/*!
 * jQuery JavaScript Library v2.2.0
 * http://jquery.com/
 *
 * Includes Sizzle.js
 * http://sizzlejs.com/
 *
 * Copyright jQuery Foundation and other contributors
 * Released under the MIT license
 * http://jquery.org/license
 *
 * Date: 2016-01-08T20:02Z
 */

(function( global, factory ) {

	if ( typeof module === "object" && typeof module.exports === "object" ) {
		// For CommonJS and CommonJS-like environments where a proper `window`
		// is present, execute the factory and get jQuery.
		// For environments that do not have a `window` with a `document`
		// (such as Node.js), expose a factory as module.exports.
		// This accentuates the need for the creation of a real `window`.
		// e.g. var jQuery = require("jquery")(window);
		// See ticket #14549 for more info.
		module.exports = global.document ?
			factory( global, true ) :
			function( w ) {
				if ( !w.document ) {
					throw new Error( "jQuery requires a window with a document" );
				}
				return factory( w );
			};
	} else {
		factory( global );
	}

// Pass this if window is not defined yet
}(typeof window !== "undefined" ? window : this, function( window, noGlobal ) {

// Support: Firefox 18+
// Can't be in strict mode, several libs including ASP.NET trace
// the stack via arguments.caller.callee and Firefox dies if
// you try to trace through "use strict" call chains. (#13335)
//"use strict";
var arr = [];

var document = window.document;

var slice = arr.slice;

var concat = arr.concat;

var push = arr.push;

var indexOf = arr.indexOf;

var class2type = {};

var toString = class2type.toString;

var hasOwn = class2type.hasOwnProperty;

var support = {};



var
	version = "2.2.0",

	// Define a local copy of jQuery
	jQuery = function( selector, context ) {

		// The jQuery object is actually just the init constructor 'enhanced'
		// Need init if jQuery is called (just allow error to be thrown if not included)
		return new jQuery.fn.init( selector, context );
	},

	// Support: Android<4.1
	// Make sure we trim BOM and NBSP
	rtrim = /^[\s\uFEFF\xA0]+|[\s\uFEFF\xA0]+$/g,

	// Matches dashed string for camelizing
	rmsPrefix = /^-ms-/,
	rdashAlpha = /-([\da-z])/gi,

	// Used by jQuery.camelCase as callback to replace()
	fcamelCase = function( all, letter ) {
		return letter.toUpperCase();
	};

jQuery.fn = jQuery.prototype = {

	// The current version of jQuery being used
	jquery: version,

	constructor: jQuery,

	// Start with an empty selector
	selector: "",

	// The default length of a jQuery object is 0
	length: 0,

	toArray: function() {
		return slice.call( this );
	},

	// Get the Nth element in the matched element set OR
	// Get the whole matched element set as a clean array
	get: function( num ) {
		return num != null ?

			// Return just the one element from the set
			( num < 0 ? this[ num + this.length ] : this[ num ] ) :

			// Return all the elements in a clean array
			slice.call( this );
	},

	// Take an array of elements and push it onto the stack
	// (returning the new matched element set)
	pushStack: function( elems ) {

		// Build a new jQuery matched element set
		var ret = jQuery.merge( this.constructor(), elems );

		// Add the old object onto the stack (as a reference)
		ret.prevObject = this;
		ret.context = this.context;

		// Return the newly-formed element set
		return ret;
	},

	// Execute a callback for every element in the matched set.
	each: function( callback ) {
		return jQuery.each( this, callback );
	},

	map: function( callback ) {
		return this.pushStack( jQuery.map( this, function( elem, i ) {
			return callback.call( elem, i, elem );
		} ) );
	},

	slice: function() {
		return this.pushStack( slice.apply( this, arguments ) );
	},

	first: function() {
		return this.eq( 0 );
	},

	last: function() {
		return this.eq( -1 );
	},

	eq: function( i ) {
		var len = this.length,
			j = +i + ( i < 0 ? len : 0 );
		return this.pushStack( j >= 0 && j < len ? [ this[ j ] ] : [] );
	},

	end: function() {
		return this.prevObject || this.constructor();
	},

	// For internal use only.
	// Behaves like an Array's method, not like a jQuery method.
	push: push,
	sort: arr.sort,
	splice: arr.splice
};

jQuery.extend = jQuery.fn.extend = function() {
	var options, name, src, copy, copyIsArray, clone,
		target = arguments[ 0 ] || {},
		i = 1,
		length = arguments.length,
		deep = false;

	// Handle a deep copy situation
	if ( typeof target === "boolean" ) {
		deep = target;

		// Skip the boolean and the target
		target = arguments[ i ] || {};
		i++;
	}

	// Handle case when target is a string or something (possible in deep copy)
	if ( typeof target !== "object" && !jQuery.isFunction( target ) ) {
		target = {};
	}

	// Extend jQuery itself if only one argument is passed
	if ( i === length ) {
		target = this;
		i--;
	}

	for ( ; i < length; i++ ) {

		// Only deal with non-null/undefined values
		if ( ( options = arguments[ i ] ) != null ) {

			// Extend the base object
			for ( name in options ) {
				src = target[ name ];
				copy = options[ name ];

				// Prevent never-ending loop
				if ( target === copy ) {
					continue;
				}

				// Recurse if we're merging plain objects or arrays
				if ( deep && copy && ( jQuery.isPlainObject( copy ) ||
					( copyIsArray = jQuery.isArray( copy ) ) ) ) {

					if ( copyIsArray ) {
						copyIsArray = false;
						clone = src && jQuery.isArray( src ) ? src : [];

					} else {
						clone = src && jQuery.isPlainObject( src ) ? src : {};
					}

					// Never move original objects, clone them
					target[ name ] = jQuery.extend( deep, clone, copy );

				// Don't bring in undefined values
				} else if ( copy !== undefined ) {
					target[ name ] = copy;
				}
			}
		}
	}

	// Return the modified object
	return target;
};

jQuery.extend( {

	// Unique for each copy of jQuery on the page
	expando: "jQuery" + ( version + Math.random() ).replace( /\D/g, "" ),

	// Assume jQuery is ready without the ready module
	isReady: true,

	error: function( msg ) {
		throw new Error( msg );
	},

	noop: function() {},

	isFunction: function( obj ) {
		return jQuery.type( obj ) === "function";
	},

	isArray: Array.isArray,

	isWindow: function( obj ) {
		return obj != null && obj === obj.window;
	},

	isNumeric: function( obj ) {

		// parseFloat NaNs numeric-cast false positives (null|true|false|"")
		// ...but misinterprets leading-number strings, particularly hex literals ("0x...")
		// subtraction forces infinities to NaN
		// adding 1 corrects loss of precision from parseFloat (#15100)
		var realStringObj = obj && obj.toString();
		return !jQuery.isArray( obj ) && ( realStringObj - parseFloat( realStringObj ) + 1 ) >= 0;
	},

	isPlainObject: function( obj ) {

		// Not plain objects:
		// - Any object or value whose internal [[Class]] property is not "[object Object]"
		// - DOM nodes
		// - window
		if ( jQuery.type( obj ) !== "object" || obj.nodeType || jQuery.isWindow( obj ) ) {
			return false;
		}

		if ( obj.constructor &&
				!hasOwn.call( obj.constructor.prototype, "isPrototypeOf" ) ) {
			return false;
		}

		// If the function hasn't returned already, we're confident that
		// |obj| is a plain object, created by {} or constructed with new Object
		return true;
	},

	isEmptyObject: function( obj ) {
		var name;
		for ( name in obj ) {
			return false;
		}
		return true;
	},

	type: function( obj ) {
		if ( obj == null ) {
			return obj + "";
		}

		// Support: Android<4.0, iOS<6 (functionish RegExp)
		return typeof obj === "object" || typeof obj === "function" ?
			class2type[ toString.call( obj ) ] || "object" :
			typeof obj;
	},

	// Evaluates a script in a global context
	globalEval: function( code ) {
		var script,
			indirect = eval;

		code = jQuery.trim( code );

		if ( code ) {

			// If the code includes a valid, prologue position
			// strict mode pragma, execute code by injecting a
			// script tag into the document.
			if ( code.indexOf( "use strict" ) === 1 ) {
				script = document.createElement( "script" );
				script.text = code;
				document.head.appendChild( script ).parentNode.removeChild( script );
			} else {

				// Otherwise, avoid the DOM node creation, insertion
				// and removal by using an indirect global eval

				indirect( code );
			}
		}
	},

	// Convert dashed to camelCase; used by the css and data modules
	// Support: IE9-11+
	// Microsoft forgot to hump their vendor prefix (#9572)
	camelCase: function( string ) {
		return string.replace( rmsPrefix, "ms-" ).replace( rdashAlpha, fcamelCase );
	},

	nodeName: function( elem, name ) {
		return elem.nodeName && elem.nodeName.toLowerCase() === name.toLowerCase();
	},

	each: function( obj, callback ) {
		var length, i = 0;

		if ( isArrayLike( obj ) ) {
			length = obj.length;
			for ( ; i < length; i++ ) {
				if ( callback.call( obj[ i ], i, obj[ i ] ) === false ) {
					break;
				}
			}
		} else {
			for ( i in obj ) {
				if ( callback.call( obj[ i ], i, obj[ i ] ) === false ) {
					break;
				}
			}
		}

		return obj;
	},

	// Support: Android<4.1
	trim: function( text ) {
		return text == null ?
			"" :
			( text + "" ).replace( rtrim, "" );
	},

	// results is for internal usage only
	makeArray: function( arr, results ) {
		var ret = results || [];

		if ( arr != null ) {
			if ( isArrayLike( Object( arr ) ) ) {
				jQuery.merge( ret,
					typeof arr === "string" ?
					[ arr ] : arr
				);
			} else {
				push.call( ret, arr );
			}
		}

		return ret;
	},

	inArray: function( elem, arr, i ) {
		return arr == null ? -1 : indexOf.call( arr, elem, i );
	},

	merge: function( first, second ) {
		var len = +second.length,
			j = 0,
			i = first.length;

		for ( ; j < len; j++ ) {
			first[ i++ ] = second[ j ];
		}

		first.length = i;

		return first;
	},

	grep: function( elems, callback, invert ) {
		var callbackInverse,
			matches = [],
			i = 0,
			length = elems.length,
			callbackExpect = !invert;

		// Go through the array, only saving the items
		// that pass the validator function
		for ( ; i < length; i++ ) {
			callbackInverse = !callback( elems[ i ], i );
			if ( callbackInverse !== callbackExpect ) {
				matches.push( elems[ i ] );
			}
		}

		return matches;
	},

	// arg is for internal usage only
	map: function( elems, callback, arg ) {
		var length, value,
			i = 0,
			ret = [];

		// Go through the array, translating each of the items to their new values
		if ( isArrayLike( elems ) ) {
			length = elems.length;
			for ( ; i < length; i++ ) {
				value = callback( elems[ i ], i, arg );

				if ( value != null ) {
					ret.push( value );
				}
			}

		// Go through every key on the object,
		} else {
			for ( i in elems ) {
				value = callback( elems[ i ], i, arg );

				if ( value != null ) {
					ret.push( value );
				}
			}
		}

		// Flatten any nested arrays
		return concat.apply( [], ret );
	},

	// A global GUID counter for objects
	guid: 1,

	// Bind a function to a context, optionally partially applying any
	// arguments.
	proxy: function( fn, context ) {
		var tmp, args, proxy;

		if ( typeof context === "string" ) {
			tmp = fn[ context ];
			context = fn;
			fn = tmp;
		}

		// Quick check to determine if target is callable, in the spec
		// this throws a TypeError, but we will just return undefined.
		if ( !jQuery.isFunction( fn ) ) {
			return undefined;
		}

		// Simulated bind
		args = slice.call( arguments, 2 );
		proxy = function() {
			return fn.apply( context || this, args.concat( slice.call( arguments ) ) );
		};

		// Set the guid of unique handler to the same of original handler, so it can be removed
		proxy.guid = fn.guid = fn.guid || jQuery.guid++;

		return proxy;
	},

	now: Date.now,

	// jQuery.support is not used in Core but other projects attach their
	// properties to it so it needs to exist.
	support: support
} );

// JSHint would error on this code due to the Symbol not being defined in ES5.
// Defining this global in .jshintrc would create a danger of using the global
// unguarded in another place, it seems safer to just disable JSHint for these
// three lines.
/* jshint ignore: start */
if ( typeof Symbol === "function" ) {
	jQuery.fn[ Symbol.iterator ] = arr[ Symbol.iterator ];
}
/* jshint ignore: end */

// Populate the class2type map
jQuery.each( "Boolean Number String Function Array Date RegExp Object Error Symbol".split( " " ),
function( i, name ) {
	class2type[ "[object " + name + "]" ] = name.toLowerCase();
} );

function isArrayLike( obj ) {

	// Support: iOS 8.2 (not reproducible in simulator)
	// `in` check used to prevent JIT error (gh-2145)
	// hasOwn isn't used here due to false negatives
	// regarding Nodelist length in IE
	var length = !!obj && "length" in obj && obj.length,
		type = jQuery.type( obj );

	if ( type === "function" || jQuery.isWindow( obj ) ) {
		return false;
	}

	return type === "array" || length === 0 ||
		typeof length === "number" && length > 0 && ( length - 1 ) in obj;
}
var Sizzle =
/*!
 * Sizzle CSS Selector Engine v2.2.1
 * http://sizzlejs.com/
 *
 * Copyright jQuery Foundation and other contributors
 * Released under the MIT license
 * http://jquery.org/license
 *
 * Date: 2015-10-17
 */
(function( window ) {

var i,
	support,
	Expr,
	getText,
	isXML,
	tokenize,
	compile,
	select,
	outermostContext,
	sortInput,
	hasDuplicate,

	// Local document vars
	setDocument,
	document,
	docElem,
	documentIsHTML,
	rbuggyQSA,
	rbuggyMatches,
	matches,
	contains,

	// Instance-specific data
	expando = "sizzle" + 1 * new Date(),
	preferredDoc = window.document,
	dirruns = 0,
	done = 0,
	classCache = createCache(),
	tokenCache = createCache(),
	compilerCache = createCache(),
	sortOrder = function( a, b ) {
		if ( a === b ) {
			hasDuplicate = true;
		}
		return 0;
	},

	// General-purpose constants
	MAX_NEGATIVE = 1 << 31,

	// Instance methods
	hasOwn = ({}).hasOwnProperty,
	arr = [],
	pop = arr.pop,
	push_native = arr.push,
	push = arr.push,
	slice = arr.slice,
	// Use a stripped-down indexOf as it's faster than native
	// http://jsperf.com/thor-indexof-vs-for/5
	indexOf = function( list, elem ) {
		var i = 0,
			len = list.length;
		for ( ; i < len; i++ ) {
			if ( list[i] === elem ) {
				return i;
			}
		}
		return -1;
	},

	booleans = "checked|selected|async|autofocus|autoplay|controls|defer|disabled|hidden|ismap|loop|multiple|open|readonly|required|scoped",

	// Regular expressions

	// http://www.w3.org/TR/css3-selectors/#whitespace
	whitespace = "[\\x20\\t\\r\\n\\f]",

	// http://www.w3.org/TR/CSS21/syndata.html#value-def-identifier
	identifier = "(?:\\\\.|[\\w-]|[^\\x00-\\xa0])+",

	// Attribute selectors: http://www.w3.org/TR/selectors/#attribute-selectors
	attributes = "\\[" + whitespace + "*(" + identifier + ")(?:" + whitespace +
		// Operator (capture 2)
		"*([*^$|!~]?=)" + whitespace +
		// "Attribute values must be CSS identifiers [capture 5] or strings [capture 3 or capture 4]"
		"*(?:'((?:\\\\.|[^\\\\'])*)'|\"((?:\\\\.|[^\\\\\"])*)\"|(" + identifier + "))|)" + whitespace +
		"*\\]",

	pseudos = ":(" + identifier + ")(?:\\((" +
		// To reduce the number of selectors needing tokenize in the preFilter, prefer arguments:
		// 1. quoted (capture 3; capture 4 or capture 5)
		"('((?:\\\\.|[^\\\\'])*)'|\"((?:\\\\.|[^\\\\\"])*)\")|" +
		// 2. simple (capture 6)
		"((?:\\\\.|[^\\\\()[\\]]|" + attributes + ")*)|" +
		// 3. anything else (capture 2)
		".*" +
		")\\)|)",

	// Leading and non-escaped trailing whitespace, capturing some non-whitespace characters preceding the latter
	rwhitespace = new RegExp( whitespace + "+", "g" ),
	rtrim = new RegExp( "^" + whitespace + "+|((?:^|[^\\\\])(?:\\\\.)*)" + whitespace + "+$", "g" ),

	rcomma = new RegExp( "^" + whitespace + "*," + whitespace + "*" ),
	rcombinators = new RegExp( "^" + whitespace + "*([>+~]|" + whitespace + ")" + whitespace + "*" ),

	rattributeQuotes = new RegExp( "=" + whitespace + "*([^\\]'\"]*?)" + whitespace + "*\\]", "g" ),

	rpseudo = new RegExp( pseudos ),
	ridentifier = new RegExp( "^" + identifier + "$" ),

	matchExpr = {
		"ID": new RegExp( "^#(" + identifier + ")" ),
		"CLASS": new RegExp( "^\\.(" + identifier + ")" ),
		"TAG": new RegExp( "^(" + identifier + "|[*])" ),
		"ATTR": new RegExp( "^" + attributes ),
		"PSEUDO": new RegExp( "^" + pseudos ),
		"CHILD": new RegExp( "^:(only|first|last|nth|nth-last)-(child|of-type)(?:\\(" + whitespace +
			"*(even|odd|(([+-]|)(\\d*)n|)" + whitespace + "*(?:([+-]|)" + whitespace +
			"*(\\d+)|))" + whitespace + "*\\)|)", "i" ),
		"bool": new RegExp( "^(?:" + booleans + ")$", "i" ),
		// For use in libraries implementing .is()
		// We use this for POS matching in `select`
		"needsContext": new RegExp( "^" + whitespace + "*[>+~]|:(even|odd|eq|gt|lt|nth|first|last)(?:\\(" +
			whitespace + "*((?:-\\d)?\\d*)" + whitespace + "*\\)|)(?=[^-]|$)", "i" )
	},

	rinputs = /^(?:input|select|textarea|button)$/i,
	rheader = /^h\d$/i,

	rnative = /^[^{]+\{\s*\[native \w/,

	// Easily-parseable/retrievable ID or TAG or CLASS selectors
	rquickExpr = /^(?:#([\w-]+)|(\w+)|\.([\w-]+))$/,

	rsibling = /[+~]/,
	rescape = /'|\\/g,

	// CSS escapes http://www.w3.org/TR/CSS21/syndata.html#escaped-characters
	runescape = new RegExp( "\\\\([\\da-f]{1,6}" + whitespace + "?|(" + whitespace + ")|.)", "ig" ),
	funescape = function( _, escaped, escapedWhitespace ) {
		var high = "0x" + escaped - 0x10000;
		// NaN means non-codepoint
		// Support: Firefox<24
		// Workaround erroneous numeric interpretation of +"0x"
		return high !== high || escapedWhitespace ?
			escaped :
			high < 0 ?
				// BMP codepoint
				String.fromCharCode( high + 0x10000 ) :
				// Supplemental Plane codepoint (surrogate pair)
				String.fromCharCode( high >> 10 | 0xD800, high & 0x3FF | 0xDC00 );
	},

	// Used for iframes
	// See setDocument()
	// Removing the function wrapper causes a "Permission Denied"
	// error in IE
	unloadHandler = function() {
		setDocument();
	};

// Optimize for push.apply( _, NodeList )
try {
	push.apply(
		(arr = slice.call( preferredDoc.childNodes )),
		preferredDoc.childNodes
	);
	// Support: Android<4.0
	// Detect silently failing push.apply
	arr[ preferredDoc.childNodes.length ].nodeType;
} catch ( e ) {
	push = { apply: arr.length ?

		// Leverage slice if possible
		function( target, els ) {
			push_native.apply( target, slice.call(els) );
		} :

		// Support: IE<9
		// Otherwise append directly
		function( target, els ) {
			var j = target.length,
				i = 0;
			// Can't trust NodeList.length
			while ( (target[j++] = els[i++]) ) {}
			target.length = j - 1;
		}
	};
}

function Sizzle( selector, context, results, seed ) {
	var m, i, elem, nid, nidselect, match, groups, newSelector,
		newContext = context && context.ownerDocument,

		// nodeType defaults to 9, since context defaults to document
		nodeType = context ? context.nodeType : 9;

	results = results || [];

	// Return early from calls with invalid selector or context
	if ( typeof selector !== "string" || !selector ||
		nodeType !== 1 && nodeType !== 9 && nodeType !== 11 ) {

		return results;
	}

	// Try to shortcut find operations (as opposed to filters) in HTML documents
	if ( !seed ) {

		if ( ( context ? context.ownerDocument || context : preferredDoc ) !== document ) {
			setDocument( context );
		}
		context = context || document;

		if ( documentIsHTML ) {

			// If the selector is sufficiently simple, try using a "get*By*" DOM method
			// (excepting DocumentFragment context, where the methods don't exist)
			if ( nodeType !== 11 && (match = rquickExpr.exec( selector )) ) {

				// ID selector
				if ( (m = match[1]) ) {

					// Document context
					if ( nodeType === 9 ) {
						if ( (elem = context.getElementById( m )) ) {

							// Support: IE, Opera, Webkit
							// TODO: identify versions
							// getElementById can match elements by name instead of ID
							if ( elem.id === m ) {
								results.push( elem );
								return results;
							}
						} else {
							return results;
						}

					// Element context
					} else {

						// Support: IE, Opera, Webkit
						// TODO: identify versions
						// getElementById can match elements by name instead of ID
						if ( newContext && (elem = newContext.getElementById( m )) &&
							contains( context, elem ) &&
							elem.id === m ) {

							results.push( elem );
							return results;
						}
					}

				// Type selector
				} else if ( match[2] ) {
					push.apply( results, context.getElementsByTagName( selector ) );
					return results;

				// Class selector
				} else if ( (m = match[3]) && support.getElementsByClassName &&
					context.getElementsByClassName ) {

					push.apply( results, context.getElementsByClassName( m ) );
					return results;
				}
			}

			// Take advantage of querySelectorAll
			if ( support.qsa &&
				!compilerCache[ selector + " " ] &&
				(!rbuggyQSA || !rbuggyQSA.test( selector )) ) {

				if ( nodeType !== 1 ) {
					newContext = context;
					newSelector = selector;

				// qSA looks outside Element context, which is not what we want
				// Thanks to Andrew Dupont for this workaround technique
				// Support: IE <=8
				// Exclude object elements
				} else if ( context.nodeName.toLowerCase() !== "object" ) {

					// Capture the context ID, setting it first if necessary
					if ( (nid = context.getAttribute( "id" )) ) {
						nid = nid.replace( rescape, "\\$&" );
					} else {
						context.setAttribute( "id", (nid = expando) );
					}

					// Prefix every selector in the list
					groups = tokenize( selector );
					i = groups.length;
					nidselect = ridentifier.test( nid ) ? "#" + nid : "[id='" + nid + "']";
					while ( i-- ) {
						groups[i] = nidselect + " " + toSelector( groups[i] );
					}
					newSelector = groups.join( "," );

					// Expand context for sibling selectors
					newContext = rsibling.test( selector ) && testContext( context.parentNode ) ||
						context;
				}

				if ( newSelector ) {
					try {
						push.apply( results,
							newContext.querySelectorAll( newSelector )
						);
						return results;
					} catch ( qsaError ) {
					} finally {
						if ( nid === expando ) {
							context.removeAttribute( "id" );
						}
					}
				}
			}
		}
	}

	// All others
	return select( selector.replace( rtrim, "$1" ), context, results, seed );
}

/**
 * Create key-value caches of limited size
 * @returns {function(string, object)} Returns the Object data after storing it on itself with
 *	property name the (space-suffixed) string and (if the cache is larger than Expr.cacheLength)
 *	deleting the oldest entry
 */
function createCache() {
	var keys = [];

	function cache( key, value ) {
		// Use (key + " ") to avoid collision with native prototype properties (see Issue #157)
		if ( keys.push( key + " " ) > Expr.cacheLength ) {
			// Only keep the most recent entries
			delete cache[ keys.shift() ];
		}
		return (cache[ key + " " ] = value);
	}
	return cache;
}

/**
 * Mark a function for special use by Sizzle
 * @param {Function} fn The function to mark
 */
function markFunction( fn ) {
	fn[ expando ] = true;
	return fn;
}

/**
 * Support testing using an element
 * @param {Function} fn Passed the created div and expects a boolean result
 */
function assert( fn ) {
	var div = document.createElement("div");

	try {
		return !!fn( div );
	} catch (e) {
		return false;
	} finally {
		// Remove from its parent by default
		if ( div.parentNode ) {
			div.parentNode.removeChild( div );
		}
		// release memory in IE
		div = null;
	}
}

/**
 * Adds the same handler for all of the specified attrs
 * @param {String} attrs Pipe-separated list of attributes
 * @param {Function} handler The method that will be applied
 */
function addHandle( attrs, handler ) {
	var arr = attrs.split("|"),
		i = arr.length;

	while ( i-- ) {
		Expr.attrHandle[ arr[i] ] = handler;
	}
}

/**
 * Checks document order of two siblings
 * @param {Element} a
 * @param {Element} b
 * @returns {Number} Returns less than 0 if a precedes b, greater than 0 if a follows b
 */
function siblingCheck( a, b ) {
	var cur = b && a,
		diff = cur && a.nodeType === 1 && b.nodeType === 1 &&
			( ~b.sourceIndex || MAX_NEGATIVE ) -
			( ~a.sourceIndex || MAX_NEGATIVE );

	// Use IE sourceIndex if available on both nodes
	if ( diff ) {
		return diff;
	}

	// Check if b follows a
	if ( cur ) {
		while ( (cur = cur.nextSibling) ) {
			if ( cur === b ) {
				return -1;
			}
		}
	}

	return a ? 1 : -1;
}

/**
 * Returns a function to use in pseudos for input types
 * @param {String} type
 */
function createInputPseudo( type ) {
	return function( elem ) {
		var name = elem.nodeName.toLowerCase();
		return name === "input" && elem.type === type;
	};
}

/**
 * Returns a function to use in pseudos for buttons
 * @param {String} type
 */
function createButtonPseudo( type ) {
	return function( elem ) {
		var name = elem.nodeName.toLowerCase();
		return (name === "input" || name === "button") && elem.type === type;
	};
}

/**
 * Returns a function to use in pseudos for positionals
 * @param {Function} fn
 */
function createPositionalPseudo( fn ) {
	return markFunction(function( argument ) {
		argument = +argument;
		return markFunction(function( seed, matches ) {
			var j,
				matchIndexes = fn( [], seed.length, argument ),
				i = matchIndexes.length;

			// Match elements found at the specified indexes
			while ( i-- ) {
				if ( seed[ (j = matchIndexes[i]) ] ) {
					seed[j] = !(matches[j] = seed[j]);
				}
			}
		});
	});
}

/**
 * Checks a node for validity as a Sizzle context
 * @param {Element|Object=} context
 * @returns {Element|Object|Boolean} The input node if acceptable, otherwise a falsy value
 */
function testContext( context ) {
	return context && typeof context.getElementsByTagName !== "undefined" && context;
}

// Expose support vars for convenience
support = Sizzle.support = {};

/**
 * Detects XML nodes
 * @param {Element|Object} elem An element or a document
 * @returns {Boolean} True iff elem is a non-HTML XML node
 */
isXML = Sizzle.isXML = function( elem ) {
	// documentElement is verified for cases where it doesn't yet exist
	// (such as loading iframes in IE - #4833)
	var documentElement = elem && (elem.ownerDocument || elem).documentElement;
	return documentElement ? documentElement.nodeName !== "HTML" : false;
};

/**
 * Sets document-related variables once based on the current document
 * @param {Element|Object} [doc] An element or document object to use to set the document
 * @returns {Object} Returns the current document
 */
setDocument = Sizzle.setDocument = function( node ) {
	var hasCompare, parent,
		doc = node ? node.ownerDocument || node : preferredDoc;

	// Return early if doc is invalid or already selected
	if ( doc === document || doc.nodeType !== 9 || !doc.documentElement ) {
		return document;
	}

	// Update global variables
	document = doc;
	docElem = document.documentElement;
	documentIsHTML = !isXML( document );

	// Support: IE 9-11, Edge
	// Accessing iframe documents after unload throws "permission denied" errors (jQuery #13936)
	if ( (parent = document.defaultView) && parent.top !== parent ) {
		// Support: IE 11
		if ( parent.addEventListener ) {
			parent.addEventListener( "unload", unloadHandler, false );

		// Support: IE 9 - 10 only
		} else if ( parent.attachEvent ) {
			parent.attachEvent( "onunload", unloadHandler );
		}
	}

	/* Attributes
	---------------------------------------------------------------------- */

	// Support: IE<8
	// Verify that getAttribute really returns attributes and not properties
	// (excepting IE8 booleans)
	support.attributes = assert(function( div ) {
		div.className = "i";
		return !div.getAttribute("className");
	});

	/* getElement(s)By*
	---------------------------------------------------------------------- */

	// Check if getElementsByTagName("*") returns only elements
	support.getElementsByTagName = assert(function( div ) {
		div.appendChild( document.createComment("") );
		return !div.getElementsByTagName("*").length;
	});

	// Support: IE<9
	support.getElementsByClassName = rnative.test( document.getElementsByClassName );

	// Support: IE<10
	// Check if getElementById returns elements by name
	// The broken getElementById methods don't pick up programatically-set names,
	// so use a roundabout getElementsByName test
	support.getById = assert(function( div ) {
		docElem.appendChild( div ).id = expando;
		return !document.getElementsByName || !document.getElementsByName( expando ).length;
	});

	// ID find and filter
	if ( support.getById ) {
		Expr.find["ID"] = function( id, context ) {
			if ( typeof context.getElementById !== "undefined" && documentIsHTML ) {
				var m = context.getElementById( id );
				return m ? [ m ] : [];
			}
		};
		Expr.filter["ID"] = function( id ) {
			var attrId = id.replace( runescape, funescape );
			return function( elem ) {
				return elem.getAttribute("id") === attrId;
			};
		};
	} else {
		// Support: IE6/7
		// getElementById is not reliable as a find shortcut
		delete Expr.find["ID"];

		Expr.filter["ID"] =  function( id ) {
			var attrId = id.replace( runescape, funescape );
			return function( elem ) {
				var node = typeof elem.getAttributeNode !== "undefined" &&
					elem.getAttributeNode("id");
				return node && node.value === attrId;
			};
		};
	}

	// Tag
	Expr.find["TAG"] = support.getElementsByTagName ?
		function( tag, context ) {
			if ( typeof context.getElementsByTagName !== "undefined" ) {
				return context.getElementsByTagName( tag );

			// DocumentFragment nodes don't have gEBTN
			} else if ( support.qsa ) {
				return context.querySelectorAll( tag );
			}
		} :

		function( tag, context ) {
			var elem,
				tmp = [],
				i = 0,
				// By happy coincidence, a (broken) gEBTN appears on DocumentFragment nodes too
				results = context.getElementsByTagName( tag );

			// Filter out possible comments
			if ( tag === "*" ) {
				while ( (elem = results[i++]) ) {
					if ( elem.nodeType === 1 ) {
						tmp.push( elem );
					}
				}

				return tmp;
			}
			return results;
		};

	// Class
	Expr.find["CLASS"] = support.getElementsByClassName && function( className, context ) {
		if ( typeof context.getElementsByClassName !== "undefined" && documentIsHTML ) {
			return context.getElementsByClassName( className );
		}
	};

	/* QSA/matchesSelector
	---------------------------------------------------------------------- */

	// QSA and matchesSelector support

	// matchesSelector(:active) reports false when true (IE9/Opera 11.5)
	rbuggyMatches = [];

	// qSa(:focus) reports false when true (Chrome 21)
	// We allow this because of a bug in IE8/9 that throws an error
	// whenever `document.activeElement` is accessed on an iframe
	// So, we allow :focus to pass through QSA all the time to avoid the IE error
	// See http://bugs.jquery.com/ticket/13378
	rbuggyQSA = [];

	if ( (support.qsa = rnative.test( document.querySelectorAll )) ) {
		// Build QSA regex
		// Regex strategy adopted from Diego Perini
		assert(function( div ) {
			// Select is set to empty string on purpose
			// This is to test IE's treatment of not explicitly
			// setting a boolean content attribute,
			// since its presence should be enough
			// http://bugs.jquery.com/ticket/12359
			docElem.appendChild( div ).innerHTML = "<a id='" + expando + "'></a>" +
				"<select id='" + expando + "-\r\\' msallowcapture=''>" +
				"<option selected=''></option></select>";

			// Support: IE8, Opera 11-12.16
			// Nothing should be selected when empty strings follow ^= or $= or *=
			// The test attribute must be unknown in Opera but "safe" for WinRT
			// http://msdn.microsoft.com/en-us/library/ie/hh465388.aspx#attribute_section
			if ( div.querySelectorAll("[msallowcapture^='']").length ) {
				rbuggyQSA.push( "[*^$]=" + whitespace + "*(?:''|\"\")" );
			}

			// Support: IE8
			// Boolean attributes and "value" are not treated correctly
			if ( !div.querySelectorAll("[selected]").length ) {
				rbuggyQSA.push( "\\[" + whitespace + "*(?:value|" + booleans + ")" );
			}

			// Support: Chrome<29, Android<4.4, Safari<7.0+, iOS<7.0+, PhantomJS<1.9.8+
			if ( !div.querySelectorAll( "[id~=" + expando + "-]" ).length ) {
				rbuggyQSA.push("~=");
			}

			// Webkit/Opera - :checked should return selected option elements
			// http://www.w3.org/TR/2011/REC-css3-selectors-20110929/#checked
			// IE8 throws error here and will not see later tests
			if ( !div.querySelectorAll(":checked").length ) {
				rbuggyQSA.push(":checked");
			}

			// Support: Safari 8+, iOS 8+
			// https://bugs.webkit.org/show_bug.cgi?id=136851
			// In-page `selector#id sibing-combinator selector` fails
			if ( !div.querySelectorAll( "a#" + expando + "+*" ).length ) {
				rbuggyQSA.push(".#.+[+~]");
			}
		});

		assert(function( div ) {
			// Support: Windows 8 Native Apps
			// The type and name attributes are restricted during .innerHTML assignment
			var input = document.createElement("input");
			input.setAttribute( "type", "hidden" );
			div.appendChild( input ).setAttribute( "name", "D" );

			// Support: IE8
			// Enforce case-sensitivity of name attribute
			if ( div.querySelectorAll("[name=d]").length ) {
				rbuggyQSA.push( "name" + whitespace + "*[*^$|!~]?=" );
			}

			// FF 3.5 - :enabled/:disabled and hidden elements (hidden elements are still enabled)
			// IE8 throws error here and will not see later tests
			if ( !div.querySelectorAll(":enabled").length ) {
				rbuggyQSA.push( ":enabled", ":disabled" );
			}

			// Opera 10-11 does not throw on post-comma invalid pseudos
			div.querySelectorAll("*,:x");
			rbuggyQSA.push(",.*:");
		});
	}

	if ( (support.matchesSelector = rnative.test( (matches = docElem.matches ||
		docElem.webkitMatchesSelector ||
		docElem.mozMatchesSelector ||
		docElem.oMatchesSelector ||
		docElem.msMatchesSelector) )) ) {

		assert(function( div ) {
			// Check to see if it's possible to do matchesSelector
			// on a disconnected node (IE 9)
			support.disconnectedMatch = matches.call( div, "div" );

			// This should fail with an exception
			// Gecko does not error, returns false instead
			matches.call( div, "[s!='']:x" );
			rbuggyMatches.push( "!=", pseudos );
		});
	}

	rbuggyQSA = rbuggyQSA.length && new RegExp( rbuggyQSA.join("|") );
	rbuggyMatches = rbuggyMatches.length && new RegExp( rbuggyMatches.join("|") );

	/* Contains
	---------------------------------------------------------------------- */
	hasCompare = rnative.test( docElem.compareDocumentPosition );

	// Element contains another
	// Purposefully self-exclusive
	// As in, an element does not contain itself
	contains = hasCompare || rnative.test( docElem.contains ) ?
		function( a, b ) {
			var adown = a.nodeType === 9 ? a.documentElement : a,
				bup = b && b.parentNode;
			return a === bup || !!( bup && bup.nodeType === 1 && (
				adown.contains ?
					adown.contains( bup ) :
					a.compareDocumentPosition && a.compareDocumentPosition( bup ) & 16
			));
		} :
		function( a, b ) {
			if ( b ) {
				while ( (b = b.parentNode) ) {
					if ( b === a ) {
						return true;
					}
				}
			}
			return false;
		};

	/* Sorting
	---------------------------------------------------------------------- */

	// Document order sorting
	sortOrder = hasCompare ?
	function( a, b ) {

		// Flag for duplicate removal
		if ( a === b ) {
			hasDuplicate = true;
			return 0;
		}

		// Sort on method existence if only one input has compareDocumentPosition
		var compare = !a.compareDocumentPosition - !b.compareDocumentPosition;
		if ( compare ) {
			return compare;
		}

		// Calculate position if both inputs belong to the same document
		compare = ( a.ownerDocument || a ) === ( b.ownerDocument || b ) ?
			a.compareDocumentPosition( b ) :

			// Otherwise we know they are disconnected
			1;

		// Disconnected nodes
		if ( compare & 1 ||
			(!support.sortDetached && b.compareDocumentPosition( a ) === compare) ) {

			// Choose the first element that is related to our preferred document
			if ( a === document || a.ownerDocument === preferredDoc && contains(preferredDoc, a) ) {
				return -1;
			}
			if ( b === document || b.ownerDocument === preferredDoc && contains(preferredDoc, b) ) {
				return 1;
			}

			// Maintain original order
			return sortInput ?
				( indexOf( sortInput, a ) - indexOf( sortInput, b ) ) :
				0;
		}

		return compare & 4 ? -1 : 1;
	} :
	function( a, b ) {
		// Exit early if the nodes are identical
		if ( a === b ) {
			hasDuplicate = true;
			return 0;
		}

		var cur,
			i = 0,
			aup = a.parentNode,
			bup = b.parentNode,
			ap = [ a ],
			bp = [ b ];

		// Parentless nodes are either documents or disconnected
		if ( !aup || !bup ) {
			return a === document ? -1 :
				b === document ? 1 :
				aup ? -1 :
				bup ? 1 :
				sortInput ?
				( indexOf( sortInput, a ) - indexOf( sortInput, b ) ) :
				0;

		// If the nodes are siblings, we can do a quick check
		} else if ( aup === bup ) {
			return siblingCheck( a, b );
		}

		// Otherwise we need full lists of their ancestors for comparison
		cur = a;
		while ( (cur = cur.parentNode) ) {
			ap.unshift( cur );
		}
		cur = b;
		while ( (cur = cur.parentNode) ) {
			bp.unshift( cur );
		}

		// Walk down the tree looking for a discrepancy
		while ( ap[i] === bp[i] ) {
			i++;
		}

		return i ?
			// Do a sibling check if the nodes have a common ancestor
			siblingCheck( ap[i], bp[i] ) :

			// Otherwise nodes in our document sort first
			ap[i] === preferredDoc ? -1 :
			bp[i] === preferredDoc ? 1 :
			0;
	};

	return document;
};

Sizzle.matches = function( expr, elements ) {
	return Sizzle( expr, null, null, elements );
};

Sizzle.matchesSelector = function( elem, expr ) {
	// Set document vars if needed
	if ( ( elem.ownerDocument || elem ) !== document ) {
		setDocument( elem );
	}

	// Make sure that attribute selectors are quoted
	expr = expr.replace( rattributeQuotes, "='$1']" );

	if ( support.matchesSelector && documentIsHTML &&
		!compilerCache[ expr + " " ] &&
		( !rbuggyMatches || !rbuggyMatches.test( expr ) ) &&
		( !rbuggyQSA     || !rbuggyQSA.test( expr ) ) ) {

		try {
			var ret = matches.call( elem, expr );

			// IE 9's matchesSelector returns false on disconnected nodes
			if ( ret || support.disconnectedMatch ||
					// As well, disconnected nodes are said to be in a document
					// fragment in IE 9
					elem.document && elem.document.nodeType !== 11 ) {
				return ret;
			}
		} catch (e) {}
	}

	return Sizzle( expr, document, null, [ elem ] ).length > 0;
};

Sizzle.contains = function( context, elem ) {
	// Set document vars if needed
	if ( ( context.ownerDocument || context ) !== document ) {
		setDocument( context );
	}
	return contains( context, elem );
};

Sizzle.attr = function( elem, name ) {
	// Set document vars if needed
	if ( ( elem.ownerDocument || elem ) !== document ) {
		setDocument( elem );
	}

	var fn = Expr.attrHandle[ name.toLowerCase() ],
		// Don't get fooled by Object.prototype properties (jQuery #13807)
		val = fn && hasOwn.call( Expr.attrHandle, name.toLowerCase() ) ?
			fn( elem, name, !documentIsHTML ) :
			undefined;

	return val !== undefined ?
		val :
		support.attributes || !documentIsHTML ?
			elem.getAttribute( name ) :
			(val = elem.getAttributeNode(name)) && val.specified ?
				val.value :
				null;
};

Sizzle.error = function( msg ) {
	throw new Error( "Syntax error, unrecognized expression: " + msg );
};

/**
 * Document sorting and removing duplicates
 * @param {ArrayLike} results
 */
Sizzle.uniqueSort = function( results ) {
	var elem,
		duplicates = [],
		j = 0,
		i = 0;

	// Unless we *know* we can detect duplicates, assume their presence
	hasDuplicate = !support.detectDuplicates;
	sortInput = !support.sortStable && results.slice( 0 );
	results.sort( sortOrder );

	if ( hasDuplicate ) {
		while ( (elem = results[i++]) ) {
			if ( elem === results[ i ] ) {
				j = duplicates.push( i );
			}
		}
		while ( j-- ) {
			results.splice( duplicates[ j ], 1 );
		}
	}

	// Clear input after sorting to release objects
	// See https://github.com/jquery/sizzle/pull/225
	sortInput = null;

	return results;
};

/**
 * Utility function for retrieving the text value of an array of DOM nodes
 * @param {Array|Element} elem
 */
getText = Sizzle.getText = function( elem ) {
	var node,
		ret = "",
		i = 0,
		nodeType = elem.nodeType;

	if ( !nodeType ) {
		// If no nodeType, this is expected to be an array
		while ( (node = elem[i++]) ) {
			// Do not traverse comment nodes
			ret += getText( node );
		}
	} else if ( nodeType === 1 || nodeType === 9 || nodeType === 11 ) {
		// Use textContent for elements
		// innerText usage removed for consistency of new lines (jQuery #11153)
		if ( typeof elem.textContent === "string" ) {
			return elem.textContent;
		} else {
			// Traverse its children
			for ( elem = elem.firstChild; elem; elem = elem.nextSibling ) {
				ret += getText( elem );
			}
		}
	} else if ( nodeType === 3 || nodeType === 4 ) {
		return elem.nodeValue;
	}
	// Do not include comment or processing instruction nodes

	return ret;
};

Expr = Sizzle.selectors = {

	// Can be adjusted by the user
	cacheLength: 50,

	createPseudo: markFunction,

	match: matchExpr,

	attrHandle: {},

	find: {},

	relative: {
		">": { dir: "parentNode", first: true },
		" ": { dir: "parentNode" },
		"+": { dir: "previousSibling", first: true },
		"~": { dir: "previousSibling" }
	},

	preFilter: {
		"ATTR": function( match ) {
			match[1] = match[1].replace( runescape, funescape );

			// Move the given value to match[3] whether quoted or unquoted
			match[3] = ( match[3] || match[4] || match[5] || "" ).replace( runescape, funescape );

			if ( match[2] === "~=" ) {
				match[3] = " " + match[3] + " ";
			}

			return match.slice( 0, 4 );
		},

		"CHILD": function( match ) {
			/* matches from matchExpr["CHILD"]
				1 type (only|nth|...)
				2 what (child|of-type)
				3 argument (even|odd|\d*|\d*n([+-]\d+)?|...)
				4 xn-component of xn+y argument ([+-]?\d*n|)
				5 sign of xn-component
				6 x of xn-component
				7 sign of y-component
				8 y of y-component
			*/
			match[1] = match[1].toLowerCase();

			if ( match[1].slice( 0, 3 ) === "nth" ) {
				// nth-* requires argument
				if ( !match[3] ) {
					Sizzle.error( match[0] );
				}

				// numeric x and y parameters for Expr.filter.CHILD
				// remember that false/true cast respectively to 0/1
				match[4] = +( match[4] ? match[5] + (match[6] || 1) : 2 * ( match[3] === "even" || match[3] === "odd" ) );
				match[5] = +( ( match[7] + match[8] ) || match[3] === "odd" );

			// other types prohibit arguments
			} else if ( match[3] ) {
				Sizzle.error( match[0] );
			}

			return match;
		},

		"PSEUDO": function( match ) {
			var excess,
				unquoted = !match[6] && match[2];

			if ( matchExpr["CHILD"].test( match[0] ) ) {
				return null;
			}

			// Accept quoted arguments as-is
			if ( match[3] ) {
				match[2] = match[4] || match[5] || "";

			// Strip excess characters from unquoted arguments
			} else if ( unquoted && rpseudo.test( unquoted ) &&
				// Get excess from tokenize (recursively)
				(excess = tokenize( unquoted, true )) &&
				// advance to the next closing parenthesis
				(excess = unquoted.indexOf( ")", unquoted.length - excess ) - unquoted.length) ) {

				// excess is a negative index
				match[0] = match[0].slice( 0, excess );
				match[2] = unquoted.slice( 0, excess );
			}

			// Return only captures needed by the pseudo filter method (type and argument)
			return match.slice( 0, 3 );
		}
	},

	filter: {

		"TAG": function( nodeNameSelector ) {
			var nodeName = nodeNameSelector.replace( runescape, funescape ).toLowerCase();
			return nodeNameSelector === "*" ?
				function() { return true; } :
				function( elem ) {
					return elem.nodeName && elem.nodeName.toLowerCase() === nodeName;
				};
		},

		"CLASS": function( className ) {
			var pattern = classCache[ className + " " ];

			return pattern ||
				(pattern = new RegExp( "(^|" + whitespace + ")" + className + "(" + whitespace + "|$)" )) &&
				classCache( className, function( elem ) {
					return pattern.test( typeof elem.className === "string" && elem.className || typeof elem.getAttribute !== "undefined" && elem.getAttribute("class") || "" );
				});
		},

		"ATTR": function( name, operator, check ) {
			return function( elem ) {
				var result = Sizzle.attr( elem, name );

				if ( result == null ) {
					return operator === "!=";
				}
				if ( !operator ) {
					return true;
				}

				result += "";

				return operator === "=" ? result === check :
					operator === "!=" ? result !== check :
					operator === "^=" ? check && result.indexOf( check ) === 0 :
					operator === "*=" ? check && result.indexOf( check ) > -1 :
					operator === "$=" ? check && result.slice( -check.length ) === check :
					operator === "~=" ? ( " " + result.replace( rwhitespace, " " ) + " " ).indexOf( check ) > -1 :
					operator === "|=" ? result === check || result.slice( 0, check.length + 1 ) === check + "-" :
					false;
			};
		},

		"CHILD": function( type, what, argument, first, last ) {
			var simple = type.slice( 0, 3 ) !== "nth",
				forward = type.slice( -4 ) !== "last",
				ofType = what === "of-type";

			return first === 1 && last === 0 ?

				// Shortcut for :nth-*(n)
				function( elem ) {
					return !!elem.parentNode;
				} :

				function( elem, context, xml ) {
					var cache, uniqueCache, outerCache, node, nodeIndex, start,
						dir = simple !== forward ? "nextSibling" : "previousSibling",
						parent = elem.parentNode,
						name = ofType && elem.nodeName.toLowerCase(),
						useCache = !xml && !ofType,
						diff = false;

					if ( parent ) {

						// :(first|last|only)-(child|of-type)
						if ( simple ) {
							while ( dir ) {
								node = elem;
								while ( (node = node[ dir ]) ) {
									if ( ofType ?
										node.nodeName.toLowerCase() === name :
										node.nodeType === 1 ) {

										return false;
									}
								}
								// Reverse direction for :only-* (if we haven't yet done so)
								start = dir = type === "only" && !start && "nextSibling";
							}
							return true;
						}

						start = [ forward ? parent.firstChild : parent.lastChild ];

						// non-xml :nth-child(...) stores cache data on `parent`
						if ( forward && useCache ) {

							// Seek `elem` from a previously-cached index

							// ...in a gzip-friendly way
							node = parent;
							outerCache = node[ expando ] || (node[ expando ] = {});

							// Support: IE <9 only
							// Defend against cloned attroperties (jQuery gh-1709)
							uniqueCache = outerCache[ node.uniqueID ] ||
								(outerCache[ node.uniqueID ] = {});

							cache = uniqueCache[ type ] || [];
							nodeIndex = cache[ 0 ] === dirruns && cache[ 1 ];
							diff = nodeIndex && cache[ 2 ];
							node = nodeIndex && parent.childNodes[ nodeIndex ];

							while ( (node = ++nodeIndex && node && node[ dir ] ||

								// Fallback to seeking `elem` from the start
								(diff = nodeIndex = 0) || start.pop()) ) {

								// When found, cache indexes on `parent` and break
								if ( node.nodeType === 1 && ++diff && node === elem ) {
									uniqueCache[ type ] = [ dirruns, nodeIndex, diff ];
									break;
								}
							}

						} else {
							// Use previously-cached element index if available
							if ( useCache ) {
								// ...in a gzip-friendly way
								node = elem;
								outerCache = node[ expando ] || (node[ expando ] = {});

								// Support: IE <9 only
								// Defend against cloned attroperties (jQuery gh-1709)
								uniqueCache = outerCache[ node.uniqueID ] ||
									(outerCache[ node.uniqueID ] = {});

								cache = uniqueCache[ type ] || [];
								nodeIndex = cache[ 0 ] === dirruns && cache[ 1 ];
								diff = nodeIndex;
							}

							// xml :nth-child(...)
							// or :nth-last-child(...) or :nth(-last)?-of-type(...)
							if ( diff === false ) {
								// Use the same loop as above to seek `elem` from the start
								while ( (node = ++nodeIndex && node && node[ dir ] ||
									(diff = nodeIndex = 0) || start.pop()) ) {

									if ( ( ofType ?
										node.nodeName.toLowerCase() === name :
										node.nodeType === 1 ) &&
										++diff ) {

										// Cache the index of each encountered element
										if ( useCache ) {
											outerCache = node[ expando ] || (node[ expando ] = {});

											// Support: IE <9 only
											// Defend against cloned attroperties (jQuery gh-1709)
											uniqueCache = outerCache[ node.uniqueID ] ||
												(outerCache[ node.uniqueID ] = {});

											uniqueCache[ type ] = [ dirruns, diff ];
										}

										if ( node === elem ) {
											break;
										}
									}
								}
							}
						}

						// Incorporate the offset, then check against cycle size
						diff -= last;
						return diff === first || ( diff % first === 0 && diff / first >= 0 );
					}
				};
		},

		"PSEUDO": function( pseudo, argument ) {
			// pseudo-class names are case-insensitive
			// http://www.w3.org/TR/selectors/#pseudo-classes
			// Prioritize by case sensitivity in case custom pseudos are added with uppercase letters
			// Remember that setFilters inherits from pseudos
			var args,
				fn = Expr.pseudos[ pseudo ] || Expr.setFilters[ pseudo.toLowerCase() ] ||
					Sizzle.error( "unsupported pseudo: " + pseudo );

			// The user may use createPseudo to indicate that
			// arguments are needed to create the filter function
			// just as Sizzle does
			if ( fn[ expando ] ) {
				return fn( argument );
			}

			// But maintain support for old signatures
			if ( fn.length > 1 ) {
				args = [ pseudo, pseudo, "", argument ];
				return Expr.setFilters.hasOwnProperty( pseudo.toLowerCase() ) ?
					markFunction(function( seed, matches ) {
						var idx,
							matched = fn( seed, argument ),
							i = matched.length;
						while ( i-- ) {
							idx = indexOf( seed, matched[i] );
							seed[ idx ] = !( matches[ idx ] = matched[i] );
						}
					}) :
					function( elem ) {
						return fn( elem, 0, args );
					};
			}

			return fn;
		}
	},

	pseudos: {
		// Potentially complex pseudos
		"not": markFunction(function( selector ) {
			// Trim the selector passed to compile
			// to avoid treating leading and trailing
			// spaces as combinators
			var input = [],
				results = [],
				matcher = compile( selector.replace( rtrim, "$1" ) );

			return matcher[ expando ] ?
				markFunction(function( seed, matches, context, xml ) {
					var elem,
						unmatched = matcher( seed, null, xml, [] ),
						i = seed.length;

					// Match elements unmatched by `matcher`
					while ( i-- ) {
						if ( (elem = unmatched[i]) ) {
							seed[i] = !(matches[i] = elem);
						}
					}
				}) :
				function( elem, context, xml ) {
					input[0] = elem;
					matcher( input, null, xml, results );
					// Don't keep the element (issue #299)
					input[0] = null;
					return !results.pop();
				};
		}),

		"has": markFunction(function( selector ) {
			return function( elem ) {
				return Sizzle( selector, elem ).length > 0;
			};
		}),

		"contains": markFunction(function( text ) {
			text = text.replace( runescape, funescape );
			return function( elem ) {
				return ( elem.textContent || elem.innerText || getText( elem ) ).indexOf( text ) > -1;
			};
		}),

		// "Whether an element is represented by a :lang() selector
		// is based solely on the element's language value
		// being equal to the identifier C,
		// or beginning with the identifier C immediately followed by "-".
		// The matching of C against the element's language value is performed case-insensitively.
		// The identifier C does not have to be a valid language name."
		// http://www.w3.org/TR/selectors/#lang-pseudo
		"lang": markFunction( function( lang ) {
			// lang value must be a valid identifier
			if ( !ridentifier.test(lang || "") ) {
				Sizzle.error( "unsupported lang: " + lang );
			}
			lang = lang.replace( runescape, funescape ).toLowerCase();
			return function( elem ) {
				var elemLang;
				do {
					if ( (elemLang = documentIsHTML ?
						elem.lang :
						elem.getAttribute("xml:lang") || elem.getAttribute("lang")) ) {

						elemLang = elemLang.toLowerCase();
						return elemLang === lang || elemLang.indexOf( lang + "-" ) === 0;
					}
				} while ( (elem = elem.parentNode) && elem.nodeType === 1 );
				return false;
			};
		}),

		// Miscellaneous
		"target": function( elem ) {
			var hash = window.location && window.location.hash;
			return hash && hash.slice( 1 ) === elem.id;
		},

		"root": function( elem ) {
			return elem === docElem;
		},

		"focus": function( elem ) {
			return elem === document.activeElement && (!document.hasFocus || document.hasFocus()) && !!(elem.type || elem.href || ~elem.tabIndex);
		},

		// Boolean properties
		"enabled": function( elem ) {
			return elem.disabled === false;
		},

		"disabled": function( elem ) {
			return elem.disabled === true;
		},

		"checked": function( elem ) {
			// In CSS3, :checked should return both checked and selected elements
			// http://www.w3.org/TR/2011/REC-css3-selectors-20110929/#checked
			var nodeName = elem.nodeName.toLowerCase();
			return (nodeName === "input" && !!elem.checked) || (nodeName === "option" && !!elem.selected);
		},

		"selected": function( elem ) {
			// Accessing this property makes selected-by-default
			// options in Safari work properly
			if ( elem.parentNode ) {
				elem.parentNode.selectedIndex;
			}

			return elem.selected === true;
		},

		// Contents
		"empty": function( elem ) {
			// http://www.w3.org/TR/selectors/#empty-pseudo
			// :empty is negated by element (1) or content nodes (text: 3; cdata: 4; entity ref: 5),
			//   but not by others (comment: 8; processing instruction: 7; etc.)
			// nodeType < 6 works because attributes (2) do not appear as children
			for ( elem = elem.firstChild; elem; elem = elem.nextSibling ) {
				if ( elem.nodeType < 6 ) {
					return false;
				}
			}
			return true;
		},

		"parent": function( elem ) {
			return !Expr.pseudos["empty"]( elem );
		},

		// Element/input types
		"header": function( elem ) {
			return rheader.test( elem.nodeName );
		},

		"input": function( elem ) {
			return rinputs.test( elem.nodeName );
		},

		"button": function( elem ) {
			var name = elem.nodeName.toLowerCase();
			return name === "input" && elem.type === "button" || name === "button";
		},

		"text": function( elem ) {
			var attr;
			return elem.nodeName.toLowerCase() === "input" &&
				elem.type === "text" &&

				// Support: IE<8
				// New HTML5 attribute values (e.g., "search") appear with elem.type === "text"
				( (attr = elem.getAttribute("type")) == null || attr.toLowerCase() === "text" );
		},

		// Position-in-collection
		"first": createPositionalPseudo(function() {
			return [ 0 ];
		}),

		"last": createPositionalPseudo(function( matchIndexes, length ) {
			return [ length - 1 ];
		}),

		"eq": createPositionalPseudo(function( matchIndexes, length, argument ) {
			return [ argument < 0 ? argument + length : argument ];
		}),

		"even": createPositionalPseudo(function( matchIndexes, length ) {
			var i = 0;
			for ( ; i < length; i += 2 ) {
				matchIndexes.push( i );
			}
			return matchIndexes;
		}),

		"odd": createPositionalPseudo(function( matchIndexes, length ) {
			var i = 1;
			for ( ; i < length; i += 2 ) {
				matchIndexes.push( i );
			}
			return matchIndexes;
		}),

		"lt": createPositionalPseudo(function( matchIndexes, length, argument ) {
			var i = argument < 0 ? argument + length : argument;
			for ( ; --i >= 0; ) {
				matchIndexes.push( i );
			}
			return matchIndexes;
		}),

		"gt": createPositionalPseudo(function( matchIndexes, length, argument ) {
			var i = argument < 0 ? argument + length : argument;
			for ( ; ++i < length; ) {
				matchIndexes.push( i );
			}
			return matchIndexes;
		})
	}
};

Expr.pseudos["nth"] = Expr.pseudos["eq"];

// Add button/input type pseudos
for ( i in { radio: true, checkbox: true, file: true, password: true, image: true } ) {
	Expr.pseudos[ i ] = createInputPseudo( i );
}
for ( i in { submit: true, reset: true } ) {
	Expr.pseudos[ i ] = createButtonPseudo( i );
}

// Easy API for creating new setFilters
function setFilters() {}
setFilters.prototype = Expr.filters = Expr.pseudos;
Expr.setFilters = new setFilters();

tokenize = Sizzle.tokenize = function( selector, parseOnly ) {
	var matched, match, tokens, type,
		soFar, groups, preFilters,
		cached = tokenCache[ selector + " " ];

	if ( cached ) {
		return parseOnly ? 0 : cached.slice( 0 );
	}

	soFar = selector;
	groups = [];
	preFilters = Expr.preFilter;

	while ( soFar ) {

		// Comma and first run
		if ( !matched || (match = rcomma.exec( soFar )) ) {
			if ( match ) {
				// Don't consume trailing commas as valid
				soFar = soFar.slice( match[0].length ) || soFar;
			}
			groups.push( (tokens = []) );
		}

		matched = false;

		// Combinators
		if ( (match = rcombinators.exec( soFar )) ) {
			matched = match.shift();
			tokens.push({
				value: matched,
				// Cast descendant combinators to space
				type: match[0].replace( rtrim, " " )
			});
			soFar = soFar.slice( matched.length );
		}

		// Filters
		for ( type in Expr.filter ) {
			if ( (match = matchExpr[ type ].exec( soFar )) && (!preFilters[ type ] ||
				(match = preFilters[ type ]( match ))) ) {
				matched = match.shift();
				tokens.push({
					value: matched,
					type: type,
					matches: match
				});
				soFar = soFar.slice( matched.length );
			}
		}

		if ( !matched ) {
			break;
		}
	}

	// Return the length of the invalid excess
	// if we're just parsing
	// Otherwise, throw an error or return tokens
	return parseOnly ?
		soFar.length :
		soFar ?
			Sizzle.error( selector ) :
			// Cache the tokens
			tokenCache( selector, groups ).slice( 0 );
};

function toSelector( tokens ) {
	var i = 0,
		len = tokens.length,
		selector = "";
	for ( ; i < len; i++ ) {
		selector += tokens[i].value;
	}
	return selector;
}

function addCombinator( matcher, combinator, base ) {
	var dir = combinator.dir,
		checkNonElements = base && dir === "parentNode",
		doneName = done++;

	return combinator.first ?
		// Check against closest ancestor/preceding element
		function( elem, context, xml ) {
			while ( (elem = elem[ dir ]) ) {
				if ( elem.nodeType === 1 || checkNonElements ) {
					return matcher( elem, context, xml );
				}
			}
		} :

		// Check against all ancestor/preceding elements
		function( elem, context, xml ) {
			var oldCache, uniqueCache, outerCache,
				newCache = [ dirruns, doneName ];

			// We can't set arbitrary data on XML nodes, so they don't benefit from combinator caching
			if ( xml ) {
				while ( (elem = elem[ dir ]) ) {
					if ( elem.nodeType === 1 || checkNonElements ) {
						if ( matcher( elem, context, xml ) ) {
							return true;
						}
					}
				}
			} else {
				while ( (elem = elem[ dir ]) ) {
					if ( elem.nodeType === 1 || checkNonElements ) {
						outerCache = elem[ expando ] || (elem[ expando ] = {});

						// Support: IE <9 only
						// Defend against cloned attroperties (jQuery gh-1709)
						uniqueCache = outerCache[ elem.uniqueID ] || (outerCache[ elem.uniqueID ] = {});

						if ( (oldCache = uniqueCache[ dir ]) &&
							oldCache[ 0 ] === dirruns && oldCache[ 1 ] === doneName ) {

							// Assign to newCache so results back-propagate to previous elements
							return (newCache[ 2 ] = oldCache[ 2 ]);
						} else {
							// Reuse newcache so results back-propagate to previous elements
							uniqueCache[ dir ] = newCache;

							// A match means we're done; a fail means we have to keep checking
							if ( (newCache[ 2 ] = matcher( elem, context, xml )) ) {
								return true;
							}
						}
					}
				}
			}
		};
}

function elementMatcher( matchers ) {
	return matchers.length > 1 ?
		function( elem, context, xml ) {
			var i = matchers.length;
			while ( i-- ) {
				if ( !matchers[i]( elem, context, xml ) ) {
					return false;
				}
			}
			return true;
		} :
		matchers[0];
}

function multipleContexts( selector, contexts, results ) {
	var i = 0,
		len = contexts.length;
	for ( ; i < len; i++ ) {
		Sizzle( selector, contexts[i], results );
	}
	return results;
}

function condense( unmatched, map, filter, context, xml ) {
	var elem,
		newUnmatched = [],
		i = 0,
		len = unmatched.length,
		mapped = map != null;

	for ( ; i < len; i++ ) {
		if ( (elem = unmatched[i]) ) {
			if ( !filter || filter( elem, context, xml ) ) {
				newUnmatched.push( elem );
				if ( mapped ) {
					map.push( i );
				}
			}
		}
	}

	return newUnmatched;
}

function setMatcher( preFilter, selector, matcher, postFilter, postFinder, postSelector ) {
	if ( postFilter && !postFilter[ expando ] ) {
		postFilter = setMatcher( postFilter );
	}
	if ( postFinder && !postFinder[ expando ] ) {
		postFinder = setMatcher( postFinder, postSelector );
	}
	return markFunction(function( seed, results, context, xml ) {
		var temp, i, elem,
			preMap = [],
			postMap = [],
			preexisting = results.length,

			// Get initial elements from seed or context
			elems = seed || multipleContexts( selector || "*", context.nodeType ? [ context ] : context, [] ),

			// Prefilter to get matcher input, preserving a map for seed-results synchronization
			matcherIn = preFilter && ( seed || !selector ) ?
				condense( elems, preMap, preFilter, context, xml ) :
				elems,

			matcherOut = matcher ?
				// If we have a postFinder, or filtered seed, or non-seed postFilter or preexisting results,
				postFinder || ( seed ? preFilter : preexisting || postFilter ) ?

					// ...intermediate processing is necessary
					[] :

					// ...otherwise use results directly
					results :
				matcherIn;

		// Find primary matches
		if ( matcher ) {
			matcher( matcherIn, matcherOut, context, xml );
		}

		// Apply postFilter
		if ( postFilter ) {
			temp = condense( matcherOut, postMap );
			postFilter( temp, [], context, xml );

			// Un-match failing elements by moving them back to matcherIn
			i = temp.length;
			while ( i-- ) {
				if ( (elem = temp[i]) ) {
					matcherOut[ postMap[i] ] = !(matcherIn[ postMap[i] ] = elem);
				}
			}
		}

		if ( seed ) {
			if ( postFinder || preFilter ) {
				if ( postFinder ) {
					// Get the final matcherOut by condensing this intermediate into postFinder contexts
					temp = [];
					i = matcherOut.length;
					while ( i-- ) {
						if ( (elem = matcherOut[i]) ) {
							// Restore matcherIn since elem is not yet a final match
							temp.push( (matcherIn[i] = elem) );
						}
					}
					postFinder( null, (matcherOut = []), temp, xml );
				}

				// Move matched elements from seed to results to keep them synchronized
				i = matcherOut.length;
				while ( i-- ) {
					if ( (elem = matcherOut[i]) &&
						(temp = postFinder ? indexOf( seed, elem ) : preMap[i]) > -1 ) {

						seed[temp] = !(results[temp] = elem);
					}
				}
			}

		// Add elements to results, through postFinder if defined
		} else {
			matcherOut = condense(
				matcherOut === results ?
					matcherOut.splice( preexisting, matcherOut.length ) :
					matcherOut
			);
			if ( postFinder ) {
				postFinder( null, results, matcherOut, xml );
			} else {
				push.apply( results, matcherOut );
			}
		}
	});
}

function matcherFromTokens( tokens ) {
	var checkContext, matcher, j,
		len = tokens.length,
		leadingRelative = Expr.relative[ tokens[0].type ],
		implicitRelative = leadingRelative || Expr.relative[" "],
		i = leadingRelative ? 1 : 0,

		// The foundational matcher ensures that elements are reachable from top-level context(s)
		matchContext = addCombinator( function( elem ) {
			return elem === checkContext;
		}, implicitRelative, true ),
		matchAnyContext = addCombinator( function( elem ) {
			return indexOf( checkContext, elem ) > -1;
		}, implicitRelative, true ),
		matchers = [ function( elem, context, xml ) {
			var ret = ( !leadingRelative && ( xml || context !== outermostContext ) ) || (
				(checkContext = context).nodeType ?
					matchContext( elem, context, xml ) :
					matchAnyContext( elem, context, xml ) );
			// Avoid hanging onto element (issue #299)
			checkContext = null;
			return ret;
		} ];

	for ( ; i < len; i++ ) {
		if ( (matcher = Expr.relative[ tokens[i].type ]) ) {
			matchers = [ addCombinator(elementMatcher( matchers ), matcher) ];
		} else {
			matcher = Expr.filter[ tokens[i].type ].apply( null, tokens[i].matches );

			// Return special upon seeing a positional matcher
			if ( matcher[ expando ] ) {
				// Find the next relative operator (if any) for proper handling
				j = ++i;
				for ( ; j < len; j++ ) {
					if ( Expr.relative[ tokens[j].type ] ) {
						break;
					}
				}
				return setMatcher(
					i > 1 && elementMatcher( matchers ),
					i > 1 && toSelector(
						// If the preceding token was a descendant combinator, insert an implicit any-element `*`
						tokens.slice( 0, i - 1 ).concat({ value: tokens[ i - 2 ].type === " " ? "*" : "" })
					).replace( rtrim, "$1" ),
					matcher,
					i < j && matcherFromTokens( tokens.slice( i, j ) ),
					j < len && matcherFromTokens( (tokens = tokens.slice( j )) ),
					j < len && toSelector( tokens )
				);
			}
			matchers.push( matcher );
		}
	}

	return elementMatcher( matchers );
}

function matcherFromGroupMatchers( elementMatchers, setMatchers ) {
	var bySet = setMatchers.length > 0,
		byElement = elementMatchers.length > 0,
		superMatcher = function( seed, context, xml, results, outermost ) {
			var elem, j, matcher,
				matchedCount = 0,
				i = "0",
				unmatched = seed && [],
				setMatched = [],
				contextBackup = outermostContext,
				// We must always have either seed elements or outermost context
				elems = seed || byElement && Expr.find["TAG"]( "*", outermost ),
				// Use integer dirruns iff this is the outermost matcher
				dirrunsUnique = (dirruns += contextBackup == null ? 1 : Math.random() || 0.1),
				len = elems.length;

			if ( outermost ) {
				outermostContext = context === document || context || outermost;
			}

			// Add elements passing elementMatchers directly to results
			// Support: IE<9, Safari
			// Tolerate NodeList properties (IE: "length"; Safari: <number>) matching elements by id
			for ( ; i !== len && (elem = elems[i]) != null; i++ ) {
				if ( byElement && elem ) {
					j = 0;
					if ( !context && elem.ownerDocument !== document ) {
						setDocument( elem );
						xml = !documentIsHTML;
					}
					while ( (matcher = elementMatchers[j++]) ) {
						if ( matcher( elem, context || document, xml) ) {
							results.push( elem );
							break;
						}
					}
					if ( outermost ) {
						dirruns = dirrunsUnique;
					}
				}

				// Track unmatched elements for set filters
				if ( bySet ) {
					// They will have gone through all possible matchers
					if ( (elem = !matcher && elem) ) {
						matchedCount--;
					}

					// Lengthen the array for every element, matched or not
					if ( seed ) {
						unmatched.push( elem );
					}
				}
			}

			// `i` is now the count of elements visited above, and adding it to `matchedCount`
			// makes the latter nonnegative.
			matchedCount += i;

			// Apply set filters to unmatched elements
			// NOTE: This can be skipped if there are no unmatched elements (i.e., `matchedCount`
			// equals `i`), unless we didn't visit _any_ elements in the above loop because we have
			// no element matchers and no seed.
			// Incrementing an initially-string "0" `i` allows `i` to remain a string only in that
			// case, which will result in a "00" `matchedCount` that differs from `i` but is also
			// numerically zero.
			if ( bySet && i !== matchedCount ) {
				j = 0;
				while ( (matcher = setMatchers[j++]) ) {
					matcher( unmatched, setMatched, context, xml );
				}

				if ( seed ) {
					// Reintegrate element matches to eliminate the need for sorting
					if ( matchedCount > 0 ) {
						while ( i-- ) {
							if ( !(unmatched[i] || setMatched[i]) ) {
								setMatched[i] = pop.call( results );
							}
						}
					}

					// Discard index placeholder values to get only actual matches
					setMatched = condense( setMatched );
				}

				// Add matches to results
				push.apply( results, setMatched );

				// Seedless set matches succeeding multiple successful matchers stipulate sorting
				if ( outermost && !seed && setMatched.length > 0 &&
					( matchedCount + setMatchers.length ) > 1 ) {

					Sizzle.uniqueSort( results );
				}
			}

			// Override manipulation of globals by nested matchers
			if ( outermost ) {
				dirruns = dirrunsUnique;
				outermostContext = contextBackup;
			}

			return unmatched;
		};

	return bySet ?
		markFunction( superMatcher ) :
		superMatcher;
}

compile = Sizzle.compile = function( selector, match /* Internal Use Only */ ) {
	var i,
		setMatchers = [],
		elementMatchers = [],
		cached = compilerCache[ selector + " " ];

	if ( !cached ) {
		// Generate a function of recursive functions that can be used to check each element
		if ( !match ) {
			match = tokenize( selector );
		}
		i = match.length;
		while ( i-- ) {
			cached = matcherFromTokens( match[i] );
			if ( cached[ expando ] ) {
				setMatchers.push( cached );
			} else {
				elementMatchers.push( cached );
			}
		}

		// Cache the compiled function
		cached = compilerCache( selector, matcherFromGroupMatchers( elementMatchers, setMatchers ) );

		// Save selector and tokenization
		cached.selector = selector;
	}
	return cached;
};

/**
 * A low-level selection function that works with Sizzle's compiled
 *  selector functions
 * @param {String|Function} selector A selector or a pre-compiled
 *  selector function built with Sizzle.compile
 * @param {Element} context
 * @param {Array} [results]
 * @param {Array} [seed] A set of elements to match against
 */
select = Sizzle.select = function( selector, context, results, seed ) {
	var i, tokens, token, type, find,
		compiled = typeof selector === "function" && selector,
		match = !seed && tokenize( (selector = compiled.selector || selector) );

	results = results || [];

	// Try to minimize operations if there is only one selector in the list and no seed
	// (the latter of which guarantees us context)
	if ( match.length === 1 ) {

		// Reduce context if the leading compound selector is an ID
		tokens = match[0] = match[0].slice( 0 );
		if ( tokens.length > 2 && (token = tokens[0]).type === "ID" &&
				support.getById && context.nodeType === 9 && documentIsHTML &&
				Expr.relative[ tokens[1].type ] ) {

			context = ( Expr.find["ID"]( token.matches[0].replace(runescape, funescape), context ) || [] )[0];
			if ( !context ) {
				return results;

			// Precompiled matchers will still verify ancestry, so step up a level
			} else if ( compiled ) {
				context = context.parentNode;
			}

			selector = selector.slice( tokens.shift().value.length );
		}

		// Fetch a seed set for right-to-left matching
		i = matchExpr["needsContext"].test( selector ) ? 0 : tokens.length;
		while ( i-- ) {
			token = tokens[i];

			// Abort if we hit a combinator
			if ( Expr.relative[ (type = token.type) ] ) {
				break;
			}
			if ( (find = Expr.find[ type ]) ) {
				// Search, expanding context for leading sibling combinators
				if ( (seed = find(
					token.matches[0].replace( runescape, funescape ),
					rsibling.test( tokens[0].type ) && testContext( context.parentNode ) || context
				)) ) {

					// If seed is empty or no tokens remain, we can return early
					tokens.splice( i, 1 );
					selector = seed.length && toSelector( tokens );
					if ( !selector ) {
						push.apply( results, seed );
						return results;
					}

					break;
				}
			}
		}
	}

	// Compile and execute a filtering function if one is not provided
	// Provide `match` to avoid retokenization if we modified the selector above
	( compiled || compile( selector, match ) )(
		seed,
		context,
		!documentIsHTML,
		results,
		!context || rsibling.test( selector ) && testContext( context.parentNode ) || context
	);
	return results;
};

// One-time assignments

// Sort stability
support.sortStable = expando.split("").sort( sortOrder ).join("") === expando;

// Support: Chrome 14-35+
// Always assume duplicates if they aren't passed to the comparison function
support.detectDuplicates = !!hasDuplicate;

// Initialize against the default document
setDocument();

// Support: Webkit<537.32 - Safari 6.0.3/Chrome 25 (fixed in Chrome 27)
// Detached nodes confoundingly follow *each other*
support.sortDetached = assert(function( div1 ) {
	// Should return 1, but returns 4 (following)
	return div1.compareDocumentPosition( document.createElement("div") ) & 1;
});

// Support: IE<8
// Prevent attribute/property "interpolation"
// http://msdn.microsoft.com/en-us/library/ms536429%28VS.85%29.aspx
if ( !assert(function( div ) {
	div.innerHTML = "<a href='#'></a>";
	return div.firstChild.getAttribute("href") === "#" ;
}) ) {
	addHandle( "type|href|height|width", function( elem, name, isXML ) {
		if ( !isXML ) {
			return elem.getAttribute( name, name.toLowerCase() === "type" ? 1 : 2 );
		}
	});
}

// Support: IE<9
// Use defaultValue in place of getAttribute("value")
if ( !support.attributes || !assert(function( div ) {
	div.innerHTML = "<input/>";
	div.firstChild.setAttribute( "value", "" );
	return div.firstChild.getAttribute( "value" ) === "";
}) ) {
	addHandle( "value", function( elem, name, isXML ) {
		if ( !isXML && elem.nodeName.toLowerCase() === "input" ) {
			return elem.defaultValue;
		}
	});
}

// Support: IE<9
// Use getAttributeNode to fetch booleans when getAttribute lies
if ( !assert(function( div ) {
	return div.getAttribute("disabled") == null;
}) ) {
	addHandle( booleans, function( elem, name, isXML ) {
		var val;
		if ( !isXML ) {
			return elem[ name ] === true ? name.toLowerCase() :
					(val = elem.getAttributeNode( name )) && val.specified ?
					val.value :
				null;
		}
	});
}

return Sizzle;

})( window );



jQuery.find = Sizzle;
jQuery.expr = Sizzle.selectors;
jQuery.expr[ ":" ] = jQuery.expr.pseudos;
jQuery.uniqueSort = jQuery.unique = Sizzle.uniqueSort;
jQuery.text = Sizzle.getText;
jQuery.isXMLDoc = Sizzle.isXML;
jQuery.contains = Sizzle.contains;



var dir = function( elem, dir, until ) {
	var matched = [],
		truncate = until !== undefined;

	while ( ( elem = elem[ dir ] ) && elem.nodeType !== 9 ) {
		if ( elem.nodeType === 1 ) {
			if ( truncate && jQuery( elem ).is( until ) ) {
				break;
			}
			matched.push( elem );
		}
	}
	return matched;
};


var siblings = function( n, elem ) {
	var matched = [];

	for ( ; n; n = n.nextSibling ) {
		if ( n.nodeType === 1 && n !== elem ) {
			matched.push( n );
		}
	}

	return matched;
};


var rneedsContext = jQuery.expr.match.needsContext;

var rsingleTag = ( /^<([\w-]+)\s*\/?>(?:<\/\1>|)$/ );



var risSimple = /^.[^:#\[\.,]*$/;

// Implement the identical functionality for filter and not
function winnow( elements, qualifier, not ) {
	if ( jQuery.isFunction( qualifier ) ) {
		return jQuery.grep( elements, function( elem, i ) {
			/* jshint -W018 */
			return !!qualifier.call( elem, i, elem ) !== not;
		} );

	}

	if ( qualifier.nodeType ) {
		return jQuery.grep( elements, function( elem ) {
			return ( elem === qualifier ) !== not;
		} );

	}

	if ( typeof qualifier === "string" ) {
		if ( risSimple.test( qualifier ) ) {
			return jQuery.filter( qualifier, elements, not );
		}

		qualifier = jQuery.filter( qualifier, elements );
	}

	return jQuery.grep( elements, function( elem ) {
		return ( indexOf.call( qualifier, elem ) > -1 ) !== not;
	} );
}

jQuery.filter = function( expr, elems, not ) {
	var elem = elems[ 0 ];

	if ( not ) {
		expr = ":not(" + expr + ")";
	}

	return elems.length === 1 && elem.nodeType === 1 ?
		jQuery.find.matchesSelector( elem, expr ) ? [ elem ] : [] :
		jQuery.find.matches( expr, jQuery.grep( elems, function( elem ) {
			return elem.nodeType === 1;
		} ) );
};

jQuery.fn.extend( {
	find: function( selector ) {
		var i,
			len = this.length,
			ret = [],
			self = this;

		if ( typeof selector !== "string" ) {
			return this.pushStack( jQuery( selector ).filter( function() {
				for ( i = 0; i < len; i++ ) {
					if ( jQuery.contains( self[ i ], this ) ) {
						return true;
					}
				}
			} ) );
		}

		for ( i = 0; i < len; i++ ) {
			jQuery.find( selector, self[ i ], ret );
		}

		// Needed because $( selector, context ) becomes $( context ).find( selector )
		ret = this.pushStack( len > 1 ? jQuery.unique( ret ) : ret );
		ret.selector = this.selector ? this.selector + " " + selector : selector;
		return ret;
	},
	filter: function( selector ) {
		return this.pushStack( winnow( this, selector || [], false ) );
	},
	not: function( selector ) {
		return this.pushStack( winnow( this, selector || [], true ) );
	},
	is: function( selector ) {
		return !!winnow(
			this,

			// If this is a positional/relative selector, check membership in the returned set
			// so $("p:first").is("p:last") won't return true for a doc with two "p".
			typeof selector === "string" && rneedsContext.test( selector ) ?
				jQuery( selector ) :
				selector || [],
			false
		).length;
	}
} );


// Initialize a jQuery object


// A central reference to the root jQuery(document)
var rootjQuery,

	// A simple way to check for HTML strings
	// Prioritize #id over <tag> to avoid XSS via location.hash (#9521)
	// Strict HTML recognition (#11290: must start with <)
	rquickExpr = /^(?:\s*(<[\w\W]+>)[^>]*|#([\w-]*))$/,

	init = jQuery.fn.init = function( selector, context, root ) {
		var match, elem;

		// HANDLE: $(""), $(null), $(undefined), $(false)
		if ( !selector ) {
			return this;
		}

		// Method init() accepts an alternate rootjQuery
		// so migrate can support jQuery.sub (gh-2101)
		root = root || rootjQuery;

		// Handle HTML strings
		if ( typeof selector === "string" ) {
			if ( selector[ 0 ] === "<" &&
				selector[ selector.length - 1 ] === ">" &&
				selector.length >= 3 ) {

				// Assume that strings that start and end with <> are HTML and skip the regex check
				match = [ null, selector, null ];

			} else {
				match = rquickExpr.exec( selector );
			}

			// Match html or make sure no context is specified for #id
			if ( match && ( match[ 1 ] || !context ) ) {

				// HANDLE: $(html) -> $(array)
				if ( match[ 1 ] ) {
					context = context instanceof jQuery ? context[ 0 ] : context;

					// Option to run scripts is true for back-compat
					// Intentionally let the error be thrown if parseHTML is not present
					jQuery.merge( this, jQuery.parseHTML(
						match[ 1 ],
						context && context.nodeType ? context.ownerDocument || context : document,
						true
					) );

					// HANDLE: $(html, props)
					if ( rsingleTag.test( match[ 1 ] ) && jQuery.isPlainObject( context ) ) {
						for ( match in context ) {

							// Properties of context are called as methods if possible
							if ( jQuery.isFunction( this[ match ] ) ) {
								this[ match ]( context[ match ] );

							// ...and otherwise set as attributes
							} else {
								this.attr( match, context[ match ] );
							}
						}
					}

					return this;

				// HANDLE: $(#id)
				} else {
					elem = document.getElementById( match[ 2 ] );

					// Support: Blackberry 4.6
					// gEBID returns nodes no longer in the document (#6963)
					if ( elem && elem.parentNode ) {

						// Inject the element directly into the jQuery object
						this.length = 1;
						this[ 0 ] = elem;
					}

					this.context = document;
					this.selector = selector;
					return this;
				}

			// HANDLE: $(expr, $(...))
			} else if ( !context || context.jquery ) {
				return ( context || root ).find( selector );

			// HANDLE: $(expr, context)
			// (which is just equivalent to: $(context).find(expr)
			} else {
				return this.constructor( context ).find( selector );
			}

		// HANDLE: $(DOMElement)
		} else if ( selector.nodeType ) {
			this.context = this[ 0 ] = selector;
			this.length = 1;
			return this;

		// HANDLE: $(function)
		// Shortcut for document ready
		} else if ( jQuery.isFunction( selector ) ) {
			return root.ready !== undefined ?
				root.ready( selector ) :

				// Execute immediately if ready is not present
				selector( jQuery );
		}

		if ( selector.selector !== undefined ) {
			this.selector = selector.selector;
			this.context = selector.context;
		}

		return jQuery.makeArray( selector, this );
	};

// Give the init function the jQuery prototype for later instantiation
init.prototype = jQuery.fn;

// Initialize central reference
rootjQuery = jQuery( document );


var rparentsprev = /^(?:parents|prev(?:Until|All))/,

	// Methods guaranteed to produce a unique set when starting from a unique set
	guaranteedUnique = {
		children: true,
		contents: true,
		next: true,
		prev: true
	};

jQuery.fn.extend( {
	has: function( target ) {
		var targets = jQuery( target, this ),
			l = targets.length;

		return this.filter( function() {
			var i = 0;
			for ( ; i < l; i++ ) {
				if ( jQuery.contains( this, targets[ i ] ) ) {
					return true;
				}
			}
		} );
	},

	closest: function( selectors, context ) {
		var cur,
			i = 0,
			l = this.length,
			matched = [],
			pos = rneedsContext.test( selectors ) || typeof selectors !== "string" ?
				jQuery( selectors, context || this.context ) :
				0;

		for ( ; i < l; i++ ) {
			for ( cur = this[ i ]; cur && cur !== context; cur = cur.parentNode ) {

				// Always skip document fragments
				if ( cur.nodeType < 11 && ( pos ?
					pos.index( cur ) > -1 :

					// Don't pass non-elements to Sizzle
					cur.nodeType === 1 &&
						jQuery.find.matchesSelector( cur, selectors ) ) ) {

					matched.push( cur );
					break;
				}
			}
		}

		return this.pushStack( matched.length > 1 ? jQuery.uniqueSort( matched ) : matched );
	},

	// Determine the position of an element within the set
	index: function( elem ) {

		// No argument, return index in parent
		if ( !elem ) {
			return ( this[ 0 ] && this[ 0 ].parentNode ) ? this.first().prevAll().length : -1;
		}

		// Index in selector
		if ( typeof elem === "string" ) {
			return indexOf.call( jQuery( elem ), this[ 0 ] );
		}

		// Locate the position of the desired element
		return indexOf.call( this,

			// If it receives a jQuery object, the first element is used
			elem.jquery ? elem[ 0 ] : elem
		);
	},

	add: function( selector, context ) {
		return this.pushStack(
			jQuery.uniqueSort(
				jQuery.merge( this.get(), jQuery( selector, context ) )
			)
		);
	},

	addBack: function( selector ) {
		return this.add( selector == null ?
			this.prevObject : this.prevObject.filter( selector )
		);
	}
} );

function sibling( cur, dir ) {
	while ( ( cur = cur[ dir ] ) && cur.nodeType !== 1 ) {}
	return cur;
}

jQuery.each( {
	parent: function( elem ) {
		var parent = elem.parentNode;
		return parent && parent.nodeType !== 11 ? parent : null;
	},
	parents: function( elem ) {
		return dir( elem, "parentNode" );
	},
	parentsUntil: function( elem, i, until ) {
		return dir( elem, "parentNode", until );
	},
	next: function( elem ) {
		return sibling( elem, "nextSibling" );
	},
	prev: function( elem ) {
		return sibling( elem, "previousSibling" );
	},
	nextAll: function( elem ) {
		return dir( elem, "nextSibling" );
	},
	prevAll: function( elem ) {
		return dir( elem, "previousSibling" );
	},
	nextUntil: function( elem, i, until ) {
		return dir( elem, "nextSibling", until );
	},
	prevUntil: function( elem, i, until ) {
		return dir( elem, "previousSibling", until );
	},
	siblings: function( elem ) {
		return siblings( ( elem.parentNode || {} ).firstChild, elem );
	},
	children: function( elem ) {
		return siblings( elem.firstChild );
	},
	contents: function( elem ) {
		return elem.contentDocument || jQuery.merge( [], elem.childNodes );
	}
}, function( name, fn ) {
	jQuery.fn[ name ] = function( until, selector ) {
		var matched = jQuery.map( this, fn, until );

		if ( name.slice( -5 ) !== "Until" ) {
			selector = until;
		}

		if ( selector && typeof selector === "string" ) {
			matched = jQuery.filter( selector, matched );
		}

		if ( this.length > 1 ) {

			// Remove duplicates
			if ( !guaranteedUnique[ name ] ) {
				jQuery.uniqueSort( matched );
			}

			// Reverse order for parents* and prev-derivatives
			if ( rparentsprev.test( name ) ) {
				matched.reverse();
			}
		}

		return this.pushStack( matched );
	};
} );
var rnotwhite = ( /\S+/g );



// Convert String-formatted options into Object-formatted ones
function createOptions( options ) {
	var object = {};
	jQuery.each( options.match( rnotwhite ) || [], function( _, flag ) {
		object[ flag ] = true;
	} );
	return object;
}

/*
 * Create a callback list using the following parameters:
 *
 *	options: an optional list of space-separated options that will change how
 *			the callback list behaves or a more traditional option object
 *
 * By default a callback list will act like an event callback list and can be
 * "fired" multiple times.
 *
 * Possible options:
 *
 *	once:			will ensure the callback list can only be fired once (like a Deferred)
 *
 *	memory:			will keep track of previous values and will call any callback added
 *					after the list has been fired right away with the latest "memorized"
 *					values (like a Deferred)
 *
 *	unique:			will ensure a callback can only be added once (no duplicate in the list)
 *
 *	stopOnFalse:	interrupt callings when a callback returns false
 *
 */
jQuery.Callbacks = function( options ) {

	// Convert options from String-formatted to Object-formatted if needed
	// (we check in cache first)
	options = typeof options === "string" ?
		createOptions( options ) :
		jQuery.extend( {}, options );

	var // Flag to know if list is currently firing
		firing,

		// Last fire value for non-forgettable lists
		memory,

		// Flag to know if list was already fired
		fired,

		// Flag to prevent firing
		locked,

		// Actual callback list
		list = [],

		// Queue of execution data for repeatable lists
		queue = [],

		// Index of currently firing callback (modified by add/remove as needed)
		firingIndex = -1,

		// Fire callbacks
		fire = function() {

			// Enforce single-firing
			locked = options.once;

			// Execute callbacks for all pending executions,
			// respecting firingIndex overrides and runtime changes
			fired = firing = true;
			for ( ; queue.length; firingIndex = -1 ) {
				memory = queue.shift();
				while ( ++firingIndex < list.length ) {

					// Run callback and check for early termination
					if ( list[ firingIndex ].apply( memory[ 0 ], memory[ 1 ] ) === false &&
						options.stopOnFalse ) {

						// Jump to end and forget the data so .add doesn't re-fire
						firingIndex = list.length;
						memory = false;
					}
				}
			}

			// Forget the data if we're done with it
			if ( !options.memory ) {
				memory = false;
			}

			firing = false;

			// Clean up if we're done firing for good
			if ( locked ) {

				// Keep an empty list if we have data for future add calls
				if ( memory ) {
					list = [];

				// Otherwise, this object is spent
				} else {
					list = "";
				}
			}
		},

		// Actual Callbacks object
		self = {

			// Add a callback or a collection of callbacks to the list
			add: function() {
				if ( list ) {

					// If we have memory from a past run, we should fire after adding
					if ( memory && !firing ) {
						firingIndex = list.length - 1;
						queue.push( memory );
					}

					( function add( args ) {
						jQuery.each( args, function( _, arg ) {
							if ( jQuery.isFunction( arg ) ) {
								if ( !options.unique || !self.has( arg ) ) {
									list.push( arg );
								}
							} else if ( arg && arg.length && jQuery.type( arg ) !== "string" ) {

								// Inspect recursively
								add( arg );
							}
						} );
					} )( arguments );

					if ( memory && !firing ) {
						fire();
					}
				}
				return this;
			},

			// Remove a callback from the list
			remove: function() {
				jQuery.each( arguments, function( _, arg ) {
					var index;
					while ( ( index = jQuery.inArray( arg, list, index ) ) > -1 ) {
						list.splice( index, 1 );

						// Handle firing indexes
						if ( index <= firingIndex ) {
							firingIndex--;
						}
					}
				} );
				return this;
			},

			// Check if a given callback is in the list.
			// If no argument is given, return whether or not list has callbacks attached.
			has: function( fn ) {
				return fn ?
					jQuery.inArray( fn, list ) > -1 :
					list.length > 0;
			},

			// Remove all callbacks from the list
			empty: function() {
				if ( list ) {
					list = [];
				}
				return this;
			},

			// Disable .fire and .add
			// Abort any current/pending executions
			// Clear all callbacks and values
			disable: function() {
				locked = queue = [];
				list = memory = "";
				return this;
			},
			disabled: function() {
				return !list;
			},

			// Disable .fire
			// Also disable .add unless we have memory (since it would have no effect)
			// Abort any pending executions
			lock: function() {
				locked = queue = [];
				if ( !memory ) {
					list = memory = "";
				}
				return this;
			},
			locked: function() {
				return !!locked;
			},

			// Call all callbacks with the given context and arguments
			fireWith: function( context, args ) {
				if ( !locked ) {
					args = args || [];
					args = [ context, args.slice ? args.slice() : args ];
					queue.push( args );
					if ( !firing ) {
						fire();
					}
				}
				return this;
			},

			// Call all the callbacks with the given arguments
			fire: function() {
				self.fireWith( this, arguments );
				return this;
			},

			// To know if the callbacks have already been called at least once
			fired: function() {
				return !!fired;
			}
		};

	return self;
};


jQuery.extend( {

	Deferred: function( func ) {
		var tuples = [

				// action, add listener, listener list, final state
				[ "resolve", "done", jQuery.Callbacks( "once memory" ), "resolved" ],
				[ "reject", "fail", jQuery.Callbacks( "once memory" ), "rejected" ],
				[ "notify", "progress", jQuery.Callbacks( "memory" ) ]
			],
			state = "pending",
			promise = {
				state: function() {
					return state;
				},
				always: function() {
					deferred.done( arguments ).fail( arguments );
					return this;
				},
				then: function( /* fnDone, fnFail, fnProgress */ ) {
					var fns = arguments;
					return jQuery.Deferred( function( newDefer ) {
						jQuery.each( tuples, function( i, tuple ) {
							var fn = jQuery.isFunction( fns[ i ] ) && fns[ i ];

							// deferred[ done | fail | progress ] for forwarding actions to newDefer
							deferred[ tuple[ 1 ] ]( function() {
								var returned = fn && fn.apply( this, arguments );
								if ( returned && jQuery.isFunction( returned.promise ) ) {
									returned.promise()
										.progress( newDefer.notify )
										.done( newDefer.resolve )
										.fail( newDefer.reject );
								} else {
									newDefer[ tuple[ 0 ] + "With" ](
										this === promise ? newDefer.promise() : this,
										fn ? [ returned ] : arguments
									);
								}
							} );
						} );
						fns = null;
					} ).promise();
				},

				// Get a promise for this deferred
				// If obj is provided, the promise aspect is added to the object
				promise: function( obj ) {
					return obj != null ? jQuery.extend( obj, promise ) : promise;
				}
			},
			deferred = {};

		// Keep pipe for back-compat
		promise.pipe = promise.then;

		// Add list-specific methods
		jQuery.each( tuples, function( i, tuple ) {
			var list = tuple[ 2 ],
				stateString = tuple[ 3 ];

			// promise[ done | fail | progress ] = list.add
			promise[ tuple[ 1 ] ] = list.add;

			// Handle state
			if ( stateString ) {
				list.add( function() {

					// state = [ resolved | rejected ]
					state = stateString;

				// [ reject_list | resolve_list ].disable; progress_list.lock
				}, tuples[ i ^ 1 ][ 2 ].disable, tuples[ 2 ][ 2 ].lock );
			}

			// deferred[ resolve | reject | notify ]
			deferred[ tuple[ 0 ] ] = function() {
				deferred[ tuple[ 0 ] + "With" ]( this === deferred ? promise : this, arguments );
				return this;
			};
			deferred[ tuple[ 0 ] + "With" ] = list.fireWith;
		} );

		// Make the deferred a promise
		promise.promise( deferred );

		// Call given func if any
		if ( func ) {
			func.call( deferred, deferred );
		}

		// All done!
		return deferred;
	},

	// Deferred helper
	when: function( subordinate /* , ..., subordinateN */ ) {
		var i = 0,
			resolveValues = slice.call( arguments ),
			length = resolveValues.length,

			// the count of uncompleted subordinates
			remaining = length !== 1 ||
				( subordinate && jQuery.isFunction( subordinate.promise ) ) ? length : 0,

			// the master Deferred.
			// If resolveValues consist of only a single Deferred, just use that.
			deferred = remaining === 1 ? subordinate : jQuery.Deferred(),

			// Update function for both resolve and progress values
			updateFunc = function( i, contexts, values ) {
				return function( value ) {
					contexts[ i ] = this;
					values[ i ] = arguments.length > 1 ? slice.call( arguments ) : value;
					if ( values === progressValues ) {
						deferred.notifyWith( contexts, values );
					} else if ( !( --remaining ) ) {
						deferred.resolveWith( contexts, values );
					}
				};
			},

			progressValues, progressContexts, resolveContexts;

		// Add listeners to Deferred subordinates; treat others as resolved
		if ( length > 1 ) {
			progressValues = new Array( length );
			progressContexts = new Array( length );
			resolveContexts = new Array( length );
			for ( ; i < length; i++ ) {
				if ( resolveValues[ i ] && jQuery.isFunction( resolveValues[ i ].promise ) ) {
					resolveValues[ i ].promise()
						.progress( updateFunc( i, progressContexts, progressValues ) )
						.done( updateFunc( i, resolveContexts, resolveValues ) )
						.fail( deferred.reject );
				} else {
					--remaining;
				}
			}
		}

		// If we're not waiting on anything, resolve the master
		if ( !remaining ) {
			deferred.resolveWith( resolveContexts, resolveValues );
		}

		return deferred.promise();
	}
} );


// The deferred used on DOM ready
var readyList;

jQuery.fn.ready = function( fn ) {

	// Add the callback
	jQuery.ready.promise().done( fn );

	return this;
};

jQuery.extend( {

	// Is the DOM ready to be used? Set to true once it occurs.
	isReady: false,

	// A counter to track how many items to wait for before
	// the ready event fires. See #6781
	readyWait: 1,

	// Hold (or release) the ready event
	holdReady: function( hold ) {
		if ( hold ) {
			jQuery.readyWait++;
		} else {
			jQuery.ready( true );
		}
	},

	// Handle when the DOM is ready
	ready: function( wait ) {

		// Abort if there are pending holds or we're already ready
		if ( wait === true ? --jQuery.readyWait : jQuery.isReady ) {
			return;
		}

		// Remember that the DOM is ready
		jQuery.isReady = true;

		// If a normal DOM Ready event fired, decrement, and wait if need be
		if ( wait !== true && --jQuery.readyWait > 0 ) {
			return;
		}

		// If there are functions bound, to execute
		readyList.resolveWith( document, [ jQuery ] );

		// Trigger any bound ready events
		if ( jQuery.fn.triggerHandler ) {
			jQuery( document ).triggerHandler( "ready" );
			jQuery( document ).off( "ready" );
		}
	}
} );

/**
 * The ready event handler and self cleanup method
 */
function completed() {
	document.removeEventListener( "DOMContentLoaded", completed );
	window.removeEventListener( "load", completed );
	jQuery.ready();
}

jQuery.ready.promise = function( obj ) {
	if ( !readyList ) {

		readyList = jQuery.Deferred();

		// Catch cases where $(document).ready() is called
		// after the browser event has already occurred.
		// Support: IE9-10 only
		// Older IE sometimes signals "interactive" too soon
		if ( document.readyState === "complete" ||
			( document.readyState !== "loading" && !document.documentElement.doScroll ) ) {

			// Handle it asynchronously to allow scripts the opportunity to delay ready
			window.setTimeout( jQuery.ready );

		} else {

			// Use the handy event callback
			document.addEventListener( "DOMContentLoaded", completed );

			// A fallback to window.onload, that will always work
			window.addEventListener( "load", completed );
		}
	}
	return readyList.promise( obj );
};

// Kick off the DOM ready check even if the user does not
jQuery.ready.promise();




// Multifunctional method to get and set values of a collection
// The value/s can optionally be executed if it's a function
var access = function( elems, fn, key, value, chainable, emptyGet, raw ) {
	var i = 0,
		len = elems.length,
		bulk = key == null;

	// Sets many values
	if ( jQuery.type( key ) === "object" ) {
		chainable = true;
		for ( i in key ) {
			access( elems, fn, i, key[ i ], true, emptyGet, raw );
		}

	// Sets one value
	} else if ( value !== undefined ) {
		chainable = true;

		if ( !jQuery.isFunction( value ) ) {
			raw = true;
		}

		if ( bulk ) {

			// Bulk operations run against the entire set
			if ( raw ) {
				fn.call( elems, value );
				fn = null;

			// ...except when executing function values
			} else {
				bulk = fn;
				fn = function( elem, key, value ) {
					return bulk.call( jQuery( elem ), value );
				};
			}
		}

		if ( fn ) {
			for ( ; i < len; i++ ) {
				fn(
					elems[ i ], key, raw ?
					value :
					value.call( elems[ i ], i, fn( elems[ i ], key ) )
				);
			}
		}
	}

	return chainable ?
		elems :

		// Gets
		bulk ?
			fn.call( elems ) :
			len ? fn( elems[ 0 ], key ) : emptyGet;
};
var acceptData = function( owner ) {

	// Accepts only:
	//  - Node
	//    - Node.ELEMENT_NODE
	//    - Node.DOCUMENT_NODE
	//  - Object
	//    - Any
	/* jshint -W018 */
	return owner.nodeType === 1 || owner.nodeType === 9 || !( +owner.nodeType );
};




function Data() {
	this.expando = jQuery.expando + Data.uid++;
}

Data.uid = 1;

Data.prototype = {

	register: function( owner, initial ) {
		var value = initial || {};

		// If it is a node unlikely to be stringify-ed or looped over
		// use plain assignment
		if ( owner.nodeType ) {
			owner[ this.expando ] = value;

		// Otherwise secure it in a non-enumerable, non-writable property
		// configurability must be true to allow the property to be
		// deleted with the delete operator
		} else {
			Object.defineProperty( owner, this.expando, {
				value: value,
				writable: true,
				configurable: true
			} );
		}
		return owner[ this.expando ];
	},
	cache: function( owner ) {

		// We can accept data for non-element nodes in modern browsers,
		// but we should not, see #8335.
		// Always return an empty object.
		if ( !acceptData( owner ) ) {
			return {};
		}

		// Check if the owner object already has a cache
		var value = owner[ this.expando ];

		// If not, create one
		if ( !value ) {
			value = {};

			// We can accept data for non-element nodes in modern browsers,
			// but we should not, see #8335.
			// Always return an empty object.
			if ( acceptData( owner ) ) {

				// If it is a node unlikely to be stringify-ed or looped over
				// use plain assignment
				if ( owner.nodeType ) {
					owner[ this.expando ] = value;

				// Otherwise secure it in a non-enumerable property
				// configurable must be true to allow the property to be
				// deleted when data is removed
				} else {
					Object.defineProperty( owner, this.expando, {
						value: value,
						configurable: true
					} );
				}
			}
		}

		return value;
	},
	set: function( owner, data, value ) {
		var prop,
			cache = this.cache( owner );

		// Handle: [ owner, key, value ] args
		if ( typeof data === "string" ) {
			cache[ data ] = value;

		// Handle: [ owner, { properties } ] args
		} else {

			// Copy the properties one-by-one to the cache object
			for ( prop in data ) {
				cache[ prop ] = data[ prop ];
			}
		}
		return cache;
	},
	get: function( owner, key ) {
		return key === undefined ?
			this.cache( owner ) :
			owner[ this.expando ] && owner[ this.expando ][ key ];
	},
	access: function( owner, key, value ) {
		var stored;

		// In cases where either:
		//
		//   1. No key was specified
		//   2. A string key was specified, but no value provided
		//
		// Take the "read" path and allow the get method to determine
		// which value to return, respectively either:
		//
		//   1. The entire cache object
		//   2. The data stored at the key
		//
		if ( key === undefined ||
				( ( key && typeof key === "string" ) && value === undefined ) ) {

			stored = this.get( owner, key );

			return stored !== undefined ?
				stored : this.get( owner, jQuery.camelCase( key ) );
		}

		// When the key is not a string, or both a key and value
		// are specified, set or extend (existing objects) with either:
		//
		//   1. An object of properties
		//   2. A key and value
		//
		this.set( owner, key, value );

		// Since the "set" path can have two possible entry points
		// return the expected data based on which path was taken[*]
		return value !== undefined ? value : key;
	},
	remove: function( owner, key ) {
		var i, name, camel,
			cache = owner[ this.expando ];

		if ( cache === undefined ) {
			return;
		}

		if ( key === undefined ) {
			this.register( owner );

		} else {

			// Support array or space separated string of keys
			if ( jQuery.isArray( key ) ) {

				// If "name" is an array of keys...
				// When data is initially created, via ("key", "val") signature,
				// keys will be converted to camelCase.
				// Since there is no way to tell _how_ a key was added, remove
				// both plain key and camelCase key. #12786
				// This will only penalize the array argument path.
				name = key.concat( key.map( jQuery.camelCase ) );
			} else {
				camel = jQuery.camelCase( key );

				// Try the string as a key before any manipulation
				if ( key in cache ) {
					name = [ key, camel ];
				} else {

					// If a key with the spaces exists, use it.
					// Otherwise, create an array by matching non-whitespace
					name = camel;
					name = name in cache ?
						[ name ] : ( name.match( rnotwhite ) || [] );
				}
			}

			i = name.length;

			while ( i-- ) {
				delete cache[ name[ i ] ];
			}
		}

		// Remove the expando if there's no more data
		if ( key === undefined || jQuery.isEmptyObject( cache ) ) {

			// Support: Chrome <= 35-45+
			// Webkit & Blink performance suffers when deleting properties
			// from DOM nodes, so set to undefined instead
			// https://code.google.com/p/chromium/issues/detail?id=378607
			if ( owner.nodeType ) {
				owner[ this.expando ] = undefined;
			} else {
				delete owner[ this.expando ];
			}
		}
	},
	hasData: function( owner ) {
		var cache = owner[ this.expando ];
		return cache !== undefined && !jQuery.isEmptyObject( cache );
	}
};
var dataPriv = new Data();

var dataUser = new Data();



//	Implementation Summary
//
//	1. Enforce API surface and semantic compatibility with 1.9.x branch
//	2. Improve the module's maintainability by reducing the storage
//		paths to a single mechanism.
//	3. Use the same single mechanism to support "private" and "user" data.
//	4. _Never_ expose "private" data to user code (TODO: Drop _data, _removeData)
//	5. Avoid exposing implementation details on user objects (eg. expando properties)
//	6. Provide a clear path for implementation upgrade to WeakMap in 2014

var rbrace = /^(?:\{[\w\W]*\}|\[[\w\W]*\])$/,
	rmultiDash = /[A-Z]/g;

function dataAttr( elem, key, data ) {
	var name;

	// If nothing was found internally, try to fetch any
	// data from the HTML5 data-* attribute
	if ( data === undefined && elem.nodeType === 1 ) {
		name = "data-" + key.replace( rmultiDash, "-$&" ).toLowerCase();
		data = elem.getAttribute( name );

		if ( typeof data === "string" ) {
			try {
				data = data === "true" ? true :
					data === "false" ? false :
					data === "null" ? null :

					// Only convert to a number if it doesn't change the string
					+data + "" === data ? +data :
					rbrace.test( data ) ? jQuery.parseJSON( data ) :
					data;
			} catch ( e ) {}

			// Make sure we set the data so it isn't changed later
			dataUser.set( elem, key, data );
		} else {
			data = undefined;
		}
	}
	return data;
}

jQuery.extend( {
	hasData: function( elem ) {
		return dataUser.hasData( elem ) || dataPriv.hasData( elem );
	},

	data: function( elem, name, data ) {
		return dataUser.access( elem, name, data );
	},

	removeData: function( elem, name ) {
		dataUser.remove( elem, name );
	},

	// TODO: Now that all calls to _data and _removeData have been replaced
	// with direct calls to dataPriv methods, these can be deprecated.
	_data: function( elem, name, data ) {
		return dataPriv.access( elem, name, data );
	},

	_removeData: function( elem, name ) {
		dataPriv.remove( elem, name );
	}
} );

jQuery.fn.extend( {
	data: function( key, value ) {
		var i, name, data,
			elem = this[ 0 ],
			attrs = elem && elem.attributes;

		// Gets all values
		if ( key === undefined ) {
			if ( this.length ) {
				data = dataUser.get( elem );

				if ( elem.nodeType === 1 && !dataPriv.get( elem, "hasDataAttrs" ) ) {
					i = attrs.length;
					while ( i-- ) {

						// Support: IE11+
						// The attrs elements can be null (#14894)
						if ( attrs[ i ] ) {
							name = attrs[ i ].name;
							if ( name.indexOf( "data-" ) === 0 ) {
								name = jQuery.camelCase( name.slice( 5 ) );
								dataAttr( elem, name, data[ name ] );
							}
						}
					}
					dataPriv.set( elem, "hasDataAttrs", true );
				}
			}

			return data;
		}

		// Sets multiple values
		if ( typeof key === "object" ) {
			return this.each( function() {
				dataUser.set( this, key );
			} );
		}

		return access( this, function( value ) {
			var data, camelKey;

			// The calling jQuery object (element matches) is not empty
			// (and therefore has an element appears at this[ 0 ]) and the
			// `value` parameter was not undefined. An empty jQuery object
			// will result in `undefined` for elem = this[ 0 ] which will
			// throw an exception if an attempt to read a data cache is made.
			if ( elem && value === undefined ) {

				// Attempt to get data from the cache
				// with the key as-is
				data = dataUser.get( elem, key ) ||

					// Try to find dashed key if it exists (gh-2779)
					// This is for 2.2.x only
					dataUser.get( elem, key.replace( rmultiDash, "-$&" ).toLowerCase() );

				if ( data !== undefined ) {
					return data;
				}

				camelKey = jQuery.camelCase( key );

				// Attempt to get data from the cache
				// with the key camelized
				data = dataUser.get( elem, camelKey );
				if ( data !== undefined ) {
					return data;
				}

				// Attempt to "discover" the data in
				// HTML5 custom data-* attrs
				data = dataAttr( elem, camelKey, undefined );
				if ( data !== undefined ) {
					return data;
				}

				// We tried really hard, but the data doesn't exist.
				return;
			}

			// Set the data...
			camelKey = jQuery.camelCase( key );
			this.each( function() {

				// First, attempt to store a copy or reference of any
				// data that might've been store with a camelCased key.
				var data = dataUser.get( this, camelKey );

				// For HTML5 data-* attribute interop, we have to
				// store property names with dashes in a camelCase form.
				// This might not apply to all properties...*
				dataUser.set( this, camelKey, value );

				// *... In the case of properties that might _actually_
				// have dashes, we need to also store a copy of that
				// unchanged property.
				if ( key.indexOf( "-" ) > -1 && data !== undefined ) {
					dataUser.set( this, key, value );
				}
			} );
		}, null, value, arguments.length > 1, null, true );
	},

	removeData: function( key ) {
		return this.each( function() {
			dataUser.remove( this, key );
		} );
	}
} );


jQuery.extend( {
	queue: function( elem, type, data ) {
		var queue;

		if ( elem ) {
			type = ( type || "fx" ) + "queue";
			queue = dataPriv.get( elem, type );

			// Speed up dequeue by getting out quickly if this is just a lookup
			if ( data ) {
				if ( !queue || jQuery.isArray( data ) ) {
					queue = dataPriv.access( elem, type, jQuery.makeArray( data ) );
				} else {
					queue.push( data );
				}
			}
			return queue || [];
		}
	},

	dequeue: function( elem, type ) {
		type = type || "fx";

		var queue = jQuery.queue( elem, type ),
			startLength = queue.length,
			fn = queue.shift(),
			hooks = jQuery._queueHooks( elem, type ),
			next = function() {
				jQuery.dequeue( elem, type );
			};

		// If the fx queue is dequeued, always remove the progress sentinel
		if ( fn === "inprogress" ) {
			fn = queue.shift();
			startLength--;
		}

		if ( fn ) {

			// Add a progress sentinel to prevent the fx queue from being
			// automatically dequeued
			if ( type === "fx" ) {
				queue.unshift( "inprogress" );
			}

			// Clear up the last queue stop function
			delete hooks.stop;
			fn.call( elem, next, hooks );
		}

		if ( !startLength && hooks ) {
			hooks.empty.fire();
		}
	},

	// Not public - generate a queueHooks object, or return the current one
	_queueHooks: function( elem, type ) {
		var key = type + "queueHooks";
		return dataPriv.get( elem, key ) || dataPriv.access( elem, key, {
			empty: jQuery.Callbacks( "once memory" ).add( function() {
				dataPriv.remove( elem, [ type + "queue", key ] );
			} )
		} );
	}
} );

jQuery.fn.extend( {
	queue: function( type, data ) {
		var setter = 2;

		if ( typeof type !== "string" ) {
			data = type;
			type = "fx";
			setter--;
		}

		if ( arguments.length < setter ) {
			return jQuery.queue( this[ 0 ], type );
		}

		return data === undefined ?
			this :
			this.each( function() {
				var queue = jQuery.queue( this, type, data );

				// Ensure a hooks for this queue
				jQuery._queueHooks( this, type );

				if ( type === "fx" && queue[ 0 ] !== "inprogress" ) {
					jQuery.dequeue( this, type );
				}
			} );
	},
	dequeue: function( type ) {
		return this.each( function() {
			jQuery.dequeue( this, type );
		} );
	},
	clearQueue: function( type ) {
		return this.queue( type || "fx", [] );
	},

	// Get a promise resolved when queues of a certain type
	// are emptied (fx is the type by default)
	promise: function( type, obj ) {
		var tmp,
			count = 1,
			defer = jQuery.Deferred(),
			elements = this,
			i = this.length,
			resolve = function() {
				if ( !( --count ) ) {
					defer.resolveWith( elements, [ elements ] );
				}
			};

		if ( typeof type !== "string" ) {
			obj = type;
			type = undefined;
		}
		type = type || "fx";

		while ( i-- ) {
			tmp = dataPriv.get( elements[ i ], type + "queueHooks" );
			if ( tmp && tmp.empty ) {
				count++;
				tmp.empty.add( resolve );
			}
		}
		resolve();
		return defer.promise( obj );
	}
} );
var pnum = ( /[+-]?(?:\d*\.|)\d+(?:[eE][+-]?\d+|)/ ).source;

var rcssNum = new RegExp( "^(?:([+-])=|)(" + pnum + ")([a-z%]*)$", "i" );


var cssExpand = [ "Top", "Right", "Bottom", "Left" ];

var isHidden = function( elem, el ) {

		// isHidden might be called from jQuery#filter function;
		// in that case, element will be second argument
		elem = el || elem;
		return jQuery.css( elem, "display" ) === "none" ||
			!jQuery.contains( elem.ownerDocument, elem );
	};



function adjustCSS( elem, prop, valueParts, tween ) {
	var adjusted,
		scale = 1,
		maxIterations = 20,
		currentValue = tween ?
			function() { return tween.cur(); } :
			function() { return jQuery.css( elem, prop, "" ); },
		initial = currentValue(),
		unit = valueParts && valueParts[ 3 ] || ( jQuery.cssNumber[ prop ] ? "" : "px" ),

		// Starting value computation is required for potential unit mismatches
		initialInUnit = ( jQuery.cssNumber[ prop ] || unit !== "px" && +initial ) &&
			rcssNum.exec( jQuery.css( elem, prop ) );

	if ( initialInUnit && initialInUnit[ 3 ] !== unit ) {

		// Trust units reported by jQuery.css
		unit = unit || initialInUnit[ 3 ];

		// Make sure we update the tween properties later on
		valueParts = valueParts || [];

		// Iteratively approximate from a nonzero starting point
		initialInUnit = +initial || 1;

		do {

			// If previous iteration zeroed out, double until we get *something*.
			// Use string for doubling so we don't accidentally see scale as unchanged below
			scale = scale || ".5";

			// Adjust and apply
			initialInUnit = initialInUnit / scale;
			jQuery.style( elem, prop, initialInUnit + unit );

		// Update scale, tolerating zero or NaN from tween.cur()
		// Break the loop if scale is unchanged or perfect, or if we've just had enough.
		} while (
			scale !== ( scale = currentValue() / initial ) && scale !== 1 && --maxIterations
		);
	}

	if ( valueParts ) {
		initialInUnit = +initialInUnit || +initial || 0;

		// Apply relative offset (+=/-=) if specified
		adjusted = valueParts[ 1 ] ?
			initialInUnit + ( valueParts[ 1 ] + 1 ) * valueParts[ 2 ] :
			+valueParts[ 2 ];
		if ( tween ) {
			tween.unit = unit;
			tween.start = initialInUnit;
			tween.end = adjusted;
		}
	}
	return adjusted;
}
var rcheckableType = ( /^(?:checkbox|radio)$/i );

var rtagName = ( /<([\w:-]+)/ );

var rscriptType = ( /^$|\/(?:java|ecma)script/i );



// We have to close these tags to support XHTML (#13200)
var wrapMap = {

	// Support: IE9
	option: [ 1, "<select multiple='multiple'>", "</select>" ],

	// XHTML parsers do not magically insert elements in the
	// same way that tag soup parsers do. So we cannot shorten
	// this by omitting <tbody> or other required elements.
	thead: [ 1, "<table>", "</table>" ],
	col: [ 2, "<table><colgroup>", "</colgroup></table>" ],
	tr: [ 2, "<table><tbody>", "</tbody></table>" ],
	td: [ 3, "<table><tbody><tr>", "</tr></tbody></table>" ],

	_default: [ 0, "", "" ]
};

// Support: IE9
wrapMap.optgroup = wrapMap.option;

wrapMap.tbody = wrapMap.tfoot = wrapMap.colgroup = wrapMap.caption = wrapMap.thead;
wrapMap.th = wrapMap.td;


function getAll( context, tag ) {

	// Support: IE9-11+
	// Use typeof to avoid zero-argument method invocation on host objects (#15151)
	var ret = typeof context.getElementsByTagName !== "undefined" ?
			context.getElementsByTagName( tag || "*" ) :
			typeof context.querySelectorAll !== "undefined" ?
				context.querySelectorAll( tag || "*" ) :
			[];

	return tag === undefined || tag && jQuery.nodeName( context, tag ) ?
		jQuery.merge( [ context ], ret ) :
		ret;
}


// Mark scripts as having already been evaluated
function setGlobalEval( elems, refElements ) {
	var i = 0,
		l = elems.length;

	for ( ; i < l; i++ ) {
		dataPriv.set(
			elems[ i ],
			"globalEval",
			!refElements || dataPriv.get( refElements[ i ], "globalEval" )
		);
	}
}


var rhtml = /<|&#?\w+;/;

function buildFragment( elems, context, scripts, selection, ignored ) {
	var elem, tmp, tag, wrap, contains, j,
		fragment = context.createDocumentFragment(),
		nodes = [],
		i = 0,
		l = elems.length;

	for ( ; i < l; i++ ) {
		elem = elems[ i ];

		if ( elem || elem === 0 ) {

			// Add nodes directly
			if ( jQuery.type( elem ) === "object" ) {

				// Support: Android<4.1, PhantomJS<2
				// push.apply(_, arraylike) throws on ancient WebKit
				jQuery.merge( nodes, elem.nodeType ? [ elem ] : elem );

			// Convert non-html into a text node
			} else if ( !rhtml.test( elem ) ) {
				nodes.push( context.createTextNode( elem ) );

			// Convert html into DOM nodes
			} else {
				tmp = tmp || fragment.appendChild( context.createElement( "div" ) );

				// Deserialize a standard representation
				tag = ( rtagName.exec( elem ) || [ "", "" ] )[ 1 ].toLowerCase();
				wrap = wrapMap[ tag ] || wrapMap._default;
				tmp.innerHTML = wrap[ 1 ] + jQuery.htmlPrefilter( elem ) + wrap[ 2 ];

				// Descend through wrappers to the right content
				j = wrap[ 0 ];
				while ( j-- ) {
					tmp = tmp.lastChild;
				}

				// Support: Android<4.1, PhantomJS<2
				// push.apply(_, arraylike) throws on ancient WebKit
				jQuery.merge( nodes, tmp.childNodes );

				// Remember the top-level container
				tmp = fragment.firstChild;

				// Ensure the created nodes are orphaned (#12392)
				tmp.textContent = "";
			}
		}
	}

	// Remove wrapper from fragment
	fragment.textContent = "";

	i = 0;
	while ( ( elem = nodes[ i++ ] ) ) {

		// Skip elements already in the context collection (trac-4087)
		if ( selection && jQuery.inArray( elem, selection ) > -1 ) {
			if ( ignored ) {
				ignored.push( elem );
			}
			continue;
		}

		contains = jQuery.contains( elem.ownerDocument, elem );

		// Append to fragment
		tmp = getAll( fragment.appendChild( elem ), "script" );

		// Preserve script evaluation history
		if ( contains ) {
			setGlobalEval( tmp );
		}

		// Capture executables
		if ( scripts ) {
			j = 0;
			while ( ( elem = tmp[ j++ ] ) ) {
				if ( rscriptType.test( elem.type || "" ) ) {
					scripts.push( elem );
				}
			}
		}
	}

	return fragment;
}


( function() {
	var fragment = document.createDocumentFragment(),
		div = fragment.appendChild( document.createElement( "div" ) ),
		input = document.createElement( "input" );

	// Support: Android 4.0-4.3, Safari<=5.1
	// Check state lost if the name is set (#11217)
	// Support: Windows Web Apps (WWA)
	// `name` and `type` must use .setAttribute for WWA (#14901)
	input.setAttribute( "type", "radio" );
	input.setAttribute( "checked", "checked" );
	input.setAttribute( "name", "t" );

	div.appendChild( input );

	// Support: Safari<=5.1, Android<4.2
	// Older WebKit doesn't clone checked state correctly in fragments
	support.checkClone = div.cloneNode( true ).cloneNode( true ).lastChild.checked;

	// Support: IE<=11+
	// Make sure textarea (and checkbox) defaultValue is properly cloned
	div.innerHTML = "<textarea>x</textarea>";
	support.noCloneChecked = !!div.cloneNode( true ).lastChild.defaultValue;
} )();


var
	rkeyEvent = /^key/,
	rmouseEvent = /^(?:mouse|pointer|contextmenu|drag|drop)|click/,
	rtypenamespace = /^([^.]*)(?:\.(.+)|)/;

function returnTrue() {
	return true;
}

function returnFalse() {
	return false;
}

// Support: IE9
// See #13393 for more info
function safeActiveElement() {
	try {
		return document.activeElement;
	} catch ( err ) { }
}

function on( elem, types, selector, data, fn, one ) {
	var origFn, type;

	// Types can be a map of types/handlers
	if ( typeof types === "object" ) {

		// ( types-Object, selector, data )
		if ( typeof selector !== "string" ) {

			// ( types-Object, data )
			data = data || selector;
			selector = undefined;
		}
		for ( type in types ) {
			on( elem, type, selector, data, types[ type ], one );
		}
		return elem;
	}

	if ( data == null && fn == null ) {

		// ( types, fn )
		fn = selector;
		data = selector = undefined;
	} else if ( fn == null ) {
		if ( typeof selector === "string" ) {

			// ( types, selector, fn )
			fn = data;
			data = undefined;
		} else {

			// ( types, data, fn )
			fn = data;
			data = selector;
			selector = undefined;
		}
	}
	if ( fn === false ) {
		fn = returnFalse;
	} else if ( !fn ) {
		return this;
	}

	if ( one === 1 ) {
		origFn = fn;
		fn = function( event ) {

			// Can use an empty set, since event contains the info
			jQuery().off( event );
			return origFn.apply( this, arguments );
		};

		// Use same guid so caller can remove using origFn
		fn.guid = origFn.guid || ( origFn.guid = jQuery.guid++ );
	}
	return elem.each( function() {
		jQuery.event.add( this, types, fn, data, selector );
	} );
}

/*
 * Helper functions for managing events -- not part of the public interface.
 * Props to Dean Edwards' addEvent library for many of the ideas.
 */
jQuery.event = {

	global: {},

	add: function( elem, types, handler, data, selector ) {

		var handleObjIn, eventHandle, tmp,
			events, t, handleObj,
			special, handlers, type, namespaces, origType,
			elemData = dataPriv.get( elem );

		// Don't attach events to noData or text/comment nodes (but allow plain objects)
		if ( !elemData ) {
			return;
		}

		// Caller can pass in an object of custom data in lieu of the handler
		if ( handler.handler ) {
			handleObjIn = handler;
			handler = handleObjIn.handler;
			selector = handleObjIn.selector;
		}

		// Make sure that the handler has a unique ID, used to find/remove it later
		if ( !handler.guid ) {
			handler.guid = jQuery.guid++;
		}

		// Init the element's event structure and main handler, if this is the first
		if ( !( events = elemData.events ) ) {
			events = elemData.events = {};
		}
		if ( !( eventHandle = elemData.handle ) ) {
			eventHandle = elemData.handle = function( e ) {

				// Discard the second event of a jQuery.event.trigger() and
				// when an event is called after a page has unloaded
				return typeof jQuery !== "undefined" && jQuery.event.triggered !== e.type ?
					jQuery.event.dispatch.apply( elem, arguments ) : undefined;
			};
		}

		// Handle multiple events separated by a space
		types = ( types || "" ).match( rnotwhite ) || [ "" ];
		t = types.length;
		while ( t-- ) {
			tmp = rtypenamespace.exec( types[ t ] ) || [];
			type = origType = tmp[ 1 ];
			namespaces = ( tmp[ 2 ] || "" ).split( "." ).sort();

			// There *must* be a type, no attaching namespace-only handlers
			if ( !type ) {
				continue;
			}

			// If event changes its type, use the special event handlers for the changed type
			special = jQuery.event.special[ type ] || {};

			// If selector defined, determine special event api type, otherwise given type
			type = ( selector ? special.delegateType : special.bindType ) || type;

			// Update special based on newly reset type
			special = jQuery.event.special[ type ] || {};

			// handleObj is passed to all event handlers
			handleObj = jQuery.extend( {
				type: type,
				origType: origType,
				data: data,
				handler: handler,
				guid: handler.guid,
				selector: selector,
				needsContext: selector && jQuery.expr.match.needsContext.test( selector ),
				namespace: namespaces.join( "." )
			}, handleObjIn );

			// Init the event handler queue if we're the first
			if ( !( handlers = events[ type ] ) ) {
				handlers = events[ type ] = [];
				handlers.delegateCount = 0;

				// Only use addEventListener if the special events handler returns false
				if ( !special.setup ||
					special.setup.call( elem, data, namespaces, eventHandle ) === false ) {

					if ( elem.addEventListener ) {
						elem.addEventListener( type, eventHandle );
					}
				}
			}

			if ( special.add ) {
				special.add.call( elem, handleObj );

				if ( !handleObj.handler.guid ) {
					handleObj.handler.guid = handler.guid;
				}
			}

			// Add to the element's handler list, delegates in front
			if ( selector ) {
				handlers.splice( handlers.delegateCount++, 0, handleObj );
			} else {
				handlers.push( handleObj );
			}

			// Keep track of which events have ever been used, for event optimization
			jQuery.event.global[ type ] = true;
		}

	},

	// Detach an event or set of events from an element
	remove: function( elem, types, handler, selector, mappedTypes ) {

		var j, origCount, tmp,
			events, t, handleObj,
			special, handlers, type, namespaces, origType,
			elemData = dataPriv.hasData( elem ) && dataPriv.get( elem );

		if ( !elemData || !( events = elemData.events ) ) {
			return;
		}

		// Once for each type.namespace in types; type may be omitted
		types = ( types || "" ).match( rnotwhite ) || [ "" ];
		t = types.length;
		while ( t-- ) {
			tmp = rtypenamespace.exec( types[ t ] ) || [];
			type = origType = tmp[ 1 ];
			namespaces = ( tmp[ 2 ] || "" ).split( "." ).sort();

			// Unbind all events (on this namespace, if provided) for the element
			if ( !type ) {
				for ( type in events ) {
					jQuery.event.remove( elem, type + types[ t ], handler, selector, true );
				}
				continue;
			}

			special = jQuery.event.special[ type ] || {};
			type = ( selector ? special.delegateType : special.bindType ) || type;
			handlers = events[ type ] || [];
			tmp = tmp[ 2 ] &&
				new RegExp( "(^|\\.)" + namespaces.join( "\\.(?:.*\\.|)" ) + "(\\.|$)" );

			// Remove matching events
			origCount = j = handlers.length;
			while ( j-- ) {
				handleObj = handlers[ j ];

				if ( ( mappedTypes || origType === handleObj.origType ) &&
					( !handler || handler.guid === handleObj.guid ) &&
					( !tmp || tmp.test( handleObj.namespace ) ) &&
					( !selector || selector === handleObj.selector ||
						selector === "**" && handleObj.selector ) ) {
					handlers.splice( j, 1 );

					if ( handleObj.selector ) {
						handlers.delegateCount--;
					}
					if ( special.remove ) {
						special.remove.call( elem, handleObj );
					}
				}
			}

			// Remove generic event handler if we removed something and no more handlers exist
			// (avoids potential for endless recursion during removal of special event handlers)
			if ( origCount && !handlers.length ) {
				if ( !special.teardown ||
					special.teardown.call( elem, namespaces, elemData.handle ) === false ) {

					jQuery.removeEvent( elem, type, elemData.handle );
				}

				delete events[ type ];
			}
		}

		// Remove data and the expando if it's no longer used
		if ( jQuery.isEmptyObject( events ) ) {
			dataPriv.remove( elem, "handle events" );
		}
	},

	dispatch: function( event ) {

		// Make a writable jQuery.Event from the native event object
		event = jQuery.event.fix( event );

		var i, j, ret, matched, handleObj,
			handlerQueue = [],
			args = slice.call( arguments ),
			handlers = ( dataPriv.get( this, "events" ) || {} )[ event.type ] || [],
			special = jQuery.event.special[ event.type ] || {};

		// Use the fix-ed jQuery.Event rather than the (read-only) native event
		args[ 0 ] = event;
		event.delegateTarget = this;

		// Call the preDispatch hook for the mapped type, and let it bail if desired
		if ( special.preDispatch && special.preDispatch.call( this, event ) === false ) {
			return;
		}

		// Determine handlers
		handlerQueue = jQuery.event.handlers.call( this, event, handlers );

		// Run delegates first; they may want to stop propagation beneath us
		i = 0;
		while ( ( matched = handlerQueue[ i++ ] ) && !event.isPropagationStopped() ) {
			event.currentTarget = matched.elem;

			j = 0;
			while ( ( handleObj = matched.handlers[ j++ ] ) &&
				!event.isImmediatePropagationStopped() ) {

				// Triggered event must either 1) have no namespace, or 2) have namespace(s)
				// a subset or equal to those in the bound event (both can have no namespace).
				if ( !event.rnamespace || event.rnamespace.test( handleObj.namespace ) ) {

					event.handleObj = handleObj;
					event.data = handleObj.data;

					ret = ( ( jQuery.event.special[ handleObj.origType ] || {} ).handle ||
						handleObj.handler ).apply( matched.elem, args );

					if ( ret !== undefined ) {
						if ( ( event.result = ret ) === false ) {
							event.preventDefault();
							event.stopPropagation();
						}
					}
				}
			}
		}

		// Call the postDispatch hook for the mapped type
		if ( special.postDispatch ) {
			special.postDispatch.call( this, event );
		}

		return event.result;
	},

	handlers: function( event, handlers ) {
		var i, matches, sel, handleObj,
			handlerQueue = [],
			delegateCount = handlers.delegateCount,
			cur = event.target;

		// Support (at least): Chrome, IE9
		// Find delegate handlers
		// Black-hole SVG <use> instance trees (#13180)
		//
		// Support: Firefox<=42+
		// Avoid non-left-click in FF but don't block IE radio events (#3861, gh-2343)
		if ( delegateCount && cur.nodeType &&
			( event.type !== "click" || isNaN( event.button ) || event.button < 1 ) ) {

			for ( ; cur !== this; cur = cur.parentNode || this ) {

				// Don't check non-elements (#13208)
				// Don't process clicks on disabled elements (#6911, #8165, #11382, #11764)
				if ( cur.nodeType === 1 && ( cur.disabled !== true || event.type !== "click" ) ) {
					matches = [];
					for ( i = 0; i < delegateCount; i++ ) {
						handleObj = handlers[ i ];

						// Don't conflict with Object.prototype properties (#13203)
						sel = handleObj.selector + " ";

						if ( matches[ sel ] === undefined ) {
							matches[ sel ] = handleObj.needsContext ?
								jQuery( sel, this ).index( cur ) > -1 :
								jQuery.find( sel, this, null, [ cur ] ).length;
						}
						if ( matches[ sel ] ) {
							matches.push( handleObj );
						}
					}
					if ( matches.length ) {
						handlerQueue.push( { elem: cur, handlers: matches } );
					}
				}
			}
		}

		// Add the remaining (directly-bound) handlers
		if ( delegateCount < handlers.length ) {
			handlerQueue.push( { elem: this, handlers: handlers.slice( delegateCount ) } );
		}

		return handlerQueue;
	},

	// Includes some event props shared by KeyEvent and MouseEvent
	props: ( "altKey bubbles cancelable ctrlKey currentTarget detail eventPhase " +
		"metaKey relatedTarget shiftKey target timeStamp view which" ).split( " " ),

	fixHooks: {},

	keyHooks: {
		props: "char charCode key keyCode".split( " " ),
		filter: function( event, original ) {

			// Add which for key events
			if ( event.which == null ) {
				event.which = original.charCode != null ? original.charCode : original.keyCode;
			}

			return event;
		}
	},

	mouseHooks: {
		props: ( "button buttons clientX clientY offsetX offsetY pageX pageY " +
			"screenX screenY toElement" ).split( " " ),
		filter: function( event, original ) {
			var eventDoc, doc, body,
				button = original.button;

			// Calculate pageX/Y if missing and clientX/Y available
			if ( event.pageX == null && original.clientX != null ) {
				eventDoc = event.target.ownerDocument || document;
				doc = eventDoc.documentElement;
				body = eventDoc.body;

				event.pageX = original.clientX +
					( doc && doc.scrollLeft || body && body.scrollLeft || 0 ) -
					( doc && doc.clientLeft || body && body.clientLeft || 0 );
				event.pageY = original.clientY +
					( doc && doc.scrollTop  || body && body.scrollTop  || 0 ) -
					( doc && doc.clientTop  || body && body.clientTop  || 0 );
			}

			// Add which for click: 1 === left; 2 === middle; 3 === right
			// Note: button is not normalized, so don't use it
			if ( !event.which && button !== undefined ) {
				event.which = ( button & 1 ? 1 : ( button & 2 ? 3 : ( button & 4 ? 2 : 0 ) ) );
			}

			return event;
		}
	},

	fix: function( event ) {
		if ( event[ jQuery.expando ] ) {
			return event;
		}

		// Create a writable copy of the event object and normalize some properties
		var i, prop, copy,
			type = event.type,
			originalEvent = event,
			fixHook = this.fixHooks[ type ];

		if ( !fixHook ) {
			this.fixHooks[ type ] = fixHook =
				rmouseEvent.test( type ) ? this.mouseHooks :
				rkeyEvent.test( type ) ? this.keyHooks :
				{};
		}
		copy = fixHook.props ? this.props.concat( fixHook.props ) : this.props;

		event = new jQuery.Event( originalEvent );

		i = copy.length;
		while ( i-- ) {
			prop = copy[ i ];
			event[ prop ] = originalEvent[ prop ];
		}

		// Support: Cordova 2.5 (WebKit) (#13255)
		// All events should have a target; Cordova deviceready doesn't
		if ( !event.target ) {
			event.target = document;
		}

		// Support: Safari 6.0+, Chrome<28
		// Target should not be a text node (#504, #13143)
		if ( event.target.nodeType === 3 ) {
			event.target = event.target.parentNode;
		}

		return fixHook.filter ? fixHook.filter( event, originalEvent ) : event;
	},

	special: {
		load: {

			// Prevent triggered image.load events from bubbling to window.load
			noBubble: true
		},
		focus: {

			// Fire native event if possible so blur/focus sequence is correct
			trigger: function() {
				if ( this !== safeActiveElement() && this.focus ) {
					this.focus();
					return false;
				}
			},
			delegateType: "focusin"
		},
		blur: {
			trigger: function() {
				if ( this === safeActiveElement() && this.blur ) {
					this.blur();
					return false;
				}
			},
			delegateType: "focusout"
		},
		click: {

			// For checkbox, fire native event so checked state will be right
			trigger: function() {
				if ( this.type === "checkbox" && this.click && jQuery.nodeName( this, "input" ) ) {
					this.click();
					return false;
				}
			},

			// For cross-browser consistency, don't fire native .click() on links
			_default: function( event ) {
				return jQuery.nodeName( event.target, "a" );
			}
		},

		beforeunload: {
			postDispatch: function( event ) {

				// Support: Firefox 20+
				// Firefox doesn't alert if the returnValue field is not set.
				if ( event.result !== undefined && event.originalEvent ) {
					event.originalEvent.returnValue = event.result;
				}
			}
		}
	}
};

jQuery.removeEvent = function( elem, type, handle ) {

	// This "if" is needed for plain objects
	if ( elem.removeEventListener ) {
		elem.removeEventListener( type, handle );
	}
};

jQuery.Event = function( src, props ) {

	// Allow instantiation without the 'new' keyword
	if ( !( this instanceof jQuery.Event ) ) {
		return new jQuery.Event( src, props );
	}

	// Event object
	if ( src && src.type ) {
		this.originalEvent = src;
		this.type = src.type;

		// Events bubbling up the document may have been marked as prevented
		// by a handler lower down the tree; reflect the correct value.
		this.isDefaultPrevented = src.defaultPrevented ||
				src.defaultPrevented === undefined &&

				// Support: Android<4.0
				src.returnValue === false ?
			returnTrue :
			returnFalse;

	// Event type
	} else {
		this.type = src;
	}

	// Put explicitly provided properties onto the event object
	if ( props ) {
		jQuery.extend( this, props );
	}

	// Create a timestamp if incoming event doesn't have one
	this.timeStamp = src && src.timeStamp || jQuery.now();

	// Mark it as fixed
	this[ jQuery.expando ] = true;
};

// jQuery.Event is based on DOM3 Events as specified by the ECMAScript Language Binding
// http://www.w3.org/TR/2003/WD-DOM-Level-3-Events-20030331/ecma-script-binding.html
jQuery.Event.prototype = {
	constructor: jQuery.Event,
	isDefaultPrevented: returnFalse,
	isPropagationStopped: returnFalse,
	isImmediatePropagationStopped: returnFalse,

	preventDefault: function() {
		var e = this.originalEvent;

		this.isDefaultPrevented = returnTrue;

		if ( e ) {
			e.preventDefault();
		}
	},
	stopPropagation: function() {
		var e = this.originalEvent;

		this.isPropagationStopped = returnTrue;

		if ( e ) {
			e.stopPropagation();
		}
	},
	stopImmediatePropagation: function() {
		var e = this.originalEvent;

		this.isImmediatePropagationStopped = returnTrue;

		if ( e ) {
			e.stopImmediatePropagation();
		}

		this.stopPropagation();
	}
};

// Create mouseenter/leave events using mouseover/out and event-time checks
// so that event delegation works in jQuery.
// Do the same for pointerenter/pointerleave and pointerover/pointerout
//
// Support: Safari 7 only
// Safari sends mouseenter too often; see:
// https://code.google.com/p/chromium/issues/detail?id=470258
// for the description of the bug (it existed in older Chrome versions as well).
jQuery.each( {
	mouseenter: "mouseover",
	mouseleave: "mouseout",
	pointerenter: "pointerover",
	pointerleave: "pointerout"
}, function( orig, fix ) {
	jQuery.event.special[ orig ] = {
		delegateType: fix,
		bindType: fix,

		handle: function( event ) {
			var ret,
				target = this,
				related = event.relatedTarget,
				handleObj = event.handleObj;

			// For mouseenter/leave call the handler if related is outside the target.
			// NB: No relatedTarget if the mouse left/entered the browser window
			if ( !related || ( related !== target && !jQuery.contains( target, related ) ) ) {
				event.type = handleObj.origType;
				ret = handleObj.handler.apply( this, arguments );
				event.type = fix;
			}
			return ret;
		}
	};
} );

jQuery.fn.extend( {
	on: function( types, selector, data, fn ) {
		return on( this, types, selector, data, fn );
	},
	one: function( types, selector, data, fn ) {
		return on( this, types, selector, data, fn, 1 );
	},
	off: function( types, selector, fn ) {
		var handleObj, type;
		if ( types && types.preventDefault && types.handleObj ) {

			// ( event )  dispatched jQuery.Event
			handleObj = types.handleObj;
			jQuery( types.delegateTarget ).off(
				handleObj.namespace ?
					handleObj.origType + "." + handleObj.namespace :
					handleObj.origType,
				handleObj.selector,
				handleObj.handler
			);
			return this;
		}
		if ( typeof types === "object" ) {

			// ( types-object [, selector] )
			for ( type in types ) {
				this.off( type, selector, types[ type ] );
			}
			return this;
		}
		if ( selector === false || typeof selector === "function" ) {

			// ( types [, fn] )
			fn = selector;
			selector = undefined;
		}
		if ( fn === false ) {
			fn = returnFalse;
		}
		return this.each( function() {
			jQuery.event.remove( this, types, fn, selector );
		} );
	}
} );


var
	rxhtmlTag = /<(?!area|br|col|embed|hr|img|input|link|meta|param)(([\w:-]+)[^>]*)\/>/gi,

	// Support: IE 10-11, Edge 10240+
	// In IE/Edge using regex groups here causes severe slowdowns.
	// See https://connect.microsoft.com/IE/feedback/details/1736512/
	rnoInnerhtml = /<script|<style|<link/i,

	// checked="checked" or checked
	rchecked = /checked\s*(?:[^=]|=\s*.checked.)/i,
	rscriptTypeMasked = /^true\/(.*)/,
	rcleanScript = /^\s*<!(?:\[CDATA\[|--)|(?:\]\]|--)>\s*$/g;

function manipulationTarget( elem, content ) {
	if ( jQuery.nodeName( elem, "table" ) &&
		jQuery.nodeName( content.nodeType !== 11 ? content : content.firstChild, "tr" ) ) {

		return elem.getElementsByTagName( "tbody" )[ 0 ] || elem;
	}

	return elem;
}

// Replace/restore the type attribute of script elements for safe DOM manipulation
function disableScript( elem ) {
	elem.type = ( elem.getAttribute( "type" ) !== null ) + "/" + elem.type;
	return elem;
}
function restoreScript( elem ) {
	var match = rscriptTypeMasked.exec( elem.type );

	if ( match ) {
		elem.type = match[ 1 ];
	} else {
		elem.removeAttribute( "type" );
	}

	return elem;
}

function cloneCopyEvent( src, dest ) {
	var i, l, type, pdataOld, pdataCur, udataOld, udataCur, events;

	if ( dest.nodeType !== 1 ) {
		return;
	}

	// 1. Copy private data: events, handlers, etc.
	if ( dataPriv.hasData( src ) ) {
		pdataOld = dataPriv.access( src );
		pdataCur = dataPriv.set( dest, pdataOld );
		events = pdataOld.events;

		if ( events ) {
			delete pdataCur.handle;
			pdataCur.events = {};

			for ( type in events ) {
				for ( i = 0, l = events[ type ].length; i < l; i++ ) {
					jQuery.event.add( dest, type, events[ type ][ i ] );
				}
			}
		}
	}

	// 2. Copy user data
	if ( dataUser.hasData( src ) ) {
		udataOld = dataUser.access( src );
		udataCur = jQuery.extend( {}, udataOld );

		dataUser.set( dest, udataCur );
	}
}

// Fix IE bugs, see support tests
function fixInput( src, dest ) {
	var nodeName = dest.nodeName.toLowerCase();

	// Fails to persist the checked state of a cloned checkbox or radio button.
	if ( nodeName === "input" && rcheckableType.test( src.type ) ) {
		dest.checked = src.checked;

	// Fails to return the selected option to the default selected state when cloning options
	} else if ( nodeName === "input" || nodeName === "textarea" ) {
		dest.defaultValue = src.defaultValue;
	}
}

function domManip( collection, args, callback, ignored ) {

	// Flatten any nested arrays
	args = concat.apply( [], args );

	var fragment, first, scripts, hasScripts, node, doc,
		i = 0,
		l = collection.length,
		iNoClone = l - 1,
		value = args[ 0 ],
		isFunction = jQuery.isFunction( value );

	// We can't cloneNode fragments that contain checked, in WebKit
	if ( isFunction ||
			( l > 1 && typeof value === "string" &&
				!support.checkClone && rchecked.test( value ) ) ) {
		return collection.each( function( index ) {
			var self = collection.eq( index );
			if ( isFunction ) {
				args[ 0 ] = value.call( this, index, self.html() );
			}
			domManip( self, args, callback, ignored );
		} );
	}

	if ( l ) {
		fragment = buildFragment( args, collection[ 0 ].ownerDocument, false, collection, ignored );
		first = fragment.firstChild;

		if ( fragment.childNodes.length === 1 ) {
			fragment = first;
		}

		// Require either new content or an interest in ignored elements to invoke the callback
		if ( first || ignored ) {
			scripts = jQuery.map( getAll( fragment, "script" ), disableScript );
			hasScripts = scripts.length;

			// Use the original fragment for the last item
			// instead of the first because it can end up
			// being emptied incorrectly in certain situations (#8070).
			for ( ; i < l; i++ ) {
				node = fragment;

				if ( i !== iNoClone ) {
					node = jQuery.clone( node, true, true );

					// Keep references to cloned scripts for later restoration
					if ( hasScripts ) {

						// Support: Android<4.1, PhantomJS<2
						// push.apply(_, arraylike) throws on ancient WebKit
						jQuery.merge( scripts, getAll( node, "script" ) );
					}
				}

				callback.call( collection[ i ], node, i );
			}

			if ( hasScripts ) {
				doc = scripts[ scripts.length - 1 ].ownerDocument;

				// Reenable scripts
				jQuery.map( scripts, restoreScript );

				// Evaluate executable scripts on first document insertion
				for ( i = 0; i < hasScripts; i++ ) {
					node = scripts[ i ];
					if ( rscriptType.test( node.type || "" ) &&
						!dataPriv.access( node, "globalEval" ) &&
						jQuery.contains( doc, node ) ) {

						if ( node.src ) {

							// Optional AJAX dependency, but won't run scripts if not present
							if ( jQuery._evalUrl ) {
								jQuery._evalUrl( node.src );
							}
						} else {
							jQuery.globalEval( node.textContent.replace( rcleanScript, "" ) );
						}
					}
				}
			}
		}
	}

	return collection;
}

function remove( elem, selector, keepData ) {
	var node,
		nodes = selector ? jQuery.filter( selector, elem ) : elem,
		i = 0;

	for ( ; ( node = nodes[ i ] ) != null; i++ ) {
		if ( !keepData && node.nodeType === 1 ) {
			jQuery.cleanData( getAll( node ) );
		}

		if ( node.parentNode ) {
			if ( keepData && jQuery.contains( node.ownerDocument, node ) ) {
				setGlobalEval( getAll( node, "script" ) );
			}
			node.parentNode.removeChild( node );
		}
	}

	return elem;
}

jQuery.extend( {
	htmlPrefilter: function( html ) {
		return html.replace( rxhtmlTag, "<$1></$2>" );
	},

	clone: function( elem, dataAndEvents, deepDataAndEvents ) {
		var i, l, srcElements, destElements,
			clone = elem.cloneNode( true ),
			inPage = jQuery.contains( elem.ownerDocument, elem );

		// Fix IE cloning issues
		if ( !support.noCloneChecked && ( elem.nodeType === 1 || elem.nodeType === 11 ) &&
				!jQuery.isXMLDoc( elem ) ) {

			// We eschew Sizzle here for performance reasons: http://jsperf.com/getall-vs-sizzle/2
			destElements = getAll( clone );
			srcElements = getAll( elem );

			for ( i = 0, l = srcElements.length; i < l; i++ ) {
				fixInput( srcElements[ i ], destElements[ i ] );
			}
		}

		// Copy the events from the original to the clone
		if ( dataAndEvents ) {
			if ( deepDataAndEvents ) {
				srcElements = srcElements || getAll( elem );
				destElements = destElements || getAll( clone );

				for ( i = 0, l = srcElements.length; i < l; i++ ) {
					cloneCopyEvent( srcElements[ i ], destElements[ i ] );
				}
			} else {
				cloneCopyEvent( elem, clone );
			}
		}

		// Preserve script evaluation history
		destElements = getAll( clone, "script" );
		if ( destElements.length > 0 ) {
			setGlobalEval( destElements, !inPage && getAll( elem, "script" ) );
		}

		// Return the cloned set
		return clone;
	},

	cleanData: function( elems ) {
		var data, elem, type,
			special = jQuery.event.special,
			i = 0;

		for ( ; ( elem = elems[ i ] ) !== undefined; i++ ) {
			if ( acceptData( elem ) ) {
				if ( ( data = elem[ dataPriv.expando ] ) ) {
					if ( data.events ) {
						for ( type in data.events ) {
							if ( special[ type ] ) {
								jQuery.event.remove( elem, type );

							// This is a shortcut to avoid jQuery.event.remove's overhead
							} else {
								jQuery.removeEvent( elem, type, data.handle );
							}
						}
					}

					// Support: Chrome <= 35-45+
					// Assign undefined instead of using delete, see Data#remove
					elem[ dataPriv.expando ] = undefined;
				}
				if ( elem[ dataUser.expando ] ) {

					// Support: Chrome <= 35-45+
					// Assign undefined instead of using delete, see Data#remove
					elem[ dataUser.expando ] = undefined;
				}
			}
		}
	}
} );

jQuery.fn.extend( {

	// Keep domManip exposed until 3.0 (gh-2225)
	domManip: domManip,

	detach: function( selector ) {
		return remove( this, selector, true );
	},

	remove: function( selector ) {
		return remove( this, selector );
	},

	text: function( value ) {
		return access( this, function( value ) {
			return value === undefined ?
				jQuery.text( this ) :
				this.empty().each( function() {
					if ( this.nodeType === 1 || this.nodeType === 11 || this.nodeType === 9 ) {
						this.textContent = value;
					}
				} );
		}, null, value, arguments.length );
	},

	append: function() {
		return domManip( this, arguments, function( elem ) {
			if ( this.nodeType === 1 || this.nodeType === 11 || this.nodeType === 9 ) {
				var target = manipulationTarget( this, elem );
				target.appendChild( elem );
			}
		} );
	},

	prepend: function() {
		return domManip( this, arguments, function( elem ) {
			if ( this.nodeType === 1 || this.nodeType === 11 || this.nodeType === 9 ) {
				var target = manipulationTarget( this, elem );
				target.insertBefore( elem, target.firstChild );
			}
		} );
	},

	before: function() {
		return domManip( this, arguments, function( elem ) {
			if ( this.parentNode ) {
				this.parentNode.insertBefore( elem, this );
			}
		} );
	},

	after: function() {
		return domManip( this, arguments, function( elem ) {
			if ( this.parentNode ) {
				this.parentNode.insertBefore( elem, this.nextSibling );
			}
		} );
	},

	empty: function() {
		var elem,
			i = 0;

		for ( ; ( elem = this[ i ] ) != null; i++ ) {
			if ( elem.nodeType === 1 ) {

				// Prevent memory leaks
				jQuery.cleanData( getAll( elem, false ) );

				// Remove any remaining nodes
				elem.textContent = "";
			}
		}

		return this;
	},

	clone: function( dataAndEvents, deepDataAndEvents ) {
		dataAndEvents = dataAndEvents == null ? false : dataAndEvents;
		deepDataAndEvents = deepDataAndEvents == null ? dataAndEvents : deepDataAndEvents;

		return this.map( function() {
			return jQuery.clone( this, dataAndEvents, deepDataAndEvents );
		} );
	},

	html: function( value ) {
		return access( this, function( value ) {
			var elem = this[ 0 ] || {},
				i = 0,
				l = this.length;

			if ( value === undefined && elem.nodeType === 1 ) {
				return elem.innerHTML;
			}

			// See if we can take a shortcut and just use innerHTML
			if ( typeof value === "string" && !rnoInnerhtml.test( value ) &&
				!wrapMap[ ( rtagName.exec( value ) || [ "", "" ] )[ 1 ].toLowerCase() ] ) {

				value = jQuery.htmlPrefilter( value );

				try {
					for ( ; i < l; i++ ) {
						elem = this[ i ] || {};

						// Remove element nodes and prevent memory leaks
						if ( elem.nodeType === 1 ) {
							jQuery.cleanData( getAll( elem, false ) );
							elem.innerHTML = value;
						}
					}

					elem = 0;

				// If using innerHTML throws an exception, use the fallback method
				} catch ( e ) {}
			}

			if ( elem ) {
				this.empty().append( value );
			}
		}, null, value, arguments.length );
	},

	replaceWith: function() {
		var ignored = [];

		// Make the changes, replacing each non-ignored context element with the new content
		return domManip( this, arguments, function( elem ) {
			var parent = this.parentNode;

			if ( jQuery.inArray( this, ignored ) < 0 ) {
				jQuery.cleanData( getAll( this ) );
				if ( parent ) {
					parent.replaceChild( elem, this );
				}
			}

		// Force callback invocation
		}, ignored );
	}
} );

jQuery.each( {
	appendTo: "append",
	prependTo: "prepend",
	insertBefore: "before",
	insertAfter: "after",
	replaceAll: "replaceWith"
}, function( name, original ) {
	jQuery.fn[ name ] = function( selector ) {
		var elems,
			ret = [],
			insert = jQuery( selector ),
			last = insert.length - 1,
			i = 0;

		for ( ; i <= last; i++ ) {
			elems = i === last ? this : this.clone( true );
			jQuery( insert[ i ] )[ original ]( elems );

			// Support: QtWebKit
			// .get() because push.apply(_, arraylike) throws
			push.apply( ret, elems.get() );
		}

		return this.pushStack( ret );
	};
} );


var iframe,
	elemdisplay = {

		// Support: Firefox
		// We have to pre-define these values for FF (#10227)
		HTML: "block",
		BODY: "block"
	};

/**
 * Retrieve the actual display of a element
 * @param {String} name nodeName of the element
 * @param {Object} doc Document object
 */

// Called only from within defaultDisplay
function actualDisplay( name, doc ) {
	var elem = jQuery( doc.createElement( name ) ).appendTo( doc.body ),

		display = jQuery.css( elem[ 0 ], "display" );

	// We don't have any data stored on the element,
	// so use "detach" method as fast way to get rid of the element
	elem.detach();

	return display;
}

/**
 * Try to determine the default display value of an element
 * @param {String} nodeName
 */
function defaultDisplay( nodeName ) {
	var doc = document,
		display = elemdisplay[ nodeName ];

	if ( !display ) {
		display = actualDisplay( nodeName, doc );

		// If the simple way fails, read from inside an iframe
		if ( display === "none" || !display ) {

			// Use the already-created iframe if possible
			iframe = ( iframe || jQuery( "<iframe frameborder='0' width='0' height='0'/>" ) )
				.appendTo( doc.documentElement );

			// Always write a new HTML skeleton so Webkit and Firefox don't choke on reuse
			doc = iframe[ 0 ].contentDocument;

			// Support: IE
			doc.write();
			doc.close();

			display = actualDisplay( nodeName, doc );
			iframe.detach();
		}

		// Store the correct default display
		elemdisplay[ nodeName ] = display;
	}

	return display;
}
var rmargin = ( /^margin/ );

var rnumnonpx = new RegExp( "^(" + pnum + ")(?!px)[a-z%]+$", "i" );

var getStyles = function( elem ) {

		// Support: IE<=11+, Firefox<=30+ (#15098, #14150)
		// IE throws on elements created in popups
		// FF meanwhile throws on frame elements through "defaultView.getComputedStyle"
		var view = elem.ownerDocument.defaultView;

		if ( !view.opener ) {
			view = window;
		}

		return view.getComputedStyle( elem );
	};

var swap = function( elem, options, callback, args ) {
	var ret, name,
		old = {};

	// Remember the old values, and insert the new ones
	for ( name in options ) {
		old[ name ] = elem.style[ name ];
		elem.style[ name ] = options[ name ];
	}

	ret = callback.apply( elem, args || [] );

	// Revert the old values
	for ( name in options ) {
		elem.style[ name ] = old[ name ];
	}

	return ret;
};


var documentElement = document.documentElement;



( function() {
	var pixelPositionVal, boxSizingReliableVal, pixelMarginRightVal, reliableMarginLeftVal,
		container = document.createElement( "div" ),
		div = document.createElement( "div" );

	// Finish early in limited (non-browser) environments
	if ( !div.style ) {
		return;
	}

	// Support: IE9-11+
	// Style of cloned element affects source element cloned (#8908)
	div.style.backgroundClip = "content-box";
	div.cloneNode( true ).style.backgroundClip = "";
	support.clearCloneStyle = div.style.backgroundClip === "content-box";

	container.style.cssText = "border:0;width:8px;height:0;top:0;left:-9999px;" +
		"padding:0;margin-top:1px;position:absolute";
	container.appendChild( div );

	// Executing both pixelPosition & boxSizingReliable tests require only one layout
	// so they're executed at the same time to save the second computation.
	function computeStyleTests() {
		div.style.cssText =

			// Support: Firefox<29, Android 2.3
			// Vendor-prefix box-sizing
			"-webkit-box-sizing:border-box;-moz-box-sizing:border-box;box-sizing:border-box;" +
			"position:relative;display:block;" +
			"margin:auto;border:1px;padding:1px;" +
			"top:1%;width:50%";
		div.innerHTML = "";
		documentElement.appendChild( container );

		var divStyle = window.getComputedStyle( div );
		pixelPositionVal = divStyle.top !== "1%";
		reliableMarginLeftVal = divStyle.marginLeft === "2px";
		boxSizingReliableVal = divStyle.width === "4px";

		// Support: Android 4.0 - 4.3 only
		// Some styles come back with percentage values, even though they shouldn't
		div.style.marginRight = "50%";
		pixelMarginRightVal = divStyle.marginRight === "4px";

		documentElement.removeChild( container );
	}

	jQuery.extend( support, {
		pixelPosition: function() {

			// This test is executed only once but we still do memoizing
			// since we can use the boxSizingReliable pre-computing.
			// No need to check if the test was already performed, though.
			computeStyleTests();
			return pixelPositionVal;
		},
		boxSizingReliable: function() {
			if ( boxSizingReliableVal == null ) {
				computeStyleTests();
			}
			return boxSizingReliableVal;
		},
		pixelMarginRight: function() {

			// Support: Android 4.0-4.3
			// We're checking for boxSizingReliableVal here instead of pixelMarginRightVal
			// since that compresses better and they're computed together anyway.
			if ( boxSizingReliableVal == null ) {
				computeStyleTests();
			}
			return pixelMarginRightVal;
		},
		reliableMarginLeft: function() {

			// Support: IE <=8 only, Android 4.0 - 4.3 only, Firefox <=3 - 37
			if ( boxSizingReliableVal == null ) {
				computeStyleTests();
			}
			return reliableMarginLeftVal;
		},
		reliableMarginRight: function() {

			// Support: Android 2.3
			// Check if div with explicit width and no margin-right incorrectly
			// gets computed margin-right based on width of container. (#3333)
			// WebKit Bug 13343 - getComputedStyle returns wrong value for margin-right
			// This support function is only executed once so no memoizing is needed.
			var ret,
				marginDiv = div.appendChild( document.createElement( "div" ) );

			// Reset CSS: box-sizing; display; margin; border; padding
			marginDiv.style.cssText = div.style.cssText =

				// Support: Android 2.3
				// Vendor-prefix box-sizing
				"-webkit-box-sizing:content-box;box-sizing:content-box;" +
				"display:block;margin:0;border:0;padding:0";
			marginDiv.style.marginRight = marginDiv.style.width = "0";
			div.style.width = "1px";
			documentElement.appendChild( container );

			ret = !parseFloat( window.getComputedStyle( marginDiv ).marginRight );

			documentElement.removeChild( container );
			div.removeChild( marginDiv );

			return ret;
		}
	} );
} )();


function curCSS( elem, name, computed ) {
	var width, minWidth, maxWidth, ret,
		style = elem.style;

	computed = computed || getStyles( elem );

	// Support: IE9
	// getPropertyValue is only needed for .css('filter') (#12537)
	if ( computed ) {
		ret = computed.getPropertyValue( name ) || computed[ name ];

		if ( ret === "" && !jQuery.contains( elem.ownerDocument, elem ) ) {
			ret = jQuery.style( elem, name );
		}

		// A tribute to the "awesome hack by Dean Edwards"
		// Android Browser returns percentage for some values,
		// but width seems to be reliably pixels.
		// This is against the CSSOM draft spec:
		// http://dev.w3.org/csswg/cssom/#resolved-values
		if ( !support.pixelMarginRight() && rnumnonpx.test( ret ) && rmargin.test( name ) ) {

			// Remember the original values
			width = style.width;
			minWidth = style.minWidth;
			maxWidth = style.maxWidth;

			// Put in the new values to get a computed value out
			style.minWidth = style.maxWidth = style.width = ret;
			ret = computed.width;

			// Revert the changed values
			style.width = width;
			style.minWidth = minWidth;
			style.maxWidth = maxWidth;
		}
	}

	return ret !== undefined ?

		// Support: IE9-11+
		// IE returns zIndex value as an integer.
		ret + "" :
		ret;
}


function addGetHookIf( conditionFn, hookFn ) {

	// Define the hook, we'll check on the first run if it's really needed.
	return {
		get: function() {
			if ( conditionFn() ) {

				// Hook not needed (or it's not possible to use it due
				// to missing dependency), remove it.
				delete this.get;
				return;
			}

			// Hook needed; redefine it so that the support test is not executed again.
			return ( this.get = hookFn ).apply( this, arguments );
		}
	};
}


var

	// Swappable if display is none or starts with table
	// except "table", "table-cell", or "table-caption"
	// See here for display values: https://developer.mozilla.org/en-US/docs/CSS/display
	rdisplayswap = /^(none|table(?!-c[ea]).+)/,

	cssShow = { position: "absolute", visibility: "hidden", display: "block" },
	cssNormalTransform = {
		letterSpacing: "0",
		fontWeight: "400"
	},

	cssPrefixes = [ "Webkit", "O", "Moz", "ms" ],
	emptyStyle = document.createElement( "div" ).style;

// Return a css property mapped to a potentially vendor prefixed property
function vendorPropName( name ) {

	// Shortcut for names that are not vendor prefixed
	if ( name in emptyStyle ) {
		return name;
	}

	// Check for vendor prefixed names
	var capName = name[ 0 ].toUpperCase() + name.slice( 1 ),
		i = cssPrefixes.length;

	while ( i-- ) {
		name = cssPrefixes[ i ] + capName;
		if ( name in emptyStyle ) {
			return name;
		}
	}
}

function setPositiveNumber( elem, value, subtract ) {

	// Any relative (+/-) values have already been
	// normalized at this point
	var matches = rcssNum.exec( value );
	return matches ?

		// Guard against undefined "subtract", e.g., when used as in cssHooks
		Math.max( 0, matches[ 2 ] - ( subtract || 0 ) ) + ( matches[ 3 ] || "px" ) :
		value;
}

function augmentWidthOrHeight( elem, name, extra, isBorderBox, styles ) {
	var i = extra === ( isBorderBox ? "border" : "content" ) ?

		// If we already have the right measurement, avoid augmentation
		4 :

		// Otherwise initialize for horizontal or vertical properties
		name === "width" ? 1 : 0,

		val = 0;

	for ( ; i < 4; i += 2 ) {

		// Both box models exclude margin, so add it if we want it
		if ( extra === "margin" ) {
			val += jQuery.css( elem, extra + cssExpand[ i ], true, styles );
		}

		if ( isBorderBox ) {

			// border-box includes padding, so remove it if we want content
			if ( extra === "content" ) {
				val -= jQuery.css( elem, "padding" + cssExpand[ i ], true, styles );
			}

			// At this point, extra isn't border nor margin, so remove border
			if ( extra !== "margin" ) {
				val -= jQuery.css( elem, "border" + cssExpand[ i ] + "Width", true, styles );
			}
		} else {

			// At this point, extra isn't content, so add padding
			val += jQuery.css( elem, "padding" + cssExpand[ i ], true, styles );

			// At this point, extra isn't content nor padding, so add border
			if ( extra !== "padding" ) {
				val += jQuery.css( elem, "border" + cssExpand[ i ] + "Width", true, styles );
			}
		}
	}

	return val;
}

function getWidthOrHeight( elem, name, extra ) {

	// Start with offset property, which is equivalent to the border-box value
	var valueIsBorderBox = true,
		val = name === "width" ? elem.offsetWidth : elem.offsetHeight,
		styles = getStyles( elem ),
		isBorderBox = jQuery.css( elem, "boxSizing", false, styles ) === "border-box";

	// Support: IE11 only
	// In IE 11 fullscreen elements inside of an iframe have
	// 100x too small dimensions (gh-1764).
	if ( document.msFullscreenElement && window.top !== window ) {

		// Support: IE11 only
		// Running getBoundingClientRect on a disconnected node
		// in IE throws an error.
		if ( elem.getClientRects().length ) {
			val = Math.round( elem.getBoundingClientRect()[ name ] * 100 );
		}
	}

	// Some non-html elements return undefined for offsetWidth, so check for null/undefined
	// svg - https://bugzilla.mozilla.org/show_bug.cgi?id=649285
	// MathML - https://bugzilla.mozilla.org/show_bug.cgi?id=491668
	if ( val <= 0 || val == null ) {

		// Fall back to computed then uncomputed css if necessary
		val = curCSS( elem, name, styles );
		if ( val < 0 || val == null ) {
			val = elem.style[ name ];
		}

		// Computed unit is not pixels. Stop here and return.
		if ( rnumnonpx.test( val ) ) {
			return val;
		}

		// Check for style in case a browser which returns unreliable values
		// for getComputedStyle silently falls back to the reliable elem.style
		valueIsBorderBox = isBorderBox &&
			( support.boxSizingReliable() || val === elem.style[ name ] );

		// Normalize "", auto, and prepare for extra
		val = parseFloat( val ) || 0;
	}

	// Use the active box-sizing model to add/subtract irrelevant styles
	return ( val +
		augmentWidthOrHeight(
			elem,
			name,
			extra || ( isBorderBox ? "border" : "content" ),
			valueIsBorderBox,
			styles
		)
	) + "px";
}

function showHide( elements, show ) {
	var display, elem, hidden,
		values = [],
		index = 0,
		length = elements.length;

	for ( ; index < length; index++ ) {
		elem = elements[ index ];
		if ( !elem.style ) {
			continue;
		}

		values[ index ] = dataPriv.get( elem, "olddisplay" );
		display = elem.style.display;
		if ( show ) {

			// Reset the inline display of this element to learn if it is
			// being hidden by cascaded rules or not
			if ( !values[ index ] && display === "none" ) {
				elem.style.display = "";
			}

			// Set elements which have been overridden with display: none
			// in a stylesheet to whatever the default browser style is
			// for such an element
			if ( elem.style.display === "" && isHidden( elem ) ) {
				values[ index ] = dataPriv.access(
					elem,
					"olddisplay",
					defaultDisplay( elem.nodeName )
				);
			}
		} else {
			hidden = isHidden( elem );

			if ( display !== "none" || !hidden ) {
				dataPriv.set(
					elem,
					"olddisplay",
					hidden ? display : jQuery.css( elem, "display" )
				);
			}
		}
	}

	// Set the display of most of the elements in a second loop
	// to avoid the constant reflow
	for ( index = 0; index < length; index++ ) {
		elem = elements[ index ];
		if ( !elem.style ) {
			continue;
		}
		if ( !show || elem.style.display === "none" || elem.style.display === "" ) {
			elem.style.display = show ? values[ index ] || "" : "none";
		}
	}

	return elements;
}

jQuery.extend( {

	// Add in style property hooks for overriding the default
	// behavior of getting and setting a style property
	cssHooks: {
		opacity: {
			get: function( elem, computed ) {
				if ( computed ) {

					// We should always get a number back from opacity
					var ret = curCSS( elem, "opacity" );
					return ret === "" ? "1" : ret;
				}
			}
		}
	},

	// Don't automatically add "px" to these possibly-unitless properties
	cssNumber: {
		"animationIterationCount": true,
		"columnCount": true,
		"fillOpacity": true,
		"flexGrow": true,
		"flexShrink": true,
		"fontWeight": true,
		"lineHeight": true,
		"opacity": true,
		"order": true,
		"orphans": true,
		"widows": true,
		"zIndex": true,
		"zoom": true
	},

	// Add in properties whose names you wish to fix before
	// setting or getting the value
	cssProps: {
		"float": "cssFloat"
	},

	// Get and set the style property on a DOM Node
	style: function( elem, name, value, extra ) {

		// Don't set styles on text and comment nodes
		if ( !elem || elem.nodeType === 3 || elem.nodeType === 8 || !elem.style ) {
			return;
		}

		// Make sure that we're working with the right name
		var ret, type, hooks,
			origName = jQuery.camelCase( name ),
			style = elem.style;

		name = jQuery.cssProps[ origName ] ||
			( jQuery.cssProps[ origName ] = vendorPropName( origName ) || origName );

		// Gets hook for the prefixed version, then unprefixed version
		hooks = jQuery.cssHooks[ name ] || jQuery.cssHooks[ origName ];

		// Check if we're setting a value
		if ( value !== undefined ) {
			type = typeof value;

			// Convert "+=" or "-=" to relative numbers (#7345)
			if ( type === "string" && ( ret = rcssNum.exec( value ) ) && ret[ 1 ] ) {
				value = adjustCSS( elem, name, ret );

				// Fixes bug #9237
				type = "number";
			}

			// Make sure that null and NaN values aren't set (#7116)
			if ( value == null || value !== value ) {
				return;
			}

			// If a number was passed in, add the unit (except for certain CSS properties)
			if ( type === "number" ) {
				value += ret && ret[ 3 ] || ( jQuery.cssNumber[ origName ] ? "" : "px" );
			}

			// Support: IE9-11+
			// background-* props affect original clone's values
			if ( !support.clearCloneStyle && value === "" && name.indexOf( "background" ) === 0 ) {
				style[ name ] = "inherit";
			}

			// If a hook was provided, use that value, otherwise just set the specified value
			if ( !hooks || !( "set" in hooks ) ||
				( value = hooks.set( elem, value, extra ) ) !== undefined ) {

				style[ name ] = value;
			}

		} else {

			// If a hook was provided get the non-computed value from there
			if ( hooks && "get" in hooks &&
				( ret = hooks.get( elem, false, extra ) ) !== undefined ) {

				return ret;
			}

			// Otherwise just get the value from the style object
			return style[ name ];
		}
	},

	css: function( elem, name, extra, styles ) {
		var val, num, hooks,
			origName = jQuery.camelCase( name );

		// Make sure that we're working with the right name
		name = jQuery.cssProps[ origName ] ||
			( jQuery.cssProps[ origName ] = vendorPropName( origName ) || origName );

		// Try prefixed name followed by the unprefixed name
		hooks = jQuery.cssHooks[ name ] || jQuery.cssHooks[ origName ];

		// If a hook was provided get the computed value from there
		if ( hooks && "get" in hooks ) {
			val = hooks.get( elem, true, extra );
		}

		// Otherwise, if a way to get the computed value exists, use that
		if ( val === undefined ) {
			val = curCSS( elem, name, styles );
		}

		// Convert "normal" to computed value
		if ( val === "normal" && name in cssNormalTransform ) {
			val = cssNormalTransform[ name ];
		}

		// Make numeric if forced or a qualifier was provided and val looks numeric
		if ( extra === "" || extra ) {
			num = parseFloat( val );
			return extra === true || isFinite( num ) ? num || 0 : val;
		}
		return val;
	}
} );

jQuery.each( [ "height", "width" ], function( i, name ) {
	jQuery.cssHooks[ name ] = {
		get: function( elem, computed, extra ) {
			if ( computed ) {

				// Certain elements can have dimension info if we invisibly show them
				// but it must have a current display style that would benefit
				return rdisplayswap.test( jQuery.css( elem, "display" ) ) &&
					elem.offsetWidth === 0 ?
						swap( elem, cssShow, function() {
							return getWidthOrHeight( elem, name, extra );
						} ) :
						getWidthOrHeight( elem, name, extra );
			}
		},

		set: function( elem, value, extra ) {
			var matches,
				styles = extra && getStyles( elem ),
				subtract = extra && augmentWidthOrHeight(
					elem,
					name,
					extra,
					jQuery.css( elem, "boxSizing", false, styles ) === "border-box",
					styles
				);

			// Convert to pixels if value adjustment is needed
			if ( subtract && ( matches = rcssNum.exec( value ) ) &&
				( matches[ 3 ] || "px" ) !== "px" ) {

				elem.style[ name ] = value;
				value = jQuery.css( elem, name );
			}

			return setPositiveNumber( elem, value, subtract );
		}
	};
} );

jQuery.cssHooks.marginLeft = addGetHookIf( support.reliableMarginLeft,
	function( elem, computed ) {
		if ( computed ) {
			return ( parseFloat( curCSS( elem, "marginLeft" ) ) ||
				elem.getBoundingClientRect().left -
					swap( elem, { marginLeft: 0 }, function() {
						return elem.getBoundingClientRect().left;
					} )
				) + "px";
		}
	}
);

// Support: Android 2.3
jQuery.cssHooks.marginRight = addGetHookIf( support.reliableMarginRight,
	function( elem, computed ) {
		if ( computed ) {
			return swap( elem, { "display": "inline-block" },
				curCSS, [ elem, "marginRight" ] );
		}
	}
);

// These hooks are used by animate to expand properties
jQuery.each( {
	margin: "",
	padding: "",
	border: "Width"
}, function( prefix, suffix ) {
	jQuery.cssHooks[ prefix + suffix ] = {
		expand: function( value ) {
			var i = 0,
				expanded = {},

				// Assumes a single number if not a string
				parts = typeof value === "string" ? value.split( " " ) : [ value ];

			for ( ; i < 4; i++ ) {
				expanded[ prefix + cssExpand[ i ] + suffix ] =
					parts[ i ] || parts[ i - 2 ] || parts[ 0 ];
			}

			return expanded;
		}
	};

	if ( !rmargin.test( prefix ) ) {
		jQuery.cssHooks[ prefix + suffix ].set = setPositiveNumber;
	}
} );

jQuery.fn.extend( {
	css: function( name, value ) {
		return access( this, function( elem, name, value ) {
			var styles, len,
				map = {},
				i = 0;

			if ( jQuery.isArray( name ) ) {
				styles = getStyles( elem );
				len = name.length;

				for ( ; i < len; i++ ) {
					map[ name[ i ] ] = jQuery.css( elem, name[ i ], false, styles );
				}

				return map;
			}

			return value !== undefined ?
				jQuery.style( elem, name, value ) :
				jQuery.css( elem, name );
		}, name, value, arguments.length > 1 );
	},
	show: function() {
		return showHide( this, true );
	},
	hide: function() {
		return showHide( this );
	},
	toggle: function( state ) {
		if ( typeof state === "boolean" ) {
			return state ? this.show() : this.hide();
		}

		return this.each( function() {
			if ( isHidden( this ) ) {
				jQuery( this ).show();
			} else {
				jQuery( this ).hide();
			}
		} );
	}
} );


function Tween( elem, options, prop, end, easing ) {
	return new Tween.prototype.init( elem, options, prop, end, easing );
}
jQuery.Tween = Tween;

Tween.prototype = {
	constructor: Tween,
	init: function( elem, options, prop, end, easing, unit ) {
		this.elem = elem;
		this.prop = prop;
		this.easing = easing || jQuery.easing._default;
		this.options = options;
		this.start = this.now = this.cur();
		this.end = end;
		this.unit = unit || ( jQuery.cssNumber[ prop ] ? "" : "px" );
	},
	cur: function() {
		var hooks = Tween.propHooks[ this.prop ];

		return hooks && hooks.get ?
			hooks.get( this ) :
			Tween.propHooks._default.get( this );
	},
	run: function( percent ) {
		var eased,
			hooks = Tween.propHooks[ this.prop ];

		if ( this.options.duration ) {
			this.pos = eased = jQuery.easing[ this.easing ](
				percent, this.options.duration * percent, 0, 1, this.options.duration
			);
		} else {
			this.pos = eased = percent;
		}
		this.now = ( this.end - this.start ) * eased + this.start;

		if ( this.options.step ) {
			this.options.step.call( this.elem, this.now, this );
		}

		if ( hooks && hooks.set ) {
			hooks.set( this );
		} else {
			Tween.propHooks._default.set( this );
		}
		return this;
	}
};

Tween.prototype.init.prototype = Tween.prototype;

Tween.propHooks = {
	_default: {
		get: function( tween ) {
			var result;

			// Use a property on the element directly when it is not a DOM element,
			// or when there is no matching style property that exists.
			if ( tween.elem.nodeType !== 1 ||
				tween.elem[ tween.prop ] != null && tween.elem.style[ tween.prop ] == null ) {
				return tween.elem[ tween.prop ];
			}

			// Passing an empty string as a 3rd parameter to .css will automatically
			// attempt a parseFloat and fallback to a string if the parse fails.
			// Simple values such as "10px" are parsed to Float;
			// complex values such as "rotate(1rad)" are returned as-is.
			result = jQuery.css( tween.elem, tween.prop, "" );

			// Empty strings, null, undefined and "auto" are converted to 0.
			return !result || result === "auto" ? 0 : result;
		},
		set: function( tween ) {

			// Use step hook for back compat.
			// Use cssHook if its there.
			// Use .style if available and use plain properties where available.
			if ( jQuery.fx.step[ tween.prop ] ) {
				jQuery.fx.step[ tween.prop ]( tween );
			} else if ( tween.elem.nodeType === 1 &&
				( tween.elem.style[ jQuery.cssProps[ tween.prop ] ] != null ||
					jQuery.cssHooks[ tween.prop ] ) ) {
				jQuery.style( tween.elem, tween.prop, tween.now + tween.unit );
			} else {
				tween.elem[ tween.prop ] = tween.now;
			}
		}
	}
};

// Support: IE9
// Panic based approach to setting things on disconnected nodes
Tween.propHooks.scrollTop = Tween.propHooks.scrollLeft = {
	set: function( tween ) {
		if ( tween.elem.nodeType && tween.elem.parentNode ) {
			tween.elem[ tween.prop ] = tween.now;
		}
	}
};

jQuery.easing = {
	linear: function( p ) {
		return p;
	},
	swing: function( p ) {
		return 0.5 - Math.cos( p * Math.PI ) / 2;
	},
	_default: "swing"
};

jQuery.fx = Tween.prototype.init;

// Back Compat <1.8 extension point
jQuery.fx.step = {};




var
	fxNow, timerId,
	rfxtypes = /^(?:toggle|show|hide)$/,
	rrun = /queueHooks$/;

// Animations created synchronously will run synchronously
function createFxNow() {
	window.setTimeout( function() {
		fxNow = undefined;
	} );
	return ( fxNow = jQuery.now() );
}

// Generate parameters to create a standard animation
function genFx( type, includeWidth ) {
	var which,
		i = 0,
		attrs = { height: type };

	// If we include width, step value is 1 to do all cssExpand values,
	// otherwise step value is 2 to skip over Left and Right
	includeWidth = includeWidth ? 1 : 0;
	for ( ; i < 4 ; i += 2 - includeWidth ) {
		which = cssExpand[ i ];
		attrs[ "margin" + which ] = attrs[ "padding" + which ] = type;
	}

	if ( includeWidth ) {
		attrs.opacity = attrs.width = type;
	}

	return attrs;
}

function createTween( value, prop, animation ) {
	var tween,
		collection = ( Animation.tweeners[ prop ] || [] ).concat( Animation.tweeners[ "*" ] ),
		index = 0,
		length = collection.length;
	for ( ; index < length; index++ ) {
		if ( ( tween = collection[ index ].call( animation, prop, value ) ) ) {

			// We're done with this property
			return tween;
		}
	}
}

function defaultPrefilter( elem, props, opts ) {
	/* jshint validthis: true */
	var prop, value, toggle, tween, hooks, oldfire, display, checkDisplay,
		anim = this,
		orig = {},
		style = elem.style,
		hidden = elem.nodeType && isHidden( elem ),
		dataShow = dataPriv.get( elem, "fxshow" );

	// Handle queue: false promises
	if ( !opts.queue ) {
		hooks = jQuery._queueHooks( elem, "fx" );
		if ( hooks.unqueued == null ) {
			hooks.unqueued = 0;
			oldfire = hooks.empty.fire;
			hooks.empty.fire = function() {
				if ( !hooks.unqueued ) {
					oldfire();
				}
			};
		}
		hooks.unqueued++;

		anim.always( function() {

			// Ensure the complete handler is called before this completes
			anim.always( function() {
				hooks.unqueued--;
				if ( !jQuery.queue( elem, "fx" ).length ) {
					hooks.empty.fire();
				}
			} );
		} );
	}

	// Height/width overflow pass
	if ( elem.nodeType === 1 && ( "height" in props || "width" in props ) ) {

		// Make sure that nothing sneaks out
		// Record all 3 overflow attributes because IE9-10 do not
		// change the overflow attribute when overflowX and
		// overflowY are set to the same value
		opts.overflow = [ style.overflow, style.overflowX, style.overflowY ];

		// Set display property to inline-block for height/width
		// animations on inline elements that are having width/height animated
		display = jQuery.css( elem, "display" );

		// Test default display if display is currently "none"
		checkDisplay = display === "none" ?
			dataPriv.get( elem, "olddisplay" ) || defaultDisplay( elem.nodeName ) : display;

		if ( checkDisplay === "inline" && jQuery.css( elem, "float" ) === "none" ) {
			style.display = "inline-block";
		}
	}

	if ( opts.overflow ) {
		style.overflow = "hidden";
		anim.always( function() {
			style.overflow = opts.overflow[ 0 ];
			style.overflowX = opts.overflow[ 1 ];
			style.overflowY = opts.overflow[ 2 ];
		} );
	}

	// show/hide pass
	for ( prop in props ) {
		value = props[ prop ];
		if ( rfxtypes.exec( value ) ) {
			delete props[ prop ];
			toggle = toggle || value === "toggle";
			if ( value === ( hidden ? "hide" : "show" ) ) {

				// If there is dataShow left over from a stopped hide or show
				// and we are going to proceed with show, we should pretend to be hidden
				if ( value === "show" && dataShow && dataShow[ prop ] !== undefined ) {
					hidden = true;
				} else {
					continue;
				}
			}
			orig[ prop ] = dataShow && dataShow[ prop ] || jQuery.style( elem, prop );

		// Any non-fx value stops us from restoring the original display value
		} else {
			display = undefined;
		}
	}

	if ( !jQuery.isEmptyObject( orig ) ) {
		if ( dataShow ) {
			if ( "hidden" in dataShow ) {
				hidden = dataShow.hidden;
			}
		} else {
			dataShow = dataPriv.access( elem, "fxshow", {} );
		}

		// Store state if its toggle - enables .stop().toggle() to "reverse"
		if ( toggle ) {
			dataShow.hidden = !hidden;
		}
		if ( hidden ) {
			jQuery( elem ).show();
		} else {
			anim.done( function() {
				jQuery( elem ).hide();
			} );
		}
		anim.done( function() {
			var prop;

			dataPriv.remove( elem, "fxshow" );
			for ( prop in orig ) {
				jQuery.style( elem, prop, orig[ prop ] );
			}
		} );
		for ( prop in orig ) {
			tween = createTween( hidden ? dataShow[ prop ] : 0, prop, anim );

			if ( !( prop in dataShow ) ) {
				dataShow[ prop ] = tween.start;
				if ( hidden ) {
					tween.end = tween.start;
					tween.start = prop === "width" || prop === "height" ? 1 : 0;
				}
			}
		}

	// If this is a noop like .hide().hide(), restore an overwritten display value
	} else if ( ( display === "none" ? defaultDisplay( elem.nodeName ) : display ) === "inline" ) {
		style.display = display;
	}
}

function propFilter( props, specialEasing ) {
	var index, name, easing, value, hooks;

	// camelCase, specialEasing and expand cssHook pass
	for ( index in props ) {
		name = jQuery.camelCase( index );
		easing = specialEasing[ name ];
		value = props[ index ];
		if ( jQuery.isArray( value ) ) {
			easing = value[ 1 ];
			value = props[ index ] = value[ 0 ];
		}

		if ( index !== name ) {
			props[ name ] = value;
			delete props[ index ];
		}

		hooks = jQuery.cssHooks[ name ];
		if ( hooks && "expand" in hooks ) {
			value = hooks.expand( value );
			delete props[ name ];

			// Not quite $.extend, this won't overwrite existing keys.
			// Reusing 'index' because we have the correct "name"
			for ( index in value ) {
				if ( !( index in props ) ) {
					props[ index ] = value[ index ];
					specialEasing[ index ] = easing;
				}
			}
		} else {
			specialEasing[ name ] = easing;
		}
	}
}

function Animation( elem, properties, options ) {
	var result,
		stopped,
		index = 0,
		length = Animation.prefilters.length,
		deferred = jQuery.Deferred().always( function() {

			// Don't match elem in the :animated selector
			delete tick.elem;
		} ),
		tick = function() {
			if ( stopped ) {
				return false;
			}
			var currentTime = fxNow || createFxNow(),
				remaining = Math.max( 0, animation.startTime + animation.duration - currentTime ),

				// Support: Android 2.3
				// Archaic crash bug won't allow us to use `1 - ( 0.5 || 0 )` (#12497)
				temp = remaining / animation.duration || 0,
				percent = 1 - temp,
				index = 0,
				length = animation.tweens.length;

			for ( ; index < length ; index++ ) {
				animation.tweens[ index ].run( percent );
			}

			deferred.notifyWith( elem, [ animation, percent, remaining ] );

			if ( percent < 1 && length ) {
				return remaining;
			} else {
				deferred.resolveWith( elem, [ animation ] );
				return false;
			}
		},
		animation = deferred.promise( {
			elem: elem,
			props: jQuery.extend( {}, properties ),
			opts: jQuery.extend( true, {
				specialEasing: {},
				easing: jQuery.easing._default
			}, options ),
			originalProperties: properties,
			originalOptions: options,
			startTime: fxNow || createFxNow(),
			duration: options.duration,
			tweens: [],
			createTween: function( prop, end ) {
				var tween = jQuery.Tween( elem, animation.opts, prop, end,
						animation.opts.specialEasing[ prop ] || animation.opts.easing );
				animation.tweens.push( tween );
				return tween;
			},
			stop: function( gotoEnd ) {
				var index = 0,

					// If we are going to the end, we want to run all the tweens
					// otherwise we skip this part
					length = gotoEnd ? animation.tweens.length : 0;
				if ( stopped ) {
					return this;
				}
				stopped = true;
				for ( ; index < length ; index++ ) {
					animation.tweens[ index ].run( 1 );
				}

				// Resolve when we played the last frame; otherwise, reject
				if ( gotoEnd ) {
					deferred.notifyWith( elem, [ animation, 1, 0 ] );
					deferred.resolveWith( elem, [ animation, gotoEnd ] );
				} else {
					deferred.rejectWith( elem, [ animation, gotoEnd ] );
				}
				return this;
			}
		} ),
		props = animation.props;

	propFilter( props, animation.opts.specialEasing );

	for ( ; index < length ; index++ ) {
		result = Animation.prefilters[ index ].call( animation, elem, props, animation.opts );
		if ( result ) {
			if ( jQuery.isFunction( result.stop ) ) {
				jQuery._queueHooks( animation.elem, animation.opts.queue ).stop =
					jQuery.proxy( result.stop, result );
			}
			return result;
		}
	}

	jQuery.map( props, createTween, animation );

	if ( jQuery.isFunction( animation.opts.start ) ) {
		animation.opts.start.call( elem, animation );
	}

	jQuery.fx.timer(
		jQuery.extend( tick, {
			elem: elem,
			anim: animation,
			queue: animation.opts.queue
		} )
	);

	// attach callbacks from options
	return animation.progress( animation.opts.progress )
		.done( animation.opts.done, animation.opts.complete )
		.fail( animation.opts.fail )
		.always( animation.opts.always );
}

jQuery.Animation = jQuery.extend( Animation, {
	tweeners: {
		"*": [ function( prop, value ) {
			var tween = this.createTween( prop, value );
			adjustCSS( tween.elem, prop, rcssNum.exec( value ), tween );
			return tween;
		} ]
	},

	tweener: function( props, callback ) {
		if ( jQuery.isFunction( props ) ) {
			callback = props;
			props = [ "*" ];
		} else {
			props = props.match( rnotwhite );
		}

		var prop,
			index = 0,
			length = props.length;

		for ( ; index < length ; index++ ) {
			prop = props[ index ];
			Animation.tweeners[ prop ] = Animation.tweeners[ prop ] || [];
			Animation.tweeners[ prop ].unshift( callback );
		}
	},

	prefilters: [ defaultPrefilter ],

	prefilter: function( callback, prepend ) {
		if ( prepend ) {
			Animation.prefilters.unshift( callback );
		} else {
			Animation.prefilters.push( callback );
		}
	}
} );

jQuery.speed = function( speed, easing, fn ) {
	var opt = speed && typeof speed === "object" ? jQuery.extend( {}, speed ) : {
		complete: fn || !fn && easing ||
			jQuery.isFunction( speed ) && speed,
		duration: speed,
		easing: fn && easing || easing && !jQuery.isFunction( easing ) && easing
	};

	opt.duration = jQuery.fx.off ? 0 : typeof opt.duration === "number" ?
		opt.duration : opt.duration in jQuery.fx.speeds ?
			jQuery.fx.speeds[ opt.duration ] : jQuery.fx.speeds._default;

	// Normalize opt.queue - true/undefined/null -> "fx"
	if ( opt.queue == null || opt.queue === true ) {
		opt.queue = "fx";
	}

	// Queueing
	opt.old = opt.complete;

	opt.complete = function() {
		if ( jQuery.isFunction( opt.old ) ) {
			opt.old.call( this );
		}

		if ( opt.queue ) {
			jQuery.dequeue( this, opt.queue );
		}
	};

	return opt;
};

jQuery.fn.extend( {
	fadeTo: function( speed, to, easing, callback ) {

		// Show any hidden elements after setting opacity to 0
		return this.filter( isHidden ).css( "opacity", 0 ).show()

			// Animate to the value specified
			.end().animate( { opacity: to }, speed, easing, callback );
	},
	animate: function( prop, speed, easing, callback ) {
		var empty = jQuery.isEmptyObject( prop ),
			optall = jQuery.speed( speed, easing, callback ),
			doAnimation = function() {

				// Operate on a copy of prop so per-property easing won't be lost
				var anim = Animation( this, jQuery.extend( {}, prop ), optall );

				// Empty animations, or finishing resolves immediately
				if ( empty || dataPriv.get( this, "finish" ) ) {
					anim.stop( true );
				}
			};
			doAnimation.finish = doAnimation;

		return empty || optall.queue === false ?
			this.each( doAnimation ) :
			this.queue( optall.queue, doAnimation );
	},
	stop: function( type, clearQueue, gotoEnd ) {
		var stopQueue = function( hooks ) {
			var stop = hooks.stop;
			delete hooks.stop;
			stop( gotoEnd );
		};

		if ( typeof type !== "string" ) {
			gotoEnd = clearQueue;
			clearQueue = type;
			type = undefined;
		}
		if ( clearQueue && type !== false ) {
			this.queue( type || "fx", [] );
		}

		return this.each( function() {
			var dequeue = true,
				index = type != null && type + "queueHooks",
				timers = jQuery.timers,
				data = dataPriv.get( this );

			if ( index ) {
				if ( data[ index ] && data[ index ].stop ) {
					stopQueue( data[ index ] );
				}
			} else {
				for ( index in data ) {
					if ( data[ index ] && data[ index ].stop && rrun.test( index ) ) {
						stopQueue( data[ index ] );
					}
				}
			}

			for ( index = timers.length; index--; ) {
				if ( timers[ index ].elem === this &&
					( type == null || timers[ index ].queue === type ) ) {

					timers[ index ].anim.stop( gotoEnd );
					dequeue = false;
					timers.splice( index, 1 );
				}
			}

			// Start the next in the queue if the last step wasn't forced.
			// Timers currently will call their complete callbacks, which
			// will dequeue but only if they were gotoEnd.
			if ( dequeue || !gotoEnd ) {
				jQuery.dequeue( this, type );
			}
		} );
	},
	finish: function( type ) {
		if ( type !== false ) {
			type = type || "fx";
		}
		return this.each( function() {
			var index,
				data = dataPriv.get( this ),
				queue = data[ type + "queue" ],
				hooks = data[ type + "queueHooks" ],
				timers = jQuery.timers,
				length = queue ? queue.length : 0;

			// Enable finishing flag on private data
			data.finish = true;

			// Empty the queue first
			jQuery.queue( this, type, [] );

			if ( hooks && hooks.stop ) {
				hooks.stop.call( this, true );
			}

			// Look for any active animations, and finish them
			for ( index = timers.length; index--; ) {
				if ( timers[ index ].elem === this && timers[ index ].queue === type ) {
					timers[ index ].anim.stop( true );
					timers.splice( index, 1 );
				}
			}

			// Look for any animations in the old queue and finish them
			for ( index = 0; index < length; index++ ) {
				if ( queue[ index ] && queue[ index ].finish ) {
					queue[ index ].finish.call( this );
				}
			}

			// Turn off finishing flag
			delete data.finish;
		} );
	}
} );

jQuery.each( [ "toggle", "show", "hide" ], function( i, name ) {
	var cssFn = jQuery.fn[ name ];
	jQuery.fn[ name ] = function( speed, easing, callback ) {
		return speed == null || typeof speed === "boolean" ?
			cssFn.apply( this, arguments ) :
			this.animate( genFx( name, true ), speed, easing, callback );
	};
} );

// Generate shortcuts for custom animations
jQuery.each( {
	slideDown: genFx( "show" ),
	slideUp: genFx( "hide" ),
	slideToggle: genFx( "toggle" ),
	fadeIn: { opacity: "show" },
	fadeOut: { opacity: "hide" },
	fadeToggle: { opacity: "toggle" }
}, function( name, props ) {
	jQuery.fn[ name ] = function( speed, easing, callback ) {
		return this.animate( props, speed, easing, callback );
	};
} );

jQuery.timers = [];
jQuery.fx.tick = function() {
	var timer,
		i = 0,
		timers = jQuery.timers;

	fxNow = jQuery.now();

	for ( ; i < timers.length; i++ ) {
		timer = timers[ i ];

		// Checks the timer has not already been removed
		if ( !timer() && timers[ i ] === timer ) {
			timers.splice( i--, 1 );
		}
	}

	if ( !timers.length ) {
		jQuery.fx.stop();
	}
	fxNow = undefined;
};

jQuery.fx.timer = function( timer ) {
	jQuery.timers.push( timer );
	if ( timer() ) {
		jQuery.fx.start();
	} else {
		jQuery.timers.pop();
	}
};

jQuery.fx.interval = 13;
jQuery.fx.start = function() {
	if ( !timerId ) {
		timerId = window.setInterval( jQuery.fx.tick, jQuery.fx.interval );
	}
};

jQuery.fx.stop = function() {
	window.clearInterval( timerId );

	timerId = null;
};

jQuery.fx.speeds = {
	slow: 600,
	fast: 200,

	// Default speed
	_default: 400
};


// Based off of the plugin by Clint Helfers, with permission.
// http://web.archive.org/web/20100324014747/http://blindsignals.com/index.php/2009/07/jquery-delay/
jQuery.fn.delay = function( time, type ) {
	time = jQuery.fx ? jQuery.fx.speeds[ time ] || time : time;
	type = type || "fx";

	return this.queue( type, function( next, hooks ) {
		var timeout = window.setTimeout( next, time );
		hooks.stop = function() {
			window.clearTimeout( timeout );
		};
	} );
};


( function() {
	var input = document.createElement( "input" ),
		select = document.createElement( "select" ),
		opt = select.appendChild( document.createElement( "option" ) );

	input.type = "checkbox";

	// Support: iOS<=5.1, Android<=4.2+
	// Default value for a checkbox should be "on"
	support.checkOn = input.value !== "";

	// Support: IE<=11+
	// Must access selectedIndex to make default options select
	support.optSelected = opt.selected;

	// Support: Android<=2.3
	// Options inside disabled selects are incorrectly marked as disabled
	select.disabled = true;
	support.optDisabled = !opt.disabled;

	// Support: IE<=11+
	// An input loses its value after becoming a radio
	input = document.createElement( "input" );
	input.value = "t";
	input.type = "radio";
	support.radioValue = input.value === "t";
} )();


var boolHook,
	attrHandle = jQuery.expr.attrHandle;

jQuery.fn.extend( {
	attr: function( name, value ) {
		return access( this, jQuery.attr, name, value, arguments.length > 1 );
	},

	removeAttr: function( name ) {
		return this.each( function() {
			jQuery.removeAttr( this, name );
		} );
	}
} );

jQuery.extend( {
	attr: function( elem, name, value ) {
		var ret, hooks,
			nType = elem.nodeType;

		// Don't get/set attributes on text, comment and attribute nodes
		if ( nType === 3 || nType === 8 || nType === 2 ) {
			return;
		}

		// Fallback to prop when attributes are not supported
		if ( typeof elem.getAttribute === "undefined" ) {
			return jQuery.prop( elem, name, value );
		}

		// All attributes are lowercase
		// Grab necessary hook if one is defined
		if ( nType !== 1 || !jQuery.isXMLDoc( elem ) ) {
			name = name.toLowerCase();
			hooks = jQuery.attrHooks[ name ] ||
				( jQuery.expr.match.bool.test( name ) ? boolHook : undefined );
		}

		if ( value !== undefined ) {
			if ( value === null ) {
				jQuery.removeAttr( elem, name );
				return;
			}

			if ( hooks && "set" in hooks &&
				( ret = hooks.set( elem, value, name ) ) !== undefined ) {
				return ret;
			}

			elem.setAttribute( name, value + "" );
			return value;
		}

		if ( hooks && "get" in hooks && ( ret = hooks.get( elem, name ) ) !== null ) {
			return ret;
		}

		ret = jQuery.find.attr( elem, name );

		// Non-existent attributes return null, we normalize to undefined
		return ret == null ? undefined : ret;
	},

	attrHooks: {
		type: {
			set: function( elem, value ) {
				if ( !support.radioValue && value === "radio" &&
					jQuery.nodeName( elem, "input" ) ) {
					var val = elem.value;
					elem.setAttribute( "type", value );
					if ( val ) {
						elem.value = val;
					}
					return value;
				}
			}
		}
	},

	removeAttr: function( elem, value ) {
		var name, propName,
			i = 0,
			attrNames = value && value.match( rnotwhite );

		if ( attrNames && elem.nodeType === 1 ) {
			while ( ( name = attrNames[ i++ ] ) ) {
				propName = jQuery.propFix[ name ] || name;

				// Boolean attributes get special treatment (#10870)
				if ( jQuery.expr.match.bool.test( name ) ) {

					// Set corresponding property to false
					elem[ propName ] = false;
				}

				elem.removeAttribute( name );
			}
		}
	}
} );

// Hooks for boolean attributes
boolHook = {
	set: function( elem, value, name ) {
		if ( value === false ) {

			// Remove boolean attributes when set to false
			jQuery.removeAttr( elem, name );
		} else {
			elem.setAttribute( name, name );
		}
		return name;
	}
};
jQuery.each( jQuery.expr.match.bool.source.match( /\w+/g ), function( i, name ) {
	var getter = attrHandle[ name ] || jQuery.find.attr;

	attrHandle[ name ] = function( elem, name, isXML ) {
		var ret, handle;
		if ( !isXML ) {

			// Avoid an infinite loop by temporarily removing this function from the getter
			handle = attrHandle[ name ];
			attrHandle[ name ] = ret;
			ret = getter( elem, name, isXML ) != null ?
				name.toLowerCase() :
				null;
			attrHandle[ name ] = handle;
		}
		return ret;
	};
} );




var rfocusable = /^(?:input|select|textarea|button)$/i,
	rclickable = /^(?:a|area)$/i;

jQuery.fn.extend( {
	prop: function( name, value ) {
		return access( this, jQuery.prop, name, value, arguments.length > 1 );
	},

	removeProp: function( name ) {
		return this.each( function() {
			delete this[ jQuery.propFix[ name ] || name ];
		} );
	}
} );

jQuery.extend( {
	prop: function( elem, name, value ) {
		var ret, hooks,
			nType = elem.nodeType;

		// Don't get/set properties on text, comment and attribute nodes
		if ( nType === 3 || nType === 8 || nType === 2 ) {
			return;
		}

		if ( nType !== 1 || !jQuery.isXMLDoc( elem ) ) {

			// Fix name and attach hooks
			name = jQuery.propFix[ name ] || name;
			hooks = jQuery.propHooks[ name ];
		}

		if ( value !== undefined ) {
			if ( hooks && "set" in hooks &&
				( ret = hooks.set( elem, value, name ) ) !== undefined ) {
				return ret;
			}

			return ( elem[ name ] = value );
		}

		if ( hooks && "get" in hooks && ( ret = hooks.get( elem, name ) ) !== null ) {
			return ret;
		}

		return elem[ name ];
	},

	propHooks: {
		tabIndex: {
			get: function( elem ) {

				// elem.tabIndex doesn't always return the
				// correct value when it hasn't been explicitly set
				// http://fluidproject.org/blog/2008/01/09/getting-setting-and-removing-tabindex-values-with-javascript/
				// Use proper attribute retrieval(#12072)
				var tabindex = jQuery.find.attr( elem, "tabindex" );

				return tabindex ?
					parseInt( tabindex, 10 ) :
					rfocusable.test( elem.nodeName ) ||
						rclickable.test( elem.nodeName ) && elem.href ?
							0 :
							-1;
			}
		}
	},

	propFix: {
		"for": "htmlFor",
		"class": "className"
	}
} );

if ( !support.optSelected ) {
	jQuery.propHooks.selected = {
		get: function( elem ) {
			var parent = elem.parentNode;
			if ( parent && parent.parentNode ) {
				parent.parentNode.selectedIndex;
			}
			return null;
		}
	};
}

jQuery.each( [
	"tabIndex",
	"readOnly",
	"maxLength",
	"cellSpacing",
	"cellPadding",
	"rowSpan",
	"colSpan",
	"useMap",
	"frameBorder",
	"contentEditable"
], function() {
	jQuery.propFix[ this.toLowerCase() ] = this;
} );




var rclass = /[\t\r\n\f]/g;

function getClass( elem ) {
	return elem.getAttribute && elem.getAttribute( "class" ) || "";
}

jQuery.fn.extend( {
	addClass: function( value ) {
		var classes, elem, cur, curValue, clazz, j, finalValue,
			i = 0;

		if ( jQuery.isFunction( value ) ) {
			return this.each( function( j ) {
				jQuery( this ).addClass( value.call( this, j, getClass( this ) ) );
			} );
		}

		if ( typeof value === "string" && value ) {
			classes = value.match( rnotwhite ) || [];

			while ( ( elem = this[ i++ ] ) ) {
				curValue = getClass( elem );
				cur = elem.nodeType === 1 &&
					( " " + curValue + " " ).replace( rclass, " " );

				if ( cur ) {
					j = 0;
					while ( ( clazz = classes[ j++ ] ) ) {
						if ( cur.indexOf( " " + clazz + " " ) < 0 ) {
							cur += clazz + " ";
						}
					}

					// Only assign if different to avoid unneeded rendering.
					finalValue = jQuery.trim( cur );
					if ( curValue !== finalValue ) {
						elem.setAttribute( "class", finalValue );
					}
				}
			}
		}

		return this;
	},

	removeClass: function( value ) {
		var classes, elem, cur, curValue, clazz, j, finalValue,
			i = 0;

		if ( jQuery.isFunction( value ) ) {
			return this.each( function( j ) {
				jQuery( this ).removeClass( value.call( this, j, getClass( this ) ) );
			} );
		}

		if ( !arguments.length ) {
			return this.attr( "class", "" );
		}

		if ( typeof value === "string" && value ) {
			classes = value.match( rnotwhite ) || [];

			while ( ( elem = this[ i++ ] ) ) {
				curValue = getClass( elem );

				// This expression is here for better compressibility (see addClass)
				cur = elem.nodeType === 1 &&
					( " " + curValue + " " ).replace( rclass, " " );

				if ( cur ) {
					j = 0;
					while ( ( clazz = classes[ j++ ] ) ) {

						// Remove *all* instances
						while ( cur.indexOf( " " + clazz + " " ) > -1 ) {
							cur = cur.replace( " " + clazz + " ", " " );
						}
					}

					// Only assign if different to avoid unneeded rendering.
					finalValue = jQuery.trim( cur );
					if ( curValue !== finalValue ) {
						elem.setAttribute( "class", finalValue );
					}
				}
			}
		}

		return this;
	},

	toggleClass: function( value, stateVal ) {
		var type = typeof value;

		if ( typeof stateVal === "boolean" && type === "string" ) {
			return stateVal ? this.addClass( value ) : this.removeClass( value );
		}

		if ( jQuery.isFunction( value ) ) {
			return this.each( function( i ) {
				jQuery( this ).toggleClass(
					value.call( this, i, getClass( this ), stateVal ),
					stateVal
				);
			} );
		}

		return this.each( function() {
			var className, i, self, classNames;

			if ( type === "string" ) {

				// Toggle individual class names
				i = 0;
				self = jQuery( this );
				classNames = value.match( rnotwhite ) || [];

				while ( ( className = classNames[ i++ ] ) ) {

					// Check each className given, space separated list
					if ( self.hasClass( className ) ) {
						self.removeClass( className );
					} else {
						self.addClass( className );
					}
				}

			// Toggle whole class name
			} else if ( value === undefined || type === "boolean" ) {
				className = getClass( this );
				if ( className ) {

					// Store className if set
					dataPriv.set( this, "__className__", className );
				}

				// If the element has a class name or if we're passed `false`,
				// then remove the whole classname (if there was one, the above saved it).
				// Otherwise bring back whatever was previously saved (if anything),
				// falling back to the empty string if nothing was stored.
				if ( this.setAttribute ) {
					this.setAttribute( "class",
						className || value === false ?
						"" :
						dataPriv.get( this, "__className__" ) || ""
					);
				}
			}
		} );
	},

	hasClass: function( selector ) {
		var className, elem,
			i = 0;

		className = " " + selector + " ";
		while ( ( elem = this[ i++ ] ) ) {
			if ( elem.nodeType === 1 &&
				( " " + getClass( elem ) + " " ).replace( rclass, " " )
					.indexOf( className ) > -1
			) {
				return true;
			}
		}

		return false;
	}
} );




var rreturn = /\r/g;

jQuery.fn.extend( {
	val: function( value ) {
		var hooks, ret, isFunction,
			elem = this[ 0 ];

		if ( !arguments.length ) {
			if ( elem ) {
				hooks = jQuery.valHooks[ elem.type ] ||
					jQuery.valHooks[ elem.nodeName.toLowerCase() ];

				if ( hooks &&
					"get" in hooks &&
					( ret = hooks.get( elem, "value" ) ) !== undefined
				) {
					return ret;
				}

				ret = elem.value;

				return typeof ret === "string" ?

					// Handle most common string cases
					ret.replace( rreturn, "" ) :

					// Handle cases where value is null/undef or number
					ret == null ? "" : ret;
			}

			return;
		}

		isFunction = jQuery.isFunction( value );

		return this.each( function( i ) {
			var val;

			if ( this.nodeType !== 1 ) {
				return;
			}

			if ( isFunction ) {
				val = value.call( this, i, jQuery( this ).val() );
			} else {
				val = value;
			}

			// Treat null/undefined as ""; convert numbers to string
			if ( val == null ) {
				val = "";

			} else if ( typeof val === "number" ) {
				val += "";

			} else if ( jQuery.isArray( val ) ) {
				val = jQuery.map( val, function( value ) {
					return value == null ? "" : value + "";
				} );
			}

			hooks = jQuery.valHooks[ this.type ] || jQuery.valHooks[ this.nodeName.toLowerCase() ];

			// If set returns undefined, fall back to normal setting
			if ( !hooks || !( "set" in hooks ) || hooks.set( this, val, "value" ) === undefined ) {
				this.value = val;
			}
		} );
	}
} );

jQuery.extend( {
	valHooks: {
		option: {
			get: function( elem ) {

				// Support: IE<11
				// option.value not trimmed (#14858)
				return jQuery.trim( elem.value );
			}
		},
		select: {
			get: function( elem ) {
				var value, option,
					options = elem.options,
					index = elem.selectedIndex,
					one = elem.type === "select-one" || index < 0,
					values = one ? null : [],
					max = one ? index + 1 : options.length,
					i = index < 0 ?
						max :
						one ? index : 0;

				// Loop through all the selected options
				for ( ; i < max; i++ ) {
					option = options[ i ];

					// IE8-9 doesn't update selected after form reset (#2551)
					if ( ( option.selected || i === index ) &&

							// Don't return options that are disabled or in a disabled optgroup
							( support.optDisabled ?
								!option.disabled : option.getAttribute( "disabled" ) === null ) &&
							( !option.parentNode.disabled ||
								!jQuery.nodeName( option.parentNode, "optgroup" ) ) ) {

						// Get the specific value for the option
						value = jQuery( option ).val();

						// We don't need an array for one selects
						if ( one ) {
							return value;
						}

						// Multi-Selects return an array
						values.push( value );
					}
				}

				return values;
			},

			set: function( elem, value ) {
				var optionSet, option,
					options = elem.options,
					values = jQuery.makeArray( value ),
					i = options.length;

				while ( i-- ) {
					option = options[ i ];
					if ( option.selected =
							jQuery.inArray( jQuery.valHooks.option.get( option ), values ) > -1
					) {
						optionSet = true;
					}
				}

				// Force browsers to behave consistently when non-matching value is set
				if ( !optionSet ) {
					elem.selectedIndex = -1;
				}
				return values;
			}
		}
	}
} );

// Radios and checkboxes getter/setter
jQuery.each( [ "radio", "checkbox" ], function() {
	jQuery.valHooks[ this ] = {
		set: function( elem, value ) {
			if ( jQuery.isArray( value ) ) {
				return ( elem.checked = jQuery.inArray( jQuery( elem ).val(), value ) > -1 );
			}
		}
	};
	if ( !support.checkOn ) {
		jQuery.valHooks[ this ].get = function( elem ) {
			return elem.getAttribute( "value" ) === null ? "on" : elem.value;
		};
	}
} );




// Return jQuery for attributes-only inclusion


var rfocusMorph = /^(?:focusinfocus|focusoutblur)$/;

jQuery.extend( jQuery.event, {

	trigger: function( event, data, elem, onlyHandlers ) {

		var i, cur, tmp, bubbleType, ontype, handle, special,
			eventPath = [ elem || document ],
			type = hasOwn.call( event, "type" ) ? event.type : event,
			namespaces = hasOwn.call( event, "namespace" ) ? event.namespace.split( "." ) : [];

		cur = tmp = elem = elem || document;

		// Don't do events on text and comment nodes
		if ( elem.nodeType === 3 || elem.nodeType === 8 ) {
			return;
		}

		// focus/blur morphs to focusin/out; ensure we're not firing them right now
		if ( rfocusMorph.test( type + jQuery.event.triggered ) ) {
			return;
		}

		if ( type.indexOf( "." ) > -1 ) {

			// Namespaced trigger; create a regexp to match event type in handle()
			namespaces = type.split( "." );
			type = namespaces.shift();
			namespaces.sort();
		}
		ontype = type.indexOf( ":" ) < 0 && "on" + type;

		// Caller can pass in a jQuery.Event object, Object, or just an event type string
		event = event[ jQuery.expando ] ?
			event :
			new jQuery.Event( type, typeof event === "object" && event );

		// Trigger bitmask: & 1 for native handlers; & 2 for jQuery (always true)
		event.isTrigger = onlyHandlers ? 2 : 3;
		event.namespace = namespaces.join( "." );
		event.rnamespace = event.namespace ?
			new RegExp( "(^|\\.)" + namespaces.join( "\\.(?:.*\\.|)" ) + "(\\.|$)" ) :
			null;

		// Clean up the event in case it is being reused
		event.result = undefined;
		if ( !event.target ) {
			event.target = elem;
		}

		// Clone any incoming data and prepend the event, creating the handler arg list
		data = data == null ?
			[ event ] :
			jQuery.makeArray( data, [ event ] );

		// Allow special events to draw outside the lines
		special = jQuery.event.special[ type ] || {};
		if ( !onlyHandlers && special.trigger && special.trigger.apply( elem, data ) === false ) {
			return;
		}

		// Determine event propagation path in advance, per W3C events spec (#9951)
		// Bubble up to document, then to window; watch for a global ownerDocument var (#9724)
		if ( !onlyHandlers && !special.noBubble && !jQuery.isWindow( elem ) ) {

			bubbleType = special.delegateType || type;
			if ( !rfocusMorph.test( bubbleType + type ) ) {
				cur = cur.parentNode;
			}
			for ( ; cur; cur = cur.parentNode ) {
				eventPath.push( cur );
				tmp = cur;
			}

			// Only add window if we got to document (e.g., not plain obj or detached DOM)
			if ( tmp === ( elem.ownerDocument || document ) ) {
				eventPath.push( tmp.defaultView || tmp.parentWindow || window );
			}
		}

		// Fire handlers on the event path
		i = 0;
		while ( ( cur = eventPath[ i++ ] ) && !event.isPropagationStopped() ) {

			event.type = i > 1 ?
				bubbleType :
				special.bindType || type;

			// jQuery handler
			handle = ( dataPriv.get( cur, "events" ) || {} )[ event.type ] &&
				dataPriv.get( cur, "handle" );
			if ( handle ) {
				handle.apply( cur, data );
			}

			// Native handler
			handle = ontype && cur[ ontype ];
			if ( handle && handle.apply && acceptData( cur ) ) {
				event.result = handle.apply( cur, data );
				if ( event.result === false ) {
					event.preventDefault();
				}
			}
		}
		event.type = type;

		// If nobody prevented the default action, do it now
		if ( !onlyHandlers && !event.isDefaultPrevented() ) {

			if ( ( !special._default ||
				special._default.apply( eventPath.pop(), data ) === false ) &&
				acceptData( elem ) ) {

				// Call a native DOM method on the target with the same name name as the event.
				// Don't do default actions on window, that's where global variables be (#6170)
				if ( ontype && jQuery.isFunction( elem[ type ] ) && !jQuery.isWindow( elem ) ) {

					// Don't re-trigger an onFOO event when we call its FOO() method
					tmp = elem[ ontype ];

					if ( tmp ) {
						elem[ ontype ] = null;
					}

					// Prevent re-triggering of the same event, since we already bubbled it above
					jQuery.event.triggered = type;
					elem[ type ]();
					jQuery.event.triggered = undefined;

					if ( tmp ) {
						elem[ ontype ] = tmp;
					}
				}
			}
		}

		return event.result;
	},

	// Piggyback on a donor event to simulate a different one
	simulate: function( type, elem, event ) {
		var e = jQuery.extend(
			new jQuery.Event(),
			event,
			{
				type: type,
				isSimulated: true

				// Previously, `originalEvent: {}` was set here, so stopPropagation call
				// would not be triggered on donor event, since in our own
				// jQuery.event.stopPropagation function we had a check for existence of
				// originalEvent.stopPropagation method, so, consequently it would be a noop.
				//
				// But now, this "simulate" function is used only for events
				// for which stopPropagation() is noop, so there is no need for that anymore.
				//
				// For the compat branch though, guard for "click" and "submit"
				// events is still used, but was moved to jQuery.event.stopPropagation function
				// because `originalEvent` should point to the original event for the constancy
				// with other events and for more focused logic
			}
		);

		jQuery.event.trigger( e, null, elem );

		if ( e.isDefaultPrevented() ) {
			event.preventDefault();
		}
	}

} );

jQuery.fn.extend( {

	trigger: function( type, data ) {
		return this.each( function() {
			jQuery.event.trigger( type, data, this );
		} );
	},
	triggerHandler: function( type, data ) {
		var elem = this[ 0 ];
		if ( elem ) {
			return jQuery.event.trigger( type, data, elem, true );
		}
	}
} );


jQuery.each( ( "blur focus focusin focusout load resize scroll unload click dblclick " +
	"mousedown mouseup mousemove mouseover mouseout mouseenter mouseleave " +
	"change select submit keydown keypress keyup error contextmenu" ).split( " " ),
	function( i, name ) {

	// Handle event binding
	jQuery.fn[ name ] = function( data, fn ) {
		return arguments.length > 0 ?
			this.on( name, null, data, fn ) :
			this.trigger( name );
	};
} );

jQuery.fn.extend( {
	hover: function( fnOver, fnOut ) {
		return this.mouseenter( fnOver ).mouseleave( fnOut || fnOver );
	}
} );




support.focusin = "onfocusin" in window;


// Support: Firefox
// Firefox doesn't have focus(in | out) events
// Related ticket - https://bugzilla.mozilla.org/show_bug.cgi?id=687787
//
// Support: Chrome, Safari
// focus(in | out) events fire after focus & blur events,
// which is spec violation - http://www.w3.org/TR/DOM-Level-3-Events/#events-focusevent-event-order
// Related ticket - https://code.google.com/p/chromium/issues/detail?id=449857
if ( !support.focusin ) {
	jQuery.each( { focus: "focusin", blur: "focusout" }, function( orig, fix ) {

		// Attach a single capturing handler on the document while someone wants focusin/focusout
		var handler = function( event ) {
			jQuery.event.simulate( fix, event.target, jQuery.event.fix( event ) );
		};

		jQuery.event.special[ fix ] = {
			setup: function() {
				var doc = this.ownerDocument || this,
					attaches = dataPriv.access( doc, fix );

				if ( !attaches ) {
					doc.addEventListener( orig, handler, true );
				}
				dataPriv.access( doc, fix, ( attaches || 0 ) + 1 );
			},
			teardown: function() {
				var doc = this.ownerDocument || this,
					attaches = dataPriv.access( doc, fix ) - 1;

				if ( !attaches ) {
					doc.removeEventListener( orig, handler, true );
					dataPriv.remove( doc, fix );

				} else {
					dataPriv.access( doc, fix, attaches );
				}
			}
		};
	} );
}
var location = window.location;

var nonce = jQuery.now();

var rquery = ( /\?/ );



// Support: Android 2.3
// Workaround failure to string-cast null input
jQuery.parseJSON = function( data ) {
	return JSON.parse( data + "" );
};


// Cross-browser xml parsing
jQuery.parseXML = function( data ) {
	var xml;
	if ( !data || typeof data !== "string" ) {
		return null;
	}

	// Support: IE9
	try {
		xml = ( new window.DOMParser() ).parseFromString( data, "text/xml" );
	} catch ( e ) {
		xml = undefined;
	}

	if ( !xml || xml.getElementsByTagName( "parsererror" ).length ) {
		jQuery.error( "Invalid XML: " + data );
	}
	return xml;
};


var
	rhash = /#.*$/,
	rts = /([?&])_=[^&]*/,
	rheaders = /^(.*?):[ \t]*([^\r\n]*)$/mg,

	// #7653, #8125, #8152: local protocol detection
	rlocalProtocol = /^(?:about|app|app-storage|.+-extension|file|res|widget):$/,
	rnoContent = /^(?:GET|HEAD)$/,
	rprotocol = /^\/\//,

	/* Prefilters
	 * 1) They are useful to introduce custom dataTypes (see ajax/jsonp.js for an example)
	 * 2) These are called:
	 *    - BEFORE asking for a transport
	 *    - AFTER param serialization (s.data is a string if s.processData is true)
	 * 3) key is the dataType
	 * 4) the catchall symbol "*" can be used
	 * 5) execution will start with transport dataType and THEN continue down to "*" if needed
	 */
	prefilters = {},

	/* Transports bindings
	 * 1) key is the dataType
	 * 2) the catchall symbol "*" can be used
	 * 3) selection will start with transport dataType and THEN go to "*" if needed
	 */
	transports = {},

	// Avoid comment-prolog char sequence (#10098); must appease lint and evade compression
	allTypes = "*/".concat( "*" ),

	// Anchor tag for parsing the document origin
	originAnchor = document.createElement( "a" );
	originAnchor.href = location.href;

// Base "constructor" for jQuery.ajaxPrefilter and jQuery.ajaxTransport
function addToPrefiltersOrTransports( structure ) {

	// dataTypeExpression is optional and defaults to "*"
	return function( dataTypeExpression, func ) {

		if ( typeof dataTypeExpression !== "string" ) {
			func = dataTypeExpression;
			dataTypeExpression = "*";
		}

		var dataType,
			i = 0,
			dataTypes = dataTypeExpression.toLowerCase().match( rnotwhite ) || [];

		if ( jQuery.isFunction( func ) ) {

			// For each dataType in the dataTypeExpression
			while ( ( dataType = dataTypes[ i++ ] ) ) {

				// Prepend if requested
				if ( dataType[ 0 ] === "+" ) {
					dataType = dataType.slice( 1 ) || "*";
					( structure[ dataType ] = structure[ dataType ] || [] ).unshift( func );

				// Otherwise append
				} else {
					( structure[ dataType ] = structure[ dataType ] || [] ).push( func );
				}
			}
		}
	};
}

// Base inspection function for prefilters and transports
function inspectPrefiltersOrTransports( structure, options, originalOptions, jqXHR ) {

	var inspected = {},
		seekingTransport = ( structure === transports );

	function inspect( dataType ) {
		var selected;
		inspected[ dataType ] = true;
		jQuery.each( structure[ dataType ] || [], function( _, prefilterOrFactory ) {
			var dataTypeOrTransport = prefilterOrFactory( options, originalOptions, jqXHR );
			if ( typeof dataTypeOrTransport === "string" &&
				!seekingTransport && !inspected[ dataTypeOrTransport ] ) {

				options.dataTypes.unshift( dataTypeOrTransport );
				inspect( dataTypeOrTransport );
				return false;
			} else if ( seekingTransport ) {
				return !( selected = dataTypeOrTransport );
			}
		} );
		return selected;
	}

	return inspect( options.dataTypes[ 0 ] ) || !inspected[ "*" ] && inspect( "*" );
}

// A special extend for ajax options
// that takes "flat" options (not to be deep extended)
// Fixes #9887
function ajaxExtend( target, src ) {
	var key, deep,
		flatOptions = jQuery.ajaxSettings.flatOptions || {};

	for ( key in src ) {
		if ( src[ key ] !== undefined ) {
			( flatOptions[ key ] ? target : ( deep || ( deep = {} ) ) )[ key ] = src[ key ];
		}
	}
	if ( deep ) {
		jQuery.extend( true, target, deep );
	}

	return target;
}

/* Handles responses to an ajax request:
 * - finds the right dataType (mediates between content-type and expected dataType)
 * - returns the corresponding response
 */
function ajaxHandleResponses( s, jqXHR, responses ) {

	var ct, type, finalDataType, firstDataType,
		contents = s.contents,
		dataTypes = s.dataTypes;

	// Remove auto dataType and get content-type in the process
	while ( dataTypes[ 0 ] === "*" ) {
		dataTypes.shift();
		if ( ct === undefined ) {
			ct = s.mimeType || jqXHR.getResponseHeader( "Content-Type" );
		}
	}

	// Check if we're dealing with a known content-type
	if ( ct ) {
		for ( type in contents ) {
			if ( contents[ type ] && contents[ type ].test( ct ) ) {
				dataTypes.unshift( type );
				break;
			}
		}
	}

	// Check to see if we have a response for the expected dataType
	if ( dataTypes[ 0 ] in responses ) {
		finalDataType = dataTypes[ 0 ];
	} else {

		// Try convertible dataTypes
		for ( type in responses ) {
			if ( !dataTypes[ 0 ] || s.converters[ type + " " + dataTypes[ 0 ] ] ) {
				finalDataType = type;
				break;
			}
			if ( !firstDataType ) {
				firstDataType = type;
			}
		}

		// Or just use first one
		finalDataType = finalDataType || firstDataType;
	}

	// If we found a dataType
	// We add the dataType to the list if needed
	// and return the corresponding response
	if ( finalDataType ) {
		if ( finalDataType !== dataTypes[ 0 ] ) {
			dataTypes.unshift( finalDataType );
		}
		return responses[ finalDataType ];
	}
}

/* Chain conversions given the request and the original response
 * Also sets the responseXXX fields on the jqXHR instance
 */
function ajaxConvert( s, response, jqXHR, isSuccess ) {
	var conv2, current, conv, tmp, prev,
		converters = {},

		// Work with a copy of dataTypes in case we need to modify it for conversion
		dataTypes = s.dataTypes.slice();

	// Create converters map with lowercased keys
	if ( dataTypes[ 1 ] ) {
		for ( conv in s.converters ) {
			converters[ conv.toLowerCase() ] = s.converters[ conv ];
		}
	}

	current = dataTypes.shift();

	// Convert to each sequential dataType
	while ( current ) {

		if ( s.responseFields[ current ] ) {
			jqXHR[ s.responseFields[ current ] ] = response;
		}

		// Apply the dataFilter if provided
		if ( !prev && isSuccess && s.dataFilter ) {
			response = s.dataFilter( response, s.dataType );
		}

		prev = current;
		current = dataTypes.shift();

		if ( current ) {

		// There's only work to do if current dataType is non-auto
			if ( current === "*" ) {

				current = prev;

			// Convert response if prev dataType is non-auto and differs from current
			} else if ( prev !== "*" && prev !== current ) {

				// Seek a direct converter
				conv = converters[ prev + " " + current ] || converters[ "* " + current ];

				// If none found, seek a pair
				if ( !conv ) {
					for ( conv2 in converters ) {

						// If conv2 outputs current
						tmp = conv2.split( " " );
						if ( tmp[ 1 ] === current ) {

							// If prev can be converted to accepted input
							conv = converters[ prev + " " + tmp[ 0 ] ] ||
								converters[ "* " + tmp[ 0 ] ];
							if ( conv ) {

								// Condense equivalence converters
								if ( conv === true ) {
									conv = converters[ conv2 ];

								// Otherwise, insert the intermediate dataType
								} else if ( converters[ conv2 ] !== true ) {
									current = tmp[ 0 ];
									dataTypes.unshift( tmp[ 1 ] );
								}
								break;
							}
						}
					}
				}

				// Apply converter (if not an equivalence)
				if ( conv !== true ) {

					// Unless errors are allowed to bubble, catch and return them
					if ( conv && s.throws ) {
						response = conv( response );
					} else {
						try {
							response = conv( response );
						} catch ( e ) {
							return {
								state: "parsererror",
								error: conv ? e : "No conversion from " + prev + " to " + current
							};
						}
					}
				}
			}
		}
	}

	return { state: "success", data: response };
}

jQuery.extend( {

	// Counter for holding the number of active queries
	active: 0,

	// Last-Modified header cache for next request
	lastModified: {},
	etag: {},

	ajaxSettings: {
		url: location.href,
		type: "GET",
		isLocal: rlocalProtocol.test( location.protocol ),
		global: true,
		processData: true,
		async: true,
		contentType: "application/x-www-form-urlencoded; charset=UTF-8",
		/*
		timeout: 0,
		data: null,
		dataType: null,
		username: null,
		password: null,
		cache: null,
		throws: false,
		traditional: false,
		headers: {},
		*/

		accepts: {
			"*": allTypes,
			text: "text/plain",
			html: "text/html",
			xml: "application/xml, text/xml",
			json: "application/json, text/javascript"
		},

		contents: {
			xml: /\bxml\b/,
			html: /\bhtml/,
			json: /\bjson\b/
		},

		responseFields: {
			xml: "responseXML",
			text: "responseText",
			json: "responseJSON"
		},

		// Data converters
		// Keys separate source (or catchall "*") and destination types with a single space
		converters: {

			// Convert anything to text
			"* text": String,

			// Text to html (true = no transformation)
			"text html": true,

			// Evaluate text as a json expression
			"text json": jQuery.parseJSON,

			// Parse text as xml
			"text xml": jQuery.parseXML
		},

		// For options that shouldn't be deep extended:
		// you can add your own custom options here if
		// and when you create one that shouldn't be
		// deep extended (see ajaxExtend)
		flatOptions: {
			url: true,
			context: true
		}
	},

	// Creates a full fledged settings object into target
	// with both ajaxSettings and settings fields.
	// If target is omitted, writes into ajaxSettings.
	ajaxSetup: function( target, settings ) {
		return settings ?

			// Building a settings object
			ajaxExtend( ajaxExtend( target, jQuery.ajaxSettings ), settings ) :

			// Extending ajaxSettings
			ajaxExtend( jQuery.ajaxSettings, target );
	},

	ajaxPrefilter: addToPrefiltersOrTransports( prefilters ),
	ajaxTransport: addToPrefiltersOrTransports( transports ),

	// Main method
	ajax: function( url, options ) {

		// If url is an object, simulate pre-1.5 signature
		if ( typeof url === "object" ) {
			options = url;
			url = undefined;
		}

		// Force options to be an object
		options = options || {};

		var transport,

			// URL without anti-cache param
			cacheURL,

			// Response headers
			responseHeadersString,
			responseHeaders,

			// timeout handle
			timeoutTimer,

			// Url cleanup var
			urlAnchor,

			// To know if global events are to be dispatched
			fireGlobals,

			// Loop variable
			i,

			// Create the final options object
			s = jQuery.ajaxSetup( {}, options ),

			// Callbacks context
			callbackContext = s.context || s,

			// Context for global events is callbackContext if it is a DOM node or jQuery collection
			globalEventContext = s.context &&
				( callbackContext.nodeType || callbackContext.jquery ) ?
					jQuery( callbackContext ) :
					jQuery.event,

			// Deferreds
			deferred = jQuery.Deferred(),
			completeDeferred = jQuery.Callbacks( "once memory" ),

			// Status-dependent callbacks
			statusCode = s.statusCode || {},

			// Headers (they are sent all at once)
			requestHeaders = {},
			requestHeadersNames = {},

			// The jqXHR state
			state = 0,

			// Default abort message
			strAbort = "canceled",

			// Fake xhr
			jqXHR = {
				readyState: 0,

				// Builds headers hashtable if needed
				getResponseHeader: function( key ) {
					var match;
					if ( state === 2 ) {
						if ( !responseHeaders ) {
							responseHeaders = {};
							while ( ( match = rheaders.exec( responseHeadersString ) ) ) {
								responseHeaders[ match[ 1 ].toLowerCase() ] = match[ 2 ];
							}
						}
						match = responseHeaders[ key.toLowerCase() ];
					}
					return match == null ? null : match;
				},

				// Raw string
				getAllResponseHeaders: function() {
					return state === 2 ? responseHeadersString : null;
				},

				// Caches the header
				setRequestHeader: function( name, value ) {
					var lname = name.toLowerCase();
					if ( !state ) {
						name = requestHeadersNames[ lname ] = requestHeadersNames[ lname ] || name;
						requestHeaders[ name ] = value;
					}
					return this;
				},

				// Overrides response content-type header
				overrideMimeType: function( type ) {
					if ( !state ) {
						s.mimeType = type;
					}
					return this;
				},

				// Status-dependent callbacks
				statusCode: function( map ) {
					var code;
					if ( map ) {
						if ( state < 2 ) {
							for ( code in map ) {

								// Lazy-add the new callback in a way that preserves old ones
								statusCode[ code ] = [ statusCode[ code ], map[ code ] ];
							}
						} else {

							// Execute the appropriate callbacks
							jqXHR.always( map[ jqXHR.status ] );
						}
					}
					return this;
				},

				// Cancel the request
				abort: function( statusText ) {
					var finalText = statusText || strAbort;
					if ( transport ) {
						transport.abort( finalText );
					}
					done( 0, finalText );
					return this;
				}
			};

		// Attach deferreds
		deferred.promise( jqXHR ).complete = completeDeferred.add;
		jqXHR.success = jqXHR.done;
		jqXHR.error = jqXHR.fail;

		// Remove hash character (#7531: and string promotion)
		// Add protocol if not provided (prefilters might expect it)
		// Handle falsy url in the settings object (#10093: consistency with old signature)
		// We also use the url parameter if available
		s.url = ( ( url || s.url || location.href ) + "" ).replace( rhash, "" )
			.replace( rprotocol, location.protocol + "//" );

		// Alias method option to type as per ticket #12004
		s.type = options.method || options.type || s.method || s.type;

		// Extract dataTypes list
		s.dataTypes = jQuery.trim( s.dataType || "*" ).toLowerCase().match( rnotwhite ) || [ "" ];

		// A cross-domain request is in order when the origin doesn't match the current origin.
		if ( s.crossDomain == null ) {
			urlAnchor = document.createElement( "a" );

			// Support: IE8-11+
			// IE throws exception if url is malformed, e.g. http://example.com:80x/
			try {
				urlAnchor.href = s.url;

				// Support: IE8-11+
				// Anchor's host property isn't correctly set when s.url is relative
				urlAnchor.href = urlAnchor.href;
				s.crossDomain = originAnchor.protocol + "//" + originAnchor.host !==
					urlAnchor.protocol + "//" + urlAnchor.host;
			} catch ( e ) {

				// If there is an error parsing the URL, assume it is crossDomain,
				// it can be rejected by the transport if it is invalid
				s.crossDomain = true;
			}
		}

		// Convert data if not already a string
		if ( s.data && s.processData && typeof s.data !== "string" ) {
			s.data = jQuery.param( s.data, s.traditional );
		}

		// Apply prefilters
		inspectPrefiltersOrTransports( prefilters, s, options, jqXHR );

		// If request was aborted inside a prefilter, stop there
		if ( state === 2 ) {
			return jqXHR;
		}

		// We can fire global events as of now if asked to
		// Don't fire events if jQuery.event is undefined in an AMD-usage scenario (#15118)
		fireGlobals = jQuery.event && s.global;

		// Watch for a new set of requests
		if ( fireGlobals && jQuery.active++ === 0 ) {
			jQuery.event.trigger( "ajaxStart" );
		}

		// Uppercase the type
		s.type = s.type.toUpperCase();

		// Determine if request has content
		s.hasContent = !rnoContent.test( s.type );

		// Save the URL in case we're toying with the If-Modified-Since
		// and/or If-None-Match header later on
		cacheURL = s.url;

		// More options handling for requests with no content
		if ( !s.hasContent ) {

			// If data is available, append data to url
			if ( s.data ) {
				cacheURL = ( s.url += ( rquery.test( cacheURL ) ? "&" : "?" ) + s.data );

				// #9682: remove data so that it's not used in an eventual retry
				delete s.data;
			}

			// Add anti-cache in url if needed
			if ( s.cache === false ) {
				s.url = rts.test( cacheURL ) ?

					// If there is already a '_' parameter, set its value
					cacheURL.replace( rts, "$1_=" + nonce++ ) :

					// Otherwise add one to the end
					cacheURL + ( rquery.test( cacheURL ) ? "&" : "?" ) + "_=" + nonce++;
			}
		}

		// Set the If-Modified-Since and/or If-None-Match header, if in ifModified mode.
		if ( s.ifModified ) {
			if ( jQuery.lastModified[ cacheURL ] ) {
				jqXHR.setRequestHeader( "If-Modified-Since", jQuery.lastModified[ cacheURL ] );
			}
			if ( jQuery.etag[ cacheURL ] ) {
				jqXHR.setRequestHeader( "If-None-Match", jQuery.etag[ cacheURL ] );
			}
		}

		// Set the correct header, if data is being sent
		if ( s.data && s.hasContent && s.contentType !== false || options.contentType ) {
			jqXHR.setRequestHeader( "Content-Type", s.contentType );
		}

		// Set the Accepts header for the server, depending on the dataType
		jqXHR.setRequestHeader(
			"Accept",
			s.dataTypes[ 0 ] && s.accepts[ s.dataTypes[ 0 ] ] ?
				s.accepts[ s.dataTypes[ 0 ] ] +
					( s.dataTypes[ 0 ] !== "*" ? ", " + allTypes + "; q=0.01" : "" ) :
				s.accepts[ "*" ]
		);

		// Check for headers option
		for ( i in s.headers ) {
			jqXHR.setRequestHeader( i, s.headers[ i ] );
		}

		// Allow custom headers/mimetypes and early abort
		if ( s.beforeSend &&
			( s.beforeSend.call( callbackContext, jqXHR, s ) === false || state === 2 ) ) {

			// Abort if not done already and return
			return jqXHR.abort();
		}

		// Aborting is no longer a cancellation
		strAbort = "abort";

		// Install callbacks on deferreds
		for ( i in { success: 1, error: 1, complete: 1 } ) {
			jqXHR[ i ]( s[ i ] );
		}

		// Get transport
		transport = inspectPrefiltersOrTransports( transports, s, options, jqXHR );

		// If no transport, we auto-abort
		if ( !transport ) {
			done( -1, "No Transport" );
		} else {
			jqXHR.readyState = 1;

			// Send global event
			if ( fireGlobals ) {
				globalEventContext.trigger( "ajaxSend", [ jqXHR, s ] );
			}

			// If request was aborted inside ajaxSend, stop there
			if ( state === 2 ) {
				return jqXHR;
			}

			// Timeout
			if ( s.async && s.timeout > 0 ) {
				timeoutTimer = window.setTimeout( function() {
					jqXHR.abort( "timeout" );
				}, s.timeout );
			}

			try {
				state = 1;
				transport.send( requestHeaders, done );
			} catch ( e ) {

				// Propagate exception as error if not done
				if ( state < 2 ) {
					done( -1, e );

				// Simply rethrow otherwise
				} else {
					throw e;
				}
			}
		}

		// Callback for when everything is done
		function done( status, nativeStatusText, responses, headers ) {
			var isSuccess, success, error, response, modified,
				statusText = nativeStatusText;

			// Called once
			if ( state === 2 ) {
				return;
			}

			// State is "done" now
			state = 2;

			// Clear timeout if it exists
			if ( timeoutTimer ) {
				window.clearTimeout( timeoutTimer );
			}

			// Dereference transport for early garbage collection
			// (no matter how long the jqXHR object will be used)
			transport = undefined;

			// Cache response headers
			responseHeadersString = headers || "";

			// Set readyState
			jqXHR.readyState = status > 0 ? 4 : 0;

			// Determine if successful
			isSuccess = status >= 200 && status < 300 || status === 304;

			// Get response data
			if ( responses ) {
				response = ajaxHandleResponses( s, jqXHR, responses );
			}

			// Convert no matter what (that way responseXXX fields are always set)
			response = ajaxConvert( s, response, jqXHR, isSuccess );

			// If successful, handle type chaining
			if ( isSuccess ) {

				// Set the If-Modified-Since and/or If-None-Match header, if in ifModified mode.
				if ( s.ifModified ) {
					modified = jqXHR.getResponseHeader( "Last-Modified" );
					if ( modified ) {
						jQuery.lastModified[ cacheURL ] = modified;
					}
					modified = jqXHR.getResponseHeader( "etag" );
					if ( modified ) {
						jQuery.etag[ cacheURL ] = modified;
					}
				}

				// if no content
				if ( status === 204 || s.type === "HEAD" ) {
					statusText = "nocontent";

				// if not modified
				} else if ( status === 304 ) {
					statusText = "notmodified";

				// If we have data, let's convert it
				} else {
					statusText = response.state;
					success = response.data;
					error = response.error;
					isSuccess = !error;
				}
			} else {

				// Extract error from statusText and normalize for non-aborts
				error = statusText;
				if ( status || !statusText ) {
					statusText = "error";
					if ( status < 0 ) {
						status = 0;
					}
				}
			}

			// Set data for the fake xhr object
			jqXHR.status = status;
			jqXHR.statusText = ( nativeStatusText || statusText ) + "";

			// Success/Error
			if ( isSuccess ) {
				deferred.resolveWith( callbackContext, [ success, statusText, jqXHR ] );
			} else {
				deferred.rejectWith( callbackContext, [ jqXHR, statusText, error ] );
			}

			// Status-dependent callbacks
			jqXHR.statusCode( statusCode );
			statusCode = undefined;

			if ( fireGlobals ) {
				globalEventContext.trigger( isSuccess ? "ajaxSuccess" : "ajaxError",
					[ jqXHR, s, isSuccess ? success : error ] );
			}

			// Complete
			completeDeferred.fireWith( callbackContext, [ jqXHR, statusText ] );

			if ( fireGlobals ) {
				globalEventContext.trigger( "ajaxComplete", [ jqXHR, s ] );

				// Handle the global AJAX counter
				if ( !( --jQuery.active ) ) {
					jQuery.event.trigger( "ajaxStop" );
				}
			}
		}

		return jqXHR;
	},

	getJSON: function( url, data, callback ) {
		return jQuery.get( url, data, callback, "json" );
	},

	getScript: function( url, callback ) {
		return jQuery.get( url, undefined, callback, "script" );
	}
} );

jQuery.each( [ "get", "post" ], function( i, method ) {
	jQuery[ method ] = function( url, data, callback, type ) {

		// Shift arguments if data argument was omitted
		if ( jQuery.isFunction( data ) ) {
			type = type || callback;
			callback = data;
			data = undefined;
		}

		// The url can be an options object (which then must have .url)
		return jQuery.ajax( jQuery.extend( {
			url: url,
			type: method,
			dataType: type,
			data: data,
			success: callback
		}, jQuery.isPlainObject( url ) && url ) );
	};
} );


jQuery._evalUrl = function( url ) {
	return jQuery.ajax( {
		url: url,

		// Make this explicit, since user can override this through ajaxSetup (#11264)
		type: "GET",
		dataType: "script",
		async: false,
		global: false,
		"throws": true
	} );
};


jQuery.fn.extend( {
	wrapAll: function( html ) {
		var wrap;

		if ( jQuery.isFunction( html ) ) {
			return this.each( function( i ) {
				jQuery( this ).wrapAll( html.call( this, i ) );
			} );
		}

		if ( this[ 0 ] ) {

			// The elements to wrap the target around
			wrap = jQuery( html, this[ 0 ].ownerDocument ).eq( 0 ).clone( true );

			if ( this[ 0 ].parentNode ) {
				wrap.insertBefore( this[ 0 ] );
			}

			wrap.map( function() {
				var elem = this;

				while ( elem.firstElementChild ) {
					elem = elem.firstElementChild;
				}

				return elem;
			} ).append( this );
		}

		return this;
	},

	wrapInner: function( html ) {
		if ( jQuery.isFunction( html ) ) {
			return this.each( function( i ) {
				jQuery( this ).wrapInner( html.call( this, i ) );
			} );
		}

		return this.each( function() {
			var self = jQuery( this ),
				contents = self.contents();

			if ( contents.length ) {
				contents.wrapAll( html );

			} else {
				self.append( html );
			}
		} );
	},

	wrap: function( html ) {
		var isFunction = jQuery.isFunction( html );

		return this.each( function( i ) {
			jQuery( this ).wrapAll( isFunction ? html.call( this, i ) : html );
		} );
	},

	unwrap: function() {
		return this.parent().each( function() {
			if ( !jQuery.nodeName( this, "body" ) ) {
				jQuery( this ).replaceWith( this.childNodes );
			}
		} ).end();
	}
} );


jQuery.expr.filters.hidden = function( elem ) {
	return !jQuery.expr.filters.visible( elem );
};
jQuery.expr.filters.visible = function( elem ) {

	// Support: Opera <= 12.12
	// Opera reports offsetWidths and offsetHeights less than zero on some elements
	// Use OR instead of AND as the element is not visible if either is true
	// See tickets #10406 and #13132
	return elem.offsetWidth > 0 || elem.offsetHeight > 0 || elem.getClientRects().length > 0;
};




var r20 = /%20/g,
	rbracket = /\[\]$/,
	rCRLF = /\r?\n/g,
	rsubmitterTypes = /^(?:submit|button|image|reset|file)$/i,
	rsubmittable = /^(?:input|select|textarea|keygen)/i;

function buildParams( prefix, obj, traditional, add ) {
	var name;

	if ( jQuery.isArray( obj ) ) {

		// Serialize array item.
		jQuery.each( obj, function( i, v ) {
			if ( traditional || rbracket.test( prefix ) ) {

				// Treat each array item as a scalar.
				add( prefix, v );

			} else {

				// Item is non-scalar (array or object), encode its numeric index.
				buildParams(
					prefix + "[" + ( typeof v === "object" && v != null ? i : "" ) + "]",
					v,
					traditional,
					add
				);
			}
		} );

	} else if ( !traditional && jQuery.type( obj ) === "object" ) {

		// Serialize object item.
		for ( name in obj ) {
			buildParams( prefix + "[" + name + "]", obj[ name ], traditional, add );
		}

	} else {

		// Serialize scalar item.
		add( prefix, obj );
	}
}

// Serialize an array of form elements or a set of
// key/values into a query string
jQuery.param = function( a, traditional ) {
	var prefix,
		s = [],
		add = function( key, value ) {

			// If value is a function, invoke it and return its value
			value = jQuery.isFunction( value ) ? value() : ( value == null ? "" : value );
			s[ s.length ] = encodeURIComponent( key ) + "=" + encodeURIComponent( value );
		};

	// Set traditional to true for jQuery <= 1.3.2 behavior.
	if ( traditional === undefined ) {
		traditional = jQuery.ajaxSettings && jQuery.ajaxSettings.traditional;
	}

	// If an array was passed in, assume that it is an array of form elements.
	if ( jQuery.isArray( a ) || ( a.jquery && !jQuery.isPlainObject( a ) ) ) {

		// Serialize the form elements
		jQuery.each( a, function() {
			add( this.name, this.value );
		} );

	} else {

		// If traditional, encode the "old" way (the way 1.3.2 or older
		// did it), otherwise encode params recursively.
		for ( prefix in a ) {
			buildParams( prefix, a[ prefix ], traditional, add );
		}
	}

	// Return the resulting serialization
	return s.join( "&" ).replace( r20, "+" );
};

jQuery.fn.extend( {
	serialize: function() {
		return jQuery.param( this.serializeArray() );
	},
	serializeArray: function() {
		return this.map( function() {

			// Can add propHook for "elements" to filter or add form elements
			var elements = jQuery.prop( this, "elements" );
			return elements ? jQuery.makeArray( elements ) : this;
		} )
		.filter( function() {
			var type = this.type;

			// Use .is( ":disabled" ) so that fieldset[disabled] works
			return this.name && !jQuery( this ).is( ":disabled" ) &&
				rsubmittable.test( this.nodeName ) && !rsubmitterTypes.test( type ) &&
				( this.checked || !rcheckableType.test( type ) );
		} )
		.map( function( i, elem ) {
			var val = jQuery( this ).val();

			return val == null ?
				null :
				jQuery.isArray( val ) ?
					jQuery.map( val, function( val ) {
						return { name: elem.name, value: val.replace( rCRLF, "\r\n" ) };
					} ) :
					{ name: elem.name, value: val.replace( rCRLF, "\r\n" ) };
		} ).get();
	}
} );


jQuery.ajaxSettings.xhr = function() {
	try {
		return new window.XMLHttpRequest();
	} catch ( e ) {}
};

var xhrSuccessStatus = {

		// File protocol always yields status code 0, assume 200
		0: 200,

		// Support: IE9
		// #1450: sometimes IE returns 1223 when it should be 204
		1223: 204
	},
	xhrSupported = jQuery.ajaxSettings.xhr();

support.cors = !!xhrSupported && ( "withCredentials" in xhrSupported );
support.ajax = xhrSupported = !!xhrSupported;

jQuery.ajaxTransport( function( options ) {
	var callback, errorCallback;

	// Cross domain only allowed if supported through XMLHttpRequest
	if ( support.cors || xhrSupported && !options.crossDomain ) {
		return {
			send: function( headers, complete ) {
				var i,
					xhr = options.xhr();

				xhr.open(
					options.type,
					options.url,
					options.async,
					options.username,
					options.password
				);

				// Apply custom fields if provided
				if ( options.xhrFields ) {
					for ( i in options.xhrFields ) {
						xhr[ i ] = options.xhrFields[ i ];
					}
				}

				// Override mime type if needed
				if ( options.mimeType && xhr.overrideMimeType ) {
					xhr.overrideMimeType( options.mimeType );
				}

				// X-Requested-With header
				// For cross-domain requests, seeing as conditions for a preflight are
				// akin to a jigsaw puzzle, we simply never set it to be sure.
				// (it can always be set on a per-request basis or even using ajaxSetup)
				// For same-domain requests, won't change header if already provided.
				if ( !options.crossDomain && !headers[ "X-Requested-With" ] ) {
					headers[ "X-Requested-With" ] = "XMLHttpRequest";
				}

				// Set headers
				for ( i in headers ) {
					xhr.setRequestHeader( i, headers[ i ] );
				}

				// Callback
				callback = function( type ) {
					return function() {
						if ( callback ) {
							callback = errorCallback = xhr.onload =
								xhr.onerror = xhr.onabort = xhr.onreadystatechange = null;

							if ( type === "abort" ) {
								xhr.abort();
							} else if ( type === "error" ) {

								// Support: IE9
								// On a manual native abort, IE9 throws
								// errors on any property access that is not readyState
								if ( typeof xhr.status !== "number" ) {
									complete( 0, "error" );
								} else {
									complete(

										// File: protocol always yields status 0; see #8605, #14207
										xhr.status,
										xhr.statusText
									);
								}
							} else {
								complete(
									xhrSuccessStatus[ xhr.status ] || xhr.status,
									xhr.statusText,

									// Support: IE9 only
									// IE9 has no XHR2 but throws on binary (trac-11426)
									// For XHR2 non-text, let the caller handle it (gh-2498)
									( xhr.responseType || "text" ) !== "text"  ||
									typeof xhr.responseText !== "string" ?
										{ binary: xhr.response } :
										{ text: xhr.responseText },
									xhr.getAllResponseHeaders()
								);
							}
						}
					};
				};

				// Listen to events
				xhr.onload = callback();
				errorCallback = xhr.onerror = callback( "error" );

				// Support: IE9
				// Use onreadystatechange to replace onabort
				// to handle uncaught aborts
				if ( xhr.onabort !== undefined ) {
					xhr.onabort = errorCallback;
				} else {
					xhr.onreadystatechange = function() {

						// Check readyState before timeout as it changes
						if ( xhr.readyState === 4 ) {

							// Allow onerror to be called first,
							// but that will not handle a native abort
							// Also, save errorCallback to a variable
							// as xhr.onerror cannot be accessed
							window.setTimeout( function() {
								if ( callback ) {
									errorCallback();
								}
							} );
						}
					};
				}

				// Create the abort callback
				callback = callback( "abort" );

				try {

					// Do send the request (this may raise an exception)
					xhr.send( options.hasContent && options.data || null );
				} catch ( e ) {

					// #14683: Only rethrow if this hasn't been notified as an error yet
					if ( callback ) {
						throw e;
					}
				}
			},

			abort: function() {
				if ( callback ) {
					callback();
				}
			}
		};
	}
} );




// Install script dataType
jQuery.ajaxSetup( {
	accepts: {
		script: "text/javascript, application/javascript, " +
			"application/ecmascript, application/x-ecmascript"
	},
	contents: {
		script: /\b(?:java|ecma)script\b/
	},
	converters: {
		"text script": function( text ) {
			jQuery.globalEval( text );
			return text;
		}
	}
} );

// Handle cache's special case and crossDomain
jQuery.ajaxPrefilter( "script", function( s ) {
	if ( s.cache === undefined ) {
		s.cache = false;
	}
	if ( s.crossDomain ) {
		s.type = "GET";
	}
} );

// Bind script tag hack transport
jQuery.ajaxTransport( "script", function( s ) {

	// This transport only deals with cross domain requests
	if ( s.crossDomain ) {
		var script, callback;
		return {
			send: function( _, complete ) {
				script = jQuery( "<script>" ).prop( {
					charset: s.scriptCharset,
					src: s.url
				} ).on(
					"load error",
					callback = function( evt ) {
						script.remove();
						callback = null;
						if ( evt ) {
							complete( evt.type === "error" ? 404 : 200, evt.type );
						}
					}
				);

				// Use native DOM manipulation to avoid our domManip AJAX trickery
				document.head.appendChild( script[ 0 ] );
			},
			abort: function() {
				if ( callback ) {
					callback();
				}
			}
		};
	}
} );




var oldCallbacks = [],
	rjsonp = /(=)\?(?=&|$)|\?\?/;

// Default jsonp settings
jQuery.ajaxSetup( {
	jsonp: "callback",
	jsonpCallback: function() {
		var callback = oldCallbacks.pop() || ( jQuery.expando + "_" + ( nonce++ ) );
		this[ callback ] = true;
		return callback;
	}
} );

// Detect, normalize options and install callbacks for jsonp requests
jQuery.ajaxPrefilter( "json jsonp", function( s, originalSettings, jqXHR ) {

	var callbackName, overwritten, responseContainer,
		jsonProp = s.jsonp !== false && ( rjsonp.test( s.url ) ?
			"url" :
			typeof s.data === "string" &&
				( s.contentType || "" )
					.indexOf( "application/x-www-form-urlencoded" ) === 0 &&
				rjsonp.test( s.data ) && "data"
		);

	// Handle iff the expected data type is "jsonp" or we have a parameter to set
	if ( jsonProp || s.dataTypes[ 0 ] === "jsonp" ) {

		// Get callback name, remembering preexisting value associated with it
		callbackName = s.jsonpCallback = jQuery.isFunction( s.jsonpCallback ) ?
			s.jsonpCallback() :
			s.jsonpCallback;

		// Insert callback into url or form data
		if ( jsonProp ) {
			s[ jsonProp ] = s[ jsonProp ].replace( rjsonp, "$1" + callbackName );
		} else if ( s.jsonp !== false ) {
			s.url += ( rquery.test( s.url ) ? "&" : "?" ) + s.jsonp + "=" + callbackName;
		}

		// Use data converter to retrieve json after script execution
		s.converters[ "script json" ] = function() {
			if ( !responseContainer ) {
				jQuery.error( callbackName + " was not called" );
			}
			return responseContainer[ 0 ];
		};

		// Force json dataType
		s.dataTypes[ 0 ] = "json";

		// Install callback
		overwritten = window[ callbackName ];
		window[ callbackName ] = function() {
			responseContainer = arguments;
		};

		// Clean-up function (fires after converters)
		jqXHR.always( function() {

			// If previous value didn't exist - remove it
			if ( overwritten === undefined ) {
				jQuery( window ).removeProp( callbackName );

			// Otherwise restore preexisting value
			} else {
				window[ callbackName ] = overwritten;
			}

			// Save back as free
			if ( s[ callbackName ] ) {

				// Make sure that re-using the options doesn't screw things around
				s.jsonpCallback = originalSettings.jsonpCallback;

				// Save the callback name for future use
				oldCallbacks.push( callbackName );
			}

			// Call if it was a function and we have a response
			if ( responseContainer && jQuery.isFunction( overwritten ) ) {
				overwritten( responseContainer[ 0 ] );
			}

			responseContainer = overwritten = undefined;
		} );

		// Delegate to script
		return "script";
	}
} );




// Support: Safari 8+
// In Safari 8 documents created via document.implementation.createHTMLDocument
// collapse sibling forms: the second one becomes a child of the first one.
// Because of that, this security measure has to be disabled in Safari 8.
// https://bugs.webkit.org/show_bug.cgi?id=137337
support.createHTMLDocument = ( function() {
	var body = document.implementation.createHTMLDocument( "" ).body;
	body.innerHTML = "<form></form><form></form>";
	return body.childNodes.length === 2;
} )();


// Argument "data" should be string of html
// context (optional): If specified, the fragment will be created in this context,
// defaults to document
// keepScripts (optional): If true, will include scripts passed in the html string
jQuery.parseHTML = function( data, context, keepScripts ) {
	if ( !data || typeof data !== "string" ) {
		return null;
	}
	if ( typeof context === "boolean" ) {
		keepScripts = context;
		context = false;
	}

	// Stop scripts or inline event handlers from being executed immediately
	// by using document.implementation
	context = context || ( support.createHTMLDocument ?
		document.implementation.createHTMLDocument( "" ) :
		document );

	var parsed = rsingleTag.exec( data ),
		scripts = !keepScripts && [];

	// Single tag
	if ( parsed ) {
		return [ context.createElement( parsed[ 1 ] ) ];
	}

	parsed = buildFragment( [ data ], context, scripts );

	if ( scripts && scripts.length ) {
		jQuery( scripts ).remove();
	}

	return jQuery.merge( [], parsed.childNodes );
};


// Keep a copy of the old load method
var _load = jQuery.fn.load;

/**
 * Load a url into a page
 */
jQuery.fn.load = function( url, params, callback ) {
	if ( typeof url !== "string" && _load ) {
		return _load.apply( this, arguments );
	}

	var selector, type, response,
		self = this,
		off = url.indexOf( " " );

	if ( off > -1 ) {
		selector = jQuery.trim( url.slice( off ) );
		url = url.slice( 0, off );
	}

	// If it's a function
	if ( jQuery.isFunction( params ) ) {

		// We assume that it's the callback
		callback = params;
		params = undefined;

	// Otherwise, build a param string
	} else if ( params && typeof params === "object" ) {
		type = "POST";
	}

	// If we have elements to modify, make the request
	if ( self.length > 0 ) {
		jQuery.ajax( {
			url: url,

			// If "type" variable is undefined, then "GET" method will be used.
			// Make value of this field explicit since
			// user can override it through ajaxSetup method
			type: type || "GET",
			dataType: "html",
			data: params
		} ).done( function( responseText ) {

			// Save response for use in complete callback
			response = arguments;

			self.html( selector ?

				// If a selector was specified, locate the right elements in a dummy div
				// Exclude scripts to avoid IE 'Permission Denied' errors
				jQuery( "<div>" ).append( jQuery.parseHTML( responseText ) ).find( selector ) :

				// Otherwise use the full result
				responseText );

		// If the request succeeds, this function gets "data", "status", "jqXHR"
		// but they are ignored because response was set above.
		// If it fails, this function gets "jqXHR", "status", "error"
		} ).always( callback && function( jqXHR, status ) {
			self.each( function() {
				callback.apply( self, response || [ jqXHR.responseText, status, jqXHR ] );
			} );
		} );
	}

	return this;
};




// Attach a bunch of functions for handling common AJAX events
jQuery.each( [
	"ajaxStart",
	"ajaxStop",
	"ajaxComplete",
	"ajaxError",
	"ajaxSuccess",
	"ajaxSend"
], function( i, type ) {
	jQuery.fn[ type ] = function( fn ) {
		return this.on( type, fn );
	};
} );




jQuery.expr.filters.animated = function( elem ) {
	return jQuery.grep( jQuery.timers, function( fn ) {
		return elem === fn.elem;
	} ).length;
};




/**
 * Gets a window from an element
 */
function getWindow( elem ) {
	return jQuery.isWindow( elem ) ? elem : elem.nodeType === 9 && elem.defaultView;
}

jQuery.offset = {
	setOffset: function( elem, options, i ) {
		var curPosition, curLeft, curCSSTop, curTop, curOffset, curCSSLeft, calculatePosition,
			position = jQuery.css( elem, "position" ),
			curElem = jQuery( elem ),
			props = {};

		// Set position first, in-case top/left are set even on static elem
		if ( position === "static" ) {
			elem.style.position = "relative";
		}

		curOffset = curElem.offset();
		curCSSTop = jQuery.css( elem, "top" );
		curCSSLeft = jQuery.css( elem, "left" );
		calculatePosition = ( position === "absolute" || position === "fixed" ) &&
			( curCSSTop + curCSSLeft ).indexOf( "auto" ) > -1;

		// Need to be able to calculate position if either
		// top or left is auto and position is either absolute or fixed
		if ( calculatePosition ) {
			curPosition = curElem.position();
			curTop = curPosition.top;
			curLeft = curPosition.left;

		} else {
			curTop = parseFloat( curCSSTop ) || 0;
			curLeft = parseFloat( curCSSLeft ) || 0;
		}

		if ( jQuery.isFunction( options ) ) {

			// Use jQuery.extend here to allow modification of coordinates argument (gh-1848)
			options = options.call( elem, i, jQuery.extend( {}, curOffset ) );
		}

		if ( options.top != null ) {
			props.top = ( options.top - curOffset.top ) + curTop;
		}
		if ( options.left != null ) {
			props.left = ( options.left - curOffset.left ) + curLeft;
		}

		if ( "using" in options ) {
			options.using.call( elem, props );

		} else {
			curElem.css( props );
		}
	}
};

jQuery.fn.extend( {
	offset: function( options ) {
		if ( arguments.length ) {
			return options === undefined ?
				this :
				this.each( function( i ) {
					jQuery.offset.setOffset( this, options, i );
				} );
		}

		var docElem, win,
			elem = this[ 0 ],
			box = { top: 0, left: 0 },
			doc = elem && elem.ownerDocument;

		if ( !doc ) {
			return;
		}

		docElem = doc.documentElement;

		// Make sure it's not a disconnected DOM node
		if ( !jQuery.contains( docElem, elem ) ) {
			return box;
		}

		box = elem.getBoundingClientRect();
		win = getWindow( doc );
		return {
			top: box.top + win.pageYOffset - docElem.clientTop,
			left: box.left + win.pageXOffset - docElem.clientLeft
		};
	},

	position: function() {
		if ( !this[ 0 ] ) {
			return;
		}

		var offsetParent, offset,
			elem = this[ 0 ],
			parentOffset = { top: 0, left: 0 };

		// Fixed elements are offset from window (parentOffset = {top:0, left: 0},
		// because it is its only offset parent
		if ( jQuery.css( elem, "position" ) === "fixed" ) {

			// Assume getBoundingClientRect is there when computed position is fixed
			offset = elem.getBoundingClientRect();

		} else {

			// Get *real* offsetParent
			offsetParent = this.offsetParent();

			// Get correct offsets
			offset = this.offset();
			if ( !jQuery.nodeName( offsetParent[ 0 ], "html" ) ) {
				parentOffset = offsetParent.offset();
			}

			// Add offsetParent borders
			// Subtract offsetParent scroll positions
			parentOffset.top += jQuery.css( offsetParent[ 0 ], "borderTopWidth", true ) -
				offsetParent.scrollTop();
			parentOffset.left += jQuery.css( offsetParent[ 0 ], "borderLeftWidth", true ) -
				offsetParent.scrollLeft();
		}

		// Subtract parent offsets and element margins
		return {
			top: offset.top - parentOffset.top - jQuery.css( elem, "marginTop", true ),
			left: offset.left - parentOffset.left - jQuery.css( elem, "marginLeft", true )
		};
	},

	// This method will return documentElement in the following cases:
	// 1) For the element inside the iframe without offsetParent, this method will return
	//    documentElement of the parent window
	// 2) For the hidden or detached element
	// 3) For body or html element, i.e. in case of the html node - it will return itself
	//
	// but those exceptions were never presented as a real life use-cases
	// and might be considered as more preferable results.
	//
	// This logic, however, is not guaranteed and can change at any point in the future
	offsetParent: function() {
		return this.map( function() {
			var offsetParent = this.offsetParent;

			while ( offsetParent && jQuery.css( offsetParent, "position" ) === "static" ) {
				offsetParent = offsetParent.offsetParent;
			}

			return offsetParent || documentElement;
		} );
	}
} );

// Create scrollLeft and scrollTop methods
jQuery.each( { scrollLeft: "pageXOffset", scrollTop: "pageYOffset" }, function( method, prop ) {
	var top = "pageYOffset" === prop;

	jQuery.fn[ method ] = function( val ) {
		return access( this, function( elem, method, val ) {
			var win = getWindow( elem );

			if ( val === undefined ) {
				return win ? win[ prop ] : elem[ method ];
			}

			if ( win ) {
				win.scrollTo(
					!top ? val : win.pageXOffset,
					top ? val : win.pageYOffset
				);

			} else {
				elem[ method ] = val;
			}
		}, method, val, arguments.length );
	};
} );

// Support: Safari<7-8+, Chrome<37-44+
// Add the top/left cssHooks using jQuery.fn.position
// Webkit bug: https://bugs.webkit.org/show_bug.cgi?id=29084
// Blink bug: https://code.google.com/p/chromium/issues/detail?id=229280
// getComputedStyle returns percent when specified for top/left/bottom/right;
// rather than make the css module depend on the offset module, just check for it here
jQuery.each( [ "top", "left" ], function( i, prop ) {
	jQuery.cssHooks[ prop ] = addGetHookIf( support.pixelPosition,
		function( elem, computed ) {
			if ( computed ) {
				computed = curCSS( elem, prop );

				// If curCSS returns percentage, fallback to offset
				return rnumnonpx.test( computed ) ?
					jQuery( elem ).position()[ prop ] + "px" :
					computed;
			}
		}
	);
} );


// Create innerHeight, innerWidth, height, width, outerHeight and outerWidth methods
jQuery.each( { Height: "height", Width: "width" }, function( name, type ) {
	jQuery.each( { padding: "inner" + name, content: type, "": "outer" + name },
		function( defaultExtra, funcName ) {

		// Margin is only for outerHeight, outerWidth
		jQuery.fn[ funcName ] = function( margin, value ) {
			var chainable = arguments.length && ( defaultExtra || typeof margin !== "boolean" ),
				extra = defaultExtra || ( margin === true || value === true ? "margin" : "border" );

			return access( this, function( elem, type, value ) {
				var doc;

				if ( jQuery.isWindow( elem ) ) {

					// As of 5/8/2012 this will yield incorrect results for Mobile Safari, but there
					// isn't a whole lot we can do. See pull request at this URL for discussion:
					// https://github.com/jquery/jquery/pull/764
					return elem.document.documentElement[ "client" + name ];
				}

				// Get document width or height
				if ( elem.nodeType === 9 ) {
					doc = elem.documentElement;

					// Either scroll[Width/Height] or offset[Width/Height] or client[Width/Height],
					// whichever is greatest
					return Math.max(
						elem.body[ "scroll" + name ], doc[ "scroll" + name ],
						elem.body[ "offset" + name ], doc[ "offset" + name ],
						doc[ "client" + name ]
					);
				}

				return value === undefined ?

					// Get width or height on the element, requesting but not forcing parseFloat
					jQuery.css( elem, type, extra ) :

					// Set width or height on the element
					jQuery.style( elem, type, value, extra );
			}, type, chainable ? margin : undefined, chainable, null );
		};
	} );
} );


jQuery.fn.extend( {

	bind: function( types, data, fn ) {
		return this.on( types, null, data, fn );
	},
	unbind: function( types, fn ) {
		return this.off( types, null, fn );
	},

	delegate: function( selector, types, data, fn ) {
		return this.on( types, selector, data, fn );
	},
	undelegate: function( selector, types, fn ) {

		// ( namespace ) or ( selector, types [, fn] )
		return arguments.length === 1 ?
			this.off( selector, "**" ) :
			this.off( types, selector || "**", fn );
	},
	size: function() {
		return this.length;
	}
} );

jQuery.fn.andSelf = jQuery.fn.addBack;




// Register as a named AMD module, since jQuery can be concatenated with other
// files that may use define, but not via a proper concatenation script that
// understands anonymous AMD modules. A named AMD is safest and most robust
// way to register. Lowercase jquery is used because AMD module names are
// derived from file names, and jQuery is normally delivered in a lowercase
// file name. Do this after creating the global so that if an AMD module wants
// to call noConflict to hide this version of jQuery, it will work.

// Note that for maximum portability, libraries that are not jQuery should
// declare themselves as anonymous modules, and avoid setting a global if an
// AMD loader is present. jQuery is a special case. For more information, see
// https://github.com/jrburke/requirejs/wiki/Updating-existing-libraries#wiki-anon

if ( typeof define === "function" && define.amd ) {
	define( "jquery", [], function() {
		return jQuery;
	} );
}



var

	// Map over jQuery in case of overwrite
	_jQuery = window.jQuery,

	// Map over the $ in case of overwrite
	_$ = window.$;

jQuery.noConflict = function( deep ) {
	if ( window.$ === jQuery ) {
		window.$ = _$;
	}

	if ( deep && window.jQuery === jQuery ) {
		window.jQuery = _jQuery;
	}

	return jQuery;
};

// Expose jQuery and $ identifiers, even in AMD
// (#7102#comment:10, https://github.com/jquery/jquery/pull/557)
// and CommonJS for browser emulators (#13566)
if ( !noGlobal ) {
	window.jQuery = window.$ = jQuery;
}

return jQuery;
}));

/*!
 * jquery-confirm v2.5.1 (http://craftpip.github.io/jquery-confirm/)
 * Author: Boniface Pereira
 * Website: www.craftpip.com
 * Contact: hey@craftpip.com
 *
 * Copyright 2013-2015 jquery-confirm
 * Licensed under MIT (https://github.com/craftpip/jquery-confirm/blob/master/LICENSE)
 */
if(typeof jQuery==="undefined"){throw new Error("jquery-confirm requires jQuery");}var jconfirm,Jconfirm;(function($){$.fn.confirm=function(options,option2){if(typeof options==="undefined"){options={};}if(typeof options==="string"){options={content:options,title:(option2)?option2:false};}$(this).each(function(){var $this=$(this);$this.on("click",function(e){e.preventDefault();var jcOption=$.extend({},options);if($this.attr("data-title")){jcOption.title=$this.attr("data-title");}if($this.attr("data-content")){jcOption.content=$this.attr("data-content");}jcOption["$target"]=$this;if($this.attr("href")&&!options.confirm){jcOption.confirm=function(){location.href=$this.attr("href");};}$.confirm(jcOption);});});return $(this);};$.confirm=function(options,option2){if(typeof options==="undefined"){options={};}if(typeof options==="string"){options={content:options,title:(option2)?option2:false};}return jconfirm(options);};$.alert=function(options,option2){if(typeof options==="undefined"){options={};}if(typeof options==="string"){options={content:options,title:(option2)?option2:false};}options.cancelButton=false;return jconfirm(options);};$.dialog=function(options,option2){if(typeof options==="undefined"){options={};}if(typeof options==="string"){options={content:options,title:(option2)?option2:false};}options.cancelButton=false;options.confirmButton=false;options.confirmKeys=[13];return jconfirm(options);};jconfirm=function(options){if(typeof options==="undefined"){options={};}if(jconfirm.defaults){$.extend(jconfirm.pluginDefaults,jconfirm.defaults);}var options=$.extend({},jconfirm.pluginDefaults,options);return new Jconfirm(options);};Jconfirm=function(options){$.extend(this,options);this._init();};Jconfirm.prototype={_init:function(){var that=this;this._rand=Math.round(Math.random()*99999);this._buildHTML();this._bindEvents();setTimeout(function(){that.open();that._watchContent();},0);},_buildHTML:function(){var that=this;this.animation="anim-"+this.animation.toLowerCase();this.closeAnimation="anim-"+this.closeAnimation.toLowerCase();this.theme="jconfirm-"+this.theme.toLowerCase();if(this.animation=="anim-none"){this.animationSpeed=0;}this._lastFocused=$("body").find(":focus");this.$el=$(this.template).appendTo(this.container).addClass(this.theme);this.$el.find(".jconfirm-box-container").addClass(this.columnClass);this.$el.find(".jconfirm-bg").css(this._getCSS(this.animationSpeed,1));this.$el.find(".jconfirm-bg").css("opacity",this.opacity);this.$b=this.$el.find(".jconfirm-box").css(this._getCSS(this.animationSpeed,this.animationBounce)).addClass(this.animation);this.$body=this.$b;if(this.rtl){this.$el.addClass("rtl");}this._contentReady=$.Deferred();this._modalReady=$.Deferred();this.$title=this.$el.find(".title");this.contentDiv=this.$el.find("div.content");this.$content=this.contentDiv;this.$contentPane=this.$el.find(".content-pane");this.$icon=this.$el.find(".icon-c");this.$closeIcon=this.$el.find(".closeIcon");this.$contentPane.css(this._getCSS(this.animationSpeed,1));this.setTitle();this.setIcon();this._setButtons();if(this.closeIconClass){this.$closeIcon.html('<i class="'+this.closeIconClass+'"></i>');}that._contentHash=this._hash(that.$content.html());$.when(this._contentReady,this._modalReady).then(function(){that.setContent();that.setTitle();that.setIcon();});this._getContent();this._imagesLoaded();if(this.autoClose){this._startCountDown();}},_unwatchContent:function(){clearInterval(this._timer);},_hash:function(){return btoa((encodeURIComponent(this.$content.html())));},_watchContent:function(){var that=this;this._timer=setInterval(function(){var now=that._hash(that.$content.html());if(that._contentHash!=now){that._contentHash=now;that.setDialogCenter();that._imagesLoaded();}},this.watchInterval);},_bindEvents:function(){var that=this;var boxClicked=false;this.$el.find(".jconfirm-scrollpane").click(function(e){if(!boxClicked){if(that.backgroundDismiss){that.cancel();that.close();}else{that.$b.addClass("hilight");setTimeout(function(){that.$b.removeClass("hilight");},800);}}boxClicked=false;});this.$el.find(".jconfirm-box").click(function(e){boxClicked=true;});if(this.$confirmButton){this.$confirmButton.click(function(e){e.preventDefault();var r=that.confirm(that.$b);that._stopCountDown();that.onAction("confirm");if(typeof r==="undefined"||r){that.close();}});}if(this.$cancelButton){this.$cancelButton.click(function(e){e.preventDefault();var r=that.cancel(that.$b);that._stopCountDown();that.onAction("cancel");if(typeof r==="undefined"||r){that.close();}});}if(this.$closeButton){this.$closeButton.click(function(e){e.preventDefault();that._stopCountDown();that.cancel();that.onAction("close");that.close();});}if(this.keyboardEnabled){setTimeout(function(){$(window).on("keyup."+this._rand,function(e){that.reactOnKey(e);});},500);}$(window).on("resize."+this._rand,function(){that.setDialogCenter(true);});},_getCSS:function(speed,bounce){return{"-webkit-transition-duration":speed/1000+"s","transition-duration":speed/1000+"s","-webkit-transition-timing-function":"cubic-bezier(.36,1.1,.2, "+bounce+")","transition-timing-function":"cubic-bezier(.36,1.1,.2, "+bounce+")"};},_imagesLoaded:function(){var that=this;$.each(this.$content.find("img:not(.loaded)"),function(i,a){var interval=setInterval(function(){var h=$(a).css("height");if(h!=="0px"){$(a).addClass("loaded");that.setDialogCenter();clearInterval(interval);}},40);});},_setButtons:function(){this.$btnc=this.$el.find(".buttons");if(this.confirmButton&&$.trim(this.confirmButton)!==""){this.$confirmButton=$('<button type="button" class="btn">'+this.confirmButton+"</button>").appendTo(this.$btnc).addClass(this.confirmButtonClass);}if(this.cancelButton&&$.trim(this.cancelButton)!==""){this.$cancelButton=$('<button type="button" class="btn">'+this.cancelButton+"</button>").appendTo(this.$btnc).addClass(this.cancelButtonClass);}if(!this.confirmButton&&!this.cancelButton){this.$btnc.hide();}if(!this.confirmButton&&!this.cancelButton&&this.closeIcon===null){this.$closeButton=this.$b.find(".closeIcon").show();}if(this.closeIcon===true){this.$closeButton=this.$b.find(".closeIcon").show();}},setTitle:function(string){this.title=(typeof string!=="undefined")?string:this.title;this.$title.html(this.title||"");},setIcon:function(iconClass){this.title=(typeof string!=="undefined")?iconClass:this.title;this.$icon.html(this.icon?'<i class="'+this.icon+'"></i>':"");},setContent:function(string){var that=this;this.content=(typeof string=="undefined")?this.content:string;if(this.content==""){this.$content.html(this.content);this.$contentPane.hide();}else{this.$content.html(this.content);this.$contentPane.show();}if(this.$content.hasClass("loading")){this.$content.removeClass("loading");this.$btnc.find("button").prop("disabled",false);}},_getContent:function(string){var that=this;string=(string)?string:this.content;this._isAjax=false;if(!this.content){this.content="";this.setContent(this.content);this._contentReady.reject();}else{if(typeof this.content==="string"){if(this.content.substr(0,4).toLowerCase()==="url:"){this._isAjax=true;this.$content.addClass("loading");this.$btnc.find("button").prop("disabled",true);var url=this.content.substring(4,this.content.length);$.get(url).done(function(html){that.content=html;that._contentReady.resolve();}).always(function(data,status,xhr){if(typeof that.contentLoaded==="function"){that.contentLoaded(data,status,xhr);}});}else{this.setContent(this.content);this._contentReady.reject();}}else{if(typeof this.content==="function"){this.$content.addClass("loading");this.$btnc.find("button").attr("disabled","disabled");var promise=this.content(this);if(typeof promise!=="object"){console.error("The content function must return jquery promise.");}else{if(typeof promise.always!=="function"){console.error("The object returned is not a jquery promise.");}else{this._isAjax=true;promise.always(function(data,status){that._contentReady.resolve();});}}}else{console.error("Invalid option for property content, passed: "+typeof this.content);}}}this.setDialogCenter();},_stopCountDown:function(){clearInterval(this.timerInterval);if(this.$cd){this.$cd.remove();}},_startCountDown:function(){var opt=this.autoClose.split("|");if(/cancel/.test(opt[0])&&this.type==="alert"){return false;}else{if(/confirm|cancel/.test(opt[0])){this.$cd=$('<span class="countdown">').appendTo(this["$"+opt[0]+"Button"]);var that=this;that.$cd.parent().click();var time=opt[1]/1000;this.timerInterval=setInterval(function(){that.$cd.html(" ("+(time-=1)+")");if(time===0){that.$cd.html("");that.$cd.parent().trigger("click");clearInterval(that.timerInterval);}},1000);}else{console.error("Invalid option "+opt[0]+", must be confirm/cancel");}}},reactOnKey:function key(e){var a=$(".jconfirm");if(a.eq(a.length-1)[0]!==this.$el[0]){return false;}var key=e.which;if(this.contentDiv.find(":input").is(":focus")&&/13|32/.test(key)){return false;}if($.inArray(key,this.cancelKeys)!==-1){if(this.$cancelButton){this.$cancelButton.click();}else{this.close();}}if($.inArray(key,this.confirmKeys)!==-1){if(this.$confirmButton){this.$confirmButton.click();}}},setDialogCenter:function(){if(this.$contentPane.css("display")=="none"){var contentHeight=0;var paneHeight=0;}else{var contentHeight=this.$content.outerHeight();var paneHeight=this.$contentPane.height();if(paneHeight==0){paneHeight=contentHeight;}}var off=100;var w=this.$content.outerWidth();this.$content.css({clip:"rect(0px "+(off+w)+"px "+contentHeight+"px -"+off+"px)"});this.$contentPane.css({height:contentHeight});var windowHeight=$(window).height();var boxHeight=this.$b.outerHeight()-paneHeight+contentHeight;var topMargin=(windowHeight-boxHeight)/2;var minMargin=100;if(boxHeight>(windowHeight-minMargin)){var style={"margin-top":minMargin/2,"margin-bottom":minMargin/2};$("body").addClass("jconfirm-noscroll");}else{var style={"margin-top":topMargin};$("body").removeClass("jconfirm-noscroll");}this.$b.css(style);},close:function(){var that=this;if(this.isClosed()){return false;}if(typeof this.onClose==="function"){this.onClose();}this._unwatchContent();that._lastFocused.focus();$(window).unbind("resize."+this._rand);if(this.keyboardEnabled){$(window).unbind("keyup."+this._rand);}that.$el.find(".jconfirm-bg").removeClass("seen");$("body").removeClass("jconfirm-noscroll");this.$b.addClass(this.closeAnimation);var closeTimer=(this.closeAnimation=="anim-none")?0:this.animationSpeed;setTimeout(function(){that.$el.remove();},closeTimer*25/100);jconfirm.record.closed+=1;jconfirm.record.currentlyOpen-=1;return true;},open:function(){var that=this;if(this.isClosed()){return false;}that.$el.find(".jconfirm-bg").addClass("seen");this.$b.removeClass(this.animation);this.$b.find("input[autofocus]:visible:first").focus();jconfirm.record.opened+=1;jconfirm.record.currentlyOpen+=1;if(typeof this.onOpen==="function"){this.onOpen();}var jcr="jconfirm-box"+this._rand;this.$b.attr("aria-labelledby",jcr).attr("tabindex",-1).focus();if(this.$title){this.$title.attr("id",jcr);}else{if(this.$content){this.$content.attr("id",jcr);}}setTimeout(function(){that.$b.css({"transition-property":that.$b.css("transition-property")+", margin"});that._modalReady.resolve();},this.animationSpeed);return true;},isClosed:function(){return this.$el.css("display")==="";}};jconfirm.pluginDefaults={template:'<div class="jconfirm"><div class="jconfirm-bg"></div><div class="jconfirm-scrollpane"><div class="container"><div class="row"><div class="jconfirm-box-container"><div class="jconfirm-box" role="dialog" aria-labelledby="labelled" tabindex="-1"><div class="closeIcon">&times;</div><div class="title-c"><span class="icon-c"></span><span class="title"></span></div><div class="content-pane"><div class="content"></div></div><div class="buttons"></div><div class="jquery-clear"></div></div></div></div></div></div></div>',title:"Hello",content:"Are you sure to continue?",contentLoaded:function(){},icon:"",opacity:0.2,confirmButton:"Okay",cancelButton:"Close",confirmButtonClass:"btn-default",cancelButtonClass:"btn-default",theme:"white",animation:"zoom",closeAnimation:"scale",animationSpeed:500,animationBounce:1.2,keyboardEnabled:false,rtl:false,confirmKeys:[13],cancelKeys:[27],container:"body",confirm:function(){},cancel:function(){},backgroundDismiss:false,autoClose:false,closeIcon:null,closeIconClass:false,watchInterval:100,columnClass:"col-md-4 col-md-offset-4 col-sm-6 col-sm-offset-3 col-xs-10 col-xs-offset-1",onOpen:function(){},onClose:function(){},onAction:function(){}};jconfirm.record={opened:0,closed:0,currentlyOpen:0};})(jQuery);
"use strict";(function(root,factory){if(typeof define==="function"&&define.amd){define([],factory)}else{var lib=factory.call(root);Object.keys(lib).forEach(function(key){root[key]=lib[key]})}})(this,function(){var topWindow=this;function error(msg){if(topWindow.console){if(topWindow.console.error){topWindow.console.error(msg)}else if(topWindow.console.log){topWindow.console.log(msg)}}}function isInIFrame(w){w=w||topWindow;return w!==w.top}function makeFailHTML(msg){return""+'<table style="background-color: #8CE; width: 100%; height: 100%;"><tr>'+'<td align="center">'+'<div style="display: table-cell; vertical-align: middle;">'+'<div style="">'+msg+"</div>"+"</div>"+"</td></tr></table>"}var GET_A_WEBGL_BROWSER=""+"This page requires a browser that supports WebGL.<br/>"+'<a href="http://get.webgl.org">Click here to upgrade your browser.</a>';var OTHER_PROBLEM=""+"It doesn't appear your computer can support WebGL.<br/>"+'<a href="http://get.webgl.org/troubleshooting/">Click here for more information.</a>';function create3DContext(canvas,opt_attribs){var names=["webgl","experimental-webgl"];var context=null;for(var ii=0;ii<names.length;++ii){try{context=canvas.getContext(names[ii],opt_attribs)}catch(e){}if(context){break}}return context}function setupWebGL(canvas,opt_attribs){function showLink(str){var container=canvas.parentNode;if(container){container.innerHTML=makeFailHTML(str)}}if(!topWindow.WebGLRenderingContext){showLink(GET_A_WEBGL_BROWSER);return null}var context=create3DContext(canvas,opt_attribs);if(!context){showLink(OTHER_PROBLEM)}return context}function updateCSSIfInIFrame(){if(isInIFrame()){document.body.className="iframe"}}function getWebGLContext(canvas,opt_attribs,opt_options){var options=opt_options||{};if(isInIFrame()){updateCSSIfInIFrame();if(!options.dontResize&&options.resize!==false){var width=canvas.clientWidth;var height=canvas.clientHeight;canvas.width=width;canvas.height=height}}else if(!options.noTitle&&options.title!==false){var title=document.title;var h1=document.createElement("h1");h1.innerText=title;document.body.insertBefore(h1,document.body.children[0])}var gl=setupWebGL(canvas,opt_attribs);return gl}function loadShader(gl,shaderSource,shaderType,opt_errorCallback){var errFn=opt_errorCallback||error;var shader=gl.createShader(shaderType);gl.shaderSource(shader,shaderSource);gl.compileShader(shader);var compiled=gl.getShaderParameter(shader,gl.COMPILE_STATUS);if(!compiled){var lastError=gl.getShaderInfoLog(shader);errFn("*** Error compiling shader '"+shader+"':"+lastError);gl.deleteShader(shader);return null}return shader}function createProgram(gl,shaders,opt_attribs,opt_locations,opt_errorCallback){var errFn=opt_errorCallback||error;var program=gl.createProgram();shaders.forEach(function(shader){gl.attachShader(program,shader)});if(opt_attribs){obj_attrib.forEach(function(attrib,ndx){gl.bindAttribLocation(program,opt_locations?opt_locations[ndx]:ndx,attrib)})}gl.linkProgram(program);var linked=gl.getProgramParameter(program,gl.LINK_STATUS);if(!linked){var lastError=gl.getProgramInfoLog(program);errFn("Error in program linking:"+lastError);gl.deleteProgram(program);return null}return program}function createShaderFromScript(gl,scriptId,opt_shaderType,opt_errorCallback){var shaderSource="";var shaderType;var shaderScript=document.getElementById(scriptId);if(!shaderScript){throw"*** Error: unknown script element"+scriptId}shaderSource=shaderScript.text;if(!opt_shaderType){if(shaderScript.type==="x-shader/x-vertex"){shaderType=gl.VERTEX_SHADER}else if(shaderScript.type==="x-shader/x-fragment"){shaderType=gl.FRAGMENT_SHADER}else if(shaderType!==gl.VERTEX_SHADER&&shaderType!==gl.FRAGMENT_SHADER){throw"*** Error: unknown shader type"}}return loadShader(gl,shaderSource,opt_shaderType?opt_shaderType:shaderType,opt_errorCallback)}var defaultShaderType=["VERTEX_SHADER","FRAGMENT_SHADER"];function createProgramFromScripts(gl,shaderScriptIds,opt_attribs,opt_locations,opt_errorCallback){var shaders=[];for(var ii=0;ii<shaderScriptIds.length;++ii){shaders.push(createShaderFromScript(gl,shaderScriptIds[ii],gl[defaultShaderType[ii]],opt_errorCallback))}return createProgram(gl,shaders,opt_attribs,opt_locations,opt_errorCallback)}function createProgramFromSources(gl,shaderSources,opt_attribs,opt_locations,opt_errorCallback){var shaders=[];for(var ii=0;ii<shaderSources.length;++ii){shaders.push(loadShader(gl,shaderSources[ii],gl[defaultShaderType[ii]],opt_errorCallback))}return createProgram(gl,shaders,opt_attribs,opt_locations,opt_errorCallback)}function getBindPointForSamplerType(gl,type){if(type===gl.SAMPLER_2D)return gl.TEXTURE_2D;if(type===gl.SAMPLER_CUBE)return gl.TEXTURE_CUBE_MAP}function createUniformSetters(gl,program){var textureUnit=0;function createUniformSetter(program,uniformInfo){var location=gl.getUniformLocation(program,uniformInfo.name);var type=uniformInfo.type;var isArray=uniformInfo.size>1&&uniformInfo.name.substr(-3)==="[0]";if(type===gl.FLOAT&&isArray){return function(v){gl.uniform1fv(location,v)}}if(type===gl.FLOAT){return function(v){gl.uniform1f(location,v)}}if(type===gl.FLOAT_VEC2){return function(v){gl.uniform2fv(location,v)}}if(type===gl.FLOAT_VEC3){return function(v){gl.uniform3fv(location,v)}}if(type===gl.FLOAT_VEC4){return function(v){gl.uniform4fv(location,v)}}if(type===gl.INT&&isArray){return function(v){gl.uniform1iv(location,v)}}if(type===gl.INT){return function(v){gl.uniform1i(location,v)}}if(type===gl.INT_VEC2){return function(v){gl.uniform2iv(location,v)}}if(type===gl.INT_VEC3){return function(v){gl.uniform3iv(location,v)}}if(type===gl.INT_VEC4){return function(v){gl.uniform4iv(location,v)}}if(type===gl.BOOL){return function(v){gl.uniform1iv(location,v)}}if(type===gl.BOOL_VEC2){return function(v){gl.uniform2iv(location,v)}}if(type===gl.BOOL_VEC3){return function(v){gl.uniform3iv(location,v)}}if(type===gl.BOOL_VEC4){return function(v){gl.uniform4iv(location,v)}}if(type===gl.FLOAT_MAT2){return function(v){gl.uniformMatrix2fv(location,false,v)}}if(type===gl.FLOAT_MAT3){return function(v){gl.uniformMatrix3fv(location,false,v)}}if(type===gl.FLOAT_MAT4){return function(v){gl.uniformMatrix4fv(location,false,v)}}if((type===gl.SAMPLER_2D||type===gl.SAMPLER_CUBE)&&isArray){var units=[];for(var ii=0;ii<info.size;++ii){units.push(textureUnit++)}return function(bindPoint,units){return function(textures){gl.uniform1iv(location,units);textures.forEach(function(texture,index){gl.activeTexture(gl.TEXTURE0+units[index]);gl.bindTexture(bindPoint,texture)})}}(getBindPointForSamplerType(gl,type),units)}if(type===gl.SAMPLER_2D||type===gl.SAMPLER_CUBE){return function(bindPoint,unit){return function(texture){gl.uniform1i(location,unit);gl.activeTexture(gl.TEXTURE0+unit);gl.bindTexture(bindPoint,texture)}}(getBindPointForSamplerType(gl,type),textureUnit++)}throw"unknown type: 0x"+type.toString(16)}var uniformSetters={};var numUniforms=gl.getProgramParameter(program,gl.ACTIVE_UNIFORMS);for(var ii=0;ii<numUniforms;++ii){var uniformInfo=gl.getActiveUniform(program,ii);if(!uniformInfo){break}var name=uniformInfo.name;if(name.substr(-3)==="[0]"){name=name.substr(0,name.length-3)}var setter=createUniformSetter(program,uniformInfo);uniformSetters[name]=setter}return uniformSetters}function setUniforms(setters,values){Object.keys(values).forEach(function(name){var setter=setters[name];if(setter){setter(values[name])}})}function createAttributeSetters(gl,program){var attribSetters={};function createAttribSetter(index){return function(b){gl.bindBuffer(gl.ARRAY_BUFFER,b.buffer);gl.enableVertexAttribArray(index);gl.vertexAttribPointer(index,b.numComponents||b.size,b.type||gl.FLOAT,b.normalize||false,b.stride||0,b.offset||0)}}var numAttribs=gl.getProgramParameter(program,gl.ACTIVE_ATTRIBUTES);for(var ii=0;ii<numAttribs;++ii){var attribInfo=gl.getActiveAttrib(program,ii);if(!attribInfo){break}var index=gl.getAttribLocation(program,attribInfo.name);attribSetters[attribInfo.name]=createAttribSetter(index)}return attribSetters}function setAttributes(setters,buffers){Object.keys(buffers).forEach(function(name){var setter=setters[name];if(setter){setter(buffers[name])}})}function createProgramInfo(gl,shaderSources,opt_attribs,opt_locations,opt_errorCallback){shaderSources=shaderSources.map(function(source){var script=document.getElementById(source);return script?script.text:source});var program=createProgramFromSources(gl,shaderSources,opt_attribs,opt_locations,opt_errorCallback);if(!program){return null}var uniformSetters=createUniformSetters(gl,program);var attribSetters=createAttributeSetters(gl,program);return{program:program,uniformSetters:uniformSetters,attribSetters:attribSetters}}function setBuffersAndAttributes(gl,setters,buffers){setAttributes(setters,buffers.attribs);if(buffers.indices){gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER,buffers.indices)}}var browserPrefixes=["","MOZ_","OP_","WEBKIT_"];function getExtensionWithKnownPrefixes(gl,name){for(var ii=0;ii<browserPrefixes.length;++ii){var prefixedName=browserPrefixes[ii]+name;var ext=gl.getExtension(prefixedName);if(ext){return ext}}}function resizeCanvasToDisplaySize(canvas,multiplier){multiplier=multiplier||1;var width=canvas.clientWidth*multiplier;var height=canvas.clientHeight*multiplier;if(canvas.width!==width||canvas.height!==height){canvas.width=width;canvas.height=height;return true}return false}function getIFrameForWindow(window){if(!isInIFrame(window)){return}var iframes=window.parent.document.getElementsByTagName("iframe");for(var ii=0;ii<iframes.length;++ii){var iframe=iframes[ii];if(iframe.contentDocument===window.document){return iframe}}}function isFrameVisible(window){try{var iframe=getIFrameForWindow(window);if(!iframe){return true}var bounds=iframe.getBoundingClientRect();var isVisible=bounds.top<window.parent.innerHeight&&bounds.bottom>=0&&bounds.left<window.parent.innerWidth&&bounds.right>=0;return isVisible&&isFrameVisible(window.parent)}catch(e){return true}}function isOnScreen(element){var isVisible=true;if(element){var bounds=element.getBoundingClientRect();isVisible=bounds.top<topWindow.innerHeight&&bounds.bottom>=0}return isVisible&&isFrameVisible(topWindow)}function augmentTypedArray(typedArray,numComponents){var cursor=0;typedArray.push=function(){for(var ii=0;ii<arguments.length;++ii){var value=arguments[ii];if(value instanceof Array||value.buffer&&value.buffer instanceof ArrayBuffer){for(var jj=0;jj<value.length;++jj){typedArray[cursor++]=value[jj]}}else{typedArray[cursor++]=value}}};typedArray.reset=function(opt_index){cursor=opt_index||0};typedArray.numComponents=numComponents;Object.defineProperty(typedArray,"numElements",{get:function(){return this.length/this.numComponents|0}});return typedArray}function createAugmentedTypedArray(numComponents,numElements,opt_type){var Type=opt_type||Float32Array;return augmentTypedArray(new Type(numComponents*numElements),numComponents)}function createBufferFromTypedArray(gl,array,type,drawType){type=type||gl.ARRAY_BUFFER;var buffer=gl.createBuffer();gl.bindBuffer(type,buffer);gl.bufferData(type,array,drawType||gl.STATIC_DRAW);return buffer}function allButIndices(name){return name!=="indices"}function createMapping(obj){var mapping={};Object.keys(obj).filter(allButIndices).forEach(function(key){mapping["a_"+key]=key});return mapping}function getGLTypeForTypedArray(gl,typedArray){if(typedArray instanceof Int8Array){return gl.BYTE}if(typedArray instanceof Uint8Array){return gl.UNSIGNED_BYTE}if(typedArray instanceof Int16Array){return gl.SHORT}if(typedArray instanceof Uint16Array){return gl.UNSIGNED_SHORT}if(typedArray instanceof Int32Array){return gl.INT}if(typedArray instanceof Uint32Array){return gl.UNSIGNED_INT}if(typedArray instanceof Float32Array){return gl.FLOAT}throw"unsupported typed array type"}function getNormalizationForTypedArray(typedArray){if(typedArray instanceof Int8Array){return true}if(typedArray instanceof Uint8Array){return true}return false}function isArrayBuffer(a){return a.buffer&&a.buffer instanceof ArrayBuffer}function guessNumComponentsFromName(name,length){var numComponents;if(name.indexOf("coord")>=0){numComponents=2}else if(name.indexOf("color")>=0){numComponents=4}else{numComponents=3}if(length%numComponents>0){throw"can not guess numComponents. You should specify it."}return numComponents}function makeTypedArray(array,name){if(isArrayBuffer(array)){return array}if(Array.isArray(array)){array={data:array}}if(!array.numComponents){array.numComponents=guessNumComponentsFromName(name,array.length)}var type=array.type;if(!type){if(name==="indices"){type=Uint16Array}}var typedArray=createAugmentedTypedArray(array.numComponents,array.data.length/array.numComponents|0,type);typedArray.push(array.data);return typedArray}function createAttribsFromArrays(gl,arrays,opt_mapping){var mapping=opt_mapping||createMapping(arrays);var attribs={};Object.keys(mapping).forEach(function(attribName){var bufferName=mapping[attribName];var array=makeTypedArray(arrays[bufferName],bufferName);attribs[attribName]={buffer:createBufferFromTypedArray(gl,array),numComponents:array.numComponents||guessNumComponentsFromName(bufferName),type:getGLTypeForTypedArray(gl,array),normalize:getNormalizationForTypedArray(array)}});return attribs}function getNumElementsFromNonIndexedArrays(arrays){var key=Object.keys(arrays)[0];var array=arrays[key];if(isArrayBuffer(array)){return array.numElements}else{return array.data.length/array.numComponents}}function createBufferInfoFromArrays(gl,arrays,opt_mapping){var bufferInfo={attribs:createAttribsFromArrays(gl,arrays,opt_mapping)};var indices=arrays.indices;if(indices){indices=makeTypedArray(indices,"indices");bufferInfo.indices=createBufferFromTypedArray(gl,indices,gl.ELEMENT_ARRAY_BUFFER);bufferInfo.numElements=indices.length}else{bufferInfo.numElements=getNumElementsFromNonIndexedArrays(arrays)}return bufferInfo}function createBuffersFromArrays(gl,arrays){var buffers={};Object.keys(arrays).forEach(function(key){var type=key==="indices"?gl.ELEMENT_ARRAY_BUFFER:gl.ARRAY_BUFFER;var array=makeTypedArray(arrays[key],name);buffers[key]=createBufferFromTypedArray(gl,array,type)});if(arrays.indices){buffers.numElements=arrays.indices.length}else if(arrays.position){buffers.numElements=arrays.position.length/3}return buffers}function drawBufferInfo(gl,type,bufferInfo,count,offset){var indices=bufferInfo.indices;var numElements=count===undefined?bufferInfo.numElements:count;offset=offset===undefined?offset:0;if(indices){gl.drawElements(type,numElements,gl.UNSIGNED_SHORT,offset)}else{gl.drawArrays(type,offset,numElements)}}function drawObjectList(gl,objectsToDraw){var lastUsedProgramInfo=null;var lastUsedBufferInfo=null;objectsToDraw.forEach(function(object){var programInfo=object.programInfo;var bufferInfo=object.bufferInfo;var bindBuffers=false;if(programInfo!==lastUsedProgramInfo){lastUsedProgramInfo=programInfo;gl.useProgram(programInfo.program);bindBuffers=true}if(bindBuffers||bufferInfo!==lastUsedBufferInfo){lastUsedBufferInfo=bufferInfo;setBuffersAndAttributes(gl,programInfo.attribSetters,bufferInfo)}setUniforms(programInfo.uniformSetters,object.uniforms);drawBufferInfo(gl,gl.TRIANGLES,bufferInfo)})}return{createAugmentedTypedArray:createAugmentedTypedArray,createAttribsFromArrays:createAttribsFromArrays,createBuffersFromArrays:createBuffersFromArrays,createBufferInfoFromArrays:createBufferInfoFromArrays,createAttributeSetters:createAttributeSetters,createProgram:createProgram,createProgramFromScripts:createProgramFromScripts,createProgramFromSources:createProgramFromSources,createProgramInfo:createProgramInfo,createUniformSetters:createUniformSetters,drawBufferInfo:drawBufferInfo,drawObjectList:drawObjectList,getWebGLContext:getWebGLContext,updateCSSIfInIFrame:updateCSSIfInIFrame,getExtensionWithKnownPrefixes:getExtensionWithKnownPrefixes,resizeCanvasToDisplaySize:resizeCanvasToDisplaySize,setAttributes:setAttributes,setBuffersAndAttributes:setBuffersAndAttributes,setUniforms:setUniforms,setupWebGL:setupWebGL}});function AgoraRender(){var gl=undefined;var program=undefined;var positionLocation=undefined;var texCoordLocation=undefined;var yTexture=undefined;var uTexture=undefined;var vTexture=undefined;var texCoordBuffer=undefined;var surfaceBuffer=undefined;var that={view:undefined,mirrorView:false,container:undefined,canvas:undefined,renderImageCount:0,initWidth:0,initHeight:0,initRotation:0,canvasUpdated:false,clientWidth:0,clientHeight:0};that.start=function(view,onFailure){initCanvas(view,that.mirrorView,view.clientWidth,view.clientHeight,that.initRotation,onFailure)};that.stop=function(){gl=undefined;program=undefined;positionLocation=undefined;texCoordLocation=undefined;deleteTexture(yTexture);deleteTexture(uTexture);deleteTexture(vTexture);yTexture=undefined;uTexture=undefined;vTexture=undefined;deleteBuffer(texCoordBuffer);deleteBuffer(surfaceBuffer);texCoordBuffer=undefined;surfaceBuffer=undefined;if(that.container){that.container.removeChild(that.canvas)}if(that.view){that.view.removeChild(that.container)}that.canvas=undefined;that.container=undefined;that.view=undefined;that.mirrorView=false};that.renderImage=function(image){if(!gl){return}if(image.width!=that.initWidth||image.height!=that.initHeight||image.rotation!=that.initRotation||image.mirror!=that.mirrorView){var view=that.view;that.stop();console.log("init canvas "+image.width+"*"+image.height+" rotation "+image.rotation);initCanvas(view,image.mirror,image.width,image.height,image.rotation,function(e){console.error("init canvas "+image.width+"*"+image.height+" rotation "+image.rotation+" failed. "+e)})}gl.bindBuffer(gl.ARRAY_BUFFER,texCoordBuffer);var xWidth=image.width+image.left+image.right;var xHeight=image.height+image.top+image.bottom;gl.bufferData(gl.ARRAY_BUFFER,new Float32Array([image.left/xWidth,image.bottom/xHeight,1-image.right/xWidth,image.bottom/xHeight,image.left/xWidth,1-image.top/xHeight,image.left/xWidth,1-image.top/xHeight,1-image.right/xWidth,image.bottom/xHeight,1-image.right/xWidth,1-image.top/xHeight]),gl.STATIC_DRAW);gl.enableVertexAttribArray(texCoordLocation);gl.vertexAttribPointer(texCoordLocation,2,gl.FLOAT,false,0,0);uploadYuv(xWidth,xHeight,image.yplane,image.uplane,image.vplane);updateCanvas(image.rotation,image.width,image.height);gl.drawArrays(gl.TRIANGLES,0,6);that.renderImageCount+=1};function uploadYuv(width,height,yplane,uplane,vplane){gl.activeTexture(gl.TEXTURE0);gl.bindTexture(gl.TEXTURE_2D,yTexture);gl.texImage2D(gl.TEXTURE_2D,0,gl.LUMINANCE,width,height,0,gl.LUMINANCE,gl.UNSIGNED_BYTE,yplane);var e=gl.getError();if(e!=gl.NO_ERROR){console.log("upload y plane ",width,height,yplane.byteLength," error",e)}gl.activeTexture(gl.TEXTURE1);gl.bindTexture(gl.TEXTURE_2D,uTexture);gl.texImage2D(gl.TEXTURE_2D,0,gl.LUMINANCE,width/2,height/2,0,gl.LUMINANCE,gl.UNSIGNED_BYTE,uplane);var e=gl.getError();if(e!=gl.NO_ERROR){console.log("upload u plane ",width,height,uplane.byteLength,"  error",e)}gl.activeTexture(gl.TEXTURE2);gl.bindTexture(gl.TEXTURE_2D,vTexture);gl.texImage2D(gl.TEXTURE_2D,0,gl.LUMINANCE,width/2,height/2,0,gl.LUMINANCE,gl.UNSIGNED_BYTE,vplane);var e=gl.getError();if(e!=gl.NO_ERROR){console.log("upload v plane ",width,height,vplane.byteLength,"  error",e)}}function deleteBuffer(buffer){if(buffer&&that.gl){that.gl.deleteBuffer(buffer)}}function deleteTexture(texture){if(texture&&that.gl){that.gl.deleteTexture(texture)}}function initCanvas(view,mirror,width,height,rotation,onFailure){that.clientWidth=view.clientWidth;that.clientHeight=view.clientHeight;that.view=view;that.mirrorView=mirror;that.canvasUpdated=false;that.container=document.createElement("div");that.container.style.width="100%";that.container.style.height="100%";that.container.style.display="flex";that.container.style.justifyContent="center";that.container.style.alignItems="center";that.view.appendChild(that.container);that.canvas=document.createElement("canvas");if(rotation==0||rotation==180){that.canvas.width=width;that.canvas.height=height}else{that.canvas.width=height;that.canvas.height=width}that.initWidth=width;that.initHeight=height;that.initRotation=rotation;if(that.mirrorView){that.canvas.style.transform="rotateY(180deg)"}that.container.appendChild(that.canvas);try{gl=that.canvas.getContext("webgl")||that.canvas.getContext("experimental-webgl")}catch(e){console.log(e)}if(!gl){gl=undefined;onFailure({error:"Browser not support! No WebGL detected."});return}gl.clearColor(0,0,0,1);gl.enable(gl.DEPTH_TEST);gl.depthFunc(gl.LEQUAL);gl.clear(gl.COLOR_BUFFER_BIT|gl.DEPTH_BUFFER_BIT);program=createProgramFromSources(gl,[AgoraRTC.vertexShaderSource,AgoraRTC.yuvShaderSource]);gl.useProgram(program);initTextures()}function initTextures(){positionLocation=gl.getAttribLocation(program,"a_position");texCoordLocation=gl.getAttribLocation(program,"a_texCoord");surfaceBuffer=gl.createBuffer();texCoordBuffer=gl.createBuffer();gl.activeTexture(gl.TEXTURE0);yTexture=gl.createTexture();gl.bindTexture(gl.TEXTURE_2D,yTexture);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_WRAP_S,gl.CLAMP_TO_EDGE);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_WRAP_T,gl.CLAMP_TO_EDGE);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_MIN_FILTER,gl.NEAREST);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_MAG_FILTER,gl.NEAREST);gl.activeTexture(gl.TEXTURE1);uTexture=gl.createTexture();gl.bindTexture(gl.TEXTURE_2D,uTexture);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_WRAP_S,gl.CLAMP_TO_EDGE);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_WRAP_T,gl.CLAMP_TO_EDGE);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_MIN_FILTER,gl.NEAREST);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_MAG_FILTER,gl.NEAREST);gl.activeTexture(gl.TEXTURE2);vTexture=gl.createTexture();gl.bindTexture(gl.TEXTURE_2D,vTexture);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_WRAP_S,gl.CLAMP_TO_EDGE);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_WRAP_T,gl.CLAMP_TO_EDGE);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_MIN_FILTER,gl.NEAREST);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_MAG_FILTER,gl.NEAREST);var y=gl.getUniformLocation(program,"Ytex");gl.uniform1i(y,0);var u=gl.getUniformLocation(program,"Utex");gl.uniform1i(u,1);var v=gl.getUniformLocation(program,"Vtex");gl.uniform1i(v,2)}function updateCanvas(rotation,width,height){if(that.canvasUpdated){return}that.canvas.style.width="100%";that.canvas.style.height="100%";try{if(rotation===0||rotation===180){if(that.clientWidth/that.clientHeight>width/height){that.canvas.style.width=that.clientHeight*width/height+"px"}else if(that.clientWidth/that.clientHeight<width/height){that.canvas.style.height=that.clientWidth*height/width+"px"}}else{if(that.clientHeight/that.clientWidth>width/height){that.canvas.style.height=that.clientWidth*width/height+"px"}else if(that.clientHeight/that.clientWidth<width/height){that.canvas.style.width=that.clientHeight*height/width+"px"}}}catch(e){console.log("updateCanvas 00001 gone "+that.canvas);console.log(that);console.error(e);return}gl.bindBuffer(gl.ARRAY_BUFFER,surfaceBuffer);gl.enableVertexAttribArray(positionLocation);gl.vertexAttribPointer(positionLocation,2,gl.FLOAT,false,0,0);var p1={x:0,y:0};var p2={x:width,y:0};var p3={x:width,y:height};var p4={x:0,y:height};var pp1=p1,pp2=p2,pp3=p3,pp4=p4;switch(rotation){case 0:break;case 90:pp1=p2;pp2=p3;pp3=p4;pp4=p1;break;case 180:pp1=p3;pp2=p4;pp3=p1;pp4=p2;break;case 270:pp1=p4;pp2=p1;pp3=p2;pp4=p3;break;default:}gl.bufferData(gl.ARRAY_BUFFER,new Float32Array([pp1.x,pp1.y,pp2.x,pp2.y,pp4.x,pp4.y,pp4.x,pp4.y,pp2.x,pp2.y,pp3.x,pp3.y]),gl.STATIC_DRAW);var resolutionLocation=gl.getUniformLocation(program,"u_resolution");gl.uniform2f(resolutionLocation,width,height);that.canvasUpdated=true}return that}var AgoraRTC=function(){"use strict";var AgoraRTC={};Object.defineProperties(AgoraRTC,{version:{get:function(){return"<%= pkg.version %>"}},name:{get:function(){return"<%= pkg.title %>"}}});return AgoraRTC}();AgoraRTC.url="wss://localhost.agora.io:8921/";AgoraRTC.macAgentInstallUrl="http://download.agora.io/AgoraWebAgent-1.6.0.pkg";AgoraRTC.winAgentInstallUrl="http://download.agora.io/AgoraWebAgentSetup-1.6.0.exe";AgoraRTC.enAgentInstallGuideUrl="http://download.agora.io/install-guide-en.html";AgoraRTC.cnAgentInstallGuideUrl="http://download.agora.io/install-guide-cn.html";AgoraRTC.vertexShaderSource="attribute vec2 a_position;"+"attribute vec2 a_texCoord;"+"uniform vec2 u_resolution;"+"varying vec2 v_texCoord;"+"void main() {"+"vec2 zeroToOne = a_position / u_resolution;"+"   vec2 zeroToTwo = zeroToOne * 2.0;"+"   vec2 clipSpace = zeroToTwo - 1.0;"+"   gl_Position = vec4(clipSpace * vec2(1, -1), 0, 1);"+"v_texCoord = a_texCoord;"+"}";AgoraRTC.yuvShaderSource="precision mediump float;"+"uniform sampler2D Ytex;"+"uniform sampler2D Utex,Vtex;"+"varying vec2 v_texCoord;"+"void main(void) {"+"  float nx,ny,r,g,b,y,u,v;"+"  mediump vec4 txl,ux,vx;"+"  nx=v_texCoord[0];"+"  ny=v_texCoord[1];"+"  y=texture2D(Ytex,vec2(nx,ny)).r;"+"  u=texture2D(Utex,vec2(nx,ny)).r;"+"  v=texture2D(Vtex,vec2(nx,ny)).r;"+"  y=1.1643*(y-0.0625);"+"  u=u-0.5;"+"  v=v-0.5;"+"  r=y+1.5958*v;"+"  g=y-0.39173*u-0.81290*v;"+"  b=y+2.017*u;"+"  gl_FragColor=vec4(r,g,b,1.0);"+"}";AgoraRTC.videoStreamSource='function onSuccess(){postMessage({type:"init",result:!0})}function onFailure(t){postMessage(t)}function sendMessage(t){"videoStat"!=t.command&&console.log(JSON.stringify(t)),that.stream&&that.stream.readyState==WebSocket.OPEN&&that.stream.send(JSON.stringify(t))}var that={};that.stream=void 0,that.init=function(t){return that.stream&&that.stream.readyState===WebSocket.OPEN?(console.warn("stream "+that.getId()+" has been initialized already"),void onSuccess()):(that.local=t.local,that.profile=t.videoProfile,that.stream=new WebSocket(t.url),that.stream.onopen=function(t){console.log((that.local?"local":"remote")+" Stream "),console.log(t),that.profile&&sendMessage({command:"setVideoProfile",profile:that.profile}),that.stream.binaryType="arraybuffer",onSuccess()},that.stream.onclose=function(t){console.log(that.local?"local":"remote"," Stream ",t),that.stream=void 0,onFailure({type:t.type,code:t.code,reason:t.reason})},that.stream.onerror=function(t){console.log(that.local?"local":"remote"," Stream ",t),onFailure({type:t.type,code:t.code,reason:t.reason})},void(that.stream.onmessage=function(t){"string"==typeof t.data?console.log(that.local?"local":"remote"," message from agent ",t.data):t.data instanceof ArrayBuffer?postMessage({type:"message",data:t.data}):t.data instanceof Blob&&console.warn("Blob image data is not supported")}))},that.close=function(){that.stream&&(that.stream.onmessage=void 0,that.stream.close()),that.stream=void 0},that.setVideoProfile=function(t){return"string"==typeof t?(that.stream?sendMessage({command:"setVideoProfile",profile:t}):that.profile=t,!0):!1},self.addEventListener("message",function(t){var e=t.data;switch(e.type){case"init":that.init(e);break;case"send":sendMessage(e.message);break;case"close":that.close()}},!1);';var AgoraCall=function(callback,parameters){if(callback){callback(parameters)}};var L={VideoProfiles:{Profile120P:"120P",Profile120P_3:"120P_3",Profile96P:"96P",Profile48P:"48P",Profile180P:"180P",Profile180P_3:"180P_3",Profile180P_4:"180P_4",Profile240P:"240P",Profile240P_3:"240P_3",Profile240P_4:"240P_4",Profile360P:"360P",Profile360P_3:"360P_3",Profile360P_4:"360P_4",Profile360P_6:"360P_6",Profile360P_7:"360P_7",Profile360P_8:"360P_8",Profile480P:"480P",Profile480P_3:"480P_3",Profile480P_4:"480P_4",Profile480P_6:"480P_6",Profile480P_8:"480P_8",Profile480P_9:"480P_9",Profile720P:"720P",Profile720P_3:"720P_3",Profile720P_5:"720P_5",Profile720P_6:"720P_6"},ErrorCode:{NO_ERROR:0,FAILED:1,INVALID_ARGUMENT:2,NOT_READY:3,NOT_SUPPORTED:4,REFUSED:5,BUFFER_TOO_SMALL:6,NOT_INITIALIZED:7,INVALID_VIEW:8,NO_PERMISSION:9,TIMEDOUT:10,CANCELED:11,TOO_OFTEN:12,BIND_SOCKET:13,NET_DOWN:14,NET_NOBUFS:15,INIT_VIDEO:16,JOIN_CHANNEL_REJECTED:17,LEAVE_CHANNEL_REJECTED:18,ALREADY_IN_USE:19,INVALID_APP_ID:101,INVALID_CHANNEL_NAME:102,CHANNEL_KEY_EXPIRED:109,INVALID_CHANNEL_KEY:110,CONNECTION_INTERRUPTED:111,CONNECTION_LOST:112,LOAD_MEDIA_ENGINE:1001,START_CALL:1002,START_CAMERA:1003,START_VIDEO_RENDER:1004,ADM_GENERAL_ERROR:1005,ADM_JAVA_RESOURCE:1006,ADM_SAMPLE_RATE:1007,ADM_INIT_PLAYOUT:1008,ADM_START_PLAYOUT:1009,ADM_STOP_PLAYOUT:1010,ADM_INIT_RECORDING:1011,ADM_START_RECORDING:1012,ADM_STOP_RECORDING:1013,ADM_RUNTIME_PLAYOUT_ERROR:1015,ADM_RUNTIME_RECORDING_ERROR:1017,ADM_RECORD_AUDIO_FAILED:1018,ADM_INIT_LOOPBACK:1022,ADM_START_LOOPBACK:1023,VDM_CAMERA_NOT_AUTHORIZED:1501}};AgoraRTC.EventDispatcher=function(spec){"use strict";var that={};spec.dispatcher={};spec.dispatcher.eventListeners={};that.addEventListener=function(eventType,listener){spec.dispatcher.eventListeners[eventType]=listener;if(!spec.dispatcher.eventListeners[eventType]){delete spec.dispatcher.eventListeners[eventType]}};that.on=that.addEventListener;that.dispatchEvent=function(event){if(spec.dispatcher.eventListeners.hasOwnProperty(event.type)){spec.dispatcher.eventListeners[event.type](event)}};return that};AgoraRTC.BasicEvent=function(spec){"use strict";var that={};that.type=spec.type;return that};AgoraRTC.StreamEvent=function(spec){"use strict";var that=AgoraRTC.BasicEvent(spec);that.stream=spec.stream;that.msg=spec.msg;return that};AgoraRTC.ClientEvent=function(spec){"use strict";var that=AgoraRTC.BasicEvent(spec);that.uid=spec.uid;that.attr=spec.attr;that.streams=spec.streams;return that};AgoraRTC.MediaEvent=function(spec){"use strict";var that=AgoraRTC.BasicEvent(spec);that.msg=spec.msg;return that};AgoraRTC.Signal=function(spec){var that=AgoraRTC.EventDispatcher(spec);that.connection=new WebSocket(spec.url);that.sendMessage=function(obj,onError){if(that.connection.readyState==WebSocket.OPEN){that.connection.send(JSON.stringify(obj))}else{console.log("connection to agent lost.");onError({error:"not connected"})}};that.close=function(){that.onEvent=onEvent;that.connection.onopen=undefined;that.connection.onclose=undefined;that.connection.onerror=undefined;that.connection.onmessage=undefined;that.connection.close()};that.connection.onopen=function(e){console.log(e);that.dispatchEvent(AgoraRTC.MediaEvent({type:"onopen",event:e}))};that.connection.onclose=function(e){console.log(e);AgoraCall(spec.onFailure,e)};that.connection.onerror=function(e){console.log(e);AgoraCall(spec.onFailure,e)};that.onEvent=onEvent;that.connection.onmessage=function(e){console.log(e);var m=JSON.parse(e.data);if(m.hasOwnProperty("command")){that.dispatchEvent(AgoraRTC.MediaEvent({type:m.command,msg:m}))}else if(m.hasOwnProperty("event")){that.onEvent(m)}};function onEvent(e){console.log(e)}return that};AgoraRTC.Stream=function(spec){var that=AgoraRTC.EventDispatcher(spec);that.stream=undefined;that.render=undefined;that.interval=undefined;that.lastRenderCount=0;that.profile=undefined;that.latency=0;var header=undefined;var yplane=undefined;var uplane=undefined;var vplane=undefined;that.init=function(onSuccess,onFailure){if(that.stream){console.warn("stream "+that.getId()+" has been initialized already");onSuccess();return}var videoStreamFileBlob=new Blob([AgoraRTC.videoStreamSource],{type:"application/javascript"});that.stream=new Worker(URL.createObjectURL(videoStreamFileBlob));that.oninit=function(e){console.log((spec.local?"local":"remote")+" Stream ");console.log(e);if(e.result===true){onSuccess()}else{onFailure()}};that.onclose=function(e){window.clearInterval(that.interval);console.log(spec.local?"local":"remote"," Stream ",e);that.stream=undefined};that.onerror=function(e){window.clearInterval(that.interval);console.log(spec.local?"local":"remote"," Stream ",e);onFailure(e)};that.onmessage=function(e){if(typeof e==="string"){console.log(spec.local?"local":"remote"," message from agent ",e)}else if(e instanceof ArrayBuffer){if(that.render){that.drawImage(e)}}else if(e instanceof Blob){console.warn("Blob image data is not supported")}};that.stream.addEventListener("message",onWorkerMessage,false);that.stream.postMessage({type:"init",local:spec.local,url:AgoraRTC.url,videoProfile:that.profile})};that.close=function(){
window.clearInterval(that.interval);if(that.stream){that.stream.removeEventListener("message",onWorkerMessage);that.stream.postMessage({type:"close"})}that.stream=undefined;that.stop()};that.play=function(div,onFailure){var view=document.getElementById(div);that.stop();that.render=AgoraRender();that.render.start(view,onFailure);if(spec.local){that.stream.postMessage({type:"send",message:{command:"preview",uid:spec.streamID,audio:spec.audio===false?false:true,video:spec.video===false?false:true,screen:spec.screen===true?true:false}})}else{that.stream.postMessage({type:"send",message:{command:"subscribe",uid:spec.streamID}})}if(that.interval){window.clearInterval(that.interval)}that.interval=window.setInterval(function(){if(that.render){var fps=that.render.renderImageCount-that.lastRenderCount;that.stream.postMessage({type:"send",message:{command:"videoStat",uid:spec.streamID,fps:fps,frameCount:that.lastRenderCount,latency:that.latency}});that.lastRenderCount=that.render.renderImageCount;that.latency=0}},1e3)};that.stop=function(){if(that.render){that.render.stop()}that.render=undefined;that.lastRenderCount=0};that.bindClient=function(client){that.client=client};that.enableAudio=function(callback){return that.client.enableAudio(that,callback)};that.disableAudio=function(callback){return that.client.disableAudio(that,callback)};that.enableVideo=function(callback){return that.client.enableVideo(that,callback)};that.disableVideo=function(callback){return that.client.disableVideo(that,callback)};that.getId=function(){return spec.streamID};that.drawImage=function(data){if(!header){header=data;if(header.byteLength!=20){console.error("invalid image header "+data.byteLength);resetImageData()}return}if(!yplane){yplane=data;if(yplane.byteLength===20){console.error("invalid image header "+data.byteLength+" "+yplane.byteLength);resetImageData()}return}if(!uplane){uplane=data;if(uplane.byteLength===20){console.error("invalid image header "+data.byteLength+" "+yplane.byteLength+" "+uplane.byteLength);resetImageData()}return}if(!vplane){vplane=data;if(yplane.byteLength!=uplane.byteLength*4||uplane.byteLength!=vplane.byteLength){console.error("invalid image header "+data.byteLength+" "+yplane.byteLength+" "+uplane.byteLength+" "+vplane.byteLength);resetImageData();return}var headerLength=20;var dv=new DataView(header);var format=dv.getUint8(0);var mirror=dv.getUint8(1);var width=dv.getUint16(2);var height=dv.getUint16(4);var left=dv.getUint16(6);var top=dv.getUint16(8);var right=dv.getUint16(10);var bottom=dv.getUint16(12);var rotation=dv.getUint16(14);var ts=dv.getUint32(16);var xWidth=width+left+right;var xHeight=height+top+bottom;var yLength=xWidth*xHeight;var yBegin=headerLength;var yEnd=yBegin+yLength;var uLength=yLength/4;var uBegin=yEnd;var uEnd=uBegin+uLength;var vLength=yLength/4;var vBegin=uEnd;var vEnd=vBegin+vLength;that.render.renderImage({mirror:mirror,width:width,height:height,left:left,top:top,right:right,bottom:bottom,rotation:rotation,yplane:new Uint8Array(yplane),uplane:new Uint8Array(uplane),vplane:new Uint8Array(vplane)});var now32=(Date.now()&4294967295)>>>0;var latency=now32-ts;if(latency>that.latency){that.latency=latency}resetImageData()}};that.setVideoProfile=function(profile){if(typeof profile=="string"){if(that.stream){that.stream.postMessage({type:"send",message:{command:"setVideoProfile",profile:profile}})}else{that.profile=profile}return true}else{return false}};function resetImageData(){header=undefined;yplane=undefined;uplane=undefined;vplane=undefined}function getReso(w,h){return{width:w,height:h}}var supportedVideoList={true:true,unspecified:true,"120p":getReso(160,120),"240p":getReso(320,240),"360p":getReso(640,360),"480p":getReso(640,480),"720p":getReso(1280,720),"1080p":getReso(1920,1080),"4k":getReso(3840,2160)};function onWorkerMessage(e){var data=e.data;switch(data.type){case"init":that.oninit(data);break;case"close":that.onclose(data);break;case"error":that.onerror(data);break;case"message":that.onmessage(data.data);break}}return that};AgoraRTC.createStream=function(spec){return AgoraRTC.Stream(spec)};AgoraRTC.Client=function(spec){"use strict";var that=AgoraRTC.EventDispatcher(spec);that.signal=undefined;that.localStream=undefined;that.remoteStreams={};that.signalOpen=false;function onSignalError(e){if(that.signalOpen){console.log("signal connection lost.");var message={reason:"LOST_CONNECTION_TO_AGENT",type:"error",eventType:e.type,code:e.code};AgoraCall(that.onFailure,message);that.dispatchEvent(message);return}if(e.type==="close"){return}var message={reason:"CLOSE_BEFORE_OPEN",type:"error",eventType:e.type,code:e.code,agentInstallUrl:getInstallUrl(),agentInstallGuideUrl:getInstallGuideUrl()};AgoraCall(that.onFailure,message);that.dispatchEvent(message)}that.init=function(appId,onSuccess,onFailure){spec.appId=appId;that.onFailure=onFailure;if(!that.signal){try{that.signal=AgoraRTC.Signal({url:AgoraRTC.url,onFailure:onSignalError})}catch(e){console.log("create signal connection failed"+e);AgoraCall(onFailure,e);return}that.signal.on("onopen",function(event){that.signalOpen=true;that.signal.onEvent=function(e){if(e.event==="error"){var name=getErrorCodeName(e.code);if(name){e.reason=name}}else if(e.event==="onUserMuteAudio"){e.event="peer-mute-audio"}else if(e.event==="onUserMuteVideo"){e.event="peer-mute-video"}that.dispatchEvent(AgoraRTC.MediaEvent({type:e.event,msg:e}))};that.signal.sendMessage({command:"initialize",appId:spec.appId},function(err){AgoraCall(onFailure,err)})})}if(that.signalOpen){that.signal.sendMessage({command:"initialize",appId:spec.appId},function(err){AgoraCall(onFailure,err)})}function onError(code){var name=getErrorCodeName(code);if(name){AgoraCall(onFailure,{reason:name});return}AgoraCall(onFailure,{reason:"UNKNOWN_ERROR",code:m.code});return}function getErrorCodeName(code){for(var k in L.ErrorCode){if(L.ErrorCode.hasOwnProperty(k)){if(L.ErrorCode[k]===code){return String(k)}}}return null}that.signal.on("initialize",function(ev){var m=ev.msg;if(m.code!=true){that.signal.close();that.signal=undefined;onError(m.code);return}AgoraCall(onSuccess,m)});that.signal.on("onError",function(ev){var m=ev.msg;onError(m.code)});console.log(that.signal)};that.renewChannelKey=function(channelKey,onSuccess,onFailure){that.signal.sendMessage({command:"renewChannelKey",channelKey:channelKey},function(e){AgoraCall(onFailure,e)});that.signal.on("renewChannelKey",function(ev){var m=ev.msg;if(m.code===true){AgoraCall(onSuccess,m);return}AgoraCall(onFailure,m)})};that.join=function(channelKey,channelName,uid,onSuccess,onFailure){if(channelName.length>64){onFailure(L.ErrorCode.INVALID_CHANNEL_NAME);return}that.signal.sendMessage({command:"joinChannel",channelKey:channelKey,channelName:channelName,uid:uid},function(e){AgoraCall(onFailure,{error:e})});that.signal.on("joinChannel",function(ev){that.signal.on("onAddVideoStream",onAddVideoStream);that.signal.on("onPeerLeave",onPeerLeave);var m=ev.msg;if(m.code==true){if(!m.uid){m.uid=0}AgoraCall(onSuccess,m.uid);return}if(m.code==L.ErrorCode.JOIN_CHANNEL_REJECTED){console.error("Command joinChannel has been rejected by agent. Is this user joined a channel already?");AgoraCall(onFailure,m.code);return}AgoraCall(onFailure,m.code)})};that.leave=function(onSuccess,onFailure){closeLocalRemoteStreams();that.signal.sendMessage({command:"leaveChannel"},function(e){console.log("leave channel failed",e)});that.signal.on("leaveChannel",function(ev){var m=ev.msg;if(m.code===true){AgoraCall(onSuccess,m);return}if(m.code==L.ErrorCode.LEAVE_CHANNEL_REJECTED){console.error("Command leaveChannel has been rejected by agent. Is this user not in a channel?");AgoraCall(onSuccess,m);return}AgoraCall(onFailure,{code:m.code})})};that.publish=function(stream,onSuccess,onFailure){that.localStream=stream;that.signal.sendMessage({command:"unmuteLocal"},function(e){AgoraCall(onFailure,{error:e})});that.signal.on("unmuteLocal",function(ev){var m=ev.msg;if(m.code!=true){AgoraCall(onFailure,{code:m.code});return}AgoraCall(onSuccess,m)});stream.bindClient(that)};that.unpublish=function(stream,onSuccess,onFailure){that.signal.sendMessage({command:"muteLocal"},function(e){AgoraCall(onFailure,{error:e})});that.signal.on("muteLocal",function(ev){var m=ev.msg;if(m.code!=true){AgoraCall(onFailure,{code:m.code});return}AgoraCall(onSuccess,m)})};that.subscribe=function(stream,onFailure){stream.init(function(){that.dispatchEvent(AgoraRTC.StreamEvent({type:"stream-subscribed",stream:stream}))},function(error){AgoraCall(onFailure,error)})};that.unsubscribe=function(streamID,onFailure){console.log("remote streams",that.remoteStreams);if(that.remoteStreams[streamID]==undefined){AgoraCall(onFailure,{error:"no such stream"});return}that.remoteStreams[streamID].close();delete that.remoteStreams[streamID]};that.enableAudio=function(stream,callback){that.signal.on("enableAudio",function(ev){var m=ev.msg;if(callback){AgoraCall(callback,m.code===true)}});that.signal.sendMessage({command:"enableAudio",streamID:stream.getId()},function(e){e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(callback,e)});return true};that.disableAudio=function(stream,callback){that.signal.on("disableAudio",function(ev){var m=ev.msg;if(callback){AgoraCall(callback,m.code===true)}});that.signal.sendMessage({command:"disableAudio",streamID:stream.getId()},function(e){e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(callback,e)});return true};that.enableVideo=function(stream,callback){that.signal.on("enableVideo",function(ev){var m=ev.msg;if(callback){AgoraCall(callback,m.code===true)}});that.signal.sendMessage({command:"enableVideo",streamID:stream.getId()},function(e){e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(callback,e)});return true};that.disableVideo=function(stream,callback){that.signal.on("disableVideo",function(ev){var m=ev.msg;if(callback){AgoraCall(callback,m.code===true)}});that.signal.sendMessage({command:"disableVideo",streamID:stream.getId()},function(e){e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(callback,e)});return true};that.getDevices=function(callback){that.signal.sendMessage({command:"getDevices"},function(e){e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(callback,e)});that.signal.on("getDevices",function(ev){var devices=ev.msg.devices;AgoraCall(callback,devices)})};that.selectDevice=function(device,onFailure){that.signal.sendMessage({command:"selectDevice",device:device},function(e){e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(onFailure,e)});that.signal.on("selectDevice",function(ev){console.log(ev)})};that.startRecording=function(recordingKey,onSuccess,onFailure){that.signal.sendMessage({command:"startRecording",recordingKey:recordingKey},function(e){e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(onFailure,e)});that.signal.on("startRecording",function(ev){var start=ev.msg;if(start.code===true){AgoraCall(onSuccess,start)}else{AgoraCall(onFailure,start)}})};that.stopRecording=function(recordingKey,onSuccess,onFailure){that.signal.sendMessage({command:"stopRecording",recordingKey:recordingKey},function(e){e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(onFailure,e)});that.signal.on("stopRecording",function(ev){var stop=ev.msg;if(stop.code===true){AgoraCall(onSuccess,stop)}else{AgoraCall(onFailure,stop)}})};that.queryRecordingStatus=function(onStatus){that.signal.sendMessage({command:"queryRecordingStatus"},function(e){e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(onStatus,e)});that.signal.on("queryRecordingStatus",function(ev){onStatus(ev.msg)})};that.close=function(){closeLocalRemoteStreams();if(that.signal){that.signal.close();that.signal=undefined}};that.getVersion=function(onSuccess,onFailure){that.signal.sendMessage({command:"getVersion"});that.signal.on("getVersion",function(ev){var m=ev.msg;if(m.code===true){onSuccess(m)}else{onFailure(m)}})};that.setParameters=function(parameters){that.signal.sendMessage({command:"setParameters",parameters:JSON.stringify(parameters)})};that.setEncryptionMode=function(encryptionMode){that.signal.sendMessage({command:"setEncryptionMode",mode:encryptionMode})};that.setEncryptionSecret=function(secret){that.signal.sendMessage({command:"setEncryptionSecret",secret:secret})};that.getWindows=function(callback){that.signal.sendMessage({command:"getWindows"},function(e){e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(callback,e)});that.signal.on("getWindows",function(ev){var windows=ev.msg.windows;AgoraCall(callback,windows)})};that.startScreenSharing=function(window,onSuccess,onFailure){that.signal.sendMessage({command:"startScreenSharing",window:window},function(e){e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(onFailure,e)});that.signal.on("startScreenSharing",function(e){AgoraCall(onSuccess,e)})};that.setScreenSharingWindow=function(window,onSuccess,onFailure){that.signal.sendMessage({command:"setScreenSharingWindow",window:window},function(e){e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(onFailure,e)});that.signal.on("setScreenSharingWindow",function(e){AgoraCall(onSuccess,e);console.log(e)})};that.stopScreenSharing=function(onSuccess,onFailure){that.signal.sendMessage({command:"stopScreenSharing"},function(e){e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(onFailure,e)});that.signal.on("stopScreenSharing",function(e){AgoraCall(onSuccess,e);console.log(e)})};function onPeerLeave(ev){var m=ev.msg;if(that.remoteStreams.hasOwnProperty(m.uid)){that.remoteStreams[m.uid].close();delete that.remoteStreams[m.uid];console.log("remote streams after peer leave ",that.remoteStreams)}that.dispatchEvent(AgoraRTC.ClientEvent({type:"peer-leave",uid:m.uid}))}function onAddVideoStream(ev){var m=ev.msg;if(!that.remoteStreams.hasOwnProperty(m.uid)){var stream=AgoraRTC.Stream({streamID:m.uid,local:false,audio:m.audio,video:m.video,screen:m.screen});stream.bindClient(that);that.remoteStreams[m.uid]=stream}that.dispatchEvent(AgoraRTC.StreamEvent({type:"stream-added",stream:that.remoteStreams[m.uid]}));console.log("remote streams",that.remoteStreams)}function getInstallUrl(){if(navigator.appVersion.indexOf("Mac")!=-1){return AgoraRTC.macAgentInstallUrl}return AgoraRTC.winAgentInstallUrl}function getInstallGuideUrl(){var userLang=navigator.language||navigator.userLanguage;if(userLang.indexOf("zh")!=-1){return AgoraRTC.cnAgentInstallGuideUrl}return AgoraRTC.enAgentInstallGuideUrl}function closeLocalRemoteStreams(){if(that.localStream){that.localStream.close()}that.localStream=undefined;for(var streamId in that.remoteStreams){if(that.remoteStreams.hasOwnProperty(streamId)){that.remoteStreams[streamId].close();delete that.remoteStreams[streamId]}}}return that};AgoraRTC.createRtcClient=function(){return AgoraRTC.Client({})};
//# sourceMappingURL=dist/AgoraRtcAgentSDK.min.js.map
"use strict";(function(root,factory){if(typeof define==="function"&&define.amd){define([],factory)}else{var lib=factory.call(root);Object.keys(lib).forEach(function(key){root[key]=lib[key]})}})(this,function(){var topWindow=this;function error(msg){if(topWindow.console){if(topWindow.console.error){topWindow.console.error(msg)}else if(topWindow.console.log){topWindow.console.log(msg)}}}function isInIFrame(w){w=w||topWindow;return w!==w.top}function makeFailHTML(msg){return""+'<table style="background-color: #8CE; width: 100%; height: 100%;"><tr>'+'<td align="center">'+'<div style="display: table-cell; vertical-align: middle;">'+'<div style="">'+msg+"</div>"+"</div>"+"</td></tr></table>"}var GET_A_WEBGL_BROWSER=""+"This page requires a browser that supports WebGL.<br/>"+'<a href="http://get.webgl.org">Click here to upgrade your browser.</a>';var OTHER_PROBLEM=""+"It doesn't appear your computer can support WebGL.<br/>"+'<a href="http://get.webgl.org/troubleshooting/">Click here for more information.</a>';function create3DContext(canvas,opt_attribs){var names=["webgl","experimental-webgl"];var context=null;for(var ii=0;ii<names.length;++ii){try{context=canvas.getContext(names[ii],opt_attribs)}catch(e){}if(context){break}}return context}function setupWebGL(canvas,opt_attribs){function showLink(str){var container=canvas.parentNode;if(container){container.innerHTML=makeFailHTML(str)}}if(!topWindow.WebGLRenderingContext){showLink(GET_A_WEBGL_BROWSER);return null}var context=create3DContext(canvas,opt_attribs);if(!context){showLink(OTHER_PROBLEM)}return context}function updateCSSIfInIFrame(){if(isInIFrame()){document.body.className="iframe"}}function getWebGLContext(canvas,opt_attribs,opt_options){var options=opt_options||{};if(isInIFrame()){updateCSSIfInIFrame();if(!options.dontResize&&options.resize!==false){var width=canvas.clientWidth;var height=canvas.clientHeight;canvas.width=width;canvas.height=height}}else if(!options.noTitle&&options.title!==false){var title=document.title;var h1=document.createElement("h1");h1.innerText=title;document.body.insertBefore(h1,document.body.children[0])}var gl=setupWebGL(canvas,opt_attribs);return gl}function loadShader(gl,shaderSource,shaderType,opt_errorCallback){var errFn=opt_errorCallback||error;var shader=gl.createShader(shaderType);gl.shaderSource(shader,shaderSource);gl.compileShader(shader);var compiled=gl.getShaderParameter(shader,gl.COMPILE_STATUS);if(!compiled){var lastError=gl.getShaderInfoLog(shader);errFn("*** Error compiling shader '"+shader+"':"+lastError);gl.deleteShader(shader);return null}return shader}function createProgram(gl,shaders,opt_attribs,opt_locations,opt_errorCallback){var errFn=opt_errorCallback||error;var program=gl.createProgram();shaders.forEach(function(shader){gl.attachShader(program,shader)});if(opt_attribs){obj_attrib.forEach(function(attrib,ndx){gl.bindAttribLocation(program,opt_locations?opt_locations[ndx]:ndx,attrib)})}gl.linkProgram(program);var linked=gl.getProgramParameter(program,gl.LINK_STATUS);if(!linked){var lastError=gl.getProgramInfoLog(program);errFn("Error in program linking:"+lastError);gl.deleteProgram(program);return null}return program}function createShaderFromScript(gl,scriptId,opt_shaderType,opt_errorCallback){var shaderSource="";var shaderType;var shaderScript=document.getElementById(scriptId);if(!shaderScript){throw"*** Error: unknown script element"+scriptId}shaderSource=shaderScript.text;if(!opt_shaderType){if(shaderScript.type==="x-shader/x-vertex"){shaderType=gl.VERTEX_SHADER}else if(shaderScript.type==="x-shader/x-fragment"){shaderType=gl.FRAGMENT_SHADER}else if(shaderType!==gl.VERTEX_SHADER&&shaderType!==gl.FRAGMENT_SHADER){throw"*** Error: unknown shader type"}}return loadShader(gl,shaderSource,opt_shaderType?opt_shaderType:shaderType,opt_errorCallback)}var defaultShaderType=["VERTEX_SHADER","FRAGMENT_SHADER"];function createProgramFromScripts(gl,shaderScriptIds,opt_attribs,opt_locations,opt_errorCallback){var shaders=[];for(var ii=0;ii<shaderScriptIds.length;++ii){shaders.push(createShaderFromScript(gl,shaderScriptIds[ii],gl[defaultShaderType[ii]],opt_errorCallback))}return createProgram(gl,shaders,opt_attribs,opt_locations,opt_errorCallback)}function createProgramFromSources(gl,shaderSources,opt_attribs,opt_locations,opt_errorCallback){var shaders=[];for(var ii=0;ii<shaderSources.length;++ii){shaders.push(loadShader(gl,shaderSources[ii],gl[defaultShaderType[ii]],opt_errorCallback))}return createProgram(gl,shaders,opt_attribs,opt_locations,opt_errorCallback)}function getBindPointForSamplerType(gl,type){if(type===gl.SAMPLER_2D)return gl.TEXTURE_2D;if(type===gl.SAMPLER_CUBE)return gl.TEXTURE_CUBE_MAP}function createUniformSetters(gl,program){var textureUnit=0;function createUniformSetter(program,uniformInfo){var location=gl.getUniformLocation(program,uniformInfo.name);var type=uniformInfo.type;var isArray=uniformInfo.size>1&&uniformInfo.name.substr(-3)==="[0]";if(type===gl.FLOAT&&isArray){return function(v){gl.uniform1fv(location,v)}}if(type===gl.FLOAT){return function(v){gl.uniform1f(location,v)}}if(type===gl.FLOAT_VEC2){return function(v){gl.uniform2fv(location,v)}}if(type===gl.FLOAT_VEC3){return function(v){gl.uniform3fv(location,v)}}if(type===gl.FLOAT_VEC4){return function(v){gl.uniform4fv(location,v)}}if(type===gl.INT&&isArray){return function(v){gl.uniform1iv(location,v)}}if(type===gl.INT){return function(v){gl.uniform1i(location,v)}}if(type===gl.INT_VEC2){return function(v){gl.uniform2iv(location,v)}}if(type===gl.INT_VEC3){return function(v){gl.uniform3iv(location,v)}}if(type===gl.INT_VEC4){return function(v){gl.uniform4iv(location,v)}}if(type===gl.BOOL){return function(v){gl.uniform1iv(location,v)}}if(type===gl.BOOL_VEC2){return function(v){gl.uniform2iv(location,v)}}if(type===gl.BOOL_VEC3){return function(v){gl.uniform3iv(location,v)}}if(type===gl.BOOL_VEC4){return function(v){gl.uniform4iv(location,v)}}if(type===gl.FLOAT_MAT2){return function(v){gl.uniformMatrix2fv(location,false,v)}}if(type===gl.FLOAT_MAT3){return function(v){gl.uniformMatrix3fv(location,false,v)}}if(type===gl.FLOAT_MAT4){return function(v){gl.uniformMatrix4fv(location,false,v)}}if((type===gl.SAMPLER_2D||type===gl.SAMPLER_CUBE)&&isArray){var units=[];for(var ii=0;ii<info.size;++ii){units.push(textureUnit++)}return function(bindPoint,units){return function(textures){gl.uniform1iv(location,units);textures.forEach(function(texture,index){gl.activeTexture(gl.TEXTURE0+units[index]);gl.bindTexture(bindPoint,texture)})}}(getBindPointForSamplerType(gl,type),units)}if(type===gl.SAMPLER_2D||type===gl.SAMPLER_CUBE){return function(bindPoint,unit){return function(texture){gl.uniform1i(location,unit);gl.activeTexture(gl.TEXTURE0+unit);gl.bindTexture(bindPoint,texture)}}(getBindPointForSamplerType(gl,type),textureUnit++)}throw"unknown type: 0x"+type.toString(16)}var uniformSetters={};var numUniforms=gl.getProgramParameter(program,gl.ACTIVE_UNIFORMS);for(var ii=0;ii<numUniforms;++ii){var uniformInfo=gl.getActiveUniform(program,ii);if(!uniformInfo){break}var name=uniformInfo.name;if(name.substr(-3)==="[0]"){name=name.substr(0,name.length-3)}var setter=createUniformSetter(program,uniformInfo);uniformSetters[name]=setter}return uniformSetters}function setUniforms(setters,values){Object.keys(values).forEach(function(name){var setter=setters[name];if(setter){setter(values[name])}})}function createAttributeSetters(gl,program){var attribSetters={};function createAttribSetter(index){return function(b){gl.bindBuffer(gl.ARRAY_BUFFER,b.buffer);gl.enableVertexAttribArray(index);gl.vertexAttribPointer(index,b.numComponents||b.size,b.type||gl.FLOAT,b.normalize||false,b.stride||0,b.offset||0)}}var numAttribs=gl.getProgramParameter(program,gl.ACTIVE_ATTRIBUTES);for(var ii=0;ii<numAttribs;++ii){var attribInfo=gl.getActiveAttrib(program,ii);if(!attribInfo){break}var index=gl.getAttribLocation(program,attribInfo.name);attribSetters[attribInfo.name]=createAttribSetter(index)}return attribSetters}function setAttributes(setters,buffers){Object.keys(buffers).forEach(function(name){var setter=setters[name];if(setter){setter(buffers[name])}})}function createProgramInfo(gl,shaderSources,opt_attribs,opt_locations,opt_errorCallback){shaderSources=shaderSources.map(function(source){var script=document.getElementById(source);return script?script.text:source});var program=createProgramFromSources(gl,shaderSources,opt_attribs,opt_locations,opt_errorCallback);if(!program){return null}var uniformSetters=createUniformSetters(gl,program);var attribSetters=createAttributeSetters(gl,program);return{program:program,uniformSetters:uniformSetters,attribSetters:attribSetters}}function setBuffersAndAttributes(gl,setters,buffers){setAttributes(setters,buffers.attribs);if(buffers.indices){gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER,buffers.indices)}}var browserPrefixes=["","MOZ_","OP_","WEBKIT_"];function getExtensionWithKnownPrefixes(gl,name){for(var ii=0;ii<browserPrefixes.length;++ii){var prefixedName=browserPrefixes[ii]+name;var ext=gl.getExtension(prefixedName);if(ext){return ext}}}function resizeCanvasToDisplaySize(canvas,multiplier){multiplier=multiplier||1;var width=canvas.clientWidth*multiplier;var height=canvas.clientHeight*multiplier;if(canvas.width!==width||canvas.height!==height){canvas.width=width;canvas.height=height;return true}return false}function getIFrameForWindow(window){if(!isInIFrame(window)){return}var iframes=window.parent.document.getElementsByTagName("iframe");for(var ii=0;ii<iframes.length;++ii){var iframe=iframes[ii];if(iframe.contentDocument===window.document){return iframe}}}function isFrameVisible(window){try{var iframe=getIFrameForWindow(window);if(!iframe){return true}var bounds=iframe.getBoundingClientRect();var isVisible=bounds.top<window.parent.innerHeight&&bounds.bottom>=0&&bounds.left<window.parent.innerWidth&&bounds.right>=0;return isVisible&&isFrameVisible(window.parent)}catch(e){return true}}function isOnScreen(element){var isVisible=true;if(element){var bounds=element.getBoundingClientRect();isVisible=bounds.top<topWindow.innerHeight&&bounds.bottom>=0}return isVisible&&isFrameVisible(topWindow)}function augmentTypedArray(typedArray,numComponents){var cursor=0;typedArray.push=function(){for(var ii=0;ii<arguments.length;++ii){var value=arguments[ii];if(value instanceof Array||value.buffer&&value.buffer instanceof ArrayBuffer){for(var jj=0;jj<value.length;++jj){typedArray[cursor++]=value[jj]}}else{typedArray[cursor++]=value}}};typedArray.reset=function(opt_index){cursor=opt_index||0};typedArray.numComponents=numComponents;Object.defineProperty(typedArray,"numElements",{get:function(){return this.length/this.numComponents|0}});return typedArray}function createAugmentedTypedArray(numComponents,numElements,opt_type){var Type=opt_type||Float32Array;return augmentTypedArray(new Type(numComponents*numElements),numComponents)}function createBufferFromTypedArray(gl,array,type,drawType){type=type||gl.ARRAY_BUFFER;var buffer=gl.createBuffer();gl.bindBuffer(type,buffer);gl.bufferData(type,array,drawType||gl.STATIC_DRAW);return buffer}function allButIndices(name){return name!=="indices"}function createMapping(obj){var mapping={};Object.keys(obj).filter(allButIndices).forEach(function(key){mapping["a_"+key]=key});return mapping}function getGLTypeForTypedArray(gl,typedArray){if(typedArray instanceof Int8Array){return gl.BYTE}if(typedArray instanceof Uint8Array){return gl.UNSIGNED_BYTE}if(typedArray instanceof Int16Array){return gl.SHORT}if(typedArray instanceof Uint16Array){return gl.UNSIGNED_SHORT}if(typedArray instanceof Int32Array){return gl.INT}if(typedArray instanceof Uint32Array){return gl.UNSIGNED_INT}if(typedArray instanceof Float32Array){return gl.FLOAT}throw"unsupported typed array type"}function getNormalizationForTypedArray(typedArray){if(typedArray instanceof Int8Array){return true}if(typedArray instanceof Uint8Array){return true}return false}function isArrayBuffer(a){return a.buffer&&a.buffer instanceof ArrayBuffer}function guessNumComponentsFromName(name,length){var numComponents;if(name.indexOf("coord")>=0){numComponents=2}else if(name.indexOf("color")>=0){numComponents=4}else{numComponents=3}if(length%numComponents>0){throw"can not guess numComponents. You should specify it."}return numComponents}function makeTypedArray(array,name){if(isArrayBuffer(array)){return array}if(Array.isArray(array)){array={data:array}}if(!array.numComponents){array.numComponents=guessNumComponentsFromName(name,array.length)}var type=array.type;if(!type){if(name==="indices"){type=Uint16Array}}var typedArray=createAugmentedTypedArray(array.numComponents,array.data.length/array.numComponents|0,type);typedArray.push(array.data);return typedArray}function createAttribsFromArrays(gl,arrays,opt_mapping){var mapping=opt_mapping||createMapping(arrays);var attribs={};Object.keys(mapping).forEach(function(attribName){var bufferName=mapping[attribName];var array=makeTypedArray(arrays[bufferName],bufferName);attribs[attribName]={buffer:createBufferFromTypedArray(gl,array),numComponents:array.numComponents||guessNumComponentsFromName(bufferName),type:getGLTypeForTypedArray(gl,array),normalize:getNormalizationForTypedArray(array)}});return attribs}function getNumElementsFromNonIndexedArrays(arrays){var key=Object.keys(arrays)[0];var array=arrays[key];if(isArrayBuffer(array)){return array.numElements}else{return array.data.length/array.numComponents}}function createBufferInfoFromArrays(gl,arrays,opt_mapping){var bufferInfo={attribs:createAttribsFromArrays(gl,arrays,opt_mapping)};var indices=arrays.indices;if(indices){indices=makeTypedArray(indices,"indices");bufferInfo.indices=createBufferFromTypedArray(gl,indices,gl.ELEMENT_ARRAY_BUFFER);bufferInfo.numElements=indices.length}else{bufferInfo.numElements=getNumElementsFromNonIndexedArrays(arrays)}return bufferInfo}function createBuffersFromArrays(gl,arrays){var buffers={};Object.keys(arrays).forEach(function(key){var type=key==="indices"?gl.ELEMENT_ARRAY_BUFFER:gl.ARRAY_BUFFER;var array=makeTypedArray(arrays[key],name);buffers[key]=createBufferFromTypedArray(gl,array,type)});if(arrays.indices){buffers.numElements=arrays.indices.length}else if(arrays.position){buffers.numElements=arrays.position.length/3}return buffers}function drawBufferInfo(gl,type,bufferInfo,count,offset){var indices=bufferInfo.indices;var numElements=count===undefined?bufferInfo.numElements:count;offset=offset===undefined?offset:0;if(indices){gl.drawElements(type,numElements,gl.UNSIGNED_SHORT,offset)}else{gl.drawArrays(type,offset,numElements)}}function drawObjectList(gl,objectsToDraw){var lastUsedProgramInfo=null;var lastUsedBufferInfo=null;objectsToDraw.forEach(function(object){var programInfo=object.programInfo;var bufferInfo=object.bufferInfo;var bindBuffers=false;if(programInfo!==lastUsedProgramInfo){lastUsedProgramInfo=programInfo;gl.useProgram(programInfo.program);bindBuffers=true}if(bindBuffers||bufferInfo!==lastUsedBufferInfo){lastUsedBufferInfo=bufferInfo;setBuffersAndAttributes(gl,programInfo.attribSetters,bufferInfo)}setUniforms(programInfo.uniformSetters,object.uniforms);drawBufferInfo(gl,gl.TRIANGLES,bufferInfo)})}return{createAugmentedTypedArray:createAugmentedTypedArray,createAttribsFromArrays:createAttribsFromArrays,createBuffersFromArrays:createBuffersFromArrays,createBufferInfoFromArrays:createBufferInfoFromArrays,createAttributeSetters:createAttributeSetters,createProgram:createProgram,createProgramFromScripts:createProgramFromScripts,createProgramFromSources:createProgramFromSources,createProgramInfo:createProgramInfo,createUniformSetters:createUniformSetters,drawBufferInfo:drawBufferInfo,drawObjectList:drawObjectList,getWebGLContext:getWebGLContext,updateCSSIfInIFrame:updateCSSIfInIFrame,getExtensionWithKnownPrefixes:getExtensionWithKnownPrefixes,resizeCanvasToDisplaySize:resizeCanvasToDisplaySize,setAttributes:setAttributes,setBuffersAndAttributes:setBuffersAndAttributes,setUniforms:setUniforms,setupWebGL:setupWebGL}});function AgoraRender(){var gl=undefined;var program=undefined;var positionLocation=undefined;var texCoordLocation=undefined;var yTexture=undefined;var uTexture=undefined;var vTexture=undefined;var texCoordBuffer=undefined;var surfaceBuffer=undefined;var that={view:undefined,mirrorView:false,container:undefined,canvas:undefined,renderImageCount:0,initWidth:0,initHeight:0,initRotation:0,canvasUpdated:false,clientWidth:0,clientHeight:0};that.start=function(view,onFailure){initCanvas(view,that.mirrorView,view.clientWidth,view.clientHeight,that.initRotation,onFailure)};that.stop=function(){gl=undefined;program=undefined;positionLocation=undefined;texCoordLocation=undefined;deleteTexture(yTexture);deleteTexture(uTexture);deleteTexture(vTexture);yTexture=undefined;uTexture=undefined;vTexture=undefined;deleteBuffer(texCoordBuffer);deleteBuffer(surfaceBuffer);texCoordBuffer=undefined;surfaceBuffer=undefined;if(that.container){that.container.removeChild(that.canvas)}if(that.view){that.view.removeChild(that.container)}that.canvas=undefined;that.container=undefined;that.view=undefined;that.mirrorView=false};that.renderImage=function(image){if(!gl){return}if(image.width!=that.initWidth||image.height!=that.initHeight||image.rotation!=that.initRotation||image.mirror!=that.mirrorView){var view=that.view;that.stop();console.log("init canvas "+image.width+"*"+image.height+" rotation "+image.rotation);initCanvas(view,image.mirror,image.width,image.height,image.rotation,function(e){console.error("init canvas "+image.width+"*"+image.height+" rotation "+image.rotation+" failed. "+e)})}gl.bindBuffer(gl.ARRAY_BUFFER,texCoordBuffer);var xWidth=image.width+image.left+image.right;var xHeight=image.height+image.top+image.bottom;gl.bufferData(gl.ARRAY_BUFFER,new Float32Array([image.left/xWidth,image.bottom/xHeight,1-image.right/xWidth,image.bottom/xHeight,image.left/xWidth,1-image.top/xHeight,image.left/xWidth,1-image.top/xHeight,1-image.right/xWidth,image.bottom/xHeight,1-image.right/xWidth,1-image.top/xHeight]),gl.STATIC_DRAW);gl.enableVertexAttribArray(texCoordLocation);gl.vertexAttribPointer(texCoordLocation,2,gl.FLOAT,false,0,0);uploadYuv(xWidth,xHeight,image.yplane,image.uplane,image.vplane);updateCanvas(image.rotation,image.width,image.height);gl.drawArrays(gl.TRIANGLES,0,6);that.renderImageCount+=1};function uploadYuv(width,height,yplane,uplane,vplane){gl.activeTexture(gl.TEXTURE0);gl.bindTexture(gl.TEXTURE_2D,yTexture);gl.texImage2D(gl.TEXTURE_2D,0,gl.LUMINANCE,width,height,0,gl.LUMINANCE,gl.UNSIGNED_BYTE,yplane);var e=gl.getError();if(e!=gl.NO_ERROR){console.log("upload y plane ",width,height,yplane.byteLength," error",e)}gl.activeTexture(gl.TEXTURE1);gl.bindTexture(gl.TEXTURE_2D,uTexture);gl.texImage2D(gl.TEXTURE_2D,0,gl.LUMINANCE,width/2,height/2,0,gl.LUMINANCE,gl.UNSIGNED_BYTE,uplane);var e=gl.getError();if(e!=gl.NO_ERROR){console.log("upload u plane ",width,height,uplane.byteLength,"  error",e)}gl.activeTexture(gl.TEXTURE2);gl.bindTexture(gl.TEXTURE_2D,vTexture);gl.texImage2D(gl.TEXTURE_2D,0,gl.LUMINANCE,width/2,height/2,0,gl.LUMINANCE,gl.UNSIGNED_BYTE,vplane);var e=gl.getError();if(e!=gl.NO_ERROR){console.log("upload v plane ",width,height,vplane.byteLength,"  error",e)}}function deleteBuffer(buffer){if(buffer&&that.gl){that.gl.deleteBuffer(buffer)}}function deleteTexture(texture){if(texture&&that.gl){that.gl.deleteTexture(texture)}}function initCanvas(view,mirror,width,height,rotation,onFailure){that.clientWidth=view.clientWidth;that.clientHeight=view.clientHeight;that.view=view;that.mirrorView=mirror;that.canvasUpdated=false;that.container=document.createElement("div");that.container.style.width="100%";that.container.style.height="100%";that.container.style.display="flex";that.container.style.justifyContent="center";that.container.style.alignItems="center";that.view.appendChild(that.container);that.canvas=document.createElement("canvas");if(rotation==0||rotation==180){that.canvas.width=width;that.canvas.height=height}else{that.canvas.width=height;that.canvas.height=width}that.initWidth=width;that.initHeight=height;that.initRotation=rotation;if(that.mirrorView){that.canvas.style.transform="rotateY(180deg)"}that.container.appendChild(that.canvas);try{gl=that.canvas.getContext("webgl")||that.canvas.getContext("experimental-webgl")}catch(e){console.log(e)}if(!gl){gl=undefined;onFailure({error:"Browser not support! No WebGL detected."});return}gl.clearColor(0,0,0,1);gl.enable(gl.DEPTH_TEST);gl.depthFunc(gl.LEQUAL);gl.clear(gl.COLOR_BUFFER_BIT|gl.DEPTH_BUFFER_BIT);program=createProgramFromSources(gl,[AgoraRTC.vertexShaderSource,AgoraRTC.yuvShaderSource]);gl.useProgram(program);initTextures()}function initTextures(){positionLocation=gl.getAttribLocation(program,"a_position");texCoordLocation=gl.getAttribLocation(program,"a_texCoord");surfaceBuffer=gl.createBuffer();texCoordBuffer=gl.createBuffer();gl.activeTexture(gl.TEXTURE0);yTexture=gl.createTexture();gl.bindTexture(gl.TEXTURE_2D,yTexture);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_WRAP_S,gl.CLAMP_TO_EDGE);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_WRAP_T,gl.CLAMP_TO_EDGE);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_MIN_FILTER,gl.NEAREST);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_MAG_FILTER,gl.NEAREST);gl.activeTexture(gl.TEXTURE1);uTexture=gl.createTexture();gl.bindTexture(gl.TEXTURE_2D,uTexture);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_WRAP_S,gl.CLAMP_TO_EDGE);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_WRAP_T,gl.CLAMP_TO_EDGE);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_MIN_FILTER,gl.NEAREST);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_MAG_FILTER,gl.NEAREST);gl.activeTexture(gl.TEXTURE2);vTexture=gl.createTexture();gl.bindTexture(gl.TEXTURE_2D,vTexture);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_WRAP_S,gl.CLAMP_TO_EDGE);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_WRAP_T,gl.CLAMP_TO_EDGE);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_MIN_FILTER,gl.NEAREST);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_MAG_FILTER,gl.NEAREST);var y=gl.getUniformLocation(program,"Ytex");gl.uniform1i(y,0);var u=gl.getUniformLocation(program,"Utex");gl.uniform1i(u,1);var v=gl.getUniformLocation(program,"Vtex");gl.uniform1i(v,2)}function updateCanvas(rotation,width,height){if(that.canvasUpdated){return}that.canvas.style.width="100%";that.canvas.style.height="100%";try{if(rotation===0||rotation===180){if(that.clientWidth/that.clientHeight>width/height){that.canvas.style.width=that.clientHeight*width/height+"px"}else if(that.clientWidth/that.clientHeight<width/height){that.canvas.style.height=that.clientWidth*height/width+"px"}}else{if(that.clientHeight/that.clientWidth>width/height){that.canvas.style.height=that.clientWidth*width/height+"px"}else if(that.clientHeight/that.clientWidth<width/height){that.canvas.style.width=that.clientHeight*height/width+"px"}}}catch(e){console.log("updateCanvas 00001 gone "+that.canvas);console.log(that);console.error(e);return}gl.bindBuffer(gl.ARRAY_BUFFER,surfaceBuffer);gl.enableVertexAttribArray(positionLocation);gl.vertexAttribPointer(positionLocation,2,gl.FLOAT,false,0,0);var p1={x:0,y:0};var p2={x:width,y:0};var p3={x:width,y:height};var p4={x:0,y:height};var pp1=p1,pp2=p2,pp3=p3,pp4=p4;switch(rotation){case 0:break;case 90:pp1=p2;pp2=p3;pp3=p4;pp4=p1;break;case 180:pp1=p3;pp2=p4;pp3=p1;pp4=p2;break;case 270:pp1=p4;pp2=p1;pp3=p2;pp4=p3;break;default:}gl.bufferData(gl.ARRAY_BUFFER,new Float32Array([pp1.x,pp1.y,pp2.x,pp2.y,pp4.x,pp4.y,pp4.x,pp4.y,pp2.x,pp2.y,pp3.x,pp3.y]),gl.STATIC_DRAW);var resolutionLocation=gl.getUniformLocation(program,"u_resolution");gl.uniform2f(resolutionLocation,width,height);that.canvasUpdated=true}return that}var AgoraRTC=function(){"use strict";var AgoraRTC={};Object.defineProperties(AgoraRTC,{version:{get:function(){return"1.6.1"}},name:{get:function(){return"AgoraWebSDK"}}});return AgoraRTC}();AgoraRTC.url="wss://localhost.agora.io:8921/";AgoraRTC.macAgentInstallUrl="http://download.agora.io/AgoraWebAgent-1.6.1.pkg";AgoraRTC.winAgentInstallUrl="http://download.agora.io/AgoraWebAgentSetup-1.6.1.exe";AgoraRTC.enAgentInstallGuideUrl="http://download.agora.io/install-guide-en.html";AgoraRTC.cnAgentInstallGuideUrl="http://download.agora.io/install-guide-cn.html";AgoraRTC.vertexShaderSource="attribute vec2 a_position;"+"attribute vec2 a_texCoord;"+"uniform vec2 u_resolution;"+"varying vec2 v_texCoord;"+"void main() {"+"vec2 zeroToOne = a_position / u_resolution;"+"   vec2 zeroToTwo = zeroToOne * 2.0;"+"   vec2 clipSpace = zeroToTwo - 1.0;"+"   gl_Position = vec4(clipSpace * vec2(1, -1), 0, 1);"+"v_texCoord = a_texCoord;"+"}";AgoraRTC.yuvShaderSource="precision mediump float;"+"uniform sampler2D Ytex;"+"uniform sampler2D Utex,Vtex;"+"varying vec2 v_texCoord;"+"void main(void) {"+"  float nx,ny,r,g,b,y,u,v;"+"  mediump vec4 txl,ux,vx;"+"  nx=v_texCoord[0];"+"  ny=v_texCoord[1];"+"  y=texture2D(Ytex,vec2(nx,ny)).r;"+"  u=texture2D(Utex,vec2(nx,ny)).r;"+"  v=texture2D(Vtex,vec2(nx,ny)).r;"+"  y=1.1643*(y-0.0625);"+"  u=u-0.5;"+"  v=v-0.5;"+"  r=y+1.5958*v;"+"  g=y-0.39173*u-0.81290*v;"+"  b=y+2.017*u;"+"  gl_FragColor=vec4(r,g,b,1.0);"+"}";AgoraRTC.videoStreamSource='function onSuccess(){postMessage({type:"init",result:!0})}function onFailure(a){postMessage(a)}function sendMessage(a){"videoStat"!=a.command&&console.log(JSON.stringify(a)),that.stream&&that.stream.readyState==WebSocket.OPEN&&that.stream.send(JSON.stringify(a))}var that={};that.stream=void 0,that.init=function(a){return that.stream&&that.stream.readyState===WebSocket.OPEN?(console.warn("stream "+that.getId()+" has been initialized already"),void onSuccess()):(that.local=a.local,that.profile=a.videoProfile,that.stream=new WebSocket(a.url),that.stream.onopen=function(a){console.log((that.local?"local":"remote")+" Stream "),console.log(a),that.profile&&sendMessage({command:"setVideoProfile",profile:that.profile.profile,swapWidthAndHeight:that.profile.swapWidthAndHeight}),that.stream.binaryType="arraybuffer",onSuccess()},that.stream.onclose=function(a){console.log(that.local?"local":"remote"," Stream ",a),that.stream=void 0,onFailure({type:a.type,code:a.code,reason:a.reason})},that.stream.onerror=function(a){console.log(that.local?"local":"remote"," Stream ",a),onFailure({type:a.type,code:a.code,reason:a.reason})},void(that.stream.onmessage=function(a){"string"==typeof a.data?console.log(that.local?"local":"remote"," message from agent ",a.data):a.data instanceof ArrayBuffer?postMessage({type:"message",data:a.data}):a.data instanceof Blob&&console.warn("Blob image data is not supported")}))},that.close=function(){that.stream&&(that.stream.onmessage=void 0,that.stream.close()),that.stream=void 0},self.addEventListener("message",function(a){var b=a.data;switch(b.type){case"init":that.init(b);break;case"send":sendMessage(b.message);break;case"close":that.close()}},!1);';var AgoraCall=function(callback,parameters){if(callback){callback(parameters)}};var L={VideoProfiles:{Profile120P:"120P",Profile120P_3:"120P_3",Profile96P:"96P",Profile48P:"48P",Profile180P:"180P",Profile180P_3:"180P_3",Profile180P_4:"180P_4",Profile240P:"240P",Profile240P_3:"240P_3",Profile240P_4:"240P_4",Profile360P:"360P",Profile360P_3:"360P_3",Profile360P_4:"360P_4",Profile360P_6:"360P_6",Profile360P_7:"360P_7",Profile360P_8:"360P_8",Profile480P:"480P",Profile480P_3:"480P_3",Profile480P_4:"480P_4",Profile480P_6:"480P_6",Profile480P_8:"480P_8",Profile480P_9:"480P_9",Profile720P:"720P",Profile720P_3:"720P_3",Profile720P_5:"720P_5",Profile720P_6:"720P_6"},ErrorCode:{NO_ERROR:0,FAILED:1,INVALID_ARGUMENT:2,NOT_READY:3,NOT_SUPPORTED:4,REFUSED:5,BUFFER_TOO_SMALL:6,NOT_INITIALIZED:7,INVALID_VIEW:8,NO_PERMISSION:9,TIMEDOUT:10,CANCELED:11,TOO_OFTEN:12,BIND_SOCKET:13,NET_DOWN:14,NET_NOBUFS:15,INIT_VIDEO:16,JOIN_CHANNEL_REJECTED:17,LEAVE_CHANNEL_REJECTED:18,ALREADY_IN_USE:19,INVALID_APP_ID:101,INVALID_CHANNEL_NAME:102,CHANNEL_KEY_EXPIRED:109,INVALID_CHANNEL_KEY:110,CONNECTION_INTERRUPTED:111,CONNECTION_LOST:112,LOAD_MEDIA_ENGINE:1001,START_CALL:1002,START_CAMERA:1003,START_VIDEO_RENDER:1004,ADM_GENERAL_ERROR:1005,ADM_JAVA_RESOURCE:1006,ADM_SAMPLE_RATE:1007,ADM_INIT_PLAYOUT:1008,ADM_START_PLAYOUT:1009,ADM_STOP_PLAYOUT:1010,ADM_INIT_RECORDING:1011,ADM_START_RECORDING:1012,ADM_STOP_RECORDING:1013,ADM_RUNTIME_PLAYOUT_ERROR:1015,ADM_RUNTIME_RECORDING_ERROR:1017,ADM_RECORD_AUDIO_FAILED:1018,ADM_INIT_LOOPBACK:1022,ADM_START_LOOPBACK:1023,VDM_CAMERA_NOT_AUTHORIZED:1501}};AgoraRTC.EventDispatcher=function(spec){"use strict";var that={};spec.dispatcher={};spec.dispatcher.eventListeners={};that.addEventListener=function(eventType,listener){spec.dispatcher.eventListeners[eventType]=listener;if(!spec.dispatcher.eventListeners[eventType]){delete spec.dispatcher.eventListeners[eventType]}};that.on=that.addEventListener;that.dispatchEvent=function(event){if(spec.dispatcher.eventListeners.hasOwnProperty(event.type)){spec.dispatcher.eventListeners[event.type](event)}};return that};AgoraRTC.BasicEvent=function(spec){"use strict";var that={};that.type=spec.type;return that};AgoraRTC.StreamEvent=function(spec){"use strict";var that=AgoraRTC.BasicEvent(spec);that.stream=spec.stream;that.msg=spec.msg;return that};AgoraRTC.ClientEvent=function(spec){"use strict";var that=AgoraRTC.BasicEvent(spec);that.uid=spec.uid;that.attr=spec.attr;that.streams=spec.streams;return that};AgoraRTC.MediaEvent=function(spec){"use strict";var that=AgoraRTC.BasicEvent(spec);that.msg=spec.msg;return that};AgoraRTC.Signal=function(spec){var that=AgoraRTC.EventDispatcher(spec);that.connection=new WebSocket(spec.url);that.sendMessage=function(obj,onError){if(that.connection.readyState==WebSocket.OPEN){that.connection.send(JSON.stringify(obj))}else{console.log("connection to agent lost.");onError({error:"not connected"})}};that.close=function(){that.onEvent=onEvent;that.connection.onopen=undefined;that.connection.onclose=undefined;that.connection.onerror=undefined;that.connection.onmessage=undefined;that.connection.close()};that.connection.onopen=function(e){console.log(e);that.dispatchEvent(AgoraRTC.MediaEvent({type:"onopen",event:e}))};that.connection.onclose=function(e){console.log(e);AgoraCall(spec.onFailure,e)};that.connection.onerror=function(e){console.log(e);AgoraCall(spec.onFailure,e)};that.onEvent=onEvent;that.connection.onmessage=function(e){console.log(e);var m=JSON.parse(e.data);if(m.hasOwnProperty("command")){that.dispatchEvent(AgoraRTC.MediaEvent({type:m.command,msg:m}))}else if(m.hasOwnProperty("event")){that.onEvent(m)}};function onEvent(e){console.log(e)}return that};AgoraRTC.Stream=function(spec){var that=AgoraRTC.EventDispatcher(spec);that.stream=undefined;that.render=undefined;that.interval=undefined;that.lastRenderCount=0;that.profile=undefined;that.latency=0;var header=undefined;var yplane=undefined;var uplane=undefined;var vplane=undefined;that.init=function(onSuccess,onFailure){if(that.stream){console.warn("stream "+that.getId()+" has been initialized already");onSuccess();return}var videoStreamFileBlob=new Blob([AgoraRTC.videoStreamSource],{type:"application/javascript"});that.stream=new Worker(URL.createObjectURL(videoStreamFileBlob));that.oninit=function(e){console.log((spec.local?"local":"remote")+" Stream ");console.log(e);if(e.result===true){onSuccess()}else{onFailure()}};that.onclose=function(e){window.clearInterval(that.interval);console.log(spec.local?"local":"remote"," Stream ",e);that.stream=undefined};that.onerror=function(e){window.clearInterval(that.interval);console.log(spec.local?"local":"remote"," Stream ",e);onFailure(e)};that.onmessage=function(e){if(typeof e==="string"){console.log(spec.local?"local":"remote"," message from agent ",e)}else if(e instanceof ArrayBuffer){if(that.render){that.drawImage(e)}}else if(e instanceof Blob){console.warn("Blob image data is not supported")}};that.stream.addEventListener("message",onWorkerMessage,false);that.stream.postMessage({type:"init",local:spec.local,url:AgoraRTC.url,videoProfile:that.profile})};that.close=function(){window.clearInterval(that.interval);if(that.stream){that.stream.removeEventListener("message",onWorkerMessage);
that.stream.postMessage({type:"close"})}that.stream=undefined;that.stop()};that.play=function(div,onFailure){var view=document.getElementById(div);that.stop();that.render=AgoraRender();that.render.start(view,onFailure);if(spec.local){that.stream.postMessage({type:"send",message:{command:"preview",uid:spec.streamID,audio:spec.audio===false?false:true,video:spec.video===false?false:true,screen:spec.screen===true?true:false}})}else{that.stream.postMessage({type:"send",message:{command:"subscribe",uid:spec.streamID}})}if(that.interval){window.clearInterval(that.interval)}that.interval=window.setInterval(function(){if(that.render){var fps=that.render.renderImageCount-that.lastRenderCount;that.stream.postMessage({type:"send",message:{command:"videoStat",uid:spec.streamID,fps:fps,frameCount:that.lastRenderCount,latency:that.latency}});that.lastRenderCount=that.render.renderImageCount;that.latency=0}},1e3)};that.stop=function(){if(that.render){that.render.stop()}that.render=undefined;that.lastRenderCount=0};that.bindClient=function(client){that.client=client};that.enableAudio=function(callback){return that.client.enableAudio(that,callback)};that.disableAudio=function(callback){return that.client.disableAudio(that,callback)};that.enableVideo=function(callback){return that.client.enableVideo(that,callback)};that.disableVideo=function(callback){return that.client.disableVideo(that,callback)};that.getId=function(){return spec.streamID};that.drawImage=function(data){if(!header){header=data;if(header.byteLength!=20){console.error("invalid image header "+data.byteLength);resetImageData()}return}if(!yplane){yplane=data;if(yplane.byteLength===20){console.error("invalid image header "+data.byteLength+" "+yplane.byteLength);resetImageData()}return}if(!uplane){uplane=data;if(uplane.byteLength===20){console.error("invalid image header "+data.byteLength+" "+yplane.byteLength+" "+uplane.byteLength);resetImageData()}return}if(!vplane){vplane=data;if(yplane.byteLength!=uplane.byteLength*4||uplane.byteLength!=vplane.byteLength){console.error("invalid image header "+data.byteLength+" "+yplane.byteLength+" "+uplane.byteLength+" "+vplane.byteLength);resetImageData();return}var headerLength=20;var dv=new DataView(header);var format=dv.getUint8(0);var mirror=dv.getUint8(1);var width=dv.getUint16(2);var height=dv.getUint16(4);var left=dv.getUint16(6);var top=dv.getUint16(8);var right=dv.getUint16(10);var bottom=dv.getUint16(12);var rotation=dv.getUint16(14);var ts=dv.getUint32(16);var xWidth=width+left+right;var xHeight=height+top+bottom;var yLength=xWidth*xHeight;var yBegin=headerLength;var yEnd=yBegin+yLength;var uLength=yLength/4;var uBegin=yEnd;var uEnd=uBegin+uLength;var vLength=yLength/4;var vBegin=uEnd;var vEnd=vBegin+vLength;that.render.renderImage({mirror:mirror,width:width,height:height,left:left,top:top,right:right,bottom:bottom,rotation:rotation,yplane:new Uint8Array(yplane),uplane:new Uint8Array(uplane),vplane:new Uint8Array(vplane)});var now32=(Date.now()&4294967295)>>>0;var latency=now32-ts;if(latency>that.latency){that.latency=latency}resetImageData()}};that.setVideoProfile=function(profile){that.profile={};if(typeof profile!=="string"&&typeof profile!=="object"){return false}if(profile.hasOwnProperty("profile")){that.profile.profile=profile.profile}if(profile.hasOwnProperty("swapWidthAndHeight")){that.profile.swapWidthAndHeight=profile.swapWidthAndHeight!==false}else{that.profile.swapWidthAndHeight=false}if(typeof profile=="string"){that.profile.profile=profile}if(that.stream){that.stream.postMessage({type:"send",message:{command:"setVideoProfile",profile:that.profile.profile,swapWidthAndHeight:that.profile.swapWidthAndHeight}})}return true};function resetImageData(){header=undefined;yplane=undefined;uplane=undefined;vplane=undefined}function getReso(w,h){return{width:w,height:h}}var supportedVideoList={true:true,unspecified:true,"120p":getReso(160,120),"240p":getReso(320,240),"360p":getReso(640,360),"480p":getReso(640,480),"720p":getReso(1280,720),"1080p":getReso(1920,1080),"4k":getReso(3840,2160)};function onWorkerMessage(e){var data=e.data;switch(data.type){case"init":that.oninit(data);break;case"close":that.onclose(data);break;case"error":that.onerror(data);break;case"message":that.onmessage(data.data);break}}return that};AgoraRTC.createStream=function(spec){return AgoraRTC.Stream(spec)};AgoraRTC.Client=function(spec){"use strict";var that=AgoraRTC.EventDispatcher(spec);that.signal=undefined;that.localStream=undefined;that.remoteStreams={};that.signalOpen=false;function onSignalError(e){if(that.signalOpen){console.log("signal connection lost.");var message={reason:"LOST_CONNECTION_TO_AGENT",type:"error",eventType:e.type,code:e.code};AgoraCall(that.onFailure,message);that.dispatchEvent(message);return}if(e.type==="close"){return}var message={reason:"CLOSE_BEFORE_OPEN",type:"error",eventType:e.type,code:e.code,agentInstallUrl:getInstallUrl(),agentInstallGuideUrl:getInstallGuideUrl()};AgoraCall(that.onFailure,message);that.dispatchEvent(message)}that.init=function(appId,onSuccess,onFailure){spec.appId=appId;that.onFailure=onFailure;if(!that.signal){try{that.signal=AgoraRTC.Signal({url:AgoraRTC.url,onFailure:onSignalError})}catch(e){console.log("create signal connection failed"+e);AgoraCall(onFailure,e);return}that.signal.on("onopen",function(event){that.signalOpen=true;that.signal.onEvent=function(e){if(e.event==="error"){var name=getErrorCodeName(e.code);if(name){e.reason=name}}else if(e.event==="onUserMuteAudio"){e.event="peer-mute-audio"}else if(e.event==="onUserMuteVideo"){e.event="peer-mute-video"}that.dispatchEvent(AgoraRTC.MediaEvent({type:e.event,msg:e}))};that.getVersion(function(ev){if(ev.version=="1.6.0"||ev.version=="1.6.1"){initialize(onSuccess,onFailure);return}var message={reason:"INCOMPATIBLE_WEBAGENT",type:"error",version:AgoraRTC.version,agentVersion:ev.version,agentInstallUrl:getInstallUrl(),agentInstallGuideUrl:getInstallGuideUrl()};that.dispatchEvent(message)},function(ev){var message={reason:"INCOMPATIBLE_WEBAGENT",type:"error",version:AgoraRTC.version,agentVersion:ev.version,agentInstallUrl:getInstallUrl(),agentInstallGuideUrl:getInstallGuideUrl()};that.dispatchEvent(message)})})}if(that.signalOpen){initialize(onSuccess,onFailure)}function initialize(onSuccess,onFailure){that.signal.on("initialize",function(ev){var m=ev.msg;if(m.code!=true){that.signal.close();that.signal=undefined;onError(m.code);return}AgoraCall(onSuccess,m)});that.signal.sendMessage({command:"initialize",appId:spec.appId},function(err){AgoraCall(onFailure,err)})}function onError(code){var name=getErrorCodeName(code);if(name){AgoraCall(onFailure,{reason:name});return}AgoraCall(onFailure,{reason:"UNKNOWN_ERROR",code:m.code});return}function getErrorCodeName(code){for(var k in L.ErrorCode){if(L.ErrorCode.hasOwnProperty(k)){if(L.ErrorCode[k]===code){return String(k)}}}return null}that.signal.on("onError",function(ev){var m=ev.msg;onError(m.code)});console.log(that.signal)};that.renewChannelKey=function(channelKey,onSuccess,onFailure){that.signal.sendMessage({command:"renewChannelKey",channelKey:channelKey},function(e){AgoraCall(onFailure,e)});that.signal.on("renewChannelKey",function(ev){var m=ev.msg;if(m.code===true){AgoraCall(onSuccess,m);return}AgoraCall(onFailure,m)})};that.join=function(channelKey,channelName,uid,onSuccess,onFailure){if(channelName.length>64){onFailure(L.ErrorCode.INVALID_CHANNEL_NAME);return}that.signal.sendMessage({command:"joinChannel",channelKey:channelKey,channelName:channelName,uid:uid},function(e){AgoraCall(onFailure,{error:e})});that.signal.on("joinChannel",function(ev){that.signal.on("onAddVideoStream",onAddVideoStream);that.signal.on("onPeerLeave",onPeerLeave);var m=ev.msg;if(m.code==true){if(!m.uid){m.uid=0}AgoraCall(onSuccess,m.uid);return}if(m.code==L.ErrorCode.JOIN_CHANNEL_REJECTED){console.error("Command joinChannel has been rejected by agent. Is this user joined a channel already?");AgoraCall(onFailure,m.code);return}AgoraCall(onFailure,m.code)})};that.leave=function(onSuccess,onFailure){closeLocalRemoteStreams();that.signal.sendMessage({command:"leaveChannel"},function(e){console.log("leave channel failed",e)});that.signal.on("leaveChannel",function(ev){var m=ev.msg;if(m.code===true){AgoraCall(onSuccess,m);return}if(m.code==L.ErrorCode.LEAVE_CHANNEL_REJECTED){console.error("Command leaveChannel has been rejected by agent. Is this user not in a channel?");AgoraCall(onSuccess,m);return}AgoraCall(onFailure,{code:m.code})})};that.publish=function(stream,onSuccess,onFailure){that.localStream=stream;that.signal.sendMessage({command:"unmuteLocal"},function(e){AgoraCall(onFailure,{error:e})});that.signal.on("unmuteLocal",function(ev){var m=ev.msg;if(m.code!=true){AgoraCall(onFailure,{code:m.code});return}AgoraCall(onSuccess,m)});stream.bindClient(that)};that.unpublish=function(stream,onSuccess,onFailure){that.signal.sendMessage({command:"muteLocal"},function(e){AgoraCall(onFailure,{error:e})});that.signal.on("muteLocal",function(ev){var m=ev.msg;if(m.code!=true){AgoraCall(onFailure,{code:m.code});return}AgoraCall(onSuccess,m)})};that.subscribe=function(stream,onFailure){stream.init(function(){that.dispatchEvent(AgoraRTC.StreamEvent({type:"stream-subscribed",stream:stream}))},function(error){AgoraCall(onFailure,error)})};that.unsubscribe=function(streamID,onFailure){console.log("remote streams",that.remoteStreams);if(that.remoteStreams[streamID]==undefined){AgoraCall(onFailure,{error:"no such stream"});return}that.remoteStreams[streamID].close();delete that.remoteStreams[streamID]};that.enableAudio=function(stream,callback){that.signal.on("enableAudio",function(ev){var m=ev.msg;if(callback){AgoraCall(callback,m.code===true)}});that.signal.sendMessage({command:"enableAudio",streamID:stream.getId()},function(e){e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(callback,e)});return true};that.disableAudio=function(stream,callback){that.signal.on("disableAudio",function(ev){var m=ev.msg;if(callback){AgoraCall(callback,m.code===true)}});that.signal.sendMessage({command:"disableAudio",streamID:stream.getId()},function(e){e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(callback,e)});return true};that.enableVideo=function(stream,callback){that.signal.on("enableVideo",function(ev){var m=ev.msg;if(callback){AgoraCall(callback,m.code===true)}});that.signal.sendMessage({command:"enableVideo",streamID:stream.getId()},function(e){e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(callback,e)});return true};that.disableVideo=function(stream,callback){that.signal.on("disableVideo",function(ev){var m=ev.msg;if(callback){AgoraCall(callback,m.code===true)}});that.signal.sendMessage({command:"disableVideo",streamID:stream.getId()},function(e){e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(callback,e)});return true};that.getDevices=function(callback){that.signal.sendMessage({command:"getDevices"},function(e){e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(callback,e)});that.signal.on("getDevices",function(ev){var devices=ev.msg.devices;AgoraCall(callback,devices)})};that.selectDevice=function(device,onFailure){that.signal.sendMessage({command:"selectDevice",device:device},function(e){e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(onFailure,e)});that.signal.on("selectDevice",function(ev){console.log(ev)})};that.startRecording=function(recordingKey,onSuccess,onFailure){that.signal.sendMessage({command:"startRecording",recordingKey:recordingKey},function(e){e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(onFailure,e)});that.signal.on("startRecording",function(ev){var start=ev.msg;if(start.code===true){AgoraCall(onSuccess,start)}else{AgoraCall(onFailure,start)}})};that.stopRecording=function(recordingKey,onSuccess,onFailure){that.signal.sendMessage({command:"stopRecording",recordingKey:recordingKey},function(e){e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(onFailure,e)});that.signal.on("stopRecording",function(ev){var stop=ev.msg;if(stop.code===true){AgoraCall(onSuccess,stop)}else{AgoraCall(onFailure,stop)}})};that.queryRecordingStatus=function(onStatus){that.signal.sendMessage({command:"queryRecordingStatus"},function(e){e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(onStatus,e)});that.signal.on("queryRecordingStatus",function(ev){onStatus(ev.msg)})};that.close=function(){closeLocalRemoteStreams();if(that.signal){that.signal.close();that.signal=undefined}};that.getVersion=function(onSuccess,onFailure){that.signal.sendMessage({command:"getVersion"});that.signal.on("getVersion",function(ev){var m=ev.msg;if(m.code===true){onSuccess(m)}else{onFailure(m)}})};that.setParameters=function(parameters){that.signal.sendMessage({command:"setParameters",parameters:JSON.stringify(parameters)})};that.setEncryptionMode=function(encryptionMode){that.signal.sendMessage({command:"setEncryptionMode",mode:encryptionMode})};that.setEncryptionSecret=function(secret){that.signal.sendMessage({command:"setEncryptionSecret",secret:secret})};that.getWindows=function(callback){that.signal.sendMessage({command:"getWindows"},function(e){e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(callback,e)});that.signal.on("getWindows",function(ev){var windows=ev.msg.windows;AgoraCall(callback,windows)})};that.startScreenSharing=function(window,onSuccess,onFailure){that.signal.sendMessage({command:"startScreenSharing",window:window},function(e){e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(onFailure,e)});that.signal.on("startScreenSharing",function(e){AgoraCall(onSuccess,e)})};that.setScreenSharingWindow=function(window,onSuccess,onFailure){that.signal.sendMessage({command:"setScreenSharingWindow",window:window},function(e){e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(onFailure,e)});that.signal.on("setScreenSharingWindow",function(e){AgoraCall(onSuccess,e);console.log(e)})};that.stopScreenSharing=function(onSuccess,onFailure){that.signal.sendMessage({command:"stopScreenSharing"},function(e){e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(onFailure,e)});that.signal.on("stopScreenSharing",function(e){AgoraCall(onSuccess,e);console.log(e)})};function onPeerLeave(ev){var m=ev.msg;if(that.remoteStreams.hasOwnProperty(m.uid)){that.remoteStreams[m.uid].close();delete that.remoteStreams[m.uid];console.log("remote streams after peer leave ",that.remoteStreams)}that.dispatchEvent(AgoraRTC.ClientEvent({type:"peer-leave",uid:m.uid}))}function onAddVideoStream(ev){var m=ev.msg;if(!that.remoteStreams.hasOwnProperty(m.uid)){var stream=AgoraRTC.Stream({streamID:m.uid,local:false,audio:m.audio,video:m.video,screen:m.screen});stream.bindClient(that);that.remoteStreams[m.uid]=stream}that.dispatchEvent(AgoraRTC.StreamEvent({type:"stream-added",stream:that.remoteStreams[m.uid]}));console.log("remote streams",that.remoteStreams)}function getInstallUrl(){if(navigator.appVersion.indexOf("Mac")!=-1){return AgoraRTC.macAgentInstallUrl}return AgoraRTC.winAgentInstallUrl}function getInstallGuideUrl(){var userLang=navigator.language||navigator.userLanguage;if(userLang.indexOf("zh")!=-1){return AgoraRTC.cnAgentInstallGuideUrl}return AgoraRTC.enAgentInstallGuideUrl}function closeLocalRemoteStreams(){if(that.localStream){that.localStream.close()}that.localStream=undefined;for(var streamId in that.remoteStreams){if(that.remoteStreams.hasOwnProperty(streamId)){that.remoteStreams[streamId].close();delete that.remoteStreams[streamId]}}}return that};AgoraRTC.createRtcClient=function(){return AgoraRTC.Client({})};
//# sourceMappingURL=dist/AgoraRtcAgentSDK.min.js.map
"use strict";(function(root,factory){if(typeof define==="function"&&define.amd){define([],factory)}else{var lib=factory.call(root);Object.keys(lib).forEach(function(key){root[key]=lib[key]})}})(this,function(){var topWindow=this;function error(msg){if(topWindow.console){if(topWindow.console.error){topWindow.console.error(msg)}else if(topWindow.console.log){topWindow.console.log(msg)}}}function isInIFrame(w){w=w||topWindow;return w!==w.top}function makeFailHTML(msg){return""+'<table style="background-color: #8CE; width: 100%; height: 100%;"><tr>'+'<td align="center">'+'<div style="display: table-cell; vertical-align: middle;">'+'<div style="">'+msg+"</div>"+"</div>"+"</td></tr></table>"}var GET_A_WEBGL_BROWSER=""+"This page requires a browser that supports WebGL.<br/>"+'<a href="http://get.webgl.org">Click here to upgrade your browser.</a>';var OTHER_PROBLEM=""+"It doesn't appear your computer can support WebGL.<br/>"+'<a href="http://get.webgl.org/troubleshooting/">Click here for more information.</a>';function create3DContext(canvas,opt_attribs){var names=["webgl","experimental-webgl"];var context=null;for(var ii=0;ii<names.length;++ii){try{context=canvas.getContext(names[ii],opt_attribs)}catch(e){}if(context){break}}return context}function setupWebGL(canvas,opt_attribs){function showLink(str){var container=canvas.parentNode;if(container){container.innerHTML=makeFailHTML(str)}}if(!topWindow.WebGLRenderingContext){showLink(GET_A_WEBGL_BROWSER);return null}var context=create3DContext(canvas,opt_attribs);if(!context){showLink(OTHER_PROBLEM)}return context}function updateCSSIfInIFrame(){if(isInIFrame()){document.body.className="iframe"}}function getWebGLContext(canvas,opt_attribs,opt_options){var options=opt_options||{};if(isInIFrame()){updateCSSIfInIFrame();if(!options.dontResize&&options.resize!==false){var width=canvas.clientWidth;var height=canvas.clientHeight;canvas.width=width;canvas.height=height}}else if(!options.noTitle&&options.title!==false){var title=document.title;var h1=document.createElement("h1");h1.innerText=title;document.body.insertBefore(h1,document.body.children[0])}var gl=setupWebGL(canvas,opt_attribs);return gl}function loadShader(gl,shaderSource,shaderType,opt_errorCallback){var errFn=opt_errorCallback||error;var shader=gl.createShader(shaderType);gl.shaderSource(shader,shaderSource);gl.compileShader(shader);var compiled=gl.getShaderParameter(shader,gl.COMPILE_STATUS);if(!compiled){var lastError=gl.getShaderInfoLog(shader);errFn("*** Error compiling shader '"+shader+"':"+lastError);gl.deleteShader(shader);return null}return shader}function createProgram(gl,shaders,opt_attribs,opt_locations,opt_errorCallback){var errFn=opt_errorCallback||error;var program=gl.createProgram();shaders.forEach(function(shader){gl.attachShader(program,shader)});if(opt_attribs){obj_attrib.forEach(function(attrib,ndx){gl.bindAttribLocation(program,opt_locations?opt_locations[ndx]:ndx,attrib)})}gl.linkProgram(program);var linked=gl.getProgramParameter(program,gl.LINK_STATUS);if(!linked){var lastError=gl.getProgramInfoLog(program);errFn("Error in program linking:"+lastError);gl.deleteProgram(program);return null}return program}function createShaderFromScript(gl,scriptId,opt_shaderType,opt_errorCallback){var shaderSource="";var shaderType;var shaderScript=document.getElementById(scriptId);if(!shaderScript){throw"*** Error: unknown script element"+scriptId}shaderSource=shaderScript.text;if(!opt_shaderType){if(shaderScript.type==="x-shader/x-vertex"){shaderType=gl.VERTEX_SHADER}else if(shaderScript.type==="x-shader/x-fragment"){shaderType=gl.FRAGMENT_SHADER}else if(shaderType!==gl.VERTEX_SHADER&&shaderType!==gl.FRAGMENT_SHADER){throw"*** Error: unknown shader type"}}return loadShader(gl,shaderSource,opt_shaderType?opt_shaderType:shaderType,opt_errorCallback)}var defaultShaderType=["VERTEX_SHADER","FRAGMENT_SHADER"];function createProgramFromScripts(gl,shaderScriptIds,opt_attribs,opt_locations,opt_errorCallback){var shaders=[];for(var ii=0;ii<shaderScriptIds.length;++ii){shaders.push(createShaderFromScript(gl,shaderScriptIds[ii],gl[defaultShaderType[ii]],opt_errorCallback))}return createProgram(gl,shaders,opt_attribs,opt_locations,opt_errorCallback)}function createProgramFromSources(gl,shaderSources,opt_attribs,opt_locations,opt_errorCallback){var shaders=[];for(var ii=0;ii<shaderSources.length;++ii){shaders.push(loadShader(gl,shaderSources[ii],gl[defaultShaderType[ii]],opt_errorCallback))}return createProgram(gl,shaders,opt_attribs,opt_locations,opt_errorCallback)}function getBindPointForSamplerType(gl,type){if(type===gl.SAMPLER_2D)return gl.TEXTURE_2D;if(type===gl.SAMPLER_CUBE)return gl.TEXTURE_CUBE_MAP}function createUniformSetters(gl,program){var textureUnit=0;function createUniformSetter(program,uniformInfo){var location=gl.getUniformLocation(program,uniformInfo.name);var type=uniformInfo.type;var isArray=uniformInfo.size>1&&uniformInfo.name.substr(-3)==="[0]";if(type===gl.FLOAT&&isArray){return function(v){gl.uniform1fv(location,v)}}if(type===gl.FLOAT){return function(v){gl.uniform1f(location,v)}}if(type===gl.FLOAT_VEC2){return function(v){gl.uniform2fv(location,v)}}if(type===gl.FLOAT_VEC3){return function(v){gl.uniform3fv(location,v)}}if(type===gl.FLOAT_VEC4){return function(v){gl.uniform4fv(location,v)}}if(type===gl.INT&&isArray){return function(v){gl.uniform1iv(location,v)}}if(type===gl.INT){return function(v){gl.uniform1i(location,v)}}if(type===gl.INT_VEC2){return function(v){gl.uniform2iv(location,v)}}if(type===gl.INT_VEC3){return function(v){gl.uniform3iv(location,v)}}if(type===gl.INT_VEC4){return function(v){gl.uniform4iv(location,v)}}if(type===gl.BOOL){return function(v){gl.uniform1iv(location,v)}}if(type===gl.BOOL_VEC2){return function(v){gl.uniform2iv(location,v)}}if(type===gl.BOOL_VEC3){return function(v){gl.uniform3iv(location,v)}}if(type===gl.BOOL_VEC4){return function(v){gl.uniform4iv(location,v)}}if(type===gl.FLOAT_MAT2){return function(v){gl.uniformMatrix2fv(location,false,v)}}if(type===gl.FLOAT_MAT3){return function(v){gl.uniformMatrix3fv(location,false,v)}}if(type===gl.FLOAT_MAT4){return function(v){gl.uniformMatrix4fv(location,false,v)}}if((type===gl.SAMPLER_2D||type===gl.SAMPLER_CUBE)&&isArray){var units=[];for(var ii=0;ii<info.size;++ii){units.push(textureUnit++)}return function(bindPoint,units){return function(textures){gl.uniform1iv(location,units);textures.forEach(function(texture,index){gl.activeTexture(gl.TEXTURE0+units[index]);gl.bindTexture(bindPoint,texture)})}}(getBindPointForSamplerType(gl,type),units)}if(type===gl.SAMPLER_2D||type===gl.SAMPLER_CUBE){return function(bindPoint,unit){return function(texture){gl.uniform1i(location,unit);gl.activeTexture(gl.TEXTURE0+unit);gl.bindTexture(bindPoint,texture)}}(getBindPointForSamplerType(gl,type),textureUnit++)}throw"unknown type: 0x"+type.toString(16)}var uniformSetters={};var numUniforms=gl.getProgramParameter(program,gl.ACTIVE_UNIFORMS);for(var ii=0;ii<numUniforms;++ii){var uniformInfo=gl.getActiveUniform(program,ii);if(!uniformInfo){break}var name=uniformInfo.name;if(name.substr(-3)==="[0]"){name=name.substr(0,name.length-3)}var setter=createUniformSetter(program,uniformInfo);uniformSetters[name]=setter}return uniformSetters}function setUniforms(setters,values){Object.keys(values).forEach(function(name){var setter=setters[name];if(setter){setter(values[name])}})}function createAttributeSetters(gl,program){var attribSetters={};function createAttribSetter(index){return function(b){gl.bindBuffer(gl.ARRAY_BUFFER,b.buffer);gl.enableVertexAttribArray(index);gl.vertexAttribPointer(index,b.numComponents||b.size,b.type||gl.FLOAT,b.normalize||false,b.stride||0,b.offset||0)}}var numAttribs=gl.getProgramParameter(program,gl.ACTIVE_ATTRIBUTES);for(var ii=0;ii<numAttribs;++ii){var attribInfo=gl.getActiveAttrib(program,ii);if(!attribInfo){break}var index=gl.getAttribLocation(program,attribInfo.name);attribSetters[attribInfo.name]=createAttribSetter(index)}return attribSetters}function setAttributes(setters,buffers){Object.keys(buffers).forEach(function(name){var setter=setters[name];if(setter){setter(buffers[name])}})}function createProgramInfo(gl,shaderSources,opt_attribs,opt_locations,opt_errorCallback){shaderSources=shaderSources.map(function(source){var script=document.getElementById(source);return script?script.text:source});var program=createProgramFromSources(gl,shaderSources,opt_attribs,opt_locations,opt_errorCallback);if(!program){return null}var uniformSetters=createUniformSetters(gl,program);var attribSetters=createAttributeSetters(gl,program);return{program:program,uniformSetters:uniformSetters,attribSetters:attribSetters}}function setBuffersAndAttributes(gl,setters,buffers){setAttributes(setters,buffers.attribs);if(buffers.indices){gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER,buffers.indices)}}var browserPrefixes=["","MOZ_","OP_","WEBKIT_"];function getExtensionWithKnownPrefixes(gl,name){for(var ii=0;ii<browserPrefixes.length;++ii){var prefixedName=browserPrefixes[ii]+name;var ext=gl.getExtension(prefixedName);if(ext){return ext}}}function resizeCanvasToDisplaySize(canvas,multiplier){multiplier=multiplier||1;var width=canvas.clientWidth*multiplier;var height=canvas.clientHeight*multiplier;if(canvas.width!==width||canvas.height!==height){canvas.width=width;canvas.height=height;return true}return false}function getIFrameForWindow(window){if(!isInIFrame(window)){return}var iframes=window.parent.document.getElementsByTagName("iframe");for(var ii=0;ii<iframes.length;++ii){var iframe=iframes[ii];if(iframe.contentDocument===window.document){return iframe}}}function isFrameVisible(window){try{var iframe=getIFrameForWindow(window);if(!iframe){return true}var bounds=iframe.getBoundingClientRect();var isVisible=bounds.top<window.parent.innerHeight&&bounds.bottom>=0&&bounds.left<window.parent.innerWidth&&bounds.right>=0;return isVisible&&isFrameVisible(window.parent)}catch(e){return true}}function isOnScreen(element){var isVisible=true;if(element){var bounds=element.getBoundingClientRect();isVisible=bounds.top<topWindow.innerHeight&&bounds.bottom>=0}return isVisible&&isFrameVisible(topWindow)}function augmentTypedArray(typedArray,numComponents){var cursor=0;typedArray.push=function(){for(var ii=0;ii<arguments.length;++ii){var value=arguments[ii];if(value instanceof Array||value.buffer&&value.buffer instanceof ArrayBuffer){for(var jj=0;jj<value.length;++jj){typedArray[cursor++]=value[jj]}}else{typedArray[cursor++]=value}}};typedArray.reset=function(opt_index){cursor=opt_index||0};typedArray.numComponents=numComponents;Object.defineProperty(typedArray,"numElements",{get:function(){return this.length/this.numComponents|0}});return typedArray}function createAugmentedTypedArray(numComponents,numElements,opt_type){var Type=opt_type||Float32Array;return augmentTypedArray(new Type(numComponents*numElements),numComponents)}function createBufferFromTypedArray(gl,array,type,drawType){type=type||gl.ARRAY_BUFFER;var buffer=gl.createBuffer();gl.bindBuffer(type,buffer);gl.bufferData(type,array,drawType||gl.STATIC_DRAW);return buffer}function allButIndices(name){return name!=="indices"}function createMapping(obj){var mapping={};Object.keys(obj).filter(allButIndices).forEach(function(key){mapping["a_"+key]=key});return mapping}function getGLTypeForTypedArray(gl,typedArray){if(typedArray instanceof Int8Array){return gl.BYTE}if(typedArray instanceof Uint8Array){return gl.UNSIGNED_BYTE}if(typedArray instanceof Int16Array){return gl.SHORT}if(typedArray instanceof Uint16Array){return gl.UNSIGNED_SHORT}if(typedArray instanceof Int32Array){return gl.INT}if(typedArray instanceof Uint32Array){return gl.UNSIGNED_INT}if(typedArray instanceof Float32Array){return gl.FLOAT}throw"unsupported typed array type"}function getNormalizationForTypedArray(typedArray){if(typedArray instanceof Int8Array){return true}if(typedArray instanceof Uint8Array){return true}return false}function isArrayBuffer(a){return a.buffer&&a.buffer instanceof ArrayBuffer}function guessNumComponentsFromName(name,length){var numComponents;if(name.indexOf("coord")>=0){numComponents=2}else if(name.indexOf("color")>=0){numComponents=4}else{numComponents=3}if(length%numComponents>0){throw"can not guess numComponents. You should specify it."}return numComponents}function makeTypedArray(array,name){if(isArrayBuffer(array)){return array}if(Array.isArray(array)){array={data:array}}if(!array.numComponents){array.numComponents=guessNumComponentsFromName(name,array.length)}var type=array.type;if(!type){if(name==="indices"){type=Uint16Array}}var typedArray=createAugmentedTypedArray(array.numComponents,array.data.length/array.numComponents|0,type);typedArray.push(array.data);return typedArray}function createAttribsFromArrays(gl,arrays,opt_mapping){var mapping=opt_mapping||createMapping(arrays);var attribs={};Object.keys(mapping).forEach(function(attribName){var bufferName=mapping[attribName];var array=makeTypedArray(arrays[bufferName],bufferName);attribs[attribName]={buffer:createBufferFromTypedArray(gl,array),numComponents:array.numComponents||guessNumComponentsFromName(bufferName),type:getGLTypeForTypedArray(gl,array),normalize:getNormalizationForTypedArray(array)}});return attribs}function getNumElementsFromNonIndexedArrays(arrays){var key=Object.keys(arrays)[0];var array=arrays[key];if(isArrayBuffer(array)){return array.numElements}else{return array.data.length/array.numComponents}}function createBufferInfoFromArrays(gl,arrays,opt_mapping){var bufferInfo={attribs:createAttribsFromArrays(gl,arrays,opt_mapping)};var indices=arrays.indices;if(indices){indices=makeTypedArray(indices,"indices");bufferInfo.indices=createBufferFromTypedArray(gl,indices,gl.ELEMENT_ARRAY_BUFFER);bufferInfo.numElements=indices.length}else{bufferInfo.numElements=getNumElementsFromNonIndexedArrays(arrays)}return bufferInfo}function createBuffersFromArrays(gl,arrays){var buffers={};Object.keys(arrays).forEach(function(key){var type=key==="indices"?gl.ELEMENT_ARRAY_BUFFER:gl.ARRAY_BUFFER;var array=makeTypedArray(arrays[key],name);buffers[key]=createBufferFromTypedArray(gl,array,type)});if(arrays.indices){buffers.numElements=arrays.indices.length}else if(arrays.position){buffers.numElements=arrays.position.length/3}return buffers}function drawBufferInfo(gl,type,bufferInfo,count,offset){var indices=bufferInfo.indices;var numElements=count===undefined?bufferInfo.numElements:count;offset=offset===undefined?offset:0;if(indices){gl.drawElements(type,numElements,gl.UNSIGNED_SHORT,offset)}else{gl.drawArrays(type,offset,numElements)}}function drawObjectList(gl,objectsToDraw){var lastUsedProgramInfo=null;var lastUsedBufferInfo=null;objectsToDraw.forEach(function(object){var programInfo=object.programInfo;var bufferInfo=object.bufferInfo;var bindBuffers=false;if(programInfo!==lastUsedProgramInfo){lastUsedProgramInfo=programInfo;gl.useProgram(programInfo.program);bindBuffers=true}if(bindBuffers||bufferInfo!==lastUsedBufferInfo){lastUsedBufferInfo=bufferInfo;setBuffersAndAttributes(gl,programInfo.attribSetters,bufferInfo)}setUniforms(programInfo.uniformSetters,object.uniforms);drawBufferInfo(gl,gl.TRIANGLES,bufferInfo)})}return{createAugmentedTypedArray:createAugmentedTypedArray,createAttribsFromArrays:createAttribsFromArrays,createBuffersFromArrays:createBuffersFromArrays,createBufferInfoFromArrays:createBufferInfoFromArrays,createAttributeSetters:createAttributeSetters,createProgram:createProgram,createProgramFromScripts:createProgramFromScripts,createProgramFromSources:createProgramFromSources,createProgramInfo:createProgramInfo,createUniformSetters:createUniformSetters,drawBufferInfo:drawBufferInfo,drawObjectList:drawObjectList,getWebGLContext:getWebGLContext,updateCSSIfInIFrame:updateCSSIfInIFrame,getExtensionWithKnownPrefixes:getExtensionWithKnownPrefixes,resizeCanvasToDisplaySize:resizeCanvasToDisplaySize,setAttributes:setAttributes,setBuffersAndAttributes:setBuffersAndAttributes,setUniforms:setUniforms,setupWebGL:setupWebGL}});function AgoraRender(){var gl=undefined;var program=undefined;var positionLocation=undefined;var texCoordLocation=undefined;var yTexture=undefined;var uTexture=undefined;var vTexture=undefined;var texCoordBuffer=undefined;var surfaceBuffer=undefined;var that={view:undefined,mirrorView:false,container:undefined,canvas:undefined,renderImageCount:0,initWidth:0,initHeight:0,initRotation:0,canvasUpdated:false,clientWidth:0,clientHeight:0};that.start=function(view,onFailure){initCanvas(view,that.mirrorView,view.clientWidth,view.clientHeight,that.initRotation,onFailure)};that.stop=function(){gl=undefined;program=undefined;positionLocation=undefined;texCoordLocation=undefined;deleteTexture(yTexture);deleteTexture(uTexture);deleteTexture(vTexture);yTexture=undefined;uTexture=undefined;vTexture=undefined;deleteBuffer(texCoordBuffer);deleteBuffer(surfaceBuffer);texCoordBuffer=undefined;surfaceBuffer=undefined;if(that.container){that.container.removeChild(that.canvas)}if(that.view){that.view.removeChild(that.container)}that.canvas=undefined;that.container=undefined;that.view=undefined;that.mirrorView=false};that.renderImage=function(image){if(!gl){return}if(image.width!=that.initWidth||image.height!=that.initHeight||image.rotation!=that.initRotation||image.mirror!=that.mirrorView){var view=that.view;that.stop();console.log("init canvas "+image.width+"*"+image.height+" rotation "+image.rotation);initCanvas(view,image.mirror,image.width,image.height,image.rotation,function(e){console.error("init canvas "+image.width+"*"+image.height+" rotation "+image.rotation+" failed. "+e)})}gl.bindBuffer(gl.ARRAY_BUFFER,texCoordBuffer);var xWidth=image.width+image.left+image.right;var xHeight=image.height+image.top+image.bottom;gl.bufferData(gl.ARRAY_BUFFER,new Float32Array([image.left/xWidth,image.bottom/xHeight,1-image.right/xWidth,image.bottom/xHeight,image.left/xWidth,1-image.top/xHeight,image.left/xWidth,1-image.top/xHeight,1-image.right/xWidth,image.bottom/xHeight,1-image.right/xWidth,1-image.top/xHeight]),gl.STATIC_DRAW);gl.enableVertexAttribArray(texCoordLocation);gl.vertexAttribPointer(texCoordLocation,2,gl.FLOAT,false,0,0);uploadYuv(xWidth,xHeight,image.yplane,image.uplane,image.vplane);updateCanvas(image.rotation,image.width,image.height);gl.drawArrays(gl.TRIANGLES,0,6);that.renderImageCount+=1};function uploadYuv(width,height,yplane,uplane,vplane){gl.activeTexture(gl.TEXTURE0);gl.bindTexture(gl.TEXTURE_2D,yTexture);gl.texImage2D(gl.TEXTURE_2D,0,gl.LUMINANCE,width,height,0,gl.LUMINANCE,gl.UNSIGNED_BYTE,yplane);var e=gl.getError();if(e!=gl.NO_ERROR){console.log("upload y plane ",width,height,yplane.byteLength," error",e)}gl.activeTexture(gl.TEXTURE1);gl.bindTexture(gl.TEXTURE_2D,uTexture);gl.texImage2D(gl.TEXTURE_2D,0,gl.LUMINANCE,width/2,height/2,0,gl.LUMINANCE,gl.UNSIGNED_BYTE,uplane);var e=gl.getError();if(e!=gl.NO_ERROR){console.log("upload u plane ",width,height,uplane.byteLength,"  error",e)}gl.activeTexture(gl.TEXTURE2);gl.bindTexture(gl.TEXTURE_2D,vTexture);gl.texImage2D(gl.TEXTURE_2D,0,gl.LUMINANCE,width/2,height/2,0,gl.LUMINANCE,gl.UNSIGNED_BYTE,vplane);var e=gl.getError();if(e!=gl.NO_ERROR){console.log("upload v plane ",width,height,vplane.byteLength,"  error",e)}}function deleteBuffer(buffer){if(buffer&&that.gl){that.gl.deleteBuffer(buffer)}}function deleteTexture(texture){if(texture&&that.gl){that.gl.deleteTexture(texture)}}function initCanvas(view,mirror,width,height,rotation,onFailure){that.clientWidth=view.clientWidth;that.clientHeight=view.clientHeight;that.view=view;that.mirrorView=mirror;that.canvasUpdated=false;that.container=document.createElement("div");that.container.style.width="100%";that.container.style.height="100%";that.container.style.display="flex";that.container.style.justifyContent="center";that.container.style.alignItems="center";that.view.appendChild(that.container);that.canvas=document.createElement("canvas");if(rotation==0||rotation==180){that.canvas.width=width;that.canvas.height=height}else{that.canvas.width=height;that.canvas.height=width}that.initWidth=width;that.initHeight=height;that.initRotation=rotation;if(that.mirrorView){that.canvas.style.transform="rotateY(180deg)"}that.container.appendChild(that.canvas);try{gl=that.canvas.getContext("webgl")||that.canvas.getContext("experimental-webgl")}catch(e){console.log(e)}if(!gl){gl=undefined;onFailure({error:"Browser not support! No WebGL detected."});return}gl.clearColor(0,0,0,1);gl.enable(gl.DEPTH_TEST);gl.depthFunc(gl.LEQUAL);gl.clear(gl.COLOR_BUFFER_BIT|gl.DEPTH_BUFFER_BIT);program=createProgramFromSources(gl,[AgoraRTC.vertexShaderSource,AgoraRTC.yuvShaderSource]);gl.useProgram(program);initTextures()}function initTextures(){positionLocation=gl.getAttribLocation(program,"a_position");texCoordLocation=gl.getAttribLocation(program,"a_texCoord");surfaceBuffer=gl.createBuffer();texCoordBuffer=gl.createBuffer();gl.activeTexture(gl.TEXTURE0);yTexture=gl.createTexture();gl.bindTexture(gl.TEXTURE_2D,yTexture);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_WRAP_S,gl.CLAMP_TO_EDGE);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_WRAP_T,gl.CLAMP_TO_EDGE);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_MIN_FILTER,gl.NEAREST);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_MAG_FILTER,gl.NEAREST);gl.activeTexture(gl.TEXTURE1);uTexture=gl.createTexture();gl.bindTexture(gl.TEXTURE_2D,uTexture);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_WRAP_S,gl.CLAMP_TO_EDGE);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_WRAP_T,gl.CLAMP_TO_EDGE);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_MIN_FILTER,gl.NEAREST);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_MAG_FILTER,gl.NEAREST);gl.activeTexture(gl.TEXTURE2);vTexture=gl.createTexture();gl.bindTexture(gl.TEXTURE_2D,vTexture);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_WRAP_S,gl.CLAMP_TO_EDGE);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_WRAP_T,gl.CLAMP_TO_EDGE);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_MIN_FILTER,gl.NEAREST);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_MAG_FILTER,gl.NEAREST);var y=gl.getUniformLocation(program,"Ytex");gl.uniform1i(y,0);var u=gl.getUniformLocation(program,"Utex");gl.uniform1i(u,1);var v=gl.getUniformLocation(program,"Vtex");gl.uniform1i(v,2)}function updateCanvas(rotation,width,height){if(that.canvasUpdated){return}that.canvas.style.width="100%";that.canvas.style.height="100%";try{if(rotation===0||rotation===180){if(that.clientWidth/that.clientHeight>width/height){that.canvas.style.width=that.clientHeight*width/height+"px"}else if(that.clientWidth/that.clientHeight<width/height){that.canvas.style.height=that.clientWidth*height/width+"px"}}else{if(that.clientHeight/that.clientWidth>width/height){that.canvas.style.height=that.clientWidth*width/height+"px"}else if(that.clientHeight/that.clientWidth<width/height){that.canvas.style.width=that.clientHeight*height/width+"px"}}}catch(e){console.log("updateCanvas 00001 gone "+that.canvas);console.log(that);console.error(e);return}gl.bindBuffer(gl.ARRAY_BUFFER,surfaceBuffer);gl.enableVertexAttribArray(positionLocation);gl.vertexAttribPointer(positionLocation,2,gl.FLOAT,false,0,0);var p1={x:0,y:0};var p2={x:width,y:0};var p3={x:width,y:height};var p4={x:0,y:height};var pp1=p1,pp2=p2,pp3=p3,pp4=p4;switch(rotation){case 0:break;case 90:pp1=p2;pp2=p3;pp3=p4;pp4=p1;break;case 180:pp1=p3;pp2=p4;pp3=p1;pp4=p2;break;case 270:pp1=p4;pp2=p1;pp3=p2;pp4=p3;break;default:}gl.bufferData(gl.ARRAY_BUFFER,new Float32Array([pp1.x,pp1.y,pp2.x,pp2.y,pp4.x,pp4.y,pp4.x,pp4.y,pp2.x,pp2.y,pp3.x,pp3.y]),gl.STATIC_DRAW);var resolutionLocation=gl.getUniformLocation(program,"u_resolution");gl.uniform2f(resolutionLocation,width,height);that.canvasUpdated=true}return that}var AgoraRTC=function(){"use strict";var AgoraRTC={};Object.defineProperties(AgoraRTC,{version:{get:function(){return"1.6.1"}},name:{get:function(){return"AgoraWebSDK"}}});return AgoraRTC}();AgoraRTC.url="wss://localhost.agora.io:8921/";AgoraRTC.macAgentInstallUrl="http://download.agora.io/AgoraWebAgent-1.6.1.pkg";AgoraRTC.winAgentInstallUrl="http://download.agora.io/AgoraWebAgentSetup-1.6.1.exe";AgoraRTC.enAgentInstallGuideUrl="http://download.agora.io/install-guide-en.html";AgoraRTC.cnAgentInstallGuideUrl="http://download.agora.io/install-guide-cn.html";AgoraRTC.vertexShaderSource="attribute vec2 a_position;"+"attribute vec2 a_texCoord;"+"uniform vec2 u_resolution;"+"varying vec2 v_texCoord;"+"void main() {"+"vec2 zeroToOne = a_position / u_resolution;"+"   vec2 zeroToTwo = zeroToOne * 2.0;"+"   vec2 clipSpace = zeroToTwo - 1.0;"+"   gl_Position = vec4(clipSpace * vec2(1, -1), 0, 1);"+"v_texCoord = a_texCoord;"+"}";AgoraRTC.yuvShaderSource="precision mediump float;"+"uniform sampler2D Ytex;"+"uniform sampler2D Utex,Vtex;"+"varying vec2 v_texCoord;"+"void main(void) {"+"  float nx,ny,r,g,b,y,u,v;"+"  mediump vec4 txl,ux,vx;"+"  nx=v_texCoord[0];"+"  ny=v_texCoord[1];"+"  y=texture2D(Ytex,vec2(nx,ny)).r;"+"  u=texture2D(Utex,vec2(nx,ny)).r;"+"  v=texture2D(Vtex,vec2(nx,ny)).r;"+"  y=1.1643*(y-0.0625);"+"  u=u-0.5;"+"  v=v-0.5;"+"  r=y+1.5958*v;"+"  g=y-0.39173*u-0.81290*v;"+"  b=y+2.017*u;"+"  gl_FragColor=vec4(r,g,b,1.0);"+"}";AgoraRTC.videoStreamSource='function onSuccess(){postMessage({type:"init",result:!0})}function onFailure(a){postMessage(a)}function sendMessage(a){"videoStat"!=a.command&&console.log(JSON.stringify(a)),that.stream&&that.stream.readyState==WebSocket.OPEN&&that.stream.send(JSON.stringify(a))}var that={};that.stream=void 0,that.init=function(a){return that.stream&&that.stream.readyState===WebSocket.OPEN?(console.warn("stream "+that.getId()+" has been initialized already"),void onSuccess()):(that.local=a.local,that.profile=a.videoProfile,that.stream=new WebSocket(a.url),that.stream.onopen=function(a){console.log((that.local?"local":"remote")+" Stream "),console.log(a),that.profile&&sendMessage({command:"setVideoProfile",profile:that.profile.profile,landscape:that.profile.landscape}),that.stream.binaryType="arraybuffer",onSuccess()},that.stream.onclose=function(a){console.log(that.local?"local":"remote"," Stream ",a),that.stream=void 0,onFailure({type:a.type,code:a.code,reason:a.reason})},that.stream.onerror=function(a){console.log(that.local?"local":"remote"," Stream ",a),onFailure({type:a.type,code:a.code,reason:a.reason})},void(that.stream.onmessage=function(a){"string"==typeof a.data?console.log(that.local?"local":"remote"," message from agent ",a.data):a.data instanceof ArrayBuffer?postMessage({type:"message",data:a.data}):a.data instanceof Blob&&console.warn("Blob image data is not supported")}))},that.close=function(){that.stream&&(that.stream.onmessage=void 0,that.stream.close()),that.stream=void 0},self.addEventListener("message",function(a){var b=a.data;switch(b.type){case"init":that.init(b);break;case"send":sendMessage(b.message);break;case"close":that.close()}},!1);';var AgoraCall=function(callback,parameters){if(callback){callback(parameters)}};var L={VideoProfiles:{Profile120P:"120P",Profile120P_3:"120P_3",Profile96P:"96P",Profile48P:"48P",Profile180P:"180P",Profile180P_3:"180P_3",Profile180P_4:"180P_4",Profile240P:"240P",Profile240P_3:"240P_3",Profile240P_4:"240P_4",Profile360P:"360P",Profile360P_3:"360P_3",Profile360P_4:"360P_4",Profile360P_6:"360P_6",Profile360P_7:"360P_7",Profile360P_8:"360P_8",Profile480P:"480P",Profile480P_3:"480P_3",Profile480P_4:"480P_4",Profile480P_6:"480P_6",Profile480P_8:"480P_8",Profile480P_9:"480P_9",Profile720P:"720P",Profile720P_3:"720P_3",Profile720P_5:"720P_5",Profile720P_6:"720P_6"},ErrorCode:{NO_ERROR:0,FAILED:1,INVALID_ARGUMENT:2,NOT_READY:3,NOT_SUPPORTED:4,REFUSED:5,BUFFER_TOO_SMALL:6,NOT_INITIALIZED:7,INVALID_VIEW:8,NO_PERMISSION:9,TIMEDOUT:10,CANCELED:11,TOO_OFTEN:12,BIND_SOCKET:13,NET_DOWN:14,NET_NOBUFS:15,INIT_VIDEO:16,JOIN_CHANNEL_REJECTED:17,LEAVE_CHANNEL_REJECTED:18,ALREADY_IN_USE:19,INVALID_APP_ID:101,INVALID_CHANNEL_NAME:102,CHANNEL_KEY_EXPIRED:109,INVALID_CHANNEL_KEY:110,CONNECTION_INTERRUPTED:111,CONNECTION_LOST:112,LOAD_MEDIA_ENGINE:1001,START_CALL:1002,START_CAMERA:1003,START_VIDEO_RENDER:1004,ADM_GENERAL_ERROR:1005,ADM_JAVA_RESOURCE:1006,ADM_SAMPLE_RATE:1007,ADM_INIT_PLAYOUT:1008,ADM_START_PLAYOUT:1009,ADM_STOP_PLAYOUT:1010,ADM_INIT_RECORDING:1011,ADM_START_RECORDING:1012,ADM_STOP_RECORDING:1013,ADM_RUNTIME_PLAYOUT_ERROR:1015,ADM_RUNTIME_RECORDING_ERROR:1017,ADM_RECORD_AUDIO_FAILED:1018,ADM_INIT_LOOPBACK:1022,ADM_START_LOOPBACK:1023,VDM_CAMERA_NOT_AUTHORIZED:1501}};AgoraRTC.EventDispatcher=function(spec){"use strict";var that={};spec.dispatcher={};spec.dispatcher.eventListeners={};that.addEventListener=function(eventType,listener){spec.dispatcher.eventListeners[eventType]=listener;if(!spec.dispatcher.eventListeners[eventType]){delete spec.dispatcher.eventListeners[eventType]}};that.on=that.addEventListener;that.dispatchEvent=function(event){if(spec.dispatcher.eventListeners.hasOwnProperty(event.type)){spec.dispatcher.eventListeners[event.type](event)}};return that};AgoraRTC.BasicEvent=function(spec){"use strict";var that={};that.type=spec.type;return that};AgoraRTC.StreamEvent=function(spec){"use strict";var that=AgoraRTC.BasicEvent(spec);that.stream=spec.stream;that.msg=spec.msg;return that};AgoraRTC.ClientEvent=function(spec){"use strict";var that=AgoraRTC.BasicEvent(spec);that.uid=spec.uid;that.attr=spec.attr;that.streams=spec.streams;return that};AgoraRTC.MediaEvent=function(spec){"use strict";var that=AgoraRTC.BasicEvent(spec);that.msg=spec.msg;return that};AgoraRTC.Signal=function(spec){var that=AgoraRTC.EventDispatcher(spec);that.connection=new WebSocket(spec.url);that.sendMessage=function(obj,onError){if(that.connection.readyState==WebSocket.OPEN){that.connection.send(JSON.stringify(obj))}else{console.log("connection to agent lost.");onError({error:"not connected"})}};that.close=function(){that.onEvent=onEvent;that.connection.onopen=undefined;that.connection.onclose=undefined;that.connection.onerror=undefined;that.connection.onmessage=undefined;that.connection.close()};that.connection.onopen=function(e){console.log(e);that.dispatchEvent(AgoraRTC.MediaEvent({type:"onopen",event:e}))};that.connection.onclose=function(e){console.log(e);AgoraCall(spec.onFailure,e)};that.connection.onerror=function(e){console.log(e);AgoraCall(spec.onFailure,e)};that.onEvent=onEvent;that.connection.onmessage=function(e){console.log(e);var m=JSON.parse(e.data);if(m.hasOwnProperty("command")){that.dispatchEvent(AgoraRTC.MediaEvent({type:m.command,msg:m}))}else if(m.hasOwnProperty("event")){that.onEvent(m)}};function onEvent(e){console.log(e)}return that};AgoraRTC.Stream=function(spec){var that=AgoraRTC.EventDispatcher(spec);that.stream=undefined;that.render=undefined;that.interval=undefined;that.lastRenderCount=0;that.profile=undefined;that.latency=0;var header=undefined;var yplane=undefined;var uplane=undefined;var vplane=undefined;that.init=function(onSuccess,onFailure){if(that.stream){console.warn("stream "+that.getId()+" has been initialized already");onSuccess();return}var videoStreamFileBlob=new Blob([AgoraRTC.videoStreamSource],{type:"application/javascript"});that.stream=new Worker(URL.createObjectURL(videoStreamFileBlob));that.oninit=function(e){console.log((spec.local?"local":"remote")+" Stream ");console.log(e);if(e.result===true){onSuccess()}else{onFailure()}};that.onclose=function(e){window.clearInterval(that.interval);console.log(spec.local?"local":"remote"," Stream ",e);that.stream=undefined};that.onerror=function(e){window.clearInterval(that.interval);console.log(spec.local?"local":"remote"," Stream ",e);onFailure(e)};that.onmessage=function(e){if(typeof e==="string"){console.log(spec.local?"local":"remote"," message from agent ",e)}else if(e instanceof ArrayBuffer){if(that.render){that.drawImage(e)}}else if(e instanceof Blob){console.warn("Blob image data is not supported")}};that.stream.addEventListener("message",onWorkerMessage,false);that.stream.postMessage({type:"init",local:spec.local,url:AgoraRTC.url,videoProfile:that.profile})};that.close=function(){window.clearInterval(that.interval);if(that.stream){that.stream.removeEventListener("message",onWorkerMessage);that.stream.postMessage({
type:"close"})}that.stream=undefined;that.stop()};that.play=function(div,onFailure){var view=document.getElementById(div);that.stop();that.render=AgoraRender();that.render.start(view,onFailure);if(spec.local){that.stream.postMessage({type:"send",message:{command:"preview",uid:spec.streamID,audio:spec.audio===false?false:true,video:spec.video===false?false:true,screen:spec.screen===true?true:false}})}else{that.stream.postMessage({type:"send",message:{command:"subscribe",uid:spec.streamID}})}if(that.interval){window.clearInterval(that.interval)}that.interval=window.setInterval(function(){if(that.render){var fps=that.render.renderImageCount-that.lastRenderCount;that.stream.postMessage({type:"send",message:{command:"videoStat",uid:spec.streamID,fps:fps,frameCount:that.lastRenderCount,latency:that.latency}});that.lastRenderCount=that.render.renderImageCount;that.latency=0}},1e3)};that.stop=function(){if(that.render){that.render.stop()}that.render=undefined;that.lastRenderCount=0};that.bindClient=function(client){that.client=client};that.enableAudio=function(callback){return that.client.enableAudio(that,callback)};that.disableAudio=function(callback){return that.client.disableAudio(that,callback)};that.enableVideo=function(callback){return that.client.enableVideo(that,callback)};that.disableVideo=function(callback){return that.client.disableVideo(that,callback)};that.getId=function(){return spec.streamID};that.drawImage=function(data){if(!header){header=data;if(header.byteLength!=20){console.error("invalid image header "+data.byteLength);resetImageData()}return}if(!yplane){yplane=data;if(yplane.byteLength===20){console.error("invalid image header "+data.byteLength+" "+yplane.byteLength);resetImageData()}return}if(!uplane){uplane=data;if(uplane.byteLength===20){console.error("invalid image header "+data.byteLength+" "+yplane.byteLength+" "+uplane.byteLength);resetImageData()}return}if(!vplane){vplane=data;if(yplane.byteLength!=uplane.byteLength*4||uplane.byteLength!=vplane.byteLength){console.error("invalid image header "+data.byteLength+" "+yplane.byteLength+" "+uplane.byteLength+" "+vplane.byteLength);resetImageData();return}var headerLength=20;var dv=new DataView(header);var format=dv.getUint8(0);var mirror=dv.getUint8(1);var width=dv.getUint16(2);var height=dv.getUint16(4);var left=dv.getUint16(6);var top=dv.getUint16(8);var right=dv.getUint16(10);var bottom=dv.getUint16(12);var rotation=dv.getUint16(14);var ts=dv.getUint32(16);var xWidth=width+left+right;var xHeight=height+top+bottom;var yLength=xWidth*xHeight;var yBegin=headerLength;var yEnd=yBegin+yLength;var uLength=yLength/4;var uBegin=yEnd;var uEnd=uBegin+uLength;var vLength=yLength/4;var vBegin=uEnd;var vEnd=vBegin+vLength;that.render.renderImage({mirror:mirror,width:width,height:height,left:left,top:top,right:right,bottom:bottom,rotation:rotation,yplane:new Uint8Array(yplane),uplane:new Uint8Array(uplane),vplane:new Uint8Array(vplane)});var now32=(Date.now()&4294967295)>>>0;var latency=now32-ts;if(latency>that.latency){that.latency=latency}resetImageData()}};that.setVideoProfile=function(profile){that.profile={};if(typeof profile!=="string"&&typeof profile!=="object"){return false}if(profile.hasOwnProperty("profile")){that.profile.profile=profile.profile}if(profile.hasOwnProperty("landscape")){that.profile.landscape=profile.landscape!==false}else{that.profile.landscape=true}if(typeof profile=="string"){that.profile.profile=profile}if(that.stream){that.stream.postMessage({type:"send",message:{command:"setVideoProfile",profile:that.profile.profile,landscape:that.profile.landscape}})}return true};function resetImageData(){header=undefined;yplane=undefined;uplane=undefined;vplane=undefined}function getReso(w,h){return{width:w,height:h}}var supportedVideoList={true:true,unspecified:true,"120p":getReso(160,120),"240p":getReso(320,240),"360p":getReso(640,360),"480p":getReso(640,480),"720p":getReso(1280,720),"1080p":getReso(1920,1080),"4k":getReso(3840,2160)};function onWorkerMessage(e){var data=e.data;switch(data.type){case"init":that.oninit(data);break;case"close":that.onclose(data);break;case"error":that.onerror(data);break;case"message":that.onmessage(data.data);break}}return that};AgoraRTC.createStream=function(spec){return AgoraRTC.Stream(spec)};AgoraRTC.Client=function(spec){"use strict";var that=AgoraRTC.EventDispatcher(spec);that.signal=undefined;that.localStream=undefined;that.remoteStreams={};that.signalOpen=false;function onSignalError(e){if(that.signalOpen){console.log("signal connection lost.");var message={reason:"LOST_CONNECTION_TO_AGENT",type:"error",eventType:e.type,code:e.code};AgoraCall(that.onFailure,message);that.dispatchEvent(message);return}if(e.type==="close"){return}var message={reason:"CLOSE_BEFORE_OPEN",type:"error",eventType:e.type,code:e.code,agentInstallUrl:getInstallUrl(),agentInstallGuideUrl:getInstallGuideUrl()};AgoraCall(that.onFailure,message);that.dispatchEvent(message)}that.init=function(appId,onSuccess,onFailure){spec.appId=appId;that.onFailure=onFailure;if(!that.signal){try{that.signal=AgoraRTC.Signal({url:AgoraRTC.url,onFailure:onSignalError})}catch(e){console.log("create signal connection failed"+e);AgoraCall(onFailure,e);return}that.signal.on("onopen",function(event){that.signalOpen=true;that.signal.onEvent=function(e){if(e.event==="error"){var name=getErrorCodeName(e.code);if(name){e.reason=name}}else if(e.event==="onUserMuteAudio"){e.event="peer-mute-audio"}else if(e.event==="onUserMuteVideo"){e.event="peer-mute-video"}that.dispatchEvent(AgoraRTC.MediaEvent({type:e.event,msg:e}))};that.getVersion(function(ev){if(ev.version=="1.6.0"||ev.version=="1.6.1"){initialize(onSuccess,onFailure);return}var message={reason:"INCOMPATIBLE_WEBAGENT",type:"error",version:AgoraRTC.version,agentVersion:ev.version,agentInstallUrl:getInstallUrl(),agentInstallGuideUrl:getInstallGuideUrl()};that.dispatchEvent(message)},function(ev){var message={reason:"INCOMPATIBLE_WEBAGENT",type:"error",version:AgoraRTC.version,agentVersion:ev.version,agentInstallUrl:getInstallUrl(),agentInstallGuideUrl:getInstallGuideUrl()};that.dispatchEvent(message)})})}if(that.signalOpen){initialize(onSuccess,onFailure)}function initialize(onSuccess,onFailure){that.signal.on("initialize",function(ev){var m=ev.msg;if(m.code!=true){that.signal.close();that.signal=undefined;onError(m.code);return}AgoraCall(onSuccess,m)});that.signal.sendMessage({command:"initialize",appId:spec.appId},function(err){AgoraCall(onFailure,err)})}function onError(code){var name=getErrorCodeName(code);if(name){AgoraCall(onFailure,{reason:name});return}AgoraCall(onFailure,{reason:"UNKNOWN_ERROR",code:m.code});return}function getErrorCodeName(code){for(var k in L.ErrorCode){if(L.ErrorCode.hasOwnProperty(k)){if(L.ErrorCode[k]===code){return String(k)}}}return null}that.signal.on("onError",function(ev){var m=ev.msg;onError(m.code)});console.log(that.signal)};that.renewChannelKey=function(channelKey,onSuccess,onFailure){that.signal.sendMessage({command:"renewChannelKey",channelKey:channelKey},function(e){AgoraCall(onFailure,e)});that.signal.on("renewChannelKey",function(ev){var m=ev.msg;if(m.code===true){AgoraCall(onSuccess,m);return}AgoraCall(onFailure,m)})};that.join=function(channelKey,channelName,uid,onSuccess,onFailure){if(channelName.length>64){onFailure(L.ErrorCode.INVALID_CHANNEL_NAME);return}that.signal.sendMessage({command:"joinChannel",channelKey:channelKey,channelName:channelName,uid:uid},function(e){AgoraCall(onFailure,{error:e})});that.signal.on("joinChannel",function(ev){that.signal.on("onAddVideoStream",onAddVideoStream);that.signal.on("onPeerLeave",onPeerLeave);var m=ev.msg;if(m.code==true){if(!m.uid){m.uid=0}AgoraCall(onSuccess,m.uid);return}if(m.code==L.ErrorCode.JOIN_CHANNEL_REJECTED){console.error("Command joinChannel has been rejected by agent. Is this user joined a channel already?");AgoraCall(onFailure,m.code);return}AgoraCall(onFailure,m.code)})};that.leave=function(onSuccess,onFailure){closeLocalRemoteStreams();that.signal.sendMessage({command:"leaveChannel"},function(e){console.log("leave channel failed",e)});that.signal.on("leaveChannel",function(ev){var m=ev.msg;if(m.code===true){AgoraCall(onSuccess,m);return}if(m.code==L.ErrorCode.LEAVE_CHANNEL_REJECTED){console.error("Command leaveChannel has been rejected by agent. Is this user not in a channel?");AgoraCall(onSuccess,m);return}AgoraCall(onFailure,{code:m.code})})};that.publish=function(stream,onSuccess,onFailure){that.localStream=stream;that.signal.sendMessage({command:"unmuteLocal"},function(e){AgoraCall(onFailure,{error:e})});that.signal.on("unmuteLocal",function(ev){var m=ev.msg;if(m.code!=true){AgoraCall(onFailure,{code:m.code});return}AgoraCall(onSuccess,m)});stream.bindClient(that)};that.unpublish=function(stream,onSuccess,onFailure){that.signal.sendMessage({command:"muteLocal"},function(e){AgoraCall(onFailure,{error:e})});that.signal.on("muteLocal",function(ev){var m=ev.msg;if(m.code!=true){AgoraCall(onFailure,{code:m.code});return}AgoraCall(onSuccess,m)})};that.subscribe=function(stream,onFailure){stream.init(function(){that.dispatchEvent(AgoraRTC.StreamEvent({type:"stream-subscribed",stream:stream}))},function(error){AgoraCall(onFailure,error)})};that.unsubscribe=function(streamID,onFailure){console.log("remote streams",that.remoteStreams);if(that.remoteStreams[streamID]==undefined){AgoraCall(onFailure,{error:"no such stream"});return}that.remoteStreams[streamID].close();delete that.remoteStreams[streamID]};that.enableAudio=function(stream,callback){that.signal.on("enableAudio",function(ev){var m=ev.msg;if(callback){AgoraCall(callback,m.code===true)}});that.signal.sendMessage({command:"enableAudio",streamID:stream.getId()},function(e){e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(callback,e)});return true};that.disableAudio=function(stream,callback){that.signal.on("disableAudio",function(ev){var m=ev.msg;if(callback){AgoraCall(callback,m.code===true)}});that.signal.sendMessage({command:"disableAudio",streamID:stream.getId()},function(e){e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(callback,e)});return true};that.enableVideo=function(stream,callback){that.signal.on("enableVideo",function(ev){var m=ev.msg;if(callback){AgoraCall(callback,m.code===true)}});that.signal.sendMessage({command:"enableVideo",streamID:stream.getId()},function(e){e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(callback,e)});return true};that.disableVideo=function(stream,callback){that.signal.on("disableVideo",function(ev){var m=ev.msg;if(callback){AgoraCall(callback,m.code===true)}});that.signal.sendMessage({command:"disableVideo",streamID:stream.getId()},function(e){e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(callback,e)});return true};that.getDevices=function(callback){that.signal.sendMessage({command:"getDevices"},function(e){e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(callback,e)});that.signal.on("getDevices",function(ev){var devices=ev.msg.devices;AgoraCall(callback,devices)})};that.selectDevice=function(device,onFailure){that.signal.sendMessage({command:"selectDevice",device:device},function(e){e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(onFailure,e)});that.signal.on("selectDevice",function(ev){console.log(ev)})};that.startRecording=function(recordingKey,onSuccess,onFailure){that.signal.sendMessage({command:"startRecording",recordingKey:recordingKey},function(e){e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(onFailure,e)});that.signal.on("startRecording",function(ev){var start=ev.msg;if(start.code===true){AgoraCall(onSuccess,start)}else{AgoraCall(onFailure,start)}})};that.stopRecording=function(recordingKey,onSuccess,onFailure){that.signal.sendMessage({command:"stopRecording",recordingKey:recordingKey},function(e){e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(onFailure,e)});that.signal.on("stopRecording",function(ev){var stop=ev.msg;if(stop.code===true){AgoraCall(onSuccess,stop)}else{AgoraCall(onFailure,stop)}})};that.queryRecordingStatus=function(onStatus){that.signal.sendMessage({command:"queryRecordingStatus"},function(e){e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(onStatus,e)});that.signal.on("queryRecordingStatus",function(ev){onStatus(ev.msg)})};that.close=function(){closeLocalRemoteStreams();if(that.signal){that.signal.close();that.signal=undefined}};that.getVersion=function(onSuccess,onFailure){that.signal.sendMessage({command:"getVersion"});that.signal.on("getVersion",function(ev){var m=ev.msg;if(m.code===true){onSuccess(m)}else{onFailure(m)}})};that.setParameters=function(parameters){that.signal.sendMessage({command:"setParameters",parameters:JSON.stringify(parameters)})};that.setEncryptionMode=function(encryptionMode){that.signal.sendMessage({command:"setEncryptionMode",mode:encryptionMode})};that.setEncryptionSecret=function(secret){that.signal.sendMessage({command:"setEncryptionSecret",secret:secret})};that.getWindows=function(callback){that.signal.sendMessage({command:"getWindows"},function(e){e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(callback,e)});that.signal.on("getWindows",function(ev){var windows=ev.msg.windows;AgoraCall(callback,windows)})};that.startScreenSharing=function(window,onSuccess,onFailure){that.signal.sendMessage({command:"startScreenSharing",window:window},function(e){e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(onFailure,e)});that.signal.on("startScreenSharing",function(e){AgoraCall(onSuccess,e)})};that.setScreenSharingWindow=function(window,onSuccess,onFailure){that.signal.sendMessage({command:"setScreenSharingWindow",window:window},function(e){e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(onFailure,e)});that.signal.on("setScreenSharingWindow",function(e){AgoraCall(onSuccess,e);console.log(e)})};that.stopScreenSharing=function(onSuccess,onFailure){that.signal.sendMessage({command:"stopScreenSharing"},function(e){e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(onFailure,e)});that.signal.on("stopScreenSharing",function(e){AgoraCall(onSuccess,e);console.log(e)})};function onPeerLeave(ev){var m=ev.msg;if(that.remoteStreams.hasOwnProperty(m.uid)){that.remoteStreams[m.uid].close();delete that.remoteStreams[m.uid];console.log("remote streams after peer leave ",that.remoteStreams)}that.dispatchEvent(AgoraRTC.ClientEvent({type:"peer-leave",uid:m.uid}))}function onAddVideoStream(ev){var m=ev.msg;if(!that.remoteStreams.hasOwnProperty(m.uid)){var stream=AgoraRTC.Stream({streamID:m.uid,local:false,audio:m.audio,video:m.video,screen:m.screen});stream.bindClient(that);that.remoteStreams[m.uid]=stream}that.dispatchEvent(AgoraRTC.StreamEvent({type:"stream-added",stream:that.remoteStreams[m.uid]}));console.log("remote streams",that.remoteStreams)}function getInstallUrl(){if(navigator.appVersion.indexOf("Mac")!=-1){return AgoraRTC.macAgentInstallUrl}return AgoraRTC.winAgentInstallUrl}function getInstallGuideUrl(){var userLang=navigator.language||navigator.userLanguage;if(userLang.indexOf("zh")!=-1){return AgoraRTC.cnAgentInstallGuideUrl}return AgoraRTC.enAgentInstallGuideUrl}function closeLocalRemoteStreams(){if(that.localStream){that.localStream.close()}that.localStream=undefined;for(var streamId in that.remoteStreams){if(that.remoteStreams.hasOwnProperty(streamId)){that.remoteStreams[streamId].close();delete that.remoteStreams[streamId]}}}return that};AgoraRTC.createRtcClient=function(){return AgoraRTC.Client({})};
//# sourceMappingURL=dist/AgoraRtcAgentSDK.min.js.map
"use strict";(function(root,factory){if(typeof define==="function"&&define.amd){define([],factory)}else{var lib=factory.call(root);Object.keys(lib).forEach(function(key){root[key]=lib[key]})}})(this,function(){var topWindow=this;function error(msg){if(topWindow.console){if(topWindow.console.error){topWindow.console.error(msg)}else if(topWindow.console.log){topWindow.console.log(msg)}}}function isInIFrame(w){w=w||topWindow;return w!==w.top}function makeFailHTML(msg){return""+'<table style="background-color: #8CE; width: 100%; height: 100%;"><tr>'+'<td align="center">'+'<div style="display: table-cell; vertical-align: middle;">'+'<div style="">'+msg+"</div>"+"</div>"+"</td></tr></table>"}var GET_A_WEBGL_BROWSER=""+"This page requires a browser that supports WebGL.<br/>"+'<a href="http://get.webgl.org">Click here to upgrade your browser.</a>';var OTHER_PROBLEM=""+"It doesn't appear your computer can support WebGL.<br/>"+'<a href="http://get.webgl.org/troubleshooting/">Click here for more information.</a>';function create3DContext(canvas,opt_attribs){var names=["webgl","experimental-webgl"];var context=null;for(var ii=0;ii<names.length;++ii){try{context=canvas.getContext(names[ii],opt_attribs)}catch(e){}if(context){break}}return context}function setupWebGL(canvas,opt_attribs){function showLink(str){var container=canvas.parentNode;if(container){container.innerHTML=makeFailHTML(str)}}if(!topWindow.WebGLRenderingContext){showLink(GET_A_WEBGL_BROWSER);return null}var context=create3DContext(canvas,opt_attribs);if(!context){showLink(OTHER_PROBLEM)}return context}function updateCSSIfInIFrame(){if(isInIFrame()){document.body.className="iframe"}}function getWebGLContext(canvas,opt_attribs,opt_options){var options=opt_options||{};if(isInIFrame()){updateCSSIfInIFrame();if(!options.dontResize&&options.resize!==false){var width=canvas.clientWidth;var height=canvas.clientHeight;canvas.width=width;canvas.height=height}}else if(!options.noTitle&&options.title!==false){var title=document.title;var h1=document.createElement("h1");h1.innerText=title;document.body.insertBefore(h1,document.body.children[0])}var gl=setupWebGL(canvas,opt_attribs);return gl}function loadShader(gl,shaderSource,shaderType,opt_errorCallback){var errFn=opt_errorCallback||error;var shader=gl.createShader(shaderType);gl.shaderSource(shader,shaderSource);gl.compileShader(shader);var compiled=gl.getShaderParameter(shader,gl.COMPILE_STATUS);if(!compiled){var lastError=gl.getShaderInfoLog(shader);errFn("*** Error compiling shader '"+shader+"':"+lastError);gl.deleteShader(shader);return null}return shader}function createProgram(gl,shaders,opt_attribs,opt_locations,opt_errorCallback){var errFn=opt_errorCallback||error;var program=gl.createProgram();shaders.forEach(function(shader){gl.attachShader(program,shader)});if(opt_attribs){obj_attrib.forEach(function(attrib,ndx){gl.bindAttribLocation(program,opt_locations?opt_locations[ndx]:ndx,attrib)})}gl.linkProgram(program);var linked=gl.getProgramParameter(program,gl.LINK_STATUS);if(!linked){var lastError=gl.getProgramInfoLog(program);errFn("Error in program linking:"+lastError);gl.deleteProgram(program);return null}return program}function createShaderFromScript(gl,scriptId,opt_shaderType,opt_errorCallback){var shaderSource="";var shaderType;var shaderScript=document.getElementById(scriptId);if(!shaderScript){throw"*** Error: unknown script element"+scriptId}shaderSource=shaderScript.text;if(!opt_shaderType){if(shaderScript.type==="x-shader/x-vertex"){shaderType=gl.VERTEX_SHADER}else if(shaderScript.type==="x-shader/x-fragment"){shaderType=gl.FRAGMENT_SHADER}else if(shaderType!==gl.VERTEX_SHADER&&shaderType!==gl.FRAGMENT_SHADER){throw"*** Error: unknown shader type"}}return loadShader(gl,shaderSource,opt_shaderType?opt_shaderType:shaderType,opt_errorCallback)}var defaultShaderType=["VERTEX_SHADER","FRAGMENT_SHADER"];function createProgramFromScripts(gl,shaderScriptIds,opt_attribs,opt_locations,opt_errorCallback){var shaders=[];for(var ii=0;ii<shaderScriptIds.length;++ii){shaders.push(createShaderFromScript(gl,shaderScriptIds[ii],gl[defaultShaderType[ii]],opt_errorCallback))}return createProgram(gl,shaders,opt_attribs,opt_locations,opt_errorCallback)}function createProgramFromSources(gl,shaderSources,opt_attribs,opt_locations,opt_errorCallback){var shaders=[];for(var ii=0;ii<shaderSources.length;++ii){shaders.push(loadShader(gl,shaderSources[ii],gl[defaultShaderType[ii]],opt_errorCallback))}return createProgram(gl,shaders,opt_attribs,opt_locations,opt_errorCallback)}function getBindPointForSamplerType(gl,type){if(type===gl.SAMPLER_2D)return gl.TEXTURE_2D;if(type===gl.SAMPLER_CUBE)return gl.TEXTURE_CUBE_MAP}function createUniformSetters(gl,program){var textureUnit=0;function createUniformSetter(program,uniformInfo){var location=gl.getUniformLocation(program,uniformInfo.name);var type=uniformInfo.type;var isArray=uniformInfo.size>1&&uniformInfo.name.substr(-3)==="[0]";if(type===gl.FLOAT&&isArray){return function(v){gl.uniform1fv(location,v)}}if(type===gl.FLOAT){return function(v){gl.uniform1f(location,v)}}if(type===gl.FLOAT_VEC2){return function(v){gl.uniform2fv(location,v)}}if(type===gl.FLOAT_VEC3){return function(v){gl.uniform3fv(location,v)}}if(type===gl.FLOAT_VEC4){return function(v){gl.uniform4fv(location,v)}}if(type===gl.INT&&isArray){return function(v){gl.uniform1iv(location,v)}}if(type===gl.INT){return function(v){gl.uniform1i(location,v)}}if(type===gl.INT_VEC2){return function(v){gl.uniform2iv(location,v)}}if(type===gl.INT_VEC3){return function(v){gl.uniform3iv(location,v)}}if(type===gl.INT_VEC4){return function(v){gl.uniform4iv(location,v)}}if(type===gl.BOOL){return function(v){gl.uniform1iv(location,v)}}if(type===gl.BOOL_VEC2){return function(v){gl.uniform2iv(location,v)}}if(type===gl.BOOL_VEC3){return function(v){gl.uniform3iv(location,v)}}if(type===gl.BOOL_VEC4){return function(v){gl.uniform4iv(location,v)}}if(type===gl.FLOAT_MAT2){return function(v){gl.uniformMatrix2fv(location,false,v)}}if(type===gl.FLOAT_MAT3){return function(v){gl.uniformMatrix3fv(location,false,v)}}if(type===gl.FLOAT_MAT4){return function(v){gl.uniformMatrix4fv(location,false,v)}}if((type===gl.SAMPLER_2D||type===gl.SAMPLER_CUBE)&&isArray){var units=[];for(var ii=0;ii<info.size;++ii){units.push(textureUnit++)}return function(bindPoint,units){return function(textures){gl.uniform1iv(location,units);textures.forEach(function(texture,index){gl.activeTexture(gl.TEXTURE0+units[index]);gl.bindTexture(bindPoint,texture)})}}(getBindPointForSamplerType(gl,type),units)}if(type===gl.SAMPLER_2D||type===gl.SAMPLER_CUBE){return function(bindPoint,unit){return function(texture){gl.uniform1i(location,unit);gl.activeTexture(gl.TEXTURE0+unit);gl.bindTexture(bindPoint,texture)}}(getBindPointForSamplerType(gl,type),textureUnit++)}throw"unknown type: 0x"+type.toString(16)}var uniformSetters={};var numUniforms=gl.getProgramParameter(program,gl.ACTIVE_UNIFORMS);for(var ii=0;ii<numUniforms;++ii){var uniformInfo=gl.getActiveUniform(program,ii);if(!uniformInfo){break}var name=uniformInfo.name;if(name.substr(-3)==="[0]"){name=name.substr(0,name.length-3)}var setter=createUniformSetter(program,uniformInfo);uniformSetters[name]=setter}return uniformSetters}function setUniforms(setters,values){Object.keys(values).forEach(function(name){var setter=setters[name];if(setter){setter(values[name])}})}function createAttributeSetters(gl,program){var attribSetters={};function createAttribSetter(index){return function(b){gl.bindBuffer(gl.ARRAY_BUFFER,b.buffer);gl.enableVertexAttribArray(index);gl.vertexAttribPointer(index,b.numComponents||b.size,b.type||gl.FLOAT,b.normalize||false,b.stride||0,b.offset||0)}}var numAttribs=gl.getProgramParameter(program,gl.ACTIVE_ATTRIBUTES);for(var ii=0;ii<numAttribs;++ii){var attribInfo=gl.getActiveAttrib(program,ii);if(!attribInfo){break}var index=gl.getAttribLocation(program,attribInfo.name);attribSetters[attribInfo.name]=createAttribSetter(index)}return attribSetters}function setAttributes(setters,buffers){Object.keys(buffers).forEach(function(name){var setter=setters[name];if(setter){setter(buffers[name])}})}function createProgramInfo(gl,shaderSources,opt_attribs,opt_locations,opt_errorCallback){shaderSources=shaderSources.map(function(source){var script=document.getElementById(source);return script?script.text:source});var program=createProgramFromSources(gl,shaderSources,opt_attribs,opt_locations,opt_errorCallback);if(!program){return null}var uniformSetters=createUniformSetters(gl,program);var attribSetters=createAttributeSetters(gl,program);return{program:program,uniformSetters:uniformSetters,attribSetters:attribSetters}}function setBuffersAndAttributes(gl,setters,buffers){setAttributes(setters,buffers.attribs);if(buffers.indices){gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER,buffers.indices)}}var browserPrefixes=["","MOZ_","OP_","WEBKIT_"];function getExtensionWithKnownPrefixes(gl,name){for(var ii=0;ii<browserPrefixes.length;++ii){var prefixedName=browserPrefixes[ii]+name;var ext=gl.getExtension(prefixedName);if(ext){return ext}}}function resizeCanvasToDisplaySize(canvas,multiplier){multiplier=multiplier||1;var width=canvas.clientWidth*multiplier;var height=canvas.clientHeight*multiplier;if(canvas.width!==width||canvas.height!==height){canvas.width=width;canvas.height=height;return true}return false}function getIFrameForWindow(window){if(!isInIFrame(window)){return}var iframes=window.parent.document.getElementsByTagName("iframe");for(var ii=0;ii<iframes.length;++ii){var iframe=iframes[ii];if(iframe.contentDocument===window.document){return iframe}}}function isFrameVisible(window){try{var iframe=getIFrameForWindow(window);if(!iframe){return true}var bounds=iframe.getBoundingClientRect();var isVisible=bounds.top<window.parent.innerHeight&&bounds.bottom>=0&&bounds.left<window.parent.innerWidth&&bounds.right>=0;return isVisible&&isFrameVisible(window.parent)}catch(e){return true}}function isOnScreen(element){var isVisible=true;if(element){var bounds=element.getBoundingClientRect();isVisible=bounds.top<topWindow.innerHeight&&bounds.bottom>=0}return isVisible&&isFrameVisible(topWindow)}function augmentTypedArray(typedArray,numComponents){var cursor=0;typedArray.push=function(){for(var ii=0;ii<arguments.length;++ii){var value=arguments[ii];if(value instanceof Array||value.buffer&&value.buffer instanceof ArrayBuffer){for(var jj=0;jj<value.length;++jj){typedArray[cursor++]=value[jj]}}else{typedArray[cursor++]=value}}};typedArray.reset=function(opt_index){cursor=opt_index||0};typedArray.numComponents=numComponents;Object.defineProperty(typedArray,"numElements",{get:function(){return this.length/this.numComponents|0}});return typedArray}function createAugmentedTypedArray(numComponents,numElements,opt_type){var Type=opt_type||Float32Array;return augmentTypedArray(new Type(numComponents*numElements),numComponents)}function createBufferFromTypedArray(gl,array,type,drawType){type=type||gl.ARRAY_BUFFER;var buffer=gl.createBuffer();gl.bindBuffer(type,buffer);gl.bufferData(type,array,drawType||gl.STATIC_DRAW);return buffer}function allButIndices(name){return name!=="indices"}function createMapping(obj){var mapping={};Object.keys(obj).filter(allButIndices).forEach(function(key){mapping["a_"+key]=key});return mapping}function getGLTypeForTypedArray(gl,typedArray){if(typedArray instanceof Int8Array){return gl.BYTE}if(typedArray instanceof Uint8Array){return gl.UNSIGNED_BYTE}if(typedArray instanceof Int16Array){return gl.SHORT}if(typedArray instanceof Uint16Array){return gl.UNSIGNED_SHORT}if(typedArray instanceof Int32Array){return gl.INT}if(typedArray instanceof Uint32Array){return gl.UNSIGNED_INT}if(typedArray instanceof Float32Array){return gl.FLOAT}throw"unsupported typed array type"}function getNormalizationForTypedArray(typedArray){if(typedArray instanceof Int8Array){return true}if(typedArray instanceof Uint8Array){return true}return false}function isArrayBuffer(a){return a.buffer&&a.buffer instanceof ArrayBuffer}function guessNumComponentsFromName(name,length){var numComponents;if(name.indexOf("coord")>=0){numComponents=2}else if(name.indexOf("color")>=0){numComponents=4}else{numComponents=3}if(length%numComponents>0){throw"can not guess numComponents. You should specify it."}return numComponents}function makeTypedArray(array,name){if(isArrayBuffer(array)){return array}if(Array.isArray(array)){array={data:array}}if(!array.numComponents){array.numComponents=guessNumComponentsFromName(name,array.length)}var type=array.type;if(!type){if(name==="indices"){type=Uint16Array}}var typedArray=createAugmentedTypedArray(array.numComponents,array.data.length/array.numComponents|0,type);typedArray.push(array.data);return typedArray}function createAttribsFromArrays(gl,arrays,opt_mapping){var mapping=opt_mapping||createMapping(arrays);var attribs={};Object.keys(mapping).forEach(function(attribName){var bufferName=mapping[attribName];var array=makeTypedArray(arrays[bufferName],bufferName);attribs[attribName]={buffer:createBufferFromTypedArray(gl,array),numComponents:array.numComponents||guessNumComponentsFromName(bufferName),type:getGLTypeForTypedArray(gl,array),normalize:getNormalizationForTypedArray(array)}});return attribs}function getNumElementsFromNonIndexedArrays(arrays){var key=Object.keys(arrays)[0];var array=arrays[key];if(isArrayBuffer(array)){return array.numElements}else{return array.data.length/array.numComponents}}function createBufferInfoFromArrays(gl,arrays,opt_mapping){var bufferInfo={attribs:createAttribsFromArrays(gl,arrays,opt_mapping)};var indices=arrays.indices;if(indices){indices=makeTypedArray(indices,"indices");bufferInfo.indices=createBufferFromTypedArray(gl,indices,gl.ELEMENT_ARRAY_BUFFER);bufferInfo.numElements=indices.length}else{bufferInfo.numElements=getNumElementsFromNonIndexedArrays(arrays)}return bufferInfo}function createBuffersFromArrays(gl,arrays){var buffers={};Object.keys(arrays).forEach(function(key){var type=key==="indices"?gl.ELEMENT_ARRAY_BUFFER:gl.ARRAY_BUFFER;var array=makeTypedArray(arrays[key],name);buffers[key]=createBufferFromTypedArray(gl,array,type)});if(arrays.indices){buffers.numElements=arrays.indices.length}else if(arrays.position){buffers.numElements=arrays.position.length/3}return buffers}function drawBufferInfo(gl,type,bufferInfo,count,offset){var indices=bufferInfo.indices;var numElements=count===undefined?bufferInfo.numElements:count;offset=offset===undefined?offset:0;if(indices){gl.drawElements(type,numElements,gl.UNSIGNED_SHORT,offset)}else{gl.drawArrays(type,offset,numElements)}}function drawObjectList(gl,objectsToDraw){var lastUsedProgramInfo=null;var lastUsedBufferInfo=null;objectsToDraw.forEach(function(object){var programInfo=object.programInfo;var bufferInfo=object.bufferInfo;var bindBuffers=false;if(programInfo!==lastUsedProgramInfo){lastUsedProgramInfo=programInfo;gl.useProgram(programInfo.program);bindBuffers=true}if(bindBuffers||bufferInfo!==lastUsedBufferInfo){lastUsedBufferInfo=bufferInfo;setBuffersAndAttributes(gl,programInfo.attribSetters,bufferInfo)}setUniforms(programInfo.uniformSetters,object.uniforms);drawBufferInfo(gl,gl.TRIANGLES,bufferInfo)})}return{createAugmentedTypedArray:createAugmentedTypedArray,createAttribsFromArrays:createAttribsFromArrays,createBuffersFromArrays:createBuffersFromArrays,createBufferInfoFromArrays:createBufferInfoFromArrays,createAttributeSetters:createAttributeSetters,createProgram:createProgram,createProgramFromScripts:createProgramFromScripts,createProgramFromSources:createProgramFromSources,createProgramInfo:createProgramInfo,createUniformSetters:createUniformSetters,drawBufferInfo:drawBufferInfo,drawObjectList:drawObjectList,getWebGLContext:getWebGLContext,updateCSSIfInIFrame:updateCSSIfInIFrame,getExtensionWithKnownPrefixes:getExtensionWithKnownPrefixes,resizeCanvasToDisplaySize:resizeCanvasToDisplaySize,setAttributes:setAttributes,setBuffersAndAttributes:setBuffersAndAttributes,setUniforms:setUniforms,setupWebGL:setupWebGL}});function AgoraRender(){var gl=undefined;var program=undefined;var positionLocation=undefined;var texCoordLocation=undefined;var yTexture=undefined;var uTexture=undefined;var vTexture=undefined;var texCoordBuffer=undefined;var surfaceBuffer=undefined;var that={view:undefined,mirrorView:false,container:undefined,canvas:undefined,renderImageCount:0,initWidth:0,initHeight:0,initRotation:0,canvasUpdated:false,clientWidth:0,clientHeight:0};that.start=function(view,onFailure){initCanvas(view,that.mirrorView,view.clientWidth,view.clientHeight,that.initRotation,onFailure)};that.stop=function(){gl=undefined;program=undefined;positionLocation=undefined;texCoordLocation=undefined;deleteTexture(yTexture);deleteTexture(uTexture);deleteTexture(vTexture);yTexture=undefined;uTexture=undefined;vTexture=undefined;deleteBuffer(texCoordBuffer);deleteBuffer(surfaceBuffer);texCoordBuffer=undefined;surfaceBuffer=undefined;if(that.container){that.container.removeChild(that.canvas)}if(that.view){that.view.removeChild(that.container)}that.canvas=undefined;that.container=undefined;that.view=undefined;that.mirrorView=false};that.renderImage=function(image){if(!gl){return}if(image.width!=that.initWidth||image.height!=that.initHeight||image.rotation!=that.initRotation||image.mirror!=that.mirrorView){var view=that.view;that.stop();console.log("init canvas "+image.width+"*"+image.height+" rotation "+image.rotation);initCanvas(view,image.mirror,image.width,image.height,image.rotation,function(e){console.error("init canvas "+image.width+"*"+image.height+" rotation "+image.rotation+" failed. "+e)})}gl.bindBuffer(gl.ARRAY_BUFFER,texCoordBuffer);var xWidth=image.width+image.left+image.right;var xHeight=image.height+image.top+image.bottom;gl.bufferData(gl.ARRAY_BUFFER,new Float32Array([image.left/xWidth,image.bottom/xHeight,1-image.right/xWidth,image.bottom/xHeight,image.left/xWidth,1-image.top/xHeight,image.left/xWidth,1-image.top/xHeight,1-image.right/xWidth,image.bottom/xHeight,1-image.right/xWidth,1-image.top/xHeight]),gl.STATIC_DRAW);gl.enableVertexAttribArray(texCoordLocation);gl.vertexAttribPointer(texCoordLocation,2,gl.FLOAT,false,0,0);uploadYuv(xWidth,xHeight,image.yplane,image.uplane,image.vplane);updateCanvas(image.rotation,image.width,image.height);gl.drawArrays(gl.TRIANGLES,0,6);that.renderImageCount+=1};function uploadYuv(width,height,yplane,uplane,vplane){gl.activeTexture(gl.TEXTURE0);gl.bindTexture(gl.TEXTURE_2D,yTexture);gl.texImage2D(gl.TEXTURE_2D,0,gl.LUMINANCE,width,height,0,gl.LUMINANCE,gl.UNSIGNED_BYTE,yplane);var e=gl.getError();if(e!=gl.NO_ERROR){console.log("upload y plane ",width,height,yplane.byteLength," error",e)}gl.activeTexture(gl.TEXTURE1);gl.bindTexture(gl.TEXTURE_2D,uTexture);gl.texImage2D(gl.TEXTURE_2D,0,gl.LUMINANCE,width/2,height/2,0,gl.LUMINANCE,gl.UNSIGNED_BYTE,uplane);var e=gl.getError();if(e!=gl.NO_ERROR){console.log("upload u plane ",width,height,uplane.byteLength,"  error",e)}gl.activeTexture(gl.TEXTURE2);gl.bindTexture(gl.TEXTURE_2D,vTexture);gl.texImage2D(gl.TEXTURE_2D,0,gl.LUMINANCE,width/2,height/2,0,gl.LUMINANCE,gl.UNSIGNED_BYTE,vplane);var e=gl.getError();if(e!=gl.NO_ERROR){console.log("upload v plane ",width,height,vplane.byteLength,"  error",e)}}function deleteBuffer(buffer){if(buffer&&that.gl){that.gl.deleteBuffer(buffer)}}function deleteTexture(texture){if(texture&&that.gl){that.gl.deleteTexture(texture)}}function initCanvas(view,mirror,width,height,rotation,onFailure){that.clientWidth=view.clientWidth;that.clientHeight=view.clientHeight;that.view=view;that.mirrorView=mirror;that.canvasUpdated=false;that.container=document.createElement("div");that.container.style.width="100%";that.container.style.height="100%";that.container.style.display="flex";that.container.style.justifyContent="center";that.container.style.alignItems="center";that.view.appendChild(that.container);that.canvas=document.createElement("canvas");if(rotation==0||rotation==180){that.canvas.width=width;that.canvas.height=height}else{that.canvas.width=height;that.canvas.height=width}that.initWidth=width;that.initHeight=height;that.initRotation=rotation;if(that.mirrorView){that.canvas.style.transform="rotateY(180deg)"}that.container.appendChild(that.canvas);try{gl=that.canvas.getContext("webgl")||that.canvas.getContext("experimental-webgl")}catch(e){console.log(e)}if(!gl){gl=undefined;onFailure({error:"Browser not support! No WebGL detected."});return}gl.clearColor(0,0,0,1);gl.enable(gl.DEPTH_TEST);gl.depthFunc(gl.LEQUAL);gl.clear(gl.COLOR_BUFFER_BIT|gl.DEPTH_BUFFER_BIT);program=createProgramFromSources(gl,[AgoraRTC.vertexShaderSource,AgoraRTC.yuvShaderSource]);gl.useProgram(program);initTextures()}function initTextures(){positionLocation=gl.getAttribLocation(program,"a_position");texCoordLocation=gl.getAttribLocation(program,"a_texCoord");surfaceBuffer=gl.createBuffer();texCoordBuffer=gl.createBuffer();gl.activeTexture(gl.TEXTURE0);yTexture=gl.createTexture();gl.bindTexture(gl.TEXTURE_2D,yTexture);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_WRAP_S,gl.CLAMP_TO_EDGE);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_WRAP_T,gl.CLAMP_TO_EDGE);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_MIN_FILTER,gl.NEAREST);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_MAG_FILTER,gl.NEAREST);gl.activeTexture(gl.TEXTURE1);uTexture=gl.createTexture();gl.bindTexture(gl.TEXTURE_2D,uTexture);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_WRAP_S,gl.CLAMP_TO_EDGE);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_WRAP_T,gl.CLAMP_TO_EDGE);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_MIN_FILTER,gl.NEAREST);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_MAG_FILTER,gl.NEAREST);gl.activeTexture(gl.TEXTURE2);vTexture=gl.createTexture();gl.bindTexture(gl.TEXTURE_2D,vTexture);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_WRAP_S,gl.CLAMP_TO_EDGE);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_WRAP_T,gl.CLAMP_TO_EDGE);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_MIN_FILTER,gl.NEAREST);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_MAG_FILTER,gl.NEAREST);var y=gl.getUniformLocation(program,"Ytex");gl.uniform1i(y,0);var u=gl.getUniformLocation(program,"Utex");gl.uniform1i(u,1);var v=gl.getUniformLocation(program,"Vtex");gl.uniform1i(v,2)}function updateCanvas(rotation,width,height){if(that.canvasUpdated){return}that.canvas.style.width="100%";that.canvas.style.height="100%";try{if(rotation===0||rotation===180){if(that.clientWidth/that.clientHeight>width/height){that.canvas.style.width=that.clientHeight*width/height+"px"}else if(that.clientWidth/that.clientHeight<width/height){that.canvas.style.height=that.clientWidth*height/width+"px"}}else{if(that.clientHeight/that.clientWidth>width/height){that.canvas.style.height=that.clientWidth*width/height+"px"}else if(that.clientHeight/that.clientWidth<width/height){that.canvas.style.width=that.clientHeight*height/width+"px"}}}catch(e){console.log("updateCanvas 00001 gone "+that.canvas);console.log(that);console.error(e);return}gl.bindBuffer(gl.ARRAY_BUFFER,surfaceBuffer);gl.enableVertexAttribArray(positionLocation);gl.vertexAttribPointer(positionLocation,2,gl.FLOAT,false,0,0);var p1={x:0,y:0};var p2={x:width,y:0};var p3={x:width,y:height};var p4={x:0,y:height};var pp1=p1,pp2=p2,pp3=p3,pp4=p4;switch(rotation){case 0:break;case 90:pp1=p2;pp2=p3;pp3=p4;pp4=p1;break;case 180:pp1=p3;pp2=p4;pp3=p1;pp4=p2;break;case 270:pp1=p4;pp2=p1;pp3=p2;pp4=p3;break;default:}gl.bufferData(gl.ARRAY_BUFFER,new Float32Array([pp1.x,pp1.y,pp2.x,pp2.y,pp4.x,pp4.y,pp4.x,pp4.y,pp2.x,pp2.y,pp3.x,pp3.y]),gl.STATIC_DRAW);var resolutionLocation=gl.getUniformLocation(program,"u_resolution");gl.uniform2f(resolutionLocation,width,height);that.canvasUpdated=true}return that}var AgoraRTC=function(){"use strict";var AgoraRTC={};Object.defineProperties(AgoraRTC,{version:{get:function(){return"1.7.0"}},name:{get:function(){return"AgoraWebSDK"}}});return AgoraRTC}();AgoraRTC.url="wss://localhost.agora.io:8921/";AgoraRTC.macAgentInstallUrl="http://download.agora.io/AgoraWebAgent-1.7.0.pkg";AgoraRTC.winAgentInstallUrl="http://download.agora.io/AgoraWebAgentSetup-1.7.0.exe";AgoraRTC.enAgentInstallGuideUrl="http://download.agora.io/install-guide-en.html";AgoraRTC.cnAgentInstallGuideUrl="http://download.agora.io/install-guide-cn.html";AgoraRTC.vertexShaderSource="attribute vec2 a_position;"+"attribute vec2 a_texCoord;"+"uniform vec2 u_resolution;"+"varying vec2 v_texCoord;"+"void main() {"+"vec2 zeroToOne = a_position / u_resolution;"+"   vec2 zeroToTwo = zeroToOne * 2.0;"+"   vec2 clipSpace = zeroToTwo - 1.0;"+"   gl_Position = vec4(clipSpace * vec2(1, -1), 0, 1);"+"v_texCoord = a_texCoord;"+"}";AgoraRTC.yuvShaderSource="precision mediump float;"+"uniform sampler2D Ytex;"+"uniform sampler2D Utex,Vtex;"+"varying vec2 v_texCoord;"+"void main(void) {"+"  float nx,ny,r,g,b,y,u,v;"+"  mediump vec4 txl,ux,vx;"+"  nx=v_texCoord[0];"+"  ny=v_texCoord[1];"+"  y=texture2D(Ytex,vec2(nx,ny)).r;"+"  u=texture2D(Utex,vec2(nx,ny)).r;"+"  v=texture2D(Vtex,vec2(nx,ny)).r;"+"  y=1.1643*(y-0.0625);"+"  u=u-0.5;"+"  v=v-0.5;"+"  r=y+1.5958*v;"+"  g=y-0.39173*u-0.81290*v;"+"  b=y+2.017*u;"+"  gl_FragColor=vec4(r,g,b,1.0);"+"}";AgoraRTC.videoStreamSource='function onSuccess(){postMessage({type:"init",result:!0})}function onFailure(a){postMessage(a)}function sendMessage(a){"videoStat"!=a.command&&console.log(JSON.stringify(a)),that.stream&&that.stream.readyState==WebSocket.OPEN&&that.stream.send(JSON.stringify(a))}var that={};that.stream=void 0,that.init=function(a){return that.stream&&that.stream.readyState===WebSocket.OPEN?(console.warn("stream "+that.getId()+" has been initialized already"),void onSuccess()):(that.local=a.local,that.profile=a.videoProfile,that.stream=new WebSocket(a.url),that.stream.onopen=function(a){console.log((that.local?"local":"remote")+" Stream "),console.log(a),that.profile&&sendMessage({command:"setVideoProfile",profile:that.profile.profile,swapWidthAndHeight:that.profile.swapWidthAndHeight}),that.stream.binaryType="arraybuffer",onSuccess()},that.stream.onclose=function(a){console.log(that.local?"local":"remote"," Stream ",a),that.stream=void 0,onFailure({type:a.type,code:a.code,reason:a.reason})},that.stream.onerror=function(a){console.log(that.local?"local":"remote"," Stream ",a),onFailure({type:a.type,code:a.code,reason:a.reason})},void(that.stream.onmessage=function(a){"string"==typeof a.data?console.log(that.local?"local":"remote"," message from agent ",a.data):a.data instanceof ArrayBuffer?postMessage({type:"message",data:a.data}):a.data instanceof Blob&&console.warn("Blob image data is not supported")}))},that.close=function(){that.stream&&(that.stream.onmessage=void 0,that.stream.close()),that.stream=void 0},self.addEventListener("message",function(a){var b=a.data;switch(b.type){case"init":that.init(b);break;case"send":sendMessage(b.message);break;case"close":that.close()}},!1);';var AgoraCall=function(callback,parameters){if(callback){callback(parameters)}};var L={VideoProfiles:{Profile120P:"120P",Profile120P_3:"120P_3",Profile96P:"96P",Profile48P:"48P",Profile180P:"180P",Profile180P_3:"180P_3",Profile180P_4:"180P_4",Profile240P:"240P",Profile240P_3:"240P_3",Profile240P_4:"240P_4",Profile360P:"360P",Profile360P_3:"360P_3",Profile360P_4:"360P_4",Profile360P_6:"360P_6",Profile360P_7:"360P_7",Profile360P_8:"360P_8",Profile480P:"480P",Profile480P_3:"480P_3",Profile480P_4:"480P_4",Profile480P_6:"480P_6",Profile480P_8:"480P_8",Profile480P_9:"480P_9",Profile720P:"720P",Profile720P_3:"720P_3",Profile720P_5:"720P_5",Profile720P_6:"720P_6"},ErrorCode:{NO_ERROR:0,FAILED:1,INVALID_ARGUMENT:2,NOT_READY:3,NOT_SUPPORTED:4,REFUSED:5,BUFFER_TOO_SMALL:6,NOT_INITIALIZED:7,INVALID_VIEW:8,NO_PERMISSION:9,TIMEDOUT:10,CANCELED:11,TOO_OFTEN:12,BIND_SOCKET:13,NET_DOWN:14,NET_NOBUFS:15,INIT_VIDEO:16,JOIN_CHANNEL_REJECTED:17,LEAVE_CHANNEL_REJECTED:18,ALREADY_IN_USE:19,INVALID_APP_ID:101,INVALID_CHANNEL_NAME:102,CHANNEL_KEY_EXPIRED:109,INVALID_CHANNEL_KEY:110,CONNECTION_INTERRUPTED:111,CONNECTION_LOST:112,LOAD_MEDIA_ENGINE:1001,START_CALL:1002,START_CAMERA:1003,START_VIDEO_RENDER:1004,ADM_GENERAL_ERROR:1005,ADM_JAVA_RESOURCE:1006,ADM_SAMPLE_RATE:1007,ADM_INIT_PLAYOUT:1008,ADM_START_PLAYOUT:1009,ADM_STOP_PLAYOUT:1010,ADM_INIT_RECORDING:1011,ADM_START_RECORDING:1012,ADM_STOP_RECORDING:1013,ADM_RUNTIME_PLAYOUT_ERROR:1015,ADM_RUNTIME_RECORDING_ERROR:1017,ADM_RECORD_AUDIO_FAILED:1018,ADM_INIT_LOOPBACK:1022,ADM_START_LOOPBACK:1023,VDM_CAMERA_NOT_AUTHORIZED:1501}};AgoraRTC.EventDispatcher=function(spec){"use strict";var that={};spec.dispatcher={};spec.dispatcher.eventListeners={};that.addEventListener=function(eventType,listener){spec.dispatcher.eventListeners[eventType]=listener;if(!spec.dispatcher.eventListeners[eventType]){delete spec.dispatcher.eventListeners[eventType]}};that.on=that.addEventListener;that.dispatchEvent=function(event){if(spec.dispatcher.eventListeners.hasOwnProperty(event.type)){spec.dispatcher.eventListeners[event.type](event)}};return that};AgoraRTC.BasicEvent=function(spec){"use strict";var that={};that.type=spec.type;return that};AgoraRTC.StreamEvent=function(spec){"use strict";var that=AgoraRTC.BasicEvent(spec);that.stream=spec.stream;that.msg=spec.msg;return that};AgoraRTC.ClientEvent=function(spec){"use strict";var that=AgoraRTC.BasicEvent(spec);that.uid=spec.uid;that.attr=spec.attr;that.streams=spec.streams;return that};AgoraRTC.MediaEvent=function(spec){"use strict";var that=AgoraRTC.BasicEvent(spec);that.msg=spec.msg;return that};AgoraRTC.Signal=function(spec){var that=AgoraRTC.EventDispatcher(spec);that.connection=new WebSocket(spec.url);that.sendMessage=function(obj,onError){if(that.connection.readyState==WebSocket.OPEN){that.connection.send(JSON.stringify(obj))}else{console.log("connection to agent lost.");onError({error:"not connected"})}};that.close=function(){that.onEvent=onEvent;that.connection.onopen=undefined;that.connection.onclose=undefined;that.connection.onerror=undefined;that.connection.onmessage=undefined;that.connection.close()};that.connection.onopen=function(e){console.log(e);that.dispatchEvent(AgoraRTC.MediaEvent({type:"onopen",event:e}))};that.connection.onclose=function(e){console.log(e);AgoraCall(spec.onFailure,e)};that.connection.onerror=function(e){console.log(e);AgoraCall(spec.onFailure,e)};that.onEvent=onEvent;that.connection.onmessage=function(e){console.log(e);var m=JSON.parse(e.data);if(m.hasOwnProperty("command")){that.dispatchEvent(AgoraRTC.MediaEvent({type:m.command,msg:m}))}else if(m.hasOwnProperty("event")){that.onEvent(m)}};function onEvent(e){console.log(e)}return that};AgoraRTC.Stream=function(spec){var that=AgoraRTC.EventDispatcher(spec);that.stream=undefined;that.render=undefined;that.interval=undefined;that.lastRenderCount=0;that.profile=undefined;that.latency=0;var header=undefined;var yplane=undefined;var uplane=undefined;var vplane=undefined;that.init=function(onSuccess,onFailure){if(that.stream){console.warn("stream "+that.getId()+" has been initialized already");onSuccess();return}var videoStreamFileBlob=new Blob([AgoraRTC.videoStreamSource],{type:"application/javascript"});that.stream=new Worker(URL.createObjectURL(videoStreamFileBlob));that.oninit=function(e){console.log((spec.local?"local":"remote")+" Stream ");console.log(e);if(e.result===true){onSuccess()}else{onFailure()}};that.onclose=function(e){window.clearInterval(that.interval);console.log(spec.local?"local":"remote"," Stream ",e);that.stream=undefined};that.onerror=function(e){window.clearInterval(that.interval);console.log(spec.local?"local":"remote"," Stream ",e);onFailure(e)};that.onmessage=function(e){if(typeof e==="string"){console.log(spec.local?"local":"remote"," message from agent ",e)}else if(e instanceof ArrayBuffer){if(that.render){that.drawImage(e)}}else if(e instanceof Blob){console.warn("Blob image data is not supported")}};that.stream.addEventListener("message",onWorkerMessage,false);that.stream.postMessage({type:"init",local:spec.local,url:AgoraRTC.url,videoProfile:that.profile})};that.close=function(){window.clearInterval(that.interval);if(that.stream){that.stream.removeEventListener("message",onWorkerMessage);
that.stream.postMessage({type:"close"})}that.stream=undefined;that.stop()};that.play=function(div,onFailure){var view=document.getElementById(div);that.stop();that.render=AgoraRender();that.render.start(view,onFailure);if(spec.local){that.stream.postMessage({type:"send",message:{command:"preview",uid:spec.streamID,audio:spec.audio===false?false:true,video:spec.video===false?false:true,screen:spec.screen===true?true:false,viewWidth:view.clientWidth,viewHeight:view.clientHeight}})}else{that.stream.postMessage({type:"send",message:{command:"subscribe",uid:spec.streamID,viewWidth:view.clientWidth,viewHeight:view.clientHeight}})}if(that.interval){window.clearInterval(that.interval)}that.interval=window.setInterval(function(){if(that.render){var fps=that.render.renderImageCount-that.lastRenderCount;that.stream.postMessage({type:"send",message:{command:"videoStat",uid:spec.streamID,fps:fps,frameCount:that.lastRenderCount,latency:that.latency}});that.lastRenderCount=that.render.renderImageCount;that.latency=0}},1e3)};that.stop=function(){if(that.render){that.render.stop()}that.render=undefined;that.lastRenderCount=0};that.bindClient=function(client){that.client=client};that.enableAudio=function(callback){return that.client.enableAudio(that,callback)};that.disableAudio=function(callback){return that.client.disableAudio(that,callback)};that.enableVideo=function(callback){return that.client.enableVideo(that,callback)};that.disableVideo=function(callback){return that.client.disableVideo(that,callback)};that.getId=function(){return spec.streamID};that.drawImage=function(data){if(!header){header=data;if(header.byteLength!=20){console.error("invalid image header "+data.byteLength);resetImageData()}return}if(!yplane){yplane=data;if(yplane.byteLength===20){console.error("invalid image header "+data.byteLength+" "+yplane.byteLength);resetImageData()}return}if(!uplane){uplane=data;if(uplane.byteLength===20){console.error("invalid image header "+data.byteLength+" "+yplane.byteLength+" "+uplane.byteLength);resetImageData()}return}if(!vplane){vplane=data;if(yplane.byteLength!=uplane.byteLength*4||uplane.byteLength!=vplane.byteLength){console.error("invalid image header "+data.byteLength+" "+yplane.byteLength+" "+uplane.byteLength+" "+vplane.byteLength);resetImageData();return}var headerLength=20;var dv=new DataView(header);var format=dv.getUint8(0);var mirror=dv.getUint8(1);var width=dv.getUint16(2);var height=dv.getUint16(4);var left=dv.getUint16(6);var top=dv.getUint16(8);var right=dv.getUint16(10);var bottom=dv.getUint16(12);var rotation=dv.getUint16(14);var ts=dv.getUint32(16);var xWidth=width+left+right;var xHeight=height+top+bottom;var yLength=xWidth*xHeight;var yBegin=headerLength;var yEnd=yBegin+yLength;var uLength=yLength/4;var uBegin=yEnd;var uEnd=uBegin+uLength;var vLength=yLength/4;var vBegin=uEnd;var vEnd=vBegin+vLength;that.render.renderImage({mirror:mirror,width:width,height:height,left:left,top:top,right:right,bottom:bottom,rotation:rotation,yplane:new Uint8Array(yplane),uplane:new Uint8Array(uplane),vplane:new Uint8Array(vplane)});var now32=(Date.now()&4294967295)>>>0;var latency=now32-ts;if(latency>that.latency){that.latency=latency}resetImageData()}};that.setVideoProfile=function(profile){that.profile={};if(typeof profile!=="string"&&typeof profile!=="object"){return false}if(profile.hasOwnProperty("profile")){that.profile.profile=profile.profile}if(profile.hasOwnProperty("swapWidthAndHeight")){that.profile.swapWidthAndHeight=profile.swapWidthAndHeight!==false}else{that.profile.swapWidthAndHeight=false}if(typeof profile=="string"){that.profile.profile=profile}if(that.stream){that.stream.postMessage({type:"send",message:{command:"setVideoProfile",profile:that.profile.profile,swapWidthAndHeight:that.profile.swapWidthAndHeight}})}return true};function resetImageData(){header=undefined;yplane=undefined;uplane=undefined;vplane=undefined}function getReso(w,h){return{width:w,height:h}}var supportedVideoList={true:true,unspecified:true,"120p":getReso(160,120),"240p":getReso(320,240),"360p":getReso(640,360),"480p":getReso(640,480),"720p":getReso(1280,720),"1080p":getReso(1920,1080),"4k":getReso(3840,2160)};function onWorkerMessage(e){var data=e.data;switch(data.type){case"init":that.oninit(data);break;case"close":that.onclose(data);break;case"error":that.onerror(data);break;case"message":that.onmessage(data.data);break}}return that};AgoraRTC.createStream=function(spec){return AgoraRTC.Stream(spec)};AgoraRTC.Client=function(spec){"use strict";var that=AgoraRTC.EventDispatcher(spec);that.signal=undefined;that.localStream=undefined;that.remoteStreams={};that.signalOpen=false;function onSignalError(e){if(that.signalOpen){console.log("signal connection lost.");var message={reason:"LOST_CONNECTION_TO_AGENT",type:"error",eventType:e.type,code:e.code};AgoraCall(that.onFailure,message);that.dispatchEvent(message);return}if(e.type==="error"){return}var message={reason:"CLOSE_BEFORE_OPEN",type:"error",eventType:e.type,code:e.code,agentInstallUrl:getInstallUrl(),agentInstallGuideUrl:getInstallGuideUrl()};AgoraCall(that.onFailure,message);that.dispatchEvent(message)}that.init=function(appId,onSuccess,onFailure){spec.appId=appId;that.onFailure=onFailure;if(!that.signal){try{that.signal=AgoraRTC.Signal({url:AgoraRTC.url,onFailure:onSignalError})}catch(e){console.log("create signal connection failed"+e);AgoraCall(onFailure,e);return}that.signal.on("onopen",function(event){that.signalOpen=true;that.signal.onEvent=function(e){if(e.event==="error"){var name=getErrorCodeName(e.code);if(name){e.reason=name}}else if(e.event==="onUserMuteAudio"){e.event="peer-mute-audio"}else if(e.event==="onUserMuteVideo"){e.event="peer-mute-video"}that.dispatchEvent(AgoraRTC.MediaEvent({type:e.event,msg:e}))};that.getVersion(function(ev){if(ev.version=="1.6.0"||ev.version=="1.6.1"||ev.version=="1.7.0"){initialize(onSuccess,onFailure);return}var message={reason:"INCOMPATIBLE_WEBAGENT",type:"error",version:AgoraRTC.version,agentVersion:ev.version,agentInstallUrl:getInstallUrl(),agentInstallGuideUrl:getInstallGuideUrl()};that.dispatchEvent(message)},function(ev){var message={reason:"INCOMPATIBLE_WEBAGENT",type:"error",version:AgoraRTC.version,agentVersion:ev.version,agentInstallUrl:getInstallUrl(),agentInstallGuideUrl:getInstallGuideUrl()};that.dispatchEvent(message)})})}if(that.signalOpen){initialize(onSuccess,onFailure)}function initialize(onSuccess,onFailure){that.signal.on("initialize",function(ev){var m=ev.msg;if(m.code!=true){that.signal.close();that.signal=undefined;onError(m.code);return}AgoraCall(onSuccess,m)});that.signal.sendMessage({command:"initialize",appId:spec.appId},function(err){AgoraCall(onFailure,err)})}function onError(code){var name=getErrorCodeName(code);if(name){AgoraCall(onFailure,{reason:name});return}AgoraCall(onFailure,{reason:"UNKNOWN_ERROR",code:m.code});return}function getErrorCodeName(code){for(var k in L.ErrorCode){if(L.ErrorCode.hasOwnProperty(k)){if(L.ErrorCode[k]===code){return String(k)}}}return null}that.signal.on("onError",function(ev){var m=ev.msg;onError(m.code)});console.log(that.signal)};that.renewChannelKey=function(channelKey,onSuccess,onFailure){that.signal.sendMessage({command:"renewChannelKey",channelKey:channelKey},function(e){AgoraCall(onFailure,e)});that.signal.on("renewChannelKey",function(ev){var m=ev.msg;if(m.code===true){AgoraCall(onSuccess,m);return}AgoraCall(onFailure,m)})};that.join=function(channelKey,channelName,uid,onSuccess,onFailure){if(channelName.length>64){onFailure(L.ErrorCode.INVALID_CHANNEL_NAME);return}that.signal.sendMessage({command:"joinChannel",channelKey:channelKey,channelName:channelName,uid:uid},function(e){AgoraCall(onFailure,{error:e})});that.signal.on("joinChannel",function(ev){that.signal.on("onAddVideoStream",onAddVideoStream);that.signal.on("onPeerLeave",onPeerLeave);var m=ev.msg;if(m.code==true){if(!m.uid){m.uid=0}AgoraCall(onSuccess,m.uid);return}if(m.code==L.ErrorCode.JOIN_CHANNEL_REJECTED){console.error("Command joinChannel has been rejected by agent. Is this user joined a channel already?");AgoraCall(onFailure,m.code);return}AgoraCall(onFailure,m.code)})};that.leave=function(onSuccess,onFailure){closeLocalRemoteStreams();that.signal.sendMessage({command:"leaveChannel"},function(e){console.log("leave channel failed",e)});that.signal.on("leaveChannel",function(ev){var m=ev.msg;if(m.code===true){AgoraCall(onSuccess,m);return}if(m.code==L.ErrorCode.LEAVE_CHANNEL_REJECTED){console.error("Command leaveChannel has been rejected by agent. Is this user not in a channel?");AgoraCall(onSuccess,m);return}AgoraCall(onFailure,{code:m.code})})};that.publish=function(stream,onSuccess,onFailure){that.localStream=stream;that.signal.sendMessage({command:"unmuteLocal"},function(e){AgoraCall(onFailure,{error:e})});that.signal.on("unmuteLocal",function(ev){var m=ev.msg;if(m.code!=true){AgoraCall(onFailure,{code:m.code});return}AgoraCall(onSuccess,m)});stream.bindClient(that)};that.unpublish=function(stream,onSuccess,onFailure){that.signal.sendMessage({command:"muteLocal"},function(e){AgoraCall(onFailure,{error:e})});that.signal.on("muteLocal",function(ev){var m=ev.msg;if(m.code!=true){AgoraCall(onFailure,{code:m.code});return}AgoraCall(onSuccess,m)})};that.subscribe=function(stream,onFailure){stream.init(function(){that.dispatchEvent(AgoraRTC.StreamEvent({type:"stream-subscribed",stream:stream}))},function(error){AgoraCall(onFailure,error)})};that.unsubscribe=function(streamID,onFailure){console.log("remote streams",that.remoteStreams);if(that.remoteStreams[streamID]==undefined){AgoraCall(onFailure,{error:"no such stream"});return}that.remoteStreams[streamID].close();delete that.remoteStreams[streamID]};that.enableAudio=function(stream,callback){that.signal.on("enableAudio",function(ev){var m=ev.msg;if(callback){AgoraCall(callback,m.code===true)}});that.signal.sendMessage({command:"enableAudio",streamID:stream.getId()},function(e){e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(callback,e)});return true};that.disableAudio=function(stream,callback){that.signal.on("disableAudio",function(ev){var m=ev.msg;if(callback){AgoraCall(callback,m.code===true)}});that.signal.sendMessage({command:"disableAudio",streamID:stream.getId()},function(e){e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(callback,e)});return true};that.enableVideo=function(stream,callback){that.signal.on("enableVideo",function(ev){var m=ev.msg;if(callback){AgoraCall(callback,m.code===true)}});that.signal.sendMessage({command:"enableVideo",streamID:stream.getId()},function(e){e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(callback,e)});return true};that.disableVideo=function(stream,callback){that.signal.on("disableVideo",function(ev){var m=ev.msg;if(callback){AgoraCall(callback,m.code===true)}});that.signal.sendMessage({command:"disableVideo",streamID:stream.getId()},function(e){e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(callback,e)});return true};that.getDevices=function(callback){that.signal.sendMessage({command:"getDevices"},function(e){e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(callback,e)});that.signal.on("getDevices",function(ev){var devices=ev.msg.devices;AgoraCall(callback,devices)})};that.selectDevice=function(device,onFailure){that.signal.sendMessage({command:"selectDevice",device:device},function(e){e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(onFailure,e)});that.signal.on("selectDevice",function(ev){console.log(ev)})};that.startRecording=function(recordingKey,onSuccess,onFailure){that.signal.sendMessage({command:"startRecording",recordingKey:recordingKey},function(e){e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(onFailure,e)});that.signal.on("startRecording",function(ev){var start=ev.msg;if(start.code===true){AgoraCall(onSuccess,start)}else{AgoraCall(onFailure,start)}})};that.stopRecording=function(recordingKey,onSuccess,onFailure){that.signal.sendMessage({command:"stopRecording",recordingKey:recordingKey},function(e){e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(onFailure,e)});that.signal.on("stopRecording",function(ev){var stop=ev.msg;if(stop.code===true){AgoraCall(onSuccess,stop)}else{AgoraCall(onFailure,stop)}})};that.queryRecordingStatus=function(onStatus){that.signal.sendMessage({command:"queryRecordingStatus"},function(e){e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(onStatus,e)});that.signal.on("queryRecordingStatus",function(ev){onStatus(ev.msg)})};that.close=function(){closeLocalRemoteStreams();if(that.signal){that.signal.close();that.signal=undefined}};that.getVersion=function(onSuccess,onFailure){that.signal.sendMessage({command:"getVersion"});that.signal.on("getVersion",function(ev){var m=ev.msg;if(m.code===true){onSuccess(m)}else{onFailure(m)}})};that.setParameters=function(parameters){that.signal.sendMessage({command:"setParameters",parameters:JSON.stringify(parameters)})};that.setEncryptionMode=function(encryptionMode){that.signal.sendMessage({command:"setEncryptionMode",mode:encryptionMode})};that.setEncryptionSecret=function(secret){that.signal.sendMessage({command:"setEncryptionSecret",secret:secret})};that.getWindows=function(callback){that.signal.sendMessage({command:"getWindows"},function(e){e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(callback,e)});that.signal.on("getWindows",function(ev){var windows=ev.msg.windows;AgoraCall(callback,windows)})};that.startScreenSharing=function(window,onSuccess,onFailure){that.signal.sendMessage({command:"startScreenSharing",window:window},function(e){e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(onFailure,e)});that.signal.on("startScreenSharing",function(e){AgoraCall(onSuccess,e)})};that.setScreenSharingWindow=function(window,onSuccess,onFailure){that.signal.sendMessage({command:"setScreenSharingWindow",window:window},function(e){e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(onFailure,e)});that.signal.on("setScreenSharingWindow",function(e){AgoraCall(onSuccess,e);console.log(e)})};that.stopScreenSharing=function(onSuccess,onFailure){that.signal.sendMessage({command:"stopScreenSharing"},function(e){e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(onFailure,e)});that.signal.on("stopScreenSharing",function(e){AgoraCall(onSuccess,e);console.log(e)})};function onPeerLeave(ev){var m=ev.msg;if(that.remoteStreams.hasOwnProperty(m.uid)){that.remoteStreams[m.uid].close();delete that.remoteStreams[m.uid];console.log("remote streams after peer leave ",that.remoteStreams)}that.dispatchEvent(AgoraRTC.ClientEvent({type:"peer-leave",uid:m.uid}))}function onAddVideoStream(ev){var m=ev.msg;if(!that.remoteStreams.hasOwnProperty(m.uid)){var stream=AgoraRTC.Stream({streamID:m.uid,local:false,audio:m.audio,video:m.video,screen:m.screen});stream.bindClient(that);that.remoteStreams[m.uid]=stream}that.dispatchEvent(AgoraRTC.StreamEvent({type:"stream-added",stream:that.remoteStreams[m.uid]}));console.log("remote streams",that.remoteStreams)}function getInstallUrl(){if(navigator.appVersion.indexOf("Mac")!=-1){return AgoraRTC.macAgentInstallUrl}return AgoraRTC.winAgentInstallUrl}function getInstallGuideUrl(){var userLang=navigator.language||navigator.userLanguage;if(userLang.indexOf("zh")!=-1){return AgoraRTC.cnAgentInstallGuideUrl}return AgoraRTC.enAgentInstallGuideUrl}function closeLocalRemoteStreams(){if(that.localStream){that.localStream.close()}that.localStream=undefined;for(var streamId in that.remoteStreams){if(that.remoteStreams.hasOwnProperty(streamId)){that.remoteStreams[streamId].close();delete that.remoteStreams[streamId]}}}return that};AgoraRTC.createRtcClient=function(){return AgoraRTC.Client({})};
//# sourceMappingURL=dist/AgoraRtcAgentSDK.min.js.map
"use strict";(function(root,factory){if(typeof define==="function"&&define.amd){define([],factory)}else{var lib=factory.call(root);Object.keys(lib).forEach(function(key){root[key]=lib[key]})}})(this,function(){var topWindow=this;function error(msg){if(topWindow.console){if(topWindow.console.error){topWindow.console.error(msg)}else if(topWindow.console.log){topWindow.console.log(msg)}}}function isInIFrame(w){w=w||topWindow;return w!==w.top}function makeFailHTML(msg){return""+'<table style="background-color: #8CE; width: 100%; height: 100%;"><tr>'+'<td align="center">'+'<div style="display: table-cell; vertical-align: middle;">'+'<div style="">'+msg+"</div>"+"</div>"+"</td></tr></table>"}var GET_A_WEBGL_BROWSER=""+"This page requires a browser that supports WebGL.<br/>"+'<a href="http://get.webgl.org">Click here to upgrade your browser.</a>';var OTHER_PROBLEM=""+"It doesn't appear your computer can support WebGL.<br/>"+'<a href="http://get.webgl.org/troubleshooting/">Click here for more information.</a>';function create3DContext(canvas,opt_attribs){var names=["webgl","experimental-webgl"];var context=null;for(var ii=0;ii<names.length;++ii){try{context=canvas.getContext(names[ii],opt_attribs)}catch(e){}if(context){break}}return context}function setupWebGL(canvas,opt_attribs){function showLink(str){var container=canvas.parentNode;if(container){container.innerHTML=makeFailHTML(str)}}if(!topWindow.WebGLRenderingContext){showLink(GET_A_WEBGL_BROWSER);return null}var context=create3DContext(canvas,opt_attribs);if(!context){showLink(OTHER_PROBLEM)}return context}function updateCSSIfInIFrame(){if(isInIFrame()){document.body.className="iframe"}}function getWebGLContext(canvas,opt_attribs,opt_options){var options=opt_options||{};if(isInIFrame()){updateCSSIfInIFrame();if(!options.dontResize&&options.resize!==false){var width=canvas.clientWidth;var height=canvas.clientHeight;canvas.width=width;canvas.height=height}}else if(!options.noTitle&&options.title!==false){var title=document.title;var h1=document.createElement("h1");h1.innerText=title;document.body.insertBefore(h1,document.body.children[0])}var gl=setupWebGL(canvas,opt_attribs);return gl}function loadShader(gl,shaderSource,shaderType,opt_errorCallback){var errFn=opt_errorCallback||error;var shader=gl.createShader(shaderType);gl.shaderSource(shader,shaderSource);gl.compileShader(shader);var compiled=gl.getShaderParameter(shader,gl.COMPILE_STATUS);if(!compiled){var lastError=gl.getShaderInfoLog(shader);errFn("*** Error compiling shader '"+shader+"':"+lastError);gl.deleteShader(shader);return null}return shader}function createProgram(gl,shaders,opt_attribs,opt_locations,opt_errorCallback){var errFn=opt_errorCallback||error;var program=gl.createProgram();shaders.forEach(function(shader){gl.attachShader(program,shader)});if(opt_attribs){obj_attrib.forEach(function(attrib,ndx){gl.bindAttribLocation(program,opt_locations?opt_locations[ndx]:ndx,attrib)})}gl.linkProgram(program);var linked=gl.getProgramParameter(program,gl.LINK_STATUS);if(!linked){var lastError=gl.getProgramInfoLog(program);errFn("Error in program linking:"+lastError);gl.deleteProgram(program);return null}return program}function createShaderFromScript(gl,scriptId,opt_shaderType,opt_errorCallback){var shaderSource="";var shaderType;var shaderScript=document.getElementById(scriptId);if(!shaderScript){throw"*** Error: unknown script element"+scriptId}shaderSource=shaderScript.text;if(!opt_shaderType){if(shaderScript.type==="x-shader/x-vertex"){shaderType=gl.VERTEX_SHADER}else if(shaderScript.type==="x-shader/x-fragment"){shaderType=gl.FRAGMENT_SHADER}else if(shaderType!==gl.VERTEX_SHADER&&shaderType!==gl.FRAGMENT_SHADER){throw"*** Error: unknown shader type"}}return loadShader(gl,shaderSource,opt_shaderType?opt_shaderType:shaderType,opt_errorCallback)}var defaultShaderType=["VERTEX_SHADER","FRAGMENT_SHADER"];function createProgramFromScripts(gl,shaderScriptIds,opt_attribs,opt_locations,opt_errorCallback){var shaders=[];for(var ii=0;ii<shaderScriptIds.length;++ii){shaders.push(createShaderFromScript(gl,shaderScriptIds[ii],gl[defaultShaderType[ii]],opt_errorCallback))}return createProgram(gl,shaders,opt_attribs,opt_locations,opt_errorCallback)}function createProgramFromSources(gl,shaderSources,opt_attribs,opt_locations,opt_errorCallback){var shaders=[];for(var ii=0;ii<shaderSources.length;++ii){shaders.push(loadShader(gl,shaderSources[ii],gl[defaultShaderType[ii]],opt_errorCallback))}return createProgram(gl,shaders,opt_attribs,opt_locations,opt_errorCallback)}function getBindPointForSamplerType(gl,type){if(type===gl.SAMPLER_2D)return gl.TEXTURE_2D;if(type===gl.SAMPLER_CUBE)return gl.TEXTURE_CUBE_MAP}function createUniformSetters(gl,program){var textureUnit=0;function createUniformSetter(program,uniformInfo){var location=gl.getUniformLocation(program,uniformInfo.name);var type=uniformInfo.type;var isArray=uniformInfo.size>1&&uniformInfo.name.substr(-3)==="[0]";if(type===gl.FLOAT&&isArray){return function(v){gl.uniform1fv(location,v)}}if(type===gl.FLOAT){return function(v){gl.uniform1f(location,v)}}if(type===gl.FLOAT_VEC2){return function(v){gl.uniform2fv(location,v)}}if(type===gl.FLOAT_VEC3){return function(v){gl.uniform3fv(location,v)}}if(type===gl.FLOAT_VEC4){return function(v){gl.uniform4fv(location,v)}}if(type===gl.INT&&isArray){return function(v){gl.uniform1iv(location,v)}}if(type===gl.INT){return function(v){gl.uniform1i(location,v)}}if(type===gl.INT_VEC2){return function(v){gl.uniform2iv(location,v)}}if(type===gl.INT_VEC3){return function(v){gl.uniform3iv(location,v)}}if(type===gl.INT_VEC4){return function(v){gl.uniform4iv(location,v)}}if(type===gl.BOOL){return function(v){gl.uniform1iv(location,v)}}if(type===gl.BOOL_VEC2){return function(v){gl.uniform2iv(location,v)}}if(type===gl.BOOL_VEC3){return function(v){gl.uniform3iv(location,v)}}if(type===gl.BOOL_VEC4){return function(v){gl.uniform4iv(location,v)}}if(type===gl.FLOAT_MAT2){return function(v){gl.uniformMatrix2fv(location,false,v)}}if(type===gl.FLOAT_MAT3){return function(v){gl.uniformMatrix3fv(location,false,v)}}if(type===gl.FLOAT_MAT4){return function(v){gl.uniformMatrix4fv(location,false,v)}}if((type===gl.SAMPLER_2D||type===gl.SAMPLER_CUBE)&&isArray){var units=[];for(var ii=0;ii<info.size;++ii){units.push(textureUnit++)}return function(bindPoint,units){return function(textures){gl.uniform1iv(location,units);textures.forEach(function(texture,index){gl.activeTexture(gl.TEXTURE0+units[index]);gl.bindTexture(bindPoint,texture)})}}(getBindPointForSamplerType(gl,type),units)}if(type===gl.SAMPLER_2D||type===gl.SAMPLER_CUBE){return function(bindPoint,unit){return function(texture){gl.uniform1i(location,unit);gl.activeTexture(gl.TEXTURE0+unit);gl.bindTexture(bindPoint,texture)}}(getBindPointForSamplerType(gl,type),textureUnit++)}throw"unknown type: 0x"+type.toString(16)}var uniformSetters={};var numUniforms=gl.getProgramParameter(program,gl.ACTIVE_UNIFORMS);for(var ii=0;ii<numUniforms;++ii){var uniformInfo=gl.getActiveUniform(program,ii);if(!uniformInfo){break}var name=uniformInfo.name;if(name.substr(-3)==="[0]"){name=name.substr(0,name.length-3)}var setter=createUniformSetter(program,uniformInfo);uniformSetters[name]=setter}return uniformSetters}function setUniforms(setters,values){Object.keys(values).forEach(function(name){var setter=setters[name];if(setter){setter(values[name])}})}function createAttributeSetters(gl,program){var attribSetters={};function createAttribSetter(index){return function(b){gl.bindBuffer(gl.ARRAY_BUFFER,b.buffer);gl.enableVertexAttribArray(index);gl.vertexAttribPointer(index,b.numComponents||b.size,b.type||gl.FLOAT,b.normalize||false,b.stride||0,b.offset||0)}}var numAttribs=gl.getProgramParameter(program,gl.ACTIVE_ATTRIBUTES);for(var ii=0;ii<numAttribs;++ii){var attribInfo=gl.getActiveAttrib(program,ii);if(!attribInfo){break}var index=gl.getAttribLocation(program,attribInfo.name);attribSetters[attribInfo.name]=createAttribSetter(index)}return attribSetters}function setAttributes(setters,buffers){Object.keys(buffers).forEach(function(name){var setter=setters[name];if(setter){setter(buffers[name])}})}function createProgramInfo(gl,shaderSources,opt_attribs,opt_locations,opt_errorCallback){shaderSources=shaderSources.map(function(source){var script=document.getElementById(source);return script?script.text:source});var program=createProgramFromSources(gl,shaderSources,opt_attribs,opt_locations,opt_errorCallback);if(!program){return null}var uniformSetters=createUniformSetters(gl,program);var attribSetters=createAttributeSetters(gl,program);return{program:program,uniformSetters:uniformSetters,attribSetters:attribSetters}}function setBuffersAndAttributes(gl,setters,buffers){setAttributes(setters,buffers.attribs);if(buffers.indices){gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER,buffers.indices)}}var browserPrefixes=["","MOZ_","OP_","WEBKIT_"];function getExtensionWithKnownPrefixes(gl,name){for(var ii=0;ii<browserPrefixes.length;++ii){var prefixedName=browserPrefixes[ii]+name;var ext=gl.getExtension(prefixedName);if(ext){return ext}}}function resizeCanvasToDisplaySize(canvas,multiplier){multiplier=multiplier||1;var width=canvas.clientWidth*multiplier;var height=canvas.clientHeight*multiplier;if(canvas.width!==width||canvas.height!==height){canvas.width=width;canvas.height=height;return true}return false}function getIFrameForWindow(window){if(!isInIFrame(window)){return}var iframes=window.parent.document.getElementsByTagName("iframe");for(var ii=0;ii<iframes.length;++ii){var iframe=iframes[ii];if(iframe.contentDocument===window.document){return iframe}}}function isFrameVisible(window){try{var iframe=getIFrameForWindow(window);if(!iframe){return true}var bounds=iframe.getBoundingClientRect();var isVisible=bounds.top<window.parent.innerHeight&&bounds.bottom>=0&&bounds.left<window.parent.innerWidth&&bounds.right>=0;return isVisible&&isFrameVisible(window.parent)}catch(e){return true}}function isOnScreen(element){var isVisible=true;if(element){var bounds=element.getBoundingClientRect();isVisible=bounds.top<topWindow.innerHeight&&bounds.bottom>=0}return isVisible&&isFrameVisible(topWindow)}function augmentTypedArray(typedArray,numComponents){var cursor=0;typedArray.push=function(){for(var ii=0;ii<arguments.length;++ii){var value=arguments[ii];if(value instanceof Array||value.buffer&&value.buffer instanceof ArrayBuffer){for(var jj=0;jj<value.length;++jj){typedArray[cursor++]=value[jj]}}else{typedArray[cursor++]=value}}};typedArray.reset=function(opt_index){cursor=opt_index||0};typedArray.numComponents=numComponents;Object.defineProperty(typedArray,"numElements",{get:function(){return this.length/this.numComponents|0}});return typedArray}function createAugmentedTypedArray(numComponents,numElements,opt_type){var Type=opt_type||Float32Array;return augmentTypedArray(new Type(numComponents*numElements),numComponents)}function createBufferFromTypedArray(gl,array,type,drawType){type=type||gl.ARRAY_BUFFER;var buffer=gl.createBuffer();gl.bindBuffer(type,buffer);gl.bufferData(type,array,drawType||gl.STATIC_DRAW);return buffer}function allButIndices(name){return name!=="indices"}function createMapping(obj){var mapping={};Object.keys(obj).filter(allButIndices).forEach(function(key){mapping["a_"+key]=key});return mapping}function getGLTypeForTypedArray(gl,typedArray){if(typedArray instanceof Int8Array){return gl.BYTE}if(typedArray instanceof Uint8Array){return gl.UNSIGNED_BYTE}if(typedArray instanceof Int16Array){return gl.SHORT}if(typedArray instanceof Uint16Array){return gl.UNSIGNED_SHORT}if(typedArray instanceof Int32Array){return gl.INT}if(typedArray instanceof Uint32Array){return gl.UNSIGNED_INT}if(typedArray instanceof Float32Array){return gl.FLOAT}throw"unsupported typed array type"}function getNormalizationForTypedArray(typedArray){if(typedArray instanceof Int8Array){return true}if(typedArray instanceof Uint8Array){return true}return false}function isArrayBuffer(a){return a.buffer&&a.buffer instanceof ArrayBuffer}function guessNumComponentsFromName(name,length){var numComponents;if(name.indexOf("coord")>=0){numComponents=2}else if(name.indexOf("color")>=0){numComponents=4}else{numComponents=3}if(length%numComponents>0){throw"can not guess numComponents. You should specify it."}return numComponents}function makeTypedArray(array,name){if(isArrayBuffer(array)){return array}if(Array.isArray(array)){array={data:array}}if(!array.numComponents){array.numComponents=guessNumComponentsFromName(name,array.length)}var type=array.type;if(!type){if(name==="indices"){type=Uint16Array}}var typedArray=createAugmentedTypedArray(array.numComponents,array.data.length/array.numComponents|0,type);typedArray.push(array.data);return typedArray}function createAttribsFromArrays(gl,arrays,opt_mapping){var mapping=opt_mapping||createMapping(arrays);var attribs={};Object.keys(mapping).forEach(function(attribName){var bufferName=mapping[attribName];var array=makeTypedArray(arrays[bufferName],bufferName);attribs[attribName]={buffer:createBufferFromTypedArray(gl,array),numComponents:array.numComponents||guessNumComponentsFromName(bufferName),type:getGLTypeForTypedArray(gl,array),normalize:getNormalizationForTypedArray(array)}});return attribs}function getNumElementsFromNonIndexedArrays(arrays){var key=Object.keys(arrays)[0];var array=arrays[key];if(isArrayBuffer(array)){return array.numElements}else{return array.data.length/array.numComponents}}function createBufferInfoFromArrays(gl,arrays,opt_mapping){var bufferInfo={attribs:createAttribsFromArrays(gl,arrays,opt_mapping)};var indices=arrays.indices;if(indices){indices=makeTypedArray(indices,"indices");bufferInfo.indices=createBufferFromTypedArray(gl,indices,gl.ELEMENT_ARRAY_BUFFER);bufferInfo.numElements=indices.length}else{bufferInfo.numElements=getNumElementsFromNonIndexedArrays(arrays)}return bufferInfo}function createBuffersFromArrays(gl,arrays){var buffers={};Object.keys(arrays).forEach(function(key){var type=key==="indices"?gl.ELEMENT_ARRAY_BUFFER:gl.ARRAY_BUFFER;var array=makeTypedArray(arrays[key],name);buffers[key]=createBufferFromTypedArray(gl,array,type)});if(arrays.indices){buffers.numElements=arrays.indices.length}else if(arrays.position){buffers.numElements=arrays.position.length/3}return buffers}function drawBufferInfo(gl,type,bufferInfo,count,offset){var indices=bufferInfo.indices;var numElements=count===undefined?bufferInfo.numElements:count;offset=offset===undefined?offset:0;if(indices){gl.drawElements(type,numElements,gl.UNSIGNED_SHORT,offset)}else{gl.drawArrays(type,offset,numElements)}}function drawObjectList(gl,objectsToDraw){var lastUsedProgramInfo=null;var lastUsedBufferInfo=null;objectsToDraw.forEach(function(object){var programInfo=object.programInfo;var bufferInfo=object.bufferInfo;var bindBuffers=false;if(programInfo!==lastUsedProgramInfo){lastUsedProgramInfo=programInfo;gl.useProgram(programInfo.program);bindBuffers=true}if(bindBuffers||bufferInfo!==lastUsedBufferInfo){lastUsedBufferInfo=bufferInfo;setBuffersAndAttributes(gl,programInfo.attribSetters,bufferInfo)}setUniforms(programInfo.uniformSetters,object.uniforms);drawBufferInfo(gl,gl.TRIANGLES,bufferInfo)})}return{createAugmentedTypedArray:createAugmentedTypedArray,createAttribsFromArrays:createAttribsFromArrays,createBuffersFromArrays:createBuffersFromArrays,createBufferInfoFromArrays:createBufferInfoFromArrays,createAttributeSetters:createAttributeSetters,createProgram:createProgram,createProgramFromScripts:createProgramFromScripts,createProgramFromSources:createProgramFromSources,createProgramInfo:createProgramInfo,createUniformSetters:createUniformSetters,drawBufferInfo:drawBufferInfo,drawObjectList:drawObjectList,getWebGLContext:getWebGLContext,updateCSSIfInIFrame:updateCSSIfInIFrame,getExtensionWithKnownPrefixes:getExtensionWithKnownPrefixes,resizeCanvasToDisplaySize:resizeCanvasToDisplaySize,setAttributes:setAttributes,setBuffersAndAttributes:setBuffersAndAttributes,setUniforms:setUniforms,setupWebGL:setupWebGL}});function AgoraRender(){var gl=undefined;var program=undefined;var positionLocation=undefined;var texCoordLocation=undefined;var yTexture=undefined;var uTexture=undefined;var vTexture=undefined;var texCoordBuffer=undefined;var surfaceBuffer=undefined;var that={view:undefined,mirrorView:false,container:undefined,canvas:undefined,renderImageCount:0,initWidth:0,initHeight:0,initRotation:0,canvasUpdated:false,clientWidth:0,clientHeight:0};that.start=function(view,onFailure){initCanvas(view,that.mirrorView,view.clientWidth,view.clientHeight,that.initRotation,onFailure)};that.stop=function(){gl=undefined;program=undefined;positionLocation=undefined;texCoordLocation=undefined;deleteTexture(yTexture);deleteTexture(uTexture);deleteTexture(vTexture);yTexture=undefined;uTexture=undefined;vTexture=undefined;deleteBuffer(texCoordBuffer);deleteBuffer(surfaceBuffer);texCoordBuffer=undefined;surfaceBuffer=undefined;if(that.container){that.container.removeChild(that.canvas)}if(that.view){that.view.removeChild(that.container)}that.canvas=undefined;that.container=undefined;that.view=undefined;that.mirrorView=false};that.renderImage=function(image){if(!gl){return}if(image.width!=that.initWidth||image.height!=that.initHeight||image.rotation!=that.initRotation||image.mirror!=that.mirrorView){var view=that.view;that.stop();console.log("init canvas "+image.width+"*"+image.height+" rotation "+image.rotation);initCanvas(view,image.mirror,image.width,image.height,image.rotation,function(e){console.error("init canvas "+image.width+"*"+image.height+" rotation "+image.rotation+" failed. "+e)})}gl.bindBuffer(gl.ARRAY_BUFFER,texCoordBuffer);var xWidth=image.width+image.left+image.right;var xHeight=image.height+image.top+image.bottom;gl.bufferData(gl.ARRAY_BUFFER,new Float32Array([image.left/xWidth,image.bottom/xHeight,1-image.right/xWidth,image.bottom/xHeight,image.left/xWidth,1-image.top/xHeight,image.left/xWidth,1-image.top/xHeight,1-image.right/xWidth,image.bottom/xHeight,1-image.right/xWidth,1-image.top/xHeight]),gl.STATIC_DRAW);gl.enableVertexAttribArray(texCoordLocation);gl.vertexAttribPointer(texCoordLocation,2,gl.FLOAT,false,0,0);uploadYuv(xWidth,xHeight,image.yplane,image.uplane,image.vplane);updateCanvas(image.rotation,image.width,image.height);gl.drawArrays(gl.TRIANGLES,0,6);that.renderImageCount+=1};function uploadYuv(width,height,yplane,uplane,vplane){gl.activeTexture(gl.TEXTURE0);gl.bindTexture(gl.TEXTURE_2D,yTexture);gl.texImage2D(gl.TEXTURE_2D,0,gl.LUMINANCE,width,height,0,gl.LUMINANCE,gl.UNSIGNED_BYTE,yplane);var e=gl.getError();if(e!=gl.NO_ERROR){console.log("upload y plane ",width,height,yplane.byteLength," error",e)}gl.activeTexture(gl.TEXTURE1);gl.bindTexture(gl.TEXTURE_2D,uTexture);gl.texImage2D(gl.TEXTURE_2D,0,gl.LUMINANCE,width/2,height/2,0,gl.LUMINANCE,gl.UNSIGNED_BYTE,uplane);var e=gl.getError();if(e!=gl.NO_ERROR){console.log("upload u plane ",width,height,uplane.byteLength,"  error",e)}gl.activeTexture(gl.TEXTURE2);gl.bindTexture(gl.TEXTURE_2D,vTexture);gl.texImage2D(gl.TEXTURE_2D,0,gl.LUMINANCE,width/2,height/2,0,gl.LUMINANCE,gl.UNSIGNED_BYTE,vplane);var e=gl.getError();if(e!=gl.NO_ERROR){console.log("upload v plane ",width,height,vplane.byteLength,"  error",e)}}function deleteBuffer(buffer){if(buffer&&that.gl){that.gl.deleteBuffer(buffer)}}function deleteTexture(texture){if(texture&&that.gl){that.gl.deleteTexture(texture)}}function initCanvas(view,mirror,width,height,rotation,onFailure){that.clientWidth=view.clientWidth;that.clientHeight=view.clientHeight;that.view=view;that.mirrorView=mirror;that.canvasUpdated=false;that.container=document.createElement("div");that.container.style.width="100%";that.container.style.height="100%";that.container.style.display="flex";that.container.style.justifyContent="center";that.container.style.alignItems="center";that.view.appendChild(that.container);that.canvas=document.createElement("canvas");if(rotation==0||rotation==180){that.canvas.width=width;that.canvas.height=height}else{that.canvas.width=height;that.canvas.height=width}that.initWidth=width;that.initHeight=height;that.initRotation=rotation;if(that.mirrorView){that.canvas.style.transform="rotateY(180deg)"}that.container.appendChild(that.canvas);try{gl=that.canvas.getContext("webgl")||that.canvas.getContext("experimental-webgl")}catch(e){console.log(e)}if(!gl){gl=undefined;onFailure({error:"Browser not support! No WebGL detected."});return}gl.clearColor(0,0,0,1);gl.enable(gl.DEPTH_TEST);gl.depthFunc(gl.LEQUAL);gl.clear(gl.COLOR_BUFFER_BIT|gl.DEPTH_BUFFER_BIT);program=createProgramFromSources(gl,[AgoraRTC.vertexShaderSource,AgoraRTC.yuvShaderSource]);gl.useProgram(program);initTextures()}function initTextures(){positionLocation=gl.getAttribLocation(program,"a_position");texCoordLocation=gl.getAttribLocation(program,"a_texCoord");surfaceBuffer=gl.createBuffer();texCoordBuffer=gl.createBuffer();gl.activeTexture(gl.TEXTURE0);yTexture=gl.createTexture();gl.bindTexture(gl.TEXTURE_2D,yTexture);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_WRAP_S,gl.CLAMP_TO_EDGE);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_WRAP_T,gl.CLAMP_TO_EDGE);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_MIN_FILTER,gl.NEAREST);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_MAG_FILTER,gl.NEAREST);gl.activeTexture(gl.TEXTURE1);uTexture=gl.createTexture();gl.bindTexture(gl.TEXTURE_2D,uTexture);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_WRAP_S,gl.CLAMP_TO_EDGE);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_WRAP_T,gl.CLAMP_TO_EDGE);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_MIN_FILTER,gl.NEAREST);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_MAG_FILTER,gl.NEAREST);gl.activeTexture(gl.TEXTURE2);vTexture=gl.createTexture();gl.bindTexture(gl.TEXTURE_2D,vTexture);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_WRAP_S,gl.CLAMP_TO_EDGE);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_WRAP_T,gl.CLAMP_TO_EDGE);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_MIN_FILTER,gl.NEAREST);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_MAG_FILTER,gl.NEAREST);var y=gl.getUniformLocation(program,"Ytex");gl.uniform1i(y,0);var u=gl.getUniformLocation(program,"Utex");gl.uniform1i(u,1);var v=gl.getUniformLocation(program,"Vtex");gl.uniform1i(v,2)}function updateCanvas(rotation,width,height){if(that.canvasUpdated){return}that.canvas.style.width="100%";that.canvas.style.height="100%";try{if(rotation===0||rotation===180){if(that.clientWidth/that.clientHeight>width/height){that.canvas.style.width=that.clientHeight*width/height+"px"}else if(that.clientWidth/that.clientHeight<width/height){that.canvas.style.height=that.clientWidth*height/width+"px"}}else{if(that.clientHeight/that.clientWidth>width/height){that.canvas.style.height=that.clientWidth*width/height+"px"}else if(that.clientHeight/that.clientWidth<width/height){that.canvas.style.width=that.clientHeight*height/width+"px"}}}catch(e){console.log("updateCanvas 00001 gone "+that.canvas);console.log(that);console.error(e);return}gl.bindBuffer(gl.ARRAY_BUFFER,surfaceBuffer);gl.enableVertexAttribArray(positionLocation);gl.vertexAttribPointer(positionLocation,2,gl.FLOAT,false,0,0);var p1={x:0,y:0};var p2={x:width,y:0};var p3={x:width,y:height};var p4={x:0,y:height};var pp1=p1,pp2=p2,pp3=p3,pp4=p4;switch(rotation){case 0:break;case 90:pp1=p2;pp2=p3;pp3=p4;pp4=p1;break;case 180:pp1=p3;pp2=p4;pp3=p1;pp4=p2;break;case 270:pp1=p4;pp2=p1;pp3=p2;pp4=p3;break;default:}gl.bufferData(gl.ARRAY_BUFFER,new Float32Array([pp1.x,pp1.y,pp2.x,pp2.y,pp4.x,pp4.y,pp4.x,pp4.y,pp2.x,pp2.y,pp3.x,pp3.y]),gl.STATIC_DRAW);var resolutionLocation=gl.getUniformLocation(program,"u_resolution");gl.uniform2f(resolutionLocation,width,height);that.canvasUpdated=true}return that}var AgoraRTC=function(){"use strict";var AgoraRTC={};Object.defineProperties(AgoraRTC,{version:{get:function(){return"1.7.2"}},name:{get:function(){return"AgoraWebSDK"}}});return AgoraRTC}();AgoraRTC.url="wss://localhost.agora.io:8921/";AgoraRTC.macAgentInstallUrl="http://download.agora.io/AgoraWebAgent-1.7.2.pkg";AgoraRTC.winAgentInstallUrl="http://download.agora.io/AgoraWebAgentSetup-1.7.2.exe";AgoraRTC.enAgentInstallGuideUrl="http://download.agora.io/install-guide-en.html";AgoraRTC.cnAgentInstallGuideUrl="http://download.agora.io/install-guide-cn.html";AgoraRTC.vertexShaderSource="attribute vec2 a_position;"+"attribute vec2 a_texCoord;"+"uniform vec2 u_resolution;"+"varying vec2 v_texCoord;"+"void main() {"+"vec2 zeroToOne = a_position / u_resolution;"+"   vec2 zeroToTwo = zeroToOne * 2.0;"+"   vec2 clipSpace = zeroToTwo - 1.0;"+"   gl_Position = vec4(clipSpace * vec2(1, -1), 0, 1);"+"v_texCoord = a_texCoord;"+"}";AgoraRTC.yuvShaderSource="precision mediump float;"+"uniform sampler2D Ytex;"+"uniform sampler2D Utex,Vtex;"+"varying vec2 v_texCoord;"+"void main(void) {"+"  float nx,ny,r,g,b,y,u,v;"+"  mediump vec4 txl,ux,vx;"+"  nx=v_texCoord[0];"+"  ny=v_texCoord[1];"+"  y=texture2D(Ytex,vec2(nx,ny)).r;"+"  u=texture2D(Utex,vec2(nx,ny)).r;"+"  v=texture2D(Vtex,vec2(nx,ny)).r;"+"  y=1.1643*(y-0.0625);"+"  u=u-0.5;"+"  v=v-0.5;"+"  r=y+1.5958*v;"+"  g=y-0.39173*u-0.81290*v;"+"  b=y+2.017*u;"+"  gl_FragColor=vec4(r,g,b,1.0);"+"}";AgoraRTC.videoStreamSource='function onSuccess(){postMessage({type:"init",result:!0})}function onFailure(a){postMessage(a)}function sendMessage(a){"videoStat"!=a.command&&console.log(JSON.stringify(a)),that.stream&&that.stream.readyState==WebSocket.OPEN&&that.stream.send(JSON.stringify(a))}var that={};that.stream=void 0,that.init=function(a){return that.stream&&that.stream.readyState===WebSocket.OPEN?(console.warn("stream "+that.getId()+" has been initialized already"),void onSuccess()):(that.local=a.local,that.profile=a.videoProfile,that.stream=new WebSocket(a.url),that.stream.onopen=function(a){console.log((that.local?"local":"remote")+" Stream "),console.log(a),that.profile&&sendMessage({command:"setVideoProfile",profile:that.profile.profile,swapWidthAndHeight:that.profile.swapWidthAndHeight}),that.stream.binaryType="arraybuffer",onSuccess()},that.stream.onclose=function(a){console.log(that.local?"local":"remote"," Stream ",a),that.stream=void 0,onFailure({type:a.type,code:a.code,reason:a.reason})},that.stream.onerror=function(a){console.log(that.local?"local":"remote"," Stream ",a),onFailure({type:a.type,code:a.code,reason:a.reason})},void(that.stream.onmessage=function(a){"string"==typeof a.data?console.log(that.local?"local":"remote"," message from agent ",a.data):a.data instanceof ArrayBuffer?postMessage({type:"message",data:a.data}):a.data instanceof Blob&&console.warn("Blob image data is not supported")}))},that.close=function(){that.stream&&(that.stream.onmessage=void 0,that.stream.close()),that.stream=void 0},self.addEventListener("message",function(a){var b=a.data;switch(b.type){case"init":that.init(b);break;case"send":sendMessage(b.message);break;case"close":that.close()}},!1);';var AgoraCall=function(callback,parameters){if(callback){callback(parameters)}};var L={VideoProfiles:{Profile120P:"120P",Profile120P_3:"120P_3",Profile96P:"96P",Profile48P:"48P",Profile180P:"180P",Profile180P_3:"180P_3",Profile180P_4:"180P_4",Profile240P:"240P",Profile240P_3:"240P_3",Profile240P_4:"240P_4",Profile360P:"360P",Profile360P_3:"360P_3",Profile360P_4:"360P_4",Profile360P_6:"360P_6",Profile360P_7:"360P_7",Profile360P_8:"360P_8",Profile480P:"480P",Profile480P_3:"480P_3",Profile480P_4:"480P_4",Profile480P_6:"480P_6",Profile480P_8:"480P_8",Profile480P_9:"480P_9",Profile720P:"720P",Profile720P_3:"720P_3",Profile720P_5:"720P_5",Profile720P_6:"720P_6"},ErrorCode:{NO_ERROR:0,FAILED:1,INVALID_ARGUMENT:2,NOT_READY:3,NOT_SUPPORTED:4,REFUSED:5,BUFFER_TOO_SMALL:6,NOT_INITIALIZED:7,INVALID_VIEW:8,NO_PERMISSION:9,TIMEDOUT:10,CANCELED:11,TOO_OFTEN:12,BIND_SOCKET:13,NET_DOWN:14,NET_NOBUFS:15,INIT_VIDEO:16,JOIN_CHANNEL_REJECTED:17,LEAVE_CHANNEL_REJECTED:18,ALREADY_IN_USE:19,INVALID_APP_ID:101,INVALID_CHANNEL_NAME:102,CHANNEL_KEY_EXPIRED:109,INVALID_CHANNEL_KEY:110,CONNECTION_INTERRUPTED:111,CONNECTION_LOST:112,LOAD_MEDIA_ENGINE:1001,START_CALL:1002,START_CAMERA:1003,START_VIDEO_RENDER:1004,ADM_GENERAL_ERROR:1005,ADM_JAVA_RESOURCE:1006,ADM_SAMPLE_RATE:1007,ADM_INIT_PLAYOUT:1008,ADM_START_PLAYOUT:1009,ADM_STOP_PLAYOUT:1010,ADM_INIT_RECORDING:1011,ADM_START_RECORDING:1012,ADM_STOP_RECORDING:1013,ADM_RUNTIME_PLAYOUT_ERROR:1015,ADM_RUNTIME_RECORDING_ERROR:1017,ADM_RECORD_AUDIO_FAILED:1018,ADM_INIT_LOOPBACK:1022,ADM_START_LOOPBACK:1023,VDM_CAMERA_NOT_AUTHORIZED:1501}};AgoraRTC.EventDispatcher=function(spec){"use strict";var that={};spec.dispatcher={};spec.dispatcher.eventListeners={};that.addEventListener=function(eventType,listener){spec.dispatcher.eventListeners[eventType]=listener;if(!spec.dispatcher.eventListeners[eventType]){delete spec.dispatcher.eventListeners[eventType]}};that.on=that.addEventListener;that.dispatchEvent=function(event){if(spec.dispatcher.eventListeners.hasOwnProperty(event.type)){spec.dispatcher.eventListeners[event.type](event)}};return that};AgoraRTC.BasicEvent=function(spec){"use strict";var that={};that.type=spec.type;return that};AgoraRTC.StreamEvent=function(spec){"use strict";var that=AgoraRTC.BasicEvent(spec);that.stream=spec.stream;that.msg=spec.msg;return that};AgoraRTC.ClientEvent=function(spec){"use strict";var that=AgoraRTC.BasicEvent(spec);that.uid=spec.uid;that.attr=spec.attr;that.streams=spec.streams;return that};AgoraRTC.MediaEvent=function(spec){"use strict";var that=AgoraRTC.BasicEvent(spec);that.msg=spec.msg;return that};AgoraRTC.Signal=function(spec){var that=AgoraRTC.EventDispatcher(spec);that.connection=new WebSocket(spec.url);that.sendMessage=function(obj,onError){if(that.connection.readyState==WebSocket.OPEN){that.connection.send(JSON.stringify(obj))}else{console.log("connection to agent lost.");onError({error:"not connected"})}};that.close=function(){that.onEvent=onEvent;that.connection.onopen=undefined;that.connection.onclose=undefined;that.connection.onerror=undefined;that.connection.onmessage=undefined;that.connection.close()};that.connection.onopen=function(e){console.log(e);that.dispatchEvent(AgoraRTC.MediaEvent({type:"onopen",event:e}))};that.connection.onclose=function(e){console.log(e);AgoraCall(spec.onFailure,e)};that.connection.onerror=function(e){console.log(e);AgoraCall(spec.onFailure,e)};that.onEvent=onEvent;that.connection.onmessage=function(e){console.log(e);var m=JSON.parse(e.data);if(m.hasOwnProperty("command")){that.dispatchEvent(AgoraRTC.MediaEvent({type:m.command,msg:m}))}else if(m.hasOwnProperty("event")){that.onEvent(m)}};function onEvent(e){console.log(e)}return that};AgoraRTC.Stream=function(spec){var that=AgoraRTC.EventDispatcher(spec);that.stream=undefined;that.render=undefined;that.interval=undefined;that.lastRenderCount=0;that.profile=undefined;that.latency=0;var header=undefined;var yplane=undefined;var uplane=undefined;var vplane=undefined;that.init=function(onSuccess,onFailure){if(that.stream){console.warn("stream "+that.getId()+" has been initialized already");onSuccess();return}var videoStreamFileBlob=new Blob([AgoraRTC.videoStreamSource],{type:"application/javascript"});that.stream=new Worker(URL.createObjectURL(videoStreamFileBlob));that.oninit=function(e){console.log((spec.local?"local":"remote")+" Stream ");console.log(e);if(e.result===true){onSuccess()}else{onFailure()}};that.onclose=function(e){window.clearInterval(that.interval);console.log(spec.local?"local":"remote"," Stream ",e);that.stream=undefined};that.onerror=function(e){window.clearInterval(that.interval);console.log(spec.local?"local":"remote"," Stream ",e);onFailure(e)};that.onmessage=function(e){if(typeof e==="string"){console.log(spec.local?"local":"remote"," message from agent ",e)}else if(e instanceof ArrayBuffer){if(that.render){that.drawImage(e)}}else if(e instanceof Blob){console.warn("Blob image data is not supported")}};that.stream.addEventListener("message",onWorkerMessage,false);that.stream.postMessage({type:"init",local:spec.local,url:AgoraRTC.url,videoProfile:that.profile})};that.close=function(){window.clearInterval(that.interval);if(that.stream){that.stream.removeEventListener("message",onWorkerMessage);
that.stream.postMessage({type:"close"})}that.stream=undefined;that.stop()};that.play=function(div,onFailure){var view=document.getElementById(div);that.stop();that.render=AgoraRender();that.render.start(view,onFailure);if(spec.local){that.stream.postMessage({type:"send",message:{command:"preview",uid:spec.streamID,audio:spec.audio===false?false:true,video:spec.video===false?false:true,screen:spec.screen===true?true:false,viewWidth:view.clientWidth,viewHeight:view.clientHeight}})}else{that.stream.postMessage({type:"send",message:{command:"subscribe",uid:spec.streamID,viewWidth:view.clientWidth,viewHeight:view.clientHeight}})}if(that.interval){window.clearInterval(that.interval)}that.interval=window.setInterval(function(){if(that.render){var fps=that.render.renderImageCount-that.lastRenderCount;that.stream.postMessage({type:"send",message:{command:"videoStat",uid:spec.streamID,fps:fps,frameCount:that.lastRenderCount,latency:that.latency}});that.lastRenderCount=that.render.renderImageCount;that.latency=0}},1e3)};that.stop=function(){if(that.render){that.render.stop()}that.render=undefined;that.lastRenderCount=0};that.bindClient=function(client){that.client=client};that.enableAudio=function(callback){return that.client.enableAudio(that,callback)};that.disableAudio=function(callback){return that.client.disableAudio(that,callback)};that.enableVideo=function(callback){return that.client.enableVideo(that,callback)};that.disableVideo=function(callback){return that.client.disableVideo(that,callback)};that.getId=function(){return spec.streamID};that.drawImage=function(data){if(!header){header=data;if(header.byteLength!=20){console.error("invalid image header "+data.byteLength);resetImageData()}return}if(!yplane){yplane=data;if(yplane.byteLength===20){console.error("invalid image header "+data.byteLength+" "+yplane.byteLength);resetImageData()}return}if(!uplane){uplane=data;if(uplane.byteLength===20){console.error("invalid image header "+data.byteLength+" "+yplane.byteLength+" "+uplane.byteLength);resetImageData()}return}if(!vplane){vplane=data;if(yplane.byteLength!=uplane.byteLength*4||uplane.byteLength!=vplane.byteLength){console.error("invalid image header "+data.byteLength+" "+yplane.byteLength+" "+uplane.byteLength+" "+vplane.byteLength);resetImageData();return}var headerLength=20;var dv=new DataView(header);var format=dv.getUint8(0);var mirror=dv.getUint8(1);var width=dv.getUint16(2);var height=dv.getUint16(4);var left=dv.getUint16(6);var top=dv.getUint16(8);var right=dv.getUint16(10);var bottom=dv.getUint16(12);var rotation=dv.getUint16(14);var ts=dv.getUint32(16);var xWidth=width+left+right;var xHeight=height+top+bottom;var yLength=xWidth*xHeight;var yBegin=headerLength;var yEnd=yBegin+yLength;var uLength=yLength/4;var uBegin=yEnd;var uEnd=uBegin+uLength;var vLength=yLength/4;var vBegin=uEnd;var vEnd=vBegin+vLength;that.render.renderImage({mirror:mirror,width:width,height:height,left:left,top:top,right:right,bottom:bottom,rotation:rotation,yplane:new Uint8Array(yplane),uplane:new Uint8Array(uplane),vplane:new Uint8Array(vplane)});var now32=(Date.now()&4294967295)>>>0;var latency=now32-ts;if(latency>that.latency){that.latency=latency}resetImageData()}};that.setVideoProfile=function(profile){that.profile={};if(typeof profile!=="string"&&typeof profile!=="object"){return false}if(profile.hasOwnProperty("profile")){that.profile.profile=profile.profile}if(profile.hasOwnProperty("swapWidthAndHeight")){that.profile.swapWidthAndHeight=profile.swapWidthAndHeight!==false}else{that.profile.swapWidthAndHeight=false}if(typeof profile=="string"){that.profile.profile=profile}if(that.stream){that.stream.postMessage({type:"send",message:{command:"setVideoProfile",profile:that.profile.profile,swapWidthAndHeight:that.profile.swapWidthAndHeight}})}return true};function resetImageData(){header=undefined;yplane=undefined;uplane=undefined;vplane=undefined}function getReso(w,h){return{width:w,height:h}}var supportedVideoList={true:true,unspecified:true,"120p":getReso(160,120),"240p":getReso(320,240),"360p":getReso(640,360),"480p":getReso(640,480),"720p":getReso(1280,720),"1080p":getReso(1920,1080),"4k":getReso(3840,2160)};function onWorkerMessage(e){var data=e.data;switch(data.type){case"init":that.oninit(data);break;case"close":that.onclose(data);break;case"error":that.onerror(data);break;case"message":that.onmessage(data.data);break}}return that};AgoraRTC.createStream=function(spec){return AgoraRTC.Stream(spec)};AgoraRTC.Client=function(spec){"use strict";var that=AgoraRTC.EventDispatcher(spec);that.signal=undefined;that.localStream=undefined;that.remoteStreams={};that.signalOpen=false;function onSignalError(e){if(that.signalOpen){console.log("signal connection lost.");var message={reason:"LOST_CONNECTION_TO_AGENT",type:"error",eventType:e.type,code:e.code};AgoraCall(that.onFailure,message);that.dispatchEvent(message);return}if(e.type==="error"){return}var message={reason:"CLOSE_BEFORE_OPEN",type:"error",eventType:e.type,code:e.code,agentInstallUrl:getInstallUrl(),agentInstallGuideUrl:getInstallGuideUrl()};AgoraCall(that.onFailure,message);that.dispatchEvent(message)}that.init=function(appId,onSuccess,onFailure){spec.appId=appId;that.onFailure=onFailure;if(!that.signal){try{that.signal=AgoraRTC.Signal({url:AgoraRTC.url,onFailure:onSignalError})}catch(e){console.log("create signal connection failed"+e);AgoraCall(onFailure,e);return}that.signal.on("onopen",function(event){that.signalOpen=true;that.signal.onEvent=function(e){if(e.event==="error"){var name=getErrorCodeName(e.code);if(name){e.reason=name}}else if(e.event==="onUserMuteAudio"){e.event="peer-mute-audio"}else if(e.event==="onUserMuteVideo"){e.event="peer-mute-video"}that.dispatchEvent(AgoraRTC.MediaEvent({type:e.event,msg:e}))};that.getVersion(function(ev){if(ev.version=="1.6.0"||ev.version=="1.6.1"||ev.version=="1.7.0"||ev.version=="1.7.1"||ev.version=="1.7.2"){initialize(onSuccess,onFailure);return}var message={reason:"INCOMPATIBLE_WEBAGENT",type:"error",version:AgoraRTC.version,agentVersion:ev.version,agentInstallUrl:getInstallUrl(),agentInstallGuideUrl:getInstallGuideUrl()};that.dispatchEvent(message)},function(ev){var message={reason:"INCOMPATIBLE_WEBAGENT",type:"error",version:AgoraRTC.version,agentVersion:ev.version,agentInstallUrl:getInstallUrl(),agentInstallGuideUrl:getInstallGuideUrl()};that.dispatchEvent(message)})})}if(that.signalOpen){initialize(onSuccess,onFailure)}function initialize(onSuccess,onFailure){that.signal.on("initialize",function(ev){var m=ev.msg;if(m.code!=true){that.signal.close();that.signal=undefined;onError(m.code);return}AgoraCall(onSuccess,m)});that.signal.sendMessage({command:"initialize",appId:spec.appId},function(err){AgoraCall(onFailure,err)})}function onError(code){var name=getErrorCodeName(code);if(name){AgoraCall(onFailure,{reason:name});return}AgoraCall(onFailure,{reason:"UNKNOWN_ERROR",code:m.code});return}function getErrorCodeName(code){for(var k in L.ErrorCode){if(L.ErrorCode.hasOwnProperty(k)){if(L.ErrorCode[k]===code){return String(k)}}}return null}that.signal.on("onError",function(ev){var m=ev.msg;onError(m.code)});console.log(that.signal)};that.renewChannelKey=function(channelKey,onSuccess,onFailure){that.signal.sendMessage({command:"renewChannelKey",channelKey:channelKey},function(e){AgoraCall(onFailure,e)});that.signal.on("renewChannelKey",function(ev){var m=ev.msg;if(m.code===true){AgoraCall(onSuccess,m);return}AgoraCall(onFailure,m)})};that.join=function(channelKey,channelName,uid,onSuccess,onFailure){if(channelName.length>64){onFailure(L.ErrorCode.INVALID_CHANNEL_NAME);return}that.signal.sendMessage({command:"joinChannel",channelKey:channelKey,channelName:channelName,uid:uid},function(e){AgoraCall(onFailure,{error:e})});that.signal.on("joinChannel",function(ev){that.signal.on("onAddVideoStream",onAddVideoStream);that.signal.on("onPeerLeave",onPeerLeave);var m=ev.msg;if(m.code==true){if(!m.uid){m.uid=0}AgoraCall(onSuccess,m.uid);return}if(m.code==L.ErrorCode.JOIN_CHANNEL_REJECTED){console.error("Command joinChannel has been rejected by agent. Is this user joined a channel already?");AgoraCall(onFailure,m.code);return}AgoraCall(onFailure,m.code)})};that.leave=function(onSuccess,onFailure){closeLocalRemoteStreams();that.signal.sendMessage({command:"leaveChannel"},function(e){console.log("leave channel failed",e)});that.signal.on("leaveChannel",function(ev){var m=ev.msg;if(m.code===true){AgoraCall(onSuccess,m);return}if(m.code==L.ErrorCode.LEAVE_CHANNEL_REJECTED){console.error("Command leaveChannel has been rejected by agent. Is this user not in a channel?");AgoraCall(onSuccess,m);return}AgoraCall(onFailure,{code:m.code})})};that.publish=function(stream,onSuccess,onFailure){that.localStream=stream;that.signal.sendMessage({command:"unmuteLocal"},function(e){AgoraCall(onFailure,{error:e})});that.signal.on("unmuteLocal",function(ev){var m=ev.msg;if(m.code!=true){AgoraCall(onFailure,{code:m.code});return}AgoraCall(onSuccess,m)});stream.bindClient(that)};that.unpublish=function(stream,onSuccess,onFailure){that.signal.sendMessage({command:"muteLocal"},function(e){AgoraCall(onFailure,{error:e})});that.signal.on("muteLocal",function(ev){var m=ev.msg;if(m.code!=true){AgoraCall(onFailure,{code:m.code});return}AgoraCall(onSuccess,m)})};that.subscribe=function(stream,onFailure){stream.init(function(){that.dispatchEvent(AgoraRTC.StreamEvent({type:"stream-subscribed",stream:stream}))},function(error){AgoraCall(onFailure,error)})};that.unsubscribe=function(streamID,onFailure){console.log("remote streams",that.remoteStreams);if(that.remoteStreams[streamID]==undefined){AgoraCall(onFailure,{error:"no such stream"});return}that.remoteStreams[streamID].close();delete that.remoteStreams[streamID]};that.enableAudio=function(stream,callback){that.signal.on("enableAudio",function(ev){var m=ev.msg;if(callback){AgoraCall(callback,m.code===true)}});that.signal.sendMessage({command:"enableAudio",streamID:stream.getId()},function(e){e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(callback,e)});return true};that.disableAudio=function(stream,callback){that.signal.on("disableAudio",function(ev){var m=ev.msg;if(callback){AgoraCall(callback,m.code===true)}});that.signal.sendMessage({command:"disableAudio",streamID:stream.getId()},function(e){e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(callback,e)});return true};that.enableVideo=function(stream,callback){that.signal.on("enableVideo",function(ev){var m=ev.msg;if(callback){AgoraCall(callback,m.code===true)}});that.signal.sendMessage({command:"enableVideo",streamID:stream.getId()},function(e){e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(callback,e)});return true};that.disableVideo=function(stream,callback){that.signal.on("disableVideo",function(ev){var m=ev.msg;if(callback){AgoraCall(callback,m.code===true)}});that.signal.sendMessage({command:"disableVideo",streamID:stream.getId()},function(e){e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(callback,e)});return true};that.getDevices=function(callback){that.signal.sendMessage({command:"getDevices"},function(e){e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(callback,e)});that.signal.on("getDevices",function(ev){var devices=ev.msg.devices;AgoraCall(callback,devices)})};that.selectDevice=function(device,onFailure){that.signal.sendMessage({command:"selectDevice",device:device},function(e){e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(onFailure,e)});that.signal.on("selectDevice",function(ev){console.log(ev)})};that.startRecording=function(recordingKey,onSuccess,onFailure){that.signal.sendMessage({command:"startRecording",recordingKey:recordingKey},function(e){e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(onFailure,e)});that.signal.on("startRecording",function(ev){var start=ev.msg;if(start.code===true){AgoraCall(onSuccess,start)}else{AgoraCall(onFailure,start)}})};that.stopRecording=function(recordingKey,onSuccess,onFailure){that.signal.sendMessage({command:"stopRecording",recordingKey:recordingKey},function(e){e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(onFailure,e)});that.signal.on("stopRecording",function(ev){var stop=ev.msg;if(stop.code===true){AgoraCall(onSuccess,stop)}else{AgoraCall(onFailure,stop)}})};that.queryRecordingStatus=function(onStatus){that.signal.sendMessage({command:"queryRecordingStatus"},function(e){e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(onStatus,e)});that.signal.on("queryRecordingStatus",function(ev){onStatus(ev.msg)})};that.close=function(){closeLocalRemoteStreams();if(that.signal){that.signal.close();that.signal=undefined}};that.getVersion=function(onSuccess,onFailure){that.signal.sendMessage({command:"getVersion"});that.signal.on("getVersion",function(ev){var m=ev.msg;if(m.code===true){onSuccess(m)}else{onFailure(m)}})};that.setParameters=function(parameters){that.signal.sendMessage({command:"setParameters",parameters:JSON.stringify(parameters)})};that.setEncryptionMode=function(encryptionMode){that.signal.sendMessage({command:"setEncryptionMode",mode:encryptionMode})};that.setEncryptionSecret=function(secret){that.signal.sendMessage({command:"setEncryptionSecret",secret:secret})};that.getWindows=function(callback){that.signal.sendMessage({command:"getWindows"},function(e){e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(callback,e)});that.signal.on("getWindows",function(ev){var windows=ev.msg.windows;AgoraCall(callback,windows)})};that.startScreenSharing=function(window,onSuccess,onFailure){that.signal.sendMessage({command:"startScreenSharing",window:window},function(e){e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(onFailure,e)});that.signal.on("startScreenSharing",function(e){AgoraCall(onSuccess,e)})};that.setScreenSharingWindow=function(window,onSuccess,onFailure){that.signal.sendMessage({command:"setScreenSharingWindow",window:window},function(e){e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(onFailure,e)});that.signal.on("setScreenSharingWindow",function(e){AgoraCall(onSuccess,e);console.log(e)})};that.stopScreenSharing=function(onSuccess,onFailure){that.signal.sendMessage({command:"stopScreenSharing"},function(e){e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(onFailure,e)});that.signal.on("stopScreenSharing",function(e){AgoraCall(onSuccess,e);console.log(e)})};function onPeerLeave(ev){var m=ev.msg;if(that.remoteStreams.hasOwnProperty(m.uid)){that.remoteStreams[m.uid].close();delete that.remoteStreams[m.uid];console.log("remote streams after peer leave ",that.remoteStreams)}that.dispatchEvent(AgoraRTC.ClientEvent({type:"peer-leave",uid:m.uid}))}function onAddVideoStream(ev){var m=ev.msg;if(!that.remoteStreams.hasOwnProperty(m.uid)){var stream=AgoraRTC.Stream({streamID:m.uid,local:false,audio:m.audio,video:m.video,screen:m.screen});stream.bindClient(that);that.remoteStreams[m.uid]=stream}that.dispatchEvent(AgoraRTC.StreamEvent({type:"stream-added",stream:that.remoteStreams[m.uid]}));console.log("remote streams",that.remoteStreams)}function getInstallUrl(){if(navigator.appVersion.indexOf("Mac")!=-1){return AgoraRTC.macAgentInstallUrl}return AgoraRTC.winAgentInstallUrl}function getInstallGuideUrl(){var userLang=navigator.language||navigator.userLanguage;if(userLang.indexOf("zh")!=-1){return AgoraRTC.cnAgentInstallGuideUrl}return AgoraRTC.enAgentInstallGuideUrl}function closeLocalRemoteStreams(){if(that.localStream){that.localStream.close()}that.localStream=undefined;for(var streamId in that.remoteStreams){if(that.remoteStreams.hasOwnProperty(streamId)){that.remoteStreams[streamId].close();delete that.remoteStreams[streamId]}}}return that};AgoraRTC.createRtcClient=function(){return AgoraRTC.Client({})};
//# sourceMappingURL=dist/AgoraRtcAgentSDK.min.js.map
"use strict";(function(root,factory){if(typeof define==="function"&&define.amd){define([],factory)}else{var lib=factory.call(root);Object.keys(lib).forEach(function(key){root[key]=lib[key]})}})(this,function(){var topWindow=this;function error(msg){if(topWindow.console){if(topWindow.console.error){topWindow.console.error(msg)}else if(topWindow.console.log){topWindow.console.log(msg)}}}function isInIFrame(w){w=w||topWindow;return w!==w.top}function makeFailHTML(msg){return""+'<table style="background-color: #8CE; width: 100%; height: 100%;"><tr>'+'<td align="center">'+'<div style="display: table-cell; vertical-align: middle;">'+'<div style="">'+msg+"</div>"+"</div>"+"</td></tr></table>"}var GET_A_WEBGL_BROWSER=""+"This page requires a browser that supports WebGL.<br/>"+'<a href="http://get.webgl.org">Click here to upgrade your browser.</a>';var OTHER_PROBLEM=""+"It doesn't appear your computer can support WebGL.<br/>"+'<a href="http://get.webgl.org/troubleshooting/">Click here for more information.</a>';function create3DContext(canvas,opt_attribs){var names=["webgl","experimental-webgl"];var context=null;for(var ii=0;ii<names.length;++ii){try{context=canvas.getContext(names[ii],opt_attribs)}catch(e){}if(context){break}}return context}function setupWebGL(canvas,opt_attribs){function showLink(str){var container=canvas.parentNode;if(container){container.innerHTML=makeFailHTML(str)}}if(!topWindow.WebGLRenderingContext){showLink(GET_A_WEBGL_BROWSER);return null}var context=create3DContext(canvas,opt_attribs);if(!context){showLink(OTHER_PROBLEM)}return context}function updateCSSIfInIFrame(){if(isInIFrame()){document.body.className="iframe"}}function getWebGLContext(canvas,opt_attribs,opt_options){var options=opt_options||{};if(isInIFrame()){updateCSSIfInIFrame();if(!options.dontResize&&options.resize!==false){var width=canvas.clientWidth;var height=canvas.clientHeight;canvas.width=width;canvas.height=height}}else if(!options.noTitle&&options.title!==false){var title=document.title;var h1=document.createElement("h1");h1.innerText=title;document.body.insertBefore(h1,document.body.children[0])}var gl=setupWebGL(canvas,opt_attribs);return gl}function loadShader(gl,shaderSource,shaderType,opt_errorCallback){var errFn=opt_errorCallback||error;var shader=gl.createShader(shaderType);gl.shaderSource(shader,shaderSource);gl.compileShader(shader);var compiled=gl.getShaderParameter(shader,gl.COMPILE_STATUS);if(!compiled){var lastError=gl.getShaderInfoLog(shader);errFn("*** Error compiling shader '"+shader+"':"+lastError);gl.deleteShader(shader);return null}return shader}function createProgram(gl,shaders,opt_attribs,opt_locations,opt_errorCallback){var errFn=opt_errorCallback||error;var program=gl.createProgram();shaders.forEach(function(shader){gl.attachShader(program,shader)});if(opt_attribs){obj_attrib.forEach(function(attrib,ndx){gl.bindAttribLocation(program,opt_locations?opt_locations[ndx]:ndx,attrib)})}gl.linkProgram(program);var linked=gl.getProgramParameter(program,gl.LINK_STATUS);if(!linked){var lastError=gl.getProgramInfoLog(program);errFn("Error in program linking:"+lastError);gl.deleteProgram(program);return null}return program}function createShaderFromScript(gl,scriptId,opt_shaderType,opt_errorCallback){var shaderSource="";var shaderType;var shaderScript=document.getElementById(scriptId);if(!shaderScript){throw"*** Error: unknown script element"+scriptId}shaderSource=shaderScript.text;if(!opt_shaderType){if(shaderScript.type==="x-shader/x-vertex"){shaderType=gl.VERTEX_SHADER}else if(shaderScript.type==="x-shader/x-fragment"){shaderType=gl.FRAGMENT_SHADER}else if(shaderType!==gl.VERTEX_SHADER&&shaderType!==gl.FRAGMENT_SHADER){throw"*** Error: unknown shader type"}}return loadShader(gl,shaderSource,opt_shaderType?opt_shaderType:shaderType,opt_errorCallback)}var defaultShaderType=["VERTEX_SHADER","FRAGMENT_SHADER"];function createProgramFromScripts(gl,shaderScriptIds,opt_attribs,opt_locations,opt_errorCallback){var shaders=[];for(var ii=0;ii<shaderScriptIds.length;++ii){shaders.push(createShaderFromScript(gl,shaderScriptIds[ii],gl[defaultShaderType[ii]],opt_errorCallback))}return createProgram(gl,shaders,opt_attribs,opt_locations,opt_errorCallback)}function createProgramFromSources(gl,shaderSources,opt_attribs,opt_locations,opt_errorCallback){var shaders=[];for(var ii=0;ii<shaderSources.length;++ii){shaders.push(loadShader(gl,shaderSources[ii],gl[defaultShaderType[ii]],opt_errorCallback))}return createProgram(gl,shaders,opt_attribs,opt_locations,opt_errorCallback)}function getBindPointForSamplerType(gl,type){if(type===gl.SAMPLER_2D)return gl.TEXTURE_2D;if(type===gl.SAMPLER_CUBE)return gl.TEXTURE_CUBE_MAP}function createUniformSetters(gl,program){var textureUnit=0;function createUniformSetter(program,uniformInfo){var location=gl.getUniformLocation(program,uniformInfo.name);var type=uniformInfo.type;var isArray=uniformInfo.size>1&&uniformInfo.name.substr(-3)==="[0]";if(type===gl.FLOAT&&isArray){return function(v){gl.uniform1fv(location,v)}}if(type===gl.FLOAT){return function(v){gl.uniform1f(location,v)}}if(type===gl.FLOAT_VEC2){return function(v){gl.uniform2fv(location,v)}}if(type===gl.FLOAT_VEC3){return function(v){gl.uniform3fv(location,v)}}if(type===gl.FLOAT_VEC4){return function(v){gl.uniform4fv(location,v)}}if(type===gl.INT&&isArray){return function(v){gl.uniform1iv(location,v)}}if(type===gl.INT){return function(v){gl.uniform1i(location,v)}}if(type===gl.INT_VEC2){return function(v){gl.uniform2iv(location,v)}}if(type===gl.INT_VEC3){return function(v){gl.uniform3iv(location,v)}}if(type===gl.INT_VEC4){return function(v){gl.uniform4iv(location,v)}}if(type===gl.BOOL){return function(v){gl.uniform1iv(location,v)}}if(type===gl.BOOL_VEC2){return function(v){gl.uniform2iv(location,v)}}if(type===gl.BOOL_VEC3){return function(v){gl.uniform3iv(location,v)}}if(type===gl.BOOL_VEC4){return function(v){gl.uniform4iv(location,v)}}if(type===gl.FLOAT_MAT2){return function(v){gl.uniformMatrix2fv(location,false,v)}}if(type===gl.FLOAT_MAT3){return function(v){gl.uniformMatrix3fv(location,false,v)}}if(type===gl.FLOAT_MAT4){return function(v){gl.uniformMatrix4fv(location,false,v)}}if((type===gl.SAMPLER_2D||type===gl.SAMPLER_CUBE)&&isArray){var units=[];for(var ii=0;ii<info.size;++ii){units.push(textureUnit++)}return function(bindPoint,units){return function(textures){gl.uniform1iv(location,units);textures.forEach(function(texture,index){gl.activeTexture(gl.TEXTURE0+units[index]);gl.bindTexture(bindPoint,texture)})}}(getBindPointForSamplerType(gl,type),units)}if(type===gl.SAMPLER_2D||type===gl.SAMPLER_CUBE){return function(bindPoint,unit){return function(texture){gl.uniform1i(location,unit);gl.activeTexture(gl.TEXTURE0+unit);gl.bindTexture(bindPoint,texture)}}(getBindPointForSamplerType(gl,type),textureUnit++)}throw"unknown type: 0x"+type.toString(16)}var uniformSetters={};var numUniforms=gl.getProgramParameter(program,gl.ACTIVE_UNIFORMS);for(var ii=0;ii<numUniforms;++ii){var uniformInfo=gl.getActiveUniform(program,ii);if(!uniformInfo){break}var name=uniformInfo.name;if(name.substr(-3)==="[0]"){name=name.substr(0,name.length-3)}var setter=createUniformSetter(program,uniformInfo);uniformSetters[name]=setter}return uniformSetters}function setUniforms(setters,values){Object.keys(values).forEach(function(name){var setter=setters[name];if(setter){setter(values[name])}})}function createAttributeSetters(gl,program){var attribSetters={};function createAttribSetter(index){return function(b){gl.bindBuffer(gl.ARRAY_BUFFER,b.buffer);gl.enableVertexAttribArray(index);gl.vertexAttribPointer(index,b.numComponents||b.size,b.type||gl.FLOAT,b.normalize||false,b.stride||0,b.offset||0)}}var numAttribs=gl.getProgramParameter(program,gl.ACTIVE_ATTRIBUTES);for(var ii=0;ii<numAttribs;++ii){var attribInfo=gl.getActiveAttrib(program,ii);if(!attribInfo){break}var index=gl.getAttribLocation(program,attribInfo.name);attribSetters[attribInfo.name]=createAttribSetter(index)}return attribSetters}function setAttributes(setters,buffers){Object.keys(buffers).forEach(function(name){var setter=setters[name];if(setter){setter(buffers[name])}})}function createProgramInfo(gl,shaderSources,opt_attribs,opt_locations,opt_errorCallback){shaderSources=shaderSources.map(function(source){var script=document.getElementById(source);return script?script.text:source});var program=createProgramFromSources(gl,shaderSources,opt_attribs,opt_locations,opt_errorCallback);if(!program){return null}var uniformSetters=createUniformSetters(gl,program);var attribSetters=createAttributeSetters(gl,program);return{program:program,uniformSetters:uniformSetters,attribSetters:attribSetters}}function setBuffersAndAttributes(gl,setters,buffers){setAttributes(setters,buffers.attribs);if(buffers.indices){gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER,buffers.indices)}}var browserPrefixes=["","MOZ_","OP_","WEBKIT_"];function getExtensionWithKnownPrefixes(gl,name){for(var ii=0;ii<browserPrefixes.length;++ii){var prefixedName=browserPrefixes[ii]+name;var ext=gl.getExtension(prefixedName);if(ext){return ext}}}function resizeCanvasToDisplaySize(canvas,multiplier){multiplier=multiplier||1;var width=canvas.clientWidth*multiplier;var height=canvas.clientHeight*multiplier;if(canvas.width!==width||canvas.height!==height){canvas.width=width;canvas.height=height;return true}return false}function getIFrameForWindow(window){if(!isInIFrame(window)){return}var iframes=window.parent.document.getElementsByTagName("iframe");for(var ii=0;ii<iframes.length;++ii){var iframe=iframes[ii];if(iframe.contentDocument===window.document){return iframe}}}function isFrameVisible(window){try{var iframe=getIFrameForWindow(window);if(!iframe){return true}var bounds=iframe.getBoundingClientRect();var isVisible=bounds.top<window.parent.innerHeight&&bounds.bottom>=0&&bounds.left<window.parent.innerWidth&&bounds.right>=0;return isVisible&&isFrameVisible(window.parent)}catch(e){return true}}function isOnScreen(element){var isVisible=true;if(element){var bounds=element.getBoundingClientRect();isVisible=bounds.top<topWindow.innerHeight&&bounds.bottom>=0}return isVisible&&isFrameVisible(topWindow)}function augmentTypedArray(typedArray,numComponents){var cursor=0;typedArray.push=function(){for(var ii=0;ii<arguments.length;++ii){var value=arguments[ii];if(value instanceof Array||value.buffer&&value.buffer instanceof ArrayBuffer){for(var jj=0;jj<value.length;++jj){typedArray[cursor++]=value[jj]}}else{typedArray[cursor++]=value}}};typedArray.reset=function(opt_index){cursor=opt_index||0};typedArray.numComponents=numComponents;Object.defineProperty(typedArray,"numElements",{get:function(){return this.length/this.numComponents|0}});return typedArray}function createAugmentedTypedArray(numComponents,numElements,opt_type){var Type=opt_type||Float32Array;return augmentTypedArray(new Type(numComponents*numElements),numComponents)}function createBufferFromTypedArray(gl,array,type,drawType){type=type||gl.ARRAY_BUFFER;var buffer=gl.createBuffer();gl.bindBuffer(type,buffer);gl.bufferData(type,array,drawType||gl.STATIC_DRAW);return buffer}function allButIndices(name){return name!=="indices"}function createMapping(obj){var mapping={};Object.keys(obj).filter(allButIndices).forEach(function(key){mapping["a_"+key]=key});return mapping}function getGLTypeForTypedArray(gl,typedArray){if(typedArray instanceof Int8Array){return gl.BYTE}if(typedArray instanceof Uint8Array){return gl.UNSIGNED_BYTE}if(typedArray instanceof Int16Array){return gl.SHORT}if(typedArray instanceof Uint16Array){return gl.UNSIGNED_SHORT}if(typedArray instanceof Int32Array){return gl.INT}if(typedArray instanceof Uint32Array){return gl.UNSIGNED_INT}if(typedArray instanceof Float32Array){return gl.FLOAT}throw"unsupported typed array type"}function getNormalizationForTypedArray(typedArray){if(typedArray instanceof Int8Array){return true}if(typedArray instanceof Uint8Array){return true}return false}function isArrayBuffer(a){return a.buffer&&a.buffer instanceof ArrayBuffer}function guessNumComponentsFromName(name,length){var numComponents;if(name.indexOf("coord")>=0){numComponents=2}else if(name.indexOf("color")>=0){numComponents=4}else{numComponents=3}if(length%numComponents>0){throw"can not guess numComponents. You should specify it."}return numComponents}function makeTypedArray(array,name){if(isArrayBuffer(array)){return array}if(Array.isArray(array)){array={data:array}}if(!array.numComponents){array.numComponents=guessNumComponentsFromName(name,array.length)}var type=array.type;if(!type){if(name==="indices"){type=Uint16Array}}var typedArray=createAugmentedTypedArray(array.numComponents,array.data.length/array.numComponents|0,type);typedArray.push(array.data);return typedArray}function createAttribsFromArrays(gl,arrays,opt_mapping){var mapping=opt_mapping||createMapping(arrays);var attribs={};Object.keys(mapping).forEach(function(attribName){var bufferName=mapping[attribName];var array=makeTypedArray(arrays[bufferName],bufferName);attribs[attribName]={buffer:createBufferFromTypedArray(gl,array),numComponents:array.numComponents||guessNumComponentsFromName(bufferName),type:getGLTypeForTypedArray(gl,array),normalize:getNormalizationForTypedArray(array)}});return attribs}function getNumElementsFromNonIndexedArrays(arrays){var key=Object.keys(arrays)[0];var array=arrays[key];if(isArrayBuffer(array)){return array.numElements}else{return array.data.length/array.numComponents}}function createBufferInfoFromArrays(gl,arrays,opt_mapping){var bufferInfo={attribs:createAttribsFromArrays(gl,arrays,opt_mapping)};var indices=arrays.indices;if(indices){indices=makeTypedArray(indices,"indices");bufferInfo.indices=createBufferFromTypedArray(gl,indices,gl.ELEMENT_ARRAY_BUFFER);bufferInfo.numElements=indices.length}else{bufferInfo.numElements=getNumElementsFromNonIndexedArrays(arrays)}return bufferInfo}function createBuffersFromArrays(gl,arrays){var buffers={};Object.keys(arrays).forEach(function(key){var type=key==="indices"?gl.ELEMENT_ARRAY_BUFFER:gl.ARRAY_BUFFER;var array=makeTypedArray(arrays[key],name);buffers[key]=createBufferFromTypedArray(gl,array,type)});if(arrays.indices){buffers.numElements=arrays.indices.length}else if(arrays.position){buffers.numElements=arrays.position.length/3}return buffers}function drawBufferInfo(gl,type,bufferInfo,count,offset){var indices=bufferInfo.indices;var numElements=count===undefined?bufferInfo.numElements:count;offset=offset===undefined?offset:0;if(indices){gl.drawElements(type,numElements,gl.UNSIGNED_SHORT,offset)}else{gl.drawArrays(type,offset,numElements)}}function drawObjectList(gl,objectsToDraw){var lastUsedProgramInfo=null;var lastUsedBufferInfo=null;objectsToDraw.forEach(function(object){var programInfo=object.programInfo;var bufferInfo=object.bufferInfo;var bindBuffers=false;if(programInfo!==lastUsedProgramInfo){lastUsedProgramInfo=programInfo;gl.useProgram(programInfo.program);bindBuffers=true}if(bindBuffers||bufferInfo!==lastUsedBufferInfo){lastUsedBufferInfo=bufferInfo;setBuffersAndAttributes(gl,programInfo.attribSetters,bufferInfo)}setUniforms(programInfo.uniformSetters,object.uniforms);drawBufferInfo(gl,gl.TRIANGLES,bufferInfo)})}return{createAugmentedTypedArray:createAugmentedTypedArray,createAttribsFromArrays:createAttribsFromArrays,createBuffersFromArrays:createBuffersFromArrays,createBufferInfoFromArrays:createBufferInfoFromArrays,createAttributeSetters:createAttributeSetters,createProgram:createProgram,createProgramFromScripts:createProgramFromScripts,createProgramFromSources:createProgramFromSources,createProgramInfo:createProgramInfo,createUniformSetters:createUniformSetters,drawBufferInfo:drawBufferInfo,drawObjectList:drawObjectList,getWebGLContext:getWebGLContext,updateCSSIfInIFrame:updateCSSIfInIFrame,getExtensionWithKnownPrefixes:getExtensionWithKnownPrefixes,resizeCanvasToDisplaySize:resizeCanvasToDisplaySize,setAttributes:setAttributes,setBuffersAndAttributes:setBuffersAndAttributes,setUniforms:setUniforms,setupWebGL:setupWebGL}});function AgoraRender(){var gl=undefined;var program=undefined;var positionLocation=undefined;var texCoordLocation=undefined;var yTexture=undefined;var uTexture=undefined;var vTexture=undefined;var texCoordBuffer=undefined;var surfaceBuffer=undefined;var that={view:undefined,mirrorView:false,container:undefined,canvas:undefined,renderImageCount:0,initWidth:0,initHeight:0,initRotation:0,canvasUpdated:false,clientWidth:0,clientHeight:0};that.start=function(view,onFailure){initCanvas(view,that.mirrorView,view.clientWidth,view.clientHeight,that.initRotation,onFailure)};that.stop=function(){gl=undefined;program=undefined;positionLocation=undefined;texCoordLocation=undefined;deleteTexture(yTexture);deleteTexture(uTexture);deleteTexture(vTexture);yTexture=undefined;uTexture=undefined;vTexture=undefined;deleteBuffer(texCoordBuffer);deleteBuffer(surfaceBuffer);texCoordBuffer=undefined;surfaceBuffer=undefined;if(that.container){that.container.removeChild(that.canvas)}if(that.view){that.view.removeChild(that.container)}that.canvas=undefined;that.container=undefined;that.view=undefined;that.mirrorView=false};that.renderImage=function(image){if(!gl){return}if(image.width!=that.initWidth||image.height!=that.initHeight||image.rotation!=that.initRotation||image.mirror!=that.mirrorView){var view=that.view;that.stop();console.log("init canvas "+image.width+"*"+image.height+" rotation "+image.rotation);initCanvas(view,image.mirror,image.width,image.height,image.rotation,function(e){console.error("init canvas "+image.width+"*"+image.height+" rotation "+image.rotation+" failed. "+e)})}gl.bindBuffer(gl.ARRAY_BUFFER,texCoordBuffer);var xWidth=image.width+image.left+image.right;var xHeight=image.height+image.top+image.bottom;gl.bufferData(gl.ARRAY_BUFFER,new Float32Array([image.left/xWidth,image.bottom/xHeight,1-image.right/xWidth,image.bottom/xHeight,image.left/xWidth,1-image.top/xHeight,image.left/xWidth,1-image.top/xHeight,1-image.right/xWidth,image.bottom/xHeight,1-image.right/xWidth,1-image.top/xHeight]),gl.STATIC_DRAW);gl.enableVertexAttribArray(texCoordLocation);gl.vertexAttribPointer(texCoordLocation,2,gl.FLOAT,false,0,0);uploadYuv(xWidth,xHeight,image.yplane,image.uplane,image.vplane);updateCanvas(image.rotation,image.width,image.height);gl.drawArrays(gl.TRIANGLES,0,6);that.renderImageCount+=1};function uploadYuv(width,height,yplane,uplane,vplane){gl.activeTexture(gl.TEXTURE0);gl.bindTexture(gl.TEXTURE_2D,yTexture);gl.texImage2D(gl.TEXTURE_2D,0,gl.LUMINANCE,width,height,0,gl.LUMINANCE,gl.UNSIGNED_BYTE,yplane);var e=gl.getError();if(e!=gl.NO_ERROR){console.log("upload y plane ",width,height,yplane.byteLength," error",e)}gl.activeTexture(gl.TEXTURE1);gl.bindTexture(gl.TEXTURE_2D,uTexture);gl.texImage2D(gl.TEXTURE_2D,0,gl.LUMINANCE,width/2,height/2,0,gl.LUMINANCE,gl.UNSIGNED_BYTE,uplane);var e=gl.getError();if(e!=gl.NO_ERROR){console.log("upload u plane ",width,height,uplane.byteLength,"  error",e)}gl.activeTexture(gl.TEXTURE2);gl.bindTexture(gl.TEXTURE_2D,vTexture);gl.texImage2D(gl.TEXTURE_2D,0,gl.LUMINANCE,width/2,height/2,0,gl.LUMINANCE,gl.UNSIGNED_BYTE,vplane);var e=gl.getError();if(e!=gl.NO_ERROR){console.log("upload v plane ",width,height,vplane.byteLength,"  error",e)}}function deleteBuffer(buffer){if(buffer&&that.gl){that.gl.deleteBuffer(buffer)}}function deleteTexture(texture){if(texture&&that.gl){that.gl.deleteTexture(texture)}}function initCanvas(view,mirror,width,height,rotation,onFailure){that.clientWidth=view.clientWidth;that.clientHeight=view.clientHeight;that.view=view;that.mirrorView=mirror;that.canvasUpdated=false;that.container=document.createElement("div");that.container.style.width="100%";that.container.style.height="100%";that.container.style.display="flex";that.container.style.justifyContent="center";that.container.style.alignItems="center";that.view.appendChild(that.container);that.canvas=document.createElement("canvas");if(rotation==0||rotation==180){that.canvas.width=width;that.canvas.height=height}else{that.canvas.width=height;that.canvas.height=width}that.initWidth=width;that.initHeight=height;that.initRotation=rotation;if(that.mirrorView){that.canvas.style.transform="rotateY(180deg)"}that.container.appendChild(that.canvas);try{gl=that.canvas.getContext("webgl")||that.canvas.getContext("experimental-webgl")}catch(e){console.log(e)}if(!gl){gl=undefined;onFailure({error:"Browser not support! No WebGL detected."});return}gl.clearColor(0,0,0,1);gl.enable(gl.DEPTH_TEST);gl.depthFunc(gl.LEQUAL);gl.clear(gl.COLOR_BUFFER_BIT|gl.DEPTH_BUFFER_BIT);program=createProgramFromSources(gl,[AgoraRTC.vertexShaderSource,AgoraRTC.yuvShaderSource]);gl.useProgram(program);initTextures()}function initTextures(){positionLocation=gl.getAttribLocation(program,"a_position");texCoordLocation=gl.getAttribLocation(program,"a_texCoord");surfaceBuffer=gl.createBuffer();texCoordBuffer=gl.createBuffer();gl.activeTexture(gl.TEXTURE0);yTexture=gl.createTexture();gl.bindTexture(gl.TEXTURE_2D,yTexture);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_WRAP_S,gl.CLAMP_TO_EDGE);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_WRAP_T,gl.CLAMP_TO_EDGE);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_MIN_FILTER,gl.NEAREST);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_MAG_FILTER,gl.NEAREST);gl.activeTexture(gl.TEXTURE1);uTexture=gl.createTexture();gl.bindTexture(gl.TEXTURE_2D,uTexture);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_WRAP_S,gl.CLAMP_TO_EDGE);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_WRAP_T,gl.CLAMP_TO_EDGE);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_MIN_FILTER,gl.NEAREST);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_MAG_FILTER,gl.NEAREST);gl.activeTexture(gl.TEXTURE2);vTexture=gl.createTexture();gl.bindTexture(gl.TEXTURE_2D,vTexture);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_WRAP_S,gl.CLAMP_TO_EDGE);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_WRAP_T,gl.CLAMP_TO_EDGE);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_MIN_FILTER,gl.NEAREST);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_MAG_FILTER,gl.NEAREST);var y=gl.getUniformLocation(program,"Ytex");gl.uniform1i(y,0);var u=gl.getUniformLocation(program,"Utex");gl.uniform1i(u,1);var v=gl.getUniformLocation(program,"Vtex");gl.uniform1i(v,2)}function updateCanvas(rotation,width,height){if(that.canvasUpdated){return}that.canvas.style.width="100%";that.canvas.style.height="100%";try{if(rotation===0||rotation===180){if(that.clientWidth/that.clientHeight>width/height){that.canvas.style.width=that.clientHeight*width/height+"px"}else if(that.clientWidth/that.clientHeight<width/height){that.canvas.style.height=that.clientWidth*height/width+"px"}}else{if(that.clientHeight/that.clientWidth>width/height){that.canvas.style.height=that.clientWidth*width/height+"px"}else if(that.clientHeight/that.clientWidth<width/height){that.canvas.style.width=that.clientHeight*height/width+"px"}}}catch(e){console.log("updateCanvas 00001 gone "+that.canvas);console.log(that);console.error(e);return}gl.bindBuffer(gl.ARRAY_BUFFER,surfaceBuffer);gl.enableVertexAttribArray(positionLocation);gl.vertexAttribPointer(positionLocation,2,gl.FLOAT,false,0,0);var p1={x:0,y:0};var p2={x:width,y:0};var p3={x:width,y:height};var p4={x:0,y:height};var pp1=p1,pp2=p2,pp3=p3,pp4=p4;switch(rotation){case 0:break;case 90:pp1=p2;pp2=p3;pp3=p4;pp4=p1;break;case 180:pp1=p3;pp2=p4;pp3=p1;pp4=p2;break;case 270:pp1=p4;pp2=p1;pp3=p2;pp4=p3;break;default:}gl.bufferData(gl.ARRAY_BUFFER,new Float32Array([pp1.x,pp1.y,pp2.x,pp2.y,pp4.x,pp4.y,pp4.x,pp4.y,pp2.x,pp2.y,pp3.x,pp3.y]),gl.STATIC_DRAW);var resolutionLocation=gl.getUniformLocation(program,"u_resolution");gl.uniform2f(resolutionLocation,width,height);that.canvasUpdated=true}return that}var AgoraRTC=function(){"use strict";var AgoraRTC={};Object.defineProperties(AgoraRTC,{version:{get:function(){return"1.7.3"}},name:{get:function(){return"AgoraWebSDK"}}});return AgoraRTC}();AgoraRTC.url="wss://localhost.agora.io:8921/";AgoraRTC.macAgentInstallUrl="http://download.agora.io/AgoraWebAgent-1.7.3.pkg";AgoraRTC.winAgentInstallUrl="http://download.agora.io/AgoraWebAgentSetup-1.7.3.exe";AgoraRTC.enAgentInstallGuideUrl="http://download.agora.io/install-guide-en.html";AgoraRTC.cnAgentInstallGuideUrl="http://download.agora.io/install-guide-cn.html";AgoraRTC.vertexShaderSource="attribute vec2 a_position;"+"attribute vec2 a_texCoord;"+"uniform vec2 u_resolution;"+"varying vec2 v_texCoord;"+"void main() {"+"vec2 zeroToOne = a_position / u_resolution;"+"   vec2 zeroToTwo = zeroToOne * 2.0;"+"   vec2 clipSpace = zeroToTwo - 1.0;"+"   gl_Position = vec4(clipSpace * vec2(1, -1), 0, 1);"+"v_texCoord = a_texCoord;"+"}";AgoraRTC.yuvShaderSource="precision mediump float;"+"uniform sampler2D Ytex;"+"uniform sampler2D Utex,Vtex;"+"varying vec2 v_texCoord;"+"void main(void) {"+"  float nx,ny,r,g,b,y,u,v;"+"  mediump vec4 txl,ux,vx;"+"  nx=v_texCoord[0];"+"  ny=v_texCoord[1];"+"  y=texture2D(Ytex,vec2(nx,ny)).r;"+"  u=texture2D(Utex,vec2(nx,ny)).r;"+"  v=texture2D(Vtex,vec2(nx,ny)).r;"+"  y=1.1643*(y-0.0625);"+"  u=u-0.5;"+"  v=v-0.5;"+"  r=y+1.5958*v;"+"  g=y-0.39173*u-0.81290*v;"+"  b=y+2.017*u;"+"  gl_FragColor=vec4(r,g,b,1.0);"+"}";AgoraRTC.videoStreamSource='function onSuccess(){postMessage({type:"init",result:!0})}function onFailure(a){postMessage(a)}function sendMessage(a){"videoStat"!=a.command&&console.log(JSON.stringify(a)),that.stream&&that.stream.readyState==WebSocket.OPEN&&that.stream.send(JSON.stringify(a))}var that={};that.stream=void 0,that.init=function(a){return that.stream&&that.stream.readyState===WebSocket.OPEN?(console.warn("stream "+that.getId()+" has been initialized already"),void onSuccess()):(that.local=a.local,that.profile=a.videoProfile,that.stream=new WebSocket(a.url),that.stream.onopen=function(a){console.log((that.local?"local":"remote")+" Stream "),console.log(a),that.profile&&sendMessage({command:"setVideoProfile",profile:that.profile.profile,swapWidthAndHeight:that.profile.swapWidthAndHeight}),that.stream.binaryType="arraybuffer",onSuccess()},that.stream.onclose=function(a){console.log(that.local?"local":"remote"," Stream ",a),that.stream=void 0,onFailure({type:a.type,code:a.code,reason:a.reason})},that.stream.onerror=function(a){console.log(that.local?"local":"remote"," Stream ",a),onFailure({type:a.type,code:a.code,reason:a.reason})},void(that.stream.onmessage=function(a){"string"==typeof a.data?console.log(that.local?"local":"remote"," message from agent ",a.data):a.data instanceof ArrayBuffer?postMessage({type:"message",data:a.data}):a.data instanceof Blob&&console.warn("Blob image data is not supported")}))},that.close=function(){that.stream&&(that.stream.onmessage=void 0,that.stream.close()),that.stream=void 0},self.addEventListener("message",function(a){var b=a.data;switch(b.type){case"init":that.init(b);break;case"send":sendMessage(b.message);break;case"close":that.close()}},!1);';var AgoraCall=function(callback,parameters){if(callback){callback(parameters)}};var L={VideoProfiles:{Profile120P:"120P",Profile120P_3:"120P_3",Profile96P:"96P",Profile48P:"48P",Profile180P:"180P",Profile180P_3:"180P_3",Profile180P_4:"180P_4",Profile240P:"240P",Profile240P_3:"240P_3",Profile240P_4:"240P_4",Profile360P:"360P",Profile360P_3:"360P_3",Profile360P_4:"360P_4",Profile360P_6:"360P_6",Profile360P_7:"360P_7",Profile360P_8:"360P_8",Profile480P:"480P",Profile480P_3:"480P_3",Profile480P_4:"480P_4",Profile480P_6:"480P_6",Profile480P_8:"480P_8",Profile480P_9:"480P_9",Profile720P:"720P",Profile720P_3:"720P_3",Profile720P_5:"720P_5",Profile720P_6:"720P_6"},ErrorCode:{NO_ERROR:0,FAILED:1,INVALID_ARGUMENT:2,NOT_READY:3,NOT_SUPPORTED:4,REFUSED:5,BUFFER_TOO_SMALL:6,NOT_INITIALIZED:7,INVALID_VIEW:8,NO_PERMISSION:9,TIMEDOUT:10,CANCELED:11,TOO_OFTEN:12,BIND_SOCKET:13,NET_DOWN:14,NET_NOBUFS:15,INIT_VIDEO:16,JOIN_CHANNEL_REJECTED:17,LEAVE_CHANNEL_REJECTED:18,ALREADY_IN_USE:19,INVALID_APP_ID:101,INVALID_CHANNEL_NAME:102,CHANNEL_KEY_EXPIRED:109,INVALID_CHANNEL_KEY:110,CONNECTION_INTERRUPTED:111,CONNECTION_LOST:112,LOAD_MEDIA_ENGINE:1001,START_CALL:1002,START_CAMERA:1003,START_VIDEO_RENDER:1004,ADM_GENERAL_ERROR:1005,ADM_JAVA_RESOURCE:1006,ADM_SAMPLE_RATE:1007,ADM_INIT_PLAYOUT:1008,ADM_START_PLAYOUT:1009,ADM_STOP_PLAYOUT:1010,ADM_INIT_RECORDING:1011,ADM_START_RECORDING:1012,ADM_STOP_RECORDING:1013,ADM_RUNTIME_PLAYOUT_ERROR:1015,ADM_RUNTIME_RECORDING_ERROR:1017,ADM_RECORD_AUDIO_FAILED:1018,ADM_INIT_LOOPBACK:1022,ADM_START_LOOPBACK:1023,VDM_CAMERA_NOT_AUTHORIZED:1501}};AgoraRTC.EventDispatcher=function(spec){"use strict";var that={};spec.dispatcher={};spec.dispatcher.eventListeners={};that.addEventListener=function(eventType,listener){spec.dispatcher.eventListeners[eventType]=listener;if(!spec.dispatcher.eventListeners[eventType]){delete spec.dispatcher.eventListeners[eventType]}};that.on=that.addEventListener;that.dispatchEvent=function(event){if(spec.dispatcher.eventListeners.hasOwnProperty(event.type)){spec.dispatcher.eventListeners[event.type](event)}};return that};AgoraRTC.BasicEvent=function(spec){"use strict";var that={};that.type=spec.type;return that};AgoraRTC.StreamEvent=function(spec){"use strict";var that=AgoraRTC.BasicEvent(spec);that.stream=spec.stream;that.msg=spec.msg;return that};AgoraRTC.ClientEvent=function(spec){"use strict";var that=AgoraRTC.BasicEvent(spec);that.uid=spec.uid;that.attr=spec.attr;that.streams=spec.streams;return that};AgoraRTC.MediaEvent=function(spec){"use strict";var that=AgoraRTC.BasicEvent(spec);that.msg=spec.msg;return that};AgoraRTC.Signal=function(spec){var that=AgoraRTC.EventDispatcher(spec);that.connection=new WebSocket(spec.url);that.sendMessage=function(obj,onError){if(that.connection.readyState==WebSocket.OPEN){that.connection.send(JSON.stringify(obj))}else{console.log("connection to agent lost.");onError({error:"not connected"})}};that.close=function(){that.onEvent=onEvent;that.connection.onopen=undefined;that.connection.onclose=undefined;that.connection.onerror=undefined;that.connection.onmessage=undefined;that.connection.close()};that.connection.onopen=function(e){console.log(e);that.dispatchEvent(AgoraRTC.MediaEvent({type:"onopen",event:e}))};that.connection.onclose=function(e){console.log(e);AgoraCall(spec.onFailure,e)};that.connection.onerror=function(e){console.log(e);AgoraCall(spec.onFailure,e)};that.onEvent=onEvent;that.connection.onmessage=function(e){console.log(e);var m=JSON.parse(e.data);if(m.hasOwnProperty("command")){that.dispatchEvent(AgoraRTC.MediaEvent({type:m.command,msg:m}))}else if(m.hasOwnProperty("event")){that.onEvent(m)}};function onEvent(e){console.log(e)}return that};AgoraRTC.Stream=function(spec){var that=AgoraRTC.EventDispatcher(spec);that.stream=undefined;that.render=undefined;that.interval=undefined;that.lastRenderCount=0;that.profile=undefined;that.latency=0;var header=undefined;var yplane=undefined;var uplane=undefined;var vplane=undefined;that.init=function(onSuccess,onFailure){if(that.stream){console.warn("stream "+that.getId()+" has been initialized already");onSuccess();return}var videoStreamFileBlob=new Blob([AgoraRTC.videoStreamSource],{type:"application/javascript"});that.stream=new Worker(URL.createObjectURL(videoStreamFileBlob));that.oninit=function(e){console.log((spec.local?"local":"remote")+" Stream ");console.log(e);if(e.result===true){onSuccess()}else{onFailure()}};that.onclose=function(e){window.clearInterval(that.interval);console.log(spec.local?"local":"remote"," Stream ",e);that.stream=undefined};that.onerror=function(e){window.clearInterval(that.interval);console.log(spec.local?"local":"remote"," Stream ",e);onFailure(e)};that.onmessage=function(e){if(typeof e==="string"){console.log(spec.local?"local":"remote"," message from agent ",e)}else if(e instanceof ArrayBuffer){if(that.render){that.drawImage(e)}}else if(e instanceof Blob){console.warn("Blob image data is not supported")}};that.stream.addEventListener("message",onWorkerMessage,false);that.stream.postMessage({type:"init",local:spec.local,url:AgoraRTC.url,videoProfile:that.profile})};that.close=function(){window.clearInterval(that.interval);if(that.stream){that.stream.removeEventListener("message",onWorkerMessage);
that.stream.postMessage({type:"close"})}that.stream=undefined;that.stop()};that.play=function(div,onFailure){var view=document.getElementById(div);that.stop();that.render=AgoraRender();that.render.start(view,onFailure);if(spec.local){that.stream.postMessage({type:"send",message:{command:"preview",uid:spec.streamID,audio:spec.audio===false?false:true,video:spec.video===false?false:true,screen:spec.screen===true?true:false,viewWidth:view.clientWidth,viewHeight:view.clientHeight}})}else{that.stream.postMessage({type:"send",message:{command:"subscribe",uid:spec.streamID,viewWidth:view.clientWidth,viewHeight:view.clientHeight}})}if(that.interval){window.clearInterval(that.interval)}that.interval=window.setInterval(function(){if(that.render){var fps=that.render.renderImageCount-that.lastRenderCount;that.stream.postMessage({type:"send",message:{command:"videoStat",uid:spec.streamID,fps:fps,frameCount:that.lastRenderCount,latency:that.latency}});that.lastRenderCount=that.render.renderImageCount;that.latency=0}},1e3)};that.stop=function(){if(that.render){that.render.stop()}that.render=undefined;that.lastRenderCount=0};that.bindClient=function(client){that.client=client};that.enableAudio=function(callback){return that.client.enableAudio(that,callback)};that.disableAudio=function(callback){return that.client.disableAudio(that,callback)};that.enableVideo=function(callback){return that.client.enableVideo(that,callback)};that.disableVideo=function(callback){return that.client.disableVideo(that,callback)};that.getId=function(){return spec.streamID};that.drawImage=function(data){if(!header){header=data;if(header.byteLength!=20){console.error("invalid image header "+data.byteLength);resetImageData()}return}if(!yplane){yplane=data;if(yplane.byteLength===20){console.error("invalid image header "+data.byteLength+" "+yplane.byteLength);resetImageData()}return}if(!uplane){uplane=data;if(uplane.byteLength===20){console.error("invalid image header "+data.byteLength+" "+yplane.byteLength+" "+uplane.byteLength);resetImageData()}return}if(!vplane){vplane=data;if(yplane.byteLength!=uplane.byteLength*4||uplane.byteLength!=vplane.byteLength){console.error("invalid image header "+data.byteLength+" "+yplane.byteLength+" "+uplane.byteLength+" "+vplane.byteLength);resetImageData();return}var headerLength=20;var dv=new DataView(header);var format=dv.getUint8(0);var mirror=dv.getUint8(1);var width=dv.getUint16(2);var height=dv.getUint16(4);var left=dv.getUint16(6);var top=dv.getUint16(8);var right=dv.getUint16(10);var bottom=dv.getUint16(12);var rotation=dv.getUint16(14);var ts=dv.getUint32(16);var xWidth=width+left+right;var xHeight=height+top+bottom;var yLength=xWidth*xHeight;var yBegin=headerLength;var yEnd=yBegin+yLength;var uLength=yLength/4;var uBegin=yEnd;var uEnd=uBegin+uLength;var vLength=yLength/4;var vBegin=uEnd;var vEnd=vBegin+vLength;that.render.renderImage({mirror:mirror,width:width,height:height,left:left,top:top,right:right,bottom:bottom,rotation:rotation,yplane:new Uint8Array(yplane),uplane:new Uint8Array(uplane),vplane:new Uint8Array(vplane)});var now32=(Date.now()&4294967295)>>>0;var latency=now32-ts;if(latency>that.latency){that.latency=latency}resetImageData()}};that.setVideoProfile=function(profile){that.profile={};if(typeof profile!=="string"&&typeof profile!=="object"){return false}if(profile.hasOwnProperty("profile")){that.profile.profile=profile.profile}if(profile.hasOwnProperty("swapWidthAndHeight")){that.profile.swapWidthAndHeight=profile.swapWidthAndHeight!==false}else{that.profile.swapWidthAndHeight=false}if(typeof profile=="string"){that.profile.profile=profile}if(that.stream){that.stream.postMessage({type:"send",message:{command:"setVideoProfile",profile:that.profile.profile,swapWidthAndHeight:that.profile.swapWidthAndHeight}})}return true};function resetImageData(){header=undefined;yplane=undefined;uplane=undefined;vplane=undefined}function getReso(w,h){return{width:w,height:h}}var supportedVideoList={true:true,unspecified:true,"120p":getReso(160,120),"240p":getReso(320,240),"360p":getReso(640,360),"480p":getReso(640,480),"720p":getReso(1280,720),"1080p":getReso(1920,1080),"4k":getReso(3840,2160)};function onWorkerMessage(e){var data=e.data;switch(data.type){case"init":that.oninit(data);break;case"close":that.onclose(data);break;case"error":that.onerror(data);break;case"message":that.onmessage(data.data);break}}return that};AgoraRTC.createStream=function(spec){return AgoraRTC.Stream(spec)};AgoraRTC.Client=function(spec){"use strict";var that=AgoraRTC.EventDispatcher(spec);that.signal=undefined;that.localStream=undefined;that.remoteStreams={};that.signalOpen=false;function onSignalError(e){if(that.signalOpen){console.log("signal connection lost.");var message={reason:"LOST_CONNECTION_TO_AGENT",type:"error",eventType:e.type,code:e.code};AgoraCall(that.onFailure,message);that.dispatchEvent(message);return}if(e.type==="error"){return}var message={reason:"CLOSE_BEFORE_OPEN",type:"error",eventType:e.type,code:e.code,agentInstallUrl:getInstallUrl(),agentInstallGuideUrl:getInstallGuideUrl()};AgoraCall(that.onFailure,message);that.dispatchEvent(message)}that.init=function(appId,onSuccess,onFailure){spec.appId=appId;that.onFailure=onFailure;if(!that.signal){try{that.signal=AgoraRTC.Signal({url:AgoraRTC.url,onFailure:onSignalError})}catch(e){console.log("create signal connection failed"+e);AgoraCall(onFailure,e);return}that.signal.on("onopen",function(event){that.signalOpen=true;that.signal.onEvent=function(e){if(e.event==="error"){var name=getErrorCodeName(e.code);if(name){e.reason=name}}else if(e.event==="onUserMuteAudio"){e.event="peer-mute-audio"}else if(e.event==="onUserMuteVideo"){e.event="peer-mute-video"}that.dispatchEvent(AgoraRTC.MediaEvent({type:e.event,msg:e}))};that.getVersion(function(ev){if(ev.version=="1.6.0"||ev.version=="1.6.1"||ev.version=="1.7.0"||ev.version=="1.7.1"||ev.version=="1.7.2"||ev.version=="1.7.3"){initialize(onSuccess,onFailure);return}var message={reason:"INCOMPATIBLE_WEBAGENT",type:"error",version:AgoraRTC.version,agentVersion:ev.version,agentInstallUrl:getInstallUrl(),agentInstallGuideUrl:getInstallGuideUrl()};that.dispatchEvent(message)},function(ev){var message={reason:"INCOMPATIBLE_WEBAGENT",type:"error",version:AgoraRTC.version,agentVersion:ev.version,agentInstallUrl:getInstallUrl(),agentInstallGuideUrl:getInstallGuideUrl()};that.dispatchEvent(message)})})}if(that.signalOpen){initialize(onSuccess,onFailure)}function initialize(onSuccess,onFailure){that.signal.on("initialize",function(ev){var m=ev.msg;if(m.code!=true){that.signal.close();that.signal=undefined;onError(m.code);return}AgoraCall(onSuccess,m)});that.signal.sendMessage({command:"initialize",appId:spec.appId},function(err){AgoraCall(onFailure,err)})}function onError(code){var name=getErrorCodeName(code);if(name){AgoraCall(onFailure,{reason:name});return}AgoraCall(onFailure,{reason:"UNKNOWN_ERROR",code:m.code});return}function getErrorCodeName(code){for(var k in L.ErrorCode){if(L.ErrorCode.hasOwnProperty(k)){if(L.ErrorCode[k]===code){return String(k)}}}return null}that.signal.on("onError",function(ev){var m=ev.msg;onError(m.code)});console.log(that.signal)};that.renewChannelKey=function(channelKey,onSuccess,onFailure){that.signal.sendMessage({command:"renewChannelKey",channelKey:channelKey},function(e){AgoraCall(onFailure,e)});that.signal.on("renewChannelKey",function(ev){var m=ev.msg;if(m.code===true){AgoraCall(onSuccess,m);return}AgoraCall(onFailure,m)})};that.join=function(channelKey,channelName,uid,onSuccess,onFailure){if(channelName.length>64){onFailure(L.ErrorCode.INVALID_CHANNEL_NAME);return}that.signal.sendMessage({command:"joinChannel",channelKey:channelKey,channelName:channelName,uid:uid},function(e){AgoraCall(onFailure,{error:e})});that.signal.on("joinChannel",function(ev){that.signal.on("onAddVideoStream",onAddVideoStream);that.signal.on("onPeerLeave",onPeerLeave);var m=ev.msg;if(m.code==true){if(!m.uid){m.uid=0}AgoraCall(onSuccess,m.uid);return}if(m.code==L.ErrorCode.JOIN_CHANNEL_REJECTED){console.error("Command joinChannel has been rejected by agent. Is this user joined a channel already?");AgoraCall(onFailure,m.code);return}AgoraCall(onFailure,m.code)})};that.leave=function(onSuccess,onFailure){closeLocalRemoteStreams();that.signal.sendMessage({command:"leaveChannel"},function(e){console.log("leave channel failed",e)});that.signal.on("leaveChannel",function(ev){var m=ev.msg;if(m.code===true){AgoraCall(onSuccess,m);return}if(m.code==L.ErrorCode.LEAVE_CHANNEL_REJECTED){console.error("Command leaveChannel has been rejected by agent. Is this user not in a channel?");AgoraCall(onSuccess,m);return}AgoraCall(onFailure,{code:m.code})})};that.publish=function(stream,onSuccess,onFailure){that.localStream=stream;that.signal.sendMessage({command:"unmuteLocal"},function(e){AgoraCall(onFailure,{error:e})});that.signal.on("unmuteLocal",function(ev){var m=ev.msg;if(m.code!=true){AgoraCall(onFailure,{code:m.code});return}AgoraCall(onSuccess,m)});stream.bindClient(that)};that.unpublish=function(stream,onSuccess,onFailure){that.signal.sendMessage({command:"muteLocal"},function(e){AgoraCall(onFailure,{error:e})});that.signal.on("muteLocal",function(ev){var m=ev.msg;if(m.code!=true){AgoraCall(onFailure,{code:m.code});return}AgoraCall(onSuccess,m)})};that.subscribe=function(stream,onFailure){stream.init(function(){that.dispatchEvent(AgoraRTC.StreamEvent({type:"stream-subscribed",stream:stream}))},function(error){AgoraCall(onFailure,error)})};that.unsubscribe=function(streamID,onFailure){console.log("remote streams",that.remoteStreams);if(that.remoteStreams[streamID]==undefined){AgoraCall(onFailure,{error:"no such stream"});return}that.remoteStreams[streamID].close();delete that.remoteStreams[streamID]};that.enableAudio=function(stream,callback){that.signal.on("enableAudio",function(ev){var m=ev.msg;if(callback){AgoraCall(callback,m.code===true)}});that.signal.sendMessage({command:"enableAudio",streamID:stream.getId()},function(e){e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(callback,e)});return true};that.disableAudio=function(stream,callback){that.signal.on("disableAudio",function(ev){var m=ev.msg;if(callback){AgoraCall(callback,m.code===true)}});that.signal.sendMessage({command:"disableAudio",streamID:stream.getId()},function(e){e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(callback,e)});return true};that.enableVideo=function(stream,callback){that.signal.on("enableVideo",function(ev){var m=ev.msg;if(callback){AgoraCall(callback,m.code===true)}});that.signal.sendMessage({command:"enableVideo",streamID:stream.getId()},function(e){e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(callback,e)});return true};that.disableVideo=function(stream,callback){that.signal.on("disableVideo",function(ev){var m=ev.msg;if(callback){AgoraCall(callback,m.code===true)}});that.signal.sendMessage({command:"disableVideo",streamID:stream.getId()},function(e){e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(callback,e)});return true};that.getDevices=function(callback){that.signal.sendMessage({command:"getDevices"},function(e){e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(callback,e)});that.signal.on("getDevices",function(ev){var devices=ev.msg.devices;AgoraCall(callback,devices)})};that.selectDevice=function(device,onFailure){that.signal.sendMessage({command:"selectDevice",device:device},function(e){e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(onFailure,e)});that.signal.on("selectDevice",function(ev){console.log(ev)})};that.startRecording=function(recordingKey,onSuccess,onFailure){that.signal.sendMessage({command:"startRecording",recordingKey:recordingKey},function(e){e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(onFailure,e)});that.signal.on("startRecording",function(ev){var start=ev.msg;if(start.code===true){AgoraCall(onSuccess,start)}else{AgoraCall(onFailure,start)}})};that.stopRecording=function(recordingKey,onSuccess,onFailure){that.signal.sendMessage({command:"stopRecording",recordingKey:recordingKey},function(e){e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(onFailure,e)});that.signal.on("stopRecording",function(ev){var stop=ev.msg;if(stop.code===true){AgoraCall(onSuccess,stop)}else{AgoraCall(onFailure,stop)}})};that.queryRecordingStatus=function(onStatus){that.signal.sendMessage({command:"queryRecordingStatus"},function(e){e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(onStatus,e)});that.signal.on("queryRecordingStatus",function(ev){onStatus(ev.msg)})};that.close=function(){closeLocalRemoteStreams();if(that.signal){that.signal.close();that.signal=undefined}};that.getVersion=function(onSuccess,onFailure){that.signal.sendMessage({command:"getVersion"});that.signal.on("getVersion",function(ev){var m=ev.msg;if(m.code===true){onSuccess(m)}else{onFailure(m)}})};that.setParameters=function(parameters){that.signal.sendMessage({command:"setParameters",parameters:JSON.stringify(parameters)})};that.setEncryptionMode=function(encryptionMode){that.signal.sendMessage({command:"setEncryptionMode",mode:encryptionMode})};that.setEncryptionSecret=function(secret){that.signal.sendMessage({command:"setEncryptionSecret",secret:secret})};that.getWindows=function(callback){that.signal.sendMessage({command:"getWindows"},function(e){e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(callback,e)});that.signal.on("getWindows",function(ev){var windows=ev.msg.windows;AgoraCall(callback,windows)})};that.startScreenSharing=function(window,onSuccess,onFailure){that.signal.sendMessage({command:"startScreenSharing",window:window},function(e){e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(onFailure,e)});that.signal.on("startScreenSharing",function(e){AgoraCall(onSuccess,e)})};that.setScreenSharingWindow=function(window,onSuccess,onFailure){that.signal.sendMessage({command:"setScreenSharingWindow",window:window},function(e){e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(onFailure,e)});that.signal.on("setScreenSharingWindow",function(e){AgoraCall(onSuccess,e);console.log(e)})};that.stopScreenSharing=function(onSuccess,onFailure){that.signal.sendMessage({command:"stopScreenSharing"},function(e){e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(onFailure,e)});that.signal.on("stopScreenSharing",function(e){AgoraCall(onSuccess,e);console.log(e)})};function onPeerLeave(ev){var m=ev.msg;if(that.remoteStreams.hasOwnProperty(m.uid)){that.remoteStreams[m.uid].close();delete that.remoteStreams[m.uid];console.log("remote streams after peer leave ",that.remoteStreams)}that.dispatchEvent(AgoraRTC.ClientEvent({type:"peer-leave",uid:m.uid}))}function onAddVideoStream(ev){var m=ev.msg;if(!that.remoteStreams.hasOwnProperty(m.uid)){var stream=AgoraRTC.Stream({streamID:m.uid,local:false,audio:m.audio,video:m.video,screen:m.screen});stream.bindClient(that);that.remoteStreams[m.uid]=stream}that.dispatchEvent(AgoraRTC.StreamEvent({type:"stream-added",stream:that.remoteStreams[m.uid]}));console.log("remote streams",that.remoteStreams)}function getInstallUrl(){if(navigator.appVersion.indexOf("Mac")!=-1){return AgoraRTC.macAgentInstallUrl}return AgoraRTC.winAgentInstallUrl}function getInstallGuideUrl(){var userLang=navigator.language||navigator.userLanguage;if(userLang.indexOf("zh")!=-1){return AgoraRTC.cnAgentInstallGuideUrl}return AgoraRTC.enAgentInstallGuideUrl}function closeLocalRemoteStreams(){if(that.localStream){that.localStream.close()}that.localStream=undefined;for(var streamId in that.remoteStreams){if(that.remoteStreams.hasOwnProperty(streamId)){that.remoteStreams[streamId].close();delete that.remoteStreams[streamId]}}}return that};AgoraRTC.createRtcClient=function(){return AgoraRTC.Client({})};
//# sourceMappingURL=dist/AgoraRtcAgentSDK.min.js.map
"use strict";(function(root,factory){if(typeof define==="function"&&define.amd){define([],factory)}else{var lib=factory.call(root);Object.keys(lib).forEach(function(key){root[key]=lib[key]})}})(this,function(){var topWindow=this;function error(msg){if(topWindow.console){if(topWindow.console.error){topWindow.console.error(msg)}else if(topWindow.console.log){topWindow.console.log(msg)}}}function isInIFrame(w){w=w||topWindow;return w!==w.top}function makeFailHTML(msg){return""+'<table style="background-color: #8CE; width: 100%; height: 100%;"><tr>'+'<td align="center">'+'<div style="display: table-cell; vertical-align: middle;">'+'<div style="">'+msg+"</div>"+"</div>"+"</td></tr></table>"}var GET_A_WEBGL_BROWSER=""+"This page requires a browser that supports WebGL.<br/>"+'<a href="http://get.webgl.org">Click here to upgrade your browser.</a>';var OTHER_PROBLEM=""+"It doesn't appear your computer can support WebGL.<br/>"+'<a href="http://get.webgl.org/troubleshooting/">Click here for more information.</a>';function create3DContext(canvas,opt_attribs){var names=["webgl","experimental-webgl"];var context=null;for(var ii=0;ii<names.length;++ii){try{context=canvas.getContext(names[ii],opt_attribs)}catch(e){}if(context){break}}return context}function setupWebGL(canvas,opt_attribs){function showLink(str){var container=canvas.parentNode;if(container){container.innerHTML=makeFailHTML(str)}}if(!topWindow.WebGLRenderingContext){showLink(GET_A_WEBGL_BROWSER);return null}var context=create3DContext(canvas,opt_attribs);if(!context){showLink(OTHER_PROBLEM)}return context}function updateCSSIfInIFrame(){if(isInIFrame()){document.body.className="iframe"}}function getWebGLContext(canvas,opt_attribs,opt_options){var options=opt_options||{};if(isInIFrame()){updateCSSIfInIFrame();if(!options.dontResize&&options.resize!==false){var width=canvas.clientWidth;var height=canvas.clientHeight;canvas.width=width;canvas.height=height}}else if(!options.noTitle&&options.title!==false){var title=document.title;var h1=document.createElement("h1");h1.innerText=title;document.body.insertBefore(h1,document.body.children[0])}var gl=setupWebGL(canvas,opt_attribs);return gl}function loadShader(gl,shaderSource,shaderType,opt_errorCallback){var errFn=opt_errorCallback||error;var shader=gl.createShader(shaderType);gl.shaderSource(shader,shaderSource);gl.compileShader(shader);var compiled=gl.getShaderParameter(shader,gl.COMPILE_STATUS);if(!compiled){var lastError=gl.getShaderInfoLog(shader);errFn("*** Error compiling shader '"+shader+"':"+lastError);gl.deleteShader(shader);return null}return shader}function createProgram(gl,shaders,opt_attribs,opt_locations,opt_errorCallback){var errFn=opt_errorCallback||error;var program=gl.createProgram();shaders.forEach(function(shader){gl.attachShader(program,shader)});if(opt_attribs){obj_attrib.forEach(function(attrib,ndx){gl.bindAttribLocation(program,opt_locations?opt_locations[ndx]:ndx,attrib)})}gl.linkProgram(program);var linked=gl.getProgramParameter(program,gl.LINK_STATUS);if(!linked){var lastError=gl.getProgramInfoLog(program);errFn("Error in program linking:"+lastError);gl.deleteProgram(program);return null}return program}function createShaderFromScript(gl,scriptId,opt_shaderType,opt_errorCallback){var shaderSource="";var shaderType;var shaderScript=document.getElementById(scriptId);if(!shaderScript){throw"*** Error: unknown script element"+scriptId}shaderSource=shaderScript.text;if(!opt_shaderType){if(shaderScript.type==="x-shader/x-vertex"){shaderType=gl.VERTEX_SHADER}else if(shaderScript.type==="x-shader/x-fragment"){shaderType=gl.FRAGMENT_SHADER}else if(shaderType!==gl.VERTEX_SHADER&&shaderType!==gl.FRAGMENT_SHADER){throw"*** Error: unknown shader type"}}return loadShader(gl,shaderSource,opt_shaderType?opt_shaderType:shaderType,opt_errorCallback)}var defaultShaderType=["VERTEX_SHADER","FRAGMENT_SHADER"];function createProgramFromScripts(gl,shaderScriptIds,opt_attribs,opt_locations,opt_errorCallback){var shaders=[];for(var ii=0;ii<shaderScriptIds.length;++ii){shaders.push(createShaderFromScript(gl,shaderScriptIds[ii],gl[defaultShaderType[ii]],opt_errorCallback))}return createProgram(gl,shaders,opt_attribs,opt_locations,opt_errorCallback)}function createProgramFromSources(gl,shaderSources,opt_attribs,opt_locations,opt_errorCallback){var shaders=[];for(var ii=0;ii<shaderSources.length;++ii){shaders.push(loadShader(gl,shaderSources[ii],gl[defaultShaderType[ii]],opt_errorCallback))}return createProgram(gl,shaders,opt_attribs,opt_locations,opt_errorCallback)}function getBindPointForSamplerType(gl,type){if(type===gl.SAMPLER_2D)return gl.TEXTURE_2D;if(type===gl.SAMPLER_CUBE)return gl.TEXTURE_CUBE_MAP}function createUniformSetters(gl,program){var textureUnit=0;function createUniformSetter(program,uniformInfo){var location=gl.getUniformLocation(program,uniformInfo.name);var type=uniformInfo.type;var isArray=uniformInfo.size>1&&uniformInfo.name.substr(-3)==="[0]";if(type===gl.FLOAT&&isArray){return function(v){gl.uniform1fv(location,v)}}if(type===gl.FLOAT){return function(v){gl.uniform1f(location,v)}}if(type===gl.FLOAT_VEC2){return function(v){gl.uniform2fv(location,v)}}if(type===gl.FLOAT_VEC3){return function(v){gl.uniform3fv(location,v)}}if(type===gl.FLOAT_VEC4){return function(v){gl.uniform4fv(location,v)}}if(type===gl.INT&&isArray){return function(v){gl.uniform1iv(location,v)}}if(type===gl.INT){return function(v){gl.uniform1i(location,v)}}if(type===gl.INT_VEC2){return function(v){gl.uniform2iv(location,v)}}if(type===gl.INT_VEC3){return function(v){gl.uniform3iv(location,v)}}if(type===gl.INT_VEC4){return function(v){gl.uniform4iv(location,v)}}if(type===gl.BOOL){return function(v){gl.uniform1iv(location,v)}}if(type===gl.BOOL_VEC2){return function(v){gl.uniform2iv(location,v)}}if(type===gl.BOOL_VEC3){return function(v){gl.uniform3iv(location,v)}}if(type===gl.BOOL_VEC4){return function(v){gl.uniform4iv(location,v)}}if(type===gl.FLOAT_MAT2){return function(v){gl.uniformMatrix2fv(location,false,v)}}if(type===gl.FLOAT_MAT3){return function(v){gl.uniformMatrix3fv(location,false,v)}}if(type===gl.FLOAT_MAT4){return function(v){gl.uniformMatrix4fv(location,false,v)}}if((type===gl.SAMPLER_2D||type===gl.SAMPLER_CUBE)&&isArray){var units=[];for(var ii=0;ii<info.size;++ii){units.push(textureUnit++)}return function(bindPoint,units){return function(textures){gl.uniform1iv(location,units);textures.forEach(function(texture,index){gl.activeTexture(gl.TEXTURE0+units[index]);gl.bindTexture(bindPoint,texture)})}}(getBindPointForSamplerType(gl,type),units)}if(type===gl.SAMPLER_2D||type===gl.SAMPLER_CUBE){return function(bindPoint,unit){return function(texture){gl.uniform1i(location,unit);gl.activeTexture(gl.TEXTURE0+unit);gl.bindTexture(bindPoint,texture)}}(getBindPointForSamplerType(gl,type),textureUnit++)}throw"unknown type: 0x"+type.toString(16)}var uniformSetters={};var numUniforms=gl.getProgramParameter(program,gl.ACTIVE_UNIFORMS);for(var ii=0;ii<numUniforms;++ii){var uniformInfo=gl.getActiveUniform(program,ii);if(!uniformInfo){break}var name=uniformInfo.name;if(name.substr(-3)==="[0]"){name=name.substr(0,name.length-3)}var setter=createUniformSetter(program,uniformInfo);uniformSetters[name]=setter}return uniformSetters}function setUniforms(setters,values){Object.keys(values).forEach(function(name){var setter=setters[name];if(setter){setter(values[name])}})}function createAttributeSetters(gl,program){var attribSetters={};function createAttribSetter(index){return function(b){gl.bindBuffer(gl.ARRAY_BUFFER,b.buffer);gl.enableVertexAttribArray(index);gl.vertexAttribPointer(index,b.numComponents||b.size,b.type||gl.FLOAT,b.normalize||false,b.stride||0,b.offset||0)}}var numAttribs=gl.getProgramParameter(program,gl.ACTIVE_ATTRIBUTES);for(var ii=0;ii<numAttribs;++ii){var attribInfo=gl.getActiveAttrib(program,ii);if(!attribInfo){break}var index=gl.getAttribLocation(program,attribInfo.name);attribSetters[attribInfo.name]=createAttribSetter(index)}return attribSetters}function setAttributes(setters,buffers){Object.keys(buffers).forEach(function(name){var setter=setters[name];if(setter){setter(buffers[name])}})}function createProgramInfo(gl,shaderSources,opt_attribs,opt_locations,opt_errorCallback){shaderSources=shaderSources.map(function(source){var script=document.getElementById(source);return script?script.text:source});var program=createProgramFromSources(gl,shaderSources,opt_attribs,opt_locations,opt_errorCallback);if(!program){return null}var uniformSetters=createUniformSetters(gl,program);var attribSetters=createAttributeSetters(gl,program);return{program:program,uniformSetters:uniformSetters,attribSetters:attribSetters}}function setBuffersAndAttributes(gl,setters,buffers){setAttributes(setters,buffers.attribs);if(buffers.indices){gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER,buffers.indices)}}var browserPrefixes=["","MOZ_","OP_","WEBKIT_"];function getExtensionWithKnownPrefixes(gl,name){for(var ii=0;ii<browserPrefixes.length;++ii){var prefixedName=browserPrefixes[ii]+name;var ext=gl.getExtension(prefixedName);if(ext){return ext}}}function resizeCanvasToDisplaySize(canvas,multiplier){multiplier=multiplier||1;var width=canvas.clientWidth*multiplier;var height=canvas.clientHeight*multiplier;if(canvas.width!==width||canvas.height!==height){canvas.width=width;canvas.height=height;return true}return false}function getIFrameForWindow(window){if(!isInIFrame(window)){return}var iframes=window.parent.document.getElementsByTagName("iframe");for(var ii=0;ii<iframes.length;++ii){var iframe=iframes[ii];if(iframe.contentDocument===window.document){return iframe}}}function isFrameVisible(window){try{var iframe=getIFrameForWindow(window);if(!iframe){return true}var bounds=iframe.getBoundingClientRect();var isVisible=bounds.top<window.parent.innerHeight&&bounds.bottom>=0&&bounds.left<window.parent.innerWidth&&bounds.right>=0;return isVisible&&isFrameVisible(window.parent)}catch(e){return true}}function isOnScreen(element){var isVisible=true;if(element){var bounds=element.getBoundingClientRect();isVisible=bounds.top<topWindow.innerHeight&&bounds.bottom>=0}return isVisible&&isFrameVisible(topWindow)}function augmentTypedArray(typedArray,numComponents){var cursor=0;typedArray.push=function(){for(var ii=0;ii<arguments.length;++ii){var value=arguments[ii];if(value instanceof Array||value.buffer&&value.buffer instanceof ArrayBuffer){for(var jj=0;jj<value.length;++jj){typedArray[cursor++]=value[jj]}}else{typedArray[cursor++]=value}}};typedArray.reset=function(opt_index){cursor=opt_index||0};typedArray.numComponents=numComponents;Object.defineProperty(typedArray,"numElements",{get:function(){return this.length/this.numComponents|0}});return typedArray}function createAugmentedTypedArray(numComponents,numElements,opt_type){var Type=opt_type||Float32Array;return augmentTypedArray(new Type(numComponents*numElements),numComponents)}function createBufferFromTypedArray(gl,array,type,drawType){type=type||gl.ARRAY_BUFFER;var buffer=gl.createBuffer();gl.bindBuffer(type,buffer);gl.bufferData(type,array,drawType||gl.STATIC_DRAW);return buffer}function allButIndices(name){return name!=="indices"}function createMapping(obj){var mapping={};Object.keys(obj).filter(allButIndices).forEach(function(key){mapping["a_"+key]=key});return mapping}function getGLTypeForTypedArray(gl,typedArray){if(typedArray instanceof Int8Array){return gl.BYTE}if(typedArray instanceof Uint8Array){return gl.UNSIGNED_BYTE}if(typedArray instanceof Int16Array){return gl.SHORT}if(typedArray instanceof Uint16Array){return gl.UNSIGNED_SHORT}if(typedArray instanceof Int32Array){return gl.INT}if(typedArray instanceof Uint32Array){return gl.UNSIGNED_INT}if(typedArray instanceof Float32Array){return gl.FLOAT}throw"unsupported typed array type"}function getNormalizationForTypedArray(typedArray){if(typedArray instanceof Int8Array){return true}if(typedArray instanceof Uint8Array){return true}return false}function isArrayBuffer(a){return a.buffer&&a.buffer instanceof ArrayBuffer}function guessNumComponentsFromName(name,length){var numComponents;if(name.indexOf("coord")>=0){numComponents=2}else if(name.indexOf("color")>=0){numComponents=4}else{numComponents=3}if(length%numComponents>0){throw"can not guess numComponents. You should specify it."}return numComponents}function makeTypedArray(array,name){if(isArrayBuffer(array)){return array}if(Array.isArray(array)){array={data:array}}if(!array.numComponents){array.numComponents=guessNumComponentsFromName(name,array.length)}var type=array.type;if(!type){if(name==="indices"){type=Uint16Array}}var typedArray=createAugmentedTypedArray(array.numComponents,array.data.length/array.numComponents|0,type);typedArray.push(array.data);return typedArray}function createAttribsFromArrays(gl,arrays,opt_mapping){var mapping=opt_mapping||createMapping(arrays);var attribs={};Object.keys(mapping).forEach(function(attribName){var bufferName=mapping[attribName];var array=makeTypedArray(arrays[bufferName],bufferName);attribs[attribName]={buffer:createBufferFromTypedArray(gl,array),numComponents:array.numComponents||guessNumComponentsFromName(bufferName),type:getGLTypeForTypedArray(gl,array),normalize:getNormalizationForTypedArray(array)}});return attribs}function getNumElementsFromNonIndexedArrays(arrays){var key=Object.keys(arrays)[0];var array=arrays[key];if(isArrayBuffer(array)){return array.numElements}else{return array.data.length/array.numComponents}}function createBufferInfoFromArrays(gl,arrays,opt_mapping){var bufferInfo={attribs:createAttribsFromArrays(gl,arrays,opt_mapping)};var indices=arrays.indices;if(indices){indices=makeTypedArray(indices,"indices");bufferInfo.indices=createBufferFromTypedArray(gl,indices,gl.ELEMENT_ARRAY_BUFFER);bufferInfo.numElements=indices.length}else{bufferInfo.numElements=getNumElementsFromNonIndexedArrays(arrays)}return bufferInfo}function createBuffersFromArrays(gl,arrays){var buffers={};Object.keys(arrays).forEach(function(key){var type=key==="indices"?gl.ELEMENT_ARRAY_BUFFER:gl.ARRAY_BUFFER;var array=makeTypedArray(arrays[key],name);buffers[key]=createBufferFromTypedArray(gl,array,type)});if(arrays.indices){buffers.numElements=arrays.indices.length}else if(arrays.position){buffers.numElements=arrays.position.length/3}return buffers}function drawBufferInfo(gl,type,bufferInfo,count,offset){var indices=bufferInfo.indices;var numElements=count===undefined?bufferInfo.numElements:count;offset=offset===undefined?offset:0;if(indices){gl.drawElements(type,numElements,gl.UNSIGNED_SHORT,offset)}else{gl.drawArrays(type,offset,numElements)}}function drawObjectList(gl,objectsToDraw){var lastUsedProgramInfo=null;var lastUsedBufferInfo=null;objectsToDraw.forEach(function(object){var programInfo=object.programInfo;var bufferInfo=object.bufferInfo;var bindBuffers=false;if(programInfo!==lastUsedProgramInfo){lastUsedProgramInfo=programInfo;gl.useProgram(programInfo.program);bindBuffers=true}if(bindBuffers||bufferInfo!==lastUsedBufferInfo){lastUsedBufferInfo=bufferInfo;setBuffersAndAttributes(gl,programInfo.attribSetters,bufferInfo)}setUniforms(programInfo.uniformSetters,object.uniforms);drawBufferInfo(gl,gl.TRIANGLES,bufferInfo)})}return{createAugmentedTypedArray:createAugmentedTypedArray,createAttribsFromArrays:createAttribsFromArrays,createBuffersFromArrays:createBuffersFromArrays,createBufferInfoFromArrays:createBufferInfoFromArrays,createAttributeSetters:createAttributeSetters,createProgram:createProgram,createProgramFromScripts:createProgramFromScripts,createProgramFromSources:createProgramFromSources,createProgramInfo:createProgramInfo,createUniformSetters:createUniformSetters,drawBufferInfo:drawBufferInfo,drawObjectList:drawObjectList,getWebGLContext:getWebGLContext,updateCSSIfInIFrame:updateCSSIfInIFrame,getExtensionWithKnownPrefixes:getExtensionWithKnownPrefixes,resizeCanvasToDisplaySize:resizeCanvasToDisplaySize,setAttributes:setAttributes,setBuffersAndAttributes:setBuffersAndAttributes,setUniforms:setUniforms,setupWebGL:setupWebGL}});function AgoraRender(){var gl=undefined;var program=undefined;var positionLocation=undefined;var texCoordLocation=undefined;var yTexture=undefined;var uTexture=undefined;var vTexture=undefined;var texCoordBuffer=undefined;var surfaceBuffer=undefined;var that={view:undefined,mirrorView:false,container:undefined,canvas:undefined,renderImageCount:0,initWidth:0,initHeight:0,initRotation:0,canvasUpdated:false,clientWidth:0,clientHeight:0};that.start=function(view,onFailure){initCanvas(view,that.mirrorView,view.clientWidth,view.clientHeight,that.initRotation,onFailure)};that.stop=function(){gl=undefined;program=undefined;positionLocation=undefined;texCoordLocation=undefined;deleteTexture(yTexture);deleteTexture(uTexture);deleteTexture(vTexture);yTexture=undefined;uTexture=undefined;vTexture=undefined;deleteBuffer(texCoordBuffer);deleteBuffer(surfaceBuffer);texCoordBuffer=undefined;surfaceBuffer=undefined;if(that.container){that.container.removeChild(that.canvas)}if(that.view){that.view.removeChild(that.container)}that.canvas=undefined;that.container=undefined;that.view=undefined;that.mirrorView=false};that.renderImage=function(image){if(!gl){return}if(image.width!=that.initWidth||image.height!=that.initHeight||image.rotation!=that.initRotation||image.mirror!=that.mirrorView){var view=that.view;that.stop();console.log("init canvas "+image.width+"*"+image.height+" rotation "+image.rotation);initCanvas(view,image.mirror,image.width,image.height,image.rotation,function(e){console.error("init canvas "+image.width+"*"+image.height+" rotation "+image.rotation+" failed. "+e)})}gl.bindBuffer(gl.ARRAY_BUFFER,texCoordBuffer);var xWidth=image.width+image.left+image.right;var xHeight=image.height+image.top+image.bottom;gl.bufferData(gl.ARRAY_BUFFER,new Float32Array([image.left/xWidth,image.bottom/xHeight,1-image.right/xWidth,image.bottom/xHeight,image.left/xWidth,1-image.top/xHeight,image.left/xWidth,1-image.top/xHeight,1-image.right/xWidth,image.bottom/xHeight,1-image.right/xWidth,1-image.top/xHeight]),gl.STATIC_DRAW);gl.enableVertexAttribArray(texCoordLocation);gl.vertexAttribPointer(texCoordLocation,2,gl.FLOAT,false,0,0);uploadYuv(xWidth,xHeight,image.yplane,image.uplane,image.vplane);updateCanvas(image.rotation,image.width,image.height);gl.drawArrays(gl.TRIANGLES,0,6);that.renderImageCount+=1};function uploadYuv(width,height,yplane,uplane,vplane){gl.activeTexture(gl.TEXTURE0);gl.bindTexture(gl.TEXTURE_2D,yTexture);gl.texImage2D(gl.TEXTURE_2D,0,gl.LUMINANCE,width,height,0,gl.LUMINANCE,gl.UNSIGNED_BYTE,yplane);var e=gl.getError();if(e!=gl.NO_ERROR){console.log("upload y plane ",width,height,yplane.byteLength," error",e)}gl.activeTexture(gl.TEXTURE1);gl.bindTexture(gl.TEXTURE_2D,uTexture);gl.texImage2D(gl.TEXTURE_2D,0,gl.LUMINANCE,width/2,height/2,0,gl.LUMINANCE,gl.UNSIGNED_BYTE,uplane);var e=gl.getError();if(e!=gl.NO_ERROR){console.log("upload u plane ",width,height,uplane.byteLength,"  error",e)}gl.activeTexture(gl.TEXTURE2);gl.bindTexture(gl.TEXTURE_2D,vTexture);gl.texImage2D(gl.TEXTURE_2D,0,gl.LUMINANCE,width/2,height/2,0,gl.LUMINANCE,gl.UNSIGNED_BYTE,vplane);var e=gl.getError();if(e!=gl.NO_ERROR){console.log("upload v plane ",width,height,vplane.byteLength,"  error",e)}}function deleteBuffer(buffer){if(buffer&&that.gl){that.gl.deleteBuffer(buffer)}}function deleteTexture(texture){if(texture&&that.gl){that.gl.deleteTexture(texture)}}function initCanvas(view,mirror,width,height,rotation,onFailure){that.clientWidth=view.clientWidth;that.clientHeight=view.clientHeight;that.view=view;that.mirrorView=mirror;that.canvasUpdated=false;that.container=document.createElement("div");that.container.style.width="100%";that.container.style.height="100%";that.container.style.display="flex";that.container.style.justifyContent="center";that.container.style.alignItems="center";that.view.appendChild(that.container);that.canvas=document.createElement("canvas");if(rotation==0||rotation==180){that.canvas.width=width;that.canvas.height=height}else{that.canvas.width=height;that.canvas.height=width}that.initWidth=width;that.initHeight=height;that.initRotation=rotation;if(that.mirrorView){that.canvas.style.transform="rotateY(180deg)"}that.container.appendChild(that.canvas);try{gl=that.canvas.getContext("webgl")||that.canvas.getContext("experimental-webgl")}catch(e){console.log(e)}if(!gl){gl=undefined;onFailure({error:"Browser not support! No WebGL detected."});return}gl.clearColor(0,0,0,1);gl.enable(gl.DEPTH_TEST);gl.depthFunc(gl.LEQUAL);gl.clear(gl.COLOR_BUFFER_BIT|gl.DEPTH_BUFFER_BIT);program=createProgramFromSources(gl,[AgoraRTC.vertexShaderSource,AgoraRTC.yuvShaderSource]);gl.useProgram(program);initTextures()}function initTextures(){positionLocation=gl.getAttribLocation(program,"a_position");texCoordLocation=gl.getAttribLocation(program,"a_texCoord");surfaceBuffer=gl.createBuffer();texCoordBuffer=gl.createBuffer();gl.activeTexture(gl.TEXTURE0);yTexture=gl.createTexture();gl.bindTexture(gl.TEXTURE_2D,yTexture);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_WRAP_S,gl.CLAMP_TO_EDGE);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_WRAP_T,gl.CLAMP_TO_EDGE);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_MIN_FILTER,gl.NEAREST);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_MAG_FILTER,gl.NEAREST);gl.activeTexture(gl.TEXTURE1);uTexture=gl.createTexture();gl.bindTexture(gl.TEXTURE_2D,uTexture);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_WRAP_S,gl.CLAMP_TO_EDGE);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_WRAP_T,gl.CLAMP_TO_EDGE);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_MIN_FILTER,gl.NEAREST);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_MAG_FILTER,gl.NEAREST);gl.activeTexture(gl.TEXTURE2);vTexture=gl.createTexture();gl.bindTexture(gl.TEXTURE_2D,vTexture);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_WRAP_S,gl.CLAMP_TO_EDGE);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_WRAP_T,gl.CLAMP_TO_EDGE);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_MIN_FILTER,gl.NEAREST);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_MAG_FILTER,gl.NEAREST);var y=gl.getUniformLocation(program,"Ytex");gl.uniform1i(y,0);var u=gl.getUniformLocation(program,"Utex");gl.uniform1i(u,1);var v=gl.getUniformLocation(program,"Vtex");gl.uniform1i(v,2)}function updateCanvas(rotation,width,height){if(that.canvasUpdated){return}that.canvas.style.width="100%";that.canvas.style.height="100%";try{if(rotation===0||rotation===180){if(that.clientWidth/that.clientHeight>width/height){that.canvas.style.width=that.clientHeight*width/height+"px"}else if(that.clientWidth/that.clientHeight<width/height){that.canvas.style.height=that.clientWidth*height/width+"px"}}else{if(that.clientHeight/that.clientWidth>width/height){that.canvas.style.height=that.clientWidth*width/height+"px"}else if(that.clientHeight/that.clientWidth<width/height){that.canvas.style.width=that.clientHeight*height/width+"px"}}}catch(e){console.log("updateCanvas 00001 gone "+that.canvas);console.log(that);console.error(e);return}gl.bindBuffer(gl.ARRAY_BUFFER,surfaceBuffer);gl.enableVertexAttribArray(positionLocation);gl.vertexAttribPointer(positionLocation,2,gl.FLOAT,false,0,0);var p1={x:0,y:0};var p2={x:width,y:0};var p3={x:width,y:height};var p4={x:0,y:height};var pp1=p1,pp2=p2,pp3=p3,pp4=p4;switch(rotation){case 0:break;case 90:pp1=p2;pp2=p3;pp3=p4;pp4=p1;break;case 180:pp1=p3;pp2=p4;pp3=p1;pp4=p2;break;case 270:pp1=p4;pp2=p1;pp3=p2;pp4=p3;break;default:}gl.bufferData(gl.ARRAY_BUFFER,new Float32Array([pp1.x,pp1.y,pp2.x,pp2.y,pp4.x,pp4.y,pp4.x,pp4.y,pp2.x,pp2.y,pp3.x,pp3.y]),gl.STATIC_DRAW);var resolutionLocation=gl.getUniformLocation(program,"u_resolution");gl.uniform2f(resolutionLocation,width,height);that.canvasUpdated=true}return that}var AgoraRTC=function(){"use strict";var AgoraRTC={};Object.defineProperties(AgoraRTC,{version:{get:function(){return"1.7.4"}},name:{get:function(){return"AgoraWebSDK"}}});return AgoraRTC}();AgoraRTC.url="wss://localhost.agora.io:8921/";AgoraRTC.macAgentInstallUrl="http://download.agora.io/AgoraWebAgent-1.7.4.pkg";AgoraRTC.winAgentInstallUrl="http://download.agora.io/AgoraWebAgentSetup-1.7.4.exe";AgoraRTC.enAgentInstallGuideUrl="http://download.agora.io/install-guide-en.html";AgoraRTC.cnAgentInstallGuideUrl="http://download.agora.io/install-guide-cn.html";AgoraRTC.vertexShaderSource="attribute vec2 a_position;"+"attribute vec2 a_texCoord;"+"uniform vec2 u_resolution;"+"varying vec2 v_texCoord;"+"void main() {"+"vec2 zeroToOne = a_position / u_resolution;"+"   vec2 zeroToTwo = zeroToOne * 2.0;"+"   vec2 clipSpace = zeroToTwo - 1.0;"+"   gl_Position = vec4(clipSpace * vec2(1, -1), 0, 1);"+"v_texCoord = a_texCoord;"+"}";AgoraRTC.yuvShaderSource="precision mediump float;"+"uniform sampler2D Ytex;"+"uniform sampler2D Utex,Vtex;"+"varying vec2 v_texCoord;"+"void main(void) {"+"  float nx,ny,r,g,b,y,u,v;"+"  mediump vec4 txl,ux,vx;"+"  nx=v_texCoord[0];"+"  ny=v_texCoord[1];"+"  y=texture2D(Ytex,vec2(nx,ny)).r;"+"  u=texture2D(Utex,vec2(nx,ny)).r;"+"  v=texture2D(Vtex,vec2(nx,ny)).r;"+"  y=1.1643*(y-0.0625);"+"  u=u-0.5;"+"  v=v-0.5;"+"  r=y+1.5958*v;"+"  g=y-0.39173*u-0.81290*v;"+"  b=y+2.017*u;"+"  gl_FragColor=vec4(r,g,b,1.0);"+"}";AgoraRTC.videoStreamSource='function onSuccess(){postMessage({type:"init",result:!0})}function onFailure(a){postMessage(a)}function sendMessage(a){"videoStat"!=a.command&&console.log(JSON.stringify(a)),that.stream&&that.stream.readyState==WebSocket.OPEN&&that.stream.send(JSON.stringify(a))}var that={};that.stream=void 0,that.init=function(a){return that.stream&&that.stream.readyState===WebSocket.OPEN?(console.warn("stream "+that.getId()+" has been initialized already"),void onSuccess()):(that.local=a.local,that.profile=a.videoProfile,that.stream=new WebSocket(a.url),that.stream.onopen=function(a){console.log((that.local?"local":"remote")+" Stream "),console.log(a),that.profile&&sendMessage({command:"setVideoProfile",profile:that.profile.profile,swapWidthAndHeight:that.profile.swapWidthAndHeight}),that.stream.binaryType="arraybuffer",onSuccess()},that.stream.onclose=function(a){console.log(that.local?"local":"remote"," Stream ",a),that.stream=void 0,onFailure({type:a.type,code:a.code,reason:a.reason})},that.stream.onerror=function(a){console.log(that.local?"local":"remote"," Stream ",a),onFailure({type:a.type,code:a.code,reason:a.reason})},void(that.stream.onmessage=function(a){"string"==typeof a.data?console.log(that.local?"local":"remote"," message from agent ",a.data):a.data instanceof ArrayBuffer?postMessage({type:"message",data:a.data}):a.data instanceof Blob&&console.warn("Blob image data is not supported")}))},that.close=function(){that.stream&&(that.stream.onmessage=void 0,that.stream.close()),that.stream=void 0},self.addEventListener("message",function(a){var b=a.data;switch(b.type){case"init":that.init(b);break;case"send":sendMessage(b.message);break;case"close":that.close()}},!1);';var AgoraCall=function(callback,parameters){if(callback){callback(parameters)}};var L={VideoProfiles:{Profile120P:"120P",Profile120P_3:"120P_3",Profile96P:"96P",Profile48P:"48P",Profile180P:"180P",Profile180P_3:"180P_3",Profile180P_4:"180P_4",Profile240P:"240P",Profile240P_3:"240P_3",Profile240P_4:"240P_4",Profile360P:"360P",Profile360P_3:"360P_3",Profile360P_4:"360P_4",Profile360P_6:"360P_6",Profile360P_7:"360P_7",Profile360P_8:"360P_8",Profile480P:"480P",Profile480P_3:"480P_3",Profile480P_4:"480P_4",Profile480P_6:"480P_6",Profile480P_8:"480P_8",Profile480P_9:"480P_9",Profile720P:"720P",Profile720P_3:"720P_3",Profile720P_5:"720P_5",Profile720P_6:"720P_6"},ErrorCode:{NO_ERROR:0,FAILED:1,INVALID_ARGUMENT:2,NOT_READY:3,NOT_SUPPORTED:4,REFUSED:5,BUFFER_TOO_SMALL:6,NOT_INITIALIZED:7,INVALID_VIEW:8,NO_PERMISSION:9,TIMEDOUT:10,CANCELED:11,TOO_OFTEN:12,BIND_SOCKET:13,NET_DOWN:14,NET_NOBUFS:15,INIT_VIDEO:16,JOIN_CHANNEL_REJECTED:17,LEAVE_CHANNEL_REJECTED:18,ALREADY_IN_USE:19,INVALID_APP_ID:101,INVALID_CHANNEL_NAME:102,CHANNEL_KEY_EXPIRED:109,INVALID_CHANNEL_KEY:110,CONNECTION_INTERRUPTED:111,CONNECTION_LOST:112,LOAD_MEDIA_ENGINE:1001,START_CALL:1002,START_CAMERA:1003,START_VIDEO_RENDER:1004,ADM_GENERAL_ERROR:1005,ADM_JAVA_RESOURCE:1006,ADM_SAMPLE_RATE:1007,ADM_INIT_PLAYOUT:1008,ADM_START_PLAYOUT:1009,ADM_STOP_PLAYOUT:1010,ADM_INIT_RECORDING:1011,ADM_START_RECORDING:1012,ADM_STOP_RECORDING:1013,ADM_RUNTIME_PLAYOUT_ERROR:1015,ADM_RUNTIME_RECORDING_ERROR:1017,ADM_RECORD_AUDIO_FAILED:1018,ADM_INIT_LOOPBACK:1022,ADM_START_LOOPBACK:1023,VDM_CAMERA_NOT_AUTHORIZED:1501}};AgoraRTC.EventDispatcher=function(spec){"use strict";var that={};spec.dispatcher={};spec.dispatcher.eventListeners={};that.addEventListener=function(eventType,listener){spec.dispatcher.eventListeners[eventType]=listener;if(!spec.dispatcher.eventListeners[eventType]){delete spec.dispatcher.eventListeners[eventType]}};that.on=that.addEventListener;that.dispatchEvent=function(event){if(spec.dispatcher.eventListeners.hasOwnProperty(event.type)){spec.dispatcher.eventListeners[event.type](event)}};return that};AgoraRTC.BasicEvent=function(spec){"use strict";var that={};that.type=spec.type;return that};AgoraRTC.StreamEvent=function(spec){"use strict";var that=AgoraRTC.BasicEvent(spec);that.stream=spec.stream;that.msg=spec.msg;return that};AgoraRTC.ClientEvent=function(spec){"use strict";var that=AgoraRTC.BasicEvent(spec);that.uid=spec.uid;that.attr=spec.attr;that.streams=spec.streams;return that};AgoraRTC.MediaEvent=function(spec){"use strict";var that=AgoraRTC.BasicEvent(spec);that.msg=spec.msg;return that};AgoraRTC.Signal=function(spec){var that=AgoraRTC.EventDispatcher(spec);that.connection=new WebSocket(spec.url);that.sendMessage=function(obj,onError){if(that.connection.readyState==WebSocket.OPEN){that.connection.send(JSON.stringify(obj))}else{console.log("connection to agent lost.");onError({error:"not connected"})}};that.close=function(){that.onEvent=onEvent;that.connection.onopen=undefined;that.connection.onclose=undefined;that.connection.onerror=undefined;that.connection.onmessage=undefined;that.connection.close()};that.connection.onopen=function(e){console.log(e);that.dispatchEvent(AgoraRTC.MediaEvent({type:"onopen",event:e}))};that.connection.onclose=function(e){console.log(e);AgoraCall(spec.onFailure,e)};that.connection.onerror=function(e){console.log(e);AgoraCall(spec.onFailure,e)};that.onEvent=onEvent;that.connection.onmessage=function(e){console.log(e);var m=JSON.parse(e.data);if(m.hasOwnProperty("command")){that.dispatchEvent(AgoraRTC.MediaEvent({type:m.command,msg:m}))}else if(m.hasOwnProperty("event")){that.onEvent(m)}};function onEvent(e){console.log(e)}return that};AgoraRTC.Stream=function(spec){var that=AgoraRTC.EventDispatcher(spec);that.stream=undefined;that.render=undefined;that.interval=undefined;that.lastRenderCount=0;that.profile=undefined;that.latency=0;var header=undefined;var yplane=undefined;var uplane=undefined;var vplane=undefined;that.init=function(onSuccess,onFailure){if(that.stream){console.warn("stream "+that.getId()+" has been initialized already");onSuccess();return}var videoStreamFileBlob=new Blob([AgoraRTC.videoStreamSource],{type:"application/javascript"});that.stream=new Worker(URL.createObjectURL(videoStreamFileBlob));that.oninit=function(e){console.log((spec.local?"local":"remote")+" Stream ");console.log(e);if(e.result===true){onSuccess()}else{onFailure()}};that.onclose=function(e){window.clearInterval(that.interval);console.log(spec.local?"local":"remote"," Stream ",e);that.stream=undefined};that.onerror=function(e){window.clearInterval(that.interval);console.log(spec.local?"local":"remote"," Stream ",e);onFailure(e)};that.onmessage=function(e){if(typeof e==="string"){console.log(spec.local?"local":"remote"," message from agent ",e)}else if(e instanceof ArrayBuffer){if(that.render){that.drawImage(e)}}else if(e instanceof Blob){console.warn("Blob image data is not supported")}};that.stream.addEventListener("message",onWorkerMessage,false);that.stream.postMessage({type:"init",local:spec.local,url:AgoraRTC.url,videoProfile:that.profile})};that.close=function(){window.clearInterval(that.interval);if(that.stream){that.stream.removeEventListener("message",onWorkerMessage);
that.stream.postMessage({type:"close"})}that.stream=undefined;that.stop()};that.play=function(div,onFailure){var view=document.getElementById(div);that.stop();that.render=AgoraRender();that.render.start(view,onFailure);if(spec.local){that.stream.postMessage({type:"send",message:{command:"preview",uid:spec.streamID,audio:spec.audio===false?false:true,video:spec.video===false?false:true,screen:spec.screen===true?true:false,viewWidth:view.clientWidth,viewHeight:view.clientHeight}})}else{that.stream.postMessage({type:"send",message:{command:"subscribe",uid:spec.streamID,viewWidth:view.clientWidth,viewHeight:view.clientHeight}})}if(that.interval){window.clearInterval(that.interval)}that.interval=window.setInterval(function(){if(that.render){var fps=that.render.renderImageCount-that.lastRenderCount;that.stream.postMessage({type:"send",message:{command:"videoStat",uid:spec.streamID,fps:fps,frameCount:that.lastRenderCount,latency:that.latency}});that.lastRenderCount=that.render.renderImageCount;that.latency=0}},1e3)};that.stop=function(){if(that.render){that.render.stop()}that.render=undefined;that.lastRenderCount=0};that.bindClient=function(client){that.client=client};that.enableAudio=function(callback){return that.client.enableAudio(that,callback)};that.disableAudio=function(callback){return that.client.disableAudio(that,callback)};that.enableVideo=function(callback){return that.client.enableVideo(that,callback)};that.disableVideo=function(callback){return that.client.disableVideo(that,callback)};that.getId=function(){return spec.streamID};that.drawImage=function(data){if(!header){header=data;if(header.byteLength!=20){console.error("invalid image header "+data.byteLength);resetImageData()}return}if(!yplane){yplane=data;if(yplane.byteLength===20){console.error("invalid image header "+data.byteLength+" "+yplane.byteLength);resetImageData()}return}if(!uplane){uplane=data;if(uplane.byteLength===20){console.error("invalid image header "+data.byteLength+" "+yplane.byteLength+" "+uplane.byteLength);resetImageData()}return}if(!vplane){vplane=data;if(yplane.byteLength!=uplane.byteLength*4||uplane.byteLength!=vplane.byteLength){console.error("invalid image header "+data.byteLength+" "+yplane.byteLength+" "+uplane.byteLength+" "+vplane.byteLength);resetImageData();return}var headerLength=20;var dv=new DataView(header);var format=dv.getUint8(0);var mirror=dv.getUint8(1);var width=dv.getUint16(2);var height=dv.getUint16(4);var left=dv.getUint16(6);var top=dv.getUint16(8);var right=dv.getUint16(10);var bottom=dv.getUint16(12);var rotation=dv.getUint16(14);var ts=dv.getUint32(16);var xWidth=width+left+right;var xHeight=height+top+bottom;var yLength=xWidth*xHeight;var yBegin=headerLength;var yEnd=yBegin+yLength;var uLength=yLength/4;var uBegin=yEnd;var uEnd=uBegin+uLength;var vLength=yLength/4;var vBegin=uEnd;var vEnd=vBegin+vLength;that.render.renderImage({mirror:mirror,width:width,height:height,left:left,top:top,right:right,bottom:bottom,rotation:rotation,yplane:new Uint8Array(yplane),uplane:new Uint8Array(uplane),vplane:new Uint8Array(vplane)});var now32=(Date.now()&4294967295)>>>0;var latency=now32-ts;if(latency>that.latency){that.latency=latency}resetImageData()}};that.setVideoProfile=function(profile){that.profile={};if(typeof profile!=="string"&&typeof profile!=="object"){return false}if(profile.hasOwnProperty("profile")){that.profile.profile=profile.profile}if(profile.hasOwnProperty("swapWidthAndHeight")){that.profile.swapWidthAndHeight=profile.swapWidthAndHeight!==false}else{that.profile.swapWidthAndHeight=false}if(typeof profile=="string"){that.profile.profile=profile}if(that.stream){that.stream.postMessage({type:"send",message:{command:"setVideoProfile",profile:that.profile.profile,swapWidthAndHeight:that.profile.swapWidthAndHeight}})}return true};function resetImageData(){header=undefined;yplane=undefined;uplane=undefined;vplane=undefined}function getReso(w,h){return{width:w,height:h}}var supportedVideoList={true:true,unspecified:true,"120p":getReso(160,120),"240p":getReso(320,240),"360p":getReso(640,360),"480p":getReso(640,480),"720p":getReso(1280,720),"1080p":getReso(1920,1080),"4k":getReso(3840,2160)};function onWorkerMessage(e){var data=e.data;switch(data.type){case"init":that.oninit(data);break;case"close":that.onclose(data);break;case"error":that.onerror(data);break;case"message":that.onmessage(data.data);break}}return that};AgoraRTC.createStream=function(spec){return AgoraRTC.Stream(spec)};AgoraRTC.Client=function(spec){"use strict";var that=AgoraRTC.EventDispatcher(spec);that.signal=undefined;that.localStream=undefined;that.remoteStreams={};that.signalOpen=false;function onSignalError(e){if(that.signalOpen){console.log("signal connection lost.");var message={reason:"LOST_CONNECTION_TO_AGENT",type:"error",eventType:e.type,code:e.code};AgoraCall(that.onFailure,message);that.dispatchEvent(message);return}if(e.type==="error"){return}var message={reason:"CLOSE_BEFORE_OPEN",type:"error",eventType:e.type,code:e.code,agentInstallUrl:getInstallUrl(),agentInstallGuideUrl:getInstallGuideUrl()};AgoraCall(that.onFailure,message);that.dispatchEvent(message)}that.init=function(appId,onSuccess,onFailure){spec.appId=appId;that.onFailure=onFailure;if(!that.signal){try{that.signal=AgoraRTC.Signal({url:AgoraRTC.url,onFailure:onSignalError})}catch(e){console.log("create signal connection failed"+e);AgoraCall(onFailure,e);return}that.signal.on("onopen",function(event){that.signalOpen=true;that.signal.onEvent=function(e){if(e.event==="error"){var name=getErrorCodeName(e.code);if(name){e.reason=name}}else if(e.event==="onUserMuteAudio"){e.event="peer-mute-audio"}else if(e.event==="onUserMuteVideo"){e.event="peer-mute-video"}that.dispatchEvent(AgoraRTC.MediaEvent({type:e.event,msg:e}))};that.getVersion(function(ev){if(ev.version=="1.6.0"||ev.version=="1.6.1"||ev.version=="1.7.0"||ev.version=="1.7.1"||ev.version=="1.7.2"||ev.version=="1.7.3"||ev.version=="1.7.4"){initialize(onSuccess,onFailure);return}var message={reason:"INCOMPATIBLE_WEBAGENT",type:"error",version:AgoraRTC.version,agentVersion:ev.version,agentInstallUrl:getInstallUrl(),agentInstallGuideUrl:getInstallGuideUrl()};that.dispatchEvent(message)},function(ev){var message={reason:"INCOMPATIBLE_WEBAGENT",type:"error",version:AgoraRTC.version,agentVersion:ev.version,agentInstallUrl:getInstallUrl(),agentInstallGuideUrl:getInstallGuideUrl()};that.dispatchEvent(message)})})}if(that.signalOpen){initialize(onSuccess,onFailure)}function initialize(onSuccess,onFailure){sendSignalCommand({command:"initialize",appId:spec.appId},function(ev){var m=ev.msg;if(m.code!=true){that.signal.close();that.signal=undefined;onError(m.code,"initialize");return}AgoraCall(onSuccess,m)},onFailure)}function onError(code,type){var name=getErrorCodeName(code);if(!name){name="UNKNOWN_ERROR"}notifyError({reason:name,type:"error",eventType:type,code:code});return}function notifyError(e){AgoraCall(onFailure,e);that.dispatchEvent(e)}function getErrorCodeName(code){for(var k in L.ErrorCode){if(L.ErrorCode.hasOwnProperty(k)){if(L.ErrorCode[k]===code){return String(k)}}}return null}that.signal.on("onError",function(ev){var m=ev.msg;onError(m.code,"error")});console.log(that.signal)};that.renewChannelKey=function(channelKey,onSuccess,onFailure){sendSignalCommand({command:"renewChannelKey",channelKey:channelKey},function(ev){var m=ev.msg;if(m.code===true){AgoraCall(onSuccess,m);return}AgoraCall(onFailure,m)},onFailure)};that.join=function(channelKey,channelName,uid,onSuccess,onFailure){if(channelName.length>64){onFailure(L.ErrorCode.INVALID_CHANNEL_NAME);return}sendSignalCommand({command:"joinChannel",channelKey:channelKey,channelName:channelName,uid:uid},function(ev){that.signal.on("onAddVideoStream",onAddVideoStream);that.signal.on("onPeerLeave",onPeerLeave);var m=ev.msg;if(m.code==true){if(!m.uid){m.uid=0}AgoraCall(onSuccess,m.uid);return}if(m.code==L.ErrorCode.JOIN_CHANNEL_REJECTED){console.error("Command joinChannel has been rejected by agent. Is this user joined a channel already?");AgoraCall(onFailure,m.code);return}AgoraCall(onFailure,m.code)},onFailure)};that.leave=function(onSuccess,onFailure){closeLocalRemoteStreams();sendSignalCommand({command:"leaveChannel"},function(ev){var m=ev.msg;if(m.code===true){AgoraCall(onSuccess,m);return}if(m.code==L.ErrorCode.LEAVE_CHANNEL_REJECTED){console.error("Command leaveChannel has been rejected by agent. Is this user not in a channel?");AgoraCall(onSuccess,m);return}AgoraCall(onFailure,{code:m.code})},function(e){console.log("leave channel failed",e)})};that.publish=function(stream,onSuccess,onFailure){that.localStream=stream;sendSignalCommand({command:"unmuteLocal"},function(ev){var m=ev.msg;if(m.code!=true){AgoraCall(onFailure,{code:m.code});return}AgoraCall(onSuccess,m)},onFailure);stream.bindClient(that)};that.unpublish=function(stream,onSuccess,onFailure){sendSignalCommand({command:"muteLocal"},function(ev){var m=ev.msg;if(m.code!=true){AgoraCall(onFailure,{code:m.code});return}AgoraCall(onSuccess,m)},function(e){AgoraCall(onFailure,{error:e})})};that.subscribe=function(stream,onFailure){stream.init(function(){that.dispatchEvent(AgoraRTC.StreamEvent({type:"stream-subscribed",stream:stream}))},function(error){AgoraCall(onFailure,error)})};that.unsubscribe=function(streamID,onFailure){console.log("remote streams",that.remoteStreams);if(that.remoteStreams[streamID]==undefined){AgoraCall(onFailure,{error:"no such stream"});return}that.remoteStreams[streamID].close();delete that.remoteStreams[streamID]};that.enableAudio=function(stream,callback){sendSignalCommand({command:"enableAudio",streamID:stream.getId()},function(ev){var m=ev.msg;if(callback){AgoraCall(callback,m.code===true)}},callback);return true};that.disableAudio=function(stream,callback){sendSignalCommand({command:"disableAudio",streamID:stream.getId()},function(ev){var m=ev.msg;if(callback){AgoraCall(callback,m.code===true)}},callback);return true};that.enableVideo=function(stream,callback){sendSignalCommand({command:"enableVideo",streamID:stream.getId()},function(ev){var m=ev.msg;if(callback){AgoraCall(callback,m.code===true)}},callback);return true};that.disableVideo=function(stream,callback){sendSignalCommand({command:"disableVideo",streamID:stream.getId()},function(ev){var m=ev.msg;if(callback){AgoraCall(callback,m.code===true)}},callback);return true};that.getDevices=function(callback){sendSignalCommand({command:"getDevices"},function(ev){var devices=ev.msg.devices;AgoraCall(callback,devices)},callback);that.signal.on("getDevices")};that.selectDevice=function(device,onFailure){sendSignalCommand({command:"selectDevice",device:device},function(ev){console.log(ev)},onFailure)};that.startRecording=function(recordingKey,onSuccess,onFailure){sendSignalCommand({command:"startRecording",recordingKey:recordingKey},function(ev){var start=ev.msg;if(start.code===true){AgoraCall(onSuccess,start)}else{AgoraCall(onFailure,start)}},onFailure)};that.stopRecording=function(recordingKey,onSuccess,onFailure){sendSignalCommand({command:"stopRecording",recordingKey:recordingKey},function(ev){var stop=ev.msg;if(stop.code===true){AgoraCall(onSuccess,stop)}else{AgoraCall(onFailure,stop)}},onFailure)};that.queryRecordingStatus=function(onStatus){sendSignalCommand({command:"queryRecordingStatus"},function(ev){onStatus(ev.msg)},onStatus)};that.close=function(){closeLocalRemoteStreams();if(that.signal){that.signal.close();that.signal=undefined}};that.getVersion=function(onSuccess,onFailure){sendSignalCommand({command:"getVersion"},function(ev){var m=ev.msg;if(m.code===true){AgoraCall(onSuccess,m)}else{AgoraCall(onFailure,m)}},onFailure)};that.setParameters=function(parameters){sendSignalCommand({command:"setParameters",parameters:JSON.stringify(parameters)})};that.setEncryptionMode=function(encryptionMode){sendSignalCommand({command:"setEncryptionMode",mode:encryptionMode})};that.setEncryptionSecret=function(secret){sendSignalCommand({command:"setEncryptionSecret",secret:secret})};that.getWindows=function(callback){sendSignalCommand({command:"getWindows"},function(ev){var windows=ev.msg.windows;AgoraCall(callback,windows)},callback)};that.startScreenSharing=function(window,onSuccess,onFailure){sendSignalCommand({command:"startScreenSharing",window:window},onSuccess,onFailure)};that.setScreenSharingWindow=function(window,onSuccess,onFailure){sendSignalCommand({command:"setScreenSharingWindow",window:window},onSuccess,onFailure)};that.stopScreenSharing=function(onSuccess,onFailure){sendSignalCommand({command:"stopScreenSharing"},onSuccess,onFailure)};that.setChannelProfile=function(profile,onSuccess,onFailure){sendSignalCommand({command:"setChannelProfile",profile:profile},onSuccess,onFailure)};that.setClientRole=function(role,permissionKey,onSuccess,onFailure){if(!permissionKey){permissionKey=""}sendSignalCommand({command:"setClientRole",role:role,key:permissionKey},onSuccess,onFailure)};that.enableDualStreamMode=function(enabled,onSuccess,onFailure){sendSignalCommand({command:"enableDualStreamMode",enabled:enabled},onSuccess,onFailure)};function sendSignalCommand(message,onSuccess,onFailure){that.signal.on(message.command,function(e){AgoraCall(onSuccess,e);console.log(e)});that.signal.sendMessage(message,function(e){console.log(e);e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(onFailure,e)})}function onPeerLeave(ev){var m=ev.msg;if(that.remoteStreams.hasOwnProperty(m.uid)){that.remoteStreams[m.uid].close();delete that.remoteStreams[m.uid];console.log("remote streams after peer leave ",that.remoteStreams)}that.dispatchEvent(AgoraRTC.ClientEvent({type:"peer-leave",uid:m.uid}))}function onAddVideoStream(ev){var m=ev.msg;if(!that.remoteStreams.hasOwnProperty(m.uid)){var stream=AgoraRTC.Stream({streamID:m.uid,local:false,audio:m.audio,video:m.video,screen:m.screen});stream.bindClient(that);that.remoteStreams[m.uid]=stream}that.dispatchEvent(AgoraRTC.StreamEvent({type:"stream-added",stream:that.remoteStreams[m.uid]}));console.log("remote streams",that.remoteStreams)}function getInstallUrl(){if(navigator.appVersion.indexOf("Mac")!=-1){return AgoraRTC.macAgentInstallUrl}return AgoraRTC.winAgentInstallUrl}function getInstallGuideUrl(){var userLang=navigator.language||navigator.userLanguage;if(userLang.indexOf("zh")!=-1){return AgoraRTC.cnAgentInstallGuideUrl}return AgoraRTC.enAgentInstallGuideUrl}function closeLocalRemoteStreams(){if(that.localStream){that.localStream.close()}that.localStream=undefined;for(var streamId in that.remoteStreams){if(that.remoteStreams.hasOwnProperty(streamId)){that.remoteStreams[streamId].close();delete that.remoteStreams[streamId]}}}return that};AgoraRTC.createRtcClient=function(){return AgoraRTC.Client({})};
//# sourceMappingURL=dist/AgoraRtcAgentSDK.min.js.map
"use strict";(function(root,factory){if(typeof define==="function"&&define.amd){define([],factory)}else{var lib=factory.call(root);Object.keys(lib).forEach(function(key){root[key]=lib[key]})}})(this,function(){var topWindow=this;function error(msg){if(topWindow.console){if(topWindow.console.error){topWindow.console.error(msg)}else if(topWindow.console.log){topWindow.console.log(msg)}}}function isInIFrame(w){w=w||topWindow;return w!==w.top}function makeFailHTML(msg){return""+'<table style="background-color: #8CE; width: 100%; height: 100%;"><tr>'+'<td align="center">'+'<div style="display: table-cell; vertical-align: middle;">'+'<div style="">'+msg+"</div>"+"</div>"+"</td></tr></table>"}var GET_A_WEBGL_BROWSER=""+"This page requires a browser that supports WebGL.<br/>"+'<a href="http://get.webgl.org">Click here to upgrade your browser.</a>';var OTHER_PROBLEM=""+"It doesn't appear your computer can support WebGL.<br/>"+'<a href="http://get.webgl.org/troubleshooting/">Click here for more information.</a>';function create3DContext(canvas,opt_attribs){var names=["webgl","experimental-webgl"];var context=null;for(var ii=0;ii<names.length;++ii){try{context=canvas.getContext(names[ii],opt_attribs)}catch(e){}if(context){break}}return context}function setupWebGL(canvas,opt_attribs){function showLink(str){var container=canvas.parentNode;if(container){container.innerHTML=makeFailHTML(str)}}if(!topWindow.WebGLRenderingContext){showLink(GET_A_WEBGL_BROWSER);return null}var context=create3DContext(canvas,opt_attribs);if(!context){showLink(OTHER_PROBLEM)}return context}function updateCSSIfInIFrame(){if(isInIFrame()){document.body.className="iframe"}}function getWebGLContext(canvas,opt_attribs,opt_options){var options=opt_options||{};if(isInIFrame()){updateCSSIfInIFrame();if(!options.dontResize&&options.resize!==false){var width=canvas.clientWidth;var height=canvas.clientHeight;canvas.width=width;canvas.height=height}}else if(!options.noTitle&&options.title!==false){var title=document.title;var h1=document.createElement("h1");h1.innerText=title;document.body.insertBefore(h1,document.body.children[0])}var gl=setupWebGL(canvas,opt_attribs);return gl}function loadShader(gl,shaderSource,shaderType,opt_errorCallback){var errFn=opt_errorCallback||error;var shader=gl.createShader(shaderType);gl.shaderSource(shader,shaderSource);gl.compileShader(shader);var compiled=gl.getShaderParameter(shader,gl.COMPILE_STATUS);if(!compiled){var lastError=gl.getShaderInfoLog(shader);errFn("*** Error compiling shader '"+shader+"':"+lastError);gl.deleteShader(shader);return null}return shader}function createProgram(gl,shaders,opt_attribs,opt_locations,opt_errorCallback){var errFn=opt_errorCallback||error;var program=gl.createProgram();shaders.forEach(function(shader){gl.attachShader(program,shader)});if(opt_attribs){obj_attrib.forEach(function(attrib,ndx){gl.bindAttribLocation(program,opt_locations?opt_locations[ndx]:ndx,attrib)})}gl.linkProgram(program);var linked=gl.getProgramParameter(program,gl.LINK_STATUS);if(!linked){var lastError=gl.getProgramInfoLog(program);errFn("Error in program linking:"+lastError);gl.deleteProgram(program);return null}return program}function createShaderFromScript(gl,scriptId,opt_shaderType,opt_errorCallback){var shaderSource="";var shaderType;var shaderScript=document.getElementById(scriptId);if(!shaderScript){throw"*** Error: unknown script element"+scriptId}shaderSource=shaderScript.text;if(!opt_shaderType){if(shaderScript.type==="x-shader/x-vertex"){shaderType=gl.VERTEX_SHADER}else if(shaderScript.type==="x-shader/x-fragment"){shaderType=gl.FRAGMENT_SHADER}else if(shaderType!==gl.VERTEX_SHADER&&shaderType!==gl.FRAGMENT_SHADER){throw"*** Error: unknown shader type"}}return loadShader(gl,shaderSource,opt_shaderType?opt_shaderType:shaderType,opt_errorCallback)}var defaultShaderType=["VERTEX_SHADER","FRAGMENT_SHADER"];function createProgramFromScripts(gl,shaderScriptIds,opt_attribs,opt_locations,opt_errorCallback){var shaders=[];for(var ii=0;ii<shaderScriptIds.length;++ii){shaders.push(createShaderFromScript(gl,shaderScriptIds[ii],gl[defaultShaderType[ii]],opt_errorCallback))}return createProgram(gl,shaders,opt_attribs,opt_locations,opt_errorCallback)}function createProgramFromSources(gl,shaderSources,opt_attribs,opt_locations,opt_errorCallback){var shaders=[];for(var ii=0;ii<shaderSources.length;++ii){shaders.push(loadShader(gl,shaderSources[ii],gl[defaultShaderType[ii]],opt_errorCallback))}return createProgram(gl,shaders,opt_attribs,opt_locations,opt_errorCallback)}function getBindPointForSamplerType(gl,type){if(type===gl.SAMPLER_2D)return gl.TEXTURE_2D;if(type===gl.SAMPLER_CUBE)return gl.TEXTURE_CUBE_MAP}function createUniformSetters(gl,program){var textureUnit=0;function createUniformSetter(program,uniformInfo){var location=gl.getUniformLocation(program,uniformInfo.name);var type=uniformInfo.type;var isArray=uniformInfo.size>1&&uniformInfo.name.substr(-3)==="[0]";if(type===gl.FLOAT&&isArray){return function(v){gl.uniform1fv(location,v)}}if(type===gl.FLOAT){return function(v){gl.uniform1f(location,v)}}if(type===gl.FLOAT_VEC2){return function(v){gl.uniform2fv(location,v)}}if(type===gl.FLOAT_VEC3){return function(v){gl.uniform3fv(location,v)}}if(type===gl.FLOAT_VEC4){return function(v){gl.uniform4fv(location,v)}}if(type===gl.INT&&isArray){return function(v){gl.uniform1iv(location,v)}}if(type===gl.INT){return function(v){gl.uniform1i(location,v)}}if(type===gl.INT_VEC2){return function(v){gl.uniform2iv(location,v)}}if(type===gl.INT_VEC3){return function(v){gl.uniform3iv(location,v)}}if(type===gl.INT_VEC4){return function(v){gl.uniform4iv(location,v)}}if(type===gl.BOOL){return function(v){gl.uniform1iv(location,v)}}if(type===gl.BOOL_VEC2){return function(v){gl.uniform2iv(location,v)}}if(type===gl.BOOL_VEC3){return function(v){gl.uniform3iv(location,v)}}if(type===gl.BOOL_VEC4){return function(v){gl.uniform4iv(location,v)}}if(type===gl.FLOAT_MAT2){return function(v){gl.uniformMatrix2fv(location,false,v)}}if(type===gl.FLOAT_MAT3){return function(v){gl.uniformMatrix3fv(location,false,v)}}if(type===gl.FLOAT_MAT4){return function(v){gl.uniformMatrix4fv(location,false,v)}}if((type===gl.SAMPLER_2D||type===gl.SAMPLER_CUBE)&&isArray){var units=[];for(var ii=0;ii<info.size;++ii){units.push(textureUnit++)}return function(bindPoint,units){return function(textures){gl.uniform1iv(location,units);textures.forEach(function(texture,index){gl.activeTexture(gl.TEXTURE0+units[index]);gl.bindTexture(bindPoint,texture)})}}(getBindPointForSamplerType(gl,type),units)}if(type===gl.SAMPLER_2D||type===gl.SAMPLER_CUBE){return function(bindPoint,unit){return function(texture){gl.uniform1i(location,unit);gl.activeTexture(gl.TEXTURE0+unit);gl.bindTexture(bindPoint,texture)}}(getBindPointForSamplerType(gl,type),textureUnit++)}throw"unknown type: 0x"+type.toString(16)}var uniformSetters={};var numUniforms=gl.getProgramParameter(program,gl.ACTIVE_UNIFORMS);for(var ii=0;ii<numUniforms;++ii){var uniformInfo=gl.getActiveUniform(program,ii);if(!uniformInfo){break}var name=uniformInfo.name;if(name.substr(-3)==="[0]"){name=name.substr(0,name.length-3)}var setter=createUniformSetter(program,uniformInfo);uniformSetters[name]=setter}return uniformSetters}function setUniforms(setters,values){Object.keys(values).forEach(function(name){var setter=setters[name];if(setter){setter(values[name])}})}function createAttributeSetters(gl,program){var attribSetters={};function createAttribSetter(index){return function(b){gl.bindBuffer(gl.ARRAY_BUFFER,b.buffer);gl.enableVertexAttribArray(index);gl.vertexAttribPointer(index,b.numComponents||b.size,b.type||gl.FLOAT,b.normalize||false,b.stride||0,b.offset||0)}}var numAttribs=gl.getProgramParameter(program,gl.ACTIVE_ATTRIBUTES);for(var ii=0;ii<numAttribs;++ii){var attribInfo=gl.getActiveAttrib(program,ii);if(!attribInfo){break}var index=gl.getAttribLocation(program,attribInfo.name);attribSetters[attribInfo.name]=createAttribSetter(index)}return attribSetters}function setAttributes(setters,buffers){Object.keys(buffers).forEach(function(name){var setter=setters[name];if(setter){setter(buffers[name])}})}function createProgramInfo(gl,shaderSources,opt_attribs,opt_locations,opt_errorCallback){shaderSources=shaderSources.map(function(source){var script=document.getElementById(source);return script?script.text:source});var program=createProgramFromSources(gl,shaderSources,opt_attribs,opt_locations,opt_errorCallback);if(!program){return null}var uniformSetters=createUniformSetters(gl,program);var attribSetters=createAttributeSetters(gl,program);return{program:program,uniformSetters:uniformSetters,attribSetters:attribSetters}}function setBuffersAndAttributes(gl,setters,buffers){setAttributes(setters,buffers.attribs);if(buffers.indices){gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER,buffers.indices)}}var browserPrefixes=["","MOZ_","OP_","WEBKIT_"];function getExtensionWithKnownPrefixes(gl,name){for(var ii=0;ii<browserPrefixes.length;++ii){var prefixedName=browserPrefixes[ii]+name;var ext=gl.getExtension(prefixedName);if(ext){return ext}}}function resizeCanvasToDisplaySize(canvas,multiplier){multiplier=multiplier||1;var width=canvas.clientWidth*multiplier;var height=canvas.clientHeight*multiplier;if(canvas.width!==width||canvas.height!==height){canvas.width=width;canvas.height=height;return true}return false}function getIFrameForWindow(window){if(!isInIFrame(window)){return}var iframes=window.parent.document.getElementsByTagName("iframe");for(var ii=0;ii<iframes.length;++ii){var iframe=iframes[ii];if(iframe.contentDocument===window.document){return iframe}}}function isFrameVisible(window){try{var iframe=getIFrameForWindow(window);if(!iframe){return true}var bounds=iframe.getBoundingClientRect();var isVisible=bounds.top<window.parent.innerHeight&&bounds.bottom>=0&&bounds.left<window.parent.innerWidth&&bounds.right>=0;return isVisible&&isFrameVisible(window.parent)}catch(e){return true}}function isOnScreen(element){var isVisible=true;if(element){var bounds=element.getBoundingClientRect();isVisible=bounds.top<topWindow.innerHeight&&bounds.bottom>=0}return isVisible&&isFrameVisible(topWindow)}function augmentTypedArray(typedArray,numComponents){var cursor=0;typedArray.push=function(){for(var ii=0;ii<arguments.length;++ii){var value=arguments[ii];if(value instanceof Array||value.buffer&&value.buffer instanceof ArrayBuffer){for(var jj=0;jj<value.length;++jj){typedArray[cursor++]=value[jj]}}else{typedArray[cursor++]=value}}};typedArray.reset=function(opt_index){cursor=opt_index||0};typedArray.numComponents=numComponents;Object.defineProperty(typedArray,"numElements",{get:function(){return this.length/this.numComponents|0}});return typedArray}function createAugmentedTypedArray(numComponents,numElements,opt_type){var Type=opt_type||Float32Array;return augmentTypedArray(new Type(numComponents*numElements),numComponents)}function createBufferFromTypedArray(gl,array,type,drawType){type=type||gl.ARRAY_BUFFER;var buffer=gl.createBuffer();gl.bindBuffer(type,buffer);gl.bufferData(type,array,drawType||gl.STATIC_DRAW);return buffer}function allButIndices(name){return name!=="indices"}function createMapping(obj){var mapping={};Object.keys(obj).filter(allButIndices).forEach(function(key){mapping["a_"+key]=key});return mapping}function getGLTypeForTypedArray(gl,typedArray){if(typedArray instanceof Int8Array){return gl.BYTE}if(typedArray instanceof Uint8Array){return gl.UNSIGNED_BYTE}if(typedArray instanceof Int16Array){return gl.SHORT}if(typedArray instanceof Uint16Array){return gl.UNSIGNED_SHORT}if(typedArray instanceof Int32Array){return gl.INT}if(typedArray instanceof Uint32Array){return gl.UNSIGNED_INT}if(typedArray instanceof Float32Array){return gl.FLOAT}throw"unsupported typed array type"}function getNormalizationForTypedArray(typedArray){if(typedArray instanceof Int8Array){return true}if(typedArray instanceof Uint8Array){return true}return false}function isArrayBuffer(a){return a.buffer&&a.buffer instanceof ArrayBuffer}function guessNumComponentsFromName(name,length){var numComponents;if(name.indexOf("coord")>=0){numComponents=2}else if(name.indexOf("color")>=0){numComponents=4}else{numComponents=3}if(length%numComponents>0){throw"can not guess numComponents. You should specify it."}return numComponents}function makeTypedArray(array,name){if(isArrayBuffer(array)){return array}if(Array.isArray(array)){array={data:array}}if(!array.numComponents){array.numComponents=guessNumComponentsFromName(name,array.length)}var type=array.type;if(!type){if(name==="indices"){type=Uint16Array}}var typedArray=createAugmentedTypedArray(array.numComponents,array.data.length/array.numComponents|0,type);typedArray.push(array.data);return typedArray}function createAttribsFromArrays(gl,arrays,opt_mapping){var mapping=opt_mapping||createMapping(arrays);var attribs={};Object.keys(mapping).forEach(function(attribName){var bufferName=mapping[attribName];var array=makeTypedArray(arrays[bufferName],bufferName);attribs[attribName]={buffer:createBufferFromTypedArray(gl,array),numComponents:array.numComponents||guessNumComponentsFromName(bufferName),type:getGLTypeForTypedArray(gl,array),normalize:getNormalizationForTypedArray(array)}});return attribs}function getNumElementsFromNonIndexedArrays(arrays){var key=Object.keys(arrays)[0];var array=arrays[key];if(isArrayBuffer(array)){return array.numElements}else{return array.data.length/array.numComponents}}function createBufferInfoFromArrays(gl,arrays,opt_mapping){var bufferInfo={attribs:createAttribsFromArrays(gl,arrays,opt_mapping)};var indices=arrays.indices;if(indices){indices=makeTypedArray(indices,"indices");bufferInfo.indices=createBufferFromTypedArray(gl,indices,gl.ELEMENT_ARRAY_BUFFER);bufferInfo.numElements=indices.length}else{bufferInfo.numElements=getNumElementsFromNonIndexedArrays(arrays)}return bufferInfo}function createBuffersFromArrays(gl,arrays){var buffers={};Object.keys(arrays).forEach(function(key){var type=key==="indices"?gl.ELEMENT_ARRAY_BUFFER:gl.ARRAY_BUFFER;var array=makeTypedArray(arrays[key],name);buffers[key]=createBufferFromTypedArray(gl,array,type)});if(arrays.indices){buffers.numElements=arrays.indices.length}else if(arrays.position){buffers.numElements=arrays.position.length/3}return buffers}function drawBufferInfo(gl,type,bufferInfo,count,offset){var indices=bufferInfo.indices;var numElements=count===undefined?bufferInfo.numElements:count;offset=offset===undefined?offset:0;if(indices){gl.drawElements(type,numElements,gl.UNSIGNED_SHORT,offset)}else{gl.drawArrays(type,offset,numElements)}}function drawObjectList(gl,objectsToDraw){var lastUsedProgramInfo=null;var lastUsedBufferInfo=null;objectsToDraw.forEach(function(object){var programInfo=object.programInfo;var bufferInfo=object.bufferInfo;var bindBuffers=false;if(programInfo!==lastUsedProgramInfo){lastUsedProgramInfo=programInfo;gl.useProgram(programInfo.program);bindBuffers=true}if(bindBuffers||bufferInfo!==lastUsedBufferInfo){lastUsedBufferInfo=bufferInfo;setBuffersAndAttributes(gl,programInfo.attribSetters,bufferInfo)}setUniforms(programInfo.uniformSetters,object.uniforms);drawBufferInfo(gl,gl.TRIANGLES,bufferInfo)})}return{createAugmentedTypedArray:createAugmentedTypedArray,createAttribsFromArrays:createAttribsFromArrays,createBuffersFromArrays:createBuffersFromArrays,createBufferInfoFromArrays:createBufferInfoFromArrays,createAttributeSetters:createAttributeSetters,createProgram:createProgram,createProgramFromScripts:createProgramFromScripts,createProgramFromSources:createProgramFromSources,createProgramInfo:createProgramInfo,createUniformSetters:createUniformSetters,drawBufferInfo:drawBufferInfo,drawObjectList:drawObjectList,getWebGLContext:getWebGLContext,updateCSSIfInIFrame:updateCSSIfInIFrame,getExtensionWithKnownPrefixes:getExtensionWithKnownPrefixes,resizeCanvasToDisplaySize:resizeCanvasToDisplaySize,setAttributes:setAttributes,setBuffersAndAttributes:setBuffersAndAttributes,setUniforms:setUniforms,setupWebGL:setupWebGL}});function AgoraRender(){var gl=undefined;var program=undefined;var positionLocation=undefined;var texCoordLocation=undefined;var yTexture=undefined;var uTexture=undefined;var vTexture=undefined;var texCoordBuffer=undefined;var surfaceBuffer=undefined;var that={view:undefined,mirrorView:false,container:undefined,canvas:undefined,renderImageCount:0,initWidth:0,initHeight:0,initRotation:0,canvasUpdated:false,clientWidth:0,clientHeight:0};that.start=function(view,onFailure){initCanvas(view,that.mirrorView,view.clientWidth,view.clientHeight,that.initRotation,onFailure)};that.stop=function(){gl=undefined;program=undefined;positionLocation=undefined;texCoordLocation=undefined;deleteTexture(yTexture);deleteTexture(uTexture);deleteTexture(vTexture);yTexture=undefined;uTexture=undefined;vTexture=undefined;deleteBuffer(texCoordBuffer);deleteBuffer(surfaceBuffer);texCoordBuffer=undefined;surfaceBuffer=undefined;if(that.container){that.container.removeChild(that.canvas)}if(that.view){that.view.removeChild(that.container)}that.canvas=undefined;that.container=undefined;that.view=undefined;that.mirrorView=false};that.renderImage=function(image){if(!gl){return}if(image.width!=that.initWidth||image.height!=that.initHeight||image.rotation!=that.initRotation||image.mirror!=that.mirrorView){var view=that.view;that.stop();console.log("init canvas "+image.width+"*"+image.height+" rotation "+image.rotation);initCanvas(view,image.mirror,image.width,image.height,image.rotation,function(e){console.error("init canvas "+image.width+"*"+image.height+" rotation "+image.rotation+" failed. "+e)})}gl.bindBuffer(gl.ARRAY_BUFFER,texCoordBuffer);var xWidth=image.width+image.left+image.right;var xHeight=image.height+image.top+image.bottom;gl.bufferData(gl.ARRAY_BUFFER,new Float32Array([image.left/xWidth,image.bottom/xHeight,1-image.right/xWidth,image.bottom/xHeight,image.left/xWidth,1-image.top/xHeight,image.left/xWidth,1-image.top/xHeight,1-image.right/xWidth,image.bottom/xHeight,1-image.right/xWidth,1-image.top/xHeight]),gl.STATIC_DRAW);gl.enableVertexAttribArray(texCoordLocation);gl.vertexAttribPointer(texCoordLocation,2,gl.FLOAT,false,0,0);uploadYuv(xWidth,xHeight,image.yplane,image.uplane,image.vplane);updateCanvas(image.rotation,image.width,image.height);gl.drawArrays(gl.TRIANGLES,0,6);that.renderImageCount+=1};function uploadYuv(width,height,yplane,uplane,vplane){gl.activeTexture(gl.TEXTURE0);gl.bindTexture(gl.TEXTURE_2D,yTexture);gl.texImage2D(gl.TEXTURE_2D,0,gl.LUMINANCE,width,height,0,gl.LUMINANCE,gl.UNSIGNED_BYTE,yplane);var e=gl.getError();if(e!=gl.NO_ERROR){console.log("upload y plane ",width,height,yplane.byteLength," error",e)}gl.activeTexture(gl.TEXTURE1);gl.bindTexture(gl.TEXTURE_2D,uTexture);gl.texImage2D(gl.TEXTURE_2D,0,gl.LUMINANCE,width/2,height/2,0,gl.LUMINANCE,gl.UNSIGNED_BYTE,uplane);var e=gl.getError();if(e!=gl.NO_ERROR){console.log("upload u plane ",width,height,uplane.byteLength,"  error",e)}gl.activeTexture(gl.TEXTURE2);gl.bindTexture(gl.TEXTURE_2D,vTexture);gl.texImage2D(gl.TEXTURE_2D,0,gl.LUMINANCE,width/2,height/2,0,gl.LUMINANCE,gl.UNSIGNED_BYTE,vplane);var e=gl.getError();if(e!=gl.NO_ERROR){console.log("upload v plane ",width,height,vplane.byteLength,"  error",e)}}function deleteBuffer(buffer){if(buffer&&that.gl){that.gl.deleteBuffer(buffer)}}function deleteTexture(texture){if(texture&&that.gl){that.gl.deleteTexture(texture)}}function initCanvas(view,mirror,width,height,rotation,onFailure){that.clientWidth=view.clientWidth;that.clientHeight=view.clientHeight;that.view=view;that.mirrorView=mirror;that.canvasUpdated=false;that.container=document.createElement("div");that.container.style.width="100%";that.container.style.height="100%";that.container.style.display="flex";that.container.style.justifyContent="center";that.container.style.alignItems="center";that.view.appendChild(that.container);that.canvas=document.createElement("canvas");if(rotation==0||rotation==180){that.canvas.width=width;that.canvas.height=height}else{that.canvas.width=height;that.canvas.height=width}that.initWidth=width;that.initHeight=height;that.initRotation=rotation;if(that.mirrorView){that.canvas.style.transform="rotateY(180deg)"}that.container.appendChild(that.canvas);try{gl=that.canvas.getContext("webgl")||that.canvas.getContext("experimental-webgl")}catch(e){console.log(e)}if(!gl){gl=undefined;onFailure({error:"Browser not support! No WebGL detected."});return}gl.clearColor(0,0,0,1);gl.enable(gl.DEPTH_TEST);gl.depthFunc(gl.LEQUAL);gl.clear(gl.COLOR_BUFFER_BIT|gl.DEPTH_BUFFER_BIT);program=createProgramFromSources(gl,[AgoraRTC.vertexShaderSource,AgoraRTC.yuvShaderSource]);gl.useProgram(program);initTextures()}function initTextures(){positionLocation=gl.getAttribLocation(program,"a_position");texCoordLocation=gl.getAttribLocation(program,"a_texCoord");surfaceBuffer=gl.createBuffer();texCoordBuffer=gl.createBuffer();gl.activeTexture(gl.TEXTURE0);yTexture=gl.createTexture();gl.bindTexture(gl.TEXTURE_2D,yTexture);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_WRAP_S,gl.CLAMP_TO_EDGE);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_WRAP_T,gl.CLAMP_TO_EDGE);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_MIN_FILTER,gl.NEAREST);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_MAG_FILTER,gl.NEAREST);gl.activeTexture(gl.TEXTURE1);uTexture=gl.createTexture();gl.bindTexture(gl.TEXTURE_2D,uTexture);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_WRAP_S,gl.CLAMP_TO_EDGE);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_WRAP_T,gl.CLAMP_TO_EDGE);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_MIN_FILTER,gl.NEAREST);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_MAG_FILTER,gl.NEAREST);gl.activeTexture(gl.TEXTURE2);vTexture=gl.createTexture();gl.bindTexture(gl.TEXTURE_2D,vTexture);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_WRAP_S,gl.CLAMP_TO_EDGE);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_WRAP_T,gl.CLAMP_TO_EDGE);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_MIN_FILTER,gl.NEAREST);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_MAG_FILTER,gl.NEAREST);var y=gl.getUniformLocation(program,"Ytex");gl.uniform1i(y,0);var u=gl.getUniformLocation(program,"Utex");gl.uniform1i(u,1);var v=gl.getUniformLocation(program,"Vtex");gl.uniform1i(v,2)}function updateCanvas(rotation,width,height){if(that.canvasUpdated){return}that.canvas.style.width="100%";that.canvas.style.height="100%";try{if(rotation===0||rotation===180){if(that.clientWidth/that.clientHeight>width/height){that.canvas.style.width=that.clientHeight*width/height+"px"}else if(that.clientWidth/that.clientHeight<width/height){that.canvas.style.height=that.clientWidth*height/width+"px"}}else{if(that.clientHeight/that.clientWidth>width/height){that.canvas.style.height=that.clientWidth*width/height+"px"}else if(that.clientHeight/that.clientWidth<width/height){that.canvas.style.width=that.clientHeight*height/width+"px"}}}catch(e){console.log("updateCanvas 00001 gone "+that.canvas);console.log(that);console.error(e);return}gl.bindBuffer(gl.ARRAY_BUFFER,surfaceBuffer);gl.enableVertexAttribArray(positionLocation);gl.vertexAttribPointer(positionLocation,2,gl.FLOAT,false,0,0);var p1={x:0,y:0};var p2={x:width,y:0};var p3={x:width,y:height};var p4={x:0,y:height};var pp1=p1,pp2=p2,pp3=p3,pp4=p4;switch(rotation){case 0:break;case 90:pp1=p2;pp2=p3;pp3=p4;pp4=p1;break;case 180:pp1=p3;pp2=p4;pp3=p1;pp4=p2;break;case 270:pp1=p4;pp2=p1;pp3=p2;pp4=p3;break;default:}gl.bufferData(gl.ARRAY_BUFFER,new Float32Array([pp1.x,pp1.y,pp2.x,pp2.y,pp4.x,pp4.y,pp4.x,pp4.y,pp2.x,pp2.y,pp3.x,pp3.y]),gl.STATIC_DRAW);var resolutionLocation=gl.getUniformLocation(program,"u_resolution");gl.uniform2f(resolutionLocation,width,height);that.canvasUpdated=true}return that}var AgoraRTC=function(){"use strict";var AgoraRTC={};Object.defineProperties(AgoraRTC,{version:{get:function(){return"1.8.0"}},name:{get:function(){return"AgoraWebSDK"}}});return AgoraRTC}();AgoraRTC.secureAgentUrl="wss://localhost.agora.io:8921/";AgoraRTC.agentUrl="ws://127.0.0.1:8922/";AgoraRTC.getAgentUrl=function(){if(location.protocol==="https:"){return AgoraRTC.secureAgentUrl}return AgoraRTC.agentUrl};AgoraRTC.macAgentInstallUrl="http://download.agora.io/AgoraWebAgent-1.8.0.pkg";AgoraRTC.winAgentInstallUrl="http://download.agora.io/AgoraWebAgentSetup-1.8.0.exe";AgoraRTC.enAgentInstallGuideUrl="http://download.agora.io/install-guide-en.html";AgoraRTC.cnAgentInstallGuideUrl="http://download.agora.io/install-guide-cn.html";AgoraRTC.vertexShaderSource="attribute vec2 a_position;"+"attribute vec2 a_texCoord;"+"uniform vec2 u_resolution;"+"varying vec2 v_texCoord;"+"void main() {"+"vec2 zeroToOne = a_position / u_resolution;"+"   vec2 zeroToTwo = zeroToOne * 2.0;"+"   vec2 clipSpace = zeroToTwo - 1.0;"+"   gl_Position = vec4(clipSpace * vec2(1, -1), 0, 1);"+"v_texCoord = a_texCoord;"+"}";AgoraRTC.yuvShaderSource="precision mediump float;"+"uniform sampler2D Ytex;"+"uniform sampler2D Utex,Vtex;"+"varying vec2 v_texCoord;"+"void main(void) {"+"  float nx,ny,r,g,b,y,u,v;"+"  mediump vec4 txl,ux,vx;"+"  nx=v_texCoord[0];"+"  ny=v_texCoord[1];"+"  y=texture2D(Ytex,vec2(nx,ny)).r;"+"  u=texture2D(Utex,vec2(nx,ny)).r;"+"  v=texture2D(Vtex,vec2(nx,ny)).r;"+"  y=1.1643*(y-0.0625);"+"  u=u-0.5;"+"  v=v-0.5;"+"  r=y+1.5958*v;"+"  g=y-0.39173*u-0.81290*v;"+"  b=y+2.017*u;"+"  gl_FragColor=vec4(r,g,b,1.0);"+"}";AgoraRTC.videoStreamSource='function onSuccess(){postMessage({type:"init",result:!0})}function onFailure(a){postMessage(a)}function sendMessage(a){"videoStat"!=a.command&&console.log(JSON.stringify(a)),that.stream&&that.stream.readyState==WebSocket.OPEN&&that.stream.send(JSON.stringify(a))}var that={};that.stream=void 0,that.init=function(a){return that.stream&&that.stream.readyState===WebSocket.OPEN?(console.warn("stream "+that.getId()+" has been initialized already"),void onSuccess()):(that.local=a.local,that.profile=a.videoProfile,that.stream=new WebSocket(a.url),that.stream.onopen=function(a){console.log((that.local?"local":"remote")+" Stream "),console.log(a),that.profile&&sendMessage({command:"setVideoProfile",profile:that.profile.profile,swapWidthAndHeight:that.profile.swapWidthAndHeight}),that.stream.binaryType="arraybuffer",onSuccess()},that.stream.onclose=function(a){console.log(that.local?"local":"remote"," Stream ",a),that.stream=void 0,onFailure({type:a.type,code:a.code,reason:a.reason})},that.stream.onerror=function(a){console.log(that.local?"local":"remote"," Stream ",a),onFailure({type:a.type,code:a.code,reason:a.reason})},void(that.stream.onmessage=function(a){"string"==typeof a.data?console.log(that.local?"local":"remote"," message from agent ",a.data):a.data instanceof ArrayBuffer?postMessage({type:"message",data:a.data}):a.data instanceof Blob&&console.warn("Blob image data is not supported")}))},that.close=function(){that.stream&&(that.stream.onmessage=void 0,that.stream.close()),that.stream=void 0},self.addEventListener("message",function(a){var b=a.data;switch(b.type){case"init":that.init(b);break;case"send":sendMessage(b.message);break;case"close":that.close()}},!1);';var AgoraCall=function(callback,parameters){if(callback){callback(parameters)}};var L={VideoProfiles:{Profile120P:"120P",Profile120P_3:"120P_3",Profile96P:"96P",Profile48P:"48P",Profile180P:"180P",Profile180P_3:"180P_3",Profile180P_4:"180P_4",Profile240P:"240P",Profile240P_3:"240P_3",Profile240P_4:"240P_4",Profile360P:"360P",Profile360P_3:"360P_3",Profile360P_4:"360P_4",Profile360P_6:"360P_6",Profile360P_7:"360P_7",Profile360P_8:"360P_8",Profile480P:"480P",Profile480P_3:"480P_3",Profile480P_4:"480P_4",Profile480P_6:"480P_6",Profile480P_8:"480P_8",Profile480P_9:"480P_9",Profile720P:"720P",Profile720P_3:"720P_3",Profile720P_5:"720P_5",Profile720P_6:"720P_6"},ErrorCode:{NO_ERROR:0,FAILED:1,INVALID_ARGUMENT:2,NOT_READY:3,NOT_SUPPORTED:4,REFUSED:5,BUFFER_TOO_SMALL:6,NOT_INITIALIZED:7,INVALID_VIEW:8,NO_PERMISSION:9,TIMEDOUT:10,CANCELED:11,TOO_OFTEN:12,BIND_SOCKET:13,NET_DOWN:14,NET_NOBUFS:15,INIT_VIDEO:16,JOIN_CHANNEL_REJECTED:17,LEAVE_CHANNEL_REJECTED:18,ALREADY_IN_USE:19,INVALID_APP_ID:101,INVALID_CHANNEL_NAME:102,CHANNEL_KEY_EXPIRED:109,INVALID_CHANNEL_KEY:110,CONNECTION_INTERRUPTED:111,CONNECTION_LOST:112,LOAD_MEDIA_ENGINE:1001,START_CALL:1002,START_CAMERA:1003,START_VIDEO_RENDER:1004,ADM_GENERAL_ERROR:1005,ADM_JAVA_RESOURCE:1006,ADM_SAMPLE_RATE:1007,ADM_INIT_PLAYOUT:1008,ADM_START_PLAYOUT:1009,ADM_STOP_PLAYOUT:1010,ADM_INIT_RECORDING:1011,ADM_START_RECORDING:1012,ADM_STOP_RECORDING:1013,ADM_RUNTIME_PLAYOUT_ERROR:1015,ADM_RUNTIME_RECORDING_ERROR:1017,ADM_RECORD_AUDIO_FAILED:1018,ADM_INIT_LOOPBACK:1022,ADM_START_LOOPBACK:1023,VDM_CAMERA_NOT_AUTHORIZED:1501}};AgoraRTC.EventDispatcher=function(spec){"use strict";var that={};spec.dispatcher={};spec.dispatcher.eventListeners={};that.addEventListener=function(eventType,listener){spec.dispatcher.eventListeners[eventType]=listener;if(!spec.dispatcher.eventListeners[eventType]){delete spec.dispatcher.eventListeners[eventType]}};that.on=that.addEventListener;that.dispatchEvent=function(event){if(spec.dispatcher.eventListeners.hasOwnProperty(event.type)){spec.dispatcher.eventListeners[event.type](event)}};return that};AgoraRTC.BasicEvent=function(spec){"use strict";var that={};that.type=spec.type;return that};AgoraRTC.StreamEvent=function(spec){"use strict";var that=AgoraRTC.BasicEvent(spec);that.stream=spec.stream;that.msg=spec.msg;return that};AgoraRTC.ClientEvent=function(spec){"use strict";var that=AgoraRTC.BasicEvent(spec);that.uid=spec.uid;that.attr=spec.attr;that.streams=spec.streams;return that};AgoraRTC.MediaEvent=function(spec){"use strict";var that=AgoraRTC.BasicEvent(spec);that.msg=spec.msg;return that};AgoraRTC.Signal=function(spec){var that=AgoraRTC.EventDispatcher(spec);that.connection=new WebSocket(spec.url);that.sendMessage=function(obj,onError){if(that.connection.readyState==WebSocket.OPEN){that.connection.send(JSON.stringify(obj))}else{console.log("connection to agent lost.");onError({error:"not connected"})}};that.close=function(){that.onEvent=onEvent;that.connection.onopen=undefined;that.connection.onclose=undefined;that.connection.onerror=undefined;that.connection.onmessage=undefined;that.connection.close()};that.connection.onopen=function(e){console.log(e);that.dispatchEvent(AgoraRTC.MediaEvent({type:"onopen",event:e}))};that.connection.onclose=function(e){console.log(e);AgoraCall(spec.onFailure,e)};that.connection.onerror=function(e){console.log(e);AgoraCall(spec.onFailure,e)};that.onEvent=onEvent;that.connection.onmessage=function(e){console.log(e);var m=JSON.parse(e.data);if(m.hasOwnProperty("command")){that.dispatchEvent(AgoraRTC.MediaEvent({type:m.command,msg:m}))}else if(m.hasOwnProperty("event")){that.onEvent(m)}};function onEvent(e){console.log(e)}return that};AgoraRTC.Stream=function(spec){var that=AgoraRTC.EventDispatcher(spec);that.stream=undefined;that.render=undefined;that.interval=undefined;that.lastRenderCount=0;that.profile=undefined;that.latency=0;var header=undefined;var yplane=undefined;var uplane=undefined;var vplane=undefined;that.init=function(onSuccess,onFailure){if(that.stream){console.warn("stream "+that.getId()+" has been initialized already");onSuccess();return}var videoStreamFileBlob=new Blob([AgoraRTC.videoStreamSource],{type:"application/javascript"});that.stream=new Worker(URL.createObjectURL(videoStreamFileBlob));that.oninit=function(e){console.log((spec.local?"local":"remote")+" Stream ");console.log(e);if(e.result===true){onSuccess()}else{onFailure()}};that.onclose=function(e){window.clearInterval(that.interval);console.log(spec.local?"local":"remote"," Stream ",e);that.stream=undefined};that.onerror=function(e){window.clearInterval(that.interval);console.log(spec.local?"local":"remote"," Stream ",e);onFailure(e)};that.onmessage=function(e){if(typeof e==="string"){console.log(spec.local?"local":"remote"," message from agent ",e)}else if(e instanceof ArrayBuffer){if(that.render){that.drawImage(e)}}else if(e instanceof Blob){console.warn("Blob image data is not supported")}};that.stream.addEventListener("message",onWorkerMessage,false);that.stream.postMessage({type:"init",local:spec.local,
url:AgoraRTC.getAgentUrl(),videoProfile:that.profile})};that.close=function(){window.clearInterval(that.interval);if(that.stream){that.stream.removeEventListener("message",onWorkerMessage);that.stream.postMessage({type:"close"})}that.stream=undefined;that.stop()};that.play=function(div,onFailure){var view=document.getElementById(div);that.stop();that.render=AgoraRender();that.render.start(view,onFailure);if(spec.local){that.stream.postMessage({type:"send",message:{command:"preview",uid:spec.streamID,audio:spec.audio===false?false:true,video:spec.video===false?false:true,screen:spec.screen===true?true:false,viewWidth:view.clientWidth,viewHeight:view.clientHeight}})}else{that.stream.postMessage({type:"send",message:{command:"subscribe",uid:spec.streamID,viewWidth:view.clientWidth,viewHeight:view.clientHeight}})}if(that.interval){window.clearInterval(that.interval)}that.interval=window.setInterval(function(){if(that.render){var fps=that.render.renderImageCount-that.lastRenderCount;that.stream.postMessage({type:"send",message:{command:"videoStat",uid:spec.streamID,fps:fps,frameCount:that.lastRenderCount,latency:that.latency}});that.lastRenderCount=that.render.renderImageCount;that.latency=0}},1e3)};that.stop=function(){if(that.render){that.render.stop()}that.render=undefined;that.lastRenderCount=0};that.bindClient=function(client){that.client=client};that.enableAudio=function(callback){return that.client.enableAudio(that,callback)};that.disableAudio=function(callback){return that.client.disableAudio(that,callback)};that.enableVideo=function(callback){return that.client.enableVideo(that,callback)};that.disableVideo=function(callback){return that.client.disableVideo(that,callback)};that.getId=function(){return spec.streamID};that.drawImage=function(data){if(!header){header=data;if(header.byteLength!=20){console.error("invalid image header "+data.byteLength);resetImageData()}return}if(!yplane){yplane=data;if(yplane.byteLength===20){console.error("invalid image header "+data.byteLength+" "+yplane.byteLength);resetImageData()}return}if(!uplane){uplane=data;if(uplane.byteLength===20){console.error("invalid image header "+data.byteLength+" "+yplane.byteLength+" "+uplane.byteLength);resetImageData()}return}if(!vplane){vplane=data;if(yplane.byteLength!=uplane.byteLength*4||uplane.byteLength!=vplane.byteLength){console.error("invalid image header "+data.byteLength+" "+yplane.byteLength+" "+uplane.byteLength+" "+vplane.byteLength);resetImageData();return}var headerLength=20;var dv=new DataView(header);var format=dv.getUint8(0);var mirror=dv.getUint8(1);var width=dv.getUint16(2);var height=dv.getUint16(4);var left=dv.getUint16(6);var top=dv.getUint16(8);var right=dv.getUint16(10);var bottom=dv.getUint16(12);var rotation=dv.getUint16(14);var ts=dv.getUint32(16);var xWidth=width+left+right;var xHeight=height+top+bottom;var yLength=xWidth*xHeight;var yBegin=headerLength;var yEnd=yBegin+yLength;var uLength=yLength/4;var uBegin=yEnd;var uEnd=uBegin+uLength;var vLength=yLength/4;var vBegin=uEnd;var vEnd=vBegin+vLength;that.render.renderImage({mirror:mirror,width:width,height:height,left:left,top:top,right:right,bottom:bottom,rotation:rotation,yplane:new Uint8Array(yplane),uplane:new Uint8Array(uplane),vplane:new Uint8Array(vplane)});var now32=(Date.now()&4294967295)>>>0;var latency=now32-ts;if(latency>that.latency){that.latency=latency}resetImageData()}};that.setVideoProfile=function(profile){that.profile={};if(typeof profile!=="string"&&typeof profile!=="object"){return false}if(profile.hasOwnProperty("profile")){that.profile.profile=profile.profile}if(profile.hasOwnProperty("swapWidthAndHeight")){that.profile.swapWidthAndHeight=profile.swapWidthAndHeight!==false}else{that.profile.swapWidthAndHeight=false}if(typeof profile=="string"){that.profile.profile=profile}if(that.stream){that.stream.postMessage({type:"send",message:{command:"setVideoProfile",profile:that.profile.profile,swapWidthAndHeight:that.profile.swapWidthAndHeight}})}return true};function resetImageData(){header=undefined;yplane=undefined;uplane=undefined;vplane=undefined}function getReso(w,h){return{width:w,height:h}}var supportedVideoList={true:true,unspecified:true,"120p":getReso(160,120),"240p":getReso(320,240),"360p":getReso(640,360),"480p":getReso(640,480),"720p":getReso(1280,720),"1080p":getReso(1920,1080),"4k":getReso(3840,2160)};function onWorkerMessage(e){var data=e.data;switch(data.type){case"init":that.oninit(data);break;case"close":that.onclose(data);break;case"error":that.onerror(data);break;case"message":that.onmessage(data.data);break}}return that};AgoraRTC.createStream=function(spec){return AgoraRTC.Stream(spec)};AgoraRTC.Client=function(spec){"use strict";var that=AgoraRTC.EventDispatcher(spec);that.signal=undefined;that.localStream=undefined;that.remoteStreams={};that.signalOpen=false;that.agentUrl=AgoraRTC.getAgentUrl();function onSignalError(e){if(that.signalOpen){console.log("signal connection lost.");var message={reason:"LOST_CONNECTION_TO_AGENT",type:"error",eventType:e.type,code:e.code};AgoraCall(that.onFailure,message);that.dispatchEvent(message);return}if(e.type==="error"){return}var message={reason:"CLOSE_BEFORE_OPEN",type:"error",eventType:e.type,code:e.code,agentInstallUrl:getInstallUrl(),agentInstallGuideUrl:getInstallGuideUrl()};AgoraCall(that.onFailure,message);that.dispatchEvent(message)}that.init=function(appId,onSuccess,onFailure){spec.appId=appId;that.onFailure=onFailure;if(!that.signal){try{that.signal=AgoraRTC.Signal({url:that.agentUrl,onFailure:onSignalError})}catch(e){console.log("create signal connection failed"+e);AgoraCall(onFailure,e);return}that.signal.on("onopen",function(event){that.signalOpen=true;that.signal.onEvent=function(e){if(e.event==="error"){var name=getErrorCodeName(e.code);if(name){e.reason=name}}else if(e.event==="onUserMuteAudio"){e.event="peer-mute-audio"}else if(e.event==="onUserMuteVideo"){e.event="peer-mute-video"}that.dispatchEvent(AgoraRTC.MediaEvent({type:e.event,msg:e}))};that.getVersion(function(ev){if(ev.version=="1.6.0"||ev.version=="1.6.1"||ev.version=="1.7.0"||ev.version=="1.7.1"||ev.version=="1.7.2"||ev.version=="1.7.3"||ev.version=="1.7.4"||ev.version=="1.8.0"){initialize(onSuccess,onFailure);return}var message={reason:"INCOMPATIBLE_WEBAGENT",type:"error",version:AgoraRTC.version,agentVersion:ev.version,agentInstallUrl:getInstallUrl(),agentInstallGuideUrl:getInstallGuideUrl()};that.dispatchEvent(message)},function(ev){var message={reason:"INCOMPATIBLE_WEBAGENT",type:"error",version:AgoraRTC.version,agentVersion:ev.version,agentInstallUrl:getInstallUrl(),agentInstallGuideUrl:getInstallGuideUrl()};that.dispatchEvent(message)})})}if(that.signalOpen){initialize(onSuccess,onFailure)}function initialize(onSuccess,onFailure){sendSignalCommand({command:"initialize",appId:spec.appId},function(ev){var m=ev.msg;if(m.code!=true){that.signal.close();that.signal=undefined;onError(m.code,"initialize");return}AgoraCall(onSuccess,m)},onFailure)}function onError(code,type){var name=getErrorCodeName(code);if(!name){name="UNKNOWN_ERROR"}notifyError({reason:name,type:"error",eventType:type,code:code});return}function notifyError(e){AgoraCall(onFailure,e);that.dispatchEvent(e)}function getErrorCodeName(code){for(var k in L.ErrorCode){if(L.ErrorCode.hasOwnProperty(k)){if(L.ErrorCode[k]===code){return String(k)}}}return null}that.signal.on("onError",function(ev){var m=ev.msg;onError(m.code,"error")});console.log(that.signal)};that.renewChannelKey=function(channelKey,onSuccess,onFailure){sendSignalCommand({command:"renewChannelKey",channelKey:channelKey},function(ev){var m=ev.msg;if(m.code===true){AgoraCall(onSuccess,m);return}AgoraCall(onFailure,m)},onFailure)};that.join=function(channelKey,channelName,uid,onSuccess,onFailure){if(channelName.length>64){onFailure(L.ErrorCode.INVALID_CHANNEL_NAME);return}sendSignalCommand({command:"joinChannel",channelKey:channelKey,channelName:channelName,uid:uid},function(ev){that.signal.on("onAddVideoStream",onAddVideoStream);that.signal.on("onPeerLeave",onPeerLeave);var m=ev.msg;if(m.code==true){if(!m.uid){m.uid=0}AgoraCall(onSuccess,m.uid);return}if(m.code==L.ErrorCode.JOIN_CHANNEL_REJECTED){console.error("Command joinChannel has been rejected by agent. Is this user joined a channel already?");AgoraCall(onFailure,m.code);return}AgoraCall(onFailure,m.code)},onFailure)};that.leave=function(onSuccess,onFailure){closeLocalRemoteStreams();sendSignalCommand({command:"leaveChannel"},function(ev){var m=ev.msg;if(m.code===true){AgoraCall(onSuccess,m);return}if(m.code==L.ErrorCode.LEAVE_CHANNEL_REJECTED){console.error("Command leaveChannel has been rejected by agent. Is this user not in a channel?");AgoraCall(onSuccess,m);return}AgoraCall(onFailure,{code:m.code})},function(e){console.log("leave channel failed",e)})};that.publish=function(stream,onSuccess,onFailure){that.localStream=stream;sendSignalCommand({command:"unmuteLocal"},function(ev){var m=ev.msg;if(m.code!=true){AgoraCall(onFailure,{code:m.code});return}AgoraCall(onSuccess,m)},onFailure);stream.bindClient(that)};that.unpublish=function(stream,onSuccess,onFailure){sendSignalCommand({command:"muteLocal"},function(ev){var m=ev.msg;if(m.code!=true){AgoraCall(onFailure,{code:m.code});return}AgoraCall(onSuccess,m)},function(e){AgoraCall(onFailure,{error:e})})};that.subscribe=function(stream,onFailure){stream.init(function(){that.dispatchEvent(AgoraRTC.StreamEvent({type:"stream-subscribed",stream:stream}))},function(error){AgoraCall(onFailure,error)})};that.unsubscribe=function(streamID,onFailure){console.log("remote streams",that.remoteStreams);if(that.remoteStreams[streamID]==undefined){AgoraCall(onFailure,{error:"no such stream"});return}that.remoteStreams[streamID].close();delete that.remoteStreams[streamID]};that.enableAudio=function(stream,callback){sendSignalCommand({command:"enableAudio",streamID:stream.getId()},function(ev){var m=ev.msg;if(callback){AgoraCall(callback,m.code===true)}},callback);return true};that.disableAudio=function(stream,callback){sendSignalCommand({command:"disableAudio",streamID:stream.getId()},function(ev){var m=ev.msg;if(callback){AgoraCall(callback,m.code===true)}},callback);return true};that.enableVideo=function(stream,callback){sendSignalCommand({command:"enableVideo",streamID:stream.getId()},function(ev){var m=ev.msg;if(callback){AgoraCall(callback,m.code===true)}},callback);return true};that.disableVideo=function(stream,callback){sendSignalCommand({command:"disableVideo",streamID:stream.getId()},function(ev){var m=ev.msg;if(callback){AgoraCall(callback,m.code===true)}},callback);return true};that.getDevices=function(callback){sendSignalCommand({command:"getDevices"},function(ev){var devices=ev.msg.devices;AgoraCall(callback,devices)},callback);that.signal.on("getDevices")};that.selectDevice=function(device,onFailure){sendSignalCommand({command:"selectDevice",device:device},function(ev){console.log(ev)},onFailure)};that.startRecording=function(recordingKey,onSuccess,onFailure){sendSignalCommand({command:"startRecording",recordingKey:recordingKey},function(ev){var start=ev.msg;if(start.code===true){AgoraCall(onSuccess,start)}else{AgoraCall(onFailure,start)}},onFailure)};that.stopRecording=function(recordingKey,onSuccess,onFailure){sendSignalCommand({command:"stopRecording",recordingKey:recordingKey},function(ev){var stop=ev.msg;if(stop.code===true){AgoraCall(onSuccess,stop)}else{AgoraCall(onFailure,stop)}},onFailure)};that.queryRecordingStatus=function(onStatus){sendSignalCommand({command:"queryRecordingStatus"},function(ev){onStatus(ev.msg)},onStatus)};that.close=function(){closeLocalRemoteStreams();if(that.signal){that.signal.close();that.signal=undefined}};that.getVersion=function(onSuccess,onFailure){sendSignalCommand({command:"getVersion"},function(ev){var m=ev.msg;if(m.code===true){AgoraCall(onSuccess,m)}else{AgoraCall(onFailure,m)}},onFailure)};that.setParameters=function(parameters){sendSignalCommand({command:"setParameters",parameters:JSON.stringify(parameters)})};that.setEncryptionMode=function(encryptionMode){sendSignalCommand({command:"setEncryptionMode",mode:encryptionMode})};that.setEncryptionSecret=function(secret){sendSignalCommand({command:"setEncryptionSecret",secret:secret})};that.getWindows=function(callback){sendSignalCommand({command:"getWindows"},function(ev){var windows=ev.msg.windows;AgoraCall(callback,windows)},callback)};that.startScreenSharing=function(window,onSuccess,onFailure){sendSignalCommand({command:"startScreenSharing",window:window},onSuccess,onFailure)};that.setScreenSharingWindow=function(window,onSuccess,onFailure){sendSignalCommand({command:"setScreenSharingWindow",window:window},onSuccess,onFailure)};that.stopScreenSharing=function(onSuccess,onFailure){sendSignalCommand({command:"stopScreenSharing"},onSuccess,onFailure)};that.setChannelProfile=function(profile,onSuccess,onFailure){sendSignalCommand({command:"setChannelProfile",profile:profile},onSuccess,onFailure)};that.setClientRole=function(role,permissionKey,onSuccess,onFailure){if(!permissionKey){permissionKey=""}sendSignalCommand({command:"setClientRole",role:role,key:permissionKey},onSuccess,onFailure)};that.enableDualStreamMode=function(enabled,onSuccess,onFailure){sendSignalCommand({command:"enableDualStreamMode",enabled:enabled},onSuccess,onFailure)};function sendSignalCommand(message,onSuccess,onFailure){that.signal.on(message.command,function(e){AgoraCall(onSuccess,e);console.log(e)});that.signal.sendMessage(message,function(e){console.log(e);e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(onFailure,e)})}function onPeerLeave(ev){var m=ev.msg;if(that.remoteStreams.hasOwnProperty(m.uid)){that.remoteStreams[m.uid].close();delete that.remoteStreams[m.uid];console.log("remote streams after peer leave ",that.remoteStreams)}that.dispatchEvent(AgoraRTC.ClientEvent({type:"peer-leave",uid:m.uid}))}function onAddVideoStream(ev){var m=ev.msg;if(!that.remoteStreams.hasOwnProperty(m.uid)){var stream=AgoraRTC.Stream({streamID:m.uid,local:false,audio:m.audio,video:m.video,screen:m.screen});stream.bindClient(that);that.remoteStreams[m.uid]=stream}that.dispatchEvent(AgoraRTC.StreamEvent({type:"stream-added",stream:that.remoteStreams[m.uid]}));console.log("remote streams",that.remoteStreams)}function getInstallUrl(){if(navigator.appVersion.indexOf("Mac")!=-1){return AgoraRTC.macAgentInstallUrl}return AgoraRTC.winAgentInstallUrl}function getInstallGuideUrl(){var userLang=navigator.language||navigator.userLanguage;if(userLang.indexOf("zh")!=-1){return AgoraRTC.cnAgentInstallGuideUrl}return AgoraRTC.enAgentInstallGuideUrl}function closeLocalRemoteStreams(){if(that.localStream){that.localStream.close()}that.localStream=undefined;for(var streamId in that.remoteStreams){if(that.remoteStreams.hasOwnProperty(streamId)){that.remoteStreams[streamId].close();delete that.remoteStreams[streamId]}}}return that};AgoraRTC.createRtcClient=function(){return AgoraRTC.Client({})};AgoraRTC.createLiveClient=function(){return AgoraRTC.Client({})};
//# sourceMappingURL=dist/AgoraRtcAgentSDK.min.js.map
"use strict";(function(root,factory){if(typeof define==="function"&&define.amd){define([],factory)}else{var lib=factory.call(root);Object.keys(lib).forEach(function(key){root[key]=lib[key]})}})(this,function(){var topWindow=this;function error(msg){if(topWindow.console){if(topWindow.console.error){topWindow.console.error(msg)}else if(topWindow.console.log){topWindow.console.log(msg)}}}function isInIFrame(w){w=w||topWindow;return w!==w.top}function makeFailHTML(msg){return""+'<table style="background-color: #8CE; width: 100%; height: 100%;"><tr>'+'<td align="center">'+'<div style="display: table-cell; vertical-align: middle;">'+'<div style="">'+msg+"</div>"+"</div>"+"</td></tr></table>"}var GET_A_WEBGL_BROWSER=""+"This page requires a browser that supports WebGL.<br/>"+'<a href="http://get.webgl.org">Click here to upgrade your browser.</a>';var OTHER_PROBLEM=""+"It doesn't appear your computer can support WebGL.<br/>"+'<a href="http://get.webgl.org/troubleshooting/">Click here for more information.</a>';function create3DContext(canvas,opt_attribs){var names=["webgl","experimental-webgl"];var context=null;for(var ii=0;ii<names.length;++ii){try{context=canvas.getContext(names[ii],opt_attribs)}catch(e){}if(context){break}}return context}function setupWebGL(canvas,opt_attribs){function showLink(str){var container=canvas.parentNode;if(container){container.innerHTML=makeFailHTML(str)}}if(!topWindow.WebGLRenderingContext){showLink(GET_A_WEBGL_BROWSER);return null}var context=create3DContext(canvas,opt_attribs);if(!context){showLink(OTHER_PROBLEM)}return context}function updateCSSIfInIFrame(){if(isInIFrame()){document.body.className="iframe"}}function getWebGLContext(canvas,opt_attribs,opt_options){var options=opt_options||{};if(isInIFrame()){updateCSSIfInIFrame();if(!options.dontResize&&options.resize!==false){var width=canvas.clientWidth;var height=canvas.clientHeight;canvas.width=width;canvas.height=height}}else if(!options.noTitle&&options.title!==false){var title=document.title;var h1=document.createElement("h1");h1.innerText=title;document.body.insertBefore(h1,document.body.children[0])}var gl=setupWebGL(canvas,opt_attribs);return gl}function loadShader(gl,shaderSource,shaderType,opt_errorCallback){var errFn=opt_errorCallback||error;var shader=gl.createShader(shaderType);gl.shaderSource(shader,shaderSource);gl.compileShader(shader);var compiled=gl.getShaderParameter(shader,gl.COMPILE_STATUS);if(!compiled){var lastError=gl.getShaderInfoLog(shader);errFn("*** Error compiling shader '"+shader+"':"+lastError);gl.deleteShader(shader);return null}return shader}function createProgram(gl,shaders,opt_attribs,opt_locations,opt_errorCallback){var errFn=opt_errorCallback||error;var program=gl.createProgram();shaders.forEach(function(shader){gl.attachShader(program,shader)});if(opt_attribs){obj_attrib.forEach(function(attrib,ndx){gl.bindAttribLocation(program,opt_locations?opt_locations[ndx]:ndx,attrib)})}gl.linkProgram(program);var linked=gl.getProgramParameter(program,gl.LINK_STATUS);if(!linked){var lastError=gl.getProgramInfoLog(program);errFn("Error in program linking:"+lastError);gl.deleteProgram(program);return null}return program}function createShaderFromScript(gl,scriptId,opt_shaderType,opt_errorCallback){var shaderSource="";var shaderType;var shaderScript=document.getElementById(scriptId);if(!shaderScript){throw"*** Error: unknown script element"+scriptId}shaderSource=shaderScript.text;if(!opt_shaderType){if(shaderScript.type==="x-shader/x-vertex"){shaderType=gl.VERTEX_SHADER}else if(shaderScript.type==="x-shader/x-fragment"){shaderType=gl.FRAGMENT_SHADER}else if(shaderType!==gl.VERTEX_SHADER&&shaderType!==gl.FRAGMENT_SHADER){throw"*** Error: unknown shader type"}}return loadShader(gl,shaderSource,opt_shaderType?opt_shaderType:shaderType,opt_errorCallback)}var defaultShaderType=["VERTEX_SHADER","FRAGMENT_SHADER"];function createProgramFromScripts(gl,shaderScriptIds,opt_attribs,opt_locations,opt_errorCallback){var shaders=[];for(var ii=0;ii<shaderScriptIds.length;++ii){shaders.push(createShaderFromScript(gl,shaderScriptIds[ii],gl[defaultShaderType[ii]],opt_errorCallback))}return createProgram(gl,shaders,opt_attribs,opt_locations,opt_errorCallback)}function createProgramFromSources(gl,shaderSources,opt_attribs,opt_locations,opt_errorCallback){var shaders=[];for(var ii=0;ii<shaderSources.length;++ii){shaders.push(loadShader(gl,shaderSources[ii],gl[defaultShaderType[ii]],opt_errorCallback))}return createProgram(gl,shaders,opt_attribs,opt_locations,opt_errorCallback)}function getBindPointForSamplerType(gl,type){if(type===gl.SAMPLER_2D)return gl.TEXTURE_2D;if(type===gl.SAMPLER_CUBE)return gl.TEXTURE_CUBE_MAP}function createUniformSetters(gl,program){var textureUnit=0;function createUniformSetter(program,uniformInfo){var location=gl.getUniformLocation(program,uniformInfo.name);var type=uniformInfo.type;var isArray=uniformInfo.size>1&&uniformInfo.name.substr(-3)==="[0]";if(type===gl.FLOAT&&isArray){return function(v){gl.uniform1fv(location,v)}}if(type===gl.FLOAT){return function(v){gl.uniform1f(location,v)}}if(type===gl.FLOAT_VEC2){return function(v){gl.uniform2fv(location,v)}}if(type===gl.FLOAT_VEC3){return function(v){gl.uniform3fv(location,v)}}if(type===gl.FLOAT_VEC4){return function(v){gl.uniform4fv(location,v)}}if(type===gl.INT&&isArray){return function(v){gl.uniform1iv(location,v)}}if(type===gl.INT){return function(v){gl.uniform1i(location,v)}}if(type===gl.INT_VEC2){return function(v){gl.uniform2iv(location,v)}}if(type===gl.INT_VEC3){return function(v){gl.uniform3iv(location,v)}}if(type===gl.INT_VEC4){return function(v){gl.uniform4iv(location,v)}}if(type===gl.BOOL){return function(v){gl.uniform1iv(location,v)}}if(type===gl.BOOL_VEC2){return function(v){gl.uniform2iv(location,v)}}if(type===gl.BOOL_VEC3){return function(v){gl.uniform3iv(location,v)}}if(type===gl.BOOL_VEC4){return function(v){gl.uniform4iv(location,v)}}if(type===gl.FLOAT_MAT2){return function(v){gl.uniformMatrix2fv(location,false,v)}}if(type===gl.FLOAT_MAT3){return function(v){gl.uniformMatrix3fv(location,false,v)}}if(type===gl.FLOAT_MAT4){return function(v){gl.uniformMatrix4fv(location,false,v)}}if((type===gl.SAMPLER_2D||type===gl.SAMPLER_CUBE)&&isArray){var units=[];for(var ii=0;ii<info.size;++ii){units.push(textureUnit++)}return function(bindPoint,units){return function(textures){gl.uniform1iv(location,units);textures.forEach(function(texture,index){gl.activeTexture(gl.TEXTURE0+units[index]);gl.bindTexture(bindPoint,texture)})}}(getBindPointForSamplerType(gl,type),units)}if(type===gl.SAMPLER_2D||type===gl.SAMPLER_CUBE){return function(bindPoint,unit){return function(texture){gl.uniform1i(location,unit);gl.activeTexture(gl.TEXTURE0+unit);gl.bindTexture(bindPoint,texture)}}(getBindPointForSamplerType(gl,type),textureUnit++)}throw"unknown type: 0x"+type.toString(16)}var uniformSetters={};var numUniforms=gl.getProgramParameter(program,gl.ACTIVE_UNIFORMS);for(var ii=0;ii<numUniforms;++ii){var uniformInfo=gl.getActiveUniform(program,ii);if(!uniformInfo){break}var name=uniformInfo.name;if(name.substr(-3)==="[0]"){name=name.substr(0,name.length-3)}var setter=createUniformSetter(program,uniformInfo);uniformSetters[name]=setter}return uniformSetters}function setUniforms(setters,values){Object.keys(values).forEach(function(name){var setter=setters[name];if(setter){setter(values[name])}})}function createAttributeSetters(gl,program){var attribSetters={};function createAttribSetter(index){return function(b){gl.bindBuffer(gl.ARRAY_BUFFER,b.buffer);gl.enableVertexAttribArray(index);gl.vertexAttribPointer(index,b.numComponents||b.size,b.type||gl.FLOAT,b.normalize||false,b.stride||0,b.offset||0)}}var numAttribs=gl.getProgramParameter(program,gl.ACTIVE_ATTRIBUTES);for(var ii=0;ii<numAttribs;++ii){var attribInfo=gl.getActiveAttrib(program,ii);if(!attribInfo){break}var index=gl.getAttribLocation(program,attribInfo.name);attribSetters[attribInfo.name]=createAttribSetter(index)}return attribSetters}function setAttributes(setters,buffers){Object.keys(buffers).forEach(function(name){var setter=setters[name];if(setter){setter(buffers[name])}})}function createProgramInfo(gl,shaderSources,opt_attribs,opt_locations,opt_errorCallback){shaderSources=shaderSources.map(function(source){var script=document.getElementById(source);return script?script.text:source});var program=createProgramFromSources(gl,shaderSources,opt_attribs,opt_locations,opt_errorCallback);if(!program){return null}var uniformSetters=createUniformSetters(gl,program);var attribSetters=createAttributeSetters(gl,program);return{program:program,uniformSetters:uniformSetters,attribSetters:attribSetters}}function setBuffersAndAttributes(gl,setters,buffers){setAttributes(setters,buffers.attribs);if(buffers.indices){gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER,buffers.indices)}}var browserPrefixes=["","MOZ_","OP_","WEBKIT_"];function getExtensionWithKnownPrefixes(gl,name){for(var ii=0;ii<browserPrefixes.length;++ii){var prefixedName=browserPrefixes[ii]+name;var ext=gl.getExtension(prefixedName);if(ext){return ext}}}function resizeCanvasToDisplaySize(canvas,multiplier){multiplier=multiplier||1;var width=canvas.clientWidth*multiplier;var height=canvas.clientHeight*multiplier;if(canvas.width!==width||canvas.height!==height){canvas.width=width;canvas.height=height;return true}return false}function getIFrameForWindow(window){if(!isInIFrame(window)){return}var iframes=window.parent.document.getElementsByTagName("iframe");for(var ii=0;ii<iframes.length;++ii){var iframe=iframes[ii];if(iframe.contentDocument===window.document){return iframe}}}function isFrameVisible(window){try{var iframe=getIFrameForWindow(window);if(!iframe){return true}var bounds=iframe.getBoundingClientRect();var isVisible=bounds.top<window.parent.innerHeight&&bounds.bottom>=0&&bounds.left<window.parent.innerWidth&&bounds.right>=0;return isVisible&&isFrameVisible(window.parent)}catch(e){return true}}function isOnScreen(element){var isVisible=true;if(element){var bounds=element.getBoundingClientRect();isVisible=bounds.top<topWindow.innerHeight&&bounds.bottom>=0}return isVisible&&isFrameVisible(topWindow)}function augmentTypedArray(typedArray,numComponents){var cursor=0;typedArray.push=function(){for(var ii=0;ii<arguments.length;++ii){var value=arguments[ii];if(value instanceof Array||value.buffer&&value.buffer instanceof ArrayBuffer){for(var jj=0;jj<value.length;++jj){typedArray[cursor++]=value[jj]}}else{typedArray[cursor++]=value}}};typedArray.reset=function(opt_index){cursor=opt_index||0};typedArray.numComponents=numComponents;Object.defineProperty(typedArray,"numElements",{get:function(){return this.length/this.numComponents|0}});return typedArray}function createAugmentedTypedArray(numComponents,numElements,opt_type){var Type=opt_type||Float32Array;return augmentTypedArray(new Type(numComponents*numElements),numComponents)}function createBufferFromTypedArray(gl,array,type,drawType){type=type||gl.ARRAY_BUFFER;var buffer=gl.createBuffer();gl.bindBuffer(type,buffer);gl.bufferData(type,array,drawType||gl.STATIC_DRAW);return buffer}function allButIndices(name){return name!=="indices"}function createMapping(obj){var mapping={};Object.keys(obj).filter(allButIndices).forEach(function(key){mapping["a_"+key]=key});return mapping}function getGLTypeForTypedArray(gl,typedArray){if(typedArray instanceof Int8Array){return gl.BYTE}if(typedArray instanceof Uint8Array){return gl.UNSIGNED_BYTE}if(typedArray instanceof Int16Array){return gl.SHORT}if(typedArray instanceof Uint16Array){return gl.UNSIGNED_SHORT}if(typedArray instanceof Int32Array){return gl.INT}if(typedArray instanceof Uint32Array){return gl.UNSIGNED_INT}if(typedArray instanceof Float32Array){return gl.FLOAT}throw"unsupported typed array type"}function getNormalizationForTypedArray(typedArray){if(typedArray instanceof Int8Array){return true}if(typedArray instanceof Uint8Array){return true}return false}function isArrayBuffer(a){return a.buffer&&a.buffer instanceof ArrayBuffer}function guessNumComponentsFromName(name,length){var numComponents;if(name.indexOf("coord")>=0){numComponents=2}else if(name.indexOf("color")>=0){numComponents=4}else{numComponents=3}if(length%numComponents>0){throw"can not guess numComponents. You should specify it."}return numComponents}function makeTypedArray(array,name){if(isArrayBuffer(array)){return array}if(Array.isArray(array)){array={data:array}}if(!array.numComponents){array.numComponents=guessNumComponentsFromName(name,array.length)}var type=array.type;if(!type){if(name==="indices"){type=Uint16Array}}var typedArray=createAugmentedTypedArray(array.numComponents,array.data.length/array.numComponents|0,type);typedArray.push(array.data);return typedArray}function createAttribsFromArrays(gl,arrays,opt_mapping){var mapping=opt_mapping||createMapping(arrays);var attribs={};Object.keys(mapping).forEach(function(attribName){var bufferName=mapping[attribName];var array=makeTypedArray(arrays[bufferName],bufferName);attribs[attribName]={buffer:createBufferFromTypedArray(gl,array),numComponents:array.numComponents||guessNumComponentsFromName(bufferName),type:getGLTypeForTypedArray(gl,array),normalize:getNormalizationForTypedArray(array)}});return attribs}function getNumElementsFromNonIndexedArrays(arrays){var key=Object.keys(arrays)[0];var array=arrays[key];if(isArrayBuffer(array)){return array.numElements}else{return array.data.length/array.numComponents}}function createBufferInfoFromArrays(gl,arrays,opt_mapping){var bufferInfo={attribs:createAttribsFromArrays(gl,arrays,opt_mapping)};var indices=arrays.indices;if(indices){indices=makeTypedArray(indices,"indices");bufferInfo.indices=createBufferFromTypedArray(gl,indices,gl.ELEMENT_ARRAY_BUFFER);bufferInfo.numElements=indices.length}else{bufferInfo.numElements=getNumElementsFromNonIndexedArrays(arrays)}return bufferInfo}function createBuffersFromArrays(gl,arrays){var buffers={};Object.keys(arrays).forEach(function(key){var type=key==="indices"?gl.ELEMENT_ARRAY_BUFFER:gl.ARRAY_BUFFER;var array=makeTypedArray(arrays[key],name);buffers[key]=createBufferFromTypedArray(gl,array,type)});if(arrays.indices){buffers.numElements=arrays.indices.length}else if(arrays.position){buffers.numElements=arrays.position.length/3}return buffers}function drawBufferInfo(gl,type,bufferInfo,count,offset){var indices=bufferInfo.indices;var numElements=count===undefined?bufferInfo.numElements:count;offset=offset===undefined?offset:0;if(indices){gl.drawElements(type,numElements,gl.UNSIGNED_SHORT,offset)}else{gl.drawArrays(type,offset,numElements)}}function drawObjectList(gl,objectsToDraw){var lastUsedProgramInfo=null;var lastUsedBufferInfo=null;objectsToDraw.forEach(function(object){var programInfo=object.programInfo;var bufferInfo=object.bufferInfo;var bindBuffers=false;if(programInfo!==lastUsedProgramInfo){lastUsedProgramInfo=programInfo;gl.useProgram(programInfo.program);bindBuffers=true}if(bindBuffers||bufferInfo!==lastUsedBufferInfo){lastUsedBufferInfo=bufferInfo;setBuffersAndAttributes(gl,programInfo.attribSetters,bufferInfo)}setUniforms(programInfo.uniformSetters,object.uniforms);drawBufferInfo(gl,gl.TRIANGLES,bufferInfo)})}return{createAugmentedTypedArray:createAugmentedTypedArray,createAttribsFromArrays:createAttribsFromArrays,createBuffersFromArrays:createBuffersFromArrays,createBufferInfoFromArrays:createBufferInfoFromArrays,createAttributeSetters:createAttributeSetters,createProgram:createProgram,createProgramFromScripts:createProgramFromScripts,createProgramFromSources:createProgramFromSources,createProgramInfo:createProgramInfo,createUniformSetters:createUniformSetters,drawBufferInfo:drawBufferInfo,drawObjectList:drawObjectList,getWebGLContext:getWebGLContext,updateCSSIfInIFrame:updateCSSIfInIFrame,getExtensionWithKnownPrefixes:getExtensionWithKnownPrefixes,resizeCanvasToDisplaySize:resizeCanvasToDisplaySize,setAttributes:setAttributes,setBuffersAndAttributes:setBuffersAndAttributes,setUniforms:setUniforms,setupWebGL:setupWebGL}});function AgoraRender(){var gl=undefined;var program=undefined;var positionLocation=undefined;var texCoordLocation=undefined;var yTexture=undefined;var uTexture=undefined;var vTexture=undefined;var texCoordBuffer=undefined;var surfaceBuffer=undefined;var that={view:undefined,mirrorView:false,container:undefined,canvas:undefined,renderImageCount:0,initWidth:0,initHeight:0,initRotation:0,canvasUpdated:false,clientWidth:0,clientHeight:0};that.start=function(view,onFailure){initCanvas(view,that.mirrorView,view.clientWidth,view.clientHeight,that.initRotation,onFailure)};that.stop=function(){gl=undefined;program=undefined;positionLocation=undefined;texCoordLocation=undefined;deleteTexture(yTexture);deleteTexture(uTexture);deleteTexture(vTexture);yTexture=undefined;uTexture=undefined;vTexture=undefined;deleteBuffer(texCoordBuffer);deleteBuffer(surfaceBuffer);texCoordBuffer=undefined;surfaceBuffer=undefined;if(that.container){that.container.removeChild(that.canvas)}if(that.view){that.view.removeChild(that.container)}that.canvas=undefined;that.container=undefined;that.view=undefined;that.mirrorView=false};that.renderImage=function(image){if(!gl){return}if(image.width!=that.initWidth||image.height!=that.initHeight||image.rotation!=that.initRotation||image.mirror!=that.mirrorView){var view=that.view;that.stop();console.log("init canvas "+image.width+"*"+image.height+" rotation "+image.rotation);initCanvas(view,image.mirror,image.width,image.height,image.rotation,function(e){console.error("init canvas "+image.width+"*"+image.height+" rotation "+image.rotation+" failed. "+e)})}gl.bindBuffer(gl.ARRAY_BUFFER,texCoordBuffer);var xWidth=image.width+image.left+image.right;var xHeight=image.height+image.top+image.bottom;gl.bufferData(gl.ARRAY_BUFFER,new Float32Array([image.left/xWidth,image.bottom/xHeight,1-image.right/xWidth,image.bottom/xHeight,image.left/xWidth,1-image.top/xHeight,image.left/xWidth,1-image.top/xHeight,1-image.right/xWidth,image.bottom/xHeight,1-image.right/xWidth,1-image.top/xHeight]),gl.STATIC_DRAW);gl.enableVertexAttribArray(texCoordLocation);gl.vertexAttribPointer(texCoordLocation,2,gl.FLOAT,false,0,0);uploadYuv(xWidth,xHeight,image.yplane,image.uplane,image.vplane);updateCanvas(image.rotation,image.width,image.height);gl.drawArrays(gl.TRIANGLES,0,6);that.renderImageCount+=1};function uploadYuv(width,height,yplane,uplane,vplane){gl.activeTexture(gl.TEXTURE0);gl.bindTexture(gl.TEXTURE_2D,yTexture);gl.texImage2D(gl.TEXTURE_2D,0,gl.LUMINANCE,width,height,0,gl.LUMINANCE,gl.UNSIGNED_BYTE,yplane);var e=gl.getError();if(e!=gl.NO_ERROR){console.log("upload y plane ",width,height,yplane.byteLength," error",e)}gl.activeTexture(gl.TEXTURE1);gl.bindTexture(gl.TEXTURE_2D,uTexture);gl.texImage2D(gl.TEXTURE_2D,0,gl.LUMINANCE,width/2,height/2,0,gl.LUMINANCE,gl.UNSIGNED_BYTE,uplane);var e=gl.getError();if(e!=gl.NO_ERROR){console.log("upload u plane ",width,height,uplane.byteLength,"  error",e)}gl.activeTexture(gl.TEXTURE2);gl.bindTexture(gl.TEXTURE_2D,vTexture);gl.texImage2D(gl.TEXTURE_2D,0,gl.LUMINANCE,width/2,height/2,0,gl.LUMINANCE,gl.UNSIGNED_BYTE,vplane);var e=gl.getError();if(e!=gl.NO_ERROR){console.log("upload v plane ",width,height,vplane.byteLength,"  error",e)}}function deleteBuffer(buffer){if(buffer&&that.gl){that.gl.deleteBuffer(buffer)}}function deleteTexture(texture){if(texture&&that.gl){that.gl.deleteTexture(texture)}}function initCanvas(view,mirror,width,height,rotation,onFailure){that.clientWidth=view.clientWidth;that.clientHeight=view.clientHeight;that.view=view;that.mirrorView=mirror;that.canvasUpdated=false;that.container=document.createElement("div");that.container.style.width="100%";that.container.style.height="100%";that.container.style.display="flex";that.container.style.justifyContent="center";that.container.style.alignItems="center";that.view.appendChild(that.container);that.canvas=document.createElement("canvas");if(rotation==0||rotation==180){that.canvas.width=width;that.canvas.height=height}else{that.canvas.width=height;that.canvas.height=width}that.initWidth=width;that.initHeight=height;that.initRotation=rotation;if(that.mirrorView){that.canvas.style.transform="rotateY(180deg)"}that.container.appendChild(that.canvas);try{gl=that.canvas.getContext("webgl")||that.canvas.getContext("experimental-webgl")}catch(e){console.log(e)}if(!gl){gl=undefined;onFailure({error:"Browser not support! No WebGL detected."});return}gl.clearColor(0,0,0,1);gl.enable(gl.DEPTH_TEST);gl.depthFunc(gl.LEQUAL);gl.clear(gl.COLOR_BUFFER_BIT|gl.DEPTH_BUFFER_BIT);program=createProgramFromSources(gl,[AgoraRTC.vertexShaderSource,AgoraRTC.yuvShaderSource]);gl.useProgram(program);initTextures()}function initTextures(){positionLocation=gl.getAttribLocation(program,"a_position");texCoordLocation=gl.getAttribLocation(program,"a_texCoord");surfaceBuffer=gl.createBuffer();texCoordBuffer=gl.createBuffer();gl.activeTexture(gl.TEXTURE0);yTexture=gl.createTexture();gl.bindTexture(gl.TEXTURE_2D,yTexture);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_WRAP_S,gl.CLAMP_TO_EDGE);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_WRAP_T,gl.CLAMP_TO_EDGE);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_MIN_FILTER,gl.NEAREST);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_MAG_FILTER,gl.NEAREST);gl.activeTexture(gl.TEXTURE1);uTexture=gl.createTexture();gl.bindTexture(gl.TEXTURE_2D,uTexture);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_WRAP_S,gl.CLAMP_TO_EDGE);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_WRAP_T,gl.CLAMP_TO_EDGE);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_MIN_FILTER,gl.NEAREST);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_MAG_FILTER,gl.NEAREST);gl.activeTexture(gl.TEXTURE2);vTexture=gl.createTexture();gl.bindTexture(gl.TEXTURE_2D,vTexture);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_WRAP_S,gl.CLAMP_TO_EDGE);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_WRAP_T,gl.CLAMP_TO_EDGE);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_MIN_FILTER,gl.NEAREST);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_MAG_FILTER,gl.NEAREST);var y=gl.getUniformLocation(program,"Ytex");gl.uniform1i(y,0);var u=gl.getUniformLocation(program,"Utex");gl.uniform1i(u,1);var v=gl.getUniformLocation(program,"Vtex");gl.uniform1i(v,2)}function updateCanvas(rotation,width,height){if(that.canvasUpdated){return}that.canvas.style.width="100%";that.canvas.style.height="100%";try{if(rotation===0||rotation===180){if(that.clientWidth/that.clientHeight>width/height){that.canvas.style.width=that.clientHeight*width/height+"px"}else if(that.clientWidth/that.clientHeight<width/height){that.canvas.style.height=that.clientWidth*height/width+"px"}}else{if(that.clientHeight/that.clientWidth>width/height){that.canvas.style.height=that.clientWidth*width/height+"px"}else if(that.clientHeight/that.clientWidth<width/height){that.canvas.style.width=that.clientHeight*height/width+"px"}}}catch(e){console.log("updateCanvas 00001 gone "+that.canvas);console.log(that);console.error(e);return}gl.bindBuffer(gl.ARRAY_BUFFER,surfaceBuffer);gl.enableVertexAttribArray(positionLocation);gl.vertexAttribPointer(positionLocation,2,gl.FLOAT,false,0,0);var p1={x:0,y:0};var p2={x:width,y:0};var p3={x:width,y:height};var p4={x:0,y:height};var pp1=p1,pp2=p2,pp3=p3,pp4=p4;switch(rotation){case 0:break;case 90:pp1=p2;pp2=p3;pp3=p4;pp4=p1;break;case 180:pp1=p3;pp2=p4;pp3=p1;pp4=p2;break;case 270:pp1=p4;pp2=p1;pp3=p2;pp4=p3;break;default:}gl.bufferData(gl.ARRAY_BUFFER,new Float32Array([pp1.x,pp1.y,pp2.x,pp2.y,pp4.x,pp4.y,pp4.x,pp4.y,pp2.x,pp2.y,pp3.x,pp3.y]),gl.STATIC_DRAW);var resolutionLocation=gl.getUniformLocation(program,"u_resolution");gl.uniform2f(resolutionLocation,width,height);that.canvasUpdated=true}return that}var AgoraRTC=function(){"use strict";var AgoraRTC={};Object.defineProperties(AgoraRTC,{version:{get:function(){return"1.8.1"}},name:{get:function(){return"AgoraWebSDK"}}});return AgoraRTC}();AgoraRTC.secureAgentUrl="wss://localhost.agora.io:8921/";AgoraRTC.agentUrl="ws://127.0.0.1:8922/";AgoraRTC.getAgentUrl=function(){if(location.protocol==="https:"){return AgoraRTC.secureAgentUrl}return AgoraRTC.agentUrl};AgoraRTC.macAgentInstallUrl="http://download.agora.io/AgoraWebAgent-1.8.1.pkg";AgoraRTC.winAgentInstallUrl="http://download.agora.io/AgoraWebAgentSetup-1.8.1.exe";AgoraRTC.enAgentInstallGuideUrl="http://download.agora.io/install-guide-en.html";AgoraRTC.cnAgentInstallGuideUrl="http://download.agora.io/install-guide-cn.html";AgoraRTC.vertexShaderSource="attribute vec2 a_position;"+"attribute vec2 a_texCoord;"+"uniform vec2 u_resolution;"+"varying vec2 v_texCoord;"+"void main() {"+"vec2 zeroToOne = a_position / u_resolution;"+"   vec2 zeroToTwo = zeroToOne * 2.0;"+"   vec2 clipSpace = zeroToTwo - 1.0;"+"   gl_Position = vec4(clipSpace * vec2(1, -1), 0, 1);"+"v_texCoord = a_texCoord;"+"}";AgoraRTC.yuvShaderSource="precision mediump float;"+"uniform sampler2D Ytex;"+"uniform sampler2D Utex,Vtex;"+"varying vec2 v_texCoord;"+"void main(void) {"+"  float nx,ny,r,g,b,y,u,v;"+"  mediump vec4 txl,ux,vx;"+"  nx=v_texCoord[0];"+"  ny=v_texCoord[1];"+"  y=texture2D(Ytex,vec2(nx,ny)).r;"+"  u=texture2D(Utex,vec2(nx,ny)).r;"+"  v=texture2D(Vtex,vec2(nx,ny)).r;"+"  y=1.1643*(y-0.0625);"+"  u=u-0.5;"+"  v=v-0.5;"+"  r=y+1.5958*v;"+"  g=y-0.39173*u-0.81290*v;"+"  b=y+2.017*u;"+"  gl_FragColor=vec4(r,g,b,1.0);"+"}";AgoraRTC.videoStreamSource='function onSuccess(){postMessage({type:"init",result:!0})}function onFailure(a){postMessage(a)}function sendMessage(a){"videoStat"!=a.command&&console.log(JSON.stringify(a)),that.stream&&that.stream.readyState==WebSocket.OPEN&&that.stream.send(JSON.stringify(a))}var that={};that.stream=void 0,that.init=function(a){return that.stream&&that.stream.readyState===WebSocket.OPEN?(console.warn("stream "+that.getId()+" has been initialized already"),void onSuccess()):(that.local=a.local,that.profile=a.videoProfile,that.stream=new WebSocket(a.url),that.stream.onopen=function(a){console.log((that.local?"local":"remote")+" Stream "),console.log(a),that.profile&&sendMessage({command:"setVideoProfile",profile:that.profile.profile,swapWidthAndHeight:that.profile.swapWidthAndHeight}),that.stream.binaryType="arraybuffer",onSuccess()},that.stream.onclose=function(a){console.log(that.local?"local":"remote"," Stream ",a),that.stream=void 0,onFailure({type:a.type,code:a.code,reason:a.reason})},that.stream.onerror=function(a){console.log(that.local?"local":"remote"," Stream ",a),onFailure({type:a.type,code:a.code,reason:a.reason})},void(that.stream.onmessage=function(a){"string"==typeof a.data?console.log(that.local?"local":"remote"," message from agent ",a.data):a.data instanceof ArrayBuffer?postMessage({type:"message",data:a.data}):a.data instanceof Blob&&console.warn("Blob image data is not supported")}))},that.close=function(){that.stream&&(that.stream.onmessage=void 0,that.stream.close()),that.stream=void 0},self.addEventListener("message",function(a){var b=a.data;switch(b.type){case"init":that.init(b);break;case"send":sendMessage(b.message);break;case"close":that.close()}},!1);';var AgoraCall=function(callback,parameters){if(callback){callback(parameters)}};var L={VideoProfiles:{Profile120P:"120P",Profile120P_3:"120P_3",Profile96P:"96P",Profile48P:"48P",Profile180P:"180P",Profile180P_3:"180P_3",Profile180P_4:"180P_4",Profile240P:"240P",Profile240P_3:"240P_3",Profile240P_4:"240P_4",Profile360P:"360P",Profile360P_3:"360P_3",Profile360P_4:"360P_4",Profile360P_6:"360P_6",Profile360P_7:"360P_7",Profile360P_8:"360P_8",Profile480P:"480P",Profile480P_3:"480P_3",Profile480P_4:"480P_4",Profile480P_6:"480P_6",Profile480P_8:"480P_8",Profile480P_9:"480P_9",Profile720P:"720P",Profile720P_3:"720P_3",Profile720P_5:"720P_5",Profile720P_6:"720P_6"},ErrorCode:{NO_ERROR:0,FAILED:1,INVALID_ARGUMENT:2,NOT_READY:3,NOT_SUPPORTED:4,REFUSED:5,BUFFER_TOO_SMALL:6,NOT_INITIALIZED:7,INVALID_VIEW:8,NO_PERMISSION:9,TIMEDOUT:10,CANCELED:11,TOO_OFTEN:12,BIND_SOCKET:13,NET_DOWN:14,NET_NOBUFS:15,INIT_VIDEO:16,JOIN_CHANNEL_REJECTED:17,LEAVE_CHANNEL_REJECTED:18,ALREADY_IN_USE:19,INVALID_APP_ID:101,INVALID_CHANNEL_NAME:102,CHANNEL_KEY_EXPIRED:109,INVALID_CHANNEL_KEY:110,CONNECTION_INTERRUPTED:111,CONNECTION_LOST:112,LOAD_MEDIA_ENGINE:1001,START_CALL:1002,START_CAMERA:1003,START_VIDEO_RENDER:1004,ADM_GENERAL_ERROR:1005,ADM_JAVA_RESOURCE:1006,ADM_SAMPLE_RATE:1007,ADM_INIT_PLAYOUT:1008,ADM_START_PLAYOUT:1009,ADM_STOP_PLAYOUT:1010,ADM_INIT_RECORDING:1011,ADM_START_RECORDING:1012,ADM_STOP_RECORDING:1013,ADM_RUNTIME_PLAYOUT_ERROR:1015,ADM_RUNTIME_RECORDING_ERROR:1017,ADM_RECORD_AUDIO_FAILED:1018,ADM_INIT_LOOPBACK:1022,ADM_START_LOOPBACK:1023,VDM_CAMERA_NOT_AUTHORIZED:1501}};AgoraRTC.EventDispatcher=function(spec){"use strict";var that={};spec.dispatcher={};spec.dispatcher.eventListeners={};that.addEventListener=function(eventType,listener){spec.dispatcher.eventListeners[eventType]=listener;if(!spec.dispatcher.eventListeners[eventType]){delete spec.dispatcher.eventListeners[eventType]}};that.on=that.addEventListener;that.dispatchEvent=function(event){if(spec.dispatcher.eventListeners.hasOwnProperty(event.type)){spec.dispatcher.eventListeners[event.type](event)}};return that};AgoraRTC.BasicEvent=function(spec){"use strict";var that={};that.type=spec.type;return that};AgoraRTC.StreamEvent=function(spec){"use strict";var that=AgoraRTC.BasicEvent(spec);that.stream=spec.stream;that.msg=spec.msg;return that};AgoraRTC.ClientEvent=function(spec){"use strict";var that=AgoraRTC.BasicEvent(spec);that.uid=spec.uid;that.attr=spec.attr;that.streams=spec.streams;return that};AgoraRTC.MediaEvent=function(spec){"use strict";var that=AgoraRTC.BasicEvent(spec);that.msg=spec.msg;return that};AgoraRTC.Signal=function(spec){var that=AgoraRTC.EventDispatcher(spec);that.connection=new WebSocket(spec.url);that.sendMessage=function(obj,onError){if(that.connection.readyState==WebSocket.OPEN){that.connection.send(JSON.stringify(obj))}else{console.log("connection to agent lost.");onError({error:"not connected"})}};that.close=function(){that.onEvent=onEvent;that.connection.onopen=undefined;that.connection.onclose=undefined;that.connection.onerror=undefined;that.connection.onmessage=undefined;that.connection.close()};that.connection.onopen=function(e){console.log(e);that.dispatchEvent(AgoraRTC.MediaEvent({type:"onopen",event:e}))};that.connection.onclose=function(e){console.log(e);AgoraCall(spec.onFailure,e)};that.connection.onerror=function(e){console.log(e);AgoraCall(spec.onFailure,e)};that.onEvent=onEvent;that.connection.onmessage=function(e){console.log(e);var m=JSON.parse(e.data);if(m.hasOwnProperty("command")){that.dispatchEvent(AgoraRTC.MediaEvent({type:m.command,msg:m}))}else if(m.hasOwnProperty("event")){that.onEvent(m)}};function onEvent(e){console.log(e)}return that};AgoraRTC.Stream=function(spec){var that=AgoraRTC.EventDispatcher(spec);that.stream=undefined;that.render=undefined;that.interval=undefined;that.lastRenderCount=0;that.profile=undefined;that.latency=0;var header=undefined;var yplane=undefined;var uplane=undefined;var vplane=undefined;that.init=function(onSuccess,onFailure){if(that.stream){console.warn("stream "+that.getId()+" has been initialized already");onSuccess();return}var videoStreamFileBlob=new Blob([AgoraRTC.videoStreamSource],{type:"application/javascript"});that.stream=new Worker(URL.createObjectURL(videoStreamFileBlob));that.oninit=function(e){console.log((spec.local?"local":"remote")+" Stream ");console.log(e);if(e.result===true){onSuccess()}else{onFailure()}};that.onclose=function(e){window.clearInterval(that.interval);console.log(spec.local?"local":"remote"," Stream ",e);that.stream=undefined};that.onerror=function(e){window.clearInterval(that.interval);console.log(spec.local?"local":"remote"," Stream ",e);onFailure(e)};that.onmessage=function(e){if(typeof e==="string"){console.log(spec.local?"local":"remote"," message from agent ",e)}else if(e instanceof ArrayBuffer){if(that.render){that.drawImage(e)}}else if(e instanceof Blob){console.warn("Blob image data is not supported")}};that.stream.addEventListener("message",onWorkerMessage,false);that.stream.postMessage({type:"init",local:spec.local,
url:AgoraRTC.getAgentUrl(),videoProfile:that.profile})};that.close=function(){window.clearInterval(that.interval);if(that.stream){that.stream.removeEventListener("message",onWorkerMessage);that.stream.postMessage({type:"close"})}that.stream=undefined;that.stop()};that.play=function(div,onFailure){var view=document.getElementById(div);that.stop();that.render=AgoraRender();that.render.start(view,onFailure);if(spec.local){that.stream.postMessage({type:"send",message:{command:"preview",uid:spec.streamID,audio:spec.audio===false?false:true,video:spec.video===false?false:true,screen:spec.screen===true?true:false,viewWidth:view.clientWidth,viewHeight:view.clientHeight}})}else{that.stream.postMessage({type:"send",message:{command:"subscribe",uid:spec.streamID,viewWidth:view.clientWidth,viewHeight:view.clientHeight}})}if(that.interval){window.clearInterval(that.interval)}that.interval=window.setInterval(function(){if(that.render){var fps=that.render.renderImageCount-that.lastRenderCount;that.stream.postMessage({type:"send",message:{command:"videoStat",uid:spec.streamID,fps:fps,frameCount:that.lastRenderCount,latency:that.latency}});that.lastRenderCount=that.render.renderImageCount;that.latency=0}},1e3)};that.stop=function(){if(that.render){that.render.stop()}that.render=undefined;that.lastRenderCount=0};that.bindClient=function(client){that.client=client};that.enableAudio=function(callback){return that.client.enableAudio(that,callback)};that.disableAudio=function(callback){return that.client.disableAudio(that,callback)};that.enableVideo=function(callback){return that.client.enableVideo(that,callback)};that.disableVideo=function(callback){return that.client.disableVideo(that,callback)};that.getId=function(){return spec.streamID};that.drawImage=function(data){if(!header){header=data;if(header.byteLength!=20){console.error("invalid image header "+data.byteLength);resetImageData()}return}if(!yplane){yplane=data;if(yplane.byteLength===20){console.error("invalid image header "+data.byteLength+" "+yplane.byteLength);resetImageData()}return}if(!uplane){uplane=data;if(uplane.byteLength===20){console.error("invalid image header "+data.byteLength+" "+yplane.byteLength+" "+uplane.byteLength);resetImageData()}return}if(!vplane){vplane=data;if(yplane.byteLength!=uplane.byteLength*4||uplane.byteLength!=vplane.byteLength){console.error("invalid image header "+data.byteLength+" "+yplane.byteLength+" "+uplane.byteLength+" "+vplane.byteLength);resetImageData();return}var headerLength=20;var dv=new DataView(header);var format=dv.getUint8(0);var mirror=dv.getUint8(1);var width=dv.getUint16(2);var height=dv.getUint16(4);var left=dv.getUint16(6);var top=dv.getUint16(8);var right=dv.getUint16(10);var bottom=dv.getUint16(12);var rotation=dv.getUint16(14);var ts=dv.getUint32(16);var xWidth=width+left+right;var xHeight=height+top+bottom;var yLength=xWidth*xHeight;var yBegin=headerLength;var yEnd=yBegin+yLength;var uLength=yLength/4;var uBegin=yEnd;var uEnd=uBegin+uLength;var vLength=yLength/4;var vBegin=uEnd;var vEnd=vBegin+vLength;that.render.renderImage({mirror:mirror,width:width,height:height,left:left,top:top,right:right,bottom:bottom,rotation:rotation,yplane:new Uint8Array(yplane),uplane:new Uint8Array(uplane),vplane:new Uint8Array(vplane)});var now32=(Date.now()&4294967295)>>>0;var latency=now32-ts;if(latency>that.latency){that.latency=latency}resetImageData()}};that.setVideoProfile=function(profile){that.profile={};if(typeof profile!=="string"&&typeof profile!=="object"){return false}if(profile.hasOwnProperty("profile")){that.profile.profile=profile.profile}if(profile.hasOwnProperty("swapWidthAndHeight")){that.profile.swapWidthAndHeight=profile.swapWidthAndHeight!==false}else{that.profile.swapWidthAndHeight=false}if(typeof profile=="string"){that.profile.profile=profile}if(that.stream){that.stream.postMessage({type:"send",message:{command:"setVideoProfile",profile:that.profile.profile,swapWidthAndHeight:that.profile.swapWidthAndHeight}})}return true};function resetImageData(){header=undefined;yplane=undefined;uplane=undefined;vplane=undefined}function getReso(w,h){return{width:w,height:h}}var supportedVideoList={true:true,unspecified:true,"120p":getReso(160,120),"240p":getReso(320,240),"360p":getReso(640,360),"480p":getReso(640,480),"720p":getReso(1280,720),"1080p":getReso(1920,1080),"4k":getReso(3840,2160)};function onWorkerMessage(e){var data=e.data;switch(data.type){case"init":that.oninit(data);break;case"close":that.onclose(data);break;case"error":that.onerror(data);break;case"message":that.onmessage(data.data);break}}return that};AgoraRTC.createStream=function(spec){return AgoraRTC.Stream(spec)};AgoraRTC.Client=function(spec){"use strict";var that=AgoraRTC.EventDispatcher(spec);that.signal=undefined;that.localStream=undefined;that.remoteStreams={};that.signalOpen=false;that.agentUrl=AgoraRTC.getAgentUrl();function onSignalError(e){if(that.signalOpen){console.log("signal connection lost.");var message={reason:"LOST_CONNECTION_TO_AGENT",type:"error",eventType:e.type,code:e.code};AgoraCall(that.onFailure,message);that.dispatchEvent(message);return}if(e.type==="error"){return}var message={reason:"CLOSE_BEFORE_OPEN",type:"error",eventType:e.type,code:e.code,agentInstallUrl:getInstallUrl(),agentInstallGuideUrl:getInstallGuideUrl()};AgoraCall(that.onFailure,message);that.dispatchEvent(message)}that.init=function(appId,onSuccess,onFailure){spec.appId=appId;that.onFailure=onFailure;if(!that.signal){try{that.signal=AgoraRTC.Signal({url:that.agentUrl,onFailure:onSignalError})}catch(e){console.log("create signal connection failed"+e);AgoraCall(onFailure,e);return}that.signal.on("onopen",function(event){that.signalOpen=true;that.signal.onEvent=function(e){if(e.event==="error"){var name=getErrorCodeName(e.code);if(name){e.reason=name}}else if(e.event==="onUserMuteAudio"){e.event="peer-mute-audio"}else if(e.event==="onUserMuteVideo"){e.event="peer-mute-video"}that.dispatchEvent(AgoraRTC.MediaEvent({type:e.event,msg:e}))};that.getVersion(function(ev){if(ev.version=="1.6.0"||ev.version=="1.6.1"||ev.version=="1.7.0"||ev.version=="1.7.1"||ev.version=="1.7.2"||ev.version=="1.7.3"||ev.version=="1.7.4"||ev.version=="1.8.0"||ev.version=="1.8.1"){initialize(onSuccess,onFailure);return}var message={reason:"INCOMPATIBLE_WEBAGENT",type:"error",version:AgoraRTC.version,agentVersion:ev.version,agentInstallUrl:getInstallUrl(),agentInstallGuideUrl:getInstallGuideUrl()};that.dispatchEvent(message)},function(ev){var message={reason:"INCOMPATIBLE_WEBAGENT",type:"error",version:AgoraRTC.version,agentVersion:ev.version,agentInstallUrl:getInstallUrl(),agentInstallGuideUrl:getInstallGuideUrl()};that.dispatchEvent(message)})})}if(that.signalOpen){initialize(onSuccess,onFailure)}function initialize(onSuccess,onFailure){sendSignalCommand({command:"initialize",appId:spec.appId},function(ev){var m=ev.msg;if(m.code!=true){that.signal.close();that.signal=undefined;onError(m.code,"initialize");return}AgoraCall(onSuccess,m)},onFailure)}function onError(code,type){var name=getErrorCodeName(code);if(!name){name="UNKNOWN_ERROR"}notifyError({reason:name,type:"error",eventType:type,code:code});return}function notifyError(e){AgoraCall(onFailure,e);that.dispatchEvent(e)}function getErrorCodeName(code){for(var k in L.ErrorCode){if(L.ErrorCode.hasOwnProperty(k)){if(L.ErrorCode[k]===code){return String(k)}}}return null}that.signal.on("onError",function(ev){var m=ev.msg;onError(m.code,"error")});console.log(that.signal)};that.renewChannelKey=function(channelKey,onSuccess,onFailure){sendSignalCommand({command:"renewChannelKey",channelKey:channelKey},function(ev){var m=ev.msg;if(m.code===true){AgoraCall(onSuccess,m);return}AgoraCall(onFailure,m)},onFailure)};that.join=function(channelKey,channelName,uid,onSuccess,onFailure){if(channelName.length>64){onFailure(L.ErrorCode.INVALID_CHANNEL_NAME);return}sendSignalCommand({command:"joinChannel",channelKey:channelKey,channelName:channelName,uid:uid},function(ev){that.signal.on("onAddVideoStream",onAddVideoStream);that.signal.on("onPeerLeave",onPeerLeave);var m=ev.msg;if(m.code==true){if(!m.uid){m.uid=0}AgoraCall(onSuccess,m.uid);return}if(m.code==L.ErrorCode.JOIN_CHANNEL_REJECTED){console.error("Command joinChannel has been rejected by agent. Is this user joined a channel already?");AgoraCall(onFailure,m.code);return}AgoraCall(onFailure,m.code)},onFailure)};that.leave=function(onSuccess,onFailure){closeLocalRemoteStreams();sendSignalCommand({command:"leaveChannel"},function(ev){var m=ev.msg;if(m.code===true){AgoraCall(onSuccess,m);return}if(m.code==L.ErrorCode.LEAVE_CHANNEL_REJECTED){console.error("Command leaveChannel has been rejected by agent. Is this user not in a channel?");AgoraCall(onSuccess,m);return}AgoraCall(onFailure,{code:m.code})},function(e){console.log("leave channel failed",e)})};that.publish=function(stream,onSuccess,onFailure){that.localStream=stream;sendSignalCommand({command:"unmuteLocal"},function(ev){var m=ev.msg;if(m.code!=true){AgoraCall(onFailure,{code:m.code});return}AgoraCall(onSuccess,m)},onFailure);stream.bindClient(that)};that.unpublish=function(stream,onSuccess,onFailure){sendSignalCommand({command:"muteLocal"},function(ev){var m=ev.msg;if(m.code!=true){AgoraCall(onFailure,{code:m.code});return}AgoraCall(onSuccess,m)},function(e){AgoraCall(onFailure,{error:e})})};that.subscribe=function(stream,onFailure){stream.init(function(){that.dispatchEvent(AgoraRTC.StreamEvent({type:"stream-subscribed",stream:stream}))},function(error){AgoraCall(onFailure,error)})};that.unsubscribe=function(streamID,onFailure){console.log("remote streams",that.remoteStreams);if(that.remoteStreams[streamID]==undefined){AgoraCall(onFailure,{error:"no such stream"});return}that.remoteStreams[streamID].close();delete that.remoteStreams[streamID]};that.enableAudio=function(stream,callback){sendSignalCommand({command:"enableAudio",streamID:stream.getId()},function(ev){var m=ev.msg;if(callback){AgoraCall(callback,m.code===true)}},callback);return true};that.disableAudio=function(stream,callback){sendSignalCommand({command:"disableAudio",streamID:stream.getId()},function(ev){var m=ev.msg;if(callback){AgoraCall(callback,m.code===true)}},callback);return true};that.enableVideo=function(stream,callback){sendSignalCommand({command:"enableVideo",streamID:stream.getId()},function(ev){var m=ev.msg;if(callback){AgoraCall(callback,m.code===true)}},callback);return true};that.disableVideo=function(stream,callback){sendSignalCommand({command:"disableVideo",streamID:stream.getId()},function(ev){var m=ev.msg;if(callback){AgoraCall(callback,m.code===true)}},callback);return true};that.getDevices=function(callback){sendSignalCommand({command:"getDevices"},function(ev){var devices=ev.msg.devices;AgoraCall(callback,devices)},callback)};that.selectDevice=function(device,onFailure){sendSignalCommand({command:"selectDevice",device:device},function(ev){console.log(ev)},onFailure)};that.startRecording=function(recordingKey,onSuccess,onFailure){sendSignalCommand({command:"startRecording",recordingKey:recordingKey},function(ev){var start=ev.msg;if(start.code===true){AgoraCall(onSuccess,start)}else{AgoraCall(onFailure,start)}},onFailure)};that.stopRecording=function(recordingKey,onSuccess,onFailure){sendSignalCommand({command:"stopRecording",recordingKey:recordingKey},function(ev){var stop=ev.msg;if(stop.code===true){AgoraCall(onSuccess,stop)}else{AgoraCall(onFailure,stop)}},onFailure)};that.queryRecordingStatus=function(onStatus){sendSignalCommand({command:"queryRecordingStatus"},function(ev){onStatus(ev.msg)},onStatus)};that.close=function(){closeLocalRemoteStreams();if(that.signal){that.signal.close();that.signal=undefined}};that.getVersion=function(onSuccess,onFailure){sendSignalCommand({command:"getVersion"},function(ev){var m=ev.msg;if(m.code===true){AgoraCall(onSuccess,m)}else{AgoraCall(onFailure,m)}},onFailure)};that.setParameters=function(parameters){sendSignalCommand({command:"setParameters",parameters:JSON.stringify(parameters)})};that.setEncryptionMode=function(encryptionMode){sendSignalCommand({command:"setEncryptionMode",mode:encryptionMode})};that.setEncryptionSecret=function(secret){sendSignalCommand({command:"setEncryptionSecret",secret:secret})};that.getWindows=function(callback){sendSignalCommand({command:"getWindows"},function(ev){var windows=ev.msg.windows;AgoraCall(callback,windows)},callback)};that.startScreenSharing=function(window,onSuccess,onFailure){sendSignalCommand({command:"startScreenSharing",window:window},onSuccess,onFailure)};that.setScreenSharingWindow=function(window,onSuccess,onFailure){sendSignalCommand({command:"setScreenSharingWindow",window:window},onSuccess,onFailure)};that.stopScreenSharing=function(onSuccess,onFailure){sendSignalCommand({command:"stopScreenSharing"},onSuccess,onFailure)};that.setChannelProfile=function(profile,onSuccess,onFailure){sendSignalCommand({command:"setChannelProfile",profile:profile},onSuccess,onFailure)};that.setClientRole=function(role,permissionKey,onSuccess,onFailure){if(!permissionKey){permissionKey=""}sendSignalCommand({command:"setClientRole",role:role,key:permissionKey},onSuccess,onFailure)};that.enableDualStreamMode=function(enabled,onSuccess,onFailure){sendSignalCommand({command:"enableDualStreamMode",enabled:enabled},onSuccess,onFailure)};function sendSignalCommand(message,onSuccess,onFailure){that.signal.on(message.command,function(e){AgoraCall(onSuccess,e);console.log(e)});that.signal.sendMessage(message,function(e){console.log(e);e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(onFailure,e)})}function onPeerLeave(ev){var m=ev.msg;if(that.remoteStreams.hasOwnProperty(m.uid)){that.remoteStreams[m.uid].close();delete that.remoteStreams[m.uid];console.log("remote streams after peer leave ",that.remoteStreams)}that.dispatchEvent(AgoraRTC.ClientEvent({type:"peer-leave",uid:m.uid}))}function onAddVideoStream(ev){var m=ev.msg;if(!that.remoteStreams.hasOwnProperty(m.uid)){var stream=AgoraRTC.Stream({streamID:m.uid,local:false,audio:m.audio,video:m.video,screen:m.screen});stream.bindClient(that);that.remoteStreams[m.uid]=stream}that.dispatchEvent(AgoraRTC.StreamEvent({type:"stream-added",stream:that.remoteStreams[m.uid]}));console.log("remote streams",that.remoteStreams)}function getInstallUrl(){if(navigator.appVersion.indexOf("Mac")!=-1){return AgoraRTC.macAgentInstallUrl}return AgoraRTC.winAgentInstallUrl}function getInstallGuideUrl(){var userLang=navigator.language||navigator.userLanguage;if(userLang.indexOf("zh")!=-1){return AgoraRTC.cnAgentInstallGuideUrl}return AgoraRTC.enAgentInstallGuideUrl}function closeLocalRemoteStreams(){if(that.localStream){that.localStream.close()}that.localStream=undefined;for(var streamId in that.remoteStreams){if(that.remoteStreams.hasOwnProperty(streamId)){that.remoteStreams[streamId].close();delete that.remoteStreams[streamId]}}}return that};AgoraRTC.createRtcClient=function(){return AgoraRTC.Client({})};AgoraRTC.createLiveClient=function(){return AgoraRTC.Client({})};
//# sourceMappingURL=dist/AgoraRtcAgentSDK.min.js.map
"use strict";(function(root,factory){if(typeof define==="function"&&define.amd){define([],factory)}else{var lib=factory.call(root);Object.keys(lib).forEach(function(key){root[key]=lib[key]})}})(this,function(){var topWindow=this;function error(msg){if(topWindow.console){if(topWindow.console.error){topWindow.console.error(msg)}else if(topWindow.console.log){topWindow.console.log(msg)}}}function isInIFrame(w){w=w||topWindow;return w!==w.top}function makeFailHTML(msg){return""+'<table style="background-color: #8CE; width: 100%; height: 100%;"><tr>'+'<td align="center">'+'<div style="display: table-cell; vertical-align: middle;">'+'<div style="">'+msg+"</div>"+"</div>"+"</td></tr></table>"}var GET_A_WEBGL_BROWSER=""+"This page requires a browser that supports WebGL.<br/>"+'<a href="http://get.webgl.org">Click here to upgrade your browser.</a>';var OTHER_PROBLEM=""+"It doesn't appear your computer can support WebGL.<br/>"+'<a href="http://get.webgl.org/troubleshooting/">Click here for more information.</a>';function create3DContext(canvas,opt_attribs){var names=["webgl","experimental-webgl"];var context=null;for(var ii=0;ii<names.length;++ii){try{context=canvas.getContext(names[ii],opt_attribs)}catch(e){}if(context){break}}return context}function setupWebGL(canvas,opt_attribs){function showLink(str){var container=canvas.parentNode;if(container){container.innerHTML=makeFailHTML(str)}}if(!topWindow.WebGLRenderingContext){showLink(GET_A_WEBGL_BROWSER);return null}var context=create3DContext(canvas,opt_attribs);if(!context){showLink(OTHER_PROBLEM)}return context}function updateCSSIfInIFrame(){if(isInIFrame()){document.body.className="iframe"}}function getWebGLContext(canvas,opt_attribs,opt_options){var options=opt_options||{};if(isInIFrame()){updateCSSIfInIFrame();if(!options.dontResize&&options.resize!==false){var width=canvas.clientWidth;var height=canvas.clientHeight;canvas.width=width;canvas.height=height}}else if(!options.noTitle&&options.title!==false){var title=document.title;var h1=document.createElement("h1");h1.innerText=title;document.body.insertBefore(h1,document.body.children[0])}var gl=setupWebGL(canvas,opt_attribs);return gl}function loadShader(gl,shaderSource,shaderType,opt_errorCallback){var errFn=opt_errorCallback||error;var shader=gl.createShader(shaderType);gl.shaderSource(shader,shaderSource);gl.compileShader(shader);var compiled=gl.getShaderParameter(shader,gl.COMPILE_STATUS);if(!compiled){var lastError=gl.getShaderInfoLog(shader);errFn("*** Error compiling shader '"+shader+"':"+lastError);gl.deleteShader(shader);return null}return shader}function createProgram(gl,shaders,opt_attribs,opt_locations,opt_errorCallback){var errFn=opt_errorCallback||error;var program=gl.createProgram();shaders.forEach(function(shader){gl.attachShader(program,shader)});if(opt_attribs){obj_attrib.forEach(function(attrib,ndx){gl.bindAttribLocation(program,opt_locations?opt_locations[ndx]:ndx,attrib)})}gl.linkProgram(program);var linked=gl.getProgramParameter(program,gl.LINK_STATUS);if(!linked){var lastError=gl.getProgramInfoLog(program);errFn("Error in program linking:"+lastError);gl.deleteProgram(program);return null}return program}function createShaderFromScript(gl,scriptId,opt_shaderType,opt_errorCallback){var shaderSource="";var shaderType;var shaderScript=document.getElementById(scriptId);if(!shaderScript){throw"*** Error: unknown script element"+scriptId}shaderSource=shaderScript.text;if(!opt_shaderType){if(shaderScript.type==="x-shader/x-vertex"){shaderType=gl.VERTEX_SHADER}else if(shaderScript.type==="x-shader/x-fragment"){shaderType=gl.FRAGMENT_SHADER}else if(shaderType!==gl.VERTEX_SHADER&&shaderType!==gl.FRAGMENT_SHADER){throw"*** Error: unknown shader type"}}return loadShader(gl,shaderSource,opt_shaderType?opt_shaderType:shaderType,opt_errorCallback)}var defaultShaderType=["VERTEX_SHADER","FRAGMENT_SHADER"];function createProgramFromScripts(gl,shaderScriptIds,opt_attribs,opt_locations,opt_errorCallback){var shaders=[];for(var ii=0;ii<shaderScriptIds.length;++ii){shaders.push(createShaderFromScript(gl,shaderScriptIds[ii],gl[defaultShaderType[ii]],opt_errorCallback))}return createProgram(gl,shaders,opt_attribs,opt_locations,opt_errorCallback)}function createProgramFromSources(gl,shaderSources,opt_attribs,opt_locations,opt_errorCallback){var shaders=[];for(var ii=0;ii<shaderSources.length;++ii){shaders.push(loadShader(gl,shaderSources[ii],gl[defaultShaderType[ii]],opt_errorCallback))}return createProgram(gl,shaders,opt_attribs,opt_locations,opt_errorCallback)}function getBindPointForSamplerType(gl,type){if(type===gl.SAMPLER_2D)return gl.TEXTURE_2D;if(type===gl.SAMPLER_CUBE)return gl.TEXTURE_CUBE_MAP}function createUniformSetters(gl,program){var textureUnit=0;function createUniformSetter(program,uniformInfo){var location=gl.getUniformLocation(program,uniformInfo.name);var type=uniformInfo.type;var isArray=uniformInfo.size>1&&uniformInfo.name.substr(-3)==="[0]";if(type===gl.FLOAT&&isArray){return function(v){gl.uniform1fv(location,v)}}if(type===gl.FLOAT){return function(v){gl.uniform1f(location,v)}}if(type===gl.FLOAT_VEC2){return function(v){gl.uniform2fv(location,v)}}if(type===gl.FLOAT_VEC3){return function(v){gl.uniform3fv(location,v)}}if(type===gl.FLOAT_VEC4){return function(v){gl.uniform4fv(location,v)}}if(type===gl.INT&&isArray){return function(v){gl.uniform1iv(location,v)}}if(type===gl.INT){return function(v){gl.uniform1i(location,v)}}if(type===gl.INT_VEC2){return function(v){gl.uniform2iv(location,v)}}if(type===gl.INT_VEC3){return function(v){gl.uniform3iv(location,v)}}if(type===gl.INT_VEC4){return function(v){gl.uniform4iv(location,v)}}if(type===gl.BOOL){return function(v){gl.uniform1iv(location,v)}}if(type===gl.BOOL_VEC2){return function(v){gl.uniform2iv(location,v)}}if(type===gl.BOOL_VEC3){return function(v){gl.uniform3iv(location,v)}}if(type===gl.BOOL_VEC4){return function(v){gl.uniform4iv(location,v)}}if(type===gl.FLOAT_MAT2){return function(v){gl.uniformMatrix2fv(location,false,v)}}if(type===gl.FLOAT_MAT3){return function(v){gl.uniformMatrix3fv(location,false,v)}}if(type===gl.FLOAT_MAT4){return function(v){gl.uniformMatrix4fv(location,false,v)}}if((type===gl.SAMPLER_2D||type===gl.SAMPLER_CUBE)&&isArray){var units=[];for(var ii=0;ii<info.size;++ii){units.push(textureUnit++)}return function(bindPoint,units){return function(textures){gl.uniform1iv(location,units);textures.forEach(function(texture,index){gl.activeTexture(gl.TEXTURE0+units[index]);gl.bindTexture(bindPoint,texture)})}}(getBindPointForSamplerType(gl,type),units)}if(type===gl.SAMPLER_2D||type===gl.SAMPLER_CUBE){return function(bindPoint,unit){return function(texture){gl.uniform1i(location,unit);gl.activeTexture(gl.TEXTURE0+unit);gl.bindTexture(bindPoint,texture)}}(getBindPointForSamplerType(gl,type),textureUnit++)}throw"unknown type: 0x"+type.toString(16)}var uniformSetters={};var numUniforms=gl.getProgramParameter(program,gl.ACTIVE_UNIFORMS);for(var ii=0;ii<numUniforms;++ii){var uniformInfo=gl.getActiveUniform(program,ii);if(!uniformInfo){break}var name=uniformInfo.name;if(name.substr(-3)==="[0]"){name=name.substr(0,name.length-3)}var setter=createUniformSetter(program,uniformInfo);uniformSetters[name]=setter}return uniformSetters}function setUniforms(setters,values){Object.keys(values).forEach(function(name){var setter=setters[name];if(setter){setter(values[name])}})}function createAttributeSetters(gl,program){var attribSetters={};function createAttribSetter(index){return function(b){gl.bindBuffer(gl.ARRAY_BUFFER,b.buffer);gl.enableVertexAttribArray(index);gl.vertexAttribPointer(index,b.numComponents||b.size,b.type||gl.FLOAT,b.normalize||false,b.stride||0,b.offset||0)}}var numAttribs=gl.getProgramParameter(program,gl.ACTIVE_ATTRIBUTES);for(var ii=0;ii<numAttribs;++ii){var attribInfo=gl.getActiveAttrib(program,ii);if(!attribInfo){break}var index=gl.getAttribLocation(program,attribInfo.name);attribSetters[attribInfo.name]=createAttribSetter(index)}return attribSetters}function setAttributes(setters,buffers){Object.keys(buffers).forEach(function(name){var setter=setters[name];if(setter){setter(buffers[name])}})}function createProgramInfo(gl,shaderSources,opt_attribs,opt_locations,opt_errorCallback){shaderSources=shaderSources.map(function(source){var script=document.getElementById(source);return script?script.text:source});var program=createProgramFromSources(gl,shaderSources,opt_attribs,opt_locations,opt_errorCallback);if(!program){return null}var uniformSetters=createUniformSetters(gl,program);var attribSetters=createAttributeSetters(gl,program);return{program:program,uniformSetters:uniformSetters,attribSetters:attribSetters}}function setBuffersAndAttributes(gl,setters,buffers){setAttributes(setters,buffers.attribs);if(buffers.indices){gl.bindBuffer(gl.ELEMENT_ARRAY_BUFFER,buffers.indices)}}var browserPrefixes=["","MOZ_","OP_","WEBKIT_"];function getExtensionWithKnownPrefixes(gl,name){for(var ii=0;ii<browserPrefixes.length;++ii){var prefixedName=browserPrefixes[ii]+name;var ext=gl.getExtension(prefixedName);if(ext){return ext}}}function resizeCanvasToDisplaySize(canvas,multiplier){multiplier=multiplier||1;var width=canvas.clientWidth*multiplier;var height=canvas.clientHeight*multiplier;if(canvas.width!==width||canvas.height!==height){canvas.width=width;canvas.height=height;return true}return false}function getIFrameForWindow(window){if(!isInIFrame(window)){return}var iframes=window.parent.document.getElementsByTagName("iframe");for(var ii=0;ii<iframes.length;++ii){var iframe=iframes[ii];if(iframe.contentDocument===window.document){return iframe}}}function isFrameVisible(window){try{var iframe=getIFrameForWindow(window);if(!iframe){return true}var bounds=iframe.getBoundingClientRect();var isVisible=bounds.top<window.parent.innerHeight&&bounds.bottom>=0&&bounds.left<window.parent.innerWidth&&bounds.right>=0;return isVisible&&isFrameVisible(window.parent)}catch(e){return true}}function isOnScreen(element){var isVisible=true;if(element){var bounds=element.getBoundingClientRect();isVisible=bounds.top<topWindow.innerHeight&&bounds.bottom>=0}return isVisible&&isFrameVisible(topWindow)}function augmentTypedArray(typedArray,numComponents){var cursor=0;typedArray.push=function(){for(var ii=0;ii<arguments.length;++ii){var value=arguments[ii];if(value instanceof Array||value.buffer&&value.buffer instanceof ArrayBuffer){for(var jj=0;jj<value.length;++jj){typedArray[cursor++]=value[jj]}}else{typedArray[cursor++]=value}}};typedArray.reset=function(opt_index){cursor=opt_index||0};typedArray.numComponents=numComponents;Object.defineProperty(typedArray,"numElements",{get:function(){return this.length/this.numComponents|0}});return typedArray}function createAugmentedTypedArray(numComponents,numElements,opt_type){var Type=opt_type||Float32Array;return augmentTypedArray(new Type(numComponents*numElements),numComponents)}function createBufferFromTypedArray(gl,array,type,drawType){type=type||gl.ARRAY_BUFFER;var buffer=gl.createBuffer();gl.bindBuffer(type,buffer);gl.bufferData(type,array,drawType||gl.STATIC_DRAW);return buffer}function allButIndices(name){return name!=="indices"}function createMapping(obj){var mapping={};Object.keys(obj).filter(allButIndices).forEach(function(key){mapping["a_"+key]=key});return mapping}function getGLTypeForTypedArray(gl,typedArray){if(typedArray instanceof Int8Array){return gl.BYTE}if(typedArray instanceof Uint8Array){return gl.UNSIGNED_BYTE}if(typedArray instanceof Int16Array){return gl.SHORT}if(typedArray instanceof Uint16Array){return gl.UNSIGNED_SHORT}if(typedArray instanceof Int32Array){return gl.INT}if(typedArray instanceof Uint32Array){return gl.UNSIGNED_INT}if(typedArray instanceof Float32Array){return gl.FLOAT}throw"unsupported typed array type"}function getNormalizationForTypedArray(typedArray){if(typedArray instanceof Int8Array){return true}if(typedArray instanceof Uint8Array){return true}return false}function isArrayBuffer(a){return a.buffer&&a.buffer instanceof ArrayBuffer}function guessNumComponentsFromName(name,length){var numComponents;if(name.indexOf("coord")>=0){numComponents=2}else if(name.indexOf("color")>=0){numComponents=4}else{numComponents=3}if(length%numComponents>0){throw"can not guess numComponents. You should specify it."}return numComponents}function makeTypedArray(array,name){if(isArrayBuffer(array)){return array}if(Array.isArray(array)){array={data:array}}if(!array.numComponents){array.numComponents=guessNumComponentsFromName(name,array.length)}var type=array.type;if(!type){if(name==="indices"){type=Uint16Array}}var typedArray=createAugmentedTypedArray(array.numComponents,array.data.length/array.numComponents|0,type);typedArray.push(array.data);return typedArray}function createAttribsFromArrays(gl,arrays,opt_mapping){var mapping=opt_mapping||createMapping(arrays);var attribs={};Object.keys(mapping).forEach(function(attribName){var bufferName=mapping[attribName];var array=makeTypedArray(arrays[bufferName],bufferName);attribs[attribName]={buffer:createBufferFromTypedArray(gl,array),numComponents:array.numComponents||guessNumComponentsFromName(bufferName),type:getGLTypeForTypedArray(gl,array),normalize:getNormalizationForTypedArray(array)}});return attribs}function getNumElementsFromNonIndexedArrays(arrays){var key=Object.keys(arrays)[0];var array=arrays[key];if(isArrayBuffer(array)){return array.numElements}else{return array.data.length/array.numComponents}}function createBufferInfoFromArrays(gl,arrays,opt_mapping){var bufferInfo={attribs:createAttribsFromArrays(gl,arrays,opt_mapping)};var indices=arrays.indices;if(indices){indices=makeTypedArray(indices,"indices");bufferInfo.indices=createBufferFromTypedArray(gl,indices,gl.ELEMENT_ARRAY_BUFFER);bufferInfo.numElements=indices.length}else{bufferInfo.numElements=getNumElementsFromNonIndexedArrays(arrays)}return bufferInfo}function createBuffersFromArrays(gl,arrays){var buffers={};Object.keys(arrays).forEach(function(key){var type=key==="indices"?gl.ELEMENT_ARRAY_BUFFER:gl.ARRAY_BUFFER;var array=makeTypedArray(arrays[key],name);buffers[key]=createBufferFromTypedArray(gl,array,type)});if(arrays.indices){buffers.numElements=arrays.indices.length}else if(arrays.position){buffers.numElements=arrays.position.length/3}return buffers}function drawBufferInfo(gl,type,bufferInfo,count,offset){var indices=bufferInfo.indices;var numElements=count===undefined?bufferInfo.numElements:count;offset=offset===undefined?offset:0;if(indices){gl.drawElements(type,numElements,gl.UNSIGNED_SHORT,offset)}else{gl.drawArrays(type,offset,numElements)}}function drawObjectList(gl,objectsToDraw){var lastUsedProgramInfo=null;var lastUsedBufferInfo=null;objectsToDraw.forEach(function(object){var programInfo=object.programInfo;var bufferInfo=object.bufferInfo;var bindBuffers=false;if(programInfo!==lastUsedProgramInfo){lastUsedProgramInfo=programInfo;gl.useProgram(programInfo.program);bindBuffers=true}if(bindBuffers||bufferInfo!==lastUsedBufferInfo){lastUsedBufferInfo=bufferInfo;setBuffersAndAttributes(gl,programInfo.attribSetters,bufferInfo)}setUniforms(programInfo.uniformSetters,object.uniforms);drawBufferInfo(gl,gl.TRIANGLES,bufferInfo)})}return{createAugmentedTypedArray:createAugmentedTypedArray,createAttribsFromArrays:createAttribsFromArrays,createBuffersFromArrays:createBuffersFromArrays,createBufferInfoFromArrays:createBufferInfoFromArrays,createAttributeSetters:createAttributeSetters,createProgram:createProgram,createProgramFromScripts:createProgramFromScripts,createProgramFromSources:createProgramFromSources,createProgramInfo:createProgramInfo,createUniformSetters:createUniformSetters,drawBufferInfo:drawBufferInfo,drawObjectList:drawObjectList,getWebGLContext:getWebGLContext,updateCSSIfInIFrame:updateCSSIfInIFrame,getExtensionWithKnownPrefixes:getExtensionWithKnownPrefixes,resizeCanvasToDisplaySize:resizeCanvasToDisplaySize,setAttributes:setAttributes,setBuffersAndAttributes:setBuffersAndAttributes,setUniforms:setUniforms,setupWebGL:setupWebGL}});function AgoraRender(){var gl=undefined;var program=undefined;var positionLocation=undefined;var texCoordLocation=undefined;var yTexture=undefined;var uTexture=undefined;var vTexture=undefined;var texCoordBuffer=undefined;var surfaceBuffer=undefined;var that={view:undefined,mirrorView:false,container:undefined,canvas:undefined,renderImageCount:0,initWidth:0,initHeight:0,initRotation:0,canvasUpdated:false,clientWidth:0,clientHeight:0};that.start=function(view,onFailure){initCanvas(view,that.mirrorView,view.clientWidth,view.clientHeight,that.initRotation,onFailure)};that.stop=function(){gl=undefined;program=undefined;positionLocation=undefined;texCoordLocation=undefined;deleteTexture(yTexture);deleteTexture(uTexture);deleteTexture(vTexture);yTexture=undefined;uTexture=undefined;vTexture=undefined;deleteBuffer(texCoordBuffer);deleteBuffer(surfaceBuffer);texCoordBuffer=undefined;surfaceBuffer=undefined;if(that.container){that.container.removeChild(that.canvas)}if(that.view){that.view.removeChild(that.container)}that.canvas=undefined;that.container=undefined;that.view=undefined;that.mirrorView=false};that.renderImage=function(image){if(!gl){return}if(image.width!=that.initWidth||image.height!=that.initHeight||image.rotation!=that.initRotation||image.mirror!=that.mirrorView){var view=that.view;that.stop();console.log("init canvas "+image.width+"*"+image.height+" rotation "+image.rotation);initCanvas(view,image.mirror,image.width,image.height,image.rotation,function(e){console.error("init canvas "+image.width+"*"+image.height+" rotation "+image.rotation+" failed. "+e)})}gl.bindBuffer(gl.ARRAY_BUFFER,texCoordBuffer);var xWidth=image.width+image.left+image.right;var xHeight=image.height+image.top+image.bottom;gl.bufferData(gl.ARRAY_BUFFER,new Float32Array([image.left/xWidth,image.bottom/xHeight,1-image.right/xWidth,image.bottom/xHeight,image.left/xWidth,1-image.top/xHeight,image.left/xWidth,1-image.top/xHeight,1-image.right/xWidth,image.bottom/xHeight,1-image.right/xWidth,1-image.top/xHeight]),gl.STATIC_DRAW);gl.enableVertexAttribArray(texCoordLocation);gl.vertexAttribPointer(texCoordLocation,2,gl.FLOAT,false,0,0);uploadYuv(xWidth,xHeight,image.yplane,image.uplane,image.vplane);updateCanvas(image.rotation,image.width,image.height);gl.drawArrays(gl.TRIANGLES,0,6);that.renderImageCount+=1};function uploadYuv(width,height,yplane,uplane,vplane){gl.activeTexture(gl.TEXTURE0);gl.bindTexture(gl.TEXTURE_2D,yTexture);gl.texImage2D(gl.TEXTURE_2D,0,gl.LUMINANCE,width,height,0,gl.LUMINANCE,gl.UNSIGNED_BYTE,yplane);var e=gl.getError();if(e!=gl.NO_ERROR){console.log("upload y plane ",width,height,yplane.byteLength," error",e)}gl.activeTexture(gl.TEXTURE1);gl.bindTexture(gl.TEXTURE_2D,uTexture);gl.texImage2D(gl.TEXTURE_2D,0,gl.LUMINANCE,width/2,height/2,0,gl.LUMINANCE,gl.UNSIGNED_BYTE,uplane);var e=gl.getError();if(e!=gl.NO_ERROR){console.log("upload u plane ",width,height,uplane.byteLength,"  error",e)}gl.activeTexture(gl.TEXTURE2);gl.bindTexture(gl.TEXTURE_2D,vTexture);gl.texImage2D(gl.TEXTURE_2D,0,gl.LUMINANCE,width/2,height/2,0,gl.LUMINANCE,gl.UNSIGNED_BYTE,vplane);var e=gl.getError();if(e!=gl.NO_ERROR){console.log("upload v plane ",width,height,vplane.byteLength,"  error",e)}}function deleteBuffer(buffer){if(buffer&&that.gl){that.gl.deleteBuffer(buffer)}}function deleteTexture(texture){if(texture&&that.gl){that.gl.deleteTexture(texture)}}function initCanvas(view,mirror,width,height,rotation,onFailure){that.clientWidth=view.clientWidth;that.clientHeight=view.clientHeight;that.view=view;that.mirrorView=mirror;that.canvasUpdated=false;that.container=document.createElement("div");that.container.style.width="100%";that.container.style.height="100%";that.container.style.display="flex";that.container.style.justifyContent="center";that.container.style.alignItems="center";that.view.appendChild(that.container);that.canvas=document.createElement("canvas");if(rotation==0||rotation==180){that.canvas.width=width;that.canvas.height=height}else{that.canvas.width=height;that.canvas.height=width}that.initWidth=width;that.initHeight=height;that.initRotation=rotation;if(that.mirrorView){that.canvas.style.transform="rotateY(180deg)"}that.container.appendChild(that.canvas);try{gl=that.canvas.getContext("webgl")||that.canvas.getContext("experimental-webgl")}catch(e){console.log(e)}if(!gl){gl=undefined;onFailure({error:"Browser not support! No WebGL detected."});return}gl.clearColor(0,0,0,1);gl.enable(gl.DEPTH_TEST);gl.depthFunc(gl.LEQUAL);gl.clear(gl.COLOR_BUFFER_BIT|gl.DEPTH_BUFFER_BIT);program=createProgramFromSources(gl,[AgoraRTC.vertexShaderSource,AgoraRTC.yuvShaderSource]);gl.useProgram(program);initTextures()}function initTextures(){positionLocation=gl.getAttribLocation(program,"a_position");texCoordLocation=gl.getAttribLocation(program,"a_texCoord");surfaceBuffer=gl.createBuffer();texCoordBuffer=gl.createBuffer();gl.activeTexture(gl.TEXTURE0);yTexture=gl.createTexture();gl.bindTexture(gl.TEXTURE_2D,yTexture);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_WRAP_S,gl.CLAMP_TO_EDGE);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_WRAP_T,gl.CLAMP_TO_EDGE);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_MIN_FILTER,gl.NEAREST);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_MAG_FILTER,gl.NEAREST);gl.activeTexture(gl.TEXTURE1);uTexture=gl.createTexture();gl.bindTexture(gl.TEXTURE_2D,uTexture);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_WRAP_S,gl.CLAMP_TO_EDGE);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_WRAP_T,gl.CLAMP_TO_EDGE);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_MIN_FILTER,gl.NEAREST);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_MAG_FILTER,gl.NEAREST);gl.activeTexture(gl.TEXTURE2);vTexture=gl.createTexture();gl.bindTexture(gl.TEXTURE_2D,vTexture);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_WRAP_S,gl.CLAMP_TO_EDGE);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_WRAP_T,gl.CLAMP_TO_EDGE);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_MIN_FILTER,gl.NEAREST);gl.texParameteri(gl.TEXTURE_2D,gl.TEXTURE_MAG_FILTER,gl.NEAREST);var y=gl.getUniformLocation(program,"Ytex");gl.uniform1i(y,0);var u=gl.getUniformLocation(program,"Utex");gl.uniform1i(u,1);var v=gl.getUniformLocation(program,"Vtex");gl.uniform1i(v,2)}function updateCanvas(rotation,width,height){if(that.canvasUpdated){return}that.canvas.style.width="100%";that.canvas.style.height="100%";try{if(rotation===0||rotation===180){if(that.clientWidth/that.clientHeight>width/height){that.canvas.style.width=that.clientHeight*width/height+"px"}else if(that.clientWidth/that.clientHeight<width/height){that.canvas.style.height=that.clientWidth*height/width+"px"}}else{if(that.clientHeight/that.clientWidth>width/height){that.canvas.style.height=that.clientWidth*width/height+"px"}else if(that.clientHeight/that.clientWidth<width/height){that.canvas.style.width=that.clientHeight*height/width+"px"}}}catch(e){console.log("updateCanvas 00001 gone "+that.canvas);console.log(that);console.error(e);return}gl.bindBuffer(gl.ARRAY_BUFFER,surfaceBuffer);gl.enableVertexAttribArray(positionLocation);gl.vertexAttribPointer(positionLocation,2,gl.FLOAT,false,0,0);var p1={x:0,y:0};var p2={x:width,y:0};var p3={x:width,y:height};var p4={x:0,y:height};var pp1=p1,pp2=p2,pp3=p3,pp4=p4;switch(rotation){case 0:break;case 90:pp1=p2;pp2=p3;pp3=p4;pp4=p1;break;case 180:pp1=p3;pp2=p4;pp3=p1;pp4=p2;break;case 270:pp1=p4;pp2=p1;pp3=p2;pp4=p3;break;default:}gl.bufferData(gl.ARRAY_BUFFER,new Float32Array([pp1.x,pp1.y,pp2.x,pp2.y,pp4.x,pp4.y,pp4.x,pp4.y,pp2.x,pp2.y,pp3.x,pp3.y]),gl.STATIC_DRAW);var resolutionLocation=gl.getUniformLocation(program,"u_resolution");gl.uniform2f(resolutionLocation,width,height);that.canvasUpdated=true}return that}var AgoraRTC=function(){"use strict";var AgoraRTC={};Object.defineProperties(AgoraRTC,{version:{get:function(){return"1.9.0"}},name:{get:function(){return"AgoraWebSDK"}}});return AgoraRTC}();AgoraRTC.secureAgentUrl="wss://localhost.agora.io:8921/";AgoraRTC.agentUrl="ws://127.0.0.1:8922/";AgoraRTC.getAgentUrl=function(){if(location.protocol==="https:"){return AgoraRTC.secureAgentUrl}return AgoraRTC.agentUrl};AgoraRTC.macAgentInstallUrl="http://download.agora.io/sdk/release/AgoraWebAgent-1.9.0.pkg";AgoraRTC.winAgentInstallUrl="http://download.agora.io/sdk/release/AgoraWebAgentSetup-1.9.0.exe";AgoraRTC.enAgentInstallGuideUrl="http://download.agora.io/install-guide-en.html";AgoraRTC.cnAgentInstallGuideUrl="http://download.agora.io/install-guide-cn.html";AgoraRTC.vertexShaderSource="attribute vec2 a_position;"+"attribute vec2 a_texCoord;"+"uniform vec2 u_resolution;"+"varying vec2 v_texCoord;"+"void main() {"+"vec2 zeroToOne = a_position / u_resolution;"+"   vec2 zeroToTwo = zeroToOne * 2.0;"+"   vec2 clipSpace = zeroToTwo - 1.0;"+"   gl_Position = vec4(clipSpace * vec2(1, -1), 0, 1);"+"v_texCoord = a_texCoord;"+"}";AgoraRTC.yuvShaderSource="precision mediump float;"+"uniform sampler2D Ytex;"+"uniform sampler2D Utex,Vtex;"+"varying vec2 v_texCoord;"+"void main(void) {"+"  float nx,ny,r,g,b,y,u,v;"+"  mediump vec4 txl,ux,vx;"+"  nx=v_texCoord[0];"+"  ny=v_texCoord[1];"+"  y=texture2D(Ytex,vec2(nx,ny)).r;"+"  u=texture2D(Utex,vec2(nx,ny)).r;"+"  v=texture2D(Vtex,vec2(nx,ny)).r;"+"  y=1.1643*(y-0.0625);"+"  u=u-0.5;"+"  v=v-0.5;"+"  r=y+1.5958*v;"+"  g=y-0.39173*u-0.81290*v;"+"  b=y+2.017*u;"+"  gl_FragColor=vec4(r,g,b,1.0);"+"}";AgoraRTC.videoStreamSource='function onSuccess(){postMessage({type:"init",result:!0})}function onFailure(a){postMessage(a)}function sendMessage(a){"videoStat"!=a.command&&console.log(JSON.stringify(a)),that.stream&&that.stream.readyState==WebSocket.OPEN&&that.stream.send(JSON.stringify(a))}var that={};that.stream=void 0,that.init=function(a){return that.stream&&that.stream.readyState===WebSocket.OPEN?(console.warn("stream "+that.getId()+" has been initialized already"),void onSuccess()):(that.local=a.local,that.profile=a.videoProfile,that.stream=new WebSocket(a.url),that.stream.onopen=function(a){console.log((that.local?"local":"remote")+" Stream "),console.log(a),that.profile&&sendMessage({command:"setVideoProfile",profile:that.profile.profile,swapWidthAndHeight:that.profile.swapWidthAndHeight}),that.stream.binaryType="arraybuffer",onSuccess()},that.stream.onclose=function(a){console.log(that.local?"local":"remote"," Stream ",a),that.stream=void 0,onFailure({type:a.type,code:a.code,reason:a.reason})},that.stream.onerror=function(a){console.log(that.local?"local":"remote"," Stream ",a),onFailure({type:a.type,code:a.code,reason:a.reason})},void(that.stream.onmessage=function(a){"string"==typeof a.data?console.log(that.local?"local":"remote"," message from agent ",a.data):a.data instanceof ArrayBuffer?postMessage({type:"message",data:a.data}):a.data instanceof Blob&&console.warn("Blob image data is not supported")}))},that.close=function(){that.stream&&(that.stream.onmessage=void 0,that.stream.close()),that.stream=void 0},self.addEventListener("message",function(a){var b=a.data;switch(b.type){case"init":that.init(b);break;case"send":sendMessage(b.message);break;case"close":that.close()}},!1);';var AgoraCall=function(callback,parameters){if(callback){callback(parameters)}};var L={VideoProfiles:{Profile120P:"120P",Profile120P_3:"120P_3",Profile180P:"180P",Profile180P_3:"180P_3",Profile180P_4:"180P_4",Profile240P:"240P",Profile240P_3:"240P_3",Profile240P_4:"240P_4",Profile360P:"360P",Profile360P_3:"360P_3",Profile360P_4:"360P_4",Profile360P_6:"360P_6",Profile360P_7:"360P_7",Profile360P_8:"360P_8",Profile360P_8:"360P_9",Profile360P_8:"360P_10",Profile360P_8:"360P_11",Profile480P:"480P",Profile480P_3:"480P_3",Profile480P_4:"480P_4",Profile480P_6:"480P_6",Profile480P_8:"480P_8",Profile480P_9:"480P_9",Profile480P_10:"480P_10",Profile720P:"720P",Profile720P_3:"720P_3",Profile720P_5:"720P_5",Profile720P_6:"720P_6"},ErrorCode:{NO_ERROR:0,FAILED:1,INVALID_ARGUMENT:2,NOT_READY:3,NOT_SUPPORTED:4,REFUSED:5,BUFFER_TOO_SMALL:6,NOT_INITIALIZED:7,INVALID_VIEW:8,NO_PERMISSION:9,TIMEDOUT:10,CANCELED:11,TOO_OFTEN:12,BIND_SOCKET:13,NET_DOWN:14,NET_NOBUFS:15,INIT_VIDEO:16,JOIN_CHANNEL_REJECTED:17,LEAVE_CHANNEL_REJECTED:18,ALREADY_IN_USE:19,INVALID_APP_ID:101,INVALID_CHANNEL_NAME:102,CHANNEL_KEY_EXPIRED:109,INVALID_CHANNEL_KEY:110,CONNECTION_INTERRUPTED:111,CONNECTION_LOST:112,LOAD_MEDIA_ENGINE:1001,START_CALL:1002,START_CAMERA:1003,START_VIDEO_RENDER:1004,ADM_GENERAL_ERROR:1005,ADM_JAVA_RESOURCE:1006,ADM_SAMPLE_RATE:1007,ADM_INIT_PLAYOUT:1008,ADM_START_PLAYOUT:1009,ADM_STOP_PLAYOUT:1010,ADM_INIT_RECORDING:1011,ADM_START_RECORDING:1012,ADM_STOP_RECORDING:1013,ADM_RUNTIME_PLAYOUT_ERROR:1015,ADM_RUNTIME_RECORDING_ERROR:1017,ADM_RECORD_AUDIO_FAILED:1018,ADM_INIT_LOOPBACK:1022,ADM_START_LOOPBACK:1023,VDM_CAMERA_NOT_AUTHORIZED:1501}};AgoraRTC.EventDispatcher=function(spec){"use strict";var that={};spec.dispatcher={};spec.dispatcher.eventListeners={};that.addEventListener=function(eventType,listener){spec.dispatcher.eventListeners[eventType]=listener;if(!spec.dispatcher.eventListeners[eventType]){delete spec.dispatcher.eventListeners[eventType]}};that.on=that.addEventListener;that.dispatchEvent=function(event){if(spec.dispatcher.eventListeners.hasOwnProperty(event.type)){spec.dispatcher.eventListeners[event.type](event)}};return that};AgoraRTC.BasicEvent=function(spec){"use strict";var that={};that.type=spec.type;return that};AgoraRTC.StreamEvent=function(spec){"use strict";var that=AgoraRTC.BasicEvent(spec);that.stream=spec.stream;that.msg=spec.msg;return that};AgoraRTC.ClientEvent=function(spec){"use strict";var that=AgoraRTC.BasicEvent(spec);that.uid=spec.uid;that.attr=spec.attr;that.streams=spec.streams;return that};AgoraRTC.MediaEvent=function(spec){"use strict";var that=AgoraRTC.BasicEvent(spec);that.msg=spec.msg;return that};AgoraRTC.Signal=function(spec){var that=AgoraRTC.EventDispatcher(spec);that.connection=new WebSocket(spec.url);that.sendMessage=function(obj,onError){if(that.connection.readyState==WebSocket.OPEN){that.connection.send(JSON.stringify(obj))}else{console.log("connection to agent lost.");onError({error:"not connected"})}};that.close=function(){that.onEvent=onEvent;that.connection.onopen=undefined;that.connection.onclose=undefined;that.connection.onerror=undefined;that.connection.onmessage=undefined;that.connection.close()};that.connection.onopen=function(e){console.log(e);that.dispatchEvent(AgoraRTC.MediaEvent({type:"onopen",event:e}))};that.connection.onclose=function(e){console.log(e);AgoraCall(spec.onFailure,e)};that.connection.onerror=function(e){console.log(e);AgoraCall(spec.onFailure,e)};that.onEvent=onEvent;that.connection.onmessage=function(e){console.log(e);var m=JSON.parse(e.data);if(m.hasOwnProperty("command")){that.dispatchEvent(AgoraRTC.MediaEvent({type:m.command,msg:m}))}else if(m.hasOwnProperty("event")){that.onEvent(m)}};function onEvent(e){console.log(e)}return that};AgoraRTC.Stream=function(spec){var that=AgoraRTC.EventDispatcher(spec);that.stream=undefined;that.render=undefined;that.interval=undefined;that.lastRenderCount=0;that.profile=undefined;that.latency=0;var header=undefined;var yplane=undefined;var uplane=undefined;var vplane=undefined;that.init=function(onSuccess,onFailure){if(that.stream){console.warn("stream "+that.getId()+" has been initialized already");onSuccess();return}var videoStreamFileBlob=new Blob([AgoraRTC.videoStreamSource],{type:"application/javascript"});that.stream=new Worker(URL.createObjectURL(videoStreamFileBlob));that.oninit=function(e){console.log((spec.local?"local":"remote")+" Stream ");console.log(e);if(e.result===true){onSuccess()}else{onFailure()}};that.onclose=function(e){window.clearInterval(that.interval);console.log(spec.local?"local":"remote"," Stream ",e);that.stream=undefined};that.onerror=function(e){window.clearInterval(that.interval);console.log(spec.local?"local":"remote"," Stream ",e);onFailure(e)};that.onmessage=function(e){if(typeof e==="string"){console.log(spec.local?"local":"remote"," message from agent ",e)}else if(e instanceof ArrayBuffer){if(that.render){that.drawImage(e)}}else if(e instanceof Blob){console.warn("Blob image data is not supported")}};that.stream.addEventListener("message",onWorkerMessage,false);
that.stream.postMessage({type:"init",local:spec.local,url:AgoraRTC.getAgentUrl(),videoProfile:that.profile})};that.close=function(){window.clearInterval(that.interval);if(that.stream){that.stream.removeEventListener("message",onWorkerMessage);that.stream.postMessage({type:"close"})}that.stream=undefined;that.stop()};that.play=function(div,onFailure){var view=document.getElementById(div);that.stop();that.render=AgoraRender();that.render.start(view,onFailure);if(spec.local){that.stream.postMessage({type:"send",message:{command:"preview",uid:spec.streamID,audio:spec.audio===false?false:true,video:spec.video===false?false:true,screen:spec.screen===true?true:false,viewWidth:view.clientWidth,viewHeight:view.clientHeight}})}else{that.stream.postMessage({type:"send",message:{command:"subscribe",uid:spec.streamID,viewWidth:view.clientWidth,viewHeight:view.clientHeight}})}if(that.interval){window.clearInterval(that.interval)}that.interval=window.setInterval(function(){if(that.render){var fps=that.render.renderImageCount-that.lastRenderCount;that.stream.postMessage({type:"send",message:{command:"videoStat",uid:spec.streamID,fps:fps,frameCount:that.lastRenderCount,latency:that.latency}});that.lastRenderCount=that.render.renderImageCount;that.latency=0}},1e3)};that.stop=function(){if(that.render){that.render.stop()}that.render=undefined;that.lastRenderCount=0};that.bindClient=function(client){that.client=client};that.enableAudio=function(callback){return that.client.enableAudio(that,callback)};that.disableAudio=function(callback){return that.client.disableAudio(that,callback)};that.enableVideo=function(callback){return that.client.enableVideo(that,callback)};that.disableVideo=function(callback){return that.client.disableVideo(that,callback)};that.getId=function(){return spec.streamID};that.drawImage=function(data){if(!header){header=data;if(header.byteLength!=20){console.error("invalid image header "+data.byteLength);resetImageData()}return}if(!yplane){yplane=data;if(yplane.byteLength===20){console.error("invalid image header "+data.byteLength+" "+yplane.byteLength);resetImageData()}return}if(!uplane){uplane=data;if(uplane.byteLength===20){console.error("invalid image header "+data.byteLength+" "+yplane.byteLength+" "+uplane.byteLength);resetImageData()}return}if(!vplane){vplane=data;if(yplane.byteLength!=uplane.byteLength*4||uplane.byteLength!=vplane.byteLength){console.error("invalid image header "+data.byteLength+" "+yplane.byteLength+" "+uplane.byteLength+" "+vplane.byteLength);resetImageData();return}var headerLength=20;var dv=new DataView(header);var format=dv.getUint8(0);var mirror=dv.getUint8(1);var width=dv.getUint16(2);var height=dv.getUint16(4);var left=dv.getUint16(6);var top=dv.getUint16(8);var right=dv.getUint16(10);var bottom=dv.getUint16(12);var rotation=dv.getUint16(14);var ts=dv.getUint32(16);var xWidth=width+left+right;var xHeight=height+top+bottom;var yLength=xWidth*xHeight;var yBegin=headerLength;var yEnd=yBegin+yLength;var uLength=yLength/4;var uBegin=yEnd;var uEnd=uBegin+uLength;var vLength=yLength/4;var vBegin=uEnd;var vEnd=vBegin+vLength;that.render.renderImage({mirror:mirror,width:width,height:height,left:left,top:top,right:right,bottom:bottom,rotation:rotation,yplane:new Uint8Array(yplane),uplane:new Uint8Array(uplane),vplane:new Uint8Array(vplane)});var now32=(Date.now()&4294967295)>>>0;var latency=now32-ts;if(latency>that.latency){that.latency=latency}resetImageData()}};that.setVideoProfile=function(profile){that.profile={};if(typeof profile!=="string"&&typeof profile!=="object"){return false}if(profile.hasOwnProperty("profile")){that.profile.profile=profile.profile}if(profile.hasOwnProperty("swapWidthAndHeight")){that.profile.swapWidthAndHeight=profile.swapWidthAndHeight!==false}else{that.profile.swapWidthAndHeight=false}if(typeof profile=="string"){that.profile.profile=profile}if(that.stream){that.stream.postMessage({type:"send",message:{command:"setVideoProfile",profile:that.profile.profile,swapWidthAndHeight:that.profile.swapWidthAndHeight}})}return true};function resetImageData(){header=undefined;yplane=undefined;uplane=undefined;vplane=undefined}function getReso(w,h){return{width:w,height:h}}var supportedVideoList={true:true,unspecified:true,"120p":getReso(160,120),"240p":getReso(320,240),"360p":getReso(640,360),"480p":getReso(640,480),"720p":getReso(1280,720),"1080p":getReso(1920,1080),"4k":getReso(3840,2160)};function onWorkerMessage(e){var data=e.data;switch(data.type){case"init":that.oninit(data);break;case"close":that.onclose(data);break;case"error":that.onerror(data);break;case"message":that.onmessage(data.data);break}}return that};AgoraRTC.createStream=function(spec){return AgoraRTC.Stream(spec)};AgoraRTC.Client=function(spec){"use strict";var that=AgoraRTC.EventDispatcher(spec);that.signal=undefined;that.localStream=undefined;that.remoteStreams={};that.signalOpen=false;that.agentUrl=AgoraRTC.getAgentUrl();function onSignalError(e){if(that.signalOpen){console.log("signal connection lost.");var message={reason:"LOST_CONNECTION_TO_AGENT",type:"error",eventType:e.type,code:e.code};AgoraCall(that.onFailure,message);that.dispatchEvent(message);return}if(e.type==="error"){return}var message={reason:"CLOSE_BEFORE_OPEN",type:"error",eventType:e.type,code:e.code,agentInstallUrl:getInstallUrl(),agentInstallGuideUrl:getInstallGuideUrl()};AgoraCall(that.onFailure,message);that.dispatchEvent(message)}that.init=function(appId,onSuccess,onFailure){spec.appId=appId;that.onFailure=onFailure;if(!that.signal){try{that.signal=AgoraRTC.Signal({url:that.agentUrl,onFailure:onSignalError})}catch(e){console.log("create signal connection failed"+e);AgoraCall(onFailure,e);return}that.signal.on("onopen",function(event){that.signalOpen=true;that.signal.onEvent=function(e){if(e.event==="error"){var name=getErrorCodeName(e.code);if(name){e.reason=name}}else if(e.event==="onUserMuteAudio"){e.event="peer-mute-audio"}else if(e.event==="onUserMuteVideo"){e.event="peer-mute-video"}that.dispatchEvent(AgoraRTC.MediaEvent({type:e.event,msg:e}))};that.getVersion(function(ev){if(ev.version=="1.6.0"||ev.version=="1.6.1"||ev.version=="1.7.0"||ev.version=="1.7.1"||ev.version=="1.7.2"||ev.version=="1.7.3"||ev.version=="1.7.4"||ev.version=="1.8.0"||ev.version=="1.8.1"||ev.version=="1.9.0"){initialize(onSuccess,onFailure);return}var message={reason:"INCOMPATIBLE_WEBAGENT",type:"error",version:AgoraRTC.version,agentVersion:ev.version,agentInstallUrl:getInstallUrl(),agentInstallGuideUrl:getInstallGuideUrl()};that.dispatchEvent(message)},function(ev){var message={reason:"INCOMPATIBLE_WEBAGENT",type:"error",version:AgoraRTC.version,agentVersion:ev.version,agentInstallUrl:getInstallUrl(),agentInstallGuideUrl:getInstallGuideUrl()};that.dispatchEvent(message)})})}if(that.signalOpen){initialize(onSuccess,onFailure)}function initialize(onSuccess,onFailure){sendSignalCommand({command:"initialize",appId:spec.appId},function(ev){var m=ev.msg;if(m.code!=true){that.signal.close();that.signal=undefined;onError(m.code,"initialize");return}AgoraCall(onSuccess,m)},onFailure)}function onError(code,type){var name=getErrorCodeName(code);if(!name){name="UNKNOWN_ERROR"}notifyError({reason:name,type:"error",eventType:type,code:code});return}function notifyError(e){AgoraCall(onFailure,e);that.dispatchEvent(e)}function getErrorCodeName(code){for(var k in L.ErrorCode){if(L.ErrorCode.hasOwnProperty(k)){if(L.ErrorCode[k]===code){return String(k)}}}return null}that.signal.on("onError",function(ev){var m=ev.msg;onError(m.code,"error")});console.log(that.signal)};that.renewChannelKey=function(channelKey,onSuccess,onFailure){sendSignalCommand({command:"renewChannelKey",channelKey:channelKey},function(ev){var m=ev.msg;if(m.code===true){AgoraCall(onSuccess,m);return}AgoraCall(onFailure,m)},onFailure)};that.join=function(channelKey,channelName,uid,onSuccess,onFailure){if(channelName.length>64){onFailure(L.ErrorCode.INVALID_CHANNEL_NAME);return}sendSignalCommand({command:"joinChannel",channelKey:channelKey,channelName:channelName,uid:uid},function(ev){that.signal.on("onAddVideoStream",onAddVideoStream);that.signal.on("onPeerLeave",onPeerLeave);var m=ev.msg;if(m.code==true){if(!m.uid){m.uid=0}AgoraCall(onSuccess,m.uid);return}if(m.code==L.ErrorCode.JOIN_CHANNEL_REJECTED){console.error("Command joinChannel has been rejected by agent. Is this user joined a channel already?");AgoraCall(onFailure,m.code);return}AgoraCall(onFailure,m.code)},onFailure)};that.leave=function(onSuccess,onFailure){closeLocalRemoteStreams();sendSignalCommand({command:"leaveChannel"},function(ev){var m=ev.msg;if(m.code===true){AgoraCall(onSuccess,m);return}if(m.code==L.ErrorCode.LEAVE_CHANNEL_REJECTED){console.error("Command leaveChannel has been rejected by agent. Is this user not in a channel?");AgoraCall(onSuccess,m);return}AgoraCall(onFailure,{code:m.code})},function(e){console.log("leave channel failed",e)})};that.publish=function(stream,onSuccess,onFailure){that.localStream=stream;sendSignalCommand({command:"unmuteLocal"},function(ev){var m=ev.msg;if(m.code!=true){AgoraCall(onFailure,{code:m.code});return}AgoraCall(onSuccess,m)},onFailure);stream.bindClient(that)};that.unpublish=function(stream,onSuccess,onFailure){sendSignalCommand({command:"muteLocal"},function(ev){var m=ev.msg;if(m.code!=true){AgoraCall(onFailure,{code:m.code});return}AgoraCall(onSuccess,m)},function(e){AgoraCall(onFailure,{error:e})})};that.subscribe=function(stream,onFailure){stream.init(function(){that.dispatchEvent(AgoraRTC.StreamEvent({type:"stream-subscribed",stream:stream}))},function(error){AgoraCall(onFailure,error)})};that.unsubscribe=function(streamID,onFailure){console.log("remote streams",that.remoteStreams);if(that.remoteStreams[streamID]==undefined){AgoraCall(onFailure,{error:"no such stream"});return}that.remoteStreams[streamID].close();delete that.remoteStreams[streamID]};that.enableAudio=function(stream,callback){sendSignalCommand({command:"enableAudio",streamID:stream.getId()},function(ev){var m=ev.msg;if(callback){AgoraCall(callback,m.code===true)}},callback);return true};that.disableAudio=function(stream,callback){sendSignalCommand({command:"disableAudio",streamID:stream.getId()},function(ev){var m=ev.msg;if(callback){AgoraCall(callback,m.code===true)}},callback);return true};that.enableVideo=function(stream,callback){sendSignalCommand({command:"enableVideo",streamID:stream.getId()},function(ev){var m=ev.msg;if(callback){AgoraCall(callback,m.code===true)}},callback);return true};that.disableVideo=function(stream,callback){sendSignalCommand({command:"disableVideo",streamID:stream.getId()},function(ev){var m=ev.msg;if(callback){AgoraCall(callback,m.code===true)}},callback);return true};that.getDevices=function(callback){sendSignalCommand({command:"getDevices"},function(ev){var devices=ev.msg.devices;AgoraCall(callback,devices)},callback)};that.selectDevice=function(device,onFailure){sendSignalCommand({command:"selectDevice",device:device},function(ev){console.log(ev)},onFailure)};that.startRecording=function(recordingKey,onSuccess,onFailure){sendSignalCommand({command:"startRecording",recordingKey:recordingKey},function(ev){var start=ev.msg;if(start.code===true){AgoraCall(onSuccess,start)}else{AgoraCall(onFailure,start)}},onFailure)};that.stopRecording=function(recordingKey,onSuccess,onFailure){sendSignalCommand({command:"stopRecording",recordingKey:recordingKey},function(ev){var stop=ev.msg;if(stop.code===true){AgoraCall(onSuccess,stop)}else{AgoraCall(onFailure,stop)}},onFailure)};that.queryRecordingStatus=function(onStatus){sendSignalCommand({command:"queryRecordingStatus"},function(ev){onStatus(ev.msg)},onStatus)};that.close=function(){closeLocalRemoteStreams();if(that.signal){that.signal.close();that.signal=undefined}};that.getVersion=function(onSuccess,onFailure){sendSignalCommand({command:"getVersion"},function(ev){var m=ev.msg;if(m.code===true){AgoraCall(onSuccess,m)}else{AgoraCall(onFailure,m)}},onFailure)};that.setParameters=function(parameters){sendSignalCommand({command:"setParameters",parameters:JSON.stringify(parameters)})};that.setEncryptionMode=function(encryptionMode){sendSignalCommand({command:"setEncryptionMode",mode:encryptionMode})};that.setEncryptionSecret=function(secret){sendSignalCommand({command:"setEncryptionSecret",secret:secret})};that.getWindows=function(callback){sendSignalCommand({command:"getWindows"},function(ev){var windows=ev.msg.windows;AgoraCall(callback,windows)},callback)};that.startScreenSharing=function(window,onSuccess,onFailure){sendSignalCommand({command:"startScreenSharing",window:window},onSuccess,onFailure)};that.setScreenSharingWindow=function(window,onSuccess,onFailure){sendSignalCommand({command:"setScreenSharingWindow",window:window},onSuccess,onFailure)};that.stopScreenSharing=function(onSuccess,onFailure){sendSignalCommand({command:"stopScreenSharing"},onSuccess,onFailure)};that.setChannelProfile=function(profile,onSuccess,onFailure){sendSignalCommand({command:"setChannelProfile",profile:profile},onSuccess,onFailure)};that.setClientRole=function(role,permissionKey,onSuccess,onFailure){if(!permissionKey){permissionKey=""}sendSignalCommand({command:"setClientRole",role:role,key:permissionKey},onSuccess,onFailure)};that.enableDualStreamMode=function(enabled,onSuccess,onFailure){sendSignalCommand({command:"enableDualStreamMode",enabled:enabled},onSuccess,onFailure)};function sendSignalCommand(message,onSuccess,onFailure){that.signal.on(message.command,function(e){AgoraCall(onSuccess,e);console.log(e)});that.signal.sendMessage(message,function(e){console.log(e);e.reason="CONNECTION_TO_AGENT_ERROR";AgoraCall(onFailure,e)})}function onPeerLeave(ev){var m=ev.msg;if(that.remoteStreams.hasOwnProperty(m.uid)){that.remoteStreams[m.uid].close();delete that.remoteStreams[m.uid];console.log("remote streams after peer leave ",that.remoteStreams)}that.dispatchEvent(AgoraRTC.ClientEvent({type:"peer-leave",uid:m.uid}))}function onAddVideoStream(ev){var m=ev.msg;if(!that.remoteStreams.hasOwnProperty(m.uid)){var stream=AgoraRTC.Stream({streamID:m.uid,local:false,audio:m.audio,video:m.video,screen:m.screen});stream.bindClient(that);that.remoteStreams[m.uid]=stream}that.dispatchEvent(AgoraRTC.StreamEvent({type:"stream-added",stream:that.remoteStreams[m.uid]}));console.log("remote streams",that.remoteStreams)}function getInstallUrl(){if(navigator.appVersion.indexOf("Mac")!=-1){return AgoraRTC.macAgentInstallUrl}return AgoraRTC.winAgentInstallUrl}function getInstallGuideUrl(){var userLang=navigator.language||navigator.userLanguage;if(userLang.indexOf("zh")!=-1){return AgoraRTC.cnAgentInstallGuideUrl}return AgoraRTC.enAgentInstallGuideUrl}function closeLocalRemoteStreams(){if(that.localStream){that.localStream.close()}that.localStream=undefined;for(var streamId in that.remoteStreams){if(that.remoteStreams.hasOwnProperty(streamId)){that.remoteStreams[streamId].close();delete that.remoteStreams[streamId]}}}return that};AgoraRTC.createRtcClient=function(){return AgoraRTC.Client({})};AgoraRTC.createLiveClient=function(){return AgoraRTC.Client({})};
//# sourceMappingURL=dist/AgoraRtcAgentSDK.min.js.map
/*!
 * Bootstrap-select v1.10.0 (http://silviomoreto.github.io/bootstrap-select)
 *
 * Copyright 2013-2016 bootstrap-select
 * Licensed under MIT (https://github.com/silviomoreto/bootstrap-select/blob/master/LICENSE)
 */

(function (root, factory) {
  if (typeof define === 'function' && define.amd) {
    // AMD. Register as an anonymous module unless amdModuleId is set
    define(["jquery"], function (a0) {
      return (factory(a0));
    });
  } else if (typeof exports === 'object') {
    // Node. Does not work with strict CommonJS, but
    // only CommonJS-like environments that support module.exports,
    // like Node.
    module.exports = factory(require("jquery"));
  } else {
    factory(jQuery);
  }
}(this, function (jQuery) {

(function ($) {
  'use strict';

  //<editor-fold desc="Shims">
  if (!String.prototype.includes) {
    (function () {
      'use strict'; // needed to support `apply`/`call` with `undefined`/`null`
      var toString = {}.toString;
      var defineProperty = (function () {
        // IE 8 only supports `Object.defineProperty` on DOM elements
        try {
          var object = {};
          var $defineProperty = Object.defineProperty;
          var result = $defineProperty(object, object, object) && $defineProperty;
        } catch (error) {
        }
        return result;
      }());
      var indexOf = ''.indexOf;
      var includes = function (search) {
        if (this == null) {
          throw new TypeError();
        }
        var string = String(this);
        if (search && toString.call(search) == '[object RegExp]') {
          throw new TypeError();
        }
        var stringLength = string.length;
        var searchString = String(search);
        var searchLength = searchString.length;
        var position = arguments.length > 1 ? arguments[1] : undefined;
        // `ToInteger`
        var pos = position ? Number(position) : 0;
        if (pos != pos) { // better `isNaN`
          pos = 0;
        }
        var start = Math.min(Math.max(pos, 0), stringLength);
        // Avoid the `indexOf` call if no match is possible
        if (searchLength + start > stringLength) {
          return false;
        }
        return indexOf.call(string, searchString, pos) != -1;
      };
      if (defineProperty) {
        defineProperty(String.prototype, 'includes', {
          'value': includes,
          'configurable': true,
          'writable': true
        });
      } else {
        String.prototype.includes = includes;
      }
    }());
  }

  if (!String.prototype.startsWith) {
    (function () {
      'use strict'; // needed to support `apply`/`call` with `undefined`/`null`
      var defineProperty = (function () {
        // IE 8 only supports `Object.defineProperty` on DOM elements
        try {
          var object = {};
          var $defineProperty = Object.defineProperty;
          var result = $defineProperty(object, object, object) && $defineProperty;
        } catch (error) {
        }
        return result;
      }());
      var toString = {}.toString;
      var startsWith = function (search) {
        if (this == null) {
          throw new TypeError();
        }
        var string = String(this);
        if (search && toString.call(search) == '[object RegExp]') {
          throw new TypeError();
        }
        var stringLength = string.length;
        var searchString = String(search);
        var searchLength = searchString.length;
        var position = arguments.length > 1 ? arguments[1] : undefined;
        // `ToInteger`
        var pos = position ? Number(position) : 0;
        if (pos != pos) { // better `isNaN`
          pos = 0;
        }
        var start = Math.min(Math.max(pos, 0), stringLength);
        // Avoid the `indexOf` call if no match is possible
        if (searchLength + start > stringLength) {
          return false;
        }
        var index = -1;
        while (++index < searchLength) {
          if (string.charCodeAt(start + index) != searchString.charCodeAt(index)) {
            return false;
          }
        }
        return true;
      };
      if (defineProperty) {
        defineProperty(String.prototype, 'startsWith', {
          'value': startsWith,
          'configurable': true,
          'writable': true
        });
      } else {
        String.prototype.startsWith = startsWith;
      }
    }());
  }

  if (!Object.keys) {
    Object.keys = function (
      o, // object
      k, // key
      r  // result array
      ){
      // initialize object and result
      r=[];
      // iterate over object keys
      for (k in o)
          // fill result array with non-prototypical keys
        r.hasOwnProperty.call(o, k) && r.push(k);
      // return result
      return r;
    };
  }

  $.fn.triggerNative = function (eventName) {
    var el = this[0],
        event;

    if (el.dispatchEvent) {
      if (typeof Event === 'function') {
        // For modern browsers
        event = new Event(eventName, {
          bubbles: true
        });
      } else {
        // For IE since it doesn't support Event constructor
        event = document.createEvent('Event');
        event.initEvent(eventName, true, false);
      }

      el.dispatchEvent(event);
    } else {
      if (el.fireEvent) {
        event = document.createEventObject();
        event.eventType = eventName;
        el.fireEvent('on' + eventName, event);
      }

      this.trigger(eventName);
    }
  };
  //</editor-fold>

  // Case insensitive contains search
  $.expr[':'].icontains = function (obj, index, meta) {
    var $obj = $(obj);
    var haystack = ($obj.data('tokens') || $obj.text()).toUpperCase();
    return haystack.includes(meta[3].toUpperCase());
  };

  // Case insensitive begins search
  $.expr[':'].ibegins = function (obj, index, meta) {
    var $obj = $(obj);
    var haystack = ($obj.data('tokens') || $obj.text()).toUpperCase();
    return haystack.startsWith(meta[3].toUpperCase());
  };

  // Case and accent insensitive contains search
  $.expr[':'].aicontains = function (obj, index, meta) {
    var $obj = $(obj);
    var haystack = ($obj.data('tokens') || $obj.data('normalizedText') || $obj.text()).toUpperCase();
    return haystack.includes(meta[3].toUpperCase());
  };

  // Case and accent insensitive begins search
  $.expr[':'].aibegins = function (obj, index, meta) {
    var $obj = $(obj);
    var haystack = ($obj.data('tokens') || $obj.data('normalizedText') || $obj.text()).toUpperCase();
    return haystack.startsWith(meta[3].toUpperCase());
  };

  /**
   * Remove all diatrics from the given text.
   * @access private
   * @param {String} text
   * @returns {String}
   */
  function normalizeToBase(text) {
    var rExps = [
      {re: /[\xC0-\xC6]/g, ch: "A"},
      {re: /[\xE0-\xE6]/g, ch: "a"},
      {re: /[\xC8-\xCB]/g, ch: "E"},
      {re: /[\xE8-\xEB]/g, ch: "e"},
      {re: /[\xCC-\xCF]/g, ch: "I"},
      {re: /[\xEC-\xEF]/g, ch: "i"},
      {re: /[\xD2-\xD6]/g, ch: "O"},
      {re: /[\xF2-\xF6]/g, ch: "o"},
      {re: /[\xD9-\xDC]/g, ch: "U"},
      {re: /[\xF9-\xFC]/g, ch: "u"},
      {re: /[\xC7-\xE7]/g, ch: "c"},
      {re: /[\xD1]/g, ch: "N"},
      {re: /[\xF1]/g, ch: "n"}
    ];
    $.each(rExps, function () {
      text = text.replace(this.re, this.ch);
    });
    return text;
  }


  function htmlEscape(html) {
    var escapeMap = {
      '&': '&amp;',
      '<': '&lt;',
      '>': '&gt;',
      '"': '&quot;',
      "'": '&#x27;',
      '`': '&#x60;'
    };
    var source = '(?:' + Object.keys(escapeMap).join('|') + ')',
        testRegexp = new RegExp(source),
        replaceRegexp = new RegExp(source, 'g'),
        string = html == null ? '' : '' + html;
    return testRegexp.test(string) ? string.replace(replaceRegexp, function (match) {
      return escapeMap[match];
    }) : string;
  }

  var Selectpicker = function (element, options, e) {
    if (e) {
      e.stopPropagation();
      e.preventDefault();
    }

    this.$element = $(element);
    this.$newElement = null;
    this.$button = null;
    this.$menu = null;
    this.$lis = null;
    this.options = options;

    // If we have no title yet, try to pull it from the html title attribute (jQuery doesnt' pick it up as it's not a
    // data-attribute)
    if (this.options.title === null) {
      this.options.title = this.$element.attr('title');
    }

    //Expose public methods
    this.val = Selectpicker.prototype.val;
    this.render = Selectpicker.prototype.render;
    this.refresh = Selectpicker.prototype.refresh;
    this.setStyle = Selectpicker.prototype.setStyle;
    this.selectAll = Selectpicker.prototype.selectAll;
    this.deselectAll = Selectpicker.prototype.deselectAll;
    this.destroy = Selectpicker.prototype.destroy;
    this.remove = Selectpicker.prototype.remove;
    this.show = Selectpicker.prototype.show;
    this.hide = Selectpicker.prototype.hide;

    this.init();
  };

  Selectpicker.VERSION = '1.10.0';

  // part of this is duplicated in i18n/defaults-en_US.js. Make sure to update both.
  Selectpicker.DEFAULTS = {
    noneSelectedText: 'Nothing selected',
    noneResultsText: 'No results matched {0}',
    countSelectedText: function (numSelected, numTotal) {
      return (numSelected == 1) ? "{0} item selected" : "{0} items selected";
    },
    maxOptionsText: function (numAll, numGroup) {
      return [
        (numAll == 1) ? 'Limit reached ({n} item max)' : 'Limit reached ({n} items max)',
        (numGroup == 1) ? 'Group limit reached ({n} item max)' : 'Group limit reached ({n} items max)'
      ];
    },
    selectAllText: 'Select All',
    deselectAllText: 'Deselect All',
    doneButton: false,
    doneButtonText: 'Close',
    multipleSeparator: ', ',
    styleBase: 'btn',
    style: 'btn-default',
    size: 'auto',
    title: null,
    selectedTextFormat: 'values',
    width: false,
    container: false,
    hideDisabled: false,
    showSubtext: false,
    showIcon: true,
    showContent: true,
    dropupAuto: true,
    header: false,
    liveSearch: false,
    liveSearchPlaceholder: null,
    liveSearchNormalize: false,
    liveSearchStyle: 'contains',
    actionsBox: false,
    iconBase: 'glyphicon',
    tickIcon: 'glyphicon-ok',
    showTick: false,
    template: {
      caret: '<span class="caret"></span>'
    },
    maxOptions: false,
    mobile: false,
    selectOnTab: false,
    dropdownAlignRight: false
  };

  Selectpicker.prototype = {

    constructor: Selectpicker,

    init: function () {
      var that = this,
          id = this.$element.attr('id');

      this.$element.addClass('bs-select-hidden');

      // store originalIndex (key) and newIndex (value) in this.liObj for fast accessibility
      // allows us to do this.$lis.eq(that.liObj[index]) instead of this.$lis.filter('[data-original-index="' + index + '"]')
      this.liObj = {};
      this.multiple = this.$element.prop('multiple');
      this.autofocus = this.$element.prop('autofocus');
      this.$newElement = this.createView();
      this.$element
        .after(this.$newElement)
        .appendTo(this.$newElement);
      this.$button = this.$newElement.children('button');
      this.$menu = this.$newElement.children('.dropdown-menu');
      this.$menuInner = this.$menu.children('.inner');
      this.$searchbox = this.$menu.find('input');

      this.$element.removeClass('bs-select-hidden');

      if (this.options.dropdownAlignRight)
        this.$menu.addClass('dropdown-menu-right');

      if (typeof id !== 'undefined') {
        this.$button.attr('data-id', id);
        $('label[for="' + id + '"]').click(function (e) {
          e.preventDefault();
          that.$button.focus();
        });
      }

      this.checkDisabled();
      this.clickListener();
      if (this.options.liveSearch) this.liveSearchListener();
      this.render();
      this.setStyle();
      this.setWidth();
      if (this.options.container) this.selectPosition();
      this.$menu.data('this', this);
      this.$newElement.data('this', this);
      if (this.options.mobile) this.mobile();

      this.$newElement.on({
        'hide.bs.dropdown': function (e) {
          that.$element.trigger('hide.bs.select', e);
        },
        'hidden.bs.dropdown': function (e) {
          that.$element.trigger('hidden.bs.select', e);
        },
        'show.bs.dropdown': function (e) {
          that.$element.trigger('show.bs.select', e);
        },
        'shown.bs.dropdown': function (e) {
          that.$element.trigger('shown.bs.select', e);
        }
      });

      if (that.$element[0].hasAttribute('required')) {
        this.$element.on('invalid', function () {
          that.$button
            .addClass('bs-invalid')
            .focus();
          
          that.$element.on({
            'focus.bs.select': function () {
              that.$button.focus();
              that.$element.off('focus.bs.select');
            },
            'shown.bs.select': function () {
              that.$element
                .val(that.$element.val()) // set the value to hide the validation message in Chrome when menu is opened
                .off('shown.bs.select');
            },
            'rendered.bs.select': function () {
              // if select is no longer invalid, remove the bs-invalid class
              if (this.validity.valid) that.$button.removeClass('bs-invalid');
              that.$element.off('rendered.bs.select');
            }
          });
          
        });
      }

      setTimeout(function () {
        that.$element.trigger('loaded.bs.select');
      });
    },

    createDropdown: function () {
      // Options
      // If we are multiple or showTick option is set, then add the show-tick class
      var showTick = (this.multiple || this.options.showTick) ? ' show-tick' : '',
          inputGroup = this.$element.parent().hasClass('input-group') ? ' input-group-btn' : '',
          autofocus = this.autofocus ? ' autofocus' : '';
      // Elements
      var header = this.options.header ? '<div class="popover-title"><button type="button" class="close" aria-hidden="true">&times;</button>' + this.options.header + '</div>' : '';
      var searchbox = this.options.liveSearch ?
      '<div class="bs-searchbox">' +
      '<input type="text" class="form-control" autocomplete="off"' +
      (null === this.options.liveSearchPlaceholder ? '' : ' placeholder="' + htmlEscape(this.options.liveSearchPlaceholder) + '"') + '>' +
      '</div>'
          : '';
      var actionsbox = this.multiple && this.options.actionsBox ?
      '<div class="bs-actionsbox">' +
      '<div class="btn-group btn-group-sm btn-block">' +
      '<button type="button" class="actions-btn bs-select-all btn btn-default">' +
      this.options.selectAllText +
      '</button>' +
      '<button type="button" class="actions-btn bs-deselect-all btn btn-default">' +
      this.options.deselectAllText +
      '</button>' +
      '</div>' +
      '</div>'
          : '';
      var donebutton = this.multiple && this.options.doneButton ?
      '<div class="bs-donebutton">' +
      '<div class="btn-group btn-block">' +
      '<button type="button" class="btn btn-sm btn-default">' +
      this.options.doneButtonText +
      '</button>' +
      '</div>' +
      '</div>'
          : '';
      var drop =
          '<div class="btn-group bootstrap-select' + showTick + inputGroup + '">' +
          '<button type="button" class="' + this.options.styleBase + ' dropdown-toggle" data-toggle="dropdown"' + autofocus + '>' +
          '<span class="filter-option pull-left"></span>&nbsp;' +
          '<span class="bs-caret">' +
          this.options.template.caret +
          '</span>' +
          '</button>' +
          '<div class="dropdown-menu open">' +
          header +
          searchbox +
          actionsbox +
          '<ul class="dropdown-menu inner" role="menu">' +
          '</ul>' +
          donebutton +
          '</div>' +
          '</div>';

      return $(drop);
    },

    createView: function () {
      var $drop = this.createDropdown(),
          li = this.createLi();

      $drop.find('ul')[0].innerHTML = li;
      return $drop;
    },

    reloadLi: function () {
      //Remove all children.
      this.destroyLi();
      //Re build
      var li = this.createLi();
      this.$menuInner[0].innerHTML = li;
    },

    destroyLi: function () {
      this.$menu.find('li').remove();
    },

    createLi: function () {
      var that = this,
          _li = [],
          optID = 0,
          titleOption = document.createElement('option'),
          liIndex = -1; // increment liIndex whenever a new <li> element is created to ensure liObj is correct

      // Helper functions
      /**
       * @param content
       * @param [index]
       * @param [classes]
       * @param [optgroup]
       * @returns {string}
       */
      var generateLI = function (content, index, classes, optgroup) {
        return '<li' +
            ((typeof classes !== 'undefined' & '' !== classes) ? ' class="' + classes + '"' : '') +
            ((typeof index !== 'undefined' & null !== index) ? ' data-original-index="' + index + '"' : '') +
            ((typeof optgroup !== 'undefined' & null !== optgroup) ? 'data-optgroup="' + optgroup + '"' : '') +
            '>' + content + '</li>';
      };

      /**
       * @param text
       * @param [classes]
       * @param [inline]
       * @param [tokens]
       * @returns {string}
       */
      var generateA = function (text, classes, inline, tokens) {
        return '<a tabindex="0"' +
            (typeof classes !== 'undefined' ? ' class="' + classes + '"' : '') +
            (typeof inline !== 'undefined' ? ' style="' + inline + '"' : '') +
            (that.options.liveSearchNormalize ? ' data-normalized-text="' + normalizeToBase(htmlEscape(text)) + '"' : '') +
            (typeof tokens !== 'undefined' || tokens !== null ? ' data-tokens="' + tokens + '"' : '') +
            '>' + text +
            '<span class="' + that.options.iconBase + ' ' + that.options.tickIcon + ' check-mark"></span>' +
            '</a>';
      };

      if (this.options.title && !this.multiple) {
        // this option doesn't create a new <li> element, but does add a new option, so liIndex is decreased
        // since liObj is recalculated on every refresh, liIndex needs to be decreased even if the titleOption is already appended
        liIndex--;

        if (!this.$element.find('.bs-title-option').length) {
          // Use native JS to prepend option (faster)
          var element = this.$element[0];
          titleOption.className = 'bs-title-option';
          titleOption.appendChild(document.createTextNode(this.options.title));
          titleOption.value = '';
          element.insertBefore(titleOption, element.firstChild);
          // Check if selected attribute is already set on an option. If not, select the titleOption option.
          if ($(element.options[element.selectedIndex]).attr('selected') === undefined) titleOption.selected = true;
        }
      }

      this.$element.find('option').each(function (index) {
        var $this = $(this);

        liIndex++;

        if ($this.hasClass('bs-title-option')) return;

        // Get the class and text for the option
        var optionClass = this.className || '',
            inline = this.style.cssText,
            text = $this.data('content') ? $this.data('content') : $this.html(),
            tokens = $this.data('tokens') ? $this.data('tokens') : null,
            subtext = typeof $this.data('subtext') !== 'undefined' ? '<small class="text-muted">' + $this.data('subtext') + '</small>' : '',
            icon = typeof $this.data('icon') !== 'undefined' ? '<span class="' + that.options.iconBase + ' ' + $this.data('icon') + '"></span> ' : '',
            isOptgroup = this.parentNode.tagName === 'OPTGROUP',
            isDisabled = this.disabled || (isOptgroup && this.parentNode.disabled);

        if (icon !== '' && isDisabled) {
          icon = '<span>' + icon + '</span>';
        }

        if (that.options.hideDisabled && isDisabled && !isOptgroup) {
          liIndex--;
          return;
        }

        if (!$this.data('content')) {
          // Prepend any icon and append any subtext to the main text.
          text = icon + '<span class="text">' + text + subtext + '</span>';
        }

        if (isOptgroup && $this.data('divider') !== true) {
          var optGroupClass = ' ' + this.parentNode.className || '';

          if ($this.index() === 0) { // Is it the first option of the optgroup?
            optID += 1;

            // Get the opt group label
            var label = this.parentNode.label,
                labelSubtext = typeof $this.parent().data('subtext') !== 'undefined' ? '<small class="text-muted">' + $this.parent().data('subtext') + '</small>' : '',
                labelIcon = $this.parent().data('icon') ? '<span class="' + that.options.iconBase + ' ' + $this.parent().data('icon') + '"></span> ' : '';

            label = labelIcon + '<span class="text">' + label + labelSubtext + '</span>';

            if (index !== 0 && _li.length > 0) { // Is it NOT the first option of the select && are there elements in the dropdown?
              liIndex++;
              _li.push(generateLI('', null, 'divider', optID + 'div'));
            }
            liIndex++;
            _li.push(generateLI(label, null, 'dropdown-header' + optGroupClass, optID));
          }

          if (that.options.hideDisabled && isDisabled) {
            liIndex--;
            return;
          }

          _li.push(generateLI(generateA(text, 'opt ' + optionClass + optGroupClass, inline, tokens), index, '', optID));
        } else if ($this.data('divider') === true) {
          _li.push(generateLI('', index, 'divider'));
        } else if ($this.data('hidden') === true) {
          _li.push(generateLI(generateA(text, optionClass, inline, tokens), index, 'hidden is-hidden'));
        } else {
          if (this.previousElementSibling && this.previousElementSibling.tagName === 'OPTGROUP') {
            liIndex++;
            _li.push(generateLI('', null, 'divider', optID + 'div'));
          }
          _li.push(generateLI(generateA(text, optionClass, inline, tokens), index));
        }

        that.liObj[index] = liIndex;
      });

      //If we are not multiple, we don't have a selected item, and we don't have a title, select the first element so something is set in the button
      if (!this.multiple && this.$element.find('option:selected').length === 0 && !this.options.title) {
        this.$element.find('option').eq(0).prop('selected', true).attr('selected', 'selected');
      }

      return _li.join('');
    },

    findLis: function () {
      if (this.$lis == null) this.$lis = this.$menu.find('li');
      return this.$lis;
    },

    /**
     * @param [updateLi] defaults to true
     */
    render: function (updateLi) {
      var that = this,
          notDisabled;

      //Update the LI to match the SELECT
      if (updateLi !== false) {
        this.$element.find('option').each(function (index) {
          var $lis = that.findLis().eq(that.liObj[index]);

          that.setDisabled(index, this.disabled || this.parentNode.tagName === 'OPTGROUP' && this.parentNode.disabled, $lis);
          that.setSelected(index, this.selected, $lis);
        });
      }

      this.tabIndex();

      var selectedItems = this.$element.find('option').map(function () {
        if (this.selected) {
          if (that.options.hideDisabled && (this.disabled || this.parentNode.tagName === 'OPTGROUP' && this.parentNode.disabled)) return;

          var $this = $(this),
              icon = $this.data('icon') && that.options.showIcon ? '<i class="' + that.options.iconBase + ' ' + $this.data('icon') + '"></i> ' : '',
              subtext;

          if (that.options.showSubtext && $this.data('subtext') && !that.multiple) {
            subtext = ' <small class="text-muted">' + $this.data('subtext') + '</small>';
          } else {
            subtext = '';
          }
          if (typeof $this.attr('title') !== 'undefined') {
            return $this.attr('title');
          } else if ($this.data('content') && that.options.showContent) {
            return $this.data('content');
          } else {
            return icon + $this.html() + subtext;
          }
        }
      }).toArray();

      //Fixes issue in IE10 occurring when no default option is selected and at least one option is disabled
      //Convert all the values into a comma delimited string
      var title = !this.multiple ? selectedItems[0] : selectedItems.join(this.options.multipleSeparator);

      //If this is multi select, and the selectText type is count, the show 1 of 2 selected etc..
      if (this.multiple && this.options.selectedTextFormat.indexOf('count') > -1) {
        var max = this.options.selectedTextFormat.split('>');
        if ((max.length > 1 && selectedItems.length > max[1]) || (max.length == 1 && selectedItems.length >= 2)) {
          notDisabled = this.options.hideDisabled ? ', [disabled]' : '';
          var totalCount = this.$element.find('option').not('[data-divider="true"], [data-hidden="true"]' + notDisabled).length,
              tr8nText = (typeof this.options.countSelectedText === 'function') ? this.options.countSelectedText(selectedItems.length, totalCount) : this.options.countSelectedText;
          title = tr8nText.replace('{0}', selectedItems.length.toString()).replace('{1}', totalCount.toString());
        }
      }

      if (this.options.title == undefined) {
        this.options.title = this.$element.attr('title');
      }

      if (this.options.selectedTextFormat == 'static') {
        title = this.options.title;
      }

      //If we dont have a title, then use the default, or if nothing is set at all, use the not selected text
      if (!title) {
        title = typeof this.options.title !== 'undefined' ? this.options.title : this.options.noneSelectedText;
      }

      //strip all html-tags and trim the result
      this.$button.attr('title', $.trim(title.replace(/<[^>]*>?/g, '')));
      this.$button.children('.filter-option').html(title);

      this.$element.trigger('rendered.bs.select');
    },

    /**
     * @param [style]
     * @param [status]
     */
    setStyle: function (style, status) {
      if (this.$element.attr('class')) {
        this.$newElement.addClass(this.$element.attr('class').replace(/selectpicker|mobile-device|bs-select-hidden|validate\[.*\]/gi, ''));
      }

      var buttonClass = style ? style : this.options.style;

      if (status == 'add') {
        this.$button.addClass(buttonClass);
      } else if (status == 'remove') {
        this.$button.removeClass(buttonClass);
      } else {
        this.$button.removeClass(this.options.style);
        this.$button.addClass(buttonClass);
      }
    },

    liHeight: function (refresh) {
      if (!refresh && (this.options.size === false || this.sizeInfo)) return;

      var newElement = document.createElement('div'),
          menu = document.createElement('div'),
          menuInner = document.createElement('ul'),
          divider = document.createElement('li'),
          li = document.createElement('li'),
          a = document.createElement('a'),
          text = document.createElement('span'),
          header = this.options.header && this.$menu.find('.popover-title').length > 0 ? this.$menu.find('.popover-title')[0].cloneNode(true) : null,
          search = this.options.liveSearch ? document.createElement('div') : null,
          actions = this.options.actionsBox && this.multiple && this.$menu.find('.bs-actionsbox').length > 0 ? this.$menu.find('.bs-actionsbox')[0].cloneNode(true) : null,
          doneButton = this.options.doneButton && this.multiple && this.$menu.find('.bs-donebutton').length > 0 ? this.$menu.find('.bs-donebutton')[0].cloneNode(true) : null;

      text.className = 'text';
      newElement.className = this.$menu[0].parentNode.className + ' open';
      menu.className = 'dropdown-menu open';
      menuInner.className = 'dropdown-menu inner';
      divider.className = 'divider';

      text.appendChild(document.createTextNode('Inner text'));
      a.appendChild(text);
      li.appendChild(a);
      menuInner.appendChild(li);
      menuInner.appendChild(divider);
      if (header) menu.appendChild(header);
      if (search) {
        // create a span instead of input as creating an input element is slower
        var input = document.createElement('span');
        search.className = 'bs-searchbox';
        input.className = 'form-control';
        search.appendChild(input);
        menu.appendChild(search);
      }
      if (actions) menu.appendChild(actions);
      menu.appendChild(menuInner);
      if (doneButton) menu.appendChild(doneButton);
      newElement.appendChild(menu);

      document.body.appendChild(newElement);

      var liHeight = a.offsetHeight,
          headerHeight = header ? header.offsetHeight : 0,
          searchHeight = search ? search.offsetHeight : 0,
          actionsHeight = actions ? actions.offsetHeight : 0,
          doneButtonHeight = doneButton ? doneButton.offsetHeight : 0,
          dividerHeight = $(divider).outerHeight(true),
          // fall back to jQuery if getComputedStyle is not supported
          menuStyle = typeof getComputedStyle === 'function' ? getComputedStyle(menu) : false,
          $menu = menuStyle ? null : $(menu),
          menuPadding = parseInt(menuStyle ? menuStyle.paddingTop : $menu.css('paddingTop')) +
                        parseInt(menuStyle ? menuStyle.paddingBottom : $menu.css('paddingBottom')) +
                        parseInt(menuStyle ? menuStyle.borderTopWidth : $menu.css('borderTopWidth')) +
                        parseInt(menuStyle ? menuStyle.borderBottomWidth : $menu.css('borderBottomWidth')),
          menuExtras =  menuPadding +
                        parseInt(menuStyle ? menuStyle.marginTop : $menu.css('marginTop')) +
                        parseInt(menuStyle ? menuStyle.marginBottom : $menu.css('marginBottom')) + 2;

      document.body.removeChild(newElement);

      this.sizeInfo = {
        liHeight: liHeight,
        headerHeight: headerHeight,
        searchHeight: searchHeight,
        actionsHeight: actionsHeight,
        doneButtonHeight: doneButtonHeight,
        dividerHeight: dividerHeight,
        menuPadding: menuPadding,
        menuExtras: menuExtras
      };
    },

    setSize: function () {
      this.findLis();
      this.liHeight();

      if (this.options.header) this.$menu.css('padding-top', 0);
      if (this.options.size === false) return;

      var that = this,
          $menu = this.$menu,
          $menuInner = this.$menuInner,
          $window = $(window),
          selectHeight = this.$newElement[0].offsetHeight,
          liHeight = this.sizeInfo['liHeight'],
          headerHeight = this.sizeInfo['headerHeight'],
          searchHeight = this.sizeInfo['searchHeight'],
          actionsHeight = this.sizeInfo['actionsHeight'],
          doneButtonHeight = this.sizeInfo['doneButtonHeight'],
          divHeight = this.sizeInfo['dividerHeight'],
          menuPadding = this.sizeInfo['menuPadding'],
          menuExtras = this.sizeInfo['menuExtras'],
          notDisabled = this.options.hideDisabled ? '.disabled' : '',
          menuHeight,
          getHeight,
          selectOffsetTop,
          selectOffsetBot,
          posVert = function () {
            selectOffsetTop = that.$newElement.offset().top - $window.scrollTop();
            selectOffsetBot = $window.height() - selectOffsetTop - selectHeight;
          };

      posVert();

      if (this.options.size === 'auto') {
        var getSize = function () {
          var minHeight,
              hasClass = function (className, include) {
                return function (element) {
                    if (include) {
                        return (element.classList ? element.classList.contains(className) : $(element).hasClass(className));
                    } else {
                        return !(element.classList ? element.classList.contains(className) : $(element).hasClass(className));
                    }
                };
              },
              lis = that.$menuInner[0].getElementsByTagName('li'),
              lisVisible = Array.prototype.filter ? Array.prototype.filter.call(lis, hasClass('hidden', false)) : that.$lis.not('.hidden'),
              optGroup = Array.prototype.filter ? Array.prototype.filter.call(lisVisible, hasClass('dropdown-header', true)) : lisVisible.filter('.dropdown-header');

          posVert();
          menuHeight = selectOffsetBot - menuExtras;

          if (that.options.container) {
            if (!$menu.data('height')) $menu.data('height', $menu.height());
            getHeight = $menu.data('height');
          } else {
            getHeight = $menu.height();
          }

          if (that.options.dropupAuto) {
            that.$newElement.toggleClass('dropup', selectOffsetTop > selectOffsetBot && (menuHeight - menuExtras) < getHeight);
          }
          if (that.$newElement.hasClass('dropup')) {
            menuHeight = selectOffsetTop - menuExtras;
          }

          if ((lisVisible.length + optGroup.length) > 3) {
            minHeight = liHeight * 3 + menuExtras - 2;
          } else {
            minHeight = 0;
          }

          $menu.css({
            'max-height': menuHeight + 'px',
            'overflow': 'hidden',
            'min-height': minHeight + headerHeight + searchHeight + actionsHeight + doneButtonHeight + 'px'
          });
          $menuInner.css({
            'max-height': menuHeight - headerHeight - searchHeight - actionsHeight - doneButtonHeight - menuPadding + 'px',
            'overflow-y': 'auto',
            'min-height': Math.max(minHeight - menuPadding, 0) + 'px'
          });
        };
        getSize();
        this.$searchbox.off('input.getSize propertychange.getSize').on('input.getSize propertychange.getSize', getSize);
        $window.off('resize.getSize scroll.getSize').on('resize.getSize scroll.getSize', getSize);
      } else if (this.options.size && this.options.size != 'auto' && this.$lis.not(notDisabled).length > this.options.size) {
        var optIndex = this.$lis.not('.divider').not(notDisabled).children().slice(0, this.options.size).last().parent().index(),
            divLength = this.$lis.slice(0, optIndex + 1).filter('.divider').length;
        menuHeight = liHeight * this.options.size + divLength * divHeight + menuPadding;

        if (that.options.container) {
          if (!$menu.data('height')) $menu.data('height', $menu.height());
          getHeight = $menu.data('height');
        } else {
          getHeight = $menu.height();
        }

        if (that.options.dropupAuto) {
          //noinspection JSUnusedAssignment
          this.$newElement.toggleClass('dropup', selectOffsetTop > selectOffsetBot && (menuHeight - menuExtras) < getHeight);
        }
        $menu.css({
          'max-height': menuHeight + headerHeight + searchHeight + actionsHeight + doneButtonHeight + 'px',
          'overflow': 'hidden',
          'min-height': ''
        });
        $menuInner.css({
          'max-height': menuHeight - menuPadding + 'px',
          'overflow-y': 'auto',
          'min-height': ''
        });
      }
    },

    setWidth: function () {
      if (this.options.width === 'auto') {
        this.$menu.css('min-width', '0');

        // Get correct width if element is hidden
        var $selectClone = this.$menu.parent().clone().appendTo('body'),
            $selectClone2 = this.options.container ? this.$newElement.clone().appendTo('body') : $selectClone,
            ulWidth = $selectClone.children('.dropdown-menu').outerWidth(),
            btnWidth = $selectClone2.css('width', 'auto').children('button').outerWidth();

        $selectClone.remove();
        $selectClone2.remove();

        // Set width to whatever's larger, button title or longest option
        this.$newElement.css('width', Math.max(ulWidth, btnWidth) + 'px');
      } else if (this.options.width === 'fit') {
        // Remove inline min-width so width can be changed from 'auto'
        this.$menu.css('min-width', '');
        this.$newElement.css('width', '').addClass('fit-width');
      } else if (this.options.width) {
        // Remove inline min-width so width can be changed from 'auto'
        this.$menu.css('min-width', '');
        this.$newElement.css('width', this.options.width);
      } else {
        // Remove inline min-width/width so width can be changed
        this.$menu.css('min-width', '');
        this.$newElement.css('width', '');
      }
      // Remove fit-width class if width is changed programmatically
      if (this.$newElement.hasClass('fit-width') && this.options.width !== 'fit') {
        this.$newElement.removeClass('fit-width');
      }
    },

    selectPosition: function () {
      this.$bsContainer = $('<div class="bs-container" />');

      var that = this,
          pos,
          actualHeight,
          getPlacement = function ($element) {
            that.$bsContainer.addClass($element.attr('class').replace(/form-control|fit-width/gi, '')).toggleClass('dropup', $element.hasClass('dropup'));
            pos = $element.offset();
            actualHeight = $element.hasClass('dropup') ? 0 : $element[0].offsetHeight;
            that.$bsContainer.css({
              'top': pos.top + actualHeight,
              'left': pos.left,
              'width': $element[0].offsetWidth
            });
          };

      this.$button.on('click', function () {
        var $this = $(this);

        if (that.isDisabled()) {
          return;
        }

        getPlacement(that.$newElement);

        that.$bsContainer
          .appendTo(that.options.container)
          .toggleClass('open', !$this.hasClass('open'))
          .append(that.$menu);
      });

      $(window).on('resize scroll', function () {
        getPlacement(that.$newElement);
      });

      this.$element.on('hide.bs.select', function () {
        that.$menu.data('height', that.$menu.height());
        that.$bsContainer.detach();
      });
    },

    setSelected: function (index, selected, $lis) {
      if (!$lis) {
        $lis = this.findLis().eq(this.liObj[index]);
      }

      $lis.toggleClass('selected', selected);
    },

    setDisabled: function (index, disabled, $lis) {
      if (!$lis) {
        $lis = this.findLis().eq(this.liObj[index]);
      }

      if (disabled) {
        $lis.addClass('disabled').children('a').attr('href', '#').attr('tabindex', -1);
      } else {
        $lis.removeClass('disabled').children('a').removeAttr('href').attr('tabindex', 0);
      }
    },

    isDisabled: function () {
      return this.$element[0].disabled;
    },

    checkDisabled: function () {
      var that = this;

      if (this.isDisabled()) {
        this.$newElement.addClass('disabled');
        this.$button.addClass('disabled').attr('tabindex', -1);
      } else {
        if (this.$button.hasClass('disabled')) {
          this.$newElement.removeClass('disabled');
          this.$button.removeClass('disabled');
        }

        if (this.$button.attr('tabindex') == -1 && !this.$element.data('tabindex')) {
          this.$button.removeAttr('tabindex');
        }
      }

      this.$button.click(function () {
        return !that.isDisabled();
      });
    },

    tabIndex: function () {
      if (this.$element.data('tabindex') !== this.$element.attr('tabindex') && 
        (this.$element.attr('tabindex') !== -98 && this.$element.attr('tabindex') !== '-98')) {
        this.$element.data('tabindex', this.$element.attr('tabindex'));
        this.$button.attr('tabindex', this.$element.data('tabindex'));
      }
      
      this.$element.attr('tabindex', -98);
    },

    clickListener: function () {
      var that = this,
          $document = $(document);

      this.$newElement.on('touchstart.dropdown', '.dropdown-menu', function (e) {
        e.stopPropagation();
      });

      $document.data('spaceSelect', false);

      this.$button.on('keyup', function (e) {
        if (/(32)/.test(e.keyCode.toString(10)) && $document.data('spaceSelect')) {
            e.preventDefault();
            $document.data('spaceSelect', false);
        }
      });

      this.$button.on('click', function () {
        that.setSize();
      });

      this.$element.on('shown.bs.select', function () {
        if (!that.options.liveSearch && !that.multiple) {
          that.$menuInner.find('.selected a').focus();
        } else if (!that.multiple) {
          var selectedIndex = that.liObj[that.$element[0].selectedIndex];

          if (typeof selectedIndex !== 'number' || that.options.size === false) return;

          // scroll to selected option
          var offset = that.$lis.eq(selectedIndex)[0].offsetTop - that.$menuInner[0].offsetTop;
          offset = offset - that.$menuInner[0].offsetHeight/2 + that.sizeInfo.liHeight/2;
          that.$menuInner[0].scrollTop = offset;
        }
      });

      this.$menuInner.on('click', 'li a', function (e) {
        var $this = $(this),
            clickedIndex = $this.parent().data('originalIndex'),
            prevValue = that.$element.val(),
            prevIndex = that.$element.prop('selectedIndex');

        // Don't close on multi choice menu
        if (that.multiple) {
          e.stopPropagation();
        }

        e.preventDefault();

        //Don't run if we have been disabled
        if (!that.isDisabled() && !$this.parent().hasClass('disabled')) {
          var $options = that.$element.find('option'),
              $option = $options.eq(clickedIndex),
              state = $option.prop('selected'),
              $optgroup = $option.parent('optgroup'),
              maxOptions = that.options.maxOptions,
              maxOptionsGrp = $optgroup.data('maxOptions') || false;

          if (!that.multiple) { // Deselect all others if not multi select box
            $options.prop('selected', false);
            $option.prop('selected', true);
            that.$menuInner.find('.selected').removeClass('selected');
            that.setSelected(clickedIndex, true);
          } else { // Toggle the one we have chosen if we are multi select.
            $option.prop('selected', !state);
            that.setSelected(clickedIndex, !state);
            $this.blur();

            if (maxOptions !== false || maxOptionsGrp !== false) {
              var maxReached = maxOptions < $options.filter(':selected').length,
                  maxReachedGrp = maxOptionsGrp < $optgroup.find('option:selected').length;

              if ((maxOptions && maxReached) || (maxOptionsGrp && maxReachedGrp)) {
                if (maxOptions && maxOptions == 1) {
                  $options.prop('selected', false);
                  $option.prop('selected', true);
                  that.$menuInner.find('.selected').removeClass('selected');
                  that.setSelected(clickedIndex, true);
                } else if (maxOptionsGrp && maxOptionsGrp == 1) {
                  $optgroup.find('option:selected').prop('selected', false);
                  $option.prop('selected', true);
                  var optgroupID = $this.parent().data('optgroup');
                  that.$menuInner.find('[data-optgroup="' + optgroupID + '"]').removeClass('selected');
                  that.setSelected(clickedIndex, true);
                } else {
                  var maxOptionsArr = (typeof that.options.maxOptionsText === 'function') ?
                          that.options.maxOptionsText(maxOptions, maxOptionsGrp) : that.options.maxOptionsText,
                      maxTxt = maxOptionsArr[0].replace('{n}', maxOptions),
                      maxTxtGrp = maxOptionsArr[1].replace('{n}', maxOptionsGrp),
                      $notify = $('<div class="notify"></div>');
                  // If {var} is set in array, replace it
                  /** @deprecated */
                  if (maxOptionsArr[2]) {
                    maxTxt = maxTxt.replace('{var}', maxOptionsArr[2][maxOptions > 1 ? 0 : 1]);
                    maxTxtGrp = maxTxtGrp.replace('{var}', maxOptionsArr[2][maxOptionsGrp > 1 ? 0 : 1]);
                  }

                  $option.prop('selected', false);

                  that.$menu.append($notify);

                  if (maxOptions && maxReached) {
                    $notify.append($('<div>' + maxTxt + '</div>'));
                    that.$element.trigger('maxReached.bs.select');
                  }

                  if (maxOptionsGrp && maxReachedGrp) {
                    $notify.append($('<div>' + maxTxtGrp + '</div>'));
                    that.$element.trigger('maxReachedGrp.bs.select');
                  }

                  setTimeout(function () {
                    that.setSelected(clickedIndex, false);
                  }, 10);

                  $notify.delay(750).fadeOut(300, function () {
                    $(this).remove();
                  });
                }
              }
            }
          }

          if (!that.multiple) {
            that.$button.focus();
          } else if (that.options.liveSearch) {
            that.$searchbox.focus();
          }

          // Trigger select 'change'
          if ((prevValue != that.$element.val() && that.multiple) || (prevIndex != that.$element.prop('selectedIndex') && !that.multiple)) {
            // $option.prop('selected') is current option state (selected/unselected). state is previous option state.
            that.$element
              .trigger('changed.bs.select', [clickedIndex, $option.prop('selected'), state])
              .triggerNative('change');
          }
        }
      });

      this.$menu.on('click', 'li.disabled a, .popover-title, .popover-title :not(.close)', function (e) {
        if (e.currentTarget == this) {
          e.preventDefault();
          e.stopPropagation();
          if (that.options.liveSearch && !$(e.target).hasClass('close')) {
            that.$searchbox.focus();
          } else {
            that.$button.focus();
          }
        }
      });

      this.$menuInner.on('click', '.divider, .dropdown-header', function (e) {
        e.preventDefault();
        e.stopPropagation();
        if (that.options.liveSearch) {
          that.$searchbox.focus();
        } else {
          that.$button.focus();
        }
      });

      this.$menu.on('click', '.popover-title .close', function () {
        that.$button.click();
      });

      this.$searchbox.on('click', function (e) {
        e.stopPropagation();
      });

      this.$menu.on('click', '.actions-btn', function (e) {
        if (that.options.liveSearch) {
          that.$searchbox.focus();
        } else {
          that.$button.focus();
        }

        e.preventDefault();
        e.stopPropagation();

        if ($(this).hasClass('bs-select-all')) {
          that.selectAll();
        } else {
          that.deselectAll();
        }
      });

      this.$element.change(function () {
        that.render(false);
      });
    },

    liveSearchListener: function () {
      var that = this,
          $no_results = $('<li class="no-results"></li>');

      this.$button.on('click.dropdown.data-api touchstart.dropdown.data-api', function () {
        that.$menuInner.find('.active').removeClass('active');
        if (!!that.$searchbox.val()) {
          that.$searchbox.val('');
          that.$lis.not('.is-hidden').removeClass('hidden');
          if (!!$no_results.parent().length) $no_results.remove();
        }
        if (!that.multiple) that.$menuInner.find('.selected').addClass('active');
        setTimeout(function () {
          that.$searchbox.focus();
        }, 10);
      });

      this.$searchbox.on('click.dropdown.data-api focus.dropdown.data-api touchend.dropdown.data-api', function (e) {
        e.stopPropagation();
      });

      this.$searchbox.on('input propertychange', function () {
        if (that.$searchbox.val()) {
          var $searchBase = that.$lis.not('.is-hidden').removeClass('hidden').children('a');
          if (that.options.liveSearchNormalize) {
            $searchBase = $searchBase.not(':a' + that._searchStyle() + '("' + normalizeToBase(that.$searchbox.val()) + '")');
          } else {
            $searchBase = $searchBase.not(':' + that._searchStyle() + '("' + that.$searchbox.val() + '")');
          }
          $searchBase.parent().addClass('hidden');

          that.$lis.filter('.dropdown-header').each(function () {
            var $this = $(this),
                optgroup = $this.data('optgroup');

            if (that.$lis.filter('[data-optgroup=' + optgroup + ']').not($this).not('.hidden').length === 0) {
              $this.addClass('hidden');
              that.$lis.filter('[data-optgroup=' + optgroup + 'div]').addClass('hidden');
            }
          });

          var $lisVisible = that.$lis.not('.hidden');

          // hide divider if first or last visible, or if followed by another divider
          $lisVisible.each(function (index) {
            var $this = $(this);

            if ($this.hasClass('divider') && (
              $this.index() === $lisVisible.first().index() ||
              $this.index() === $lisVisible.last().index() ||
              $lisVisible.eq(index + 1).hasClass('divider'))) {
              $this.addClass('hidden');
            }
          });

          if (!that.$lis.not('.hidden, .no-results').length) {
            if (!!$no_results.parent().length) {
              $no_results.remove();
            }
            $no_results.html(that.options.noneResultsText.replace('{0}', '"' + htmlEscape(that.$searchbox.val()) + '"')).show();
            that.$menuInner.append($no_results);
          } else if (!!$no_results.parent().length) {
            $no_results.remove();
          }
        } else {
          that.$lis.not('.is-hidden').removeClass('hidden');
          if (!!$no_results.parent().length) {
            $no_results.remove();
          }
        }

        that.$lis.filter('.active').removeClass('active');
        if (that.$searchbox.val()) that.$lis.not('.hidden, .divider, .dropdown-header').eq(0).addClass('active').children('a').focus();
        $(this).focus();
      });
    },

    _searchStyle: function () {
      var styles = {
        begins: 'ibegins',
        startsWith: 'ibegins'
      };

      return styles[this.options.liveSearchStyle] || 'icontains';
    },

    val: function (value) {
      if (typeof value !== 'undefined') {
        this.$element.val(value);
        this.render();

        return this.$element;
      } else {
        return this.$element.val();
      }
    },

    changeAll: function (status) {
      if (typeof status === 'undefined') status = true;

      this.findLis();

      var $options = this.$element.find('option'),
          $lisVisible = this.$lis.not('.divider, .dropdown-header, .disabled, .hidden').toggleClass('selected', status),
          lisVisLen = $lisVisible.length,
          selectedOptions = [];

      for (var i = 0; i < lisVisLen; i++) {
        var origIndex = $lisVisible[i].getAttribute('data-original-index');
        selectedOptions[selectedOptions.length] = $options.eq(origIndex)[0];
      }

      $(selectedOptions).prop('selected', status);

      this.render(false);

      this.$element
        .trigger('changed.bs.select')
        .triggerNative('change');
    },

    selectAll: function () {
      return this.changeAll(true);
    },

    deselectAll: function () {
      return this.changeAll(false);
    },

    toggle: function (e) {
      e = e || window.event;
      
      if (e) e.stopPropagation();

      this.$button.trigger('click');
    },

    keydown: function (e) {
      var $this = $(this),
          $parent = $this.is('input') ? $this.parent().parent() : $this.parent(),
          $items,
          that = $parent.data('this'),
          index,
          next,
          first,
          last,
          prev,
          nextPrev,
          prevIndex,
          isActive,
          selector = ':not(.disabled, .hidden, .dropdown-header, .divider)',
          keyCodeMap = {
            32: ' ',
            48: '0',
            49: '1',
            50: '2',
            51: '3',
            52: '4',
            53: '5',
            54: '6',
            55: '7',
            56: '8',
            57: '9',
            59: ';',
            65: 'a',
            66: 'b',
            67: 'c',
            68: 'd',
            69: 'e',
            70: 'f',
            71: 'g',
            72: 'h',
            73: 'i',
            74: 'j',
            75: 'k',
            76: 'l',
            77: 'm',
            78: 'n',
            79: 'o',
            80: 'p',
            81: 'q',
            82: 'r',
            83: 's',
            84: 't',
            85: 'u',
            86: 'v',
            87: 'w',
            88: 'x',
            89: 'y',
            90: 'z',
            96: '0',
            97: '1',
            98: '2',
            99: '3',
            100: '4',
            101: '5',
            102: '6',
            103: '7',
            104: '8',
            105: '9'
          };

      if (that.options.liveSearch) $parent = $this.parent().parent();

      if (that.options.container) $parent = that.$menu;

      $items = $('[role=menu] li', $parent);

      isActive = that.$newElement.hasClass('open');

      if (!isActive && (e.keyCode >= 48 && e.keyCode <= 57 || e.keyCode >= 96 && e.keyCode <= 105 || e.keyCode >= 65 && e.keyCode <= 90)) {
        if (!that.options.container) {
          that.setSize();
          that.$menu.parent().addClass('open');
          isActive = true;
        } else {
          that.$button.trigger('click');
        }
        that.$searchbox.focus();
      }

      if (that.options.liveSearch) {
        if (/(^9$|27)/.test(e.keyCode.toString(10)) && isActive && that.$menu.find('.active').length === 0) {
          e.preventDefault();
          that.$menu.parent().removeClass('open');
          if (that.options.container) that.$newElement.removeClass('open');
          that.$button.focus();
        }
        // $items contains li elements when liveSearch is enabled
        $items = $('[role=menu] li' + selector, $parent);
        if (!$this.val() && !/(38|40)/.test(e.keyCode.toString(10))) {
          if ($items.filter('.active').length === 0) {
            $items = that.$menuInner.find('li');
            if (that.options.liveSearchNormalize) {
              $items = $items.filter(':a' + that._searchStyle() + '(' + normalizeToBase(keyCodeMap[e.keyCode]) + ')');
            } else {
              $items = $items.filter(':' + that._searchStyle() + '(' + keyCodeMap[e.keyCode] + ')');
            }
          }
        }
      }

      if (!$items.length) return;

      if (/(38|40)/.test(e.keyCode.toString(10))) {
        index = $items.index($items.find('a').filter(':focus').parent());
        first = $items.filter(selector).first().index();
        last = $items.filter(selector).last().index();
        next = $items.eq(index).nextAll(selector).eq(0).index();
        prev = $items.eq(index).prevAll(selector).eq(0).index();
        nextPrev = $items.eq(next).prevAll(selector).eq(0).index();

        if (that.options.liveSearch) {
          $items.each(function (i) {
            if (!$(this).hasClass('disabled')) {
              $(this).data('index', i);
            }
          });
          index = $items.index($items.filter('.active'));
          first = $items.first().data('index');
          last = $items.last().data('index');
          next = $items.eq(index).nextAll().eq(0).data('index');
          prev = $items.eq(index).prevAll().eq(0).data('index');
          nextPrev = $items.eq(next).prevAll().eq(0).data('index');
        }

        prevIndex = $this.data('prevIndex');

        if (e.keyCode == 38) {
          if (that.options.liveSearch) index--;
          if (index != nextPrev && index > prev) index = prev;
          if (index < first) index = first;
          if (index == prevIndex) index = last;
        } else if (e.keyCode == 40) {
          if (that.options.liveSearch) index++;
          if (index == -1) index = 0;
          if (index != nextPrev && index < next) index = next;
          if (index > last) index = last;
          if (index == prevIndex) index = first;
        }

        $this.data('prevIndex', index);

        if (!that.options.liveSearch) {
          $items.eq(index).children('a').focus();
        } else {
          e.preventDefault();
          if (!$this.hasClass('dropdown-toggle')) {
            $items.removeClass('active').eq(index).addClass('active').children('a').focus();
            $this.focus();
          }
        }

      } else if (!$this.is('input')) {
        var keyIndex = [],
            count,
            prevKey;

        $items.each(function () {
          if (!$(this).hasClass('disabled')) {
            if ($.trim($(this).children('a').text().toLowerCase()).substring(0, 1) == keyCodeMap[e.keyCode]) {
              keyIndex.push($(this).index());
            }
          }
        });

        count = $(document).data('keycount');
        count++;
        $(document).data('keycount', count);

        prevKey = $.trim($(':focus').text().toLowerCase()).substring(0, 1);

        if (prevKey != keyCodeMap[e.keyCode]) {
          count = 1;
          $(document).data('keycount', count);
        } else if (count >= keyIndex.length) {
          $(document).data('keycount', 0);
          if (count > keyIndex.length) count = 1;
        }

        $items.eq(keyIndex[count - 1]).children('a').focus();
      }

      // Select focused option if "Enter", "Spacebar" or "Tab" (when selectOnTab is true) are pressed inside the menu.
      if ((/(13|32)/.test(e.keyCode.toString(10)) || (/(^9$)/.test(e.keyCode.toString(10)) && that.options.selectOnTab)) && isActive) {
        if (!/(32)/.test(e.keyCode.toString(10))) e.preventDefault();
        if (!that.options.liveSearch) {
          var elem = $(':focus');
          elem.click();
          // Bring back focus for multiselects
          elem.focus();
          // Prevent screen from scrolling if the user hit the spacebar
          e.preventDefault();
          // Fixes spacebar selection of dropdown items in FF & IE
          $(document).data('spaceSelect', true);
        } else if (!/(32)/.test(e.keyCode.toString(10))) {
          that.$menuInner.find('.active a').click();
          $this.focus();
        }
        $(document).data('keycount', 0);
      }

      if ((/(^9$|27)/.test(e.keyCode.toString(10)) && isActive && (that.multiple || that.options.liveSearch)) || (/(27)/.test(e.keyCode.toString(10)) && !isActive)) {
        that.$menu.parent().removeClass('open');
        if (that.options.container) that.$newElement.removeClass('open');
        that.$button.focus();
      }
    },

    mobile: function () {
      this.$element.addClass('mobile-device');
    },

    refresh: function () {
      this.$lis = null;
      this.liObj = {};
      this.reloadLi();
      this.render();
      this.checkDisabled();
      this.liHeight(true);
      this.setStyle();
      this.setWidth();
      if (this.$lis) this.$searchbox.trigger('propertychange');

      this.$element.trigger('refreshed.bs.select');
    },

    hide: function () {
      this.$newElement.hide();
    },

    show: function () {
      this.$newElement.show();
    },

    remove: function () {
      this.$newElement.remove();
      this.$element.remove();
    },

    destroy: function () {
        this.$newElement.before(this.$element).remove();

        if (this.$bsContainer) {
            this.$bsContainer.remove();
        } else {
            this.$menu.remove();
        }

        this.$element
          .off('.bs.select')
          .removeData('selectpicker')
          .removeClass('bs-select-hidden selectpicker');
    }
  };

  // SELECTPICKER PLUGIN DEFINITION
  // ==============================
  function Plugin(option, event) {
    // get the args of the outer function..
    var args = arguments;
    // The arguments of the function are explicitly re-defined from the argument list, because the shift causes them
    // to get lost/corrupted in android 2.3 and IE9 #715 #775
    var _option = option,
        _event = event;
    [].shift.apply(args);

    var value;
    var chain = this.each(function () {
      var $this = $(this);
      if ($this.is('select')) {
        var data = $this.data('selectpicker'),
            options = typeof _option == 'object' && _option;

        if (!data) {
          var config = $.extend({}, Selectpicker.DEFAULTS, $.fn.selectpicker.defaults || {}, $this.data(), options);
          config.template = $.extend({}, Selectpicker.DEFAULTS.template, ($.fn.selectpicker.defaults ? $.fn.selectpicker.defaults.template : {}), $this.data().template, options.template);
          $this.data('selectpicker', (data = new Selectpicker(this, config, _event)));
        } else if (options) {
          for (var i in options) {
            if (options.hasOwnProperty(i)) {
              data.options[i] = options[i];
            }
          }
        }

        if (typeof _option == 'string') {
          if (data[_option] instanceof Function) {
            value = data[_option].apply(data, args);
          } else {
            value = data.options[_option];
          }
        }
      }
    });

    if (typeof value !== 'undefined') {
      //noinspection JSUnusedAssignment
      return value;
    } else {
      return chain;
    }
  }

  var old = $.fn.selectpicker;
  $.fn.selectpicker = Plugin;
  $.fn.selectpicker.Constructor = Selectpicker;

  // SELECTPICKER NO CONFLICT
  // ========================
  $.fn.selectpicker.noConflict = function () {
    $.fn.selectpicker = old;
    return this;
  };

  $(document)
      .data('keycount', 0)
      .on('keydown.bs.select', '.bootstrap-select [data-toggle=dropdown], .bootstrap-select [role="menu"], .bs-searchbox input', Selectpicker.prototype.keydown)
      .on('focusin.modal', '.bootstrap-select [data-toggle=dropdown], .bootstrap-select [role="menu"], .bs-searchbox input', function (e) {
        e.stopPropagation();
      });

  // SELECTPICKER DATA-API
  // =====================
  $(window).on('load.bs.select.data-api', function () {
    $('.selectpicker').each(function () {
      var $selectpicker = $(this);
      Plugin.call($selectpicker, $selectpicker.data());
    })
  });
})(jQuery);


}));

/*! =======================================================
                      VERSION  6.0.17
========================================================= */
"use strict";

function _typeof(obj) { return obj && typeof Symbol !== "undefined" && obj.constructor === Symbol ? "symbol" : typeof obj; }

/*! =========================================================
 * bootstrap-slider.js
 *
 * Maintainers:
 *		Kyle Kemp
 *			- Twitter: @seiyria
 *			- Github:  seiyria
 *		Rohit Kalkur
 *			- Twitter: @Rovolutionary
 *			- Github:  rovolution
 *
 * =========================================================
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ========================================================= */

/**
 * Bridget makes jQuery widgets
 * v1.0.1
 * MIT license
 */

(function (factory) {
	if (typeof define === "function" && define.amd) {
		define(["jquery"], factory);
	} else if ((typeof module === "undefined" ? "undefined" : _typeof(module)) === "object" && module.exports) {
		var jQuery;
		try {
			jQuery = require("jquery");
		} catch (err) {
			jQuery = null;
		}
		module.exports = factory(jQuery);
	} else if (window) {
		window.Slider = factory(window.jQuery);
	}
})(function ($) {
	// Reference to Slider constructor
	var Slider;

	(function ($) {

		'use strict';

		// -------------------------- utils -------------------------- //

		var slice = Array.prototype.slice;

		function noop() {}

		// -------------------------- definition -------------------------- //

		function defineBridget($) {

			// bail if no jQuery
			if (!$) {
				return;
			}

			// -------------------------- addOptionMethod -------------------------- //

			/**
    * adds option method -> $().plugin('option', {...})
    * @param {Function} PluginClass - constructor class
    */
			function addOptionMethod(PluginClass) {
				// don't overwrite original option method
				if (PluginClass.prototype.option) {
					return;
				}

				// option setter
				PluginClass.prototype.option = function (opts) {
					// bail out if not an object
					if (!$.isPlainObject(opts)) {
						return;
					}
					this.options = $.extend(true, this.options, opts);
				};
			}

			// -------------------------- plugin bridge -------------------------- //

			// helper function for logging errors
			// $.error breaks jQuery chaining
			var logError = typeof console === 'undefined' ? noop : function (message) {
				console.error(message);
			};

			/**
    * jQuery plugin bridge, access methods like $elem.plugin('method')
    * @param {String} namespace - plugin name
    * @param {Function} PluginClass - constructor class
    */
			function bridge(namespace, PluginClass) {
				// add to jQuery fn namespace
				$.fn[namespace] = function (options) {
					if (typeof options === 'string') {
						// call plugin method when first argument is a string
						// get arguments for method
						var args = slice.call(arguments, 1);

						for (var i = 0, len = this.length; i < len; i++) {
							var elem = this[i];
							var instance = $.data(elem, namespace);
							if (!instance) {
								logError("cannot call methods on " + namespace + " prior to initialization; " + "attempted to call '" + options + "'");
								continue;
							}
							if (!$.isFunction(instance[options]) || options.charAt(0) === '_') {
								logError("no such method '" + options + "' for " + namespace + " instance");
								continue;
							}

							// trigger method with arguments
							var returnValue = instance[options].apply(instance, args);

							// break look and return first value if provided
							if (returnValue !== undefined && returnValue !== instance) {
								return returnValue;
							}
						}
						// return this if no return value
						return this;
					} else {
						var objects = this.map(function () {
							var instance = $.data(this, namespace);
							if (instance) {
								// apply options & init
								instance.option(options);
								instance._init();
							} else {
								// initialize new instance
								instance = new PluginClass(this, options);
								$.data(this, namespace, instance);
							}
							return $(this);
						});

						if (!objects || objects.length > 1) {
							return objects;
						} else {
							return objects[0];
						}
					}
				};
			}

			// -------------------------- bridget -------------------------- //

			/**
    * converts a Prototypical class into a proper jQuery plugin
    *   the class must have a ._init method
    * @param {String} namespace - plugin name, used in $().pluginName
    * @param {Function} PluginClass - constructor class
    */
			$.bridget = function (namespace, PluginClass) {
				addOptionMethod(PluginClass);
				bridge(namespace, PluginClass);
			};

			return $.bridget;
		}

		// get jquery from browser global
		defineBridget($);
	})($);

	/*************************************************
 			BOOTSTRAP-SLIDER SOURCE CODE
 	**************************************************/

	(function ($) {

		var ErrorMsgs = {
			formatInvalidInputErrorMsg: function formatInvalidInputErrorMsg(input) {
				return "Invalid input value '" + input + "' passed in";
			},
			callingContextNotSliderInstance: "Calling context element does not have instance of Slider bound to it. Check your code to make sure the JQuery object returned from the call to the slider() initializer is calling the method"
		};

		var SliderScale = {
			linear: {
				toValue: function toValue(percentage) {
					var rawValue = percentage / 100 * (this.options.max - this.options.min);
					var shouldAdjustWithBase = true;
					if (this.options.ticks_positions.length > 0) {
						var minv,
						    maxv,
						    minp,
						    maxp = 0;
						for (var i = 1; i < this.options.ticks_positions.length; i++) {
							if (percentage <= this.options.ticks_positions[i]) {
								minv = this.options.ticks[i - 1];
								minp = this.options.ticks_positions[i - 1];
								maxv = this.options.ticks[i];
								maxp = this.options.ticks_positions[i];

								break;
							}
						}
						var partialPercentage = (percentage - minp) / (maxp - minp);
						rawValue = minv + partialPercentage * (maxv - minv);
						shouldAdjustWithBase = false;
					}

					var adjustment = shouldAdjustWithBase ? this.options.min : 0;
					var value = adjustment + Math.round(rawValue / this.options.step) * this.options.step;
					if (value < this.options.min) {
						return this.options.min;
					} else if (value > this.options.max) {
						return this.options.max;
					} else {
						return value;
					}
				},
				toPercentage: function toPercentage(value) {
					if (this.options.max === this.options.min) {
						return 0;
					}

					if (this.options.ticks_positions.length > 0) {
						var minv,
						    maxv,
						    minp,
						    maxp = 0;
						for (var i = 0; i < this.options.ticks.length; i++) {
							if (value <= this.options.ticks[i]) {
								minv = i > 0 ? this.options.ticks[i - 1] : 0;
								minp = i > 0 ? this.options.ticks_positions[i - 1] : 0;
								maxv = this.options.ticks[i];
								maxp = this.options.ticks_positions[i];

								break;
							}
						}
						if (i > 0) {
							var partialPercentage = (value - minv) / (maxv - minv);
							return minp + partialPercentage * (maxp - minp);
						}
					}

					return 100 * (value - this.options.min) / (this.options.max - this.options.min);
				}
			},

			logarithmic: {
				/* Based on http://stackoverflow.com/questions/846221/logarithmic-slider */
				toValue: function toValue(percentage) {
					var min = this.options.min === 0 ? 0 : Math.log(this.options.min);
					var max = Math.log(this.options.max);
					var value = Math.exp(min + (max - min) * percentage / 100);
					value = this.options.min + Math.round((value - this.options.min) / this.options.step) * this.options.step;
					/* Rounding to the nearest step could exceed the min or
      * max, so clip to those values. */
					if (value < this.options.min) {
						return this.options.min;
					} else if (value > this.options.max) {
						return this.options.max;
					} else {
						return value;
					}
				},
				toPercentage: function toPercentage(value) {
					if (this.options.max === this.options.min) {
						return 0;
					} else {
						var max = Math.log(this.options.max);
						var min = this.options.min === 0 ? 0 : Math.log(this.options.min);
						var v = value === 0 ? 0 : Math.log(value);
						return 100 * (v - min) / (max - min);
					}
				}
			}
		};

		/*************************************************
  						CONSTRUCTOR
  	**************************************************/
		Slider = function (element, options) {
			createNewSlider.call(this, element, options);
			return this;
		};

		function createNewSlider(element, options) {

			/*
   	The internal state object is used to store data about the current 'state' of slider.
   		This includes values such as the `value`, `enabled`, etc...
   */
			this._state = {
				value: null,
				enabled: null,
				offset: null,
				size: null,
				percentage: null,
				inDrag: false,
				over: false
			};

			if (typeof element === "string") {
				this.element = document.querySelector(element);
			} else if (element instanceof HTMLElement) {
				this.element = element;
			}

			/*************************************************
   					Process Options
   	**************************************************/
			options = options ? options : {};
			var optionTypes = Object.keys(this.defaultOptions);

			for (var i = 0; i < optionTypes.length; i++) {
				var optName = optionTypes[i];

				// First check if an option was passed in via the constructor
				var val = options[optName];
				// If no data attrib, then check data atrributes
				val = typeof val !== 'undefined' ? val : getDataAttrib(this.element, optName);
				// Finally, if nothing was specified, use the defaults
				val = val !== null ? val : this.defaultOptions[optName];

				// Set all options on the instance of the Slider
				if (!this.options) {
					this.options = {};
				}
				this.options[optName] = val;
			}

			/*
   	Validate `tooltip_position` against 'orientation`
   	- if `tooltip_position` is incompatible with orientation, swith it to a default compatible with specified `orientation`
   		-- default for "vertical" -> "right"
   		-- default for "horizontal" -> "left"
   */
			if (this.options.orientation === "vertical" && (this.options.tooltip_position === "top" || this.options.tooltip_position === "bottom")) {

				this.options.tooltip_position = "right";
			} else if (this.options.orientation === "horizontal" && (this.options.tooltip_position === "left" || this.options.tooltip_position === "right")) {

				this.options.tooltip_position = "top";
			}

			function getDataAttrib(element, optName) {
				var dataName = "data-slider-" + optName.replace(/_/g, '-');
				var dataValString = element.getAttribute(dataName);

				try {
					return JSON.parse(dataValString);
				} catch (err) {
					return dataValString;
				}
			}

			/*************************************************
   					Create Markup
   	**************************************************/

			var origWidth = this.element.style.width;
			var updateSlider = false;
			var parent = this.element.parentNode;
			var sliderTrackSelection;
			var sliderTrackLow, sliderTrackHigh;
			var sliderMinHandle;
			var sliderMaxHandle;

			if (this.sliderElem) {
				updateSlider = true;
			} else {
				/* Create elements needed for slider */
				this.sliderElem = document.createElement("div");
				this.sliderElem.className = "slider";

				/* Create slider track elements */
				var sliderTrack = document.createElement("div");
				sliderTrack.className = "slider-track";

				sliderTrackLow = document.createElement("div");
				sliderTrackLow.className = "slider-track-low";

				sliderTrackSelection = document.createElement("div");
				sliderTrackSelection.className = "slider-selection";

				sliderTrackHigh = document.createElement("div");
				sliderTrackHigh.className = "slider-track-high";

				sliderMinHandle = document.createElement("div");
				sliderMinHandle.className = "slider-handle min-slider-handle";
				sliderMinHandle.setAttribute('role', 'slider');
				sliderMinHandle.setAttribute('aria-valuemin', this.options.min);
				sliderMinHandle.setAttribute('aria-valuemax', this.options.max);

				sliderMaxHandle = document.createElement("div");
				sliderMaxHandle.className = "slider-handle max-slider-handle";
				sliderMaxHandle.setAttribute('role', 'slider');
				sliderMaxHandle.setAttribute('aria-valuemin', this.options.min);
				sliderMaxHandle.setAttribute('aria-valuemax', this.options.max);

				sliderTrack.appendChild(sliderTrackLow);
				sliderTrack.appendChild(sliderTrackSelection);
				sliderTrack.appendChild(sliderTrackHigh);

				/* Add aria-labelledby to handle's */
				var isLabelledbyArray = Array.isArray(this.options.labelledby);
				if (isLabelledbyArray && this.options.labelledby[0]) {
					sliderMinHandle.setAttribute('aria-labelledby', this.options.labelledby[0]);
				}
				if (isLabelledbyArray && this.options.labelledby[1]) {
					sliderMaxHandle.setAttribute('aria-labelledby', this.options.labelledby[1]);
				}
				if (!isLabelledbyArray && this.options.labelledby) {
					sliderMinHandle.setAttribute('aria-labelledby', this.options.labelledby);
					sliderMaxHandle.setAttribute('aria-labelledby', this.options.labelledby);
				}

				/* Create ticks */
				this.ticks = [];
				if (Array.isArray(this.options.ticks) && this.options.ticks.length > 0) {
					for (i = 0; i < this.options.ticks.length; i++) {
						var tick = document.createElement('div');
						tick.className = 'slider-tick';

						this.ticks.push(tick);
						sliderTrack.appendChild(tick);
					}

					sliderTrackSelection.className += " tick-slider-selection";
				}

				sliderTrack.appendChild(sliderMinHandle);
				sliderTrack.appendChild(sliderMaxHandle);

				this.tickLabels = [];
				if (Array.isArray(this.options.ticks_labels) && this.options.ticks_labels.length > 0) {
					this.tickLabelContainer = document.createElement('div');
					this.tickLabelContainer.className = 'slider-tick-label-container';

					for (i = 0; i < this.options.ticks_labels.length; i++) {
						var label = document.createElement('div');
						var noTickPositionsSpecified = this.options.ticks_positions.length === 0;
						var tickLabelsIndex = this.options.reversed && noTickPositionsSpecified ? this.options.ticks_labels.length - (i + 1) : i;
						label.className = 'slider-tick-label';
						label.innerHTML = this.options.ticks_labels[tickLabelsIndex];

						this.tickLabels.push(label);
						this.tickLabelContainer.appendChild(label);
					}
				}

				var createAndAppendTooltipSubElements = function createAndAppendTooltipSubElements(tooltipElem) {
					var arrow = document.createElement("div");
					arrow.className = "tooltip-arrow";

					var inner = document.createElement("div");
					inner.className = "tooltip-inner";

					tooltipElem.appendChild(arrow);
					tooltipElem.appendChild(inner);
				};

				/* Create tooltip elements */
				var sliderTooltip = document.createElement("div");
				sliderTooltip.className = "tooltip tooltip-main";
				sliderTooltip.setAttribute('role', 'presentation');
				createAndAppendTooltipSubElements(sliderTooltip);

				var sliderTooltipMin = document.createElement("div");
				sliderTooltipMin.className = "tooltip tooltip-min";
				sliderTooltipMin.setAttribute('role', 'presentation');
				createAndAppendTooltipSubElements(sliderTooltipMin);

				var sliderTooltipMax = document.createElement("div");
				sliderTooltipMax.className = "tooltip tooltip-max";
				sliderTooltipMax.setAttribute('role', 'presentation');
				createAndAppendTooltipSubElements(sliderTooltipMax);

				/* Append components to sliderElem */
				this.sliderElem.appendChild(sliderTrack);
				this.sliderElem.appendChild(sliderTooltip);
				this.sliderElem.appendChild(sliderTooltipMin);
				this.sliderElem.appendChild(sliderTooltipMax);

				if (this.tickLabelContainer) {
					this.sliderElem.appendChild(this.tickLabelContainer);
				}

				/* Append slider element to parent container, right before the original <input> element */
				parent.insertBefore(this.sliderElem, this.element);

				/* Hide original <input> element */
				this.element.style.display = "none";
			}
			/* If JQuery exists, cache JQ references */
			if ($) {
				this.$element = $(this.element);
				this.$sliderElem = $(this.sliderElem);
			}

			/*************************************************
   						Setup
   	**************************************************/
			this.eventToCallbackMap = {};
			this.sliderElem.id = this.options.id;

			this.touchCapable = 'ontouchstart' in window || window.DocumentTouch && document instanceof window.DocumentTouch;

			this.tooltip = this.sliderElem.querySelector('.tooltip-main');
			this.tooltipInner = this.tooltip.querySelector('.tooltip-inner');

			this.tooltip_min = this.sliderElem.querySelector('.tooltip-min');
			this.tooltipInner_min = this.tooltip_min.querySelector('.tooltip-inner');

			this.tooltip_max = this.sliderElem.querySelector('.tooltip-max');
			this.tooltipInner_max = this.tooltip_max.querySelector('.tooltip-inner');

			if (SliderScale[this.options.scale]) {
				this.options.scale = SliderScale[this.options.scale];
			}

			if (updateSlider === true) {
				// Reset classes
				this._removeClass(this.sliderElem, 'slider-horizontal');
				this._removeClass(this.sliderElem, 'slider-vertical');
				this._removeClass(this.tooltip, 'hide');
				this._removeClass(this.tooltip_min, 'hide');
				this._removeClass(this.tooltip_max, 'hide');

				// Undo existing inline styles for track
				["left", "top", "width", "height"].forEach(function (prop) {
					this._removeProperty(this.trackLow, prop);
					this._removeProperty(this.trackSelection, prop);
					this._removeProperty(this.trackHigh, prop);
				}, this);

				// Undo inline styles on handles
				[this.handle1, this.handle2].forEach(function (handle) {
					this._removeProperty(handle, 'left');
					this._removeProperty(handle, 'top');
				}, this);

				// Undo inline styles and classes on tooltips
				[this.tooltip, this.tooltip_min, this.tooltip_max].forEach(function (tooltip) {
					this._removeProperty(tooltip, 'left');
					this._removeProperty(tooltip, 'top');
					this._removeProperty(tooltip, 'margin-left');
					this._removeProperty(tooltip, 'margin-top');

					this._removeClass(tooltip, 'right');
					this._removeClass(tooltip, 'top');
				}, this);
			}

			if (this.options.orientation === 'vertical') {
				this._addClass(this.sliderElem, 'slider-vertical');
				this.stylePos = 'top';
				this.mousePos = 'pageY';
				this.sizePos = 'offsetHeight';
			} else {
				this._addClass(this.sliderElem, 'slider-horizontal');
				this.sliderElem.style.width = origWidth;
				this.options.orientation = 'horizontal';
				this.stylePos = 'left';
				this.mousePos = 'pageX';
				this.sizePos = 'offsetWidth';
			}
			this._setTooltipPosition();
			/* In case ticks are specified, overwrite the min and max bounds */
			if (Array.isArray(this.options.ticks) && this.options.ticks.length > 0) {
				this.options.max = Math.max.apply(Math, this.options.ticks);
				this.options.min = Math.min.apply(Math, this.options.ticks);
			}

			if (Array.isArray(this.options.value)) {
				this.options.range = true;
				this._state.value = this.options.value;
			} else if (this.options.range) {
				// User wants a range, but value is not an array
				this._state.value = [this.options.value, this.options.max];
			} else {
				this._state.value = this.options.value;
			}

			this.trackLow = sliderTrackLow || this.trackLow;
			this.trackSelection = sliderTrackSelection || this.trackSelection;
			this.trackHigh = sliderTrackHigh || this.trackHigh;

			if (this.options.selection === 'none') {
				this._addClass(this.trackLow, 'hide');
				this._addClass(this.trackSelection, 'hide');
				this._addClass(this.trackHigh, 'hide');
			}

			this.handle1 = sliderMinHandle || this.handle1;
			this.handle2 = sliderMaxHandle || this.handle2;

			if (updateSlider === true) {
				// Reset classes
				this._removeClass(this.handle1, 'round triangle');
				this._removeClass(this.handle2, 'round triangle hide');

				for (i = 0; i < this.ticks.length; i++) {
					this._removeClass(this.ticks[i], 'round triangle hide');
				}
			}

			var availableHandleModifiers = ['round', 'triangle', 'custom'];
			var isValidHandleType = availableHandleModifiers.indexOf(this.options.handle) !== -1;
			if (isValidHandleType) {
				this._addClass(this.handle1, this.options.handle);
				this._addClass(this.handle2, this.options.handle);

				for (i = 0; i < this.ticks.length; i++) {
					this._addClass(this.ticks[i], this.options.handle);
				}
			}

			this._state.offset = this._offset(this.sliderElem);
			this._state.size = this.sliderElem[this.sizePos];
			this.setValue(this._state.value);

			/******************************************
   				Bind Event Listeners
   	******************************************/

			// Bind keyboard handlers
			this.handle1Keydown = this._keydown.bind(this, 0);
			this.handle1.addEventListener("keydown", this.handle1Keydown, false);

			this.handle2Keydown = this._keydown.bind(this, 1);
			this.handle2.addEventListener("keydown", this.handle2Keydown, false);

			this.mousedown = this._mousedown.bind(this);
			if (this.touchCapable) {
				// Bind touch handlers
				this.sliderElem.addEventListener("touchstart", this.mousedown, false);
			}
			this.sliderElem.addEventListener("mousedown", this.mousedown, false);

			// Bind window handlers
			this.resize = this._resize.bind(this);
			window.addEventListener("resize", this.resize, false);

			// Bind tooltip-related handlers
			if (this.options.tooltip === 'hide') {
				this._addClass(this.tooltip, 'hide');
				this._addClass(this.tooltip_min, 'hide');
				this._addClass(this.tooltip_max, 'hide');
			} else if (this.options.tooltip === 'always') {
				this._showTooltip();
				this._alwaysShowTooltip = true;
			} else {
				this.showTooltip = this._showTooltip.bind(this);
				this.hideTooltip = this._hideTooltip.bind(this);

				this.sliderElem.addEventListener("mouseenter", this.showTooltip, false);
				this.sliderElem.addEventListener("mouseleave", this.hideTooltip, false);

				this.handle1.addEventListener("focus", this.showTooltip, false);
				this.handle1.addEventListener("blur", this.hideTooltip, false);

				this.handle2.addEventListener("focus", this.showTooltip, false);
				this.handle2.addEventListener("blur", this.hideTooltip, false);
			}

			if (this.options.enabled) {
				this.enable();
			} else {
				this.disable();
			}
		}

		/*************************************************
  				INSTANCE PROPERTIES/METHODS
  	- Any methods bound to the prototype are considered
  part of the plugin's `public` interface
  	**************************************************/
		Slider.prototype = {
			_init: function _init() {}, // NOTE: Must exist to support bridget

			constructor: Slider,

			defaultOptions: {
				id: "",
				min: 0,
				max: 10,
				step: 1,
				precision: 0,
				orientation: 'horizontal',
				value: 5,
				range: false,
				selection: 'before',
				tooltip: 'show',
				tooltip_split: false,
				handle: 'round',
				reversed: false,
				enabled: true,
				formatter: function formatter(val) {
					if (Array.isArray(val)) {
						return val[0] + " : " + val[1];
					} else {
						return val;
					}
				},
				natural_arrow_keys: false,
				ticks: [],
				ticks_positions: [],
				ticks_labels: [],
				ticks_snap_bounds: 0,
				scale: 'linear',
				focus: false,
				tooltip_position: null,
				labelledby: null
			},

			getElement: function getElement() {
				return this.sliderElem;
			},

			getValue: function getValue() {
				if (this.options.range) {
					return this._state.value;
				} else {
					return this._state.value[0];
				}
			},

			setValue: function setValue(val, triggerSlideEvent, triggerChangeEvent) {
				if (!val) {
					val = 0;
				}
				var oldValue = this.getValue();
				this._state.value = this._validateInputValue(val);
				var applyPrecision = this._applyPrecision.bind(this);

				if (this.options.range) {
					this._state.value[0] = applyPrecision(this._state.value[0]);
					this._state.value[1] = applyPrecision(this._state.value[1]);

					this._state.value[0] = Math.max(this.options.min, Math.min(this.options.max, this._state.value[0]));
					this._state.value[1] = Math.max(this.options.min, Math.min(this.options.max, this._state.value[1]));
				} else {
					this._state.value = applyPrecision(this._state.value);
					this._state.value = [Math.max(this.options.min, Math.min(this.options.max, this._state.value))];
					this._addClass(this.handle2, 'hide');
					if (this.options.selection === 'after') {
						this._state.value[1] = this.options.max;
					} else {
						this._state.value[1] = this.options.min;
					}
				}

				if (this.options.max > this.options.min) {
					this._state.percentage = [this._toPercentage(this._state.value[0]), this._toPercentage(this._state.value[1]), this.options.step * 100 / (this.options.max - this.options.min)];
				} else {
					this._state.percentage = [0, 0, 100];
				}

				this._layout();
				var newValue = this.options.range ? this._state.value : this._state.value[0];

				this._setDataVal(newValue);
				if (triggerSlideEvent === true) {
					this._trigger('slide', newValue);
				}
				if (oldValue !== newValue && triggerChangeEvent === true) {
					this._trigger('change', {
						oldValue: oldValue,
						newValue: newValue
					});
				}

				return this;
			},

			destroy: function destroy() {
				// Remove event handlers on slider elements
				this._removeSliderEventHandlers();

				// Remove the slider from the DOM
				this.sliderElem.parentNode.removeChild(this.sliderElem);
				/* Show original <input> element */
				this.element.style.display = "";

				// Clear out custom event bindings
				this._cleanUpEventCallbacksMap();

				// Remove data values
				this.element.removeAttribute("data");

				// Remove JQuery handlers/data
				if ($) {
					this._unbindJQueryEventHandlers();
					this.$element.removeData('slider');
				}
			},

			disable: function disable() {
				this._state.enabled = false;
				this.handle1.removeAttribute("tabindex");
				this.handle2.removeAttribute("tabindex");
				this._addClass(this.sliderElem, 'slider-disabled');
				this._trigger('slideDisabled');

				return this;
			},

			enable: function enable() {
				this._state.enabled = true;
				this.handle1.setAttribute("tabindex", 0);
				this.handle2.setAttribute("tabindex", 0);
				this._removeClass(this.sliderElem, 'slider-disabled');
				this._trigger('slideEnabled');

				return this;
			},

			toggle: function toggle() {
				if (this._state.enabled) {
					this.disable();
				} else {
					this.enable();
				}
				return this;
			},

			isEnabled: function isEnabled() {
				return this._state.enabled;
			},

			on: function on(evt, callback) {
				this._bindNonQueryEventHandler(evt, callback);
				return this;
			},

			off: function off(evt, callback) {
				if ($) {
					this.$element.off(evt, callback);
					this.$sliderElem.off(evt, callback);
				} else {
					this._unbindNonQueryEventHandler(evt, callback);
				}
			},

			getAttribute: function getAttribute(attribute) {
				if (attribute) {
					return this.options[attribute];
				} else {
					return this.options;
				}
			},

			setAttribute: function setAttribute(attribute, value) {
				this.options[attribute] = value;
				return this;
			},

			refresh: function refresh() {
				this._removeSliderEventHandlers();
				createNewSlider.call(this, this.element, this.options);
				if ($) {
					// Bind new instance of slider to the element
					$.data(this.element, 'slider', this);
				}
				return this;
			},

			relayout: function relayout() {
				this._layout();
				return this;
			},

			/******************************+
   				HELPERS
   	- Any method that is not part of the public interface.
   - Place it underneath this comment block and write its signature like so:
   	  					_fnName : function() {...}
   	********************************/
			_removeSliderEventHandlers: function _removeSliderEventHandlers() {
				// Remove keydown event listeners
				this.handle1.removeEventListener("keydown", this.handle1Keydown, false);
				this.handle2.removeEventListener("keydown", this.handle2Keydown, false);

				if (this.showTooltip) {
					this.handle1.removeEventListener("focus", this.showTooltip, false);
					this.handle2.removeEventListener("focus", this.showTooltip, false);
				}
				if (this.hideTooltip) {
					this.handle1.removeEventListener("blur", this.hideTooltip, false);
					this.handle2.removeEventListener("blur", this.hideTooltip, false);
				}

				// Remove event listeners from sliderElem
				if (this.showTooltip) {
					this.sliderElem.removeEventListener("mouseenter", this.showTooltip, false);
				}
				if (this.hideTooltip) {
					this.sliderElem.removeEventListener("mouseleave", this.hideTooltip, false);
				}
				this.sliderElem.removeEventListener("touchstart", this.mousedown, false);
				this.sliderElem.removeEventListener("mousedown", this.mousedown, false);

				// Remove window event listener
				window.removeEventListener("resize", this.resize, false);
			},
			_bindNonQueryEventHandler: function _bindNonQueryEventHandler(evt, callback) {
				if (this.eventToCallbackMap[evt] === undefined) {
					this.eventToCallbackMap[evt] = [];
				}
				this.eventToCallbackMap[evt].push(callback);
			},
			_unbindNonQueryEventHandler: function _unbindNonQueryEventHandler(evt, callback) {
				var callbacks = this.eventToCallbackMap[evt];
				if (callbacks !== undefined) {
					for (var i = 0; i < callbacks.length; i++) {
						if (callbacks[i] === callback) {
							callbacks.splice(i, 1);
							break;
						}
					}
				}
			},
			_cleanUpEventCallbacksMap: function _cleanUpEventCallbacksMap() {
				var eventNames = Object.keys(this.eventToCallbackMap);
				for (var i = 0; i < eventNames.length; i++) {
					var eventName = eventNames[i];
					this.eventToCallbackMap[eventName] = null;
				}
			},
			_showTooltip: function _showTooltip() {
				if (this.options.tooltip_split === false) {
					this._addClass(this.tooltip, 'in');
					this.tooltip_min.style.display = 'none';
					this.tooltip_max.style.display = 'none';
				} else {
					this._addClass(this.tooltip_min, 'in');
					this._addClass(this.tooltip_max, 'in');
					this.tooltip.style.display = 'none';
				}
				this._state.over = true;
			},
			_hideTooltip: function _hideTooltip() {
				if (this._state.inDrag === false && this.alwaysShowTooltip !== true) {
					this._removeClass(this.tooltip, 'in');
					this._removeClass(this.tooltip_min, 'in');
					this._removeClass(this.tooltip_max, 'in');
				}
				this._state.over = false;
			},
			_layout: function _layout() {
				var positionPercentages;

				if (this.options.reversed) {
					positionPercentages = [100 - this._state.percentage[0], this.options.range ? 100 - this._state.percentage[1] : this._state.percentage[1]];
				} else {
					positionPercentages = [this._state.percentage[0], this._state.percentage[1]];
				}

				this.handle1.style[this.stylePos] = positionPercentages[0] + '%';
				this.handle1.setAttribute('aria-valuenow', this._state.value[0]);

				this.handle2.style[this.stylePos] = positionPercentages[1] + '%';
				this.handle2.setAttribute('aria-valuenow', this._state.value[1]);

				/* Position ticks and labels */
				if (Array.isArray(this.options.ticks) && this.options.ticks.length > 0) {

					var styleSize = this.options.orientation === 'vertical' ? 'height' : 'width';
					var styleMargin = this.options.orientation === 'vertical' ? 'marginTop' : 'marginLeft';
					var labelSize = this._state.size / (this.options.ticks.length - 1);

					if (this.tickLabelContainer) {
						var extraMargin = 0;
						if (this.options.ticks_positions.length === 0) {
							if (this.options.orientation !== 'vertical') {
								this.tickLabelContainer.style[styleMargin] = -labelSize / 2 + 'px';
							}

							extraMargin = this.tickLabelContainer.offsetHeight;
						} else {
							/* Chidren are position absolute, calculate height by finding the max offsetHeight of a child */
							for (i = 0; i < this.tickLabelContainer.childNodes.length; i++) {
								if (this.tickLabelContainer.childNodes[i].offsetHeight > extraMargin) {
									extraMargin = this.tickLabelContainer.childNodes[i].offsetHeight;
								}
							}
						}
						if (this.options.orientation === 'horizontal') {
							this.sliderElem.style.marginBottom = extraMargin + 'px';
						}
					}
					for (var i = 0; i < this.options.ticks.length; i++) {

						var percentage = this.options.ticks_positions[i] || this._toPercentage(this.options.ticks[i]);

						if (this.options.reversed) {
							percentage = 100 - percentage;
						}

						this.ticks[i].style[this.stylePos] = percentage + '%';

						/* Set class labels to denote whether ticks are in the selection */
						this._removeClass(this.ticks[i], 'in-selection');
						if (!this.options.range) {
							if (this.options.selection === 'after' && percentage >= positionPercentages[0]) {
								this._addClass(this.ticks[i], 'in-selection');
							} else if (this.options.selection === 'before' && percentage <= positionPercentages[0]) {
								this._addClass(this.ticks[i], 'in-selection');
							}
						} else if (percentage >= positionPercentages[0] && percentage <= positionPercentages[1]) {
							this._addClass(this.ticks[i], 'in-selection');
						}

						if (this.tickLabels[i]) {
							this.tickLabels[i].style[styleSize] = labelSize + 'px';

							if (this.options.orientation !== 'vertical' && this.options.ticks_positions[i] !== undefined) {
								this.tickLabels[i].style.position = 'absolute';
								this.tickLabels[i].style[this.stylePos] = percentage + '%';
								this.tickLabels[i].style[styleMargin] = -labelSize / 2 + 'px';
							} else if (this.options.orientation === 'vertical') {
								this.tickLabels[i].style['marginLeft'] = this.sliderElem.offsetWidth + 'px';
								this.tickLabelContainer.style['marginTop'] = this.sliderElem.offsetWidth / 2 * -1 + 'px';
							}
						}
					}
				}

				var formattedTooltipVal;

				if (this.options.range) {
					formattedTooltipVal = this.options.formatter(this._state.value);
					this._setText(this.tooltipInner, formattedTooltipVal);
					this.tooltip.style[this.stylePos] = (positionPercentages[1] + positionPercentages[0]) / 2 + '%';

					if (this.options.orientation === 'vertical') {
						this._css(this.tooltip, 'margin-top', -this.tooltip.offsetHeight / 2 + 'px');
					} else {
						this._css(this.tooltip, 'margin-left', -this.tooltip.offsetWidth / 2 + 'px');
					}

					if (this.options.orientation === 'vertical') {
						this._css(this.tooltip, 'margin-top', -this.tooltip.offsetHeight / 2 + 'px');
					} else {
						this._css(this.tooltip, 'margin-left', -this.tooltip.offsetWidth / 2 + 'px');
					}

					var innerTooltipMinText = this.options.formatter(this._state.value[0]);
					this._setText(this.tooltipInner_min, innerTooltipMinText);

					var innerTooltipMaxText = this.options.formatter(this._state.value[1]);
					this._setText(this.tooltipInner_max, innerTooltipMaxText);

					this.tooltip_min.style[this.stylePos] = positionPercentages[0] + '%';

					if (this.options.orientation === 'vertical') {
						this._css(this.tooltip_min, 'margin-top', -this.tooltip_min.offsetHeight / 2 + 'px');
					} else {
						this._css(this.tooltip_min, 'margin-left', -this.tooltip_min.offsetWidth / 2 + 'px');
					}

					this.tooltip_max.style[this.stylePos] = positionPercentages[1] + '%';

					if (this.options.orientation === 'vertical') {
						this._css(this.tooltip_max, 'margin-top', -this.tooltip_max.offsetHeight / 2 + 'px');
					} else {
						this._css(this.tooltip_max, 'margin-left', -this.tooltip_max.offsetWidth / 2 + 'px');
					}
				} else {
					formattedTooltipVal = this.options.formatter(this._state.value[0]);
					this._setText(this.tooltipInner, formattedTooltipVal);

					this.tooltip.style[this.stylePos] = positionPercentages[0] + '%';
					if (this.options.orientation === 'vertical') {
						this._css(this.tooltip, 'margin-top', -this.tooltip.offsetHeight / 2 + 'px');
					} else {
						this._css(this.tooltip, 'margin-left', -this.tooltip.offsetWidth / 2 + 'px');
					}
				}

				if (this.options.orientation === 'vertical') {
					this.trackLow.style.top = '0';
					this.trackLow.style.height = Math.min(positionPercentages[0], positionPercentages[1]) + '%';

					this.trackSelection.style.top = Math.min(positionPercentages[0], positionPercentages[1]) + '%';
					this.trackSelection.style.height = Math.abs(positionPercentages[0] - positionPercentages[1]) + '%';

					this.trackHigh.style.bottom = '0';
					this.trackHigh.style.height = 100 - Math.min(positionPercentages[0], positionPercentages[1]) - Math.abs(positionPercentages[0] - positionPercentages[1]) + '%';
				} else {
					this.trackLow.style.left = '0';
					this.trackLow.style.width = Math.min(positionPercentages[0], positionPercentages[1]) + '%';

					this.trackSelection.style.left = Math.min(positionPercentages[0], positionPercentages[1]) + '%';
					this.trackSelection.style.width = Math.abs(positionPercentages[0] - positionPercentages[1]) + '%';

					this.trackHigh.style.right = '0';
					this.trackHigh.style.width = 100 - Math.min(positionPercentages[0], positionPercentages[1]) - Math.abs(positionPercentages[0] - positionPercentages[1]) + '%';

					var offset_min = this.tooltip_min.getBoundingClientRect();
					var offset_max = this.tooltip_max.getBoundingClientRect();

					if (this.options.tooltip_position === 'bottom') {
						if (offset_min.right > offset_max.left) {
							this._removeClass(this.tooltip_max, 'bottom');
							this._addClass(this.tooltip_max, 'top');
							this.tooltip_max.style.top = '';
							this.tooltip_max.style.bottom = 22 + 'px';
						} else {
							this._removeClass(this.tooltip_max, 'top');
							this._addClass(this.tooltip_max, 'bottom');
							this.tooltip_max.style.top = this.tooltip_min.style.top;
							this.tooltip_max.style.bottom = '';
						}
					} else {
						if (offset_min.right > offset_max.left) {
							this._removeClass(this.tooltip_max, 'top');
							this._addClass(this.tooltip_max, 'bottom');
							this.tooltip_max.style.top = 18 + 'px';
						} else {
							this._removeClass(this.tooltip_max, 'bottom');
							this._addClass(this.tooltip_max, 'top');
							this.tooltip_max.style.top = this.tooltip_min.style.top;
						}
					}
				}
			},
			_resize: function _resize(ev) {
				/*jshint unused:false*/
				this._state.offset = this._offset(this.sliderElem);
				this._state.size = this.sliderElem[this.sizePos];
				this._layout();
			},
			_removeProperty: function _removeProperty(element, prop) {
				if (element.style.removeProperty) {
					element.style.removeProperty(prop);
				} else {
					element.style.removeAttribute(prop);
				}
			},
			_mousedown: function _mousedown(ev) {
				if (!this._state.enabled) {
					return false;
				}

				this._state.offset = this._offset(this.sliderElem);
				this._state.size = this.sliderElem[this.sizePos];

				var percentage = this._getPercentage(ev);

				if (this.options.range) {
					var diff1 = Math.abs(this._state.percentage[0] - percentage);
					var diff2 = Math.abs(this._state.percentage[1] - percentage);
					this._state.dragged = diff1 < diff2 ? 0 : 1;
				} else {
					this._state.dragged = 0;
				}

				this._state.percentage[this._state.dragged] = percentage;
				this._layout();

				if (this.touchCapable) {
					document.removeEventListener("touchmove", this.mousemove, false);
					document.removeEventListener("touchend", this.mouseup, false);
				}

				if (this.mousemove) {
					document.removeEventListener("mousemove", this.mousemove, false);
				}
				if (this.mouseup) {
					document.removeEventListener("mouseup", this.mouseup, false);
				}

				this.mousemove = this._mousemove.bind(this);
				this.mouseup = this._mouseup.bind(this);

				if (this.touchCapable) {
					// Touch: Bind touch events:
					document.addEventListener("touchmove", this.mousemove, false);
					document.addEventListener("touchend", this.mouseup, false);
				}
				// Bind mouse events:
				document.addEventListener("mousemove", this.mousemove, false);
				document.addEventListener("mouseup", this.mouseup, false);

				this._state.inDrag = true;
				var newValue = this._calculateValue();

				this._trigger('slideStart', newValue);

				this._setDataVal(newValue);
				this.setValue(newValue, false, true);

				this._pauseEvent(ev);

				if (this.options.focus) {
					this._triggerFocusOnHandle(this._state.dragged);
				}

				return true;
			},
			_triggerFocusOnHandle: function _triggerFocusOnHandle(handleIdx) {
				if (handleIdx === 0) {
					this.handle1.focus();
				}
				if (handleIdx === 1) {
					this.handle2.focus();
				}
			},
			_keydown: function _keydown(handleIdx, ev) {
				if (!this._state.enabled) {
					return false;
				}

				var dir;
				switch (ev.keyCode) {
					case 37: // left
					case 40:
						// down
						dir = -1;
						break;
					case 39: // right
					case 38:
						// up
						dir = 1;
						break;
				}
				if (!dir) {
					return;
				}

				// use natural arrow keys instead of from min to max
				if (this.options.natural_arrow_keys) {
					var ifVerticalAndNotReversed = this.options.orientation === 'vertical' && !this.options.reversed;
					var ifHorizontalAndReversed = this.options.orientation === 'horizontal' && this.options.reversed;

					if (ifVerticalAndNotReversed || ifHorizontalAndReversed) {
						dir = -dir;
					}
				}

				var val = this._state.value[handleIdx] + dir * this.options.step;
				if (this.options.range) {
					val = [!handleIdx ? val : this._state.value[0], handleIdx ? val : this._state.value[1]];
				}

				this._trigger('slideStart', val);
				this._setDataVal(val);
				this.setValue(val, true, true);

				this._setDataVal(val);
				this._trigger('slideStop', val);
				this._layout();

				this._pauseEvent(ev);

				return false;
			},
			_pauseEvent: function _pauseEvent(ev) {
				if (ev.stopPropagation) {
					ev.stopPropagation();
				}
				if (ev.preventDefault) {
					ev.preventDefault();
				}
				ev.cancelBubble = true;
				ev.returnValue = false;
			},
			_mousemove: function _mousemove(ev) {
				if (!this._state.enabled) {
					return false;
				}

				var percentage = this._getPercentage(ev);
				this._adjustPercentageForRangeSliders(percentage);
				this._state.percentage[this._state.dragged] = percentage;
				this._layout();

				var val = this._calculateValue(true);
				this.setValue(val, true, true);

				return false;
			},
			_adjustPercentageForRangeSliders: function _adjustPercentageForRangeSliders(percentage) {
				if (this.options.range) {
					var precision = this._getNumDigitsAfterDecimalPlace(percentage);
					precision = precision ? precision - 1 : 0;
					var percentageWithAdjustedPrecision = this._applyToFixedAndParseFloat(percentage, precision);
					if (this._state.dragged === 0 && this._applyToFixedAndParseFloat(this._state.percentage[1], precision) < percentageWithAdjustedPrecision) {
						this._state.percentage[0] = this._state.percentage[1];
						this._state.dragged = 1;
					} else if (this._state.dragged === 1 && this._applyToFixedAndParseFloat(this._state.percentage[0], precision) > percentageWithAdjustedPrecision) {
						this._state.percentage[1] = this._state.percentage[0];
						this._state.dragged = 0;
					}
				}
			},
			_mouseup: function _mouseup() {
				if (!this._state.enabled) {
					return false;
				}
				if (this.touchCapable) {
					// Touch: Unbind touch event handlers:
					document.removeEventListener("touchmove", this.mousemove, false);
					document.removeEventListener("touchend", this.mouseup, false);
				}
				// Unbind mouse event handlers:
				document.removeEventListener("mousemove", this.mousemove, false);
				document.removeEventListener("mouseup", this.mouseup, false);

				this._state.inDrag = false;
				if (this._state.over === false) {
					this._hideTooltip();
				}
				var val = this._calculateValue(true);

				this._layout();
				this._setDataVal(val);
				this._trigger('slideStop', val);

				return false;
			},
			_calculateValue: function _calculateValue(snapToClosestTick) {
				var val;
				if (this.options.range) {
					val = [this.options.min, this.options.max];
					if (this._state.percentage[0] !== 0) {
						val[0] = this._toValue(this._state.percentage[0]);
						val[0] = this._applyPrecision(val[0]);
					}
					if (this._state.percentage[1] !== 100) {
						val[1] = this._toValue(this._state.percentage[1]);
						val[1] = this._applyPrecision(val[1]);
					}
				} else {
					val = this._toValue(this._state.percentage[0]);
					val = parseFloat(val);
					val = this._applyPrecision(val);
				}

				if (snapToClosestTick) {
					var min = [val, Infinity];
					for (var i = 0; i < this.options.ticks.length; i++) {
						var diff = Math.abs(this.options.ticks[i] - val);
						if (diff <= min[1]) {
							min = [this.options.ticks[i], diff];
						}
					}
					if (min[1] <= this.options.ticks_snap_bounds) {
						return min[0];
					}
				}

				return val;
			},
			_applyPrecision: function _applyPrecision(val) {
				var precision = this.options.precision || this._getNumDigitsAfterDecimalPlace(this.options.step);
				return this._applyToFixedAndParseFloat(val, precision);
			},
			_getNumDigitsAfterDecimalPlace: function _getNumDigitsAfterDecimalPlace(num) {
				var match = ('' + num).match(/(?:\.(\d+))?(?:[eE]([+-]?\d+))?$/);
				if (!match) {
					return 0;
				}
				return Math.max(0, (match[1] ? match[1].length : 0) - (match[2] ? +match[2] : 0));
			},
			_applyToFixedAndParseFloat: function _applyToFixedAndParseFloat(num, toFixedInput) {
				var truncatedNum = num.toFixed(toFixedInput);
				return parseFloat(truncatedNum);
			},
			/*
   	Credits to Mike Samuel for the following method!
   	Source: http://stackoverflow.com/questions/10454518/javascript-how-to-retrieve-the-number-of-decimals-of-a-string-number
   */
			_getPercentage: function _getPercentage(ev) {
				if (this.touchCapable && (ev.type === 'touchstart' || ev.type === 'touchmove')) {
					ev = ev.touches[0];
				}

				var eventPosition = ev[this.mousePos];
				var sliderOffset = this._state.offset[this.stylePos];
				var distanceToSlide = eventPosition - sliderOffset;
				// Calculate what percent of the length the slider handle has slid
				var percentage = distanceToSlide / this._state.size * 100;
				percentage = Math.round(percentage / this._state.percentage[2]) * this._state.percentage[2];
				if (this.options.reversed) {
					percentage = 100 - percentage;
				}

				// Make sure the percent is within the bounds of the slider.
				// 0% corresponds to the 'min' value of the slide
				// 100% corresponds to the 'max' value of the slide
				return Math.max(0, Math.min(100, percentage));
			},
			_validateInputValue: function _validateInputValue(val) {
				if (typeof val === 'number') {
					return val;
				} else if (Array.isArray(val)) {
					this._validateArray(val);
					return val;
				} else {
					throw new Error(ErrorMsgs.formatInvalidInputErrorMsg(val));
				}
			},
			_validateArray: function _validateArray(val) {
				for (var i = 0; i < val.length; i++) {
					var input = val[i];
					if (typeof input !== 'number') {
						throw new Error(ErrorMsgs.formatInvalidInputErrorMsg(input));
					}
				}
			},
			_setDataVal: function _setDataVal(val) {
				this.element.setAttribute('data-value', val);
				this.element.setAttribute('value', val);
				this.element.value = val;
			},
			_trigger: function _trigger(evt, val) {
				val = val || val === 0 ? val : undefined;

				var callbackFnArray = this.eventToCallbackMap[evt];
				if (callbackFnArray && callbackFnArray.length) {
					for (var i = 0; i < callbackFnArray.length; i++) {
						var callbackFn = callbackFnArray[i];
						callbackFn(val);
					}
				}

				/* If JQuery exists, trigger JQuery events */
				if ($) {
					this._triggerJQueryEvent(evt, val);
				}
			},
			_triggerJQueryEvent: function _triggerJQueryEvent(evt, val) {
				var eventData = {
					type: evt,
					value: val
				};
				this.$element.trigger(eventData);
				this.$sliderElem.trigger(eventData);
			},
			_unbindJQueryEventHandlers: function _unbindJQueryEventHandlers() {
				this.$element.off();
				this.$sliderElem.off();
			},
			_setText: function _setText(element, text) {
				if (typeof element.textContent !== "undefined") {
					element.textContent = text;
				} else if (typeof element.innerText !== "undefined") {
					element.innerText = text;
				}
			},
			_removeClass: function _removeClass(element, classString) {
				var classes = classString.split(" ");
				var newClasses = element.className;

				for (var i = 0; i < classes.length; i++) {
					var classTag = classes[i];
					var regex = new RegExp("(?:\\s|^)" + classTag + "(?:\\s|$)");
					newClasses = newClasses.replace(regex, " ");
				}

				element.className = newClasses.trim();
			},
			_addClass: function _addClass(element, classString) {
				var classes = classString.split(" ");
				var newClasses = element.className;

				for (var i = 0; i < classes.length; i++) {
					var classTag = classes[i];
					var regex = new RegExp("(?:\\s|^)" + classTag + "(?:\\s|$)");
					var ifClassExists = regex.test(newClasses);

					if (!ifClassExists) {
						newClasses += " " + classTag;
					}
				}

				element.className = newClasses.trim();
			},
			_offsetLeft: function _offsetLeft(obj) {
				return obj.getBoundingClientRect().left;
			},
			_offsetTop: function _offsetTop(obj) {
				var offsetTop = obj.offsetTop;
				while ((obj = obj.offsetParent) && !isNaN(obj.offsetTop)) {
					offsetTop += obj.offsetTop;
					if (obj.tagName !== 'BODY') {
						offsetTop -= obj.scrollTop;
					}
				}
				return offsetTop;
			},
			_offset: function _offset(obj) {
				return {
					left: this._offsetLeft(obj),
					top: this._offsetTop(obj)
				};
			},
			_css: function _css(elementRef, styleName, value) {
				if ($) {
					$.style(elementRef, styleName, value);
				} else {
					var style = styleName.replace(/^-ms-/, "ms-").replace(/-([\da-z])/gi, function (all, letter) {
						return letter.toUpperCase();
					});
					elementRef.style[style] = value;
				}
			},
			_toValue: function _toValue(percentage) {
				return this.options.scale.toValue.apply(this, [percentage]);
			},
			_toPercentage: function _toPercentage(value) {
				return this.options.scale.toPercentage.apply(this, [value]);
			},
			_setTooltipPosition: function _setTooltipPosition() {
				var tooltips = [this.tooltip, this.tooltip_min, this.tooltip_max];
				if (this.options.orientation === 'vertical') {
					var tooltipPos = this.options.tooltip_position || 'right';
					var oppositeSide = tooltipPos === 'left' ? 'right' : 'left';
					tooltips.forEach((function (tooltip) {
						this._addClass(tooltip, tooltipPos);
						tooltip.style[oppositeSide] = '100%';
					}).bind(this));
				} else if (this.options.tooltip_position === 'bottom') {
					tooltips.forEach((function (tooltip) {
						this._addClass(tooltip, 'bottom');
						tooltip.style.top = 22 + 'px';
					}).bind(this));
				} else {
					tooltips.forEach((function (tooltip) {
						this._addClass(tooltip, 'top');
						tooltip.style.top = -this.tooltip.outerHeight - 14 + 'px';
					}).bind(this));
				}
			}
		};

		/*********************************
  		Attach to global namespace
  	*********************************/
		if ($) {
			var namespace = $.fn.slider ? 'bootstrapSlider' : 'slider';
			$.bridget(namespace, Slider);
		}
	})($);

	return Slider;
});

/*!
 * Bootstrap v3.3.6 (http://getbootstrap.com)
 * Copyright 2011-2015 Twitter, Inc.
 * Licensed under the MIT license
 */

if (typeof jQuery === 'undefined') {
  throw new Error('Bootstrap\'s JavaScript requires jQuery')
}

+function ($) {
  'use strict';
  var version = $.fn.jquery.split(' ')[0].split('.')
  if ((version[0] < 2 && version[1] < 9) || (version[0] == 1 && version[1] == 9 && version[2] < 1) || (version[0] > 2)) {
    throw new Error('Bootstrap\'s JavaScript requires jQuery version 1.9.1 or higher, but lower than version 3')
  }
}(jQuery);

/* ========================================================================
 * Bootstrap: transition.js v3.3.6
 * http://getbootstrap.com/javascript/#transitions
 * ========================================================================
 * Copyright 2011-2015 Twitter, Inc.
 * Licensed under MIT (https://github.com/twbs/bootstrap/blob/master/LICENSE)
 * ======================================================================== */


+function ($) {
  'use strict';

  // CSS TRANSITION SUPPORT (Shoutout: http://www.modernizr.com/)
  // ============================================================

  function transitionEnd() {
    var el = document.createElement('bootstrap')

    var transEndEventNames = {
      WebkitTransition : 'webkitTransitionEnd',
      MozTransition    : 'transitionend',
      OTransition      : 'oTransitionEnd otransitionend',
      transition       : 'transitionend'
    }

    for (var name in transEndEventNames) {
      if (el.style[name] !== undefined) {
        return { end: transEndEventNames[name] }
      }
    }

    return false // explicit for ie8 (  ._.)
  }

  // http://blog.alexmaccaw.com/css-transitions
  $.fn.emulateTransitionEnd = function (duration) {
    var called = false
    var $el = this
    $(this).one('bsTransitionEnd', function () { called = true })
    var callback = function () { if (!called) $($el).trigger($.support.transition.end) }
    setTimeout(callback, duration)
    return this
  }

  $(function () {
    $.support.transition = transitionEnd()

    if (!$.support.transition) return

    $.event.special.bsTransitionEnd = {
      bindType: $.support.transition.end,
      delegateType: $.support.transition.end,
      handle: function (e) {
        if ($(e.target).is(this)) return e.handleObj.handler.apply(this, arguments)
      }
    }
  })

}(jQuery);

/* ========================================================================
 * Bootstrap: alert.js v3.3.6
 * http://getbootstrap.com/javascript/#alerts
 * ========================================================================
 * Copyright 2011-2015 Twitter, Inc.
 * Licensed under MIT (https://github.com/twbs/bootstrap/blob/master/LICENSE)
 * ======================================================================== */


+function ($) {
  'use strict';

  // ALERT CLASS DEFINITION
  // ======================

  var dismiss = '[data-dismiss="alert"]'
  var Alert   = function (el) {
    $(el).on('click', dismiss, this.close)
  }

  Alert.VERSION = '3.3.6'

  Alert.TRANSITION_DURATION = 150

  Alert.prototype.close = function (e) {
    var $this    = $(this)
    var selector = $this.attr('data-target')

    if (!selector) {
      selector = $this.attr('href')
      selector = selector && selector.replace(/.*(?=#[^\s]*$)/, '') // strip for ie7
    }

    var $parent = $(selector)

    if (e) e.preventDefault()

    if (!$parent.length) {
      $parent = $this.closest('.alert')
    }

    $parent.trigger(e = $.Event('close.bs.alert'))

    if (e.isDefaultPrevented()) return

    $parent.removeClass('in')

    function removeElement() {
      // detach from parent, fire event then clean up data
      $parent.detach().trigger('closed.bs.alert').remove()
    }

    $.support.transition && $parent.hasClass('fade') ?
      $parent
        .one('bsTransitionEnd', removeElement)
        .emulateTransitionEnd(Alert.TRANSITION_DURATION) :
      removeElement()
  }


  // ALERT PLUGIN DEFINITION
  // =======================

  function Plugin(option) {
    return this.each(function () {
      var $this = $(this)
      var data  = $this.data('bs.alert')

      if (!data) $this.data('bs.alert', (data = new Alert(this)))
      if (typeof option == 'string') data[option].call($this)
    })
  }

  var old = $.fn.alert

  $.fn.alert             = Plugin
  $.fn.alert.Constructor = Alert


  // ALERT NO CONFLICT
  // =================

  $.fn.alert.noConflict = function () {
    $.fn.alert = old
    return this
  }


  // ALERT DATA-API
  // ==============

  $(document).on('click.bs.alert.data-api', dismiss, Alert.prototype.close)

}(jQuery);

/* ========================================================================
 * Bootstrap: button.js v3.3.6
 * http://getbootstrap.com/javascript/#buttons
 * ========================================================================
 * Copyright 2011-2015 Twitter, Inc.
 * Licensed under MIT (https://github.com/twbs/bootstrap/blob/master/LICENSE)
 * ======================================================================== */


+function ($) {
  'use strict';

  // BUTTON PUBLIC CLASS DEFINITION
  // ==============================

  var Button = function (element, options) {
    this.$element  = $(element)
    this.options   = $.extend({}, Button.DEFAULTS, options)
    this.isLoading = false
  }

  Button.VERSION  = '3.3.6'

  Button.DEFAULTS = {
    loadingText: 'loading...'
  }

  Button.prototype.setState = function (state) {
    var d    = 'disabled'
    var $el  = this.$element
    var val  = $el.is('input') ? 'val' : 'html'
    var data = $el.data()

    state += 'Text'

    if (data.resetText == null) $el.data('resetText', $el[val]())

    // push to event loop to allow forms to submit
    setTimeout($.proxy(function () {
      $el[val](data[state] == null ? this.options[state] : data[state])

      if (state == 'loadingText') {
        this.isLoading = true
        $el.addClass(d).attr(d, d)
      } else if (this.isLoading) {
        this.isLoading = false
        $el.removeClass(d).removeAttr(d)
      }
    }, this), 0)
  }

  Button.prototype.toggle = function () {
    var changed = true
    var $parent = this.$element.closest('[data-toggle="buttons"]')

    if ($parent.length) {
      var $input = this.$element.find('input')
      if ($input.prop('type') == 'radio') {
        if ($input.prop('checked')) changed = false
        $parent.find('.active').removeClass('active')
        this.$element.addClass('active')
      } else if ($input.prop('type') == 'checkbox') {
        if (($input.prop('checked')) !== this.$element.hasClass('active')) changed = false
        this.$element.toggleClass('active')
      }
      $input.prop('checked', this.$element.hasClass('active'))
      if (changed) $input.trigger('change')
    } else {
      this.$element.attr('aria-pressed', !this.$element.hasClass('active'))
      this.$element.toggleClass('active')
    }
  }


  // BUTTON PLUGIN DEFINITION
  // ========================

  function Plugin(option) {
    return this.each(function () {
      var $this   = $(this)
      var data    = $this.data('bs.button')
      var options = typeof option == 'object' && option

      if (!data) $this.data('bs.button', (data = new Button(this, options)))

      if (option == 'toggle') data.toggle()
      else if (option) data.setState(option)
    })
  }

  var old = $.fn.button

  $.fn.button             = Plugin
  $.fn.button.Constructor = Button


  // BUTTON NO CONFLICT
  // ==================

  $.fn.button.noConflict = function () {
    $.fn.button = old
    return this
  }


  // BUTTON DATA-API
  // ===============

  $(document)
    .on('click.bs.button.data-api', '[data-toggle^="button"]', function (e) {
      var $btn = $(e.target)
      if (!$btn.hasClass('btn')) $btn = $btn.closest('.btn')
      Plugin.call($btn, 'toggle')
      if (!($(e.target).is('input[type="radio"]') || $(e.target).is('input[type="checkbox"]'))) e.preventDefault()
    })
    .on('focus.bs.button.data-api blur.bs.button.data-api', '[data-toggle^="button"]', function (e) {
      $(e.target).closest('.btn').toggleClass('focus', /^focus(in)?$/.test(e.type))
    })

}(jQuery);

/* ========================================================================
 * Bootstrap: carousel.js v3.3.6
 * http://getbootstrap.com/javascript/#carousel
 * ========================================================================
 * Copyright 2011-2015 Twitter, Inc.
 * Licensed under MIT (https://github.com/twbs/bootstrap/blob/master/LICENSE)
 * ======================================================================== */


+function ($) {
  'use strict';

  // CAROUSEL CLASS DEFINITION
  // =========================

  var Carousel = function (element, options) {
    this.$element    = $(element)
    this.$indicators = this.$element.find('.carousel-indicators')
    this.options     = options
    this.paused      = null
    this.sliding     = null
    this.interval    = null
    this.$active     = null
    this.$items      = null

    this.options.keyboard && this.$element.on('keydown.bs.carousel', $.proxy(this.keydown, this))

    this.options.pause == 'hover' && !('ontouchstart' in document.documentElement) && this.$element
      .on('mouseenter.bs.carousel', $.proxy(this.pause, this))
      .on('mouseleave.bs.carousel', $.proxy(this.cycle, this))
  }

  Carousel.VERSION  = '3.3.6'

  Carousel.TRANSITION_DURATION = 600

  Carousel.DEFAULTS = {
    interval: 5000,
    pause: 'hover',
    wrap: true,
    keyboard: true
  }

  Carousel.prototype.keydown = function (e) {
    if (/input|textarea/i.test(e.target.tagName)) return
    switch (e.which) {
      case 37: this.prev(); break
      case 39: this.next(); break
      default: return
    }

    e.preventDefault()
  }

  Carousel.prototype.cycle = function (e) {
    e || (this.paused = false)

    this.interval && clearInterval(this.interval)

    this.options.interval
      && !this.paused
      && (this.interval = setInterval($.proxy(this.next, this), this.options.interval))

    return this
  }

  Carousel.prototype.getItemIndex = function (item) {
    this.$items = item.parent().children('.item')
    return this.$items.index(item || this.$active)
  }

  Carousel.prototype.getItemForDirection = function (direction, active) {
    var activeIndex = this.getItemIndex(active)
    var willWrap = (direction == 'prev' && activeIndex === 0)
                || (direction == 'next' && activeIndex == (this.$items.length - 1))
    if (willWrap && !this.options.wrap) return active
    var delta = direction == 'prev' ? -1 : 1
    var itemIndex = (activeIndex + delta) % this.$items.length
    return this.$items.eq(itemIndex)
  }

  Carousel.prototype.to = function (pos) {
    var that        = this
    var activeIndex = this.getItemIndex(this.$active = this.$element.find('.item.active'))

    if (pos > (this.$items.length - 1) || pos < 0) return

    if (this.sliding)       return this.$element.one('slid.bs.carousel', function () { that.to(pos) }) // yes, "slid"
    if (activeIndex == pos) return this.pause().cycle()

    return this.slide(pos > activeIndex ? 'next' : 'prev', this.$items.eq(pos))
  }

  Carousel.prototype.pause = function (e) {
    e || (this.paused = true)

    if (this.$element.find('.next, .prev').length && $.support.transition) {
      this.$element.trigger($.support.transition.end)
      this.cycle(true)
    }

    this.interval = clearInterval(this.interval)

    return this
  }

  Carousel.prototype.next = function () {
    if (this.sliding) return
    return this.slide('next')
  }

  Carousel.prototype.prev = function () {
    if (this.sliding) return
    return this.slide('prev')
  }

  Carousel.prototype.slide = function (type, next) {
    var $active   = this.$element.find('.item.active')
    var $next     = next || this.getItemForDirection(type, $active)
    var isCycling = this.interval
    var direction = type == 'next' ? 'left' : 'right'
    var that      = this

    if ($next.hasClass('active')) return (this.sliding = false)

    var relatedTarget = $next[0]
    var slideEvent = $.Event('slide.bs.carousel', {
      relatedTarget: relatedTarget,
      direction: direction
    })
    this.$element.trigger(slideEvent)
    if (slideEvent.isDefaultPrevented()) return

    this.sliding = true

    isCycling && this.pause()

    if (this.$indicators.length) {
      this.$indicators.find('.active').removeClass('active')
      var $nextIndicator = $(this.$indicators.children()[this.getItemIndex($next)])
      $nextIndicator && $nextIndicator.addClass('active')
    }

    var slidEvent = $.Event('slid.bs.carousel', { relatedTarget: relatedTarget, direction: direction }) // yes, "slid"
    if ($.support.transition && this.$element.hasClass('slide')) {
      $next.addClass(type)
      $next[0].offsetWidth // force reflow
      $active.addClass(direction)
      $next.addClass(direction)
      $active
        .one('bsTransitionEnd', function () {
          $next.removeClass([type, direction].join(' ')).addClass('active')
          $active.removeClass(['active', direction].join(' '))
          that.sliding = false
          setTimeout(function () {
            that.$element.trigger(slidEvent)
          }, 0)
        })
        .emulateTransitionEnd(Carousel.TRANSITION_DURATION)
    } else {
      $active.removeClass('active')
      $next.addClass('active')
      this.sliding = false
      this.$element.trigger(slidEvent)
    }

    isCycling && this.cycle()

    return this
  }


  // CAROUSEL PLUGIN DEFINITION
  // ==========================

  function Plugin(option) {
    return this.each(function () {
      var $this   = $(this)
      var data    = $this.data('bs.carousel')
      var options = $.extend({}, Carousel.DEFAULTS, $this.data(), typeof option == 'object' && option)
      var action  = typeof option == 'string' ? option : options.slide

      if (!data) $this.data('bs.carousel', (data = new Carousel(this, options)))
      if (typeof option == 'number') data.to(option)
      else if (action) data[action]()
      else if (options.interval) data.pause().cycle()
    })
  }

  var old = $.fn.carousel

  $.fn.carousel             = Plugin
  $.fn.carousel.Constructor = Carousel


  // CAROUSEL NO CONFLICT
  // ====================

  $.fn.carousel.noConflict = function () {
    $.fn.carousel = old
    return this
  }


  // CAROUSEL DATA-API
  // =================

  var clickHandler = function (e) {
    var href
    var $this   = $(this)
    var $target = $($this.attr('data-target') || (href = $this.attr('href')) && href.replace(/.*(?=#[^\s]+$)/, '')) // strip for ie7
    if (!$target.hasClass('carousel')) return
    var options = $.extend({}, $target.data(), $this.data())
    var slideIndex = $this.attr('data-slide-to')
    if (slideIndex) options.interval = false

    Plugin.call($target, options)

    if (slideIndex) {
      $target.data('bs.carousel').to(slideIndex)
    }

    e.preventDefault()
  }

  $(document)
    .on('click.bs.carousel.data-api', '[data-slide]', clickHandler)
    .on('click.bs.carousel.data-api', '[data-slide-to]', clickHandler)

  $(window).on('load', function () {
    $('[data-ride="carousel"]').each(function () {
      var $carousel = $(this)
      Plugin.call($carousel, $carousel.data())
    })
  })

}(jQuery);

/* ========================================================================
 * Bootstrap: collapse.js v3.3.6
 * http://getbootstrap.com/javascript/#collapse
 * ========================================================================
 * Copyright 2011-2015 Twitter, Inc.
 * Licensed under MIT (https://github.com/twbs/bootstrap/blob/master/LICENSE)
 * ======================================================================== */


+function ($) {
  'use strict';

  // COLLAPSE PUBLIC CLASS DEFINITION
  // ================================

  var Collapse = function (element, options) {
    this.$element      = $(element)
    this.options       = $.extend({}, Collapse.DEFAULTS, options)
    this.$trigger      = $('[data-toggle="collapse"][href="#' + element.id + '"],' +
                           '[data-toggle="collapse"][data-target="#' + element.id + '"]')
    this.transitioning = null

    if (this.options.parent) {
      this.$parent = this.getParent()
    } else {
      this.addAriaAndCollapsedClass(this.$element, this.$trigger)
    }

    if (this.options.toggle) this.toggle()
  }

  Collapse.VERSION  = '3.3.6'

  Collapse.TRANSITION_DURATION = 350

  Collapse.DEFAULTS = {
    toggle: true
  }

  Collapse.prototype.dimension = function () {
    var hasWidth = this.$element.hasClass('width')
    return hasWidth ? 'width' : 'height'
  }

  Collapse.prototype.show = function () {
    if (this.transitioning || this.$element.hasClass('in')) return

    var activesData
    var actives = this.$parent && this.$parent.children('.panel').children('.in, .collapsing')

    if (actives && actives.length) {
      activesData = actives.data('bs.collapse')
      if (activesData && activesData.transitioning) return
    }

    var startEvent = $.Event('show.bs.collapse')
    this.$element.trigger(startEvent)
    if (startEvent.isDefaultPrevented()) return

    if (actives && actives.length) {
      Plugin.call(actives, 'hide')
      activesData || actives.data('bs.collapse', null)
    }

    var dimension = this.dimension()

    this.$element
      .removeClass('collapse')
      .addClass('collapsing')[dimension](0)
      .attr('aria-expanded', true)

    this.$trigger
      .removeClass('collapsed')
      .attr('aria-expanded', true)

    this.transitioning = 1

    var complete = function () {
      this.$element
        .removeClass('collapsing')
        .addClass('collapse in')[dimension]('')
      this.transitioning = 0
      this.$element
        .trigger('shown.bs.collapse')
    }

    if (!$.support.transition) return complete.call(this)

    var scrollSize = $.camelCase(['scroll', dimension].join('-'))

    this.$element
      .one('bsTransitionEnd', $.proxy(complete, this))
      .emulateTransitionEnd(Collapse.TRANSITION_DURATION)[dimension](this.$element[0][scrollSize])
  }

  Collapse.prototype.hide = function () {
    if (this.transitioning || !this.$element.hasClass('in')) return

    var startEvent = $.Event('hide.bs.collapse')
    this.$element.trigger(startEvent)
    if (startEvent.isDefaultPrevented()) return

    var dimension = this.dimension()

    this.$element[dimension](this.$element[dimension]())[0].offsetHeight

    this.$element
      .addClass('collapsing')
      .removeClass('collapse in')
      .attr('aria-expanded', false)

    this.$trigger
      .addClass('collapsed')
      .attr('aria-expanded', false)

    this.transitioning = 1

    var complete = function () {
      this.transitioning = 0
      this.$element
        .removeClass('collapsing')
        .addClass('collapse')
        .trigger('hidden.bs.collapse')
    }

    if (!$.support.transition) return complete.call(this)

    this.$element
      [dimension](0)
      .one('bsTransitionEnd', $.proxy(complete, this))
      .emulateTransitionEnd(Collapse.TRANSITION_DURATION)
  }

  Collapse.prototype.toggle = function () {
    this[this.$element.hasClass('in') ? 'hide' : 'show']()
  }

  Collapse.prototype.getParent = function () {
    return $(this.options.parent)
      .find('[data-toggle="collapse"][data-parent="' + this.options.parent + '"]')
      .each($.proxy(function (i, element) {
        var $element = $(element)
        this.addAriaAndCollapsedClass(getTargetFromTrigger($element), $element)
      }, this))
      .end()
  }

  Collapse.prototype.addAriaAndCollapsedClass = function ($element, $trigger) {
    var isOpen = $element.hasClass('in')

    $element.attr('aria-expanded', isOpen)
    $trigger
      .toggleClass('collapsed', !isOpen)
      .attr('aria-expanded', isOpen)
  }

  function getTargetFromTrigger($trigger) {
    var href
    var target = $trigger.attr('data-target')
      || (href = $trigger.attr('href')) && href.replace(/.*(?=#[^\s]+$)/, '') // strip for ie7

    return $(target)
  }


  // COLLAPSE PLUGIN DEFINITION
  // ==========================

  function Plugin(option) {
    return this.each(function () {
      var $this   = $(this)
      var data    = $this.data('bs.collapse')
      var options = $.extend({}, Collapse.DEFAULTS, $this.data(), typeof option == 'object' && option)

      if (!data && options.toggle && /show|hide/.test(option)) options.toggle = false
      if (!data) $this.data('bs.collapse', (data = new Collapse(this, options)))
      if (typeof option == 'string') data[option]()
    })
  }

  var old = $.fn.collapse

  $.fn.collapse             = Plugin
  $.fn.collapse.Constructor = Collapse


  // COLLAPSE NO CONFLICT
  // ====================

  $.fn.collapse.noConflict = function () {
    $.fn.collapse = old
    return this
  }


  // COLLAPSE DATA-API
  // =================

  $(document).on('click.bs.collapse.data-api', '[data-toggle="collapse"]', function (e) {
    var $this   = $(this)

    if (!$this.attr('data-target')) e.preventDefault()

    var $target = getTargetFromTrigger($this)
    var data    = $target.data('bs.collapse')
    var option  = data ? 'toggle' : $this.data()

    Plugin.call($target, option)
  })

}(jQuery);

/* ========================================================================
 * Bootstrap: dropdown.js v3.3.6
 * http://getbootstrap.com/javascript/#dropdowns
 * ========================================================================
 * Copyright 2011-2015 Twitter, Inc.
 * Licensed under MIT (https://github.com/twbs/bootstrap/blob/master/LICENSE)
 * ======================================================================== */


+function ($) {
  'use strict';

  // DROPDOWN CLASS DEFINITION
  // =========================

  var backdrop = '.dropdown-backdrop'
  var toggle   = '[data-toggle="dropdown"]'
  var Dropdown = function (element) {
    $(element).on('click.bs.dropdown', this.toggle)
  }

  Dropdown.VERSION = '3.3.6'

  function getParent($this) {
    var selector = $this.attr('data-target')

    if (!selector) {
      selector = $this.attr('href')
      selector = selector && /#[A-Za-z]/.test(selector) && selector.replace(/.*(?=#[^\s]*$)/, '') // strip for ie7
    }

    var $parent = selector && $(selector)

    return $parent && $parent.length ? $parent : $this.parent()
  }

  function clearMenus(e) {
    if (e && e.which === 3) return
    $(backdrop).remove()
    $(toggle).each(function () {
      var $this         = $(this)
      var $parent       = getParent($this)
      var relatedTarget = { relatedTarget: this }

      if (!$parent.hasClass('open')) return

      if (e && e.type == 'click' && /input|textarea/i.test(e.target.tagName) && $.contains($parent[0], e.target)) return

      $parent.trigger(e = $.Event('hide.bs.dropdown', relatedTarget))

      if (e.isDefaultPrevented()) return

      $this.attr('aria-expanded', 'false')
      $parent.removeClass('open').trigger($.Event('hidden.bs.dropdown', relatedTarget))
    })
  }

  Dropdown.prototype.toggle = function (e) {
    var $this = $(this)

    if ($this.is('.disabled, :disabled')) return

    var $parent  = getParent($this)
    var isActive = $parent.hasClass('open')

    clearMenus()

    if (!isActive) {
      if ('ontouchstart' in document.documentElement && !$parent.closest('.navbar-nav').length) {
        // if mobile we use a backdrop because click events don't delegate
        $(document.createElement('div'))
          .addClass('dropdown-backdrop')
          .insertAfter($(this))
          .on('click', clearMenus)
      }

      var relatedTarget = { relatedTarget: this }
      $parent.trigger(e = $.Event('show.bs.dropdown', relatedTarget))

      if (e.isDefaultPrevented()) return

      $this
        .trigger('focus')
        .attr('aria-expanded', 'true')

      $parent
        .toggleClass('open')
        .trigger($.Event('shown.bs.dropdown', relatedTarget))
    }

    return false
  }

  Dropdown.prototype.keydown = function (e) {
    if (!/(38|40|27|32)/.test(e.which) || /input|textarea/i.test(e.target.tagName)) return

    var $this = $(this)

    e.preventDefault()
    e.stopPropagation()

    if ($this.is('.disabled, :disabled')) return

    var $parent  = getParent($this)
    var isActive = $parent.hasClass('open')

    if (!isActive && e.which != 27 || isActive && e.which == 27) {
      if (e.which == 27) $parent.find(toggle).trigger('focus')
      return $this.trigger('click')
    }

    var desc = ' li:not(.disabled):visible a'
    var $items = $parent.find('.dropdown-menu' + desc)

    if (!$items.length) return

    var index = $items.index(e.target)

    if (e.which == 38 && index > 0)                 index--         // up
    if (e.which == 40 && index < $items.length - 1) index++         // down
    if (!~index)                                    index = 0

    $items.eq(index).trigger('focus')
  }


  // DROPDOWN PLUGIN DEFINITION
  // ==========================

  function Plugin(option) {
    return this.each(function () {
      var $this = $(this)
      var data  = $this.data('bs.dropdown')

      if (!data) $this.data('bs.dropdown', (data = new Dropdown(this)))
      if (typeof option == 'string') data[option].call($this)
    })
  }

  var old = $.fn.dropdown

  $.fn.dropdown             = Plugin
  $.fn.dropdown.Constructor = Dropdown


  // DROPDOWN NO CONFLICT
  // ====================

  $.fn.dropdown.noConflict = function () {
    $.fn.dropdown = old
    return this
  }


  // APPLY TO STANDARD DROPDOWN ELEMENTS
  // ===================================

  $(document)
    .on('click.bs.dropdown.data-api', clearMenus)
    .on('click.bs.dropdown.data-api', '.dropdown form', function (e) { e.stopPropagation() })
    .on('click.bs.dropdown.data-api', toggle, Dropdown.prototype.toggle)
    .on('keydown.bs.dropdown.data-api', toggle, Dropdown.prototype.keydown)
    .on('keydown.bs.dropdown.data-api', '.dropdown-menu', Dropdown.prototype.keydown)

}(jQuery);

/* ========================================================================
 * Bootstrap: modal.js v3.3.6
 * http://getbootstrap.com/javascript/#modals
 * ========================================================================
 * Copyright 2011-2015 Twitter, Inc.
 * Licensed under MIT (https://github.com/twbs/bootstrap/blob/master/LICENSE)
 * ======================================================================== */


+function ($) {
  'use strict';

  // MODAL CLASS DEFINITION
  // ======================

  var Modal = function (element, options) {
    this.options             = options
    this.$body               = $(document.body)
    this.$element            = $(element)
    this.$dialog             = this.$element.find('.modal-dialog')
    this.$backdrop           = null
    this.isShown             = null
    this.originalBodyPad     = null
    this.scrollbarWidth      = 0
    this.ignoreBackdropClick = false

    if (this.options.remote) {
      this.$element
        .find('.modal-content')
        .load(this.options.remote, $.proxy(function () {
          this.$element.trigger('loaded.bs.modal')
        }, this))
    }
  }

  Modal.VERSION  = '3.3.6'

  Modal.TRANSITION_DURATION = 300
  Modal.BACKDROP_TRANSITION_DURATION = 150

  Modal.DEFAULTS = {
    backdrop: true,
    keyboard: true,
    show: true
  }

  Modal.prototype.toggle = function (_relatedTarget) {
    return this.isShown ? this.hide() : this.show(_relatedTarget)
  }

  Modal.prototype.show = function (_relatedTarget) {
    var that = this
    var e    = $.Event('show.bs.modal', { relatedTarget: _relatedTarget })

    this.$element.trigger(e)

    if (this.isShown || e.isDefaultPrevented()) return

    this.isShown = true

    this.checkScrollbar()
    this.setScrollbar()
    this.$body.addClass('modal-open')

    this.escape()
    this.resize()

    this.$element.on('click.dismiss.bs.modal', '[data-dismiss="modal"]', $.proxy(this.hide, this))

    this.$dialog.on('mousedown.dismiss.bs.modal', function () {
      that.$element.one('mouseup.dismiss.bs.modal', function (e) {
        if ($(e.target).is(that.$element)) that.ignoreBackdropClick = true
      })
    })

    this.backdrop(function () {
      var transition = $.support.transition && that.$element.hasClass('fade')

      if (!that.$element.parent().length) {
        that.$element.appendTo(that.$body) // don't move modals dom position
      }

      that.$element
        .show()
        .scrollTop(0)

      that.adjustDialog()

      if (transition) {
        that.$element[0].offsetWidth // force reflow
      }

      that.$element.addClass('in')

      that.enforceFocus()

      var e = $.Event('shown.bs.modal', { relatedTarget: _relatedTarget })

      transition ?
        that.$dialog // wait for modal to slide in
          .one('bsTransitionEnd', function () {
            that.$element.trigger('focus').trigger(e)
          })
          .emulateTransitionEnd(Modal.TRANSITION_DURATION) :
        that.$element.trigger('focus').trigger(e)
    })
  }

  Modal.prototype.hide = function (e) {
    if (e) e.preventDefault()

    e = $.Event('hide.bs.modal')

    this.$element.trigger(e)

    if (!this.isShown || e.isDefaultPrevented()) return

    this.isShown = false

    this.escape()
    this.resize()

    $(document).off('focusin.bs.modal')

    this.$element
      .removeClass('in')
      .off('click.dismiss.bs.modal')
      .off('mouseup.dismiss.bs.modal')

    this.$dialog.off('mousedown.dismiss.bs.modal')

    $.support.transition && this.$element.hasClass('fade') ?
      this.$element
        .one('bsTransitionEnd', $.proxy(this.hideModal, this))
        .emulateTransitionEnd(Modal.TRANSITION_DURATION) :
      this.hideModal()
  }

  Modal.prototype.enforceFocus = function () {
    $(document)
      .off('focusin.bs.modal') // guard against infinite focus loop
      .on('focusin.bs.modal', $.proxy(function (e) {
        if (this.$element[0] !== e.target && !this.$element.has(e.target).length) {
          this.$element.trigger('focus')
        }
      }, this))
  }

  Modal.prototype.escape = function () {
    if (this.isShown && this.options.keyboard) {
      this.$element.on('keydown.dismiss.bs.modal', $.proxy(function (e) {
        e.which == 27 && this.hide()
      }, this))
    } else if (!this.isShown) {
      this.$element.off('keydown.dismiss.bs.modal')
    }
  }

  Modal.prototype.resize = function () {
    if (this.isShown) {
      $(window).on('resize.bs.modal', $.proxy(this.handleUpdate, this))
    } else {
      $(window).off('resize.bs.modal')
    }
  }

  Modal.prototype.hideModal = function () {
    var that = this
    this.$element.hide()
    this.backdrop(function () {
      that.$body.removeClass('modal-open')
      that.resetAdjustments()
      that.resetScrollbar()
      that.$element.trigger('hidden.bs.modal')
    })
  }

  Modal.prototype.removeBackdrop = function () {
    this.$backdrop && this.$backdrop.remove()
    this.$backdrop = null
  }

  Modal.prototype.backdrop = function (callback) {
    var that = this
    var animate = this.$element.hasClass('fade') ? 'fade' : ''

    if (this.isShown && this.options.backdrop) {
      var doAnimate = $.support.transition && animate

      this.$backdrop = $(document.createElement('div'))
        .addClass('modal-backdrop ' + animate)
        .appendTo(this.$body)

      this.$element.on('click.dismiss.bs.modal', $.proxy(function (e) {
        if (this.ignoreBackdropClick) {
          this.ignoreBackdropClick = false
          return
        }
        if (e.target !== e.currentTarget) return
        this.options.backdrop == 'static'
          ? this.$element[0].focus()
          : this.hide()
      }, this))

      if (doAnimate) this.$backdrop[0].offsetWidth // force reflow

      this.$backdrop.addClass('in')

      if (!callback) return

      doAnimate ?
        this.$backdrop
          .one('bsTransitionEnd', callback)
          .emulateTransitionEnd(Modal.BACKDROP_TRANSITION_DURATION) :
        callback()

    } else if (!this.isShown && this.$backdrop) {
      this.$backdrop.removeClass('in')

      var callbackRemove = function () {
        that.removeBackdrop()
        callback && callback()
      }
      $.support.transition && this.$element.hasClass('fade') ?
        this.$backdrop
          .one('bsTransitionEnd', callbackRemove)
          .emulateTransitionEnd(Modal.BACKDROP_TRANSITION_DURATION) :
        callbackRemove()

    } else if (callback) {
      callback()
    }
  }

  // these following methods are used to handle overflowing modals

  Modal.prototype.handleUpdate = function () {
    this.adjustDialog()
  }

  Modal.prototype.adjustDialog = function () {
    var modalIsOverflowing = this.$element[0].scrollHeight > document.documentElement.clientHeight

    this.$element.css({
      paddingLeft:  !this.bodyIsOverflowing && modalIsOverflowing ? this.scrollbarWidth : '',
      paddingRight: this.bodyIsOverflowing && !modalIsOverflowing ? this.scrollbarWidth : ''
    })
  }

  Modal.prototype.resetAdjustments = function () {
    this.$element.css({
      paddingLeft: '',
      paddingRight: ''
    })
  }

  Modal.prototype.checkScrollbar = function () {
    var fullWindowWidth = window.innerWidth
    if (!fullWindowWidth) { // workaround for missing window.innerWidth in IE8
      var documentElementRect = document.documentElement.getBoundingClientRect()
      fullWindowWidth = documentElementRect.right - Math.abs(documentElementRect.left)
    }
    this.bodyIsOverflowing = document.body.clientWidth < fullWindowWidth
    this.scrollbarWidth = this.measureScrollbar()
  }

  Modal.prototype.setScrollbar = function () {
    var bodyPad = parseInt((this.$body.css('padding-right') || 0), 10)
    this.originalBodyPad = document.body.style.paddingRight || ''
    if (this.bodyIsOverflowing) this.$body.css('padding-right', bodyPad + this.scrollbarWidth)
  }

  Modal.prototype.resetScrollbar = function () {
    this.$body.css('padding-right', this.originalBodyPad)
  }

  Modal.prototype.measureScrollbar = function () { // thx walsh
    var scrollDiv = document.createElement('div')
    scrollDiv.className = 'modal-scrollbar-measure'
    this.$body.append(scrollDiv)
    var scrollbarWidth = scrollDiv.offsetWidth - scrollDiv.clientWidth
    this.$body[0].removeChild(scrollDiv)
    return scrollbarWidth
  }


  // MODAL PLUGIN DEFINITION
  // =======================

  function Plugin(option, _relatedTarget) {
    return this.each(function () {
      var $this   = $(this)
      var data    = $this.data('bs.modal')
      var options = $.extend({}, Modal.DEFAULTS, $this.data(), typeof option == 'object' && option)

      if (!data) $this.data('bs.modal', (data = new Modal(this, options)))
      if (typeof option == 'string') data[option](_relatedTarget)
      else if (options.show) data.show(_relatedTarget)
    })
  }

  var old = $.fn.modal

  $.fn.modal             = Plugin
  $.fn.modal.Constructor = Modal


  // MODAL NO CONFLICT
  // =================

  $.fn.modal.noConflict = function () {
    $.fn.modal = old
    return this
  }


  // MODAL DATA-API
  // ==============

  $(document).on('click.bs.modal.data-api', '[data-toggle="modal"]', function (e) {
    var $this   = $(this)
    var href    = $this.attr('href')
    var $target = $($this.attr('data-target') || (href && href.replace(/.*(?=#[^\s]+$)/, ''))) // strip for ie7
    var option  = $target.data('bs.modal') ? 'toggle' : $.extend({ remote: !/#/.test(href) && href }, $target.data(), $this.data())

    if ($this.is('a')) e.preventDefault()

    $target.one('show.bs.modal', function (showEvent) {
      if (showEvent.isDefaultPrevented()) return // only register focus restorer if modal will actually get shown
      $target.one('hidden.bs.modal', function () {
        $this.is(':visible') && $this.trigger('focus')
      })
    })
    Plugin.call($target, option, this)
  })

}(jQuery);

/* ========================================================================
 * Bootstrap: tooltip.js v3.3.6
 * http://getbootstrap.com/javascript/#tooltip
 * Inspired by the original jQuery.tipsy by Jason Frame
 * ========================================================================
 * Copyright 2011-2015 Twitter, Inc.
 * Licensed under MIT (https://github.com/twbs/bootstrap/blob/master/LICENSE)
 * ======================================================================== */


+function ($) {
  'use strict';

  // TOOLTIP PUBLIC CLASS DEFINITION
  // ===============================

  var Tooltip = function (element, options) {
    this.type       = null
    this.options    = null
    this.enabled    = null
    this.timeout    = null
    this.hoverState = null
    this.$element   = null
    this.inState    = null

    this.init('tooltip', element, options)
  }

  Tooltip.VERSION  = '3.3.6'

  Tooltip.TRANSITION_DURATION = 150

  Tooltip.DEFAULTS = {
    animation: true,
    placement: 'top',
    selector: false,
    template: '<div class="tooltip" role="tooltip"><div class="tooltip-arrow"></div><div class="tooltip-inner"></div></div>',
    trigger: 'hover focus',
    title: '',
    delay: 0,
    html: false,
    container: false,
    viewport: {
      selector: 'body',
      padding: 0
    }
  }

  Tooltip.prototype.init = function (type, element, options) {
    this.enabled   = true
    this.type      = type
    this.$element  = $(element)
    this.options   = this.getOptions(options)
    this.$viewport = this.options.viewport && $($.isFunction(this.options.viewport) ? this.options.viewport.call(this, this.$element) : (this.options.viewport.selector || this.options.viewport))
    this.inState   = { click: false, hover: false, focus: false }

    if (this.$element[0] instanceof document.constructor && !this.options.selector) {
      throw new Error('`selector` option must be specified when initializing ' + this.type + ' on the window.document object!')
    }

    var triggers = this.options.trigger.split(' ')

    for (var i = triggers.length; i--;) {
      var trigger = triggers[i]

      if (trigger == 'click') {
        this.$element.on('click.' + this.type, this.options.selector, $.proxy(this.toggle, this))
      } else if (trigger != 'manual') {
        var eventIn  = trigger == 'hover' ? 'mouseenter' : 'focusin'
        var eventOut = trigger == 'hover' ? 'mouseleave' : 'focusout'

        this.$element.on(eventIn  + '.' + this.type, this.options.selector, $.proxy(this.enter, this))
        this.$element.on(eventOut + '.' + this.type, this.options.selector, $.proxy(this.leave, this))
      }
    }

    this.options.selector ?
      (this._options = $.extend({}, this.options, { trigger: 'manual', selector: '' })) :
      this.fixTitle()
  }

  Tooltip.prototype.getDefaults = function () {
    return Tooltip.DEFAULTS
  }

  Tooltip.prototype.getOptions = function (options) {
    options = $.extend({}, this.getDefaults(), this.$element.data(), options)

    if (options.delay && typeof options.delay == 'number') {
      options.delay = {
        show: options.delay,
        hide: options.delay
      }
    }

    return options
  }

  Tooltip.prototype.getDelegateOptions = function () {
    var options  = {}
    var defaults = this.getDefaults()

    this._options && $.each(this._options, function (key, value) {
      if (defaults[key] != value) options[key] = value
    })

    return options
  }

  Tooltip.prototype.enter = function (obj) {
    var self = obj instanceof this.constructor ?
      obj : $(obj.currentTarget).data('bs.' + this.type)

    if (!self) {
      self = new this.constructor(obj.currentTarget, this.getDelegateOptions())
      $(obj.currentTarget).data('bs.' + this.type, self)
    }

    if (obj instanceof $.Event) {
      self.inState[obj.type == 'focusin' ? 'focus' : 'hover'] = true
    }

    if (self.tip().hasClass('in') || self.hoverState == 'in') {
      self.hoverState = 'in'
      return
    }

    clearTimeout(self.timeout)

    self.hoverState = 'in'

    if (!self.options.delay || !self.options.delay.show) return self.show()

    self.timeout = setTimeout(function () {
      if (self.hoverState == 'in') self.show()
    }, self.options.delay.show)
  }

  Tooltip.prototype.isInStateTrue = function () {
    for (var key in this.inState) {
      if (this.inState[key]) return true
    }

    return false
  }

  Tooltip.prototype.leave = function (obj) {
    var self = obj instanceof this.constructor ?
      obj : $(obj.currentTarget).data('bs.' + this.type)

    if (!self) {
      self = new this.constructor(obj.currentTarget, this.getDelegateOptions())
      $(obj.currentTarget).data('bs.' + this.type, self)
    }

    if (obj instanceof $.Event) {
      self.inState[obj.type == 'focusout' ? 'focus' : 'hover'] = false
    }

    if (self.isInStateTrue()) return

    clearTimeout(self.timeout)

    self.hoverState = 'out'

    if (!self.options.delay || !self.options.delay.hide) return self.hide()

    self.timeout = setTimeout(function () {
      if (self.hoverState == 'out') self.hide()
    }, self.options.delay.hide)
  }

  Tooltip.prototype.show = function () {
    var e = $.Event('show.bs.' + this.type)

    if (this.hasContent() && this.enabled) {
      this.$element.trigger(e)

      var inDom = $.contains(this.$element[0].ownerDocument.documentElement, this.$element[0])
      if (e.isDefaultPrevented() || !inDom) return
      var that = this

      var $tip = this.tip()

      var tipId = this.getUID(this.type)

      this.setContent()
      $tip.attr('id', tipId)
      this.$element.attr('aria-describedby', tipId)

      if (this.options.animation) $tip.addClass('fade')

      var placement = typeof this.options.placement == 'function' ?
        this.options.placement.call(this, $tip[0], this.$element[0]) :
        this.options.placement

      var autoToken = /\s?auto?\s?/i
      var autoPlace = autoToken.test(placement)
      if (autoPlace) placement = placement.replace(autoToken, '') || 'top'

      $tip
        .detach()
        .css({ top: 0, left: 0, display: 'block' })
        .addClass(placement)
        .data('bs.' + this.type, this)

      this.options.container ? $tip.appendTo(this.options.container) : $tip.insertAfter(this.$element)
      this.$element.trigger('inserted.bs.' + this.type)

      var pos          = this.getPosition()
      var actualWidth  = $tip[0].offsetWidth
      var actualHeight = $tip[0].offsetHeight

      if (autoPlace) {
        var orgPlacement = placement
        var viewportDim = this.getPosition(this.$viewport)

        placement = placement == 'bottom' && pos.bottom + actualHeight > viewportDim.bottom ? 'top'    :
                    placement == 'top'    && pos.top    - actualHeight < viewportDim.top    ? 'bottom' :
                    placement == 'right'  && pos.right  + actualWidth  > viewportDim.width  ? 'left'   :
                    placement == 'left'   && pos.left   - actualWidth  < viewportDim.left   ? 'right'  :
                    placement

        $tip
          .removeClass(orgPlacement)
          .addClass(placement)
      }

      var calculatedOffset = this.getCalculatedOffset(placement, pos, actualWidth, actualHeight)

      this.applyPlacement(calculatedOffset, placement)

      var complete = function () {
        var prevHoverState = that.hoverState
        that.$element.trigger('shown.bs.' + that.type)
        that.hoverState = null

        if (prevHoverState == 'out') that.leave(that)
      }

      $.support.transition && this.$tip.hasClass('fade') ?
        $tip
          .one('bsTransitionEnd', complete)
          .emulateTransitionEnd(Tooltip.TRANSITION_DURATION) :
        complete()
    }
  }

  Tooltip.prototype.applyPlacement = function (offset, placement) {
    var $tip   = this.tip()
    var width  = $tip[0].offsetWidth
    var height = $tip[0].offsetHeight

    // manually read margins because getBoundingClientRect includes difference
    var marginTop = parseInt($tip.css('margin-top'), 10)
    var marginLeft = parseInt($tip.css('margin-left'), 10)

    // we must check for NaN for ie 8/9
    if (isNaN(marginTop))  marginTop  = 0
    if (isNaN(marginLeft)) marginLeft = 0

    offset.top  += marginTop
    offset.left += marginLeft

    // $.fn.offset doesn't round pixel values
    // so we use setOffset directly with our own function B-0
    $.offset.setOffset($tip[0], $.extend({
      using: function (props) {
        $tip.css({
          top: Math.round(props.top),
          left: Math.round(props.left)
        })
      }
    }, offset), 0)

    $tip.addClass('in')

    // check to see if placing tip in new offset caused the tip to resize itself
    var actualWidth  = $tip[0].offsetWidth
    var actualHeight = $tip[0].offsetHeight

    if (placement == 'top' && actualHeight != height) {
      offset.top = offset.top + height - actualHeight
    }

    var delta = this.getViewportAdjustedDelta(placement, offset, actualWidth, actualHeight)

    if (delta.left) offset.left += delta.left
    else offset.top += delta.top

    var isVertical          = /top|bottom/.test(placement)
    var arrowDelta          = isVertical ? delta.left * 2 - width + actualWidth : delta.top * 2 - height + actualHeight
    var arrowOffsetPosition = isVertical ? 'offsetWidth' : 'offsetHeight'

    $tip.offset(offset)
    this.replaceArrow(arrowDelta, $tip[0][arrowOffsetPosition], isVertical)
  }

  Tooltip.prototype.replaceArrow = function (delta, dimension, isVertical) {
    this.arrow()
      .css(isVertical ? 'left' : 'top', 50 * (1 - delta / dimension) + '%')
      .css(isVertical ? 'top' : 'left', '')
  }

  Tooltip.prototype.setContent = function () {
    var $tip  = this.tip()
    var title = this.getTitle()

    $tip.find('.tooltip-inner')[this.options.html ? 'html' : 'text'](title)
    $tip.removeClass('fade in top bottom left right')
  }

  Tooltip.prototype.hide = function (callback) {
    var that = this
    var $tip = $(this.$tip)
    var e    = $.Event('hide.bs.' + this.type)

    function complete() {
      if (that.hoverState != 'in') $tip.detach()
      that.$element
        .removeAttr('aria-describedby')
        .trigger('hidden.bs.' + that.type)
      callback && callback()
    }

    this.$element.trigger(e)

    if (e.isDefaultPrevented()) return

    $tip.removeClass('in')

    $.support.transition && $tip.hasClass('fade') ?
      $tip
        .one('bsTransitionEnd', complete)
        .emulateTransitionEnd(Tooltip.TRANSITION_DURATION) :
      complete()

    this.hoverState = null

    return this
  }

  Tooltip.prototype.fixTitle = function () {
    var $e = this.$element
    if ($e.attr('title') || typeof $e.attr('data-original-title') != 'string') {
      $e.attr('data-original-title', $e.attr('title') || '').attr('title', '')
    }
  }

  Tooltip.prototype.hasContent = function () {
    return this.getTitle()
  }

  Tooltip.prototype.getPosition = function ($element) {
    $element   = $element || this.$element

    var el     = $element[0]
    var isBody = el.tagName == 'BODY'

    var elRect    = el.getBoundingClientRect()
    if (elRect.width == null) {
      // width and height are missing in IE8, so compute them manually; see https://github.com/twbs/bootstrap/issues/14093
      elRect = $.extend({}, elRect, { width: elRect.right - elRect.left, height: elRect.bottom - elRect.top })
    }
    var elOffset  = isBody ? { top: 0, left: 0 } : $element.offset()
    var scroll    = { scroll: isBody ? document.documentElement.scrollTop || document.body.scrollTop : $element.scrollTop() }
    var outerDims = isBody ? { width: $(window).width(), height: $(window).height() } : null

    return $.extend({}, elRect, scroll, outerDims, elOffset)
  }

  Tooltip.prototype.getCalculatedOffset = function (placement, pos, actualWidth, actualHeight) {
    return placement == 'bottom' ? { top: pos.top + pos.height,   left: pos.left + pos.width / 2 - actualWidth / 2 } :
           placement == 'top'    ? { top: pos.top - actualHeight, left: pos.left + pos.width / 2 - actualWidth / 2 } :
           placement == 'left'   ? { top: pos.top + pos.height / 2 - actualHeight / 2, left: pos.left - actualWidth } :
        /* placement == 'right' */ { top: pos.top + pos.height / 2 - actualHeight / 2, left: pos.left + pos.width }

  }

  Tooltip.prototype.getViewportAdjustedDelta = function (placement, pos, actualWidth, actualHeight) {
    var delta = { top: 0, left: 0 }
    if (!this.$viewport) return delta

    var viewportPadding = this.options.viewport && this.options.viewport.padding || 0
    var viewportDimensions = this.getPosition(this.$viewport)

    if (/right|left/.test(placement)) {
      var topEdgeOffset    = pos.top - viewportPadding - viewportDimensions.scroll
      var bottomEdgeOffset = pos.top + viewportPadding - viewportDimensions.scroll + actualHeight
      if (topEdgeOffset < viewportDimensions.top) { // top overflow
        delta.top = viewportDimensions.top - topEdgeOffset
      } else if (bottomEdgeOffset > viewportDimensions.top + viewportDimensions.height) { // bottom overflow
        delta.top = viewportDimensions.top + viewportDimensions.height - bottomEdgeOffset
      }
    } else {
      var leftEdgeOffset  = pos.left - viewportPadding
      var rightEdgeOffset = pos.left + viewportPadding + actualWidth
      if (leftEdgeOffset < viewportDimensions.left) { // left overflow
        delta.left = viewportDimensions.left - leftEdgeOffset
      } else if (rightEdgeOffset > viewportDimensions.right) { // right overflow
        delta.left = viewportDimensions.left + viewportDimensions.width - rightEdgeOffset
      }
    }

    return delta
  }

  Tooltip.prototype.getTitle = function () {
    var title
    var $e = this.$element
    var o  = this.options

    title = $e.attr('data-original-title')
      || (typeof o.title == 'function' ? o.title.call($e[0]) :  o.title)

    return title
  }

  Tooltip.prototype.getUID = function (prefix) {
    do prefix += ~~(Math.random() * 1000000)
    while (document.getElementById(prefix))
    return prefix
  }

  Tooltip.prototype.tip = function () {
    if (!this.$tip) {
      this.$tip = $(this.options.template)
      if (this.$tip.length != 1) {
        throw new Error(this.type + ' `template` option must consist of exactly 1 top-level element!')
      }
    }
    return this.$tip
  }

  Tooltip.prototype.arrow = function () {
    return (this.$arrow = this.$arrow || this.tip().find('.tooltip-arrow'))
  }

  Tooltip.prototype.enable = function () {
    this.enabled = true
  }

  Tooltip.prototype.disable = function () {
    this.enabled = false
  }

  Tooltip.prototype.toggleEnabled = function () {
    this.enabled = !this.enabled
  }

  Tooltip.prototype.toggle = function (e) {
    var self = this
    if (e) {
      self = $(e.currentTarget).data('bs.' + this.type)
      if (!self) {
        self = new this.constructor(e.currentTarget, this.getDelegateOptions())
        $(e.currentTarget).data('bs.' + this.type, self)
      }
    }

    if (e) {
      self.inState.click = !self.inState.click
      if (self.isInStateTrue()) self.enter(self)
      else self.leave(self)
    } else {
      self.tip().hasClass('in') ? self.leave(self) : self.enter(self)
    }
  }

  Tooltip.prototype.destroy = function () {
    var that = this
    clearTimeout(this.timeout)
    this.hide(function () {
      that.$element.off('.' + that.type).removeData('bs.' + that.type)
      if (that.$tip) {
        that.$tip.detach()
      }
      that.$tip = null
      that.$arrow = null
      that.$viewport = null
    })
  }


  // TOOLTIP PLUGIN DEFINITION
  // =========================

  function Plugin(option) {
    return this.each(function () {
      var $this   = $(this)
      var data    = $this.data('bs.tooltip')
      var options = typeof option == 'object' && option

      if (!data && /destroy|hide/.test(option)) return
      if (!data) $this.data('bs.tooltip', (data = new Tooltip(this, options)))
      if (typeof option == 'string') data[option]()
    })
  }

  var old = $.fn.tooltip

  $.fn.tooltip             = Plugin
  $.fn.tooltip.Constructor = Tooltip


  // TOOLTIP NO CONFLICT
  // ===================

  $.fn.tooltip.noConflict = function () {
    $.fn.tooltip = old
    return this
  }

}(jQuery);

/* ========================================================================
 * Bootstrap: popover.js v3.3.6
 * http://getbootstrap.com/javascript/#popovers
 * ========================================================================
 * Copyright 2011-2015 Twitter, Inc.
 * Licensed under MIT (https://github.com/twbs/bootstrap/blob/master/LICENSE)
 * ======================================================================== */


+function ($) {
  'use strict';

  // POPOVER PUBLIC CLASS DEFINITION
  // ===============================

  var Popover = function (element, options) {
    this.init('popover', element, options)
  }

  if (!$.fn.tooltip) throw new Error('Popover requires tooltip.js')

  Popover.VERSION  = '3.3.6'

  Popover.DEFAULTS = $.extend({}, $.fn.tooltip.Constructor.DEFAULTS, {
    placement: 'right',
    trigger: 'click',
    content: '',
    template: '<div class="popover" role="tooltip"><div class="arrow"></div><h3 class="popover-title"></h3><div class="popover-content"></div></div>'
  })


  // NOTE: POPOVER EXTENDS tooltip.js
  // ================================

  Popover.prototype = $.extend({}, $.fn.tooltip.Constructor.prototype)

  Popover.prototype.constructor = Popover

  Popover.prototype.getDefaults = function () {
    return Popover.DEFAULTS
  }

  Popover.prototype.setContent = function () {
    var $tip    = this.tip()
    var title   = this.getTitle()
    var content = this.getContent()

    $tip.find('.popover-title')[this.options.html ? 'html' : 'text'](title)
    $tip.find('.popover-content').children().detach().end()[ // we use append for html objects to maintain js events
      this.options.html ? (typeof content == 'string' ? 'html' : 'append') : 'text'
    ](content)

    $tip.removeClass('fade top bottom left right in')

    // IE8 doesn't accept hiding via the `:empty` pseudo selector, we have to do
    // this manually by checking the contents.
    if (!$tip.find('.popover-title').html()) $tip.find('.popover-title').hide()
  }

  Popover.prototype.hasContent = function () {
    return this.getTitle() || this.getContent()
  }

  Popover.prototype.getContent = function () {
    var $e = this.$element
    var o  = this.options

    return $e.attr('data-content')
      || (typeof o.content == 'function' ?
            o.content.call($e[0]) :
            o.content)
  }

  Popover.prototype.arrow = function () {
    return (this.$arrow = this.$arrow || this.tip().find('.arrow'))
  }


  // POPOVER PLUGIN DEFINITION
  // =========================

  function Plugin(option) {
    return this.each(function () {
      var $this   = $(this)
      var data    = $this.data('bs.popover')
      var options = typeof option == 'object' && option

      if (!data && /destroy|hide/.test(option)) return
      if (!data) $this.data('bs.popover', (data = new Popover(this, options)))
      if (typeof option == 'string') data[option]()
    })
  }

  var old = $.fn.popover

  $.fn.popover             = Plugin
  $.fn.popover.Constructor = Popover


  // POPOVER NO CONFLICT
  // ===================

  $.fn.popover.noConflict = function () {
    $.fn.popover = old
    return this
  }

}(jQuery);

/* ========================================================================
 * Bootstrap: scrollspy.js v3.3.6
 * http://getbootstrap.com/javascript/#scrollspy
 * ========================================================================
 * Copyright 2011-2015 Twitter, Inc.
 * Licensed under MIT (https://github.com/twbs/bootstrap/blob/master/LICENSE)
 * ======================================================================== */


+function ($) {
  'use strict';

  // SCROLLSPY CLASS DEFINITION
  // ==========================

  function ScrollSpy(element, options) {
    this.$body          = $(document.body)
    this.$scrollElement = $(element).is(document.body) ? $(window) : $(element)
    this.options        = $.extend({}, ScrollSpy.DEFAULTS, options)
    this.selector       = (this.options.target || '') + ' .nav li > a'
    this.offsets        = []
    this.targets        = []
    this.activeTarget   = null
    this.scrollHeight   = 0

    this.$scrollElement.on('scroll.bs.scrollspy', $.proxy(this.process, this))
    this.refresh()
    this.process()
  }

  ScrollSpy.VERSION  = '3.3.6'

  ScrollSpy.DEFAULTS = {
    offset: 10
  }

  ScrollSpy.prototype.getScrollHeight = function () {
    return this.$scrollElement[0].scrollHeight || Math.max(this.$body[0].scrollHeight, document.documentElement.scrollHeight)
  }

  ScrollSpy.prototype.refresh = function () {
    var that          = this
    var offsetMethod  = 'offset'
    var offsetBase    = 0

    this.offsets      = []
    this.targets      = []
    this.scrollHeight = this.getScrollHeight()

    if (!$.isWindow(this.$scrollElement[0])) {
      offsetMethod = 'position'
      offsetBase   = this.$scrollElement.scrollTop()
    }

    this.$body
      .find(this.selector)
      .map(function () {
        var $el   = $(this)
        var href  = $el.data('target') || $el.attr('href')
        var $href = /^#./.test(href) && $(href)

        return ($href
          && $href.length
          && $href.is(':visible')
          && [[$href[offsetMethod]().top + offsetBase, href]]) || null
      })
      .sort(function (a, b) { return a[0] - b[0] })
      .each(function () {
        that.offsets.push(this[0])
        that.targets.push(this[1])
      })
  }

  ScrollSpy.prototype.process = function () {
    var scrollTop    = this.$scrollElement.scrollTop() + this.options.offset
    var scrollHeight = this.getScrollHeight()
    var maxScroll    = this.options.offset + scrollHeight - this.$scrollElement.height()
    var offsets      = this.offsets
    var targets      = this.targets
    var activeTarget = this.activeTarget
    var i

    if (this.scrollHeight != scrollHeight) {
      this.refresh()
    }

    if (scrollTop >= maxScroll) {
      return activeTarget != (i = targets[targets.length - 1]) && this.activate(i)
    }

    if (activeTarget && scrollTop < offsets[0]) {
      this.activeTarget = null
      return this.clear()
    }

    for (i = offsets.length; i--;) {
      activeTarget != targets[i]
        && scrollTop >= offsets[i]
        && (offsets[i + 1] === undefined || scrollTop < offsets[i + 1])
        && this.activate(targets[i])
    }
  }

  ScrollSpy.prototype.activate = function (target) {
    this.activeTarget = target

    this.clear()

    var selector = this.selector +
      '[data-target="' + target + '"],' +
      this.selector + '[href="' + target + '"]'

    var active = $(selector)
      .parents('li')
      .addClass('active')

    if (active.parent('.dropdown-menu').length) {
      active = active
        .closest('li.dropdown')
        .addClass('active')
    }

    active.trigger('activate.bs.scrollspy')
  }

  ScrollSpy.prototype.clear = function () {
    $(this.selector)
      .parentsUntil(this.options.target, '.active')
      .removeClass('active')
  }


  // SCROLLSPY PLUGIN DEFINITION
  // ===========================

  function Plugin(option) {
    return this.each(function () {
      var $this   = $(this)
      var data    = $this.data('bs.scrollspy')
      var options = typeof option == 'object' && option

      if (!data) $this.data('bs.scrollspy', (data = new ScrollSpy(this, options)))
      if (typeof option == 'string') data[option]()
    })
  }

  var old = $.fn.scrollspy

  $.fn.scrollspy             = Plugin
  $.fn.scrollspy.Constructor = ScrollSpy


  // SCROLLSPY NO CONFLICT
  // =====================

  $.fn.scrollspy.noConflict = function () {
    $.fn.scrollspy = old
    return this
  }


  // SCROLLSPY DATA-API
  // ==================

  $(window).on('load.bs.scrollspy.data-api', function () {
    $('[data-spy="scroll"]').each(function () {
      var $spy = $(this)
      Plugin.call($spy, $spy.data())
    })
  })

}(jQuery);

/* ========================================================================
 * Bootstrap: tab.js v3.3.6
 * http://getbootstrap.com/javascript/#tabs
 * ========================================================================
 * Copyright 2011-2015 Twitter, Inc.
 * Licensed under MIT (https://github.com/twbs/bootstrap/blob/master/LICENSE)
 * ======================================================================== */


+function ($) {
  'use strict';

  // TAB CLASS DEFINITION
  // ====================

  var Tab = function (element) {
    // jscs:disable requireDollarBeforejQueryAssignment
    this.element = $(element)
    // jscs:enable requireDollarBeforejQueryAssignment
  }

  Tab.VERSION = '3.3.6'

  Tab.TRANSITION_DURATION = 150

  Tab.prototype.show = function () {
    var $this    = this.element
    var $ul      = $this.closest('ul:not(.dropdown-menu)')
    var selector = $this.data('target')

    if (!selector) {
      selector = $this.attr('href')
      selector = selector && selector.replace(/.*(?=#[^\s]*$)/, '') // strip for ie7
    }

    if ($this.parent('li').hasClass('active')) return

    var $previous = $ul.find('.active:last a')
    var hideEvent = $.Event('hide.bs.tab', {
      relatedTarget: $this[0]
    })
    var showEvent = $.Event('show.bs.tab', {
      relatedTarget: $previous[0]
    })

    $previous.trigger(hideEvent)
    $this.trigger(showEvent)

    if (showEvent.isDefaultPrevented() || hideEvent.isDefaultPrevented()) return

    var $target = $(selector)

    this.activate($this.closest('li'), $ul)
    this.activate($target, $target.parent(), function () {
      $previous.trigger({
        type: 'hidden.bs.tab',
        relatedTarget: $this[0]
      })
      $this.trigger({
        type: 'shown.bs.tab',
        relatedTarget: $previous[0]
      })
    })
  }

  Tab.prototype.activate = function (element, container, callback) {
    var $active    = container.find('> .active')
    var transition = callback
      && $.support.transition
      && ($active.length && $active.hasClass('fade') || !!container.find('> .fade').length)

    function next() {
      $active
        .removeClass('active')
        .find('> .dropdown-menu > .active')
          .removeClass('active')
        .end()
        .find('[data-toggle="tab"]')
          .attr('aria-expanded', false)

      element
        .addClass('active')
        .find('[data-toggle="tab"]')
          .attr('aria-expanded', true)

      if (transition) {
        element[0].offsetWidth // reflow for transition
        element.addClass('in')
      } else {
        element.removeClass('fade')
      }

      if (element.parent('.dropdown-menu').length) {
        element
          .closest('li.dropdown')
            .addClass('active')
          .end()
          .find('[data-toggle="tab"]')
            .attr('aria-expanded', true)
      }

      callback && callback()
    }

    $active.length && transition ?
      $active
        .one('bsTransitionEnd', next)
        .emulateTransitionEnd(Tab.TRANSITION_DURATION) :
      next()

    $active.removeClass('in')
  }


  // TAB PLUGIN DEFINITION
  // =====================

  function Plugin(option) {
    return this.each(function () {
      var $this = $(this)
      var data  = $this.data('bs.tab')

      if (!data) $this.data('bs.tab', (data = new Tab(this)))
      if (typeof option == 'string') data[option]()
    })
  }

  var old = $.fn.tab

  $.fn.tab             = Plugin
  $.fn.tab.Constructor = Tab


  // TAB NO CONFLICT
  // ===============

  $.fn.tab.noConflict = function () {
    $.fn.tab = old
    return this
  }


  // TAB DATA-API
  // ============

  var clickHandler = function (e) {
    e.preventDefault()
    Plugin.call($(this), 'show')
  }

  $(document)
    .on('click.bs.tab.data-api', '[data-toggle="tab"]', clickHandler)
    .on('click.bs.tab.data-api', '[data-toggle="pill"]', clickHandler)

}(jQuery);

/* ========================================================================
 * Bootstrap: affix.js v3.3.6
 * http://getbootstrap.com/javascript/#affix
 * ========================================================================
 * Copyright 2011-2015 Twitter, Inc.
 * Licensed under MIT (https://github.com/twbs/bootstrap/blob/master/LICENSE)
 * ======================================================================== */


+function ($) {
  'use strict';

  // AFFIX CLASS DEFINITION
  // ======================

  var Affix = function (element, options) {
    this.options = $.extend({}, Affix.DEFAULTS, options)

    this.$target = $(this.options.target)
      .on('scroll.bs.affix.data-api', $.proxy(this.checkPosition, this))
      .on('click.bs.affix.data-api',  $.proxy(this.checkPositionWithEventLoop, this))

    this.$element     = $(element)
    this.affixed      = null
    this.unpin        = null
    this.pinnedOffset = null

    this.checkPosition()
  }

  Affix.VERSION  = '3.3.6'

  Affix.RESET    = 'affix affix-top affix-bottom'

  Affix.DEFAULTS = {
    offset: 0,
    target: window
  }

  Affix.prototype.getState = function (scrollHeight, height, offsetTop, offsetBottom) {
    var scrollTop    = this.$target.scrollTop()
    var position     = this.$element.offset()
    var targetHeight = this.$target.height()

    if (offsetTop != null && this.affixed == 'top') return scrollTop < offsetTop ? 'top' : false

    if (this.affixed == 'bottom') {
      if (offsetTop != null) return (scrollTop + this.unpin <= position.top) ? false : 'bottom'
      return (scrollTop + targetHeight <= scrollHeight - offsetBottom) ? false : 'bottom'
    }

    var initializing   = this.affixed == null
    var colliderTop    = initializing ? scrollTop : position.top
    var colliderHeight = initializing ? targetHeight : height

    if (offsetTop != null && scrollTop <= offsetTop) return 'top'
    if (offsetBottom != null && (colliderTop + colliderHeight >= scrollHeight - offsetBottom)) return 'bottom'

    return false
  }

  Affix.prototype.getPinnedOffset = function () {
    if (this.pinnedOffset) return this.pinnedOffset
    this.$element.removeClass(Affix.RESET).addClass('affix')
    var scrollTop = this.$target.scrollTop()
    var position  = this.$element.offset()
    return (this.pinnedOffset = position.top - scrollTop)
  }

  Affix.prototype.checkPositionWithEventLoop = function () {
    setTimeout($.proxy(this.checkPosition, this), 1)
  }

  Affix.prototype.checkPosition = function () {
    if (!this.$element.is(':visible')) return

    var height       = this.$element.height()
    var offset       = this.options.offset
    var offsetTop    = offset.top
    var offsetBottom = offset.bottom
    var scrollHeight = Math.max($(document).height(), $(document.body).height())

    if (typeof offset != 'object')         offsetBottom = offsetTop = offset
    if (typeof offsetTop == 'function')    offsetTop    = offset.top(this.$element)
    if (typeof offsetBottom == 'function') offsetBottom = offset.bottom(this.$element)

    var affix = this.getState(scrollHeight, height, offsetTop, offsetBottom)

    if (this.affixed != affix) {
      if (this.unpin != null) this.$element.css('top', '')

      var affixType = 'affix' + (affix ? '-' + affix : '')
      var e         = $.Event(affixType + '.bs.affix')

      this.$element.trigger(e)

      if (e.isDefaultPrevented()) return

      this.affixed = affix
      this.unpin = affix == 'bottom' ? this.getPinnedOffset() : null

      this.$element
        .removeClass(Affix.RESET)
        .addClass(affixType)
        .trigger(affixType.replace('affix', 'affixed') + '.bs.affix')
    }

    if (affix == 'bottom') {
      this.$element.offset({
        top: scrollHeight - height - offsetBottom
      })
    }
  }


  // AFFIX PLUGIN DEFINITION
  // =======================

  function Plugin(option) {
    return this.each(function () {
      var $this   = $(this)
      var data    = $this.data('bs.affix')
      var options = typeof option == 'object' && option

      if (!data) $this.data('bs.affix', (data = new Affix(this, options)))
      if (typeof option == 'string') data[option]()
    })
  }

  var old = $.fn.affix

  $.fn.affix             = Plugin
  $.fn.affix.Constructor = Affix


  // AFFIX NO CONFLICT
  // =================

  $.fn.affix.noConflict = function () {
    $.fn.affix = old
    return this
  }


  // AFFIX DATA-API
  // ==============

  $(window).on('load', function () {
    $('[data-spy="affix"]').each(function () {
      var $spy = $(this)
      var data = $spy.data()

      data.offset = data.offset || {}

      if (data.offsetBottom != null) data.offset.bottom = data.offsetBottom
      if (data.offsetTop    != null) data.offset.top    = data.offsetTop

      Plugin.call($spy, data)
    })
  })

}(jQuery);

/*
 HTML5 Shiv v3.6.2 | @afarkas @jdalton @jon_neal @rem | MIT/GPL2 Licensed
*/
(function(l,f){function m(){var a=e.elements;return"string"==typeof a?a.split(" "):a}function i(a){var b=n[a[o]];b||(b={},h++,a[o]=h,n[h]=b);return b}function p(a,b,c){b||(b=f);if(g)return b.createElement(a);c||(c=i(b));b=c.cache[a]?c.cache[a].cloneNode():r.test(a)?(c.cache[a]=c.createElem(a)).cloneNode():c.createElem(a);return b.canHaveChildren&&!s.test(a)?c.frag.appendChild(b):b}function t(a,b){if(!b.cache)b.cache={},b.createElem=a.createElement,b.createFrag=a.createDocumentFragment,b.frag=b.createFrag();
a.createElement=function(c){return!e.shivMethods?b.createElem(c):p(c,a,b)};a.createDocumentFragment=Function("h,f","return function(){var n=f.cloneNode(),c=n.createElement;h.shivMethods&&("+m().join().replace(/\w+/g,function(a){b.createElem(a);b.frag.createElement(a);return'c("'+a+'")'})+");return n}")(e,b.frag)}function q(a){a||(a=f);var b=i(a);if(e.shivCSS&&!j&&!b.hasCSS){var c,d=a;c=d.createElement("p");d=d.getElementsByTagName("head")[0]||d.documentElement;c.innerHTML="x<style>article,aside,figcaption,figure,footer,header,hgroup,main,nav,section{display:block}mark{background:#FF0;color:#000}</style>";
c=d.insertBefore(c.lastChild,d.firstChild);b.hasCSS=!!c}g||t(a,b);return a}var k=l.html5||{},s=/^<|^(?:button|map|select|textarea|object|iframe|option|optgroup)$/i,r=/^(?:a|b|code|div|fieldset|h1|h2|h3|h4|h5|h6|i|label|li|ol|p|q|span|strong|style|table|tbody|td|th|tr|ul)$/i,j,o="_html5shiv",h=0,n={},g;(function(){try{var a=f.createElement("a");a.innerHTML="<xyz></xyz>";j="hidden"in a;var b;if(!(b=1==a.childNodes.length)){f.createElement("a");var c=f.createDocumentFragment();b="undefined"==typeof c.cloneNode||
"undefined"==typeof c.createDocumentFragment||"undefined"==typeof c.createElement}g=b}catch(d){g=j=!0}})();var e={elements:k.elements||"abbr article aside audio bdi canvas data datalist details figcaption figure footer header hgroup main mark meter nav output progress section summary time video",version:"3.6.2",shivCSS:!1!==k.shivCSS,supportsUnknownElements:g,shivMethods:!1!==k.shivMethods,type:"default",shivDocument:q,createElement:p,createDocumentFragment:function(a,b){a||(a=f);if(g)return a.createDocumentFragment();
for(var b=b||i(a),c=b.frag.cloneNode(),d=0,e=m(),h=e.length;d<h;d++)c.createElement(e[d]);return c}};l.html5=e;q(f)})(this,document);

/*!
 * JavaScript Cookie v2.1.0
 * https://github.com/js-cookie/js-cookie
 *
 * Copyright 2006, 2015 Klaus Hartl & Fagner Brack
 * Released under the MIT license
 */
(function (factory) {
	if (typeof define === 'function' && define.amd) {
		define(factory);
	} else if (typeof exports === 'object') {
		module.exports = factory();
	} else {
		var _OldCookies = window.Cookies;
		var api = window.Cookies = factory();
		api.noConflict = function () {
			window.Cookies = _OldCookies;
			return api;
		};
	}
}(function () {
	function extend () {
		var i = 0;
		var result = {};
		for (; i < arguments.length; i++) {
			var attributes = arguments[ i ];
			for (var key in attributes) {
				result[key] = attributes[key];
			}
		}
		return result;
	}

	function init (converter) {
		function api (key, value, attributes) {
			var result;

			// Write

			if (arguments.length > 1) {
				attributes = extend({
					path: '/'
				}, api.defaults, attributes);

				if (typeof attributes.expires === 'number') {
					var expires = new Date();
					expires.setMilliseconds(expires.getMilliseconds() + attributes.expires * 864e+5);
					attributes.expires = expires;
				}

				try {
					result = JSON.stringify(value);
					if (/^[\{\[]/.test(result)) {
						value = result;
					}
				} catch (e) {}

				if (!converter.write) {
					value = encodeURIComponent(String(value))
						.replace(/%(23|24|26|2B|3A|3C|3E|3D|2F|3F|40|5B|5D|5E|60|7B|7D|7C)/g, decodeURIComponent);
				} else {
					value = converter.write(value, key);
				}

				key = encodeURIComponent(String(key));
				key = key.replace(/%(23|24|26|2B|5E|60|7C)/g, decodeURIComponent);
				key = key.replace(/[\(\)]/g, escape);

				return (document.cookie = [
					key, '=', value,
					attributes.expires && '; expires=' + attributes.expires.toUTCString(), // use expires attribute, max-age is not supported by IE
					attributes.path    && '; path=' + attributes.path,
					attributes.domain  && '; domain=' + attributes.domain,
					attributes.secure ? '; secure' : ''
				].join(''));
			}

			// Read

			if (!key) {
				result = {};
			}

			// To prevent the for loop in the first place assign an empty array
			// in case there are no cookies at all. Also prevents odd result when
			// calling "get()"
			var cookies = document.cookie ? document.cookie.split('; ') : [];
			var rdecode = /(%[0-9A-Z]{2})+/g;
			var i = 0;

			for (; i < cookies.length; i++) {
				var parts = cookies[i].split('=');
				var name = parts[0].replace(rdecode, decodeURIComponent);
				var cookie = parts.slice(1).join('=');

				if (cookie.charAt(0) === '"') {
					cookie = cookie.slice(1, -1);
				}

				try {
					cookie = converter.read ?
						converter.read(cookie, name) : converter(cookie, name) ||
						cookie.replace(rdecode, decodeURIComponent);

					if (this.json) {
						try {
							cookie = JSON.parse(cookie);
						} catch (e) {}
					}

					if (key === name) {
						result = cookie;
						break;
					}

					if (!key) {
						result[name] = cookie;
					}
				} catch (e) {}
			}

			return result;
		}

		api.get = api.set = api;
		api.getJSON = function () {
			return api.apply({
				json: true
			}, [].slice.call(arguments));
		};
		api.defaults = {};

		api.remove = function (key, attributes) {
			api(key, '', extend(attributes, {
				expires: -1
			}));
		};

		api.withConverter = init;

		return api;
	}

	return init(function () {});
}));

/*! Respond.js v1.4.2: min/max-width media query polyfill
 * Copyright 2014 Scott Jehl
 * Licensed under MIT
 * http://j.mp/respondjs */

!function(a){"use strict";a.matchMedia=a.matchMedia||function(a){var b,c=a.documentElement,d=c.firstElementChild||c.firstChild,e=a.createElement("body"),f=a.createElement("div");return f.id="mq-test-1",f.style.cssText="position:absolute;top:-100em",e.style.background="none",e.appendChild(f),function(a){return f.innerHTML='&shy;<style media="'+a+'"> #mq-test-1 { width: 42px; }</style>',c.insertBefore(e,d),b=42===f.offsetWidth,c.removeChild(e),{matches:b,media:a}}}(a.document)}(this),function(a){"use strict";function b(){v(!0)}var c={};a.respond=c,c.update=function(){};var d=[],e=function(){var b=!1;try{b=new a.XMLHttpRequest}catch(c){b=new a.ActiveXObject("Microsoft.XMLHTTP")}return function(){return b}}(),f=function(a,b){var c=e();c&&(c.open("GET",a,!0),c.onreadystatechange=function(){4!==c.readyState||200!==c.status&&304!==c.status||b(c.responseText)},4!==c.readyState&&c.send(null))},g=function(a){return a.replace(c.regex.minmaxwh,"").match(c.regex.other)};if(c.ajax=f,c.queue=d,c.unsupportedmq=g,c.regex={media:/@media[^\{]+\{([^\{\}]*\{[^\}\{]*\})+/gi,keyframes:/@(?:\-(?:o|moz|webkit)\-)?keyframes[^\{]+\{(?:[^\{\}]*\{[^\}\{]*\})+[^\}]*\}/gi,comments:/\/\*[^*]*\*+([^/][^*]*\*+)*\//gi,urls:/(url\()['"]?([^\/\)'"][^:\)'"]+)['"]?(\))/g,findStyles:/@media *([^\{]+)\{([\S\s]+?)$/,only:/(only\s+)?([a-zA-Z]+)\s?/,minw:/\(\s*min\-width\s*:\s*(\s*[0-9\.]+)(px|em)\s*\)/,maxw:/\(\s*max\-width\s*:\s*(\s*[0-9\.]+)(px|em)\s*\)/,minmaxwh:/\(\s*m(in|ax)\-(height|width)\s*:\s*(\s*[0-9\.]+)(px|em)\s*\)/gi,other:/\([^\)]*\)/g},c.mediaQueriesSupported=a.matchMedia&&null!==a.matchMedia("only all")&&a.matchMedia("only all").matches,!c.mediaQueriesSupported){var h,i,j,k=a.document,l=k.documentElement,m=[],n=[],o=[],p={},q=30,r=k.getElementsByTagName("head")[0]||l,s=k.getElementsByTagName("base")[0],t=r.getElementsByTagName("link"),u=function(){var a,b=k.createElement("div"),c=k.body,d=l.style.fontSize,e=c&&c.style.fontSize,f=!1;return b.style.cssText="position:absolute;font-size:1em;width:1em",c||(c=f=k.createElement("body"),c.style.background="none"),l.style.fontSize="100%",c.style.fontSize="100%",c.appendChild(b),f&&l.insertBefore(c,l.firstChild),a=b.offsetWidth,f?l.removeChild(c):c.removeChild(b),l.style.fontSize=d,e&&(c.style.fontSize=e),a=j=parseFloat(a)},v=function(b){var c="clientWidth",d=l[c],e="CSS1Compat"===k.compatMode&&d||k.body[c]||d,f={},g=t[t.length-1],p=(new Date).getTime();if(b&&h&&q>p-h)return a.clearTimeout(i),i=a.setTimeout(v,q),void 0;h=p;for(var s in m)if(m.hasOwnProperty(s)){var w=m[s],x=w.minw,y=w.maxw,z=null===x,A=null===y,B="em";x&&(x=parseFloat(x)*(x.indexOf(B)>-1?j||u():1)),y&&(y=parseFloat(y)*(y.indexOf(B)>-1?j||u():1)),w.hasquery&&(z&&A||!(z||e>=x)||!(A||y>=e))||(f[w.media]||(f[w.media]=[]),f[w.media].push(n[w.rules]))}for(var C in o)o.hasOwnProperty(C)&&o[C]&&o[C].parentNode===r&&r.removeChild(o[C]);o.length=0;for(var D in f)if(f.hasOwnProperty(D)){var E=k.createElement("style"),F=f[D].join("\n");E.type="text/css",E.media=D,r.insertBefore(E,g.nextSibling),E.styleSheet?E.styleSheet.cssText=F:E.appendChild(k.createTextNode(F)),o.push(E)}},w=function(a,b,d){var e=a.replace(c.regex.comments,"").replace(c.regex.keyframes,"").match(c.regex.media),f=e&&e.length||0;b=b.substring(0,b.lastIndexOf("/"));var h=function(a){return a.replace(c.regex.urls,"$1"+b+"$2$3")},i=!f&&d;b.length&&(b+="/"),i&&(f=1);for(var j=0;f>j;j++){var k,l,o,p;i?(k=d,n.push(h(a))):(k=e[j].match(c.regex.findStyles)&&RegExp.$1,n.push(RegExp.$2&&h(RegExp.$2))),o=k.split(","),p=o.length;for(var q=0;p>q;q++)l=o[q],g(l)||m.push({media:l.split("(")[0].match(c.regex.only)&&RegExp.$2||"all",rules:n.length-1,hasquery:l.indexOf("(")>-1,minw:l.match(c.regex.minw)&&parseFloat(RegExp.$1)+(RegExp.$2||""),maxw:l.match(c.regex.maxw)&&parseFloat(RegExp.$1)+(RegExp.$2||"")})}v()},x=function(){if(d.length){var b=d.shift();f(b.href,function(c){w(c,b.href,b.media),p[b.href]=!0,a.setTimeout(function(){x()},0)})}},y=function(){for(var b=0;b<t.length;b++){var c=t[b],e=c.href,f=c.media,g=c.rel&&"stylesheet"===c.rel.toLowerCase();e&&g&&!p[e]&&(c.styleSheet&&c.styleSheet.rawCssText?(w(c.styleSheet.rawCssText,e,f),p[e]=!0):(!/^([a-zA-Z:]*\/\/)/.test(e)&&!s||e.replace(RegExp.$1,"").split("/")[0]===a.location.host)&&("//"===e.substring(0,2)&&(e=a.location.protocol+e),d.push({href:e,media:f})))}x()};y(),c.update=y,c.getEmValue=u,a.addEventListener?a.addEventListener("resize",b,!1):a.attachEvent&&a.attachEvent("onresize",b)}}(this);
/*!
 * screenfull
 * v3.0.0 - 2015-11-24
 * (c) Sindre Sorhus; MIT License
 */
(function () {
    'use strict';

    var isCommonjs = typeof module !== 'undefined' && module.exports;
    var keyboardAllowed = typeof Element !== 'undefined' && 'ALLOW_KEYBOARD_INPUT' in Element;

    var fn = (function () {
        var val;
        var valLength;

        var fnMap = [
            [
                'requestFullscreen',
                'exitFullscreen',
                'fullscreenElement',
                'fullscreenEnabled',
                'fullscreenchange',
                'fullscreenerror'
            ],
            // new WebKit
            [
                'webkitRequestFullscreen',
                'webkitExitFullscreen',
                'webkitFullscreenElement',
                'webkitFullscreenEnabled',
                'webkitfullscreenchange',
                'webkitfullscreenerror'

            ],
            // old WebKit (Safari 5.1)
            [
                'webkitRequestFullScreen',
                'webkitCancelFullScreen',
                'webkitCurrentFullScreenElement',
                'webkitCancelFullScreen',
                'webkitfullscreenchange',
                'webkitfullscreenerror'

            ],
            [
                'mozRequestFullScreen',
                'mozCancelFullScreen',
                'mozFullScreenElement',
                'mozFullScreenEnabled',
                'mozfullscreenchange',
                'mozfullscreenerror'
            ],
            [
                'msRequestFullscreen',
                'msExitFullscreen',
                'msFullscreenElement',
                'msFullscreenEnabled',
                'MSFullscreenChange',
                'MSFullscreenError'
            ]
        ];

        var i = 0;
        var l = fnMap.length;
        var ret = {};

        for (; i < l; i++) {
            val = fnMap[i];
            if (val && val[1] in document) {
                for (i = 0, valLength = val.length; i < valLength; i++) {
                    ret[fnMap[0][i]] = val[i];
                }
                return ret;
            }
        }

        return false;
    })();

    var screenfull = {
        request: function (elem) {
            var request = fn.requestFullscreen;

            elem = elem || document.documentElement;

            // Work around Safari 5.1 bug: reports support for
            // keyboard in fullscreen even though it doesn't.
            // Browser sniffing, since the alternative with
            // setTimeout is even worse.
            if (/5\.1[\.\d]* Safari/.test(navigator.userAgent)) {
                elem[request]();
            } else {
                elem[request](keyboardAllowed && Element.ALLOW_KEYBOARD_INPUT);
            }
        },
        exit: function () {
            document[fn.exitFullscreen]();
        },
        toggle: function (elem) {
            if (this.isFullscreen) {
                this.exit();
            } else {
                this.request(elem);
            }
        },
        raw: fn
    };

    if (!fn) {
        if (isCommonjs) {
            module.exports = false;
        } else {
            window.screenfull = false;
        }

        return;
    }

    Object.defineProperties(screenfull, {
        isFullscreen: {
            get: function () {
                return Boolean(document[fn.fullscreenElement]);
            }
        },
        element: {
            enumerable: true,
            get: function () {
                return document[fn.fullscreenElement];
            }
        },
        enabled: {
            enumerable: true,
            get: function () {
                // Coerce to boolean in case of old WebKit
                return Boolean(document[fn.fullscreenEnabled]);
            }
        }
    });

    if (isCommonjs) {
        module.exports = screenfull;
    } else {
        window.screenfull = screenfull;
    }
})();

!function(e){if("object"==typeof exports&&"undefined"!=typeof module)module.exports=e();else if("function"==typeof define&&define.amd)define([],e);else{var f;"undefined"!=typeof window?f=window:"undefined"!=typeof global?f=global:"undefined"!=typeof self&&(f=self),f.io=e()}}(function(){var define,module,exports;return (function e(t,n,r){function s(o,u){if(!n[o]){if(!t[o]){var a=typeof require=="function"&&require;if(!u&&a)return a(o,!0);if(i)return i(o,!0);throw new Error("Cannot find module '"+o+"'")}var f=n[o]={exports:{}};t[o][0].call(f.exports,function(e){var n=t[o][1][e];return s(n?n:e)},f,f.exports,e,t,n,r)}return n[o].exports}var i=typeof require=="function"&&require;for(var o=0;o<r.length;o++)s(r[o]);return s})({1:[function(_dereq_,module,exports){

module.exports = _dereq_('./lib/');

},{"./lib/":2}],2:[function(_dereq_,module,exports){

/**
 * Module dependencies.
 */

var url = _dereq_('./url');
var parser = _dereq_('socket.io-parser');
var Manager = _dereq_('./manager');
var debug = _dereq_('debug')('socket.io-client');

/**
 * Module exports.
 */

module.exports = exports = lookup;

/**
 * Managers cache.
 */

var cache = exports.managers = {};

/**
 * Looks up an existing `Manager` for multiplexing.
 * If the user summons:
 *
 *   `io('http://localhost/a');`
 *   `io('http://localhost/b');`
 *
 * We reuse the existing instance based on same scheme/port/host,
 * and we initialize sockets for each namespace.
 *
 * @api public
 */

function lookup(uri, opts) {
  if (typeof uri == 'object') {
    opts = uri;
    uri = undefined;
  }

  opts = opts || {};

  var parsed = url(uri);
  var source = parsed.source;
  var id = parsed.id;
  var path = parsed.path;
  var sameNamespace = (cache[id] && cache[id].nsps[path] &&
                       path == cache[id].nsps[path].nsp);
  var newConnection = opts.forceNew || opts['force new connection'] ||
                      false === opts.multiplex || sameNamespace;

  var io;

  if (newConnection) {
    debug('ignoring socket cache for %s', source);
    io = Manager(source, opts);
  } else {
    if (!cache[id]) {
      debug('new io instance for %s', source);
      cache[id] = Manager(source, opts);
    }
    io = cache[id];
  }

  return io.socket(parsed.path);
}

/**
 * Protocol version.
 *
 * @api public
 */

exports.protocol = parser.protocol;

/**
 * `connect`.
 *
 * @param {String} uri
 * @api public
 */

exports.connect = lookup;

/**
 * Expose constructors for standalone build.
 *
 * @api public
 */

exports.Manager = _dereq_('./manager');
exports.Socket = _dereq_('./socket');

},{"./manager":3,"./socket":5,"./url":6,"debug":10,"socket.io-parser":44}],3:[function(_dereq_,module,exports){

/**
 * Module dependencies.
 */

var url = _dereq_('./url');
var eio = _dereq_('engine.io-client');
var Socket = _dereq_('./socket');
var Emitter = _dereq_('component-emitter');
var parser = _dereq_('socket.io-parser');
var on = _dereq_('./on');
var bind = _dereq_('component-bind');
var object = _dereq_('object-component');
var debug = _dereq_('debug')('socket.io-client:manager');
var indexOf = _dereq_('indexof');
var Backoff = _dereq_('backo2');

/**
 * Module exports
 */

module.exports = Manager;

/**
 * `Manager` constructor.
 *
 * @param {String} engine instance or engine uri/opts
 * @param {Object} options
 * @api public
 */

function Manager(uri, opts){
  if (!(this instanceof Manager)) return new Manager(uri, opts);
  if (uri && ('object' == typeof uri)) {
    opts = uri;
    uri = undefined;
  }
  opts = opts || {};

  opts.path = opts.path || '/socket.io';
  this.nsps = {};
  this.subs = [];
  this.opts = opts;
  this.reconnection(opts.reconnection !== false);
  this.reconnectionAttempts(opts.reconnectionAttempts || Infinity);
  this.reconnectionDelay(opts.reconnectionDelay || 1000);
  this.reconnectionDelayMax(opts.reconnectionDelayMax || 5000);
  this.randomizationFactor(opts.randomizationFactor || 0.5);
  this.backoff = new Backoff({
    min: this.reconnectionDelay(),
    max: this.reconnectionDelayMax(),
    jitter: this.randomizationFactor()
  });
  this.timeout(null == opts.timeout ? 20000 : opts.timeout);
  this.readyState = 'closed';
  this.uri = uri;
  this.connected = [];
  this.encoding = false;
  this.packetBuffer = [];
  this.encoder = new parser.Encoder();
  this.decoder = new parser.Decoder();
  this.autoConnect = opts.autoConnect !== false;
  if (this.autoConnect) this.open();
}

/**
 * Propagate given event to sockets and emit on `this`
 *
 * @api private
 */

Manager.prototype.emitAll = function() {
  this.emit.apply(this, arguments);
  for (var nsp in this.nsps) {
    this.nsps[nsp].emit.apply(this.nsps[nsp], arguments);
  }
};

/**
 * Update `socket.id` of all sockets
 *
 * @api private
 */

Manager.prototype.updateSocketIds = function(){
  for (var nsp in this.nsps) {
    this.nsps[nsp].id = this.engine.id;
  }
};

/**
 * Mix in `Emitter`.
 */

Emitter(Manager.prototype);

/**
 * Sets the `reconnection` config.
 *
 * @param {Boolean} true/false if it should automatically reconnect
 * @return {Manager} self or value
 * @api public
 */

Manager.prototype.reconnection = function(v){
  if (!arguments.length) return this._reconnection;
  this._reconnection = !!v;
  return this;
};

/**
 * Sets the reconnection attempts config.
 *
 * @param {Number} max reconnection attempts before giving up
 * @return {Manager} self or value
 * @api public
 */

Manager.prototype.reconnectionAttempts = function(v){
  if (!arguments.length) return this._reconnectionAttempts;
  this._reconnectionAttempts = v;
  return this;
};

/**
 * Sets the delay between reconnections.
 *
 * @param {Number} delay
 * @return {Manager} self or value
 * @api public
 */

Manager.prototype.reconnectionDelay = function(v){
  if (!arguments.length) return this._reconnectionDelay;
  this._reconnectionDelay = v;
  this.backoff && this.backoff.setMin(v);
  return this;
};

Manager.prototype.randomizationFactor = function(v){
  if (!arguments.length) return this._randomizationFactor;
  this._randomizationFactor = v;
  this.backoff && this.backoff.setJitter(v);
  return this;
};

/**
 * Sets the maximum delay between reconnections.
 *
 * @param {Number} delay
 * @return {Manager} self or value
 * @api public
 */

Manager.prototype.reconnectionDelayMax = function(v){
  if (!arguments.length) return this._reconnectionDelayMax;
  this._reconnectionDelayMax = v;
  this.backoff && this.backoff.setMax(v);
  return this;
};

/**
 * Sets the connection timeout. `false` to disable
 *
 * @return {Manager} self or value
 * @api public
 */

Manager.prototype.timeout = function(v){
  if (!arguments.length) return this._timeout;
  this._timeout = v;
  return this;
};

/**
 * Starts trying to reconnect if reconnection is enabled and we have not
 * started reconnecting yet
 *
 * @api private
 */

Manager.prototype.maybeReconnectOnOpen = function() {
  // Only try to reconnect if it's the first time we're connecting
  if (!this.reconnecting && this._reconnection && this.backoff.attempts === 0) {
    // keeps reconnection from firing twice for the same reconnection loop
    this.reconnect();
  }
};


/**
 * Sets the current transport `socket`.
 *
 * @param {Function} optional, callback
 * @return {Manager} self
 * @api public
 */

Manager.prototype.open =
Manager.prototype.connect = function(fn){
  debug('readyState %s', this.readyState);
  if (~this.readyState.indexOf('open')) return this;

  debug('opening %s', this.uri);
  this.engine = eio(this.uri, this.opts);
  var socket = this.engine;
  var self = this;
  this.readyState = 'opening';
  this.skipReconnect = false;

  // emit `open`
  var openSub = on(socket, 'open', function() {
    self.onopen();
    fn && fn();
  });

  // emit `connect_error`
  var errorSub = on(socket, 'error', function(data){
    debug('connect_error');
    self.cleanup();
    self.readyState = 'closed';
    self.emitAll('connect_error', data);
    if (fn) {
      var err = new Error('Connection error');
      err.data = data;
      fn(err);
    } else {
      // Only do this if there is no fn to handle the error
      self.maybeReconnectOnOpen();
    }
  });

  // emit `connect_timeout`
  if (false !== this._timeout) {
    var timeout = this._timeout;
    debug('connect attempt will timeout after %d', timeout);

    // set timer
    var timer = setTimeout(function(){
      debug('connect attempt timed out after %d', timeout);
      openSub.destroy();
      socket.close();
      socket.emit('error', 'timeout');
      self.emitAll('connect_timeout', timeout);
    }, timeout);

    this.subs.push({
      destroy: function(){
        clearTimeout(timer);
      }
    });
  }

  this.subs.push(openSub);
  this.subs.push(errorSub);

  return this;
};

/**
 * Called upon transport open.
 *
 * @api private
 */

Manager.prototype.onopen = function(){
  debug('open');

  // clear old subs
  this.cleanup();

  // mark as open
  this.readyState = 'open';
  this.emit('open');

  // add new subs
  var socket = this.engine;
  this.subs.push(on(socket, 'data', bind(this, 'ondata')));
  this.subs.push(on(this.decoder, 'decoded', bind(this, 'ondecoded')));
  this.subs.push(on(socket, 'error', bind(this, 'onerror')));
  this.subs.push(on(socket, 'close', bind(this, 'onclose')));
};

/**
 * Called with data.
 *
 * @api private
 */

Manager.prototype.ondata = function(data){
  this.decoder.add(data);
};

/**
 * Called when parser fully decodes a packet.
 *
 * @api private
 */

Manager.prototype.ondecoded = function(packet) {
  this.emit('packet', packet);
};

/**
 * Called upon socket error.
 *
 * @api private
 */

Manager.prototype.onerror = function(err){
  debug('error', err);
  this.emitAll('error', err);
};

/**
 * Creates a new socket for the given `nsp`.
 *
 * @return {Socket}
 * @api public
 */

Manager.prototype.socket = function(nsp){
  var socket = this.nsps[nsp];
  if (!socket) {
    socket = new Socket(this, nsp);
    this.nsps[nsp] = socket;
    var self = this;
    socket.on('connect', function(){
      socket.id = self.engine.id;
      if (!~indexOf(self.connected, socket)) {
        self.connected.push(socket);
      }
    });
  }
  return socket;
};

/**
 * Called upon a socket close.
 *
 * @param {Socket} socket
 */

Manager.prototype.destroy = function(socket){
  var index = indexOf(this.connected, socket);
  if (~index) this.connected.splice(index, 1);
  if (this.connected.length) return;

  this.close();
};

/**
 * Writes a packet.
 *
 * @param {Object} packet
 * @api private
 */

Manager.prototype.packet = function(packet){
  debug('writing packet %j', packet);
  var self = this;

  if (!self.encoding) {
    // encode, then write to engine with result
    self.encoding = true;
    this.encoder.encode(packet, function(encodedPackets) {
      for (var i = 0; i < encodedPackets.length; i++) {
        self.engine.write(encodedPackets[i]);
      }
      self.encoding = false;
      self.processPacketQueue();
    });
  } else { // add packet to the queue
    self.packetBuffer.push(packet);
  }
};

/**
 * If packet buffer is non-empty, begins encoding the
 * next packet in line.
 *
 * @api private
 */

Manager.prototype.processPacketQueue = function() {
  if (this.packetBuffer.length > 0 && !this.encoding) {
    var pack = this.packetBuffer.shift();
    this.packet(pack);
  }
};

/**
 * Clean up transport subscriptions and packet buffer.
 *
 * @api private
 */

Manager.prototype.cleanup = function(){
  var sub;
  while (sub = this.subs.shift()) sub.destroy();

  this.packetBuffer = [];
  this.encoding = false;

  this.decoder.destroy();
};

/**
 * Close the current socket.
 *
 * @api private
 */

Manager.prototype.close =
Manager.prototype.disconnect = function(){
  this.skipReconnect = true;
  this.backoff.reset();
  this.readyState = 'closed';
  this.engine && this.engine.close();
};

/**
 * Called upon engine close.
 *
 * @api private
 */

Manager.prototype.onclose = function(reason){
  debug('close');
  this.cleanup();
  this.backoff.reset();
  this.readyState = 'closed';
  this.emit('close', reason);
  if (this._reconnection && !this.skipReconnect) {
    this.reconnect();
  }
};

/**
 * Attempt a reconnection.
 *
 * @api private
 */

Manager.prototype.reconnect = function(){
  if (this.reconnecting || this.skipReconnect) return this;

  var self = this;

  if (this.backoff.attempts >= this._reconnectionAttempts) {
    debug('reconnect failed');
    this.backoff.reset();
    this.emitAll('reconnect_failed');
    this.reconnecting = false;
  } else {
    var delay = this.backoff.duration();
    debug('will wait %dms before reconnect attempt', delay);

    this.reconnecting = true;
    var timer = setTimeout(function(){
      if (self.skipReconnect) return;

      debug('attempting reconnect');
      self.emitAll('reconnect_attempt', self.backoff.attempts);
      self.emitAll('reconnecting', self.backoff.attempts);

      // check again for the case socket closed in above events
      if (self.skipReconnect) return;

      self.open(function(err){
        if (err) {
          debug('reconnect attempt error');
          self.reconnecting = false;
          self.reconnect();
          self.emitAll('reconnect_error', err.data);
        } else {
          debug('reconnect success');
          self.onreconnect();
        }
      });
    }, delay);

    this.subs.push({
      destroy: function(){
        clearTimeout(timer);
      }
    });
  }
};

/**
 * Called upon successful reconnect.
 *
 * @api private
 */

Manager.prototype.onreconnect = function(){
  var attempt = this.backoff.attempts;
  this.reconnecting = false;
  this.backoff.reset();
  this.updateSocketIds();
  this.emitAll('reconnect', attempt);
};

},{"./on":4,"./socket":5,"./url":6,"backo2":7,"component-bind":8,"component-emitter":9,"debug":10,"engine.io-client":11,"indexof":40,"object-component":41,"socket.io-parser":44}],4:[function(_dereq_,module,exports){

/**
 * Module exports.
 */

module.exports = on;

/**
 * Helper for subscriptions.
 *
 * @param {Object|EventEmitter} obj with `Emitter` mixin or `EventEmitter`
 * @param {String} event name
 * @param {Function} callback
 * @api public
 */

function on(obj, ev, fn) {
  obj.on(ev, fn);
  return {
    destroy: function(){
      obj.removeListener(ev, fn);
    }
  };
}

},{}],5:[function(_dereq_,module,exports){

/**
 * Module dependencies.
 */

var parser = _dereq_('socket.io-parser');
var Emitter = _dereq_('component-emitter');
var toArray = _dereq_('to-array');
var on = _dereq_('./on');
var bind = _dereq_('component-bind');
var debug = _dereq_('debug')('socket.io-client:socket');
var hasBin = _dereq_('has-binary');

/**
 * Module exports.
 */

module.exports = exports = Socket;

/**
 * Internal events (blacklisted).
 * These events can't be emitted by the user.
 *
 * @api private
 */

var events = {
  connect: 1,
  connect_error: 1,
  connect_timeout: 1,
  disconnect: 1,
  error: 1,
  reconnect: 1,
  reconnect_attempt: 1,
  reconnect_failed: 1,
  reconnect_error: 1,
  reconnecting: 1
};

/**
 * Shortcut to `Emitter#emit`.
 */

var emit = Emitter.prototype.emit;

/**
 * `Socket` constructor.
 *
 * @api public
 */

function Socket(io, nsp){
  this.io = io;
  this.nsp = nsp;
  this.json = this; // compat
  this.ids = 0;
  this.acks = {};
  if (this.io.autoConnect) this.open();
  this.receiveBuffer = [];
  this.sendBuffer = [];
  this.connected = false;
  this.disconnected = true;
}

/**
 * Mix in `Emitter`.
 */

Emitter(Socket.prototype);

/**
 * Subscribe to open, close and packet events
 *
 * @api private
 */

Socket.prototype.subEvents = function() {
  if (this.subs) return;

  var io = this.io;
  this.subs = [
    on(io, 'open', bind(this, 'onopen')),
    on(io, 'packet', bind(this, 'onpacket')),
    on(io, 'close', bind(this, 'onclose'))
  ];
};

/**
 * "Opens" the socket.
 *
 * @api public
 */

Socket.prototype.open =
Socket.prototype.connect = function(){
  if (this.connected) return this;

  this.subEvents();
  this.io.open(); // ensure open
  if ('open' == this.io.readyState) this.onopen();
  return this;
};

/**
 * Sends a `message` event.
 *
 * @return {Socket} self
 * @api public
 */

Socket.prototype.send = function(){
  var args = toArray(arguments);
  args.unshift('message');
  this.emit.apply(this, args);
  return this;
};

/**
 * Override `emit`.
 * If the event is in `events`, it's emitted normally.
 *
 * @param {String} event name
 * @return {Socket} self
 * @api public
 */

Socket.prototype.emit = function(ev){
  if (events.hasOwnProperty(ev)) {
    emit.apply(this, arguments);
    return this;
  }

  var args = toArray(arguments);
  var parserType = parser.EVENT; // default
  if (hasBin(args)) { parserType = parser.BINARY_EVENT; } // binary
  var packet = { type: parserType, data: args };

  // event ack callback
  if ('function' == typeof args[args.length - 1]) {
    debug('emitting packet with ack id %d', this.ids);
    this.acks[this.ids] = args.pop();
    packet.id = this.ids++;
  }

  if (this.connected) {
    this.packet(packet);
  } else {
    this.sendBuffer.push(packet);
  }

  return this;
};

/**
 * Sends a packet.
 *
 * @param {Object} packet
 * @api private
 */

Socket.prototype.packet = function(packet){
  packet.nsp = this.nsp;
  this.io.packet(packet);
};

/**
 * Called upon engine `open`.
 *
 * @api private
 */

Socket.prototype.onopen = function(){
  debug('transport is open - connecting');

  // write connect packet if necessary
  if ('/' != this.nsp) {
    this.packet({ type: parser.CONNECT });
  }
};

/**
 * Called upon engine `close`.
 *
 * @param {String} reason
 * @api private
 */

Socket.prototype.onclose = function(reason){
  debug('close (%s)', reason);
  this.connected = false;
  this.disconnected = true;
  delete this.id;
  this.emit('disconnect', reason);
};

/**
 * Called with socket packet.
 *
 * @param {Object} packet
 * @api private
 */

Socket.prototype.onpacket = function(packet){
  if (packet.nsp != this.nsp) return;

  switch (packet.type) {
    case parser.CONNECT:
      this.onconnect();
      break;

    case parser.EVENT:
      this.onevent(packet);
      break;

    case parser.BINARY_EVENT:
      this.onevent(packet);
      break;

    case parser.ACK:
      this.onack(packet);
      break;

    case parser.BINARY_ACK:
      this.onack(packet);
      break;

    case parser.DISCONNECT:
      this.ondisconnect();
      break;

    case parser.ERROR:
      this.emit('error', packet.data);
      break;
  }
};

/**
 * Called upon a server event.
 *
 * @param {Object} packet
 * @api private
 */

Socket.prototype.onevent = function(packet){
  var args = packet.data || [];
  debug('emitting event %j', args);

  if (null != packet.id) {
    debug('attaching ack callback to event');
    args.push(this.ack(packet.id));
  }

  if (this.connected) {
    emit.apply(this, args);
  } else {
    this.receiveBuffer.push(args);
  }
};

/**
 * Produces an ack callback to emit with an event.
 *
 * @api private
 */

Socket.prototype.ack = function(id){
  var self = this;
  var sent = false;
  return function(){
    // prevent double callbacks
    if (sent) return;
    sent = true;
    var args = toArray(arguments);
    debug('sending ack %j', args);

    var type = hasBin(args) ? parser.BINARY_ACK : parser.ACK;
    self.packet({
      type: type,
      id: id,
      data: args
    });
  };
};

/**
 * Called upon a server acknowlegement.
 *
 * @param {Object} packet
 * @api private
 */

Socket.prototype.onack = function(packet){
  debug('calling ack %s with %j', packet.id, packet.data);
  var fn = this.acks[packet.id];
  fn.apply(this, packet.data);
  delete this.acks[packet.id];
};

/**
 * Called upon server connect.
 *
 * @api private
 */

Socket.prototype.onconnect = function(){
  this.connected = true;
  this.disconnected = false;
  this.emit('connect');
  this.emitBuffered();
};

/**
 * Emit buffered events (received and emitted).
 *
 * @api private
 */

Socket.prototype.emitBuffered = function(){
  var i;
  for (i = 0; i < this.receiveBuffer.length; i++) {
    emit.apply(this, this.receiveBuffer[i]);
  }
  this.receiveBuffer = [];

  for (i = 0; i < this.sendBuffer.length; i++) {
    this.packet(this.sendBuffer[i]);
  }
  this.sendBuffer = [];
};

/**
 * Called upon server disconnect.
 *
 * @api private
 */

Socket.prototype.ondisconnect = function(){
  debug('server disconnect (%s)', this.nsp);
  this.destroy();
  this.onclose('io server disconnect');
};

/**
 * Called upon forced client/server side disconnections,
 * this method ensures the manager stops tracking us and
 * that reconnections don't get triggered for this.
 *
 * @api private.
 */

Socket.prototype.destroy = function(){
  if (this.subs) {
    // clean subscriptions to avoid reconnections
    for (var i = 0; i < this.subs.length; i++) {
      this.subs[i].destroy();
    }
    this.subs = null;
  }

  this.io.destroy(this);
};

/**
 * Disconnects the socket manually.
 *
 * @return {Socket} self
 * @api public
 */

Socket.prototype.close =
Socket.prototype.disconnect = function(){
  if (this.connected) {
    debug('performing disconnect (%s)', this.nsp);
    this.packet({ type: parser.DISCONNECT });
  }

  // remove socket from pool
  this.destroy();

  if (this.connected) {
    // fire events
    this.onclose('io client disconnect');
  }
  return this;
};

},{"./on":4,"component-bind":8,"component-emitter":9,"debug":10,"has-binary":36,"socket.io-parser":44,"to-array":48}],6:[function(_dereq_,module,exports){
(function (global){

/**
 * Module dependencies.
 */

var parseuri = _dereq_('parseuri');
var debug = _dereq_('debug')('socket.io-client:url');

/**
 * Module exports.
 */

module.exports = url;

/**
 * URL parser.
 *
 * @param {String} url
 * @param {Object} An object meant to mimic window.location.
 *                 Defaults to window.location.
 * @api public
 */

function url(uri, loc){
  var obj = uri;

  // default to window.location
  var loc = loc || global.location;
  if (null == uri) uri = loc.protocol + '//' + loc.host;

  // relative path support
  if ('string' == typeof uri) {
    if ('/' == uri.charAt(0)) {
      if ('/' == uri.charAt(1)) {
        uri = loc.protocol + uri;
      } else {
        uri = loc.hostname + uri;
      }
    }

    if (!/^(https?|wss?):\/\//.test(uri)) {
      debug('protocol-less url %s', uri);
      if ('undefined' != typeof loc) {
        uri = loc.protocol + '//' + uri;
      } else {
        uri = 'https://' + uri;
      }
    }

    // parse
    debug('parse %s', uri);
    obj = parseuri(uri);
  }

  // make sure we treat `localhost:80` and `localhost` equally
  if (!obj.port) {
    if (/^(http|ws)$/.test(obj.protocol)) {
      obj.port = '80';
    }
    else if (/^(http|ws)s$/.test(obj.protocol)) {
      obj.port = '443';
    }
  }

  obj.path = obj.path || '/';

  // define unique id
  obj.id = obj.protocol + '://' + obj.host + ':' + obj.port;
  // define href
  obj.href = obj.protocol + '://' + obj.host + (loc && loc.port == obj.port ? '' : (':' + obj.port));

  return obj;
}

}).call(this,typeof self !== "undefined" ? self : typeof window !== "undefined" ? window : {})
},{"debug":10,"parseuri":42}],7:[function(_dereq_,module,exports){

/**
 * Expose `Backoff`.
 */

module.exports = Backoff;

/**
 * Initialize backoff timer with `opts`.
 *
 * - `min` initial timeout in milliseconds [100]
 * - `max` max timeout [10000]
 * - `jitter` [0]
 * - `factor` [2]
 *
 * @param {Object} opts
 * @api public
 */

function Backoff(opts) {
  opts = opts || {};
  this.ms = opts.min || 100;
  this.max = opts.max || 10000;
  this.factor = opts.factor || 2;
  this.jitter = opts.jitter > 0 && opts.jitter <= 1 ? opts.jitter : 0;
  this.attempts = 0;
}

/**
 * Return the backoff duration.
 *
 * @return {Number}
 * @api public
 */

Backoff.prototype.duration = function(){
  var ms = this.ms * Math.pow(this.factor, this.attempts++);
  if (this.jitter) {
    var rand =  Math.random();
    var deviation = Math.floor(rand * this.jitter * ms);
    ms = (Math.floor(rand * 10) & 1) == 0  ? ms - deviation : ms + deviation;
  }
  return Math.min(ms, this.max) | 0;
};

/**
 * Reset the number of attempts.
 *
 * @api public
 */

Backoff.prototype.reset = function(){
  this.attempts = 0;
};

/**
 * Set the minimum duration
 *
 * @api public
 */

Backoff.prototype.setMin = function(min){
  this.ms = min;
};

/**
 * Set the maximum duration
 *
 * @api public
 */

Backoff.prototype.setMax = function(max){
  this.max = max;
};

/**
 * Set the jitter
 *
 * @api public
 */

Backoff.prototype.setJitter = function(jitter){
  this.jitter = jitter;
};


},{}],8:[function(_dereq_,module,exports){
/**
 * Slice reference.
 */

var slice = [].slice;

/**
 * Bind `obj` to `fn`.
 *
 * @param {Object} obj
 * @param {Function|String} fn or string
 * @return {Function}
 * @api public
 */

module.exports = function(obj, fn){
  if ('string' == typeof fn) fn = obj[fn];
  if ('function' != typeof fn) throw new Error('bind() requires a function');
  var args = slice.call(arguments, 2);
  return function(){
    return fn.apply(obj, args.concat(slice.call(arguments)));
  }
};

},{}],9:[function(_dereq_,module,exports){

/**
 * Expose `Emitter`.
 */

module.exports = Emitter;

/**
 * Initialize a new `Emitter`.
 *
 * @api public
 */

function Emitter(obj) {
  if (obj) return mixin(obj);
};

/**
 * Mixin the emitter properties.
 *
 * @param {Object} obj
 * @return {Object}
 * @api private
 */

function mixin(obj) {
  for (var key in Emitter.prototype) {
    obj[key] = Emitter.prototype[key];
  }
  return obj;
}

/**
 * Listen on the given `event` with `fn`.
 *
 * @param {String} event
 * @param {Function} fn
 * @return {Emitter}
 * @api public
 */

Emitter.prototype.on =
Emitter.prototype.addEventListener = function(event, fn){
  this._callbacks = this._callbacks || {};
  (this._callbacks[event] = this._callbacks[event] || [])
    .push(fn);
  return this;
};

/**
 * Adds an `event` listener that will be invoked a single
 * time then automatically removed.
 *
 * @param {String} event
 * @param {Function} fn
 * @return {Emitter}
 * @api public
 */

Emitter.prototype.once = function(event, fn){
  var self = this;
  this._callbacks = this._callbacks || {};

  function on() {
    self.off(event, on);
    fn.apply(this, arguments);
  }

  on.fn = fn;
  this.on(event, on);
  return this;
};

/**
 * Remove the given callback for `event` or all
 * registered callbacks.
 *
 * @param {String} event
 * @param {Function} fn
 * @return {Emitter}
 * @api public
 */

Emitter.prototype.off =
Emitter.prototype.removeListener =
Emitter.prototype.removeAllListeners =
Emitter.prototype.removeEventListener = function(event, fn){
  this._callbacks = this._callbacks || {};

  // all
  if (0 == arguments.length) {
    this._callbacks = {};
    return this;
  }

  // specific event
  var callbacks = this._callbacks[event];
  if (!callbacks) return this;

  // remove all handlers
  if (1 == arguments.length) {
    delete this._callbacks[event];
    return this;
  }

  // remove specific handler
  var cb;
  for (var i = 0; i < callbacks.length; i++) {
    cb = callbacks[i];
    if (cb === fn || cb.fn === fn) {
      callbacks.splice(i, 1);
      break;
    }
  }
  return this;
};

/**
 * Emit `event` with the given args.
 *
 * @param {String} event
 * @param {Mixed} ...
 * @return {Emitter}
 */

Emitter.prototype.emit = function(event){
  this._callbacks = this._callbacks || {};
  var args = [].slice.call(arguments, 1)
    , callbacks = this._callbacks[event];

  if (callbacks) {
    callbacks = callbacks.slice(0);
    for (var i = 0, len = callbacks.length; i < len; ++i) {
      callbacks[i].apply(this, args);
    }
  }

  return this;
};

/**
 * Return array of callbacks for `event`.
 *
 * @param {String} event
 * @return {Array}
 * @api public
 */

Emitter.prototype.listeners = function(event){
  this._callbacks = this._callbacks || {};
  return this._callbacks[event] || [];
};

/**
 * Check if this emitter has `event` handlers.
 *
 * @param {String} event
 * @return {Boolean}
 * @api public
 */

Emitter.prototype.hasListeners = function(event){
  return !! this.listeners(event).length;
};

},{}],10:[function(_dereq_,module,exports){

/**
 * Expose `debug()` as the module.
 */

module.exports = debug;

/**
 * Create a debugger with the given `name`.
 *
 * @param {String} name
 * @return {Type}
 * @api public
 */

function debug(name) {
  if (!debug.enabled(name)) return function(){};

  return function(fmt){
    fmt = coerce(fmt);

    var curr = new Date;
    var ms = curr - (debug[name] || curr);
    debug[name] = curr;

    fmt = name
      + ' '
      + fmt
      + ' +' + debug.humanize(ms);

    // This hackery is required for IE8
    // where `console.log` doesn't have 'apply'
    window.console
      && console.log
      && Function.prototype.apply.call(console.log, console, arguments);
  }
}

/**
 * The currently active debug mode names.
 */

debug.names = [];
debug.skips = [];

/**
 * Enables a debug mode by name. This can include modes
 * separated by a colon and wildcards.
 *
 * @param {String} name
 * @api public
 */

debug.enable = function(name) {
  try {
    localStorage.debug = name;
  } catch(e){}

  var split = (name || '').split(/[\s,]+/)
    , len = split.length;

  for (var i = 0; i < len; i++) {
    name = split[i].replace('*', '.*?');
    if (name[0] === '-') {
      debug.skips.push(new RegExp('^' + name.substr(1) + '$'));
    }
    else {
      debug.names.push(new RegExp('^' + name + '$'));
    }
  }
};

/**
 * Disable debug output.
 *
 * @api public
 */

debug.disable = function(){
  debug.enable('');
};

/**
 * Humanize the given `ms`.
 *
 * @param {Number} m
 * @return {String}
 * @api private
 */

debug.humanize = function(ms) {
  var sec = 1000
    , min = 60 * 1000
    , hour = 60 * min;

  if (ms >= hour) return (ms / hour).toFixed(1) + 'h';
  if (ms >= min) return (ms / min).toFixed(1) + 'm';
  if (ms >= sec) return (ms / sec | 0) + 's';
  return ms + 'ms';
};

/**
 * Returns true if the given mode name is enabled, false otherwise.
 *
 * @param {String} name
 * @return {Boolean}
 * @api public
 */

debug.enabled = function(name) {
  for (var i = 0, len = debug.skips.length; i < len; i++) {
    if (debug.skips[i].test(name)) {
      return false;
    }
  }
  for (var i = 0, len = debug.names.length; i < len; i++) {
    if (debug.names[i].test(name)) {
      return true;
    }
  }
  return false;
};

/**
 * Coerce `val`.
 */

function coerce(val) {
  if (val instanceof Error) return val.stack || val.message;
  return val;
}

// persist

try {
  if (window.localStorage) debug.enable(localStorage.debug);
} catch(e){}

},{}],11:[function(_dereq_,module,exports){

module.exports =  _dereq_('./lib/');

},{"./lib/":12}],12:[function(_dereq_,module,exports){

module.exports = _dereq_('./socket');

/**
 * Exports parser
 *
 * @api public
 *
 */
module.exports.parser = _dereq_('engine.io-parser');

},{"./socket":13,"engine.io-parser":25}],13:[function(_dereq_,module,exports){
(function (global){
/**
 * Module dependencies.
 */

var transports = _dereq_('./transports');
var Emitter = _dereq_('component-emitter');
var debug = _dereq_('debug')('engine.io-client:socket');
var index = _dereq_('indexof');
var parser = _dereq_('engine.io-parser');
var parseuri = _dereq_('parseuri');
var parsejson = _dereq_('parsejson');
var parseqs = _dereq_('parseqs');

/**
 * Module exports.
 */

module.exports = Socket;

/**
 * Noop function.
 *
 * @api private
 */

function noop(){}

/**
 * Socket constructor.
 *
 * @param {String|Object} uri or options
 * @param {Object} options
 * @api public
 */

function Socket(uri, opts){
  if (!(this instanceof Socket)) return new Socket(uri, opts);

  opts = opts || {};

  if (uri && 'object' == typeof uri) {
    opts = uri;
    uri = null;
  }

  if (uri) {
    uri = parseuri(uri);
    opts.host = uri.host;
    opts.secure = uri.protocol == 'https' || uri.protocol == 'wss';
    opts.port = uri.port;
    if (uri.query) opts.query = uri.query;
  }

  this.secure = null != opts.secure ? opts.secure :
    (global.location && 'https:' == location.protocol);

  if (opts.host) {
    var pieces = opts.host.split(':');
    opts.hostname = pieces.shift();
    if (pieces.length) {
      opts.port = pieces.pop();
    } else if (!opts.port) {
      // if no port is specified manually, use the protocol default
      opts.port = this.secure ? '443' : '80';
    }
  }

  this.agent = opts.agent || false;
  this.hostname = opts.hostname ||
    (global.location ? location.hostname : 'localhost');
  this.port = opts.port || (global.location && location.port ?
       location.port :
       (this.secure ? 443 : 80));
  this.query = opts.query || {};
  if ('string' == typeof this.query) this.query = parseqs.decode(this.query);
  this.upgrade = false !== opts.upgrade;
  this.path = (opts.path || '/engine.io').replace(/\/$/, '') + '/';
  this.forceJSONP = !!opts.forceJSONP;
  this.jsonp = false !== opts.jsonp;
  this.forceBase64 = !!opts.forceBase64;
  this.enablesXDR = !!opts.enablesXDR;
  this.timestampParam = opts.timestampParam || 't';
  this.timestampRequests = opts.timestampRequests;
  this.transports = opts.transports || ['polling', 'websocket'];
  this.readyState = '';
  this.writeBuffer = [];
  this.callbackBuffer = [];
  this.policyPort = opts.policyPort || 843;
  this.rememberUpgrade = opts.rememberUpgrade || false;
  this.binaryType = null;
  this.onlyBinaryUpgrades = opts.onlyBinaryUpgrades;

  // SSL options for Node.js client
  this.pfx = opts.pfx || null;
  this.key = opts.key || null;
  this.passphrase = opts.passphrase || null;
  this.cert = opts.cert || null;
  this.ca = opts.ca || null;
  this.ciphers = opts.ciphers || null;
  this.rejectUnauthorized = opts.rejectUnauthorized || null;

  this.open();
}

Socket.priorWebsocketSuccess = false;

/**
 * Mix in `Emitter`.
 */

Emitter(Socket.prototype);

/**
 * Protocol version.
 *
 * @api public
 */

Socket.protocol = parser.protocol; // this is an int

/**
 * Expose deps for legacy compatibility
 * and standalone browser access.
 */

Socket.Socket = Socket;
Socket.Transport = _dereq_('./transport');
Socket.transports = _dereq_('./transports');
Socket.parser = _dereq_('engine.io-parser');

/**
 * Creates transport of the given type.
 *
 * @param {String} transport name
 * @return {Transport}
 * @api private
 */

Socket.prototype.createTransport = function (name) {
  debug('creating transport "%s"', name);
  var query = clone(this.query);

  // append engine.io protocol identifier
  query.EIO = parser.protocol;

  // transport name
  query.transport = name;

  // session id if we already have one
  if (this.id) query.sid = this.id;

  var transport = new transports[name]({
    agent: this.agent,
    hostname: this.hostname,
    port: this.port,
    secure: this.secure,
    path: this.path,
    query: query,
    forceJSONP: this.forceJSONP,
    jsonp: this.jsonp,
    forceBase64: this.forceBase64,
    enablesXDR: this.enablesXDR,
    timestampRequests: this.timestampRequests,
    timestampParam: this.timestampParam,
    policyPort: this.policyPort,
    socket: this,
    pfx: this.pfx,
    key: this.key,
    passphrase: this.passphrase,
    cert: this.cert,
    ca: this.ca,
    ciphers: this.ciphers,
    rejectUnauthorized: this.rejectUnauthorized
  });

  return transport;
};

function clone (obj) {
  var o = {};
  for (var i in obj) {
    if (obj.hasOwnProperty(i)) {
      o[i] = obj[i];
    }
  }
  return o;
}

/**
 * Initializes transport to use and starts probe.
 *
 * @api private
 */
Socket.prototype.open = function () {
  var transport;
  if (this.rememberUpgrade && Socket.priorWebsocketSuccess && this.transports.indexOf('websocket') != -1) {
    transport = 'websocket';
  } else if (0 == this.transports.length) {
    // Emit error on next tick so it can be listened to
    var self = this;
    setTimeout(function() {
      self.emit('error', 'No transports available');
    }, 0);
    return;
  } else {
    transport = this.transports[0];
  }
  this.readyState = 'opening';

  // Retry with the next transport if the transport is disabled (jsonp: false)
  var transport;
  try {
    transport = this.createTransport(transport);
  } catch (e) {
    this.transports.shift();
    this.open();
    return;
  }

  transport.open();
  this.setTransport(transport);
};

/**
 * Sets the current transport. Disables the existing one (if any).
 *
 * @api private
 */

Socket.prototype.setTransport = function(transport){
  debug('setting transport %s', transport.name);
  var self = this;

  if (this.transport) {
    debug('clearing existing transport %s', this.transport.name);
    this.transport.removeAllListeners();
  }

  // set up transport
  this.transport = transport;

  // set up transport listeners
  transport
  .on('drain', function(){
    self.onDrain();
  })
  .on('packet', function(packet){
    self.onPacket(packet);
  })
  .on('error', function(e){
    self.onError(e);
  })
  .on('close', function(){
    self.onClose('transport close');
  });
};

/**
 * Probes a transport.
 *
 * @param {String} transport name
 * @api private
 */

Socket.prototype.probe = function (name) {
  debug('probing transport "%s"', name);
  var transport = this.createTransport(name, { probe: 1 })
    , failed = false
    , self = this;

  Socket.priorWebsocketSuccess = false;

  function onTransportOpen(){
    if (self.onlyBinaryUpgrades) {
      var upgradeLosesBinary = !this.supportsBinary && self.transport.supportsBinary;
      failed = failed || upgradeLosesBinary;
    }
    if (failed) return;

    debug('probe transport "%s" opened', name);
    transport.send([{ type: 'ping', data: 'probe' }]);
    transport.once('packet', function (msg) {
      if (failed) return;
      if ('pong' == msg.type && 'probe' == msg.data) {
        debug('probe transport "%s" pong', name);
        self.upgrading = true;
        self.emit('upgrading', transport);
        if (!transport) return;
        Socket.priorWebsocketSuccess = 'websocket' == transport.name;

        debug('pausing current transport "%s"', self.transport.name);
        self.transport.pause(function () {
          if (failed) return;
          if ('closed' == self.readyState) return;
          debug('changing transport and sending upgrade packet');

          cleanup();

          self.setTransport(transport);
          transport.send([{ type: 'upgrade' }]);
          self.emit('upgrade', transport);
          transport = null;
          self.upgrading = false;
          self.flush();
        });
      } else {
        debug('probe transport "%s" failed', name);
        var err = new Error('probe error');
        err.transport = transport.name;
        self.emit('upgradeError', err);
      }
    });
  }

  function freezeTransport() {
    if (failed) return;

    // Any callback called by transport should be ignored since now
    failed = true;

    cleanup();

    transport.close();
    transport = null;
  }

  //Handle any error that happens while probing
  function onerror(err) {
    var error = new Error('probe error: ' + err);
    error.transport = transport.name;

    freezeTransport();

    debug('probe transport "%s" failed because of error: %s', name, err);

    self.emit('upgradeError', error);
  }

  function onTransportClose(){
    onerror("transport closed");
  }

  //When the socket is closed while we're probing
  function onclose(){
    onerror("socket closed");
  }

  //When the socket is upgraded while we're probing
  function onupgrade(to){
    if (transport && to.name != transport.name) {
      debug('"%s" works - aborting "%s"', to.name, transport.name);
      freezeTransport();
    }
  }

  //Remove all listeners on the transport and on self
  function cleanup(){
    transport.removeListener('open', onTransportOpen);
    transport.removeListener('error', onerror);
    transport.removeListener('close', onTransportClose);
    self.removeListener('close', onclose);
    self.removeListener('upgrading', onupgrade);
  }

  transport.once('open', onTransportOpen);
  transport.once('error', onerror);
  transport.once('close', onTransportClose);

  this.once('close', onclose);
  this.once('upgrading', onupgrade);

  transport.open();

};

/**
 * Called when connection is deemed open.
 *
 * @api public
 */

Socket.prototype.onOpen = function () {
  debug('socket open');
  this.readyState = 'open';
  Socket.priorWebsocketSuccess = 'websocket' == this.transport.name;
  this.emit('open');
  this.flush();

  // we check for `readyState` in case an `open`
  // listener already closed the socket
  if ('open' == this.readyState && this.upgrade && this.transport.pause) {
    debug('starting upgrade probes');
    for (var i = 0, l = this.upgrades.length; i < l; i++) {
      this.probe(this.upgrades[i]);
    }
  }
};

/**
 * Handles a packet.
 *
 * @api private
 */

Socket.prototype.onPacket = function (packet) {
  if ('opening' == this.readyState || 'open' == this.readyState) {
    debug('socket receive: type "%s", data "%s"', packet.type, packet.data);

    this.emit('packet', packet);

    // Socket is live - any packet counts
    this.emit('heartbeat');

    switch (packet.type) {
      case 'open':
        this.onHandshake(parsejson(packet.data));
        break;

      case 'pong':
        this.setPing();
        break;

      case 'error':
        var err = new Error('server error');
        err.code = packet.data;
        this.emit('error', err);
        break;

      case 'message':
        this.emit('data', packet.data);
        this.emit('message', packet.data);
        break;
    }
  } else {
    debug('packet received with socket readyState "%s"', this.readyState);
  }
};

/**
 * Called upon handshake completion.
 *
 * @param {Object} handshake obj
 * @api private
 */

Socket.prototype.onHandshake = function (data) {
  this.emit('handshake', data);
  this.id = data.sid;
  this.transport.query.sid = data.sid;
  this.upgrades = this.filterUpgrades(data.upgrades);
  this.pingInterval = data.pingInterval;
  this.pingTimeout = data.pingTimeout;
  this.onOpen();
  // In case open handler closes socket
  if  ('closed' == this.readyState) return;
  this.setPing();

  // Prolong liveness of socket on heartbeat
  this.removeListener('heartbeat', this.onHeartbeat);
  this.on('heartbeat', this.onHeartbeat);
};

/**
 * Resets ping timeout.
 *
 * @api private
 */

Socket.prototype.onHeartbeat = function (timeout) {
  clearTimeout(this.pingTimeoutTimer);
  var self = this;
  self.pingTimeoutTimer = setTimeout(function () {
    if ('closed' == self.readyState) return;
    self.onClose('ping timeout');
  }, timeout || (self.pingInterval + self.pingTimeout));
};

/**
 * Pings server every `this.pingInterval` and expects response
 * within `this.pingTimeout` or closes connection.
 *
 * @api private
 */

Socket.prototype.setPing = function () {
  var self = this;
  clearTimeout(self.pingIntervalTimer);
  self.pingIntervalTimer = setTimeout(function () {
    debug('writing ping packet - expecting pong within %sms', self.pingTimeout);
    self.ping();
    self.onHeartbeat(self.pingTimeout);
  }, self.pingInterval);
};

/**
* Sends a ping packet.
*
* @api public
*/

Socket.prototype.ping = function () {
  this.sendPacket('ping');
};

/**
 * Called on `drain` event
 *
 * @api private
 */

Socket.prototype.onDrain = function() {
  for (var i = 0; i < this.prevBufferLen; i++) {
    if (this.callbackBuffer[i]) {
      this.callbackBuffer[i]();
    }
  }

  this.writeBuffer.splice(0, this.prevBufferLen);
  this.callbackBuffer.splice(0, this.prevBufferLen);

  // setting prevBufferLen = 0 is very important
  // for example, when upgrading, upgrade packet is sent over,
  // and a nonzero prevBufferLen could cause problems on `drain`
  this.prevBufferLen = 0;

  if (this.writeBuffer.length == 0) {
    this.emit('drain');
  } else {
    this.flush();
  }
};

/**
 * Flush write buffers.
 *
 * @api private
 */

Socket.prototype.flush = function () {
  // console.log(new Date().getTime(), this.readyState, this.transport.writable, this.upgrading, this.writeBuffer.length)
  // if (!this.transport.writable && this.writeBuffer.length){
  //   console.log(11);
  // }
  if ('closed' != this.readyState && this.transport.writable &&
    !this.upgrading && this.writeBuffer.length) {
    // console.log(new Date().getTime(), 'flushing %d packets in socket', this.writeBuffer.length);
    this.transport.send(this.writeBuffer);
    // keep track of current length of writeBuffer
    // splice writeBuffer and callbackBuffer on `drain`
    this.prevBufferLen = this.writeBuffer.length;
    this.emit('flush');
  }
};

/**
 * Sends a message.
 *
 * @param {String} message.
 * @param {Function} callback function.
 * @return {Socket} for chaining.
 * @api public
 */

Socket.prototype.write =
Socket.prototype.send = function (msg, fn) {
  this.sendPacket('message', msg, fn);
  return this;
};

/**
 * Sends a packet.
 *
 * @param {String} packet type.
 * @param {String} data.
 * @param {Function} callback function.
 * @api private
 */

Socket.prototype.sendPacket = function (type, data, fn) {
  // console.log(new Date().getTime(), 'Socket.prototype.sendPacket');
  if ('closing' == this.readyState || 'closed' == this.readyState) {
    return;
  }

  var packet = { type: type, data: data };
  this.emit('packetCreate', packet);
  this.writeBuffer.push(packet);
  this.callbackBuffer.push(fn);
  this.flush();
};

/**
 * Closes the connection.
 *
 * @api private
 */

Socket.prototype.close = function () {
  if ('opening' == this.readyState || 'open' == this.readyState) {
    this.readyState = 'closing';

    var self = this;

    function close() {
      self.onClose('forced close');
      debug('socket closing - telling transport to close');
      self.transport.close();
    }

    function cleanupAndClose() {
      self.removeListener('upgrade', cleanupAndClose);
      self.removeListener('upgradeError', cleanupAndClose);
      close();
    }

    function waitForUpgrade() {
      // wait for upgrade to finish since we can't send packets while pausing a transport
      self.once('upgrade', cleanupAndClose);
      self.once('upgradeError', cleanupAndClose);
    }

    if (this.writeBuffer.length) {
      this.once('drain', function() {
        if (this.upgrading) {
          waitForUpgrade();
        } else {
          close();
        }
      });
    } else if (this.upgrading) {
      waitForUpgrade();
    } else {
      close();
    }
  }

  return this;
};

/**
 * Called upon transport error
 *
 * @api private
 */

Socket.prototype.onError = function (err) {
  debug('socket error %j', err);
  Socket.priorWebsocketSuccess = false;
  this.emit('error', err);
  this.onClose('transport error', err);
};

/**
 * Called upon transport close.
 *
 * @api private
 */

Socket.prototype.onClose = function (reason, desc) {
  if ('opening' == this.readyState || 'open' == this.readyState || 'closing' == this.readyState) {
    debug('socket close with reason: "%s"', reason);
    var self = this;

    // clear timers
    clearTimeout(this.pingIntervalTimer);
    clearTimeout(this.pingTimeoutTimer);

    // clean buffers in next tick, so developers can still
    // grab the buffers on `close` event
    setTimeout(function() {
      self.writeBuffer = [];
      self.callbackBuffer = [];
      self.prevBufferLen = 0;
    }, 0);

    // stop event from firing again for transport
    this.transport.removeAllListeners('close');

    // ensure transport won't stay open
    this.transport.close();

    // ignore further transport communication
    this.transport.removeAllListeners();

    // set ready state
    this.readyState = 'closed';

    // clear session id
    this.id = null;

    // emit close event
    this.emit('close', reason, desc);
  }
};

/**
 * Filters upgrades, returning only those matching client transports.
 *
 * @param {Array} server upgrades
 * @api private
 *
 */

Socket.prototype.filterUpgrades = function (upgrades) {
  var filteredUpgrades = [];
  for (var i = 0, j = upgrades.length; i<j; i++) {
    if (~index(this.transports, upgrades[i])) filteredUpgrades.push(upgrades[i]);
  }
  return filteredUpgrades;
};

}).call(this,typeof self !== "undefined" ? self : typeof window !== "undefined" ? window : {})
},{"./transport":14,"./transports":15,"component-emitter":9,"debug":22,"engine.io-parser":25,"indexof":40,"parsejson":32,"parseqs":33,"parseuri":34}],14:[function(_dereq_,module,exports){
/**
 * Module dependencies.
 */

var parser = _dereq_('engine.io-parser');
var Emitter = _dereq_('component-emitter');

/**
 * Module exports.
 */

module.exports = Transport;

/**
 * Transport abstract constructor.
 *
 * @param {Object} options.
 * @api private
 */

function Transport (opts) {
  this.path = opts.path;
  this.hostname = opts.hostname;
  this.port = opts.port;
  this.secure = opts.secure;
  this.query = opts.query;
  this.timestampParam = opts.timestampParam;
  this.timestampRequests = opts.timestampRequests;
  this.readyState = '';
  this.agent = opts.agent || false;
  this.socket = opts.socket;
  this.enablesXDR = opts.enablesXDR;

  // SSL options for Node.js client
  this.pfx = opts.pfx;
  this.key = opts.key;
  this.passphrase = opts.passphrase;
  this.cert = opts.cert;
  this.ca = opts.ca;
  this.ciphers = opts.ciphers;
  this.rejectUnauthorized = opts.rejectUnauthorized;
}

/**
 * Mix in `Emitter`.
 */

Emitter(Transport.prototype);

/**
 * A counter used to prevent collisions in the timestamps used
 * for cache busting.
 */

Transport.timestamps = 0;

/**
 * Emits an error.
 *
 * @param {String} str
 * @return {Transport} for chaining
 * @api public
 */

Transport.prototype.onError = function (msg, desc) {
  var err = new Error(msg);
  err.type = 'TransportError';
  err.description = desc;
  this.emit('error', err);
  return this;
};

/**
 * Opens the transport.
 *
 * @api public
 */

Transport.prototype.open = function () {
  if ('closed' == this.readyState || '' == this.readyState) {
    this.readyState = 'opening';
    this.doOpen();
  }

  return this;
};

/**
 * Closes the transport.
 *
 * @api private
 */

Transport.prototype.close = function () {
  if ('opening' == this.readyState || 'open' == this.readyState) {
    this.doClose();
    this.onClose();
  }

  return this;
};

/**
 * Sends multiple packets.
 *
 * @param {Array} packets
 * @api private
 */

Transport.prototype.send = function(packets){
  if ('open' == this.readyState) {
    this.write(packets);
  } else {
    throw new Error('Transport not open');
  }
};

/**
 * Called upon open
 *
 * @api private
 */

Transport.prototype.onOpen = function () {
  this.readyState = 'open';
  this.writable = true;
  this.emit('open');
};

/**
 * Called with data.
 *
 * @param {String} data
 * @api private
 */

Transport.prototype.onData = function(data){
  var packet = parser.decodePacket(data, this.socket.binaryType);
  this.onPacket(packet);
};

/**
 * Called with a decoded packet.
 */

Transport.prototype.onPacket = function (packet) {
  this.emit('packet', packet);
};

/**
 * Called upon close.
 *
 * @api private
 */

Transport.prototype.onClose = function () {
  this.readyState = 'closed';
  this.emit('close');
};

},{"component-emitter":9,"engine.io-parser":25}],15:[function(_dereq_,module,exports){
(function (global){
/**
 * Module dependencies
 */

var XMLHttpRequest = _dereq_('xmlhttprequest');
var XHR = _dereq_('./polling-xhr');
var JSONP = _dereq_('./polling-jsonp');
var websocket = _dereq_('./websocket');

/**
 * Export transports.
 */

exports.polling = polling;
exports.websocket = websocket;

/**
 * Polling transport polymorphic constructor.
 * Decides on xhr vs jsonp based on feature detection.
 *
 * @api private
 */

function polling(opts){
  var xhr;
  var xd = false;
  var xs = false;
  var jsonp = false !== opts.jsonp;

  if (global.location) {
    var isSSL = 'https:' == location.protocol;
    var port = location.port;

    // some user agents have empty `location.port`
    if (!port) {
      port = isSSL ? 443 : 80;
    }

    xd = opts.hostname != location.hostname || port != opts.port;
    xs = opts.secure != isSSL;
  }

  opts.xdomain = xd;
  opts.xscheme = xs;
  xhr = new XMLHttpRequest(opts);

  if ('open' in xhr && !opts.forceJSONP) {
    return new XHR(opts);
  } else {
    if (!jsonp) throw new Error('JSONP disabled');
    return new JSONP(opts);
  }
}

}).call(this,typeof self !== "undefined" ? self : typeof window !== "undefined" ? window : {})
},{"./polling-jsonp":16,"./polling-xhr":17,"./websocket":19,"xmlhttprequest":20}],16:[function(_dereq_,module,exports){
(function (global){

/**
 * Module requirements.
 */

var Polling = _dereq_('./polling');
var inherit = _dereq_('component-inherit');

/**
 * Module exports.
 */

module.exports = JSONPPolling;

/**
 * Cached regular expressions.
 */

var rNewline = /\n/g;
var rEscapedNewline = /\\n/g;

/**
 * Global JSONP callbacks.
 */

var callbacks;

/**
 * Callbacks count.
 */

var index = 0;

/**
 * Noop.
 */

function empty () { }

/**
 * JSONP Polling constructor.
 *
 * @param {Object} opts.
 * @api public
 */

function JSONPPolling (opts) {
  Polling.call(this, opts);

  this.query = this.query || {};

  // define global callbacks array if not present
  // we do this here (lazily) to avoid unneeded global pollution
  if (!callbacks) {
    // we need to consider multiple engines in the same page
    if (!global.___eio) global.___eio = [];
    callbacks = global.___eio;
  }

  // callback identifier
  this.index = callbacks.length;

  // add callback to jsonp global
  var self = this;
  callbacks.push(function (msg) {
    self.onData(msg);
  });

  // append to query string
  this.query.j = this.index;

  // prevent spurious errors from being emitted when the window is unloaded
  if (global.document && global.addEventListener) {
    global.addEventListener('beforeunload', function () {
      if (self.script) self.script.onerror = empty;
    }, false);
  }
}

/**
 * Inherits from Polling.
 */

inherit(JSONPPolling, Polling);

/*
 * JSONP only supports binary as base64 encoded strings
 */

JSONPPolling.prototype.supportsBinary = false;

/**
 * Closes the socket.
 *
 * @api private
 */

JSONPPolling.prototype.doClose = function () {
  if (this.script) {
    this.script.parentNode.removeChild(this.script);
    this.script = null;
  }

  if (this.form) {
    this.form.parentNode.removeChild(this.form);
    this.form = null;
    this.iframe = null;
  }

  Polling.prototype.doClose.call(this);
};

/**
 * Starts a poll cycle.
 *
 * @api private
 */

JSONPPolling.prototype.doPoll = function () {
  var self = this;
  var script = document.createElement('script');

  if (this.script) {
    this.script.parentNode.removeChild(this.script);
    this.script = null;
  }

  script.async = true;
  script.src = this.uri();
  script.onerror = function(e){
    self.onError('jsonp poll error',e);
  };

  var insertAt = document.getElementsByTagName('script')[0];
  insertAt.parentNode.insertBefore(script, insertAt);
  this.script = script;

  var isUAgecko = 'undefined' != typeof navigator && /gecko/i.test(navigator.userAgent);
  
  if (isUAgecko) {
    setTimeout(function () {
      var iframe = document.createElement('iframe');
      document.body.appendChild(iframe);
      document.body.removeChild(iframe);
    }, 100);
  }
};

/**
 * Writes with a hidden iframe.
 *
 * @param {String} data to send
 * @param {Function} called upon flush.
 * @api private
 */

JSONPPolling.prototype.doWrite = function (data, fn) {
  var self = this;

  if (!this.form) {
    var form = document.createElement('form');
    var area = document.createElement('textarea');
    var id = this.iframeId = 'eio_iframe_' + this.index;
    var iframe;

    form.className = 'socketio';
    form.style.position = 'absolute';
    form.style.top = '-1000px';
    form.style.left = '-1000px';
    form.target = id;
    form.method = 'POST';
    form.setAttribute('accept-charset', 'utf-8');
    area.name = 'd';
    form.appendChild(area);
    document.body.appendChild(form);

    this.form = form;
    this.area = area;
  }

  this.form.action = this.uri();

  function complete () {
    initIframe();
    fn();
  }

  function initIframe () {
    if (self.iframe) {
      try {
        self.form.removeChild(self.iframe);
      } catch (e) {
        self.onError('jsonp polling iframe removal error', e);
      }
    }

    try {
      // ie6 dynamic iframes with target="" support (thanks Chris Lambacher)
      var html = '<iframe src="javascript:0" name="'+ self.iframeId +'">';
      iframe = document.createElement(html);
    } catch (e) {
      iframe = document.createElement('iframe');
      iframe.name = self.iframeId;
      iframe.src = 'javascript:0';
    }

    iframe.id = self.iframeId;

    self.form.appendChild(iframe);
    self.iframe = iframe;
  }

  initIframe();

  // escape \n to prevent it from being converted into \r\n by some UAs
  // double escaping is required for escaped new lines because unescaping of new lines can be done safely on server-side
  data = data.replace(rEscapedNewline, '\\\n');
  this.area.value = data.replace(rNewline, '\\n');

  try {
    this.form.submit();
  } catch(e) {}

  if (this.iframe.attachEvent) {
    this.iframe.onreadystatechange = function(){
      if (self.iframe.readyState == 'complete') {
        complete();
      }
    };
  } else {
    this.iframe.onload = complete;
  }
};

}).call(this,typeof self !== "undefined" ? self : typeof window !== "undefined" ? window : {})
},{"./polling":18,"component-inherit":21}],17:[function(_dereq_,module,exports){
(function (global){
/**
 * Module requirements.
 */

var XMLHttpRequest = _dereq_('xmlhttprequest');
var Polling = _dereq_('./polling');
var Emitter = _dereq_('component-emitter');
var inherit = _dereq_('component-inherit');
var debug = _dereq_('debug')('engine.io-client:polling-xhr');

/**
 * Module exports.
 */

module.exports = XHR;
module.exports.Request = Request;

/**
 * Empty function
 */

function empty(){}

/**
 * XHR Polling constructor.
 *
 * @param {Object} opts
 * @api public
 */

function XHR(opts){
  Polling.call(this, opts);

  if (global.location) {
    var isSSL = 'https:' == location.protocol;
    var port = location.port;

    // some user agents have empty `location.port`
    if (!port) {
      port = isSSL ? 443 : 80;
    }

    this.xd = opts.hostname != global.location.hostname ||
      port != opts.port;
    this.xs = opts.secure != isSSL;
  }
}

/**
 * Inherits from Polling.
 */

inherit(XHR, Polling);

/**
 * XHR supports binary
 */

XHR.prototype.supportsBinary = true;

/**
 * Creates a request.
 *
 * @param {String} method
 * @api private
 */

XHR.prototype.request = function(opts){
  opts = opts || {};
  opts.uri = this.uri();
  opts.xd = this.xd;
  opts.xs = this.xs;
  opts.agent = this.agent || false;
  opts.supportsBinary = this.supportsBinary;
  opts.enablesXDR = this.enablesXDR;

  // SSL options for Node.js client
  opts.pfx = this.pfx;
  opts.key = this.key;
  opts.passphrase = this.passphrase;
  opts.cert = this.cert;
  opts.ca = this.ca;
  opts.ciphers = this.ciphers;
  opts.rejectUnauthorized = this.rejectUnauthorized;

  return new Request(opts);
};

/**
 * Sends data.
 *
 * @param {String} data to send.
 * @param {Function} called upon flush.
 * @api private
 */

XHR.prototype.doWrite = function(data, fn){
  var isBinary = typeof data !== 'string' && data !== undefined;
  var req = this.request({ method: 'POST', data: data, isBinary: isBinary });
  var self = this;
  req.on('success', fn);
  req.on('error', function(err){
    self.onError('xhr post error', err);
  });
  this.sendXhr = req;
};

/**
 * Starts a poll cycle.
 *
 * @api private
 */

XHR.prototype.doPoll = function(){
  debug('xhr poll');
  var req = this.request();
  var self = this;
  req.on('data', function(data){
    self.onData(data);
  });
  req.on('error', function(err){
    self.onError('xhr poll error', err);
  });
  this.pollXhr = req;
};

/**
 * Request constructor
 *
 * @param {Object} options
 * @api public
 */

function Request(opts){
  this.method = opts.method || 'GET';
  this.uri = opts.uri;
  this.xd = !!opts.xd;
  this.xs = !!opts.xs;
  this.async = false !== opts.async;
  this.data = undefined != opts.data ? opts.data : null;
  this.agent = opts.agent;
  this.isBinary = opts.isBinary;
  this.supportsBinary = opts.supportsBinary;
  this.enablesXDR = opts.enablesXDR;

  // SSL options for Node.js client
  this.pfx = opts.pfx;
  this.key = opts.key;
  this.passphrase = opts.passphrase;
  this.cert = opts.cert;
  this.ca = opts.ca;
  this.ciphers = opts.ciphers;
  this.rejectUnauthorized = opts.rejectUnauthorized;

  this.create();
}

/**
 * Mix in `Emitter`.
 */

Emitter(Request.prototype);

/**
 * Creates the XHR object and sends the request.
 *
 * @api private
 */

Request.prototype.create = function(){
  var opts = { agent: this.agent, xdomain: this.xd, xscheme: this.xs, enablesXDR: this.enablesXDR };

  // SSL options for Node.js client
  opts.pfx = this.pfx;
  opts.key = this.key;
  opts.passphrase = this.passphrase;
  opts.cert = this.cert;
  opts.ca = this.ca;
  opts.ciphers = this.ciphers;
  opts.rejectUnauthorized = this.rejectUnauthorized;

  var xhr = this.xhr = new XMLHttpRequest(opts);
  var self = this;

  try {
    debug('xhr open %s: %s', this.method, this.uri);
    xhr.open(this.method, this.uri, this.async);
    if (this.supportsBinary) {
      // This has to be done after open because Firefox is stupid
      // http://stackoverflow.com/questions/13216903/get-binary-data-with-xmlhttprequest-in-a-firefox-extension
      xhr.responseType = 'arraybuffer';
    }

    if ('POST' == this.method) {
      try {
        if (this.isBinary) {
          xhr.setRequestHeader('Content-type', 'application/octet-stream');
        } else {
          xhr.setRequestHeader('Content-type', 'text/plain;charset=UTF-8');
        }
      } catch (e) {}
    }

    // ie6 check
    if ('withCredentials' in xhr) {
      xhr.withCredentials = true;
    }

    if (this.hasXDR()) {
      xhr.onload = function(){
        self.onLoad();
      };
      xhr.onerror = function(){
        self.onError(xhr.responseText);
      };
    } else {
      xhr.onreadystatechange = function(){
        if (4 != xhr.readyState) return;
        if (200 == xhr.status || 1223 == xhr.status) {
          self.onLoad();
        } else {
          // make sure the `error` event handler that's user-set
          // does not throw in the same tick and gets caught here
          setTimeout(function(){
            self.onError(xhr.status);
          }, 0);
        }
      };
    }

    debug('xhr data %s', this.data);
    xhr.send(this.data);
  } catch (e) {
    // Need to defer since .create() is called directly fhrom the constructor
    // and thus the 'error' event can only be only bound *after* this exception
    // occurs.  Therefore, also, we cannot throw here at all.
    setTimeout(function() {
      self.onError(e);
    }, 0);
    return;
  }

  if (global.document) {
    this.index = Request.requestsCount++;
    Request.requests[this.index] = this;
  }
};

/**
 * Called upon successful response.
 *
 * @api private
 */

Request.prototype.onSuccess = function(){
  this.emit('success');
  this.cleanup();
};

/**
 * Called if we have data.
 *
 * @api private
 */

Request.prototype.onData = function(data){
  this.emit('data', data);
  this.onSuccess();
};

/**
 * Called upon error.
 *
 * @api private
 */

Request.prototype.onError = function(err){
  this.emit('error', err);
  this.cleanup(true);
};

/**
 * Cleans up house.
 *
 * @api private
 */

Request.prototype.cleanup = function(fromError){
  if ('undefined' == typeof this.xhr || null === this.xhr) {
    return;
  }
  // xmlhttprequest
  if (this.hasXDR()) {
    this.xhr.onload = this.xhr.onerror = empty;
  } else {
    this.xhr.onreadystatechange = empty;
  }

  if (fromError) {
    try {
      this.xhr.abort();
    } catch(e) {}
  }

  if (global.document) {
    delete Request.requests[this.index];
  }

  this.xhr = null;
};

/**
 * Called upon load.
 *
 * @api private
 */

Request.prototype.onLoad = function(){
  var data;
  try {
    var contentType;
    try {
      contentType = this.xhr.getResponseHeader('Content-Type').split(';')[0];
    } catch (e) {}
    if (contentType === 'application/octet-stream') {
      data = this.xhr.response;
    } else {
      if (!this.supportsBinary) {
        data = this.xhr.responseText;
      } else {
        data = 'ok';
      }
    }
  } catch (e) {
    this.onError(e);
  }
  if (null != data) {
    this.onData(data);
  }
};

/**
 * Check if it has XDomainRequest.
 *
 * @api private
 */

Request.prototype.hasXDR = function(){
  return 'undefined' !== typeof global.XDomainRequest && !this.xs && this.enablesXDR;
};

/**
 * Aborts the request.
 *
 * @api public
 */

Request.prototype.abort = function(){
  this.cleanup();
};

/**
 * Aborts pending requests when unloading the window. This is needed to prevent
 * memory leaks (e.g. when using IE) and to ensure that no spurious error is
 * emitted.
 */

if (global.document) {
  Request.requestsCount = 0;
  Request.requests = {};
  if (global.attachEvent) {
    global.attachEvent('onunload', unloadHandler);
  } else if (global.addEventListener) {
    global.addEventListener('beforeunload', unloadHandler, false);
  }
}

function unloadHandler() {
  for (var i in Request.requests) {
    if (Request.requests.hasOwnProperty(i)) {
      Request.requests[i].abort();
    }
  }
}

}).call(this,typeof self !== "undefined" ? self : typeof window !== "undefined" ? window : {})
},{"./polling":18,"component-emitter":9,"component-inherit":21,"debug":22,"xmlhttprequest":20}],18:[function(_dereq_,module,exports){
/**
 * Module dependencies.
 */

var Transport = _dereq_('../transport');
var parseqs = _dereq_('parseqs');
var parser = _dereq_('engine.io-parser');
var inherit = _dereq_('component-inherit');
var debug = _dereq_('debug')('engine.io-client:polling');

/**
 * Module exports.
 */

module.exports = Polling;

/**
 * Is XHR2 supported?
 */

var hasXHR2 = (function() {
  var XMLHttpRequest = _dereq_('xmlhttprequest');
  var xhr = new XMLHttpRequest({ xdomain: false });
  return null != xhr.responseType;
})();

/**
 * Polling interface.
 *
 * @param {Object} opts
 * @api private
 */

function Polling(opts){
  var forceBase64 = (opts && opts.forceBase64);
  if (!hasXHR2 || forceBase64) {
    this.supportsBinary = false;
  }
  Transport.call(this, opts);
}

/**
 * Inherits from Transport.
 */

inherit(Polling, Transport);

/**
 * Transport name.
 */

Polling.prototype.name = 'polling';

/**
 * Opens the socket (triggers polling). We write a PING message to determine
 * when the transport is open.
 *
 * @api private
 */

Polling.prototype.doOpen = function(){
  this.poll();
};

/**
 * Pauses polling.
 *
 * @param {Function} callback upon buffers are flushed and transport is paused
 * @api private
 */

Polling.prototype.pause = function(onPause){
  var pending = 0;
  var self = this;

  this.readyState = 'pausing';

  function pause(){
    debug('paused');
    self.readyState = 'paused';
    onPause();
  }

  if (this.polling || !this.writable) {
    var total = 0;

    if (this.polling) {
      debug('we are currently polling - waiting to pause');
      total++;
      this.once('pollComplete', function(){
        debug('pre-pause polling complete');
        --total || pause();
      });
    }

    if (!this.writable) {
      debug('we are currently writing - waiting to pause');
      total++;
      this.once('drain', function(){
        debug('pre-pause writing complete');
        --total || pause();
      });
    }
  } else {
    pause();
  }
};

/**
 * Starts polling cycle.
 *
 * @api public
 */

Polling.prototype.poll = function(){
  debug('polling');
  this.polling = true;
  this.doPoll();
  this.emit('poll');
};

/**
 * Overloads onData to detect payloads.
 *
 * @api private
 */

Polling.prototype.onData = function(data){
  var self = this;
  debug('polling got data %s', data);
  var callback = function(packet, index, total) {
    // if its the first message we consider the transport open
    if ('opening' == self.readyState) {
      self.onOpen();
    }

    // if its a close packet, we close the ongoing requests
    if ('close' == packet.type) {
      self.onClose();
      return false;
    }

    // otherwise bypass onData and handle the message
    self.onPacket(packet);
  };

  // decode payload
  parser.decodePayload(data, this.socket.binaryType, callback);

  // if an event did not trigger closing
  if ('closed' != this.readyState) {
    // if we got data we're not polling
    this.polling = false;
    this.emit('pollComplete');

    if ('open' == this.readyState) {
      this.poll();
    } else {
      debug('ignoring poll - transport state "%s"', this.readyState);
    }
  }
};

/**
 * For polling, send a close packet.
 *
 * @api private
 */

Polling.prototype.doClose = function(){
  var self = this;

  function close(){
    debug('writing close packet');
    self.write([{ type: 'close' }]);
  }

  if ('open' == this.readyState) {
    debug('transport open - closing');
    close();
  } else {
    // in case we're trying to close while
    // handshaking is in progress (GH-164)
    debug('transport not open - deferring close');
    this.once('open', close);
  }
};

/**
 * Writes a packets payload.
 *
 * @param {Array} data packets
 * @param {Function} drain callback
 * @api private
 */

Polling.prototype.write = function(packets){
  var self = this;
  this.writable = false;
  var callbackfn = function() {
    self.writable = true;
    self.emit('drain');
  };

  var self = this;
  parser.encodePayload(packets, this.supportsBinary, function(data) {
    self.doWrite(data, callbackfn);
  });
};

/**
 * Generates uri for connection.
 *
 * @api private
 */

Polling.prototype.uri = function(){
  var query = this.query || {};
  var schema = this.secure ? 'https' : 'http';
  var port = '';

  // cache busting is forced
  if (false !== this.timestampRequests) {
    query[this.timestampParam] = +new Date + '-' + Transport.timestamps++;
  }

  if (!this.supportsBinary && !query.sid) {
    query.b64 = 1;
  }

  query = parseqs.encode(query);

  // avoid port if default for schema
  if (this.port && (('https' == schema && this.port != 443) ||
     ('http' == schema && this.port != 80))) {
    port = ':' + this.port;
  }

  // prepend ? to query
  if (query.length) {
    query = '?' + query;
  }

  return schema + '://' + this.hostname + port + this.path + query;
};

},{"../transport":14,"component-inherit":21,"debug":22,"engine.io-parser":25,"parseqs":33,"xmlhttprequest":20}],19:[function(_dereq_,module,exports){
/**
 * Module dependencies.
 */

var Transport = _dereq_('../transport');
var parser = _dereq_('engine.io-parser');
var parseqs = _dereq_('parseqs');
var inherit = _dereq_('component-inherit');
var debug = _dereq_('debug')('engine.io-client:websocket');

/**
 * `ws` exposes a WebSocket-compatible interface in
 * Node, or the `WebSocket` or `MozWebSocket` globals
 * in the browser.
 */

var WebSocket = _dereq_('ws');

/**
 * Module exports.
 */

module.exports = WS;

/**
 * WebSocket transport constructor.
 *
 * @api {Object} connection options
 * @api public
 */

function WS(opts){
  var forceBase64 = (opts && opts.forceBase64);
  if (forceBase64) {
    this.supportsBinary = false;
  }
  Transport.call(this, opts);
}

/**
 * Inherits from Transport.
 */

inherit(WS, Transport);

/**
 * Transport name.
 *
 * @api public
 */

WS.prototype.name = 'websocket';

/*
 * WebSockets support binary
 */

WS.prototype.supportsBinary = true;

/**
 * Opens socket.
 *
 * @api private
 */

WS.prototype.doOpen = function(){
  if (!this.check()) {
    // let probe timeout
    return;
  }

  var self = this;
  var uri = this.uri();
  var protocols = void(0);
  var opts = { agent: this.agent };

  // SSL options for Node.js client
  opts.pfx = this.pfx;
  opts.key = this.key;
  opts.passphrase = this.passphrase;
  opts.cert = this.cert;
  opts.ca = this.ca;
  opts.ciphers = this.ciphers;
  opts.rejectUnauthorized = this.rejectUnauthorized;

  this.ws = new WebSocket(uri, protocols, opts);

  if (this.ws.binaryType === undefined) {
    this.supportsBinary = false;
  }

  this.ws.binaryType = 'arraybuffer';
  this.addEventListeners();
};

/**
 * Adds event listeners to the socket
 *
 * @api private
 */

WS.prototype.addEventListeners = function(){
  var self = this;

  this.ws.onopen = function(){
    self.onOpen();
  };
  this.ws.onclose = function(){
    self.onClose();
  };
  this.ws.onmessage = function(ev){
    self.onData(ev.data);
  };
  this.ws.onerror = function(e){
    self.onError('websocket error', e);
  };
};

/**
 * Override `onData` to use a timer on iOS.
 * See: https://gist.github.com/mloughran/2052006
 *
 * @api private
 */

if ('undefined' != typeof navigator
  && /iPad|iPhone|iPod/i.test(navigator.userAgent)) {
  WS.prototype.onData = function(data){
    var self = this;
    setTimeout(function(){
      Transport.prototype.onData.call(self, data);
    }, 0);
  };
}

/**
 * Writes data to socket.
 *
 * @param {Array} array of packets.
 * @api private
 */

WS.prototype.write = function(packets){
  var self = this;
  // console.log(new Date().getTime(), "WS.prototype.write");
  this.writable = false;
  // encodePacket efficient as it uses WS framing
  // no need for encodePayload
  for (var i = 0, l = packets.length; i < l; i++) {
    parser.encodePacket(packets[i], this.supportsBinary, function(data) {
      //Sometimes the websocket has already been closed but the browser didn't
      //have a chance of informing us about it yet, in that case send will
      //throw an error
      try {
        // console.log(new Date().getTime(), "ws.send", data);
        self.ws.send(data);
      } catch (e){
        debug('websocket closed before onclose event');
      }
    });
  }

  function ondrain() {
    // console.log(new Date().getTime(), "emit('drain')");
    self.writable = true;
    self.emit('drain');
  }
  // fake drain
  // defer to next tick to allow Socket to clear writeBuffer
  // console.log(new Date().getTime(), "setTimeout(ondrain, 0)");
  setTimeout(ondrain, 1);
  // ondrain();
};

/**
 * Called upon close
 *
 * @api private
 */

WS.prototype.onClose = function(){
  Transport.prototype.onClose.call(this);
};

/**
 * Closes socket.
 *
 * @api private
 */

WS.prototype.doClose = function(){
  if (typeof this.ws !== 'undefined') {
    this.ws.close();
  }
};

/**
 * Generates uri for connection.
 *
 * @api private
 */

WS.prototype.uri = function(){
  var query = this.query || {};
  var schema = this.secure ? 'wss' : 'ws';
  var port = '';

  // avoid port if default for schema
  if (this.port && (('wss' == schema && this.port != 443)
    || ('ws' == schema && this.port != 80))) {
    port = ':' + this.port;
  }

  // append timestamp to URI
  if (this.timestampRequests) {
    query[this.timestampParam] = +new Date;
  }

  // communicate binary support capabilities
  if (!this.supportsBinary) {
    query.b64 = 1;
  }

  query = parseqs.encode(query);

  // prepend ? to query
  if (query.length) {
    query = '?' + query;
  }

  return schema + '://' + this.hostname + port + this.path + query;
};

/**
 * Feature detection for WebSocket.
 *
 * @return {Boolean} whether this transport is available.
 * @api public
 */

WS.prototype.check = function(){
  return !!WebSocket && !('__initialize' in WebSocket && this.name === WS.prototype.name);
};

},{"../transport":14,"component-inherit":21,"debug":22,"engine.io-parser":25,"parseqs":33,"ws":35}],20:[function(_dereq_,module,exports){
// browser shim for xmlhttprequest module
var hasCORS = _dereq_('has-cors');

module.exports = function(opts) {
  var xdomain = opts.xdomain;

  // scheme must be same when usign XDomainRequest
  // http://blogs.msdn.com/b/ieinternals/archive/2010/05/13/xdomainrequest-restrictions-limitations-and-workarounds.aspx
  var xscheme = opts.xscheme;

  // XDomainRequest has a flow of not sending cookie, therefore it should be disabled as a default.
  // https://github.com/Automattic/engine.io-client/pull/217
  var enablesXDR = opts.enablesXDR;

  // XMLHttpRequest can be disabled on IE
  try {
    if ('undefined' != typeof XMLHttpRequest && (!xdomain || hasCORS)) {
      return new XMLHttpRequest();
    }
  } catch (e) { }

  // Use XDomainRequest for IE8 if enablesXDR is true
  // because loading bar keeps flashing when using jsonp-polling
  // https://github.com/yujiosaka/socke.io-ie8-loading-example
  try {
    if ('undefined' != typeof XDomainRequest && !xscheme && enablesXDR) {
      return new XDomainRequest();
    }
  } catch (e) { }

  if (!xdomain) {
    try {
      return new ActiveXObject('Microsoft.XMLHTTP');
    } catch(e) { }
  }
}

},{"has-cors":38}],21:[function(_dereq_,module,exports){

module.exports = function(a, b){
  var fn = function(){};
  fn.prototype = b.prototype;
  a.prototype = new fn;
  a.prototype.constructor = a;
};
},{}],22:[function(_dereq_,module,exports){

/**
 * This is the web browser implementation of `debug()`.
 *
 * Expose `debug()` as the module.
 */

exports = module.exports = _dereq_('./debug');
exports.log = log;
exports.formatArgs = formatArgs;
exports.save = save;
exports.load = load;
exports.useColors = useColors;

/**
 * Colors.
 */

exports.colors = [
  'lightseagreen',
  'forestgreen',
  'goldenrod',
  'dodgerblue',
  'darkorchid',
  'crimson'
];

/**
 * Currently only WebKit-based Web Inspectors, Firefox >= v31,
 * and the Firebug extension (any Firefox version) are known
 * to support "%c" CSS customizations.
 *
 * TODO: add a `localStorage` variable to explicitly enable/disable colors
 */

function useColors() {
  // is webkit? http://stackoverflow.com/a/16459606/376773
  return ('WebkitAppearance' in document.documentElement.style) ||
    // is firebug? http://stackoverflow.com/a/398120/376773
    (window.console && (console.firebug || (console.exception && console.table))) ||
    // is firefox >= v31?
    // https://developer.mozilla.org/en-US/docs/Tools/Web_Console#Styling_messages
    (navigator.userAgent.toLowerCase().match(/firefox\/(\d+)/) && parseInt(RegExp.$1, 10) >= 31);
}

/**
 * Map %j to `JSON.stringify()`, since no Web Inspectors do that by default.
 */

exports.formatters.j = function(v) {
  return JSON.stringify(v);
};


/**
 * Colorize log arguments if enabled.
 *
 * @api public
 */

function formatArgs() {
  var args = arguments;
  var useColors = this.useColors;

  args[0] = (useColors ? '%c' : '')
    + this.namespace
    + (useColors ? ' %c' : ' ')
    + args[0]
    + (useColors ? '%c ' : ' ')
    + '+' + exports.humanize(this.diff);

  if (!useColors) return args;

  var c = 'color: ' + this.color;
  args = [args[0], c, 'color: inherit'].concat(Array.prototype.slice.call(args, 1));

  // the final "%c" is somewhat tricky, because there could be other
  // arguments passed either before or after the %c, so we need to
  // figure out the correct index to insert the CSS into
  var index = 0;
  var lastC = 0;
  args[0].replace(/%[a-z%]/g, function(match) {
    if ('%' === match) return;
    index++;
    if ('%c' === match) {
      // we only are interested in the *last* %c
      // (the user may have provided their own)
      lastC = index;
    }
  });

  args.splice(lastC, 0, c);
  return args;
}

/**
 * Invokes `console.log()` when available.
 * No-op when `console.log` is not a "function".
 *
 * @api public
 */

function log() {
  // This hackery is required for IE8,
  // where the `console.log` function doesn't have 'apply'
  return 'object' == typeof console
    && 'function' == typeof console.log
    && Function.prototype.apply.call(console.log, console, arguments);
}

/**
 * Save `namespaces`.
 *
 * @param {String} namespaces
 * @api private
 */

function save(namespaces) {
  try {
    if (null == namespaces) {
      localStorage.removeItem('debug');
    } else {
      localStorage.debug = namespaces;
    }
  } catch(e) {}
}

/**
 * Load `namespaces`.
 *
 * @return {String} returns the previously persisted debug modes
 * @api private
 */

function load() {
  var r;
  try {
    r = localStorage.debug;
  } catch(e) {}
  return r;
}

/**
 * Enable namespaces listed in `localStorage.debug` initially.
 */

exports.enable(load());

},{"./debug":23}],23:[function(_dereq_,module,exports){

/**
 * This is the common logic for both the Node.js and web browser
 * implementations of `debug()`.
 *
 * Expose `debug()` as the module.
 */

exports = module.exports = debug;
exports.coerce = coerce;
exports.disable = disable;
exports.enable = enable;
exports.enabled = enabled;
exports.humanize = _dereq_('ms');

/**
 * The currently active debug mode names, and names to skip.
 */

exports.names = [];
exports.skips = [];

/**
 * Map of special "%n" handling functions, for the debug "format" argument.
 *
 * Valid key names are a single, lowercased letter, i.e. "n".
 */

exports.formatters = {};

/**
 * Previously assigned color.
 */

var prevColor = 0;

/**
 * Previous log timestamp.
 */

var prevTime;

/**
 * Select a color.
 *
 * @return {Number}
 * @api private
 */

function selectColor() {
  return exports.colors[prevColor++ % exports.colors.length];
}

/**
 * Create a debugger with the given `namespace`.
 *
 * @param {String} namespace
 * @return {Function}
 * @api public
 */

function debug(namespace) {

  // define the `disabled` version
  function disabled() {
  }
  disabled.enabled = false;

  // define the `enabled` version
  function enabled() {

    var self = enabled;

    // set `diff` timestamp
    var curr = +new Date();
    var ms = curr - (prevTime || curr);
    self.diff = ms;
    self.prev = prevTime;
    self.curr = curr;
    prevTime = curr;

    // add the `color` if not set
    if (null == self.useColors) self.useColors = exports.useColors();
    if (null == self.color && self.useColors) self.color = selectColor();

    var args = Array.prototype.slice.call(arguments);

    args[0] = exports.coerce(args[0]);

    if ('string' !== typeof args[0]) {
      // anything else let's inspect with %o
      args = ['%o'].concat(args);
    }

    // apply any `formatters` transformations
    var index = 0;
    args[0] = args[0].replace(/%([a-z%])/g, function(match, format) {
      // if we encounter an escaped % then don't increase the array index
      if (match === '%') return match;
      index++;
      var formatter = exports.formatters[format];
      if ('function' === typeof formatter) {
        var val = args[index];
        match = formatter.call(self, val);

        // now we need to remove `args[index]` since it's inlined in the `format`
        args.splice(index, 1);
        index--;
      }
      return match;
    });

    if ('function' === typeof exports.formatArgs) {
      args = exports.formatArgs.apply(self, args);
    }
    var logFn = enabled.log || exports.log || console.log.bind(console);
    logFn.apply(self, args);
  }
  enabled.enabled = true;

  var fn = exports.enabled(namespace) ? enabled : disabled;

  fn.namespace = namespace;

  return fn;
}

/**
 * Enables a debug mode by namespaces. This can include modes
 * separated by a colon and wildcards.
 *
 * @param {String} namespaces
 * @api public
 */

function enable(namespaces) {
  exports.save(namespaces);

  var split = (namespaces || '').split(/[\s,]+/);
  var len = split.length;

  for (var i = 0; i < len; i++) {
    if (!split[i]) continue; // ignore empty strings
    namespaces = split[i].replace(/\*/g, '.*?');
    if (namespaces[0] === '-') {
      exports.skips.push(new RegExp('^' + namespaces.substr(1) + '$'));
    } else {
      exports.names.push(new RegExp('^' + namespaces + '$'));
    }
  }
}

/**
 * Disable debug output.
 *
 * @api public
 */

function disable() {
  exports.enable('');
}

/**
 * Returns true if the given mode name is enabled, false otherwise.
 *
 * @param {String} name
 * @return {Boolean}
 * @api public
 */

function enabled(name) {
  var i, len;
  for (i = 0, len = exports.skips.length; i < len; i++) {
    if (exports.skips[i].test(name)) {
      return false;
    }
  }
  for (i = 0, len = exports.names.length; i < len; i++) {
    if (exports.names[i].test(name)) {
      return true;
    }
  }
  return false;
}

/**
 * Coerce `val`.
 *
 * @param {Mixed} val
 * @return {Mixed}
 * @api private
 */

function coerce(val) {
  if (val instanceof Error) return val.stack || val.message;
  return val;
}

},{"ms":24}],24:[function(_dereq_,module,exports){
/**
 * Helpers.
 */

var s = 1000;
var m = s * 60;
var h = m * 60;
var d = h * 24;
var y = d * 365.25;

/**
 * Parse or format the given `val`.
 *
 * Options:
 *
 *  - `long` verbose formatting [false]
 *
 * @param {String|Number} val
 * @param {Object} options
 * @return {String|Number}
 * @api public
 */

module.exports = function(val, options){
  options = options || {};
  if ('string' == typeof val) return parse(val);
  return options.long
    ? long(val)
    : short(val);
};

/**
 * Parse the given `str` and return milliseconds.
 *
 * @param {String} str
 * @return {Number}
 * @api private
 */

function parse(str) {
  var match = /^((?:\d+)?\.?\d+) *(ms|seconds?|s|minutes?|m|hours?|h|days?|d|years?|y)?$/i.exec(str);
  if (!match) return;
  var n = parseFloat(match[1]);
  var type = (match[2] || 'ms').toLowerCase();
  switch (type) {
    case 'years':
    case 'year':
    case 'y':
      return n * y;
    case 'days':
    case 'day':
    case 'd':
      return n * d;
    case 'hours':
    case 'hour':
    case 'h':
      return n * h;
    case 'minutes':
    case 'minute':
    case 'm':
      return n * m;
    case 'seconds':
    case 'second':
    case 's':
      return n * s;
    case 'ms':
      return n;
  }
}

/**
 * Short format for `ms`.
 *
 * @param {Number} ms
 * @return {String}
 * @api private
 */

function short(ms) {
  if (ms >= d) return Math.round(ms / d) + 'd';
  if (ms >= h) return Math.round(ms / h) + 'h';
  if (ms >= m) return Math.round(ms / m) + 'm';
  if (ms >= s) return Math.round(ms / s) + 's';
  return ms + 'ms';
}

/**
 * Long format for `ms`.
 *
 * @param {Number} ms
 * @return {String}
 * @api private
 */

function long(ms) {
  return plural(ms, d, 'day')
    || plural(ms, h, 'hour')
    || plural(ms, m, 'minute')
    || plural(ms, s, 'second')
    || ms + ' ms';
}

/**
 * Pluralization helper.
 */

function plural(ms, n, name) {
  if (ms < n) return;
  if (ms < n * 1.5) return Math.floor(ms / n) + ' ' + name;
  return Math.ceil(ms / n) + ' ' + name + 's';
}

},{}],25:[function(_dereq_,module,exports){
(function (global){
/**
 * Module dependencies.
 */

var keys = _dereq_('./keys');
var hasBinary = _dereq_('has-binary');
var sliceBuffer = _dereq_('arraybuffer.slice');
var base64encoder = _dereq_('base64-arraybuffer');
var after = _dereq_('after');
var utf8 = _dereq_('utf8');

/**
 * Check if we are running an android browser. That requires us to use
 * ArrayBuffer with polling transports...
 *
 * http://ghinda.net/jpeg-blob-ajax-android/
 */

var isAndroid = navigator.userAgent.match(/Android/i);

/**
 * Check if we are running in PhantomJS.
 * Uploading a Blob with PhantomJS does not work correctly, as reported here:
 * https://github.com/ariya/phantomjs/issues/11395
 * @type boolean
 */
var isPhantomJS = /PhantomJS/i.test(navigator.userAgent);

/**
 * When true, avoids using Blobs to encode payloads.
 * @type boolean
 */
var dontSendBlobs = isAndroid || isPhantomJS;

/**
 * Current protocol version.
 */

exports.protocol = 3;

/**
 * Packet types.
 */

var packets = exports.packets = {
    open:     0    // non-ws
  , close:    1    // non-ws
  , ping:     2
  , pong:     3
  , message:  4
  , upgrade:  5
  , noop:     6
};

var packetslist = keys(packets);

/**
 * Premade error packet.
 */

var err = { type: 'error', data: 'parser error' };

/**
 * Create a blob api even for blob builder when vendor prefixes exist
 */

var Blob = _dereq_('blob');

/**
 * Encodes a packet.
 *
 *     <packet type id> [ <data> ]
 *
 * Example:
 *
 *     5hello world
 *     3
 *     4
 *
 * Binary is encoded in an identical principle
 *
 * @api private
 */

exports.encodePacket = function (packet, supportsBinary, utf8encode, callback) {
  if ('function' == typeof supportsBinary) {
    callback = supportsBinary;
    supportsBinary = false;
  }

  if ('function' == typeof utf8encode) {
    callback = utf8encode;
    utf8encode = null;
  }

  var data = (packet.data === undefined)
    ? undefined
    : packet.data.buffer || packet.data;

  if (global.ArrayBuffer && data instanceof ArrayBuffer) {
    return encodeArrayBuffer(packet, supportsBinary, callback);
  } else if (Blob && data instanceof global.Blob) {
    return encodeBlob(packet, supportsBinary, callback);
  }

  // might be an object with { base64: true, data: dataAsBase64String }
  if (data && data.base64) {
    return encodeBase64Object(packet, callback);
  }

  // Sending data as a utf-8 string
  var encoded = packets[packet.type];

  // data fragment is optional
  if (undefined !== packet.data) {
    encoded += utf8encode ? utf8.encode(String(packet.data)) : String(packet.data);
  }

  return callback('' + encoded);

};

function encodeBase64Object(packet, callback) {
  // packet data is an object { base64: true, data: dataAsBase64String }
  var message = 'b' + exports.packets[packet.type] + packet.data.data;
  return callback(message);
}

/**
 * Encode packet helpers for binary types
 */

function encodeArrayBuffer(packet, supportsBinary, callback) {
  if (!supportsBinary) {
    return exports.encodeBase64Packet(packet, callback);
  }

  var data = packet.data;
  var contentArray = new Uint8Array(data);
  var resultBuffer = new Uint8Array(1 + data.byteLength);

  resultBuffer[0] = packets[packet.type];
  for (var i = 0; i < contentArray.length; i++) {
    resultBuffer[i+1] = contentArray[i];
  }

  return callback(resultBuffer.buffer);
}

function encodeBlobAsArrayBuffer(packet, supportsBinary, callback) {
  if (!supportsBinary) {
    return exports.encodeBase64Packet(packet, callback);
  }

  var fr = new FileReader();
  fr.onload = function() {
    packet.data = fr.result;
    exports.encodePacket(packet, supportsBinary, true, callback);
  };
  return fr.readAsArrayBuffer(packet.data);
}

function encodeBlob(packet, supportsBinary, callback) {
  if (!supportsBinary) {
    return exports.encodeBase64Packet(packet, callback);
  }

  if (dontSendBlobs) {
    return encodeBlobAsArrayBuffer(packet, supportsBinary, callback);
  }

  var length = new Uint8Array(1);
  length[0] = packets[packet.type];
  var blob = new Blob([length.buffer, packet.data]);

  return callback(blob);
}

/**
 * Encodes a packet with binary data in a base64 string
 *
 * @param {Object} packet, has `type` and `data`
 * @return {String} base64 encoded message
 */

exports.encodeBase64Packet = function(packet, callback) {
  var message = 'b' + exports.packets[packet.type];
  if (Blob && packet.data instanceof Blob) {
    var fr = new FileReader();
    fr.onload = function() {
      var b64 = fr.result.split(',')[1];
      callback(message + b64);
    };
    return fr.readAsDataURL(packet.data);
  }

  var b64data;
  try {
    b64data = String.fromCharCode.apply(null, new Uint8Array(packet.data));
  } catch (e) {
    // iPhone Safari doesn't let you apply with typed arrays
    var typed = new Uint8Array(packet.data);
    var basic = new Array(typed.length);
    for (var i = 0; i < typed.length; i++) {
      basic[i] = typed[i];
    }
    b64data = String.fromCharCode.apply(null, basic);
  }
  message += global.btoa(b64data);
  return callback(message);
};

/**
 * Decodes a packet. Changes format to Blob if requested.
 *
 * @return {Object} with `type` and `data` (if any)
 * @api private
 */

exports.decodePacket = function (data, binaryType, utf8decode) {
  // String data
  if (typeof data == 'string' || data === undefined) {
    if (data.charAt(0) == 'b') {
      return exports.decodeBase64Packet(data.substr(1), binaryType);
    }

    if (utf8decode) {
      try {
        data = utf8.decode(data);
      } catch (e) {
        return err;
      }
    }
    var type = data.charAt(0);

    if (Number(type) != type || !packetslist[type]) {
      return err;
    }

    if (data.length > 1) {
      return { type: packetslist[type], data: data.substring(1) };
    } else {
      return { type: packetslist[type] };
    }
  }

  var asArray = new Uint8Array(data);
  var type = asArray[0];
  var rest = sliceBuffer(data, 1);
  if (Blob && binaryType === 'blob') {
    rest = new Blob([rest]);
  }
  return { type: packetslist[type], data: rest };
};

/**
 * Decodes a packet encoded in a base64 string
 *
 * @param {String} base64 encoded message
 * @return {Object} with `type` and `data` (if any)
 */

exports.decodeBase64Packet = function(msg, binaryType) {
  var type = packetslist[msg.charAt(0)];
  if (!global.ArrayBuffer) {
    return { type: type, data: { base64: true, data: msg.substr(1) } };
  }

  var data = base64encoder.decode(msg.substr(1));

  if (binaryType === 'blob' && Blob) {
    data = new Blob([data]);
  }

  return { type: type, data: data };
};

/**
 * Encodes multiple messages (payload).
 *
 *     <length>:data
 *
 * Example:
 *
 *     11:hello world2:hi
 *
 * If any contents are binary, they will be encoded as base64 strings. Base64
 * encoded strings are marked with a b before the length specifier
 *
 * @param {Array} packets
 * @api private
 */

exports.encodePayload = function (packets, supportsBinary, callback) {
  if (typeof supportsBinary == 'function') {
    callback = supportsBinary;
    supportsBinary = null;
  }

  var isBinary = hasBinary(packets);

  if (supportsBinary && isBinary) {
    if (Blob && !dontSendBlobs) {
      return exports.encodePayloadAsBlob(packets, callback);
    }

    return exports.encodePayloadAsArrayBuffer(packets, callback);
  }

  if (!packets.length) {
    return callback('0:');
  }

  function setLengthHeader(message) {
    return message.length + ':' + message;
  }

  function encodeOne(packet, doneCallback) {
    exports.encodePacket(packet, !isBinary ? false : supportsBinary, true, function(message) {
      doneCallback(null, setLengthHeader(message));
    });
  }

  map(packets, encodeOne, function(err, results) {
    return callback(results.join(''));
  });
};

/**
 * Async array map using after
 */

function map(ary, each, done) {
  var result = new Array(ary.length);
  var next = after(ary.length, done);

  var eachWithIndex = function(i, el, cb) {
    each(el, function(error, msg) {
      result[i] = msg;
      cb(error, result);
    });
  };

  for (var i = 0; i < ary.length; i++) {
    eachWithIndex(i, ary[i], next);
  }
}

/*
 * Decodes data when a payload is maybe expected. Possible binary contents are
 * decoded from their base64 representation
 *
 * @param {String} data, callback method
 * @api public
 */

exports.decodePayload = function (data, binaryType, callback) {
  if (typeof data != 'string') {
    return exports.decodePayloadAsBinary(data, binaryType, callback);
  }

  if (typeof binaryType === 'function') {
    callback = binaryType;
    binaryType = null;
  }

  var packet;
  if (data == '') {
    // parser error - ignoring payload
    return callback(err, 0, 1);
  }

  var length = ''
    , n, msg;

  for (var i = 0, l = data.length; i < l; i++) {
    var chr = data.charAt(i);

    if (':' != chr) {
      length += chr;
    } else {
      if ('' == length || (length != (n = Number(length)))) {
        // parser error - ignoring payload
        return callback(err, 0, 1);
      }

      msg = data.substr(i + 1, n);

      if (length != msg.length) {
        // parser error - ignoring payload
        return callback(err, 0, 1);
      }

      if (msg.length) {
        packet = exports.decodePacket(msg, binaryType, true);

        if (err.type == packet.type && err.data == packet.data) {
          // parser error in individual packet - ignoring payload
          return callback(err, 0, 1);
        }

        var ret = callback(packet, i + n, l);
        if (false === ret) return;
      }

      // advance cursor
      i += n;
      length = '';
    }
  }

  if (length != '') {
    // parser error - ignoring payload
    return callback(err, 0, 1);
  }

};

/**
 * Encodes multiple messages (payload) as binary.
 *
 * <1 = binary, 0 = string><number from 0-9><number from 0-9>[...]<number
 * 255><data>
 *
 * Example:
 * 1 3 255 1 2 3, if the binary contents are interpreted as 8 bit integers
 *
 * @param {Array} packets
 * @return {ArrayBuffer} encoded payload
 * @api private
 */

exports.encodePayloadAsArrayBuffer = function(packets, callback) {
  if (!packets.length) {
    return callback(new ArrayBuffer(0));
  }

  function encodeOne(packet, doneCallback) {
    exports.encodePacket(packet, true, true, function(data) {
      return doneCallback(null, data);
    });
  }

  map(packets, encodeOne, function(err, encodedPackets) {
    var totalLength = encodedPackets.reduce(function(acc, p) {
      var len;
      if (typeof p === 'string'){
        len = p.length;
      } else {
        len = p.byteLength;
      }
      return acc + len.toString().length + len + 2; // string/binary identifier + separator = 2
    }, 0);

    var resultArray = new Uint8Array(totalLength);

    var bufferIndex = 0;
    encodedPackets.forEach(function(p) {
      var isString = typeof p === 'string';
      var ab = p;
      if (isString) {
        var view = new Uint8Array(p.length);
        for (var i = 0; i < p.length; i++) {
          view[i] = p.charCodeAt(i);
        }
        ab = view.buffer;
      }

      if (isString) { // not true binary
        resultArray[bufferIndex++] = 0;
      } else { // true binary
        resultArray[bufferIndex++] = 1;
      }

      var lenStr = ab.byteLength.toString();
      for (var i = 0; i < lenStr.length; i++) {
        resultArray[bufferIndex++] = parseInt(lenStr[i]);
      }
      resultArray[bufferIndex++] = 255;

      var view = new Uint8Array(ab);
      for (var i = 0; i < view.length; i++) {
        resultArray[bufferIndex++] = view[i];
      }
    });

    return callback(resultArray.buffer);
  });
};

/**
 * Encode as Blob
 */

exports.encodePayloadAsBlob = function(packets, callback) {
  function encodeOne(packet, doneCallback) {
    exports.encodePacket(packet, true, true, function(encoded) {
      var binaryIdentifier = new Uint8Array(1);
      binaryIdentifier[0] = 1;
      if (typeof encoded === 'string') {
        var view = new Uint8Array(encoded.length);
        for (var i = 0; i < encoded.length; i++) {
          view[i] = encoded.charCodeAt(i);
        }
        encoded = view.buffer;
        binaryIdentifier[0] = 0;
      }

      var len = (encoded instanceof ArrayBuffer)
        ? encoded.byteLength
        : encoded.size;

      var lenStr = len.toString();
      var lengthAry = new Uint8Array(lenStr.length + 1);
      for (var i = 0; i < lenStr.length; i++) {
        lengthAry[i] = parseInt(lenStr[i]);
      }
      lengthAry[lenStr.length] = 255;

      if (Blob) {
        var blob = new Blob([binaryIdentifier.buffer, lengthAry.buffer, encoded]);
        doneCallback(null, blob);
      }
    });
  }

  map(packets, encodeOne, function(err, results) {
    return callback(new Blob(results));
  });
};

/*
 * Decodes data when a payload is maybe expected. Strings are decoded by
 * interpreting each byte as a key code for entries marked to start with 0. See
 * description of encodePayloadAsBinary
 *
 * @param {ArrayBuffer} data, callback method
 * @api public
 */

exports.decodePayloadAsBinary = function (data, binaryType, callback) {
  if (typeof binaryType === 'function') {
    callback = binaryType;
    binaryType = null;
  }

  var bufferTail = data;
  var buffers = [];

  var numberTooLong = false;
  while (bufferTail.byteLength > 0) {
    var tailArray = new Uint8Array(bufferTail);
    var isString = tailArray[0] === 0;
    var msgLength = '';

    for (var i = 1; ; i++) {
      if (tailArray[i] == 255) break;

      if (msgLength.length > 310) {
        numberTooLong = true;
        break;
      }

      msgLength += tailArray[i];
    }

    if(numberTooLong) return callback(err, 0, 1);

    bufferTail = sliceBuffer(bufferTail, 2 + msgLength.length);
    msgLength = parseInt(msgLength);

    var msg = sliceBuffer(bufferTail, 0, msgLength);
    if (isString) {
      try {
        msg = String.fromCharCode.apply(null, new Uint8Array(msg));
      } catch (e) {
        // iPhone Safari doesn't let you apply to typed arrays
        var typed = new Uint8Array(msg);
        msg = '';
        for (var i = 0; i < typed.length; i++) {
          msg += String.fromCharCode(typed[i]);
        }
      }
    }

    buffers.push(msg);
    bufferTail = sliceBuffer(bufferTail, msgLength);
  }

  var total = buffers.length;
  buffers.forEach(function(buffer, i) {
    callback(exports.decodePacket(buffer, binaryType, true), i, total);
  });
};

}).call(this,typeof self !== "undefined" ? self : typeof window !== "undefined" ? window : {})
},{"./keys":26,"after":27,"arraybuffer.slice":28,"base64-arraybuffer":29,"blob":30,"has-binary":36,"utf8":31}],26:[function(_dereq_,module,exports){

/**
 * Gets the keys for an object.
 *
 * @return {Array} keys
 * @api private
 */

module.exports = Object.keys || function keys (obj){
  var arr = [];
  var has = Object.prototype.hasOwnProperty;

  for (var i in obj) {
    if (has.call(obj, i)) {
      arr.push(i);
    }
  }
  return arr;
};

},{}],27:[function(_dereq_,module,exports){
module.exports = after

function after(count, callback, err_cb) {
    var bail = false
    err_cb = err_cb || noop
    proxy.count = count

    return (count === 0) ? callback() : proxy

    function proxy(err, result) {
        if (proxy.count <= 0) {
            throw new Error('after called too many times')
        }
        --proxy.count

        // after first error, rest are passed to err_cb
        if (err) {
            bail = true
            callback(err)
            // future error callbacks will go to error handler
            callback = err_cb
        } else if (proxy.count === 0 && !bail) {
            callback(null, result)
        }
    }
}

function noop() {}

},{}],28:[function(_dereq_,module,exports){
/**
 * An abstraction for slicing an arraybuffer even when
 * ArrayBuffer.prototype.slice is not supported
 *
 * @api public
 */

module.exports = function(arraybuffer, start, end) {
  var bytes = arraybuffer.byteLength;
  start = start || 0;
  end = end || bytes;

  if (arraybuffer.slice) { return arraybuffer.slice(start, end); }

  if (start < 0) { start += bytes; }
  if (end < 0) { end += bytes; }
  if (end > bytes) { end = bytes; }

  if (start >= bytes || start >= end || bytes === 0) {
    return new ArrayBuffer(0);
  }

  var abv = new Uint8Array(arraybuffer);
  var result = new Uint8Array(end - start);
  for (var i = start, ii = 0; i < end; i++, ii++) {
    result[ii] = abv[i];
  }
  return result.buffer;
};

},{}],29:[function(_dereq_,module,exports){
/*
 * base64-arraybuffer
 * https://github.com/niklasvh/base64-arraybuffer
 *
 * Copyright (c) 2012 Niklas von Hertzen
 * Licensed under the MIT license.
 */
(function(chars){
  "use strict";

  exports.encode = function(arraybuffer) {
    var bytes = new Uint8Array(arraybuffer),
    i, len = bytes.length, base64 = "";

    for (i = 0; i < len; i+=3) {
      base64 += chars[bytes[i] >> 2];
      base64 += chars[((bytes[i] & 3) << 4) | (bytes[i + 1] >> 4)];
      base64 += chars[((bytes[i + 1] & 15) << 2) | (bytes[i + 2] >> 6)];
      base64 += chars[bytes[i + 2] & 63];
    }

    if ((len % 3) === 2) {
      base64 = base64.substring(0, base64.length - 1) + "=";
    } else if (len % 3 === 1) {
      base64 = base64.substring(0, base64.length - 2) + "==";
    }

    return base64;
  };

  exports.decode =  function(base64) {
    var bufferLength = base64.length * 0.75,
    len = base64.length, i, p = 0,
    encoded1, encoded2, encoded3, encoded4;

    if (base64[base64.length - 1] === "=") {
      bufferLength--;
      if (base64[base64.length - 2] === "=") {
        bufferLength--;
      }
    }

    var arraybuffer = new ArrayBuffer(bufferLength),
    bytes = new Uint8Array(arraybuffer);

    for (i = 0; i < len; i+=4) {
      encoded1 = chars.indexOf(base64[i]);
      encoded2 = chars.indexOf(base64[i+1]);
      encoded3 = chars.indexOf(base64[i+2]);
      encoded4 = chars.indexOf(base64[i+3]);

      bytes[p++] = (encoded1 << 2) | (encoded2 >> 4);
      bytes[p++] = ((encoded2 & 15) << 4) | (encoded3 >> 2);
      bytes[p++] = ((encoded3 & 3) << 6) | (encoded4 & 63);
    }

    return arraybuffer;
  };
})("ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/");

},{}],30:[function(_dereq_,module,exports){
(function (global){
/**
 * Create a blob builder even when vendor prefixes exist
 */

var BlobBuilder = global.BlobBuilder
  || global.WebKitBlobBuilder
  || global.MSBlobBuilder
  || global.MozBlobBuilder;

/**
 * Check if Blob constructor is supported
 */

var blobSupported = (function() {
  try {
    var b = new Blob(['hi']);
    return b.size == 2;
  } catch(e) {
    return false;
  }
})();

/**
 * Check if BlobBuilder is supported
 */

var blobBuilderSupported = BlobBuilder
  && BlobBuilder.prototype.append
  && BlobBuilder.prototype.getBlob;

function BlobBuilderConstructor(ary, options) {
  options = options || {};

  var bb = new BlobBuilder();
  for (var i = 0; i < ary.length; i++) {
    bb.append(ary[i]);
  }
  return (options.type) ? bb.getBlob(options.type) : bb.getBlob();
};

module.exports = (function() {
  if (blobSupported) {
    return global.Blob;
  } else if (blobBuilderSupported) {
    return BlobBuilderConstructor;
  } else {
    return undefined;
  }
})();

}).call(this,typeof self !== "undefined" ? self : typeof window !== "undefined" ? window : {})
},{}],31:[function(_dereq_,module,exports){
(function (global){
/*! http://mths.be/utf8js v2.0.0 by @mathias */
;(function(root) {

	// Detect free variables `exports`
	var freeExports = typeof exports == 'object' && exports;

	// Detect free variable `module`
	var freeModule = typeof module == 'object' && module &&
		module.exports == freeExports && module;

	// Detect free variable `global`, from Node.js or Browserified code,
	// and use it as `root`
	var freeGlobal = typeof global == 'object' && global;
	if (freeGlobal.global === freeGlobal || freeGlobal.window === freeGlobal) {
		root = freeGlobal;
	}

	/*--------------------------------------------------------------------------*/

	var stringFromCharCode = String.fromCharCode;

	// Taken from http://mths.be/punycode
	function ucs2decode(string) {
		var output = [];
		var counter = 0;
		var length = string.length;
		var value;
		var extra;
		while (counter < length) {
			value = string.charCodeAt(counter++);
			if (value >= 0xD800 && value <= 0xDBFF && counter < length) {
				// high surrogate, and there is a next character
				extra = string.charCodeAt(counter++);
				if ((extra & 0xFC00) == 0xDC00) { // low surrogate
					output.push(((value & 0x3FF) << 10) + (extra & 0x3FF) + 0x10000);
				} else {
					// unmatched surrogate; only append this code unit, in case the next
					// code unit is the high surrogate of a surrogate pair
					output.push(value);
					counter--;
				}
			} else {
				output.push(value);
			}
		}
		return output;
	}

	// Taken from http://mths.be/punycode
	function ucs2encode(array) {
		var length = array.length;
		var index = -1;
		var value;
		var output = '';
		while (++index < length) {
			value = array[index];
			if (value > 0xFFFF) {
				value -= 0x10000;
				output += stringFromCharCode(value >>> 10 & 0x3FF | 0xD800);
				value = 0xDC00 | value & 0x3FF;
			}
			output += stringFromCharCode(value);
		}
		return output;
	}

	/*--------------------------------------------------------------------------*/

	function createByte(codePoint, shift) {
		return stringFromCharCode(((codePoint >> shift) & 0x3F) | 0x80);
	}

	function encodeCodePoint(codePoint) {
		if ((codePoint & 0xFFFFFF80) == 0) { // 1-byte sequence
			return stringFromCharCode(codePoint);
		}
		var symbol = '';
		if ((codePoint & 0xFFFFF800) == 0) { // 2-byte sequence
			symbol = stringFromCharCode(((codePoint >> 6) & 0x1F) | 0xC0);
		}
		else if ((codePoint & 0xFFFF0000) == 0) { // 3-byte sequence
			symbol = stringFromCharCode(((codePoint >> 12) & 0x0F) | 0xE0);
			symbol += createByte(codePoint, 6);
		}
		else if ((codePoint & 0xFFE00000) == 0) { // 4-byte sequence
			symbol = stringFromCharCode(((codePoint >> 18) & 0x07) | 0xF0);
			symbol += createByte(codePoint, 12);
			symbol += createByte(codePoint, 6);
		}
		symbol += stringFromCharCode((codePoint & 0x3F) | 0x80);
		return symbol;
	}

	function utf8encode(string) {
		var codePoints = ucs2decode(string);

		// console.log(JSON.stringify(codePoints.map(function(x) {
		// 	return 'U+' + x.toString(16).toUpperCase();
		// })));

		var length = codePoints.length;
		var index = -1;
		var codePoint;
		var byteString = '';
		while (++index < length) {
			codePoint = codePoints[index];
			byteString += encodeCodePoint(codePoint);
		}
		return byteString;
	}

	/*--------------------------------------------------------------------------*/

	function readContinuationByte() {
		if (byteIndex >= byteCount) {
			throw Error('Invalid byte index');
		}

		var continuationByte = byteArray[byteIndex] & 0xFF;
		byteIndex++;

		if ((continuationByte & 0xC0) == 0x80) {
			return continuationByte & 0x3F;
		}

		// If we end up here, it’s not a continuation byte
		throw Error('Invalid continuation byte');
	}

	function decodeSymbol() {
		var byte1;
		var byte2;
		var byte3;
		var byte4;
		var codePoint;

		if (byteIndex > byteCount) {
			throw Error('Invalid byte index');
		}

		if (byteIndex == byteCount) {
			return false;
		}

		// Read first byte
		byte1 = byteArray[byteIndex] & 0xFF;
		byteIndex++;

		// 1-byte sequence (no continuation bytes)
		if ((byte1 & 0x80) == 0) {
			return byte1;
		}

		// 2-byte sequence
		if ((byte1 & 0xE0) == 0xC0) {
			var byte2 = readContinuationByte();
			codePoint = ((byte1 & 0x1F) << 6) | byte2;
			if (codePoint >= 0x80) {
				return codePoint;
			} else {
				throw Error('Invalid continuation byte');
			}
		}

		// 3-byte sequence (may include unpaired surrogates)
		if ((byte1 & 0xF0) == 0xE0) {
			byte2 = readContinuationByte();
			byte3 = readContinuationByte();
			codePoint = ((byte1 & 0x0F) << 12) | (byte2 << 6) | byte3;
			if (codePoint >= 0x0800) {
				return codePoint;
			} else {
				throw Error('Invalid continuation byte');
			}
		}

		// 4-byte sequence
		if ((byte1 & 0xF8) == 0xF0) {
			byte2 = readContinuationByte();
			byte3 = readContinuationByte();
			byte4 = readContinuationByte();
			codePoint = ((byte1 & 0x0F) << 0x12) | (byte2 << 0x0C) |
				(byte3 << 0x06) | byte4;
			if (codePoint >= 0x010000 && codePoint <= 0x10FFFF) {
				return codePoint;
			}
		}

		throw Error('Invalid UTF-8 detected');
	}

	var byteArray;
	var byteCount;
	var byteIndex;
	function utf8decode(byteString) {
		byteArray = ucs2decode(byteString);
		byteCount = byteArray.length;
		byteIndex = 0;
		var codePoints = [];
		var tmp;
		while ((tmp = decodeSymbol()) !== false) {
			codePoints.push(tmp);
		}
		return ucs2encode(codePoints);
	}

	/*--------------------------------------------------------------------------*/

	var utf8 = {
		'version': '2.0.0',
		'encode': utf8encode,
		'decode': utf8decode
	};

	// Some AMD build optimizers, like r.js, check for specific condition patterns
	// like the following:
	if (
		typeof define == 'function' &&
		typeof define.amd == 'object' &&
		define.amd
	) {
		define(function() {
			return utf8;
		});
	}	else if (freeExports && !freeExports.nodeType) {
		if (freeModule) { // in Node.js or RingoJS v0.8.0+
			freeModule.exports = utf8;
		} else { // in Narwhal or RingoJS v0.7.0-
			var object = {};
			var hasOwnProperty = object.hasOwnProperty;
			for (var key in utf8) {
				hasOwnProperty.call(utf8, key) && (freeExports[key] = utf8[key]);
			}
		}
	} else { // in Rhino or a web browser
		root.utf8 = utf8;
	}

}(this));

}).call(this,typeof self !== "undefined" ? self : typeof window !== "undefined" ? window : {})
},{}],32:[function(_dereq_,module,exports){
(function (global){
/**
 * JSON parse.
 *
 * @see Based on jQuery#parseJSON (MIT) and JSON2
 * @api private
 */

var rvalidchars = /^[\],:{}\s]*$/;
var rvalidescape = /\\(?:["\\\/bfnrt]|u[0-9a-fA-F]{4})/g;
var rvalidtokens = /"[^"\\\n\r]*"|true|false|null|-?\d+(?:\.\d*)?(?:[eE][+\-]?\d+)?/g;
var rvalidbraces = /(?:^|:|,)(?:\s*\[)+/g;
var rtrimLeft = /^\s+/;
var rtrimRight = /\s+$/;

module.exports = function parsejson(data) {
  if ('string' != typeof data || !data) {
    return null;
  }

  data = data.replace(rtrimLeft, '').replace(rtrimRight, '');

  // Attempt to parse using the native JSON parser first
  if (global.JSON && JSON.parse) {
    return JSON.parse(data);
  }

  if (rvalidchars.test(data.replace(rvalidescape, '@')
      .replace(rvalidtokens, ']')
      .replace(rvalidbraces, ''))) {
    return (new Function('return ' + data))();
  }
};
}).call(this,typeof self !== "undefined" ? self : typeof window !== "undefined" ? window : {})
},{}],33:[function(_dereq_,module,exports){
/**
 * Compiles a querystring
 * Returns string representation of the object
 *
 * @param {Object}
 * @api private
 */

exports.encode = function (obj) {
  var str = '';

  for (var i in obj) {
    if (obj.hasOwnProperty(i)) {
      if (str.length) str += '&';
      str += encodeURIComponent(i) + '=' + encodeURIComponent(obj[i]);
    }
  }

  return str;
};

/**
 * Parses a simple querystring into an object
 *
 * @param {String} qs
 * @api private
 */

exports.decode = function(qs){
  var qry = {};
  var pairs = qs.split('&');
  for (var i = 0, l = pairs.length; i < l; i++) {
    var pair = pairs[i].split('=');
    qry[decodeURIComponent(pair[0])] = decodeURIComponent(pair[1]);
  }
  return qry;
};

},{}],34:[function(_dereq_,module,exports){
/**
 * Parses an URI
 *
 * @author Steven Levithan <stevenlevithan.com> (MIT license)
 * @api private
 */

var re = /^(?:(?![^:@]+:[^:@\/]*@)(http|https|ws|wss):\/\/)?((?:(([^:@]*)(?::([^:@]*))?)?@)?((?:[a-f0-9]{0,4}:){2,7}[a-f0-9]{0,4}|[^:\/?#]*)(?::(\d*))?)(((\/(?:[^?#](?![^?#\/]*\.[^?#\/.]+(?:[?#]|$)))*\/?)?([^?#\/]*))(?:\?([^#]*))?(?:#(.*))?)/;

var parts = [
    'source', 'protocol', 'authority', 'userInfo', 'user', 'password', 'host', 'port', 'relative', 'path', 'directory', 'file', 'query', 'anchor'
];

module.exports = function parseuri(str) {
    var src = str,
        b = str.indexOf('['),
        e = str.indexOf(']');

    if (b != -1 && e != -1) {
        str = str.substring(0, b) + str.substring(b, e).replace(/:/g, ';') + str.substring(e, str.length);
    }

    var m = re.exec(str || ''),
        uri = {},
        i = 14;

    while (i--) {
        uri[parts[i]] = m[i] || '';
    }

    if (b != -1 && e != -1) {
        uri.source = src;
        uri.host = uri.host.substring(1, uri.host.length - 1).replace(/;/g, ':');
        uri.authority = uri.authority.replace('[', '').replace(']', '').replace(/;/g, ':');
        uri.ipv6uri = true;
    }

    return uri;
};

},{}],35:[function(_dereq_,module,exports){

/**
 * Module dependencies.
 */

var global = (function() { return this; })();

/**
 * WebSocket constructor.
 */

var WebSocket = global.WebSocket || global.MozWebSocket;

/**
 * Module exports.
 */

module.exports = WebSocket ? ws : null;

/**
 * WebSocket constructor.
 *
 * The third `opts` options object gets ignored in web browsers, since it's
 * non-standard, and throws a TypeError if passed to the constructor.
 * See: https://github.com/einaros/ws/issues/227
 *
 * @param {String} uri
 * @param {Array} protocols (optional)
 * @param {Object) opts (optional)
 * @api public
 */

function ws(uri, protocols, opts) {
  var instance;
  if (protocols) {
    instance = new WebSocket(uri, protocols);
  } else {
    instance = new WebSocket(uri);
  }
  return instance;
}

if (WebSocket) ws.prototype = WebSocket.prototype;

},{}],36:[function(_dereq_,module,exports){
(function (global){

/*
 * Module requirements.
 */

var isArray = _dereq_('isarray');

/**
 * Module exports.
 */

module.exports = hasBinary;

/**
 * Checks for binary data.
 *
 * Right now only Buffer and ArrayBuffer are supported..
 *
 * @param {Object} anything
 * @api public
 */

function hasBinary(data) {

  function _hasBinary(obj) {
    if (!obj) return false;

    if ( (global.Buffer && global.Buffer.isBuffer(obj)) ||
         (global.ArrayBuffer && obj instanceof ArrayBuffer) ||
         (global.Blob && obj instanceof Blob) ||
         (global.File && obj instanceof File)
        ) {
      return true;
    }

    if (isArray(obj)) {
      for (var i = 0; i < obj.length; i++) {
          if (_hasBinary(obj[i])) {
              return true;
          }
      }
    } else if (obj && 'object' == typeof obj) {
      if (obj.toJSON) {
        obj = obj.toJSON();
      }

      for (var key in obj) {
        if (obj.hasOwnProperty(key) && _hasBinary(obj[key])) {
          return true;
        }
      }
    }

    return false;
  }

  return _hasBinary(data);
}

}).call(this,typeof self !== "undefined" ? self : typeof window !== "undefined" ? window : {})
},{"isarray":37}],37:[function(_dereq_,module,exports){
module.exports = Array.isArray || function (arr) {
  return Object.prototype.toString.call(arr) == '[object Array]';
};

},{}],38:[function(_dereq_,module,exports){

/**
 * Module dependencies.
 */

var global = _dereq_('global');

/**
 * Module exports.
 *
 * Logic borrowed from Modernizr:
 *
 *   - https://github.com/Modernizr/Modernizr/blob/master/feature-detects/cors.js
 */

try {
  module.exports = 'XMLHttpRequest' in global &&
    'withCredentials' in new global.XMLHttpRequest();
} catch (err) {
  // if XMLHttp support is disabled in IE then it will throw
  // when trying to create
  module.exports = false;
}

},{"global":39}],39:[function(_dereq_,module,exports){

/**
 * Returns `this`. Execute this without a "context" (i.e. without it being
 * attached to an object of the left-hand side), and `this` points to the
 * "global" scope of the current JS execution.
 */

module.exports = (function () { return this; })();

},{}],40:[function(_dereq_,module,exports){

var indexOf = [].indexOf;

module.exports = function(arr, obj){
  if (indexOf) return arr.indexOf(obj);
  for (var i = 0; i < arr.length; ++i) {
    if (arr[i] === obj) return i;
  }
  return -1;
};
},{}],41:[function(_dereq_,module,exports){

/**
 * HOP ref.
 */

var has = Object.prototype.hasOwnProperty;

/**
 * Return own keys in `obj`.
 *
 * @param {Object} obj
 * @return {Array}
 * @api public
 */

exports.keys = Object.keys || function(obj){
  var keys = [];
  for (var key in obj) {
    if (has.call(obj, key)) {
      keys.push(key);
    }
  }
  return keys;
};

/**
 * Return own values in `obj`.
 *
 * @param {Object} obj
 * @return {Array}
 * @api public
 */

exports.values = function(obj){
  var vals = [];
  for (var key in obj) {
    if (has.call(obj, key)) {
      vals.push(obj[key]);
    }
  }
  return vals;
};

/**
 * Merge `b` into `a`.
 *
 * @param {Object} a
 * @param {Object} b
 * @return {Object} a
 * @api public
 */

exports.merge = function(a, b){
  for (var key in b) {
    if (has.call(b, key)) {
      a[key] = b[key];
    }
  }
  return a;
};

/**
 * Return length of `obj`.
 *
 * @param {Object} obj
 * @return {Number}
 * @api public
 */

exports.length = function(obj){
  return exports.keys(obj).length;
};

/**
 * Check if `obj` is empty.
 *
 * @param {Object} obj
 * @return {Boolean}
 * @api public
 */

exports.isEmpty = function(obj){
  return 0 == exports.length(obj);
};
},{}],42:[function(_dereq_,module,exports){
/**
 * Parses an URI
 *
 * @author Steven Levithan <stevenlevithan.com> (MIT license)
 * @api private
 */

var re = /^(?:(?![^:@]+:[^:@\/]*@)(http|https|ws|wss):\/\/)?((?:(([^:@]*)(?::([^:@]*))?)?@)?((?:[a-f0-9]{0,4}:){2,7}[a-f0-9]{0,4}|[^:\/?#]*)(?::(\d*))?)(((\/(?:[^?#](?![^?#\/]*\.[^?#\/.]+(?:[?#]|$)))*\/?)?([^?#\/]*))(?:\?([^#]*))?(?:#(.*))?)/;

var parts = [
    'source', 'protocol', 'authority', 'userInfo', 'user', 'password', 'host'
  , 'port', 'relative', 'path', 'directory', 'file', 'query', 'anchor'
];

module.exports = function parseuri(str) {
  var m = re.exec(str || '')
    , uri = {}
    , i = 14;

  while (i--) {
    uri[parts[i]] = m[i] || '';
  }

  return uri;
};

},{}],43:[function(_dereq_,module,exports){
(function (global){
/*global Blob,File*/

/**
 * Module requirements
 */

var isArray = _dereq_('isarray');
var isBuf = _dereq_('./is-buffer');

/**
 * Replaces every Buffer | ArrayBuffer in packet with a numbered placeholder.
 * Anything with blobs or files should be fed through removeBlobs before coming
 * here.
 *
 * @param {Object} packet - socket.io event packet
 * @return {Object} with deconstructed packet and list of buffers
 * @api public
 */

exports.deconstructPacket = function(packet){
  var buffers = [];
  var packetData = packet.data;

  function _deconstructPacket(data) {
    if (!data) return data;

    if (isBuf(data)) {
      var placeholder = { _placeholder: true, num: buffers.length };
      buffers.push(data);
      return placeholder;
    } else if (isArray(data)) {
      var newData = new Array(data.length);
      for (var i = 0; i < data.length; i++) {
        newData[i] = _deconstructPacket(data[i]);
      }
      return newData;
    } else if ('object' == typeof data && !(data instanceof Date)) {
      var newData = {};
      for (var key in data) {
        newData[key] = _deconstructPacket(data[key]);
      }
      return newData;
    }
    return data;
  }

  var pack = packet;
  pack.data = _deconstructPacket(packetData);
  pack.attachments = buffers.length; // number of binary 'attachments'
  return {packet: pack, buffers: buffers};
};

/**
 * Reconstructs a binary packet from its placeholder packet and buffers
 *
 * @param {Object} packet - event packet with placeholders
 * @param {Array} buffers - binary buffers to put in placeholder positions
 * @return {Object} reconstructed packet
 * @api public
 */

exports.reconstructPacket = function(packet, buffers) {
  var curPlaceHolder = 0;

  function _reconstructPacket(data) {
    if (data && data._placeholder) {
      var buf = buffers[data.num]; // appropriate buffer (should be natural order anyway)
      return buf;
    } else if (isArray(data)) {
      for (var i = 0; i < data.length; i++) {
        data[i] = _reconstructPacket(data[i]);
      }
      return data;
    } else if (data && 'object' == typeof data) {
      for (var key in data) {
        data[key] = _reconstructPacket(data[key]);
      }
      return data;
    }
    return data;
  }

  packet.data = _reconstructPacket(packet.data);
  packet.attachments = undefined; // no longer useful
  return packet;
};

/**
 * Asynchronously removes Blobs or Files from data via
 * FileReader's readAsArrayBuffer method. Used before encoding
 * data as msgpack. Calls callback with the blobless data.
 *
 * @param {Object} data
 * @param {Function} callback
 * @api private
 */

exports.removeBlobs = function(data, callback) {
  function _removeBlobs(obj, curKey, containingObject) {
    if (!obj) return obj;

    // convert any blob
    if ((global.Blob && obj instanceof Blob) ||
        (global.File && obj instanceof File)) {
      pendingBlobs++;

      // async filereader
      var fileReader = new FileReader();
      fileReader.onload = function() { // this.result == arraybuffer
        if (containingObject) {
          containingObject[curKey] = this.result;
        }
        else {
          bloblessData = this.result;
        }

        // if nothing pending its callback time
        if(! --pendingBlobs) {
          callback(bloblessData);
        }
      };

      fileReader.readAsArrayBuffer(obj); // blob -> arraybuffer
    } else if (isArray(obj)) { // handle array
      for (var i = 0; i < obj.length; i++) {
        _removeBlobs(obj[i], i, obj);
      }
    } else if (obj && 'object' == typeof obj && !isBuf(obj)) { // and object
      for (var key in obj) {
        _removeBlobs(obj[key], key, obj);
      }
    }
  }

  var pendingBlobs = 0;
  var bloblessData = data;
  _removeBlobs(bloblessData);
  if (!pendingBlobs) {
    callback(bloblessData);
  }
};

}).call(this,typeof self !== "undefined" ? self : typeof window !== "undefined" ? window : {})
},{"./is-buffer":45,"isarray":46}],44:[function(_dereq_,module,exports){

/**
 * Module dependencies.
 */

var debug = _dereq_('debug')('socket.io-parser');
var json = _dereq_('json3');
var isArray = _dereq_('isarray');
var Emitter = _dereq_('component-emitter');
var binary = _dereq_('./binary');
var isBuf = _dereq_('./is-buffer');

/**
 * Protocol version.
 *
 * @api public
 */

exports.protocol = 4;

/**
 * Packet types.
 *
 * @api public
 */

exports.types = [
  'CONNECT',
  'DISCONNECT',
  'EVENT',
  'BINARY_EVENT',
  'ACK',
  'BINARY_ACK',
  'ERROR'
];

/**
 * Packet type `connect`.
 *
 * @api public
 */

exports.CONNECT = 0;

/**
 * Packet type `disconnect`.
 *
 * @api public
 */

exports.DISCONNECT = 1;

/**
 * Packet type `event`.
 *
 * @api public
 */

exports.EVENT = 2;

/**
 * Packet type `ack`.
 *
 * @api public
 */

exports.ACK = 3;

/**
 * Packet type `error`.
 *
 * @api public
 */

exports.ERROR = 4;

/**
 * Packet type 'binary event'
 *
 * @api public
 */

exports.BINARY_EVENT = 5;

/**
 * Packet type `binary ack`. For acks with binary arguments.
 *
 * @api public
 */

exports.BINARY_ACK = 6;

/**
 * Encoder constructor.
 *
 * @api public
 */

exports.Encoder = Encoder;

/**
 * Decoder constructor.
 *
 * @api public
 */

exports.Decoder = Decoder;

/**
 * A socket.io Encoder instance
 *
 * @api public
 */

function Encoder() {}

/**
 * Encode a packet as a single string if non-binary, or as a
 * buffer sequence, depending on packet type.
 *
 * @param {Object} obj - packet object
 * @param {Function} callback - function to handle encodings (likely engine.write)
 * @return Calls callback with Array of encodings
 * @api public
 */

Encoder.prototype.encode = function(obj, callback){
  debug('encoding packet %j', obj);

  if (exports.BINARY_EVENT == obj.type || exports.BINARY_ACK == obj.type) {
    encodeAsBinary(obj, callback);
  }
  else {
    var encoding = encodeAsString(obj);
    callback([encoding]);
  }
};

/**
 * Encode packet as string.
 *
 * @param {Object} packet
 * @return {String} encoded
 * @api private
 */

function encodeAsString(obj) {
  var str = '';
  var nsp = false;

  // first is type
  str += obj.type;

  // attachments if we have them
  if (exports.BINARY_EVENT == obj.type || exports.BINARY_ACK == obj.type) {
    str += obj.attachments;
    str += '-';
  }

  // if we have a namespace other than `/`
  // we append it followed by a comma `,`
  if (obj.nsp && '/' != obj.nsp) {
    nsp = true;
    str += obj.nsp;
  }

  // immediately followed by the id
  if (null != obj.id) {
    if (nsp) {
      str += ',';
      nsp = false;
    }
    str += obj.id;
  }

  // json data
  if (null != obj.data) {
    if (nsp) str += ',';
    str += json.stringify(obj.data);
  }

  debug('encoded %j as %s', obj, str);
  return str;
}

/**
 * Encode packet as 'buffer sequence' by removing blobs, and
 * deconstructing packet into object with placeholders and
 * a list of buffers.
 *
 * @param {Object} packet
 * @return {Buffer} encoded
 * @api private
 */

function encodeAsBinary(obj, callback) {

  function writeEncoding(bloblessData) {
    var deconstruction = binary.deconstructPacket(bloblessData);
    var pack = encodeAsString(deconstruction.packet);
    var buffers = deconstruction.buffers;

    buffers.unshift(pack); // add packet info to beginning of data list
    callback(buffers); // write all the buffers
  }

  binary.removeBlobs(obj, writeEncoding);
}

/**
 * A socket.io Decoder instance
 *
 * @return {Object} decoder
 * @api public
 */

function Decoder() {
  this.reconstructor = null;
}

/**
 * Mix in `Emitter` with Decoder.
 */

Emitter(Decoder.prototype);

/**
 * Decodes an ecoded packet string into packet JSON.
 *
 * @param {String} obj - encoded packet
 * @return {Object} packet
 * @api public
 */

Decoder.prototype.add = function(obj) {
  var packet;
  if ('string' == typeof obj) {
    packet = decodeString(obj);
    if (exports.BINARY_EVENT == packet.type || exports.BINARY_ACK == packet.type) { // binary packet's json
      this.reconstructor = new BinaryReconstructor(packet);

      // no attachments, labeled binary but no binary data to follow
      if (this.reconstructor.reconPack.attachments == 0) {
        this.emit('decoded', packet);
      }
    } else { // non-binary full packet
      this.emit('decoded', packet);
    }
  }
  else if (isBuf(obj) || obj.base64) { // raw binary data
    if (!this.reconstructor) {
      throw new Error('got binary data when not reconstructing a packet');
    } else {
      packet = this.reconstructor.takeBinaryData(obj);
      if (packet) { // received final buffer
        this.reconstructor = null;
        this.emit('decoded', packet);
      }
    }
  }
  else {
    throw new Error('Unknown type: ' + obj);
  }
};

/**
 * Decode a packet String (JSON data)
 *
 * @param {String} str
 * @return {Object} packet
 * @api private
 */

function decodeString(str) {
  var p = {};
  var i = 0;

  // look up type
  p.type = Number(str.charAt(0));
  if (null == exports.types[p.type]) return error();

  // look up attachments if type binary
  if (exports.BINARY_EVENT == p.type || exports.BINARY_ACK == p.type) {
    p.attachments = '';
    while (str.charAt(++i) != '-') {
      p.attachments += str.charAt(i);
    }
    p.attachments = Number(p.attachments);
  }

  // look up namespace (if any)
  if ('/' == str.charAt(i + 1)) {
    p.nsp = '';
    while (++i) {
      var c = str.charAt(i);
      if (',' == c) break;
      p.nsp += c;
      if (i + 1 == str.length) break;
    }
  } else {
    p.nsp = '/';
  }

  // look up id
  var next = str.charAt(i + 1);
  if ('' != next && Number(next) == next) {
    p.id = '';
    while (++i) {
      var c = str.charAt(i);
      if (null == c || Number(c) != c) {
        --i;
        break;
      }
      p.id += str.charAt(i);
      if (i + 1 == str.length) break;
    }
    p.id = Number(p.id);
  }

  // look up json data
  if (str.charAt(++i)) {
    try {
      p.data = json.parse(str.substr(i));
    } catch(e){
      return error();
    }
  }

  debug('decoded %s as %j', str, p);
  return p;
}

/**
 * Deallocates a parser's resources
 *
 * @api public
 */

Decoder.prototype.destroy = function() {
  if (this.reconstructor) {
    this.reconstructor.finishedReconstruction();
  }
};

/**
 * A manager of a binary event's 'buffer sequence'. Should
 * be constructed whenever a packet of type BINARY_EVENT is
 * decoded.
 *
 * @param {Object} packet
 * @return {BinaryReconstructor} initialized reconstructor
 * @api private
 */

function BinaryReconstructor(packet) {
  this.reconPack = packet;
  this.buffers = [];
}

/**
 * Method to be called when binary data received from connection
 * after a BINARY_EVENT packet.
 *
 * @param {Buffer | ArrayBuffer} binData - the raw binary data received
 * @return {null | Object} returns null if more binary data is expected or
 *   a reconstructed packet object if all buffers have been received.
 * @api private
 */

BinaryReconstructor.prototype.takeBinaryData = function(binData) {
  this.buffers.push(binData);
  if (this.buffers.length == this.reconPack.attachments) { // done with buffer list
    var packet = binary.reconstructPacket(this.reconPack, this.buffers);
    this.finishedReconstruction();
    return packet;
  }
  return null;
};

/**
 * Cleans up binary packet reconstruction variables.
 *
 * @api private
 */

BinaryReconstructor.prototype.finishedReconstruction = function() {
  this.reconPack = null;
  this.buffers = [];
};

function error(data){
  return {
    type: exports.ERROR,
    data: 'parser error'
  };
}

},{"./binary":43,"./is-buffer":45,"component-emitter":9,"debug":10,"isarray":46,"json3":47}],45:[function(_dereq_,module,exports){
(function (global){

module.exports = isBuf;

/**
 * Returns true if obj is a buffer or an arraybuffer.
 *
 * @api private
 */

function isBuf(obj) {
  return (global.Buffer && global.Buffer.isBuffer(obj)) ||
         (global.ArrayBuffer && obj instanceof ArrayBuffer);
}

}).call(this,typeof self !== "undefined" ? self : typeof window !== "undefined" ? window : {})
},{}],46:[function(_dereq_,module,exports){
module.exports=_dereq_(37)
},{}],47:[function(_dereq_,module,exports){
/*! JSON v3.2.6 | http://bestiejs.github.io/json3 | Copyright 2012-2013, Kit Cambridge | http://kit.mit-license.org */
;(function (window) {
  // Convenience aliases.
  var getClass = {}.toString, isProperty, forEach, undef;

  // Detect the `define` function exposed by asynchronous module loaders. The
  // strict `define` check is necessary for compatibility with `r.js`.
  var isLoader = typeof define === "function" && define.amd;

  // Detect native implementations.
  var nativeJSON = typeof JSON == "object" && JSON;

  // Set up the JSON 3 namespace, preferring the CommonJS `exports` object if
  // available.
  var JSON3 = typeof exports == "object" && exports && !exports.nodeType && exports;

  if (JSON3 && nativeJSON) {
    // Explicitly delegate to the native `stringify` and `parse`
    // implementations in CommonJS environments.
    JSON3.stringify = nativeJSON.stringify;
    JSON3.parse = nativeJSON.parse;
  } else {
    // Export for web browsers, JavaScript engines, and asynchronous module
    // loaders, using the global `JSON` object if available.
    JSON3 = window.JSON = nativeJSON || {};
  }

  // Test the `Date#getUTC*` methods. Based on work by @Yaffle.
  var isExtended = new Date(-3509827334573292);
  try {
    // The `getUTCFullYear`, `Month`, and `Date` methods return nonsensical
    // results for certain dates in Opera >= 10.53.
    isExtended = isExtended.getUTCFullYear() == -109252 && isExtended.getUTCMonth() === 0 && isExtended.getUTCDate() === 1 &&
      // Safari < 2.0.2 stores the internal millisecond time value correctly,
      // but clips the values returned by the date methods to the range of
      // signed 32-bit integers ([-2 ** 31, 2 ** 31 - 1]).
      isExtended.getUTCHours() == 10 && isExtended.getUTCMinutes() == 37 && isExtended.getUTCSeconds() == 6 && isExtended.getUTCMilliseconds() == 708;
  } catch (exception) {}

  // Internal: Determines whether the native `JSON.stringify` and `parse`
  // implementations are spec-compliant. Based on work by Ken Snyder.
  function has(name) {
    if (has[name] !== undef) {
      // Return cached feature test result.
      return has[name];
    }

    var isSupported;
    if (name == "bug-string-char-index") {
      // IE <= 7 doesn't support accessing string characters using square
      // bracket notation. IE 8 only supports this for primitives.
      isSupported = "a"[0] != "a";
    } else if (name == "json") {
      // Indicates whether both `JSON.stringify` and `JSON.parse` are
      // supported.
      isSupported = has("json-stringify") && has("json-parse");
    } else {
      var value, serialized = '{"a":[1,true,false,null,"\\u0000\\b\\n\\f\\r\\t"]}';
      // Test `JSON.stringify`.
      if (name == "json-stringify") {
        var stringify = JSON3.stringify, stringifySupported = typeof stringify == "function" && isExtended;
        if (stringifySupported) {
          // A test function object with a custom `toJSON` method.
          (value = function () {
            return 1;
          }).toJSON = value;
          try {
            stringifySupported =
              // Firefox 3.1b1 and b2 serialize string, number, and boolean
              // primitives as object literals.
              stringify(0) === "0" &&
              // FF 3.1b1, b2, and JSON 2 serialize wrapped primitives as object
              // literals.
              stringify(new Number()) === "0" &&
              stringify(new String()) == '""' &&
              // FF 3.1b1, 2 throw an error if the value is `null`, `undefined`, or
              // does not define a canonical JSON representation (this applies to
              // objects with `toJSON` properties as well, *unless* they are nested
              // within an object or array).
              stringify(getClass) === undef &&
              // IE 8 serializes `undefined` as `"undefined"`. Safari <= 5.1.7 and
              // FF 3.1b3 pass this test.
              stringify(undef) === undef &&
              // Safari <= 5.1.7 and FF 3.1b3 throw `Error`s and `TypeError`s,
              // respectively, if the value is omitted entirely.
              stringify() === undef &&
              // FF 3.1b1, 2 throw an error if the given value is not a number,
              // string, array, object, Boolean, or `null` literal. This applies to
              // objects with custom `toJSON` methods as well, unless they are nested
              // inside object or array literals. YUI 3.0.0b1 ignores custom `toJSON`
              // methods entirely.
              stringify(value) === "1" &&
              stringify([value]) == "[1]" &&
              // Prototype <= 1.6.1 serializes `[undefined]` as `"[]"` instead of
              // `"[null]"`.
              stringify([undef]) == "[null]" &&
              // YUI 3.0.0b1 fails to serialize `null` literals.
              stringify(null) == "null" &&
              // FF 3.1b1, 2 halts serialization if an array contains a function:
              // `[1, true, getClass, 1]` serializes as "[1,true,],". FF 3.1b3
              // elides non-JSON values from objects and arrays, unless they
              // define custom `toJSON` methods.
              stringify([undef, getClass, null]) == "[null,null,null]" &&
              // Simple serialization test. FF 3.1b1 uses Unicode escape sequences
              // where character escape codes are expected (e.g., `\b` => `\u0008`).
              stringify({ "a": [value, true, false, null, "\x00\b\n\f\r\t"] }) == serialized &&
              // FF 3.1b1 and b2 ignore the `filter` and `width` arguments.
              stringify(null, value) === "1" &&
              stringify([1, 2], null, 1) == "[\n 1,\n 2\n]" &&
              // JSON 2, Prototype <= 1.7, and older WebKit builds incorrectly
              // serialize extended years.
              stringify(new Date(-8.64e15)) == '"-271821-04-20T00:00:00.000Z"' &&
              // The milliseconds are optional in ES 5, but required in 5.1.
              stringify(new Date(8.64e15)) == '"+275760-09-13T00:00:00.000Z"' &&
              // Firefox <= 11.0 incorrectly serializes years prior to 0 as negative
              // four-digit years instead of six-digit years. Credits: @Yaffle.
              stringify(new Date(-621987552e5)) == '"-000001-01-01T00:00:00.000Z"' &&
              // Safari <= 5.1.5 and Opera >= 10.53 incorrectly serialize millisecond
              // values less than 1000. Credits: @Yaffle.
              stringify(new Date(-1)) == '"1969-12-31T23:59:59.999Z"';
          } catch (exception) {
            stringifySupported = false;
          }
        }
        isSupported = stringifySupported;
      }
      // Test `JSON.parse`.
      if (name == "json-parse") {
        var parse = JSON3.parse;
        if (typeof parse == "function") {
          try {
            // FF 3.1b1, b2 will throw an exception if a bare literal is provided.
            // Conforming implementations should also coerce the initial argument to
            // a string prior to parsing.
            if (parse("0") === 0 && !parse(false)) {
              // Simple parsing test.
              value = parse(serialized);
              var parseSupported = value["a"].length == 5 && value["a"][0] === 1;
              if (parseSupported) {
                try {
                  // Safari <= 5.1.2 and FF 3.1b1 allow unescaped tabs in strings.
                  parseSupported = !parse('"\t"');
                } catch (exception) {}
                if (parseSupported) {
                  try {
                    // FF 4.0 and 4.0.1 allow leading `+` signs and leading
                    // decimal points. FF 4.0, 4.0.1, and IE 9-10 also allow
                    // certain octal literals.
                    parseSupported = parse("01") !== 1;
                  } catch (exception) {}
                }
                if (parseSupported) {
                  try {
                    // FF 4.0, 4.0.1, and Rhino 1.7R3-R4 allow trailing decimal
                    // points. These environments, along with FF 3.1b1 and 2,
                    // also allow trailing commas in JSON objects and arrays.
                    parseSupported = parse("1.") !== 1;
                  } catch (exception) {}
                }
              }
            }
          } catch (exception) {
            parseSupported = false;
          }
        }
        isSupported = parseSupported;
      }
    }
    return has[name] = !!isSupported;
  }

  if (!has("json")) {
    // Common `[[Class]]` name aliases.
    var functionClass = "[object Function]";
    var dateClass = "[object Date]";
    var numberClass = "[object Number]";
    var stringClass = "[object String]";
    var arrayClass = "[object Array]";
    var booleanClass = "[object Boolean]";

    // Detect incomplete support for accessing string characters by index.
    var charIndexBuggy = has("bug-string-char-index");

    // Define additional utility methods if the `Date` methods are buggy.
    if (!isExtended) {
      var floor = Math.floor;
      // A mapping between the months of the year and the number of days between
      // January 1st and the first of the respective month.
      var Months = [0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334];
      // Internal: Calculates the number of days between the Unix epoch and the
      // first day of the given month.
      var getDay = function (year, month) {
        return Months[month] + 365 * (year - 1970) + floor((year - 1969 + (month = +(month > 1))) / 4) - floor((year - 1901 + month) / 100) + floor((year - 1601 + month) / 400);
      };
    }

    // Internal: Determines if a property is a direct property of the given
    // object. Delegates to the native `Object#hasOwnProperty` method.
    if (!(isProperty = {}.hasOwnProperty)) {
      isProperty = function (property) {
        var members = {}, constructor;
        if ((members.__proto__ = null, members.__proto__ = {
          // The *proto* property cannot be set multiple times in recent
          // versions of Firefox and SeaMonkey.
          "toString": 1
        }, members).toString != getClass) {
          // Safari <= 2.0.3 doesn't implement `Object#hasOwnProperty`, but
          // supports the mutable *proto* property.
          isProperty = function (property) {
            // Capture and break the object's prototype chain (see section 8.6.2
            // of the ES 5.1 spec). The parenthesized expression prevents an
            // unsafe transformation by the Closure Compiler.
            var original = this.__proto__, result = property in (this.__proto__ = null, this);
            // Restore the original prototype chain.
            this.__proto__ = original;
            return result;
          };
        } else {
          // Capture a reference to the top-level `Object` constructor.
          constructor = members.constructor;
          // Use the `constructor` property to simulate `Object#hasOwnProperty` in
          // other environments.
          isProperty = function (property) {
            var parent = (this.constructor || constructor).prototype;
            return property in this && !(property in parent && this[property] === parent[property]);
          };
        }
        members = null;
        return isProperty.call(this, property);
      };
    }

    // Internal: A set of primitive types used by `isHostType`.
    var PrimitiveTypes = {
      'boolean': 1,
      'number': 1,
      'string': 1,
      'undefined': 1
    };

    // Internal: Determines if the given object `property` value is a
    // non-primitive.
    var isHostType = function (object, property) {
      var type = typeof object[property];
      return type == 'object' ? !!object[property] : !PrimitiveTypes[type];
    };

    // Internal: Normalizes the `for...in` iteration algorithm across
    // environments. Each enumerated key is yielded to a `callback` function.
    forEach = function (object, callback) {
      var size = 0, Properties, members, property;

      // Tests for bugs in the current environment's `for...in` algorithm. The
      // `valueOf` property inherits the non-enumerable flag from
      // `Object.prototype` in older versions of IE, Netscape, and Mozilla.
      (Properties = function () {
        this.valueOf = 0;
      }).prototype.valueOf = 0;

      // Iterate over a new instance of the `Properties` class.
      members = new Properties();
      for (property in members) {
        // Ignore all properties inherited from `Object.prototype`.
        if (isProperty.call(members, property)) {
          size++;
        }
      }
      Properties = members = null;

      // Normalize the iteration algorithm.
      if (!size) {
        // A list of non-enumerable properties inherited from `Object.prototype`.
        members = ["valueOf", "toString", "toLocaleString", "propertyIsEnumerable", "isPrototypeOf", "hasOwnProperty", "constructor"];
        // IE <= 8, Mozilla 1.0, and Netscape 6.2 ignore shadowed non-enumerable
        // properties.
        forEach = function (object, callback) {
          var isFunction = getClass.call(object) == functionClass, property, length;
          var hasProperty = !isFunction && typeof object.constructor != 'function' && isHostType(object, 'hasOwnProperty') ? object.hasOwnProperty : isProperty;
          for (property in object) {
            // Gecko <= 1.0 enumerates the `prototype` property of functions under
            // certain conditions; IE does not.
            if (!(isFunction && property == "prototype") && hasProperty.call(object, property)) {
              callback(property);
            }
          }
          // Manually invoke the callback for each non-enumerable property.
          for (length = members.length; property = members[--length]; hasProperty.call(object, property) && callback(property));
        };
      } else if (size == 2) {
        // Safari <= 2.0.4 enumerates shadowed properties twice.
        forEach = function (object, callback) {
          // Create a set of iterated properties.
          var members = {}, isFunction = getClass.call(object) == functionClass, property;
          for (property in object) {
            // Store each property name to prevent double enumeration. The
            // `prototype` property of functions is not enumerated due to cross-
            // environment inconsistencies.
            if (!(isFunction && property == "prototype") && !isProperty.call(members, property) && (members[property] = 1) && isProperty.call(object, property)) {
              callback(property);
            }
          }
        };
      } else {
        // No bugs detected; use the standard `for...in` algorithm.
        forEach = function (object, callback) {
          var isFunction = getClass.call(object) == functionClass, property, isConstructor;
          for (property in object) {
            if (!(isFunction && property == "prototype") && isProperty.call(object, property) && !(isConstructor = property === "constructor")) {
              callback(property);
            }
          }
          // Manually invoke the callback for the `constructor` property due to
          // cross-environment inconsistencies.
          if (isConstructor || isProperty.call(object, (property = "constructor"))) {
            callback(property);
          }
        };
      }
      return forEach(object, callback);
    };

    // Public: Serializes a JavaScript `value` as a JSON string. The optional
    // `filter` argument may specify either a function that alters how object and
    // array members are serialized, or an array of strings and numbers that
    // indicates which properties should be serialized. The optional `width`
    // argument may be either a string or number that specifies the indentation
    // level of the output.
    if (!has("json-stringify")) {
      // Internal: A map of control characters and their escaped equivalents.
      var Escapes = {
        92: "\\\\",
        34: '\\"',
        8: "\\b",
        12: "\\f",
        10: "\\n",
        13: "\\r",
        9: "\\t"
      };

      // Internal: Converts `value` into a zero-padded string such that its
      // length is at least equal to `width`. The `width` must be <= 6.
      var leadingZeroes = "000000";
      var toPaddedString = function (width, value) {
        // The `|| 0` expression is necessary to work around a bug in
        // Opera <= 7.54u2 where `0 == -0`, but `String(-0) !== "0"`.
        return (leadingZeroes + (value || 0)).slice(-width);
      };

      // Internal: Double-quotes a string `value`, replacing all ASCII control
      // characters (characters with code unit values between 0 and 31) with
      // their escaped equivalents. This is an implementation of the
      // `Quote(value)` operation defined in ES 5.1 section 15.12.3.
      var unicodePrefix = "\\u00";
      var quote = function (value) {
        var result = '"', index = 0, length = value.length, isLarge = length > 10 && charIndexBuggy, symbols;
        if (isLarge) {
          symbols = value.split("");
        }
        for (; index < length; index++) {
          var charCode = value.charCodeAt(index);
          // If the character is a control character, append its Unicode or
          // shorthand escape sequence; otherwise, append the character as-is.
          switch (charCode) {
            case 8: case 9: case 10: case 12: case 13: case 34: case 92:
              result += Escapes[charCode];
              break;
            default:
              if (charCode < 32) {
                result += unicodePrefix + toPaddedString(2, charCode.toString(16));
                break;
              }
              result += isLarge ? symbols[index] : charIndexBuggy ? value.charAt(index) : value[index];
          }
        }
        return result + '"';
      };

      // Internal: Recursively serializes an object. Implements the
      // `Str(key, holder)`, `JO(value)`, and `JA(value)` operations.
      var serialize = function (property, object, callback, properties, whitespace, indentation, stack) {
        var value, className, year, month, date, time, hours, minutes, seconds, milliseconds, results, element, index, length, prefix, result;
        try {
          // Necessary for host object support.
          value = object[property];
        } catch (exception) {}
        if (typeof value == "object" && value) {
          className = getClass.call(value);
          if (className == dateClass && !isProperty.call(value, "toJSON")) {
            if (value > -1 / 0 && value < 1 / 0) {
              // Dates are serialized according to the `Date#toJSON` method
              // specified in ES 5.1 section 15.9.5.44. See section 15.9.1.15
              // for the ISO 8601 date time string format.
              if (getDay) {
                // Manually compute the year, month, date, hours, minutes,
                // seconds, and milliseconds if the `getUTC*` methods are
                // buggy. Adapted from @Yaffle's `date-shim` project.
                date = floor(value / 864e5);
                for (year = floor(date / 365.2425) + 1970 - 1; getDay(year + 1, 0) <= date; year++);
                for (month = floor((date - getDay(year, 0)) / 30.42); getDay(year, month + 1) <= date; month++);
                date = 1 + date - getDay(year, month);
                // The `time` value specifies the time within the day (see ES
                // 5.1 section 15.9.1.2). The formula `(A % B + B) % B` is used
                // to compute `A modulo B`, as the `%` operator does not
                // correspond to the `modulo` operation for negative numbers.
                time = (value % 864e5 + 864e5) % 864e5;
                // The hours, minutes, seconds, and milliseconds are obtained by
                // decomposing the time within the day. See section 15.9.1.10.
                hours = floor(time / 36e5) % 24;
                minutes = floor(time / 6e4) % 60;
                seconds = floor(time / 1e3) % 60;
                milliseconds = time % 1e3;
              } else {
                year = value.getUTCFullYear();
                month = value.getUTCMonth();
                date = value.getUTCDate();
                hours = value.getUTCHours();
                minutes = value.getUTCMinutes();
                seconds = value.getUTCSeconds();
                milliseconds = value.getUTCMilliseconds();
              }
              // Serialize extended years correctly.
              value = (year <= 0 || year >= 1e4 ? (year < 0 ? "-" : "+") + toPaddedString(6, year < 0 ? -year : year) : toPaddedString(4, year)) +
                "-" + toPaddedString(2, month + 1) + "-" + toPaddedString(2, date) +
                // Months, dates, hours, minutes, and seconds should have two
                // digits; milliseconds should have three.
                "T" + toPaddedString(2, hours) + ":" + toPaddedString(2, minutes) + ":" + toPaddedString(2, seconds) +
                // Milliseconds are optional in ES 5.0, but required in 5.1.
                "." + toPaddedString(3, milliseconds) + "Z";
            } else {
              value = null;
            }
          } else if (typeof value.toJSON == "function" && ((className != numberClass && className != stringClass && className != arrayClass) || isProperty.call(value, "toJSON"))) {
            // Prototype <= 1.6.1 adds non-standard `toJSON` methods to the
            // `Number`, `String`, `Date`, and `Array` prototypes. JSON 3
            // ignores all `toJSON` methods on these objects unless they are
            // defined directly on an instance.
            value = value.toJSON(property);
          }
        }
        if (callback) {
          // If a replacement function was provided, call it to obtain the value
          // for serialization.
          value = callback.call(object, property, value);
        }
        if (value === null) {
          return "null";
        }
        className = getClass.call(value);
        if (className == booleanClass) {
          // Booleans are represented literally.
          return "" + value;
        } else if (className == numberClass) {
          // JSON numbers must be finite. `Infinity` and `NaN` are serialized as
          // `"null"`.
          return value > -1 / 0 && value < 1 / 0 ? "" + value : "null";
        } else if (className == stringClass) {
          // Strings are double-quoted and escaped.
          return quote("" + value);
        }
        // Recursively serialize objects and arrays.
        if (typeof value == "object") {
          // Check for cyclic structures. This is a linear search; performance
          // is inversely proportional to the number of unique nested objects.
          for (length = stack.length; length--;) {
            if (stack[length] === value) {
              // Cyclic structures cannot be serialized by `JSON.stringify`.
              throw TypeError();
            }
          }
          // Add the object to the stack of traversed objects.
          stack.push(value);
          results = [];
          // Save the current indentation level and indent one additional level.
          prefix = indentation;
          indentation += whitespace;
          if (className == arrayClass) {
            // Recursively serialize array elements.
            for (index = 0, length = value.length; index < length; index++) {
              element = serialize(index, value, callback, properties, whitespace, indentation, stack);
              results.push(element === undef ? "null" : element);
            }
            result = results.length ? (whitespace ? "[\n" + indentation + results.join(",\n" + indentation) + "\n" + prefix + "]" : ("[" + results.join(",") + "]")) : "[]";
          } else {
            // Recursively serialize object members. Members are selected from
            // either a user-specified list of property names, or the object
            // itself.
            forEach(properties || value, function (property) {
              var element = serialize(property, value, callback, properties, whitespace, indentation, stack);
              if (element !== undef) {
                // According to ES 5.1 section 15.12.3: "If `gap` {whitespace}
                // is not the empty string, let `member` {quote(property) + ":"}
                // be the concatenation of `member` and the `space` character."
                // The "`space` character" refers to the literal space
                // character, not the `space` {width} argument provided to
                // `JSON.stringify`.
                results.push(quote(property) + ":" + (whitespace ? " " : "") + element);
              }
            });
            result = results.length ? (whitespace ? "{\n" + indentation + results.join(",\n" + indentation) + "\n" + prefix + "}" : ("{" + results.join(",") + "}")) : "{}";
          }
          // Remove the object from the traversed object stack.
          stack.pop();
          return result;
        }
      };

      // Public: `JSON.stringify`. See ES 5.1 section 15.12.3.
      JSON3.stringify = function (source, filter, width) {
        var whitespace, callback, properties, className;
        if (typeof filter == "function" || typeof filter == "object" && filter) {
          if ((className = getClass.call(filter)) == functionClass) {
            callback = filter;
          } else if (className == arrayClass) {
            // Convert the property names array into a makeshift set.
            properties = {};
            for (var index = 0, length = filter.length, value; index < length; value = filter[index++], ((className = getClass.call(value)), className == stringClass || className == numberClass) && (properties[value] = 1));
          }
        }
        if (width) {
          if ((className = getClass.call(width)) == numberClass) {
            // Convert the `width` to an integer and create a string containing
            // `width` number of space characters.
            if ((width -= width % 1) > 0) {
              for (whitespace = "", width > 10 && (width = 10); whitespace.length < width; whitespace += " ");
            }
          } else if (className == stringClass) {
            whitespace = width.length <= 10 ? width : width.slice(0, 10);
          }
        }
        // Opera <= 7.54u2 discards the values associated with empty string keys
        // (`""`) only if they are used directly within an object member list
        // (e.g., `!("" in { "": 1})`).
        return serialize("", (value = {}, value[""] = source, value), callback, properties, whitespace, "", []);
      };
    }

    // Public: Parses a JSON source string.
    if (!has("json-parse")) {
      var fromCharCode = String.fromCharCode;

      // Internal: A map of escaped control characters and their unescaped
      // equivalents.
      var Unescapes = {
        92: "\\",
        34: '"',
        47: "/",
        98: "\b",
        116: "\t",
        110: "\n",
        102: "\f",
        114: "\r"
      };

      // Internal: Stores the parser state.
      var Index, Source;

      // Internal: Resets the parser state and throws a `SyntaxError`.
      var abort = function() {
        Index = Source = null;
        throw SyntaxError();
      };

      // Internal: Returns the next token, or `"$"` if the parser has reached
      // the end of the source string. A token may be a string, number, `null`
      // literal, or Boolean literal.
      var lex = function () {
        var source = Source, length = source.length, value, begin, position, isSigned, charCode;
        while (Index < length) {
          charCode = source.charCodeAt(Index);
          switch (charCode) {
            case 9: case 10: case 13: case 32:
              // Skip whitespace tokens, including tabs, carriage returns, line
              // feeds, and space characters.
              Index++;
              break;
            case 123: case 125: case 91: case 93: case 58: case 44:
              // Parse a punctuator token (`{`, `}`, `[`, `]`, `:`, or `,`) at
              // the current position.
              value = charIndexBuggy ? source.charAt(Index) : source[Index];
              Index++;
              return value;
            case 34:
              // `"` delimits a JSON string; advance to the next character and
              // begin parsing the string. String tokens are prefixed with the
              // sentinel `@` character to distinguish them from punctuators and
              // end-of-string tokens.
              for (value = "@", Index++; Index < length;) {
                charCode = source.charCodeAt(Index);
                if (charCode < 32) {
                  // Unescaped ASCII control characters (those with a code unit
                  // less than the space character) are not permitted.
                  abort();
                } else if (charCode == 92) {
                  // A reverse solidus (`\`) marks the beginning of an escaped
                  // control character (including `"`, `\`, and `/`) or Unicode
                  // escape sequence.
                  charCode = source.charCodeAt(++Index);
                  switch (charCode) {
                    case 92: case 34: case 47: case 98: case 116: case 110: case 102: case 114:
                      // Revive escaped control characters.
                      value += Unescapes[charCode];
                      Index++;
                      break;
                    case 117:
                      // `\u` marks the beginning of a Unicode escape sequence.
                      // Advance to the first character and validate the
                      // four-digit code point.
                      begin = ++Index;
                      for (position = Index + 4; Index < position; Index++) {
                        charCode = source.charCodeAt(Index);
                        // A valid sequence comprises four hexdigits (case-
                        // insensitive) that form a single hexadecimal value.
                        if (!(charCode >= 48 && charCode <= 57 || charCode >= 97 && charCode <= 102 || charCode >= 65 && charCode <= 70)) {
                          // Invalid Unicode escape sequence.
                          abort();
                        }
                      }
                      // Revive the escaped character.
                      value += fromCharCode("0x" + source.slice(begin, Index));
                      break;
                    default:
                      // Invalid escape sequence.
                      abort();
                  }
                } else {
                  if (charCode == 34) {
                    // An unescaped double-quote character marks the end of the
                    // string.
                    break;
                  }
                  charCode = source.charCodeAt(Index);
                  begin = Index;
                  // Optimize for the common case where a string is valid.
                  while (charCode >= 32 && charCode != 92 && charCode != 34) {
                    charCode = source.charCodeAt(++Index);
                  }
                  // Append the string as-is.
                  value += source.slice(begin, Index);
                }
              }
              if (source.charCodeAt(Index) == 34) {
                // Advance to the next character and return the revived string.
                Index++;
                return value;
              }
              // Unterminated string.
              abort();
            default:
              // Parse numbers and literals.
              begin = Index;
              // Advance past the negative sign, if one is specified.
              if (charCode == 45) {
                isSigned = true;
                charCode = source.charCodeAt(++Index);
              }
              // Parse an integer or floating-point value.
              if (charCode >= 48 && charCode <= 57) {
                // Leading zeroes are interpreted as octal literals.
                if (charCode == 48 && ((charCode = source.charCodeAt(Index + 1)), charCode >= 48 && charCode <= 57)) {
                  // Illegal octal literal.
                  abort();
                }
                isSigned = false;
                // Parse the integer component.
                for (; Index < length && ((charCode = source.charCodeAt(Index)), charCode >= 48 && charCode <= 57); Index++);
                // Floats cannot contain a leading decimal point; however, this
                // case is already accounted for by the parser.
                if (source.charCodeAt(Index) == 46) {
                  position = ++Index;
                  // Parse the decimal component.
                  for (; position < length && ((charCode = source.charCodeAt(position)), charCode >= 48 && charCode <= 57); position++);
                  if (position == Index) {
                    // Illegal trailing decimal.
                    abort();
                  }
                  Index = position;
                }
                // Parse exponents. The `e` denoting the exponent is
                // case-insensitive.
                charCode = source.charCodeAt(Index);
                if (charCode == 101 || charCode == 69) {
                  charCode = source.charCodeAt(++Index);
                  // Skip past the sign following the exponent, if one is
                  // specified.
                  if (charCode == 43 || charCode == 45) {
                    Index++;
                  }
                  // Parse the exponential component.
                  for (position = Index; position < length && ((charCode = source.charCodeAt(position)), charCode >= 48 && charCode <= 57); position++);
                  if (position == Index) {
                    // Illegal empty exponent.
                    abort();
                  }
                  Index = position;
                }
                // Coerce the parsed value to a JavaScript number.
                return +source.slice(begin, Index);
              }
              // A negative sign may only precede numbers.
              if (isSigned) {
                abort();
              }
              // `true`, `false`, and `null` literals.
              if (source.slice(Index, Index + 4) == "true") {
                Index += 4;
                return true;
              } else if (source.slice(Index, Index + 5) == "false") {
                Index += 5;
                return false;
              } else if (source.slice(Index, Index + 4) == "null") {
                Index += 4;
                return null;
              }
              // Unrecognized token.
              abort();
          }
        }
        // Return the sentinel `$` character if the parser has reached the end
        // of the source string.
        return "$";
      };

      // Internal: Parses a JSON `value` token.
      var get = function (value) {
        var results, hasMembers;
        if (value == "$") {
          // Unexpected end of input.
          abort();
        }
        if (typeof value == "string") {
          if ((charIndexBuggy ? value.charAt(0) : value[0]) == "@") {
            // Remove the sentinel `@` character.
            return value.slice(1);
          }
          // Parse object and array literals.
          if (value == "[") {
            // Parses a JSON array, returning a new JavaScript array.
            results = [];
            for (;; hasMembers || (hasMembers = true)) {
              value = lex();
              // A closing square bracket marks the end of the array literal.
              if (value == "]") {
                break;
              }
              // If the array literal contains elements, the current token
              // should be a comma separating the previous element from the
              // next.
              if (hasMembers) {
                if (value == ",") {
                  value = lex();
                  if (value == "]") {
                    // Unexpected trailing `,` in array literal.
                    abort();
                  }
                } else {
                  // A `,` must separate each array element.
                  abort();
                }
              }
              // Elisions and leading commas are not permitted.
              if (value == ",") {
                abort();
              }
              results.push(get(value));
            }
            return results;
          } else if (value == "{") {
            // Parses a JSON object, returning a new JavaScript object.
            results = {};
            for (;; hasMembers || (hasMembers = true)) {
              value = lex();
              // A closing curly brace marks the end of the object literal.
              if (value == "}") {
                break;
              }
              // If the object literal contains members, the current token
              // should be a comma separator.
              if (hasMembers) {
                if (value == ",") {
                  value = lex();
                  if (value == "}") {
                    // Unexpected trailing `,` in object literal.
                    abort();
                  }
                } else {
                  // A `,` must separate each object member.
                  abort();
                }
              }
              // Leading commas are not permitted, object property names must be
              // double-quoted strings, and a `:` must separate each property
              // name and value.
              if (value == "," || typeof value != "string" || (charIndexBuggy ? value.charAt(0) : value[0]) != "@" || lex() != ":") {
                abort();
              }
              results[value.slice(1)] = get(lex());
            }
            return results;
          }
          // Unexpected token encountered.
          abort();
        }
        return value;
      };

      // Internal: Updates a traversed object member.
      var update = function(source, property, callback) {
        var element = walk(source, property, callback);
        if (element === undef) {
          delete source[property];
        } else {
          source[property] = element;
        }
      };

      // Internal: Recursively traverses a parsed JSON object, invoking the
      // `callback` function for each value. This is an implementation of the
      // `Walk(holder, name)` operation defined in ES 5.1 section 15.12.2.
      var walk = function (source, property, callback) {
        var value = source[property], length;
        if (typeof value == "object" && value) {
          // `forEach` can't be used to traverse an array in Opera <= 8.54
          // because its `Object#hasOwnProperty` implementation returns `false`
          // for array indices (e.g., `![1, 2, 3].hasOwnProperty("0")`).
          if (getClass.call(value) == arrayClass) {
            for (length = value.length; length--;) {
              update(value, length, callback);
            }
          } else {
            forEach(value, function (property) {
              update(value, property, callback);
            });
          }
        }
        return callback.call(source, property, value);
      };

      // Public: `JSON.parse`. See ES 5.1 section 15.12.2.
      JSON3.parse = function (source, callback) {
        var result, value;
        Index = 0;
        Source = "" + source;
        result = get(lex());
        // If a JSON string contains multiple tokens, it is invalid.
        if (lex() != "$") {
          abort();
        }
        // Reset the parser state.
        Index = Source = null;
        return callback && getClass.call(callback) == functionClass ? walk((value = {}, value[""] = result, value), "", callback) : result;
      };
    }
  }

  // Export for asynchronous module loaders.
  if (isLoader) {
    define(function () {
      return JSON3;
    });
  }
}(this));

},{}],48:[function(_dereq_,module,exports){
module.exports = toArray

function toArray(list, index) {
    var array = []

    index = index || 0

    for (var i = index || 0; i < list.length; i++) {
        array[i - index] = list[i]
    }

    return array
}

},{}]},{},[1])
(1)
});
